@@ -5,7 +5,7 @@
 // CoderInfo implementation
 //
 
-#define MAGICK_IMPLEMENTATION  1
+#define MAGICKCORE_IMPLEMENTATION  1
 
 #include "Magick++/Include.h"
 #include "Magick++/CoderInfo.h"
