@@ -7,7 +7,7 @@
 // A wrapper around DrawInfo, ImageInfo, and QuantizeInfo
 //
 
-#define MAGICK_IMPLEMENTATION
+#define MAGICKCORE_IMPLEMENTATION  1
 
 #include "Magick++/Include.h"
 #include <string>
@@ -18,6 +18,9 @@
 #include "Magick++/Functions.h"
 #include "Magick++/Exception.h"
 
+#define MagickPI  3.14159265358979323846264338327950288419716939937510
+#define DegreesToRadians(x)  (MagickPI*(x)/180.0)
+
 // Constructor
 Magick::Options::Options( void )
   : _imageInfo(static_cast<ImageInfo*>(AcquireMagickMemory(sizeof(ImageInfo)))),
@@ -59,15 +62,15 @@ Magick::Options::Options( const MagickLib::ImageInfo* imageInfo_,
 Magick::Options::~Options()
 {
   // Destroy image info
-  DestroyImageInfo( _imageInfo );
+   _imageInfo =DestroyImageInfo( _imageInfo );
   _imageInfo=0;
 
   // Destroy quantization info
-  DestroyQuantizeInfo( _quantizeInfo );
+   _quantizeInfo =DestroyQuantizeInfo( _quantizeInfo );
   _quantizeInfo=0;
 
   // Destroy drawing info
-  DestroyDrawInfo( _drawInfo );
+   _drawInfo =DestroyDrawInfo( _drawInfo );
   _drawInfo=0;
 }
 
@@ -436,6 +439,21 @@ Magick::ResolutionType Magick::Options::resolutionUnits ( void ) const
   return static_cast<Magick::ResolutionType>(_imageInfo->units);
 }
 
+void Magick::Options::samplingFactor ( const std::string &samplingFactor_ )
+{
+  if ( samplingFactor_.length() == 0 )
+    _imageInfo->sampling_factor=(char *) RelinquishMagickMemory(_imageInfo->sampling_factor);
+  else
+    Magick::CloneString( &_imageInfo->sampling_factor, samplingFactor_ );
+}
+std::string Magick::Options::samplingFactor ( void ) const
+{
+  if ( _imageInfo->sampling_factor )
+    return std::string( _imageInfo->sampling_factor );
+
+  return std::string();
+}
+
 void Magick::Options::size ( const Geometry &geometry_ )
 {
   _imageInfo->size=(char *) RelinquishMagickMemory(_imageInfo->size);
