@@ -18,7 +18,7 @@
 %                                March  2004                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -60,6 +60,7 @@
 #include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/profile.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/utility.h"
@@ -482,7 +483,7 @@ static boolean ReadICCProfile(j_decompress_ptr jpeg_info)
   p=profile->datum;
   for (i=(long) profile->length-1; i >= 0; i--)
     *p++=(unsigned char) GetCharacter(jpeg_info);
-  icc_profile=GetImageProfile(image,"icc");
+  icc_profile=(StringInfo *) GetImageProfile(image,"icc");
   if (icc_profile != (StringInfo *) NULL)
     ConcatenateStringInfo(icc_profile,profile);
   else
@@ -545,7 +546,7 @@ static boolean ReadIPTCProfile(j_decompress_ptr jpeg_info)
 #endif
   error_manager=(ErrorManager *) jpeg_info->client_data;
   image=error_manager->image;
-  iptc_profile=GetImageProfile(image,"iptc");
+  iptc_profile=(StringInfo *) GetImageProfile(image,"iptc");
   if (iptc_profile == (StringInfo *) NULL)
     {
 #ifdef GET_ONLY_IPTC_DATA
@@ -600,7 +601,7 @@ static boolean ReadIPTCProfile(j_decompress_ptr jpeg_info)
   p=profile->datum;
   for (i=(long) profile->length-1; i >= 0; i--)
     *p++=(unsigned char) GetCharacter(jpeg_info);
-  iptc_profile=GetImageProfile(image,"iptc");
+  iptc_profile=(StringInfo *) GetImageProfile(image,"iptc");
   if (iptc_profile != (StringInfo *) NULL)
     ConcatenateStringInfo(iptc_profile,profile);
   else
@@ -791,9 +792,6 @@ static MagickBooleanType DecodeImage(Image *image,
 #define BI_RLE8  1
 #endif
 
-  int
-    count;
-
   long
     y;
 
@@ -808,13 +806,16 @@ static MagickBooleanType DecodeImage(Image *image,
     *p,
     *q;
 
+  ssize_t
+    count;
+
   unsigned char
     byte;
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(pixels != (unsigned char *) NULL);
   (void) ResetMagickMemory(pixels,0,(size_t) image->columns*image->rows*
     sizeof(*pixels));
@@ -826,24 +827,24 @@ static MagickBooleanType DecodeImage(Image *image,
   {
     if ((p < pixels) || (p >= q))
       break;
-    count=ReadBlobByte(image);
-    if (count == EOF)
+    count=(ssize_t) ReadBlobByte(image);
+    if ((int) count == EOF)
       break;
     if (count != 0)
       {
-        count=Min((unsigned long) count,(size_t) (q-p));
+        count=Min(count,(ssize_t) (q-p));
         /*
           Encoded mode.
         */
         byte=(unsigned char) ReadBlobByte(image);
         if (compression == BI_RLE8)
           {
-            for (i=0; i < count; i++)
+            for (i=0; i < (long) count; i++)
               *p++=(unsigned char) byte;
           }
         else
           {
-            for (i=0; i < count; i++)
+            for (i=0; i < (long) count; i++)
               *p++=(unsigned char)
                 ((i & 0x01) != 0 ? (byte & 0x0f) : ((byte >> 4) & 0x0f));
           }
@@ -854,7 +855,7 @@ static MagickBooleanType DecodeImage(Image *image,
         /*
           Escape mode.
         */
-        count=ReadBlobByte(image);
+        count=(ssize_t) ReadBlobByte(image);
         if (count == 0x01)
           return(MagickTrue);
         switch (count)
@@ -884,12 +885,12 @@ static MagickBooleanType DecodeImage(Image *image,
             /*
               Absolute mode.
             */
-            count=Min((unsigned long) count,(size_t) (q-p));
+            count=Min(count,(ssize_t) (q-p));
             if (compression == BI_RLE8)
-              for (i=0; i < count; i++)
+              for (i=0; i < (long) count; i++)
                 *p++=(unsigned char) ReadBlobByte(image);
             else
-              for (i=0; i < count; i++)
+              for (i=0; i < (long) count; i++)
               {
                 if ((i & 0x01) == 0)
                   byte=(unsigned char) ReadBlobByte(image);
@@ -1017,6 +1018,9 @@ static Image *ReadAVIImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickBooleanType
     status;
 
+  MagickOffsetType
+    offset;
+
   PixelPacket
     *colormap;
 
@@ -1083,7 +1087,8 @@ static Image *ReadAVIImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
@@ -1138,7 +1143,8 @@ static Image *ReadAVIImage(const ImageInfo *image_info,ExceptionInfo *exception)
       break;
     if ((chunk_size & 0x01) != 0)
       chunk_size++;
-    if ((GetBlobSize(image) != 0) && (chunk_size > GetBlobSize(image)))
+    if ((GetBlobSize(image) != 0) &&
+        ((MagickSizeType) chunk_size > GetBlobSize(image)))
       ThrowReaderException(CorruptImageError,"NotEnoughPixelData");
     if (image_info->verbose != MagickFalse)
       {
@@ -1216,8 +1222,8 @@ static Image *ReadAVIImage(const ImageInfo *image_info,ExceptionInfo *exception)
               jpeg_info.scale_denom=(unsigned int) scale_factor;
               jpeg_calc_output_dimensions(&jpeg_info);
               if (image->debug != MagickFalse)
-                (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Scale factor: %ld",
-                (long) scale_factor);
+                (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                  "Scale factor: %ld",(long) scale_factor);
             }
             if (image_info->subrange != 0)
             {
@@ -1226,14 +1232,15 @@ static Image *ReadAVIImage(const ImageInfo *image_info,ExceptionInfo *exception)
             }
 #if (JPEG_LIB_VERSION >= 61) && defined(D_PROGRESSIVE_SUPPORTED)
 #ifdef D_LOSSLESS_SUPPORTED
-            image->interlace=
-              jpeg_info.process == JPROC_PROGRESSIVE ? PlaneInterlace : NoInterlace;
+            image->interlace=jpeg_info.process == JPROC_PROGRESSIVE ?
+              PlaneInterlace : NoInterlace;
             image->compression=jpeg_info.process == JPROC_LOSSLESS ?
-LosslessJPEGCompression : JPEGCompression;
+              LosslessJPEGCompression : JPEGCompression;
             if (jpeg_info.data_precision > 8)
-              (void) ThrowMagickException(exception,GetMagickModule(),OptionError,
-              "12-bit JPEG not supported. Reducing pixel data to 8 bits","`%s'",
-              image->filename);
+              (void) ThrowMagickException(exception,GetMagickModule(),
+                OptionError,
+                "12-bit JPEG not supported. Reducing pixel data to 8 bits",
+                "`%s'",image->filename);
 #else
             image->interlace=jpeg_info.progressive_mode != 0 ? PlaneInterlace :
             NoInterlace;
@@ -1267,18 +1274,19 @@ LosslessJPEGCompression : JPEGCompression;
             {
               if (image->interlace == PlaneInterlace)
                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                "Interlace: progressive");
+                  "Interlace: progressive");
               else
                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                "Interlace: nonprogressive");
-              (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Data precision: %d",
-                (int) jpeg_info.data_precision);
-              (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Geometry: %dx%d",
-                (int) jpeg_info.output_width,(int) jpeg_info.output_height);
+                  "Interlace: nonprogressive");
+              (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                "Data precision: %d",(int) jpeg_info.data_precision);
+              (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                "Geometry: %dx%d",(int) jpeg_info.output_width,(int)
+                jpeg_info.output_height);
 #ifdef D_LOSSLESS_SUPPORTED
               if (image->compression==LosslessJPEGCompression)
                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                "Quality: 100 (lossless)");
+                  "Quality: 100 (lossless)");
               else
 #endif
               {
@@ -1348,10 +1356,10 @@ LosslessJPEGCompression : JPEGCompression;
                       {
                         if ((hashval > hash[i]) || (sum > sums[i]))
                           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                          "Quality: %ld (approximate)",i+1);
+                            "Quality: %ld (approximate)",i+1);
                         else
                           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                          "Quality: %ld",i+1);
+                            "Quality: %ld",i+1);
                         break;
                       }
                     }
@@ -1397,10 +1405,10 @@ LosslessJPEGCompression : JPEGCompression;
                         {
                           if ((hashval > bwhash[i]) || (sum > bwsum[i]))
                             (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                            "Quality: %ld (approximate)",i+1);
+                              "Quality: %ld (approximate)",i+1);
                           else
                             (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                            "Quality: %ld",i+1);
+                              "Quality: %ld",i+1);
                           break;
                         }
                       }
@@ -1429,13 +1437,15 @@ LosslessJPEGCompression : JPEGCompression;
                   (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                     "Colorspace: GRAYSCALE");
                   (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                    "Sampling factors: (%d,%d)",jpeg_info.comp_info[0].h_samp_factor,
+                    "Sampling factors: (%d,%d)",
+                    jpeg_info.comp_info[0].h_samp_factor,
                     jpeg_info.comp_info[0].v_samp_factor);
                   break;
                 }
               case JCS_RGB:
                 {
-                  (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Colorspace: RGB");
+                  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                    "Colorspace: RGB");
                   (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                     "Sampling factors: (%d,%d),(%d,%d),(%d,%d)",
                     jpeg_info.comp_info[0].h_samp_factor,
@@ -1448,8 +1458,8 @@ LosslessJPEGCompression : JPEGCompression;
                 }
               default:
                 {
-                  (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Colorspace: %d",
-                    jpeg_info.out_color_space);
+                  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                    "Colorspace: %d",jpeg_info.out_color_space);
                   (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                     "Sampling factors: (%d,%d),(%d,%d),(%d,%d),(%d,%d)",
                     jpeg_info.comp_info[0].h_samp_factor,
@@ -1500,11 +1510,15 @@ LosslessJPEGCompression : JPEGCompression;
                 else
                   for (x=0; x < (long) image->columns; x++)
                   {
-                    q->red=ScaleShortToQuantum(16*GETJSAMPLE(*p++));
-                    q->green=ScaleShortToQuantum(16*GETJSAMPLE(*p++));
-                    q->blue=ScaleShortToQuantum(16*GETJSAMPLE(*p++));
+                    q->red=ScaleShortToQuantum((unsigned short)
+                      (16*GETJSAMPLE(*p++)));
+                    q->green=ScaleShortToQuantum((unsigned short)
+                      (16*GETJSAMPLE(*p++)));
+                    q->blue=ScaleShortToQuantum((unsigned short)
+                      (16*GETJSAMPLE(*p++)));
                     if (image->colorspace == CMYKColorspace)
-                      indexes[x]=ScaleShortToQuantum(16*GETJSAMPLE(*p++));
+                      indexes[x]=ScaleShortToQuantum((unsigned short)
+                        (16*GETJSAMPLE(*p++)));
                     q++;
                   }
               }
@@ -1521,11 +1535,14 @@ LosslessJPEGCompression : JPEGCompression;
                 else
                   for (x=0; x < (long) image->columns; x++)
                   {
-                    q->red=ScaleCharToQuantum(GETJSAMPLE(*p++));
-                    q->green=ScaleCharToQuantum(GETJSAMPLE(*p++));
-                    q->blue=ScaleCharToQuantum(GETJSAMPLE(*p++));
+                    q->red=ScaleCharToQuantum((unsigned char) GETJSAMPLE(*p++));
+                    q->green=ScaleCharToQuantum((unsigned char)
+                      GETJSAMPLE(*p++));
+                    q->blue=ScaleCharToQuantum((unsigned char)
+                      GETJSAMPLE(*p++));
                     if (image->colorspace == CMYKColorspace)
-                      indexes[x]=ScaleCharToQuantum(GETJSAMPLE(*p++));
+                      indexes[x]=ScaleCharToQuantum((unsigned char)
+                        GETJSAMPLE(*p++));
                     q++;
                   }
                   if (SyncImagePixels(image) == MagickFalse)
@@ -1562,10 +1579,10 @@ LosslessJPEGCompression : JPEGCompression;
                   indexes=GetIndexes(image);
                   for (x=0; x < (long) image->columns; x++)
                   {
-                    q->red=(Quantum) (MaxRGB-q->red);
-                    q->green=(Quantum) (MaxRGB-q->green);
-                    q->blue=(Quantum) (MaxRGB-q->blue);
-                    indexes[x]=(IndexPacket) (MaxRGB-indexes[x]);
+                    q->red=(Quantum) (QuantumRange-q->red);
+                    q->green=(Quantum) (QuantumRange-q->green);
+                    q->blue=(Quantum) (QuantumRange-q->blue);
+                    indexes[x]=(IndexPacket) (QuantumRange-indexes[x]);
                     q++;
                   }
                   if (SyncImagePixels(image) == MagickFalse)
@@ -1682,11 +1699,12 @@ LosslessJPEGCompression : JPEGCompression;
                 }
                 if (SyncImagePixels(image) == MagickFalse)
                   break;
+                offset=(MagickOffsetType) image->rows-y-1;
                 if (image->previous == (Image *) NULL)
                 if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                    (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
+                    (QuantumTick(offset,image->rows) != MagickFalse))
                   {
-                    status=image->progress_monitor(LoadImageTag,image->rows-y-1,
+                    status=image->progress_monitor(LoadImageTag,offset,
                       image->rows,image->client_data);
                     if (status == MagickFalse)
                       break;
@@ -1725,11 +1743,12 @@ LosslessJPEGCompression : JPEGCompression;
                 }
                 if (SyncImagePixels(image) == MagickFalse)
                   break;
+                offset=(MagickOffsetType) image->rows-y-1;
                 if (image->previous == (Image *) NULL)
                 if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                    (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
+                    (QuantumTick(offset,image->rows) != MagickFalse))
                   {
-                    status=image->progress_monitor(LoadImageTag,image->rows-y-1,
+                    status=image->progress_monitor(LoadImageTag,offset,
                       image->rows,image->client_data);
                     if (status == MagickFalse)
                       break;
@@ -1760,11 +1779,12 @@ LosslessJPEGCompression : JPEGCompression;
                 }
                 if (SyncImagePixels(image) == MagickFalse)
                   break;
+                offset=(MagickOffsetType) image->rows-y-1;
                 if (image->previous == (Image *) NULL)
                 if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                    (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
+                    (QuantumTick(offset,image->rows) != MagickFalse))
                   {
-                    status=image->progress_monitor(LoadImageTag,image->rows-y-1,
+                    status=image->progress_monitor(LoadImageTag,offset,
                       image->rows,image->client_data);
                     if (status == MagickFalse)
                       break;
@@ -1799,11 +1819,12 @@ LosslessJPEGCompression : JPEGCompression;
                 }
                 if (SyncImagePixels(image) == MagickFalse)
                   break;
+                offset=(MagickOffsetType) image->rows-y-1;
                 if (image->previous == (Image *) NULL)
                 if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                    (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
+                    (QuantumTick(offset,image->rows) != MagickFalse))
                   {
-                    status=image->progress_monitor(LoadImageTag,image->rows-y-1,
+                    status=image->progress_monitor(LoadImageTag,offset,
                       image->rows,image->client_data);
                     if (status == MagickFalse)
                       break;
@@ -1835,11 +1856,12 @@ LosslessJPEGCompression : JPEGCompression;
                 }
                 if (SyncImagePixels(image) == MagickFalse)
                   break;
+                offset=(MagickOffsetType) image->rows-y-1;
                 if (image->previous == (Image *) NULL)
                 if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                    (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
+                    (QuantumTick(offset,image->rows) != MagickFalse))
                   {
-                    status=image->progress_monitor(LoadImageTag,image->rows-y-1,
+                    status=image->progress_monitor(LoadImageTag,offset,
                       image->rows,image->client_data);
                     if (status == MagickFalse)
                       break;
@@ -1863,7 +1885,7 @@ LosslessJPEGCompression : JPEGCompression;
               Allocate next image structure.
             */
             AllocateNextImage(image_info,image);
-            if (image->next == (Image *) NULL)
+            if (GetNextImageInList(image) == (Image *) NULL)
               {
                 image=DestroyImageList(image);
                 return((Image *) NULL);
@@ -1968,9 +1990,12 @@ LosslessJPEGCompression : JPEGCompression;
                     "MemoryAllocationFailed");
                 for (i=0; i < (long) number_colors; i++)
                 {
-                  colormap[i].blue=ScaleCharToQuantum(ReadBlobByte(image));
-                  colormap[i].green=ScaleCharToQuantum(ReadBlobByte(image));
-                  colormap[i].red=ScaleCharToQuantum(ReadBlobByte(image));
+                  colormap[i].blue=ScaleCharToQuantum((unsigned char)
+                    ReadBlobByte(image));
+                  colormap[i].green=ScaleCharToQuantum((unsigned char)
+                    ReadBlobByte(image));
+                  colormap[i].red=ScaleCharToQuantum((unsigned char)
+                    ReadBlobByte(image));
                   (void) ReadBlobByte(image);
                   chunk_size-=4;
                 }
@@ -2100,8 +2125,8 @@ ModuleExport void RegisterAVIImage(void)
   entry=SetMagickInfo("AVI");
   entry->decoder=(DecoderHandler *) ReadAVIImage;
   entry->magick=(MagickHandler *) IsAVI;
-  entry->description=AcquireString("Microsoft Audio/Visual Interleaved");
-  entry->module=AcquireString("AVI");
+  entry->description=ConstantString("Microsoft Audio/Visual Interleaved");
+  entry->module=ConstantString("AVI");
   (void) RegisterMagickInfo(entry);
 }
 
