@@ -18,7 +18,7 @@
 %                               December 2001                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -55,6 +55,7 @@
 #include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/profile.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/transform.h"
@@ -148,7 +149,7 @@ static MagickBooleanType
 %  The format of the DecodeImage method is:
 %
 %      MagickBooleanType DecodeImage(Image *image,
-%        const MagickBooleanType compression,unsigned char *pixels)
+%        const unsigned long compression,unsigned char *pixels)
 %
 %  A description of each parameter follows:
 %
@@ -165,7 +166,7 @@ static MagickBooleanType
 %
 */
 static MagickBooleanType DecodeImage(Image *image,
-  const MagickBooleanType compression,unsigned char *pixels)
+  const unsigned long compression,unsigned char *pixels)
 {
   int
     count;
@@ -190,7 +191,7 @@ static MagickBooleanType DecodeImage(Image *image,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(pixels != (unsigned char *) NULL);
   (void) ResetMagickMemory(pixels,0,(size_t) image->columns*image->rows*
     sizeof(*pixels));
@@ -360,7 +361,7 @@ static size_t EncodeImage(Image *image,const unsigned long bytes_per_line,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(pixels != (const unsigned char *) NULL);
   assert(compressed_pixels != (unsigned char *) NULL);
   p=pixels;
@@ -489,6 +490,7 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
     status;
 
   MagickOffsetType
+    offset,
     start_position;
 
   register IndexPacket
@@ -530,7 +532,8 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
@@ -776,16 +779,12 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
       ThrowReaderException(CorruptImageError,"NegativeOrZeroImageSize");
     if (bmp_info.height == 0)
       ThrowReaderException(CorruptImageError,"NegativeOrZeroImageSize");
-    if ((bmp_info.height < 0) && (bmp_info.compression !=0))
-      ThrowReaderException(CorruptImageError,"CompressionNotValid");
     if (bmp_info.planes != 1)
       ThrowReaderException(CorruptImageError,"StaticPlanesValueNotEqualToOne");
     if ((bmp_info.bits_per_pixel != 1) && (bmp_info.bits_per_pixel != 4) &&
         (bmp_info.bits_per_pixel != 8) && (bmp_info.bits_per_pixel != 16) &&
         (bmp_info.bits_per_pixel != 24) && (bmp_info.bits_per_pixel != 32))
       ThrowReaderException(CorruptImageError,"UnrecognizedBitsPerPixel");
-    if (bmp_info.number_colors > (1UL << bmp_info.bits_per_pixel))
-      ThrowReaderException(CorruptImageError,"UnrecognizedNumberOfColors");
     if (bmp_info.compression > 3)
       ThrowReaderException(CorruptImageError,"UnrecognizedImageCompression");
     if ((bmp_info.compression == 1) && (bmp_info.bits_per_pixel != 8))
@@ -811,7 +810,7 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
     image->columns=(unsigned long) bmp_info.width;
     image->rows=(unsigned long) AbsoluteValue(bmp_info.height);
     image->depth=8;
-    image->matte=(MagickBooleanType) (bmp_info.alpha_mask != 0);
+    image->matte=bmp_info.alpha_mask != 0 ? MagickTrue : MagickFalse;
     if ((bmp_info.number_colors != 0) || (bmp_info.bits_per_pixel < 16))
       {
         image->storage_class=PseudoClass;
@@ -888,8 +887,7 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Convert run-length encoded raster pixels.
         */
-        status=DecodeImage(image,
-          bmp_info.compression != 0 ? MagickTrue : MagickFalse,pixels);
+        status=DecodeImage(image,bmp_info.compression,pixels);
         if (status == MagickFalse)
           ThrowReaderException(CorruptImageError,
             "UnableToRunlengthDecodeImage");
@@ -1073,12 +1071,13 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
           }
           if (SyncImagePixels(image) == MagickFalse)
             break;
+          offset=(MagickOffsetType) image->rows-y-1;
           if (image->previous == (Image *) NULL)
             if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
+                (QuantumTick(offset,image->rows) != MagickFalse))
               {
-                status=image->progress_monitor(LoadImageTag,(MagickOffsetType)
-                  image->rows-y-1,image->rows,image->client_data);
+                status=image->progress_monitor(LoadImageTag,offset,image->rows,
+                  image->client_data);
                 if (status == MagickFalse)
                   break;
               }
@@ -1126,26 +1125,27 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
               blue|=((blue & 0xe000) >> 5);
             if (quantum_bits.blue <= 8)
               blue|=((blue & 0xff00) >> 8);
-            q->red=ScaleShortToQuantum(red);
-            q->green=ScaleShortToQuantum(green);
-            q->blue=ScaleShortToQuantum(blue);
+            q->red=ScaleShortToQuantum((unsigned short) red);
+            q->green=ScaleShortToQuantum((unsigned short) green);
+            q->blue=ScaleShortToQuantum((unsigned short) blue);
             if (image->matte != MagickFalse)
               {
                 opacity=((pixel & bmp_info.alpha_mask) << shift.opacity) >> 16;
                 if (quantum_bits.opacity <= 8)
                   q->opacity|=((q->opacity & 0xff00) >> 8);
-                q->opacity=ScaleShortToQuantum(opacity);
+                q->opacity=ScaleShortToQuantum((unsigned short) opacity);
               }
             q++;
           }
           if (SyncImagePixels(image) == MagickFalse)
             break;
+          offset=(MagickOffsetType) image->rows-y-1;
           if (image->previous == (Image *) NULL)
             if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
+                (QuantumTick(offset,image->rows) != MagickFalse))
               {
-                status=image->progress_monitor(LoadImageTag,(MagickOffsetType)
-                  image->rows-y-1,image->rows,image->client_data);
+                status=image->progress_monitor(LoadImageTag,offset,image->rows,
+                  image->client_data);
                 if (status == MagickFalse)
                   break;
               }
@@ -1173,12 +1173,13 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
           }
           if (SyncImagePixels(image) == MagickFalse)
             break;
+          offset=(MagickOffsetType) image->rows-y-1;
           if (image->previous == (Image *) NULL)
             if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
+                (QuantumTick(offset,image->rows) != MagickFalse))
               {
-                status=image->progress_monitor(LoadImageTag,(MagickOffsetType)
-                  image->rows-y-1,image->rows,image->client_data);
+                status=image->progress_monitor(LoadImageTag,offset,image->rows,
+                  image->client_data);
                 if (status == MagickFalse)
                   break;
               }
@@ -1224,21 +1225,22 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 opacity=((pixel & bmp_info.alpha_mask) << shift.opacity) >> 16;
                 if (quantum_bits.opacity == 8)
                   opacity|=(opacity >> 8);
-                  q->opacity=ScaleShortToQuantum(opacity);
+                  q->opacity=ScaleShortToQuantum((unsigned short) opacity);
               }
-            q->red=ScaleShortToQuantum(red);
-            q->green=ScaleShortToQuantum(green);
-            q->blue=ScaleShortToQuantum(blue);
+            q->red=ScaleShortToQuantum((unsigned short) red);
+            q->green=ScaleShortToQuantum((unsigned short) green);
+            q->blue=ScaleShortToQuantum((unsigned short) blue);
             q++;
           }
           if (SyncImagePixels(image) == MagickFalse)
             break;
+          offset=(MagickOffsetType) image->rows-y-1;
           if (image->previous == (Image *) NULL)
             if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-                (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
+                (QuantumTick(offset,image->rows) != MagickFalse))
               {
-                status=image->progress_monitor(LoadImageTag,(MagickOffsetType)
-                  image->rows-y-1,image->rows,image->client_data);
+                status=image->progress_monitor(LoadImageTag,offset,image->rows,
+                  image->client_data);
                 if (status == MagickFalse)
                   break;
               }
@@ -1288,7 +1290,7 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
           Acquire next image structure.
         */
         AllocateNextImage(image_info,image);
-        if (image->next == (Image *) NULL)
+        if (GetNextImageInList(image) == (Image *) NULL)
           {
             image=DestroyImageList(image);
             return((Image *) NULL);
@@ -1339,24 +1341,24 @@ ModuleExport void RegisterBMPImage(void)
   entry->decoder=(DecoderHandler *) ReadBMPImage;
   entry->encoder=(EncoderHandler *) WriteBMPImage;
   entry->magick=(MagickHandler *) IsBMP;
-  entry->description=AcquireString("Microsoft Windows bitmap image");
-  entry->module=AcquireString("BMP");
+  entry->description=ConstantString("Microsoft Windows bitmap image");
+  entry->module=ConstantString("BMP");
   entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("BMP2");
   entry->encoder=(EncoderHandler *) WriteBMPImage;
   entry->magick=(MagickHandler *) IsBMP;
-  entry->description=AcquireString("Microsoft Windows bitmap image v2");
-  entry->module=AcquireString("BMP");
+  entry->description=ConstantString("Microsoft Windows bitmap image v2");
+  entry->module=ConstantString("BMP");
   entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("BMP3");
   entry->encoder=(EncoderHandler *) WriteBMPImage;
   entry->magick=(MagickHandler *) IsBMP;
-  entry->description=AcquireString("Microsoft Windows bitmap image v3");
-  entry->module=AcquireString("BMP");
+  entry->description=ConstantString("Microsoft Windows bitmap image v3");
+  entry->module=ConstantString("BMP");
   entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
   (void) RegisterMagickInfo(entry);
@@ -1420,6 +1422,9 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image)
   BMPInfo
     bmp_info;
 
+  const StringInfo
+    *profile;
+
   long
     y;
 
@@ -1443,9 +1448,6 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image)
   register unsigned char
     *q;
 
-  StringInfo
-    *profile;
-
   unsigned char
     *bmp_data,
     *pixels;
@@ -1462,7 +1464,7 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
@@ -1478,7 +1480,8 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image)
     /*
       Initialize BMP raster file header.
     */
-    (void) SetImageColorspace(image,RGBColorspace);
+    if (image_info->colorspace == UndefinedColorspace)
+      (void) SetImageColorspace(image,RGBColorspace);
     (void) ResetMagickMemory(&bmp_info,0,sizeof(bmp_info));
     bmp_info.file_size=14+12;
     if (type > 2)
@@ -1486,7 +1489,7 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image)
     bmp_info.offset_bits=bmp_info.file_size;
     bmp_info.compression=BI_RGB;
     if ((image->storage_class == PseudoClass) && (image->colors > 256))
-      image->storage_class=DirectClass;
+      (void) SetImageStorageClass(image,DirectClass);
     if (image->storage_class != DirectClass)
       {
         /*
@@ -1505,10 +1508,10 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image)
           bmp_info.bits_per_pixel=8;
         bmp_info.number_colors=1UL << bmp_info.bits_per_pixel;
         if (image->matte != MagickFalse)
-          image->storage_class=DirectClass;
+          (void) SetImageStorageClass(image,DirectClass);
         else
           if (bmp_info.number_colors < image->colors)
-            image->storage_class=DirectClass;
+            (void) SetImageStorageClass(image,DirectClass);
           else
             {
               bmp_info.file_size+=3*(1UL << bmp_info.bits_per_pixel);
@@ -1528,15 +1531,15 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image)
         bmp_info.number_colors=0;
         bmp_info.bits_per_pixel=(unsigned short)
           ((type > 3) && (image->matte != MagickFalse) ? 32 : 24);
-        bmp_info.compression=(MagickBooleanType)
-          ((type > 3) && (image->matte != MagickFalse) ? BI_BITFIELDS : BI_RGB);
+        bmp_info.compression=(unsigned int) ((type > 3) &&
+          (image->matte != MagickFalse) ?  BI_BITFIELDS : BI_RGB);
       }
     bytes_per_line=4*((image->columns*bmp_info.bits_per_pixel+31)/32);
     bmp_info.ba_offset=0;
     profile=GetImageProfile(image,"icc");
-    have_color_info=(MagickBooleanType)
-      (((image->rendering_intent != UndefinedIntent) ||
-       (profile != (StringInfo *) NULL) || (image->gamma != 0.0)));
+    have_color_info=(image->rendering_intent != UndefinedIntent) ||
+      (profile != (StringInfo *) NULL) || (image->gamma != 0.0) ?  MagickTrue :
+      MagickFalse;
     if (type == 2)
       bmp_info.size=12;
     else
@@ -1994,7 +1997,7 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image)
         "  Pixels:  %lu bytes",bmp_info.image_size);
     (void) WriteBlob(image,(size_t) bmp_info.image_size,pixels);
     pixels=(unsigned char *) RelinquishMagickMemory(pixels);
-    if (image->next == (Image *) NULL)
+    if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
