@@ -17,7 +17,7 @@
 %                               February 2002                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -52,6 +52,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/option.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 
@@ -122,16 +123,21 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   if (*image_info->filename != '@')
     caption=AcquireString(image_info->filename);
   else
-    caption=(char *) FileToBlob(image_info->filename+1,&length,exception);
+    {
+      caption=FileToString(image_info->filename+1,~0,exception);
+      if ((caption != (char *) NULL) && (*caption != '\0'))
+        if (caption[strlen(caption)-1] == '\n')
+          caption[strlen(caption)-1]='\0';
+    }
   if (caption == (char *) NULL)
     return((Image *) NULL);
-  StripString(caption);
   image=AllocateImage(image_info);
   if (image->columns == 0)
     ThrowReaderException(OptionError,"MustSpecifyImageSize");
@@ -139,16 +145,25 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
     Format caption.
   */
   draw_info=CloneDrawInfo(image_info,(DrawInfo *) NULL);
-  draw_info->fill=image_info->pen;
   draw_info->text=AcquireString(caption);
   gravity=GetImageOption(image_info,"gravity");
   if (gravity != (char *) NULL)
-    draw_info->gravity=(GravityType)
-      ParseMagickOption(MagickGravityOptions,MagickFalse,gravity);
-  p=caption;
+    draw_info->gravity=(GravityType) ParseMagickOption(MagickGravityOptions,
+      MagickFalse,gravity);
+  length=strlen(caption)+1;
+  for (p=caption; *p != '\0'; p++)
+  {
+    if (*p == '\r')
+      *p=' ';
+    if ((*p == '\\') && (*(p+1) == 'n'))
+      {
+        (void) CopyMagickString(p,p+1,length-(p-caption));
+        *p='\n';
+      }
+  }
   q=draw_info->text;
   s=(char *) NULL;
-  for (i=0; *p != '\0'; p++)
+  for (p=caption; *p != '\0'; p++)
   {
     if (isspace((int) ((unsigned char) *p)) != 0)
       s=p;
@@ -157,24 +172,28 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
     status=GetTypeMetrics(image,draw_info,&metrics);
     if (status == MagickFalse)
       ThrowReaderException(TypeError,"UnableToGetTypeMetrics");
-    if ((metrics.width+metrics.max_advance/2) < (double) image->columns)
-      continue;
-    if (s != '\0')
+    if (*p != '\n')
+      if ((metrics.width+metrics.max_advance/2) < (double) image->columns)
+        continue;
+    if ((s != (char *) NULL) && (*s != '\0'))
       {
         *s='\n';
-        i++;
         p=s+1;
         s=(char *) NULL;
       }
     q=draw_info->text;
   }
+  i=0;
+  for (p=caption; *p != '\0'; p++)
+    if (*p == '\n')
+      i++;
   if (image->rows == 0)
     image->rows=(unsigned long) ((i+1)*(metrics.ascent-metrics.descent));
   SetImageBackgroundColor(image);
   /*
     Draw caption.
   */
-  (void) strcpy(draw_info->text,caption);
+  (void) CopyMagickString(draw_info->text,caption,length);
   (void) FormatMagickString(geometry,MaxTextExtent,"+%g+%g",
     metrics.max_advance/4,metrics.ascent);
   if (draw_info->gravity == UndefinedGravity)
@@ -216,8 +235,8 @@ ModuleExport void RegisterCAPTIONImage(void)
   entry=SetMagickInfo("CAPTION");
   entry->decoder=(DecoderHandler *) ReadCAPTIONImage;
   entry->adjoin=MagickFalse;
-  entry->description=AcquireString("Image caption");
-  entry->module=AcquireString("CAPTION");
+  entry->description=ConstantString("Image caption");
+  entry->module=ConstantString("CAPTION");
   (void) RegisterMagickInfo(entry);
 }
 
