@@ -20,7 +20,7 @@
 %                               October 2003                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -58,9 +58,31 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 
+/*
+  Typedef declaration.
+*/
+typedef struct _CINFileInfo
+{
+  unsigned long
+    magic,
+    image_offset,
+    generic_length,
+    industry_length,
+    user_length,
+    file_size;
+
+  char
+    version[8],
+    filename[100],
+    date[12],
+    time[12],
+    reserve[36];
+} CINFileInfo;
+
 /*
   Forward declaractions.
 */
@@ -131,6 +153,73 @@ static MagickBooleanType IsCIN(const unsigned char *magick,const size_t length)
 %
 %
 */
+
+static size_t GetBytesPerRow(unsigned long columns,
+  unsigned long samples_per_pixel,unsigned long bits_per_pixel,
+  MagickBooleanType pad)
+{
+  size_t
+    bytes_per_row;
+
+  switch (bits_per_pixel)
+  {
+    case 1:
+    {
+      bytes_per_row=4*(((size_t) samples_per_pixel*columns*
+        bits_per_pixel+31)/32);
+      break;
+    }
+    case 8:
+    default:
+    {
+      bytes_per_row=4*(((size_t) samples_per_pixel*columns*
+        bits_per_pixel+31)/32);
+      break;
+    }
+    case 10:
+    {
+      if (pad == MagickFalse)
+        {
+          bytes_per_row=4*(((size_t) samples_per_pixel*columns*
+            bits_per_pixel+31)/32);
+          break;
+        }
+      bytes_per_row=4*(((size_t) (32*((samples_per_pixel*columns+2)/3))+31)/32);
+      break;
+    }
+    case 12:
+    {
+      if (pad == MagickFalse)
+        {
+          bytes_per_row=4*(((size_t) samples_per_pixel*columns*
+            bits_per_pixel+31)/32);
+          break;
+        }
+      bytes_per_row=2*(((size_t) (16*samples_per_pixel*columns)+15)/16);
+      break;
+    }
+    case 16:
+    {
+      bytes_per_row=2*(((size_t) samples_per_pixel*columns*
+        bits_per_pixel+15)/16);
+      break;
+    }
+    case 32:
+    {
+      bytes_per_row=4*(((size_t) samples_per_pixel*columns*
+        bits_per_pixel+31)/32);
+      break;
+    }
+    case 64:
+    {
+      bytes_per_row=8*(((size_t) samples_per_pixel*columns*
+        bits_per_pixel+63)/64);
+      break;
+    }
+  }
+  return(bytes_per_row);
+}
+
 static Image *ReadCINImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
@@ -149,24 +238,30 @@ static Image *ReadCINImage(const ImageInfo *image_info,
   MagickBooleanType
     status;
 
+  QuantumType
+    quantum_type;
+
+  register const unsigned char
+    *p;
+
   register long
-    x;
+    i;
 
   register PixelPacket
     *q;
 
-  register long
-    i;
+  size_t
+    length;
 
   ssize_t
     count;
 
   unsigned char
-    colortype;
+    colortype,
+    *pixels;
 
   unsigned long
-    headersize,
-    pixel;
+    headersize;
 
   /*
     Open image file.
@@ -192,16 +287,16 @@ static Image *ReadCINImage(const ImageInfo *image_info,
   if ((count != 4) ||
       ((LocaleNCompare((char *) magick,"\200\052\137\327",4) != 0)))
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
-  headersize=ReadBlobMSBLong(image);
+  headersize=ReadBlobLong(image);
   for (i=0; i < 185; i++)
     (void) ReadBlobByte(image);
   colortype=(unsigned char) ReadBlobByte(image);
   for (i=0; i < 4; i++)
     (void) ReadBlobByte(image);
-  image->depth=ReadBlobByte(image) > 8 ? 16UL  : 8UL;
+  image->depth=(unsigned long) ReadBlobByte(image);
   (void) ReadBlobByte(image);
-  image->columns=ReadBlobMSBLong(image);
-  image->rows=ReadBlobMSBLong(image);
+  image->columns=ReadBlobLong(image);
+  image->rows=ReadBlobLong(image);
   (void) SeekBlob(image,(MagickOffsetType) headersize,SEEK_SET);
   if (image_info->ping)
     {
@@ -211,64 +306,44 @@ static Image *ReadCINImage(const ImageInfo *image_info,
   /*
     Convert CIN raster image to pixel packets.
   */
-  switch ((int) colortype)
+  quantum_type=RGBPadQuantum;
+  length=GetBytesPerRow(image->columns,3,image->depth,MagickTrue);
+  if (colortype == MonoColorType)
+    {
+      quantum_type=GrayPadQuantum;
+      length=GetBytesPerRow(image->columns,1,image->depth,MagickTrue);
+    }
+  pixels=(unsigned char *) AcquireMagickMemory(length);
+  if (pixels == (unsigned char *) NULL)
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  for (y=0; y < (long) image->rows; y++)
   {
-    case (MonoColorType):
-    {
-      q=SetImagePixels(image,0,0,image->columns,image->rows);
-      for (x=0; x < (long) ((image->columns*image->rows)/3); x++)
-      {
-        pixel=ReadBlobMSBLong(image);
-        q->red=ScaleXToQuantum((pixel >> 0) & 0x3ff,1023);
-        q->green=q->red;
-        q->blue=q->red;
-        q++;
-        q->red=ScaleXToQuantum((pixel >> 10) & 0x3ff,1023);
-        q->green=q->red;
-        q->blue=q->red;
-        q++;
-        q->red=ScaleXToQuantum((pixel >> 20) & 0x3ff,1023);
-        q->green=q->red;
-        q->blue=q->red;
-        q++;
-      }
+    q=SetImagePixels(image,0,y,image->columns,1);
+    if (q == (PixelPacket *) NULL)
       break;
-    }
-    case (RGBColorType):
-    {
-      for (y=0; y < (long) image->rows; y++)
-      {
-        q=SetImagePixels(image,0,y,image->columns,1);
-        if (q == (PixelPacket *) NULL)
-          break;
-        for (x=0; x < (long) image->columns; x++)
+    p=ReadBlobStream(image,length,pixels,&count);
+    if ((size_t) count != length)
+      break;
+    status=ExportQuantumPixels(image,quantum_type,0,p);
+    if (status == MagickFalse)
+      break;
+    if (SyncImagePixels(image) == MagickFalse)
+      break;
+    if (image->previous == (Image *) NULL)
+      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+          (QuantumTick(y,image->rows) != MagickFalse))
         {
-          pixel=ReadBlobMSBLong(image);
-          q->red=ScaleXToQuantum((pixel >> 22) & 0x3ff,1023);
-          q->green=ScaleXToQuantum((pixel >> 12) & 0x3ff,1023);
-          q->blue=ScaleXToQuantum((pixel >> 2) & 0x3ff,1023);
-          q++;
+          status=image->progress_monitor(LoadImageTag,y,image->rows,
+            image->client_data);
+          if (status == MagickFalse)
+            break;
         }
-        if (!SyncImagePixels(image))
-          break;
-        if (image->previous == (Image *) NULL)
-          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-              (QuantumTick(y,image->rows) != MagickFalse))
-            {
-              status=image->progress_monitor(LoadImageTag,y,image->rows,
-                image->client_data);
-              if (status == MagickFalse)
-                break;
-            }
-      }
-      break;
-    }
-    default:
-      ThrowReaderException(CorruptImageError,"ColorTypeNotSupported");
   }
+  pixels=(unsigned char *) RelinquishMagickMemory(pixels);
   if (EOFBlob(image))
     ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
       image->filename);
+  image->colorspace=LogColorspace;
   CloseBlob(image);
   return(GetFirstImageInList(image));
 }
@@ -304,8 +379,8 @@ ModuleExport void RegisterCINImage(void)
   entry->decoder=(DecoderHandler *) ReadCINImage;
   entry->encoder=(EncoderHandler *) WriteCINImage;
   entry->magick=(MagickHandler *) IsCIN;
-  entry->description=AcquireString("Cineon Image File");
-  entry->module=AcquireString("CIN");
+  entry->description=ConstantString("Cineon Image File");
+  entry->module=ConstantString("CIN");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -360,6 +435,9 @@ ModuleExport void UnregisterCINImage(void)
 */
 static MagickBooleanType WriteCINImage(const ImageInfo *image_info,Image *image)
 {
+  char
+    filename[MaxTextExtent];
+
   long
     y;
 
@@ -370,6 +448,9 @@ static MagickBooleanType WriteCINImage(const ImageInfo *image_info,Image *image)
     max_color,
     min_color;
 
+  QuantumType
+    quantum_type;
+
   register const PixelPacket
     *p;
 
@@ -377,8 +458,14 @@ static MagickBooleanType WriteCINImage(const ImageInfo *image_info,Image *image)
     i,
     x;
 
-  unsigned long
-    pixel;
+  size_t
+    length;
+
+  ssize_t
+    count;
+
+  unsigned char
+    *pixels;
 
   /*
     Open output image file.
@@ -392,7 +479,9 @@ static MagickBooleanType WriteCINImage(const ImageInfo *image_info,Image *image)
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  (void) SetImageColorspace(image,RGBColorspace);
+  image->depth=10;
+  if (image_info->colorspace == UndefinedColorspace)
+    (void) SetImageColorspace(image,LogColorspace);
   /*
     Compute range for each color component (10-bits).
   */
@@ -433,68 +522,69 @@ static MagickBooleanType WriteCINImage(const ImageInfo *image_info,Image *image)
   /*
     Write image header.
   */
-  (void) WriteBlobMSBLong(image,0x802A5FD7UL);  /* magick number */
-  (void) WriteBlobMSBLong(image,0x0800);  /* Offset to image */
-  (void) WriteBlobMSBLong(image,0x400);  /* Generic section header length */
-  (void) WriteBlobMSBLong(image,0x400);  /* Industry Specific lenght */
-  (void) WriteBlobMSBLong(image,0x0);  /* variable length section */
-  (void) WriteBlobMSBLong(image,4*image->columns*image->rows+0x2000);  /* total image file size */
+  (void) WriteBlobLong(image,0x802A5FD7UL);  /* magick number */
+  (void) WriteBlobLong(image,0x0800);  /* Offset to image */
+  (void) WriteBlobLong(image,0x400);  /* Generic section header length */
+  (void) WriteBlobLong(image,0x400);  /* Industry Specific lenght */
+  (void) WriteBlobLong(image,0x0);  /* variable length section */
+  (void) WriteBlobLong(image,4*image->columns*image->rows+0x2000);  /* total image file size */
   (void) WriteBlobString(image,"V4.5");  /* version */
-  (void) WriteBlobMSBLong(image,0x0);
-  (void) WriteBlobString(image,image->filename);
-  for (i=0 ; i < (long) (100-strlen(image->filename)); i++ )
-    (void) WriteBlobByte(image,0);
+  (void) WriteBlobLong(image,0x0);
+  (void) CopyMagickString(filename,image->filename,100);
+  (void) WriteBlobString(image,filename);
+  for (i=0; i < (long) (100-strlen(filename)); i++)
+    (void) WriteBlobByte(image,'\0');
   (void) WriteBlobString(image,"yyyy:mm:dd  ");  /* creation date. */
   (void) WriteBlobString(image,"hh:mm:ssxxx ");  /* creation time. */
   for (i=0 ; i < 36 ; i++)
-    (void) WriteBlobByte(image,0);
+    (void) WriteBlobByte(image,'\0');
   (void) WriteBlobByte(image,0);  /* 0 left to right top to bottom */
   (void) WriteBlobByte(image,3);  /* 3 channels */
   (void) WriteBlobByte(image,0x0);  /* alignment */
   (void) WriteBlobByte(image,0x0);  /* red */
   (void) WriteBlobByte(image,0);  /* Channel 1 designator byte 0 */
   (void) WriteBlobByte(image,1);  /* Channel 1 designator byte 1 */
-  (void) WriteBlobByte(image,10);  /* Bits per pixel... */
+  (void) WriteBlobByte(image,(unsigned char) image->depth);  /* Bits per pixel... */
   (void) WriteBlobByte(image,0);  /* alignment */
-  (void) WriteBlobMSBLong(image,image->columns);  /* pixels per line */
-  (void) WriteBlobMSBLong(image,image->rows);  /* lines per image */
-  (void) WriteBlobMSBLong(image,min_color.red);  /* Minimum data value */
-  (void) WriteBlobMSBLong(image,0x0);  /* Minimum quantity represented */
-  (void) WriteBlobMSBLong(image,max_color.red);  /* Maximum data value */
-  (void) WriteBlobMSBLong(image,0x40000000);  /* Maximum quantity represented */
+  (void) WriteBlobLong(image,image->columns);  /* pixels per line */
+  (void) WriteBlobLong(image,image->rows);  /* lines per image */
+  (void) WriteBlobLong(image,min_color.red);  /* Minimum data value */
+  (void) WriteBlobLong(image,0x0);  /* Minimum quantity represented */
+  (void) WriteBlobLong(image,max_color.red);  /* Maximum data value */
+  (void) WriteBlobLong(image,0x40000000);  /* Maximum quantity represented */
   (void) WriteBlobByte(image,0);  /* green */
   (void) WriteBlobByte(image,2);
-  (void) WriteBlobByte(image,10);
+  (void) WriteBlobByte(image,(unsigned char) image->depth);
   (void) WriteBlobByte(image,0);
-  (void) WriteBlobMSBLong(image,image->columns);
-  (void) WriteBlobMSBLong(image,image->rows);
-  (void) WriteBlobMSBLong(image,min_color.green);
-  (void) WriteBlobMSBLong(image,0x0);
-  (void) WriteBlobMSBLong(image,max_color.green);
-  (void) WriteBlobMSBLong(image,0x40000000);
+  (void) WriteBlobLong(image,image->columns);
+  (void) WriteBlobLong(image,image->rows);
+  (void) WriteBlobLong(image,min_color.green);
+  (void) WriteBlobLong(image,0x0);
+  (void) WriteBlobLong(image,max_color.green);
+  (void) WriteBlobLong(image,0x40000000);
   (void) WriteBlobByte(image,0);  /* blue */
   (void) WriteBlobByte(image,3);
-  (void) WriteBlobByte(image,10);
+  (void) WriteBlobByte(image,(unsigned char) image->depth);
   (void) WriteBlobByte(image,0);
-  (void) WriteBlobMSBLong(image,image->columns);
-  (void) WriteBlobMSBLong(image,image->rows);
-  (void) WriteBlobMSBLong(image,min_color.blue);
-  (void) WriteBlobMSBLong(image,0x0);
-  (void) WriteBlobMSBLong(image,max_color.blue);
-  (void) WriteBlobMSBLong(image,0x40000000);
+  (void) WriteBlobLong(image,image->columns);
+  (void) WriteBlobLong(image,image->rows);
+  (void) WriteBlobLong(image,min_color.blue);
+  (void) WriteBlobLong(image,0x0);
+  (void) WriteBlobLong(image,max_color.blue);
+  (void) WriteBlobLong(image,0x40000000);
   /*
 	  Pad channel 4-8.
   */
   for (i=0; i < (5*28); i++)
     (void) WriteBlobByte(image,0x00);
-  (void) WriteBlobMSBLong(image,0x4EFF0000);  /* White point(colour temp.) x */
-  (void) WriteBlobMSBLong(image,0x4EFF0000);  /* White point(colour temp.) y */
-  (void) WriteBlobMSBLong(image,0x4EFF0000);  /* Red primary chromaticity x */
-  (void) WriteBlobMSBLong(image,0x4EFF0000);  /* Red primary chromaticity y */
-  (void) WriteBlobMSBLong(image,0x4EFF0000);  /* Green primary chromaticity x */
-  (void) WriteBlobMSBLong(image,0x4EFF0000);  /* Green primary chromaticity y */
-  (void) WriteBlobMSBLong(image,0x4EFF0000);  /* Blue primary chromaticity x */
-  (void) WriteBlobMSBLong(image,0x4EFF0000);  /* Blue primary chromaticity y */
+  (void) WriteBlobLong(image,0x4EFF0000);  /* White point(colour temp.) x */
+  (void) WriteBlobLong(image,0x4EFF0000);  /* White point(colour temp.) y */
+  (void) WriteBlobLong(image,0x4EFF0000);  /* Red primary chromaticity x */
+  (void) WriteBlobLong(image,0x4EFF0000);  /* Red primary chromaticity y */
+  (void) WriteBlobLong(image,0x4EFF0000);  /* Green primary chromaticity x */
+  (void) WriteBlobLong(image,0x4EFF0000);  /* Green primary chromaticity y */
+  (void) WriteBlobLong(image,0x4EFF0000);  /* Blue primary chromaticity x */
+  (void) WriteBlobLong(image,0x4EFF0000);  /* Blue primary chromaticity y */
   /*
     Label text.
   */
@@ -504,15 +594,15 @@ static MagickBooleanType WriteCINImage(const ImageInfo *image_info,Image *image)
   (void) WriteBlobByte(image,5);  /* packing longword (32bit) boundaries */
   (void) WriteBlobByte(image,0);  /* data unsigned */
   (void) WriteBlobByte(image,0);  /* image sense: positive image */
-  (void) WriteBlobMSBLong(image,0x0);  /* end of line padding */
-  (void) WriteBlobMSBLong(image,0x0);  /* end of channel padding */
+  (void) WriteBlobLong(image,0x0);  /* end of line padding */
+  (void) WriteBlobLong(image,0x0);  /* end of channel padding */
   /*
     Reseved for future Use.
   */
   for (i=0; i < 20; i++)
     (void) WriteBlobByte(image,0x00);
-  (void) WriteBlobMSBLong(image,0x0);  /* x offset */
-  (void) WriteBlobMSBLong(image,0x0);  /* y offset */
+  (void) WriteBlobLong(image,0x0);  /* x offset */
+  (void) WriteBlobLong(image,0x0);  /* y offset */
   (void) WriteBlobString(image,image->filename);
   for (i=0 ; i < (long) (100-strlen(image->filename)); i++ )
     (void) WriteBlobByte(image,0);
@@ -533,9 +623,9 @@ static MagickBooleanType WriteCINImage(const ImageInfo *image_info,Image *image)
     (void) WriteBlobByte(image,0);
   for (i=0; i < 32; i++)
     (void) WriteBlobByte(image,0);
-  (void) WriteBlobMSBLong(image,0x4326AB85);  /* X input device pitch */
-  (void) WriteBlobMSBLong(image,0x4326AB85);  /* Y input device pitch */
-  (void) WriteBlobMSBLong(image,0x3F800000);  /* Image gamma */
+  (void) WriteBlobLong(image,0x4326AB85);  /* X input device pitch */
+  (void) WriteBlobLong(image,0x4326AB85);  /* Y input device pitch */
+  (void) WriteBlobLong(image,0x3F800000);  /* Image gamma */
   /*
     Reserved for future use.
   */
@@ -543,12 +633,12 @@ static MagickBooleanType WriteCINImage(const ImageInfo *image_info,Image *image)
     (void) WriteBlobByte(image,0);
   for (i=0 ; i < 4; i++)
     (void) WriteBlobByte(image,0);
-  (void) WriteBlobMSBLong(image,0x0);
-  (void) WriteBlobMSBLong(image,0x0);
+  (void) WriteBlobLong(image,0x0);
+  (void) WriteBlobLong(image,0x0);
   for (i=0 ; i < 32; i++)
     (void) WriteBlobByte(image,0);
-  (void) WriteBlobMSBLong(image,0x0);
-  (void) WriteBlobMSBLong(image,0x0);
+  (void) WriteBlobLong(image,0x0);
+  (void) WriteBlobLong(image,0x0);
   for (i=0 ; i < 32; i++)
     (void) WriteBlobByte(image,0);
   for (i=0 ; i < 200; i++)
@@ -558,21 +648,29 @@ static MagickBooleanType WriteCINImage(const ImageInfo *image_info,Image *image)
   /*
     Convert pixel packets to CIN raster image.
   */
+  quantum_type=RGBPadQuantum;
+  length=GetBytesPerRow(image->columns,3,image->depth,MagickTrue);
+  if (0)
+    {
+      quantum_type=GrayPadQuantum;
+      length=GetBytesPerRow(image->columns,3,image->depth,MagickTrue);
+    }
+  pixels=(unsigned char *) AcquireMagickMemory(length);
+  if (pixels == (unsigned char *) NULL)
+    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
   for (y=0; y < (long) image->rows; y++)
   {
     p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    for (x=0; x < (long) image->columns; x++)
-    {
-      pixel=(unsigned long)
-        (((long) ((1023L*p->red+QuantumRange/2)/QuantumRange) & 0x3ff) << 22) |
-        (((long) ((1023L*p->green+QuantumRange/2)/QuantumRange) &0x3ff) << 12) |
-        (((long) ((1023L*p->blue+QuantumRange/2)/QuantumRange) &0x3ff) << 2);
-      (void) WriteBlobMSBLong(image,pixel);
-      p++;
-    }
+    status=ImportQuantumPixels(image,quantum_type,0,pixels);
+    if (status == MagickFalse)
+      break;
+    count=WriteBlob(image,length,pixels);
+    if (count != (ssize_t) length)
+      break;
   }
+  pixels=(unsigned char *) RelinquishMagickMemory(pixels);
   CloseBlob(image);
   return(status);
 }
