@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -48,6 +48,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 
@@ -87,8 +88,8 @@ ModuleExport void RegisterCLIPImage(void)
 
   entry=SetMagickInfo("CLIP");
   entry->encoder=(EncoderHandler *) WriteCLIPImage;
-  entry->description=AcquireString("Image Clip Mask");
-  entry->module=AcquireString("CLIP");
+  entry->description=ConstantString("Image Clip Mask");
+  entry->module=ConstantString("CLIP");
   (void) RegisterMagickInfo(entry);
 }
 
