@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -53,6 +53,7 @@
 #include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/pixel.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/statistic.h"
 #include "magick/string_.h"
@@ -120,7 +121,7 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
     packet_size;
 
   unsigned char
-    *scanline;
+    *pixels;
 
   unsigned long
     depth,
@@ -129,7 +130,8 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
@@ -150,18 +152,18 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
         (void) ReadBlobByte(image);
     }
   /*
-    Allocate memory for a scanline.
+    Allocate memory for a pixels.
   */
-  depth=GetImageQuantumDepth(image,MagickTrue);
+  depth=GetImageQuantumDepth(image,MagickFalse);
   packet_size=(size_t) (4*depth/8);
   if (LocaleCompare(image_info->magick,"CMYKA") == 0)
     {
       packet_size+=depth/8;
       image->matte=MagickTrue;
     }
-  scanline=(unsigned char *)
-    AcquireMagickMemory(packet_size*image->extract_info.width);
-  if (scanline == (unsigned char *) NULL)
+  pixels=(unsigned char *)
+    AcquireMagickMemory(packet_size*image->extract_info.width*sizeof(*pixels));
+  if (pixels == (unsigned char *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   if (image_info->number_scenes != 0)
     while (image->scene < image_info->scene)
@@ -172,7 +174,7 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
       image->scene++;
       for (y=0; y < (long) image->rows; y++)
       {
-        count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+        count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
         if (count != (ssize_t) (packet_size*image->extract_info.width))
           ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
       }
@@ -197,16 +199,16 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
         */
         for (y=0; y < image->extract_info.y; y++)
         {
-          count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
           if (count != (ssize_t) (packet_size*image->extract_info.width))
             break;
         }
         for (y=0; y < (long) image->rows; y++)
         {
-          if ((y > 0) || (image->previous == (Image *) NULL))
+          if ((y > 0) || (GetPreviousImageInList(image) == (Image *) NULL))
             {
               count=ReadBlob(image,packet_size*image->extract_info.width,
-                scanline);
+                pixels);
               if (count != (ssize_t) (packet_size*image->extract_info.width))
                 break;
             }
@@ -214,9 +216,9 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           if (q == (PixelPacket *) NULL)
             break;
           if (image->matte == MagickFalse)
-            (void) ExportQuantumPixels(image,CMYKQuantum,0,scanline+offset);
+            (void) ExportQuantumPixels(image,CMYKQuantum,0,pixels+offset);
           else
-            (void) ExportQuantumPixels(image,CMYKAQuantum,0,scanline+offset);
+            (void) ExportQuantumPixels(image,CMYKAQuantum,0,pixels+offset);
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
@@ -232,7 +234,7 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
         width=image->extract_info.height-image->rows-image->extract_info.y;
         for (i=0; i < (long) width; i++)
         {
-          count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
           if (count != (ssize_t) (packet_size*image->extract_info.width))
             break;
         }
@@ -246,42 +248,42 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
         packet_size=(size_t) (image->depth > 8 ? 2 : 1);
         for (y=0; y < image->extract_info.y; y++)
         {
-          count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
           if (count != (ssize_t) (packet_size*image->extract_info.width))
             break;
         }
         for (y=0; y < (long) image->rows; y++)
         {
-          if ((y > 0) || (image->previous == (Image *) NULL))
+          if ((y > 0) || (GetPreviousImageInList(image) == (Image *) NULL))
             {
               count=ReadBlob(image,packet_size*image->extract_info.width,
-                scanline);
+                pixels);
               if (count != (ssize_t) (packet_size*image->extract_info.width))
                 break;
             }
           q=SetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          (void) ExportQuantumPixels(image,CyanQuantum,0,scanline+offset);
-          count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          (void) ExportQuantumPixels(image,CyanQuantum,0,pixels+offset);
+          count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
           if (count != (ssize_t) (packet_size*image->extract_info.width))
             break;
-          (void) ExportQuantumPixels(image,MagentaQuantum,0,scanline+offset);
-          count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          (void) ExportQuantumPixels(image,MagentaQuantum,0,pixels+offset);
+          count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
           if (count != (ssize_t) (packet_size*image->extract_info.width))
             break;
-          (void) ExportQuantumPixels(image,YellowQuantum,0,scanline+offset);
-          count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          (void) ExportQuantumPixels(image,YellowQuantum,0,pixels+offset);
+          count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
           if (count != (ssize_t) (packet_size*image->extract_info.width))
             break;
-          (void) ExportQuantumPixels(image,BlackQuantum,0,scanline+offset);
+          (void) ExportQuantumPixels(image,BlackQuantum,0,pixels+offset);
           if (image->matte != MagickFalse)
             {
               count=ReadBlob(image,packet_size*image->extract_info.width,
-                scanline);
+                pixels);
               if (count != (ssize_t) (packet_size*image->extract_info.width))
                 break;
-              (void) ExportQuantumPixels(image,AlphaQuantum,0,scanline+offset);
+              (void) ExportQuantumPixels(image,AlphaQuantum,0,pixels+offset);
             }
           if (SyncImagePixels(image) == MagickFalse)
             break;
@@ -298,7 +300,7 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
         width=image->extract_info.height-image->rows-image->extract_info.y;
         for (i=0; i < (long) width; i++)
         {
-          count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
           if (count != (ssize_t) (packet_size*image->extract_info.width))
             break;
         }
@@ -326,7 +328,7 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
         packet_size=(size_t) (image->depth > 8 ? 2 : 1);
         for (y=0; y < image->extract_info.y; y++)
         {
-          count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
           if (count != (ssize_t) (packet_size*image->extract_info.width))
             break;
         }
@@ -334,17 +336,17 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
         span=image->rows*(image->matte != MagickFalse ? 5 : 4);
         for (y=0; y < (long) image->rows; y++)
         {
-          if ((y > 0) || (image->previous == (Image *) NULL))
+          if ((y > 0) || (GetPreviousImageInList(image) == (Image *) NULL))
             {
               count=ReadBlob(image,packet_size*image->extract_info.width,
-                scanline);
+                pixels);
               if (count != (ssize_t) (packet_size*image->extract_info.width))
                 break;
             }
           q=SetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          (void) ExportQuantumPixels(image,CyanQuantum,0,scanline+offset);
+          (void) ExportQuantumPixels(image,CyanQuantum,0,pixels+offset);
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
@@ -361,7 +363,7 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
         width=image->extract_info.height-image->rows-image->extract_info.y;
         for (i=0; i < (long) width; i++)
         {
-          count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
           if (count != (ssize_t) (packet_size*image->extract_info.width))
             break;
         }
@@ -378,19 +380,19 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           }
         for (y=0; y < image->extract_info.y; y++)
         {
-          count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
           if (count != (ssize_t) (packet_size*image->extract_info.width))
             break;
         }
         for (y=0; y < (long) image->rows; y++)
         {
-          count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
           if (count != (ssize_t) (packet_size*image->extract_info.width))
             break;
           q=GetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          (void) ExportQuantumPixels(image,MagentaQuantum,0,scanline+offset);
+          (void) ExportQuantumPixels(image,MagentaQuantum,0,pixels+offset);
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
@@ -407,7 +409,7 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
         width=image->extract_info.height-image->rows-image->extract_info.y;
         for (i=0; i < (long) width; i++)
         {
-          count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
           if (count != (ssize_t) (packet_size*image->extract_info.width))
             break;
         }
@@ -424,19 +426,19 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           }
         for (y=0; y < image->extract_info.y; y++)
         {
-          count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
           if (count != (ssize_t) (packet_size*image->extract_info.width))
             break;
         }
         for (y=0; y < (long) image->rows; y++)
         {
-          count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
           if (count != (ssize_t) (packet_size*image->extract_info.width))
             break;
           q=GetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          (void) ExportQuantumPixels(image,YellowQuantum,0,scanline+offset);
+          (void) ExportQuantumPixels(image,YellowQuantum,0,pixels+offset);
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
@@ -453,7 +455,7 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
         width=image->extract_info.height-image->rows-image->extract_info.y;
         for (i=0; i < (long) width; i++)
         {
-          count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
           if (count != (ssize_t) (packet_size*image->extract_info.width))
             break;
         }
@@ -470,19 +472,19 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           }
         for (y=0; y < image->extract_info.y; y++)
         {
-          count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
           if (count != (ssize_t) (packet_size*image->extract_info.width))
             break;
         }
         for (y=0; y < (long) image->rows; y++)
         {
-          count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
           if (count != (ssize_t) (packet_size*image->extract_info.width))
             break;
           q=GetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          (void) ExportQuantumPixels(image,BlackQuantum,0,scanline+offset);
+          (void) ExportQuantumPixels(image,BlackQuantum,0,pixels+offset);
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
@@ -499,7 +501,7 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
         width=image->extract_info.height-image->rows-image->extract_info.y;
         for (i=0; i < (long) width; i++)
         {
-          count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
           if (count != (ssize_t) (packet_size*image->extract_info.width))
             break;
         }
@@ -522,20 +524,20 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
             for (y=0; y < image->extract_info.y; y++)
             {
               count=ReadBlob(image,packet_size*image->extract_info.width,
-                scanline);
+                pixels);
               if (count != (ssize_t) (packet_size*image->extract_info.width))
                 break;
             }
             for (y=0; y < (long) image->rows; y++)
             {
               count=ReadBlob(image,packet_size*image->extract_info.width,
-                scanline);
+                pixels);
               if (count != (ssize_t) (packet_size*image->extract_info.width))
                 break;
               q=GetImagePixels(image,0,y,image->columns,1);
               if (q == (PixelPacket *) NULL)
                 break;
-              (void) ExportQuantumPixels(image,AlphaQuantum,0,scanline+offset);
+              (void) ExportQuantumPixels(image,AlphaQuantum,0,pixels+offset);
               if (SyncImagePixels(image) == MagickFalse)
                 break;
               if (image->previous == (Image *) NULL)
@@ -553,7 +555,7 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
             for (i=0; i < (long) width; i++)
             {
               count=ReadBlob(image,packet_size*image->extract_info.width,
-                scanline);
+                pixels);
               if (count != (ssize_t) (packet_size*image->extract_info.width))
                 break;
             }
@@ -574,14 +576,14 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
         break;
     if (image_info->interlace == PartitionInterlace)
       break;
-    count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+    count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
     if (count == (ssize_t) (packet_size*image->extract_info.width))
       {
         /*
           Allocate next image structure.
         */
         AllocateNextImage(image_info,image);
-        if (image->next == (Image *) NULL)
+        if (GetNextImageInList(image) == (Image *) NULL)
           {
             image=DestroyImageList(image);
             return((Image *) NULL);
@@ -596,7 +598,7 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           }
       }
   } while (count == (ssize_t) (packet_size*image->extract_info.width));
-  scanline=(unsigned char *) RelinquishMagickMemory(scanline);
+  pixels=(unsigned char *) RelinquishMagickMemory(pixels);
   CloseBlob(image);
   return(GetFirstImageInList(image));
 }
@@ -634,18 +636,18 @@ ModuleExport void RegisterCMYKImage(void)
   entry->encoder=(EncoderHandler *) WriteCMYKImage;
   entry->raw=MagickTrue;
   entry->endian_support=MagickTrue;
-  entry->description=
-    AcquireString("Raw cyan, magenta, yellow, and black samples");
-  entry->module=AcquireString("CMYK");
+  entry->description=ConstantString(
+    "Raw cyan, magenta, yellow, and black samples");
+  entry->module=ConstantString("CMYK");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("CMYKA");
   entry->decoder=(DecoderHandler *) ReadCMYKImage;
   entry->encoder=(EncoderHandler *) WriteCMYKImage;
   entry->raw=MagickTrue;
   entry->endian_support=MagickTrue;
-  entry->description=
-    AcquireString("Raw cyan, magenta, yellow, black, and opacity samples");
-  entry->module=AcquireString("CMYK");
+  entry->description=ConstantString(
+    "Raw cyan, magenta, yellow, black, and opacity samples");
+  entry->module=ConstantString("CMYK");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -733,12 +735,13 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
-  depth=GetImageQuantumDepth(image,MagickTrue);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  depth=GetImageQuantumDepth(image,MagickFalse);
   packet_size=(size_t) (4*depth/8);
   if (LocaleCompare(image_info->magick,"CMYKA") == 0)
     packet_size+=depth/8;
-  pixels=(unsigned char *) AcquireMagickMemory(packet_size*image->columns);
+  pixels=(unsigned char *)
+    AcquireMagickMemory(packet_size*image->columns*sizeof(*pixels));
   if (pixels == (unsigned char *) NULL)
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
   if (image_info->interlace != PartitionInterlace)
@@ -756,7 +759,8 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
     /*
       Convert MIFF to CMYK raster pixels.
     */
-    (void) SetImageColorspace(image,CMYKColorspace);
+    if (image_info->colorspace == UndefinedColorspace)
+      (void) SetImageColorspace(image,CMYKColorspace);
     if (LocaleCompare(image_info->magick,"CMYKA") == 0)
       if (image->matte == MagickFalse)
         SetImageOpacity(image,OpaqueOpacity);
@@ -964,7 +968,7 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
         break;
       }
     }
-    if (image->next == (Image *) NULL)
+    if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
