@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -56,6 +56,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/transform.h"
@@ -167,7 +168,7 @@ static MagickBooleanType DecodeImage(Image *image,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(pixels != (unsigned char *) NULL);
   (void) ResetMagickMemory(pixels,0,(size_t) image->columns*image->rows*
     sizeof(*pixels));
@@ -337,7 +338,7 @@ static size_t EncodeImage(Image *image,const unsigned long bytes_per_line,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(pixels != (const unsigned char *) NULL);
   assert(compressed_pixels != (unsigned char *) NULL);
   p=pixels;
@@ -493,7 +494,8 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
@@ -897,9 +899,9 @@ ModuleExport void RegisterDIBImage(void)
   entry->magick=(MagickHandler *) IsDIB;
   entry->adjoin=MagickFalse;
   entry->stealth=MagickTrue;
-  entry->description=
-    AcquireString("Microsoft Windows 3.X Packed Device-Independent Bitmap");
-  entry->module=AcquireString("DIB");
+  entry->description=ConstantString(
+    "Microsoft Windows 3.X Packed Device-Independent Bitmap");
+  entry->module=ConstantString("DIB");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -992,14 +994,15 @@ static MagickBooleanType WriteDIBImage(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
   /*
     Initialize DIB raster file header.
   */
-  (void) SetImageColorspace(image,RGBColorspace);
+  if (image_info->colorspace == UndefinedColorspace)
+    (void) SetImageColorspace(image,RGBColorspace);
   if (image->storage_class == DirectClass)
     {
       /*
