@@ -16,13 +16,13 @@
 %                              Bill Radcliffe                                 %
 %                                   2001                                      %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -57,6 +57,7 @@
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 /*
@@ -529,20 +530,20 @@ ModuleExport void RegisterEMFImage(void)
 #if defined(HasWINGDI32)
   entry->decoder=ReadEMFImage;
 #endif
-  entry->description=
-    AcquireString("Windows WIN32 API rendered Enhanced Meta File");
+  entry->description=ConstantString(
+    "Windows WIN32 API rendered Enhanced Meta File");
   entry->magick=(MagickHandler *) IsEMF;
   entry->blob_support=MagickFalse;
-  entry->module=AcquireString("WMF");
+  entry->module=ConstantString("WMF");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("WMFWIN32");
 #if defined(HasWINGDI32)
   entry->decoder=ReadEMFImage;
 #endif
-  entry->description=AcquireString("Windows WIN32 API rendered Meta File");
+  entry->description=ConstantString("Windows WIN32 API rendered Meta File");
   entry->magick=(MagickHandler *) IsWMF;
   entry->blob_support=MagickFalse;
-  entry->module=AcquireString("WMFWIN32");
+  entry->module=ConstantString("WMFWIN32");
   (void) RegisterMagickInfo(entry);
 }
 
