@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -52,6 +52,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 
@@ -138,7 +139,8 @@ static Image *ReadFAXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
@@ -162,9 +164,9 @@ static Image *ReadFAXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Monochrome colormap.
   */
-  image->colormap[0].red=MaxRGB;
-  image->colormap[0].green=MaxRGB;
-  image->colormap[0].blue=MaxRGB;
+  image->colormap[0].red=QuantumRange;
+  image->colormap[0].green=QuantumRange;
+  image->colormap[0].blue=QuantumRange;
   image->colormap[1].red=0;
   image->colormap[1].green=0;
   image->colormap[1].blue=0;
@@ -224,17 +226,17 @@ ModuleExport void RegisterFAXImage(void)
   entry->decoder=(DecoderHandler *) ReadFAXImage;
   entry->encoder=(EncoderHandler *) WriteFAXImage;
   entry->magick=(MagickHandler *) IsFAX;
-  entry->description=AcquireString("Group 3 FAX");
-  entry->note=AcquireString(Note);
-  entry->module=AcquireString("FAX");
+  entry->description=ConstantString("Group 3 FAX");
+  entry->note=ConstantString(Note);
+  entry->module=ConstantString("FAX");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("G3");
   entry->decoder=(DecoderHandler *) ReadFAXImage;
   entry->encoder=(EncoderHandler *) WriteFAXImage;
   entry->magick=(MagickHandler *) IsFAX;
   entry->adjoin=MagickFalse;
-  entry->description=AcquireString("Group 3 FAX");
-  entry->module=AcquireString("FAX");
+  entry->description=ConstantString("Group 3 FAX");
+  entry->module=ConstantString("FAX");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -308,21 +310,22 @@ static MagickBooleanType WriteFAXImage(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
   write_info=CloneImageInfo(image_info);
-  (void) strcpy(write_info->magick,"FAX");
+  (void) CopyMagickString(write_info->magick,"FAX",MaxTextExtent);
   scene=0;
   do
   {
     /*
       Convert MIFF to monochrome.
     */
-    (void) SetImageColorspace(image,RGBColorspace);
+    if (image_info->colorspace == UndefinedColorspace)
+      (void) SetImageColorspace(image,RGBColorspace);
     status=HuffmanEncodeImage(write_info,image);
-    if (image->next == (Image *) NULL)
+    if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
