@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -55,6 +55,7 @@
 #include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/pixel.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/statistic.h"
 #include "magick/string_.h"
@@ -90,7 +91,6 @@ static MagickBooleanType
 %
 %    o length: Specifies the length of the magick string.
 %
-%
 */
 static MagickBooleanType IsFITS(const unsigned char *magick,const size_t length)
 {
@@ -133,7 +133,6 @@ static MagickBooleanType IsFITS(const unsigned char *magick,const size_t length)
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
 static Image *ReadFITSImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
@@ -242,8 +241,9 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
   fits_info.bits_per_pixel=8;
   fits_info.columns=1;
   fits_info.rows=1;
+  fits_info.rows=1;
   fits_info.number_scenes=1;
-  fits_info.number_axes=1;
+  fits_info.number_axes=0;
   fits_info.min_data=0.0;
   fits_info.max_data=0.0;
   fits_info.zero=0.0;
@@ -257,7 +257,7 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
       image=DestroyImage(image);
       return((Image *) NULL);
     }
-  for ( ; ; )
+  for ( ; c != EOF; )
   {
     if (isalnum((int) ((unsigned char) c)) == 0)
       c=ReadBlobByte(image);
@@ -272,7 +272,7 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
         p=keyword;
         do
         {
-          if ((size_t) (p-keyword) < (MaxTextExtent-1))
+          if ((size_t) (p-keyword) < (MaxTextExtent/2))
             *p++=(char) c;
           c=ReadBlobByte(image);
         } while ((isalnum((int) ((unsigned char) c)) != MagickFalse) ||
@@ -292,7 +292,7 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
         p=value;
         while (isalnum(c) || (c == '-') || (c == '+') || (c == '.'))
         {
-          if ((size_t) (p-value) < (MaxTextExtent-1))
+          if ((size_t) (p-value) < (MaxTextExtent/2))
             *p++=c;
           c=ReadBlobByte(image);
         }
@@ -322,11 +322,11 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
           fits_info.scale=atof(value);
         (void) SetImageAttribute(image,keyword,value);
       }
-    while ((TellBlob(image) % 80) != 0)
+    while (((TellBlob(image) % 80) != 0) && (c != EOF))
       c=ReadBlobByte(image);
     c=ReadBlobByte(image);
   }
-  while ((TellBlob(image) % 2880) != 0)
+  while (((TellBlob(image) % 2880) != 0) && (c != EOF))
     c=ReadBlobByte(image);
   /*
     Verify that required image information is defined.
@@ -485,6 +485,9 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
           if (scaled_pixel > fits_info.max_data)
             fits_info.max_data=scaled_pixel;
         }
+        if (fits_info.bits_per_pixel == 16)
+          if (pixel > 32767)
+            pixel-=65536L;
       }
     /*
       Convert FITS pixels to pixel packets.
@@ -546,6 +549,9 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
           }
         scaled_pixel=scale*
           (pixel*fits_info.scale-fits_info.min_data-fits_info.zero);
+        if (fits_info.bits_per_pixel == 16)
+          if (scaled_pixel > 32767)
+            scaled_pixel-=65536L;
         index=(IndexPacket) ((scaled_pixel < 0.0) ? 0 :
           ((unsigned long) scaled_pixel > ((1UL << image->depth)-1)) ?
           ((1UL << image->depth)-1) : (unsigned long) (scaled_pixel+0.5));
@@ -635,8 +641,8 @@ ModuleExport void RegisterFITSImage(void)
   entry->encoder=(EncoderHandler *) WriteFITSImage;
   entry->magick=(MagickHandler *) IsFITS;
   entry->adjoin=MagickFalse;
-  entry->description=AcquireString("Flexible Image Transport System");
-  entry->module=AcquireString("FITS");
+  entry->description=ConstantString("Flexible Image Transport System");
+  entry->module=ConstantString("FITS");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -695,7 +701,7 @@ static MagickBooleanType WriteFITSImage(const ImageInfo *image_info,
   Image *image)
 {
   char
-    buffer[81],
+    buffer[MaxTextExtent],
     *fits_info;
 
   long
@@ -731,7 +737,8 @@ static MagickBooleanType WriteFITSImage(const ImageInfo *image_info,
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  (void) SetImageColorspace(image,RGBColorspace);
+  if (image_info->colorspace == UndefinedColorspace)
+    (void) SetImageColorspace(image,RGBColorspace);
   /*
     Allocate image memory.
   */
@@ -747,31 +754,34 @@ static MagickBooleanType WriteFITSImage(const ImageInfo *image_info,
   */
   for (i=0; i < 2880; i++)
     fits_info[i]=' ';
-  (void) strcpy(buffer,"SIMPLE  =                    T");
-  (void) strncpy(fits_info+0,buffer,strlen(buffer));
+  (void) CopyMagickString(buffer,"SIMPLE  =                    T",
+    MaxTextExtent);
+  (void) CopyMagickString(fits_info+0,buffer,2880);
   (void) FormatMagickString(buffer,MaxTextExtent,"BITPIX  =           %10lu",
     image->depth);
-  (void) strncpy(fits_info+80,buffer,strlen(buffer));
-  (void) strcpy(buffer,"NAXIS   =                    2");
-  (void) strncpy(fits_info+160,buffer,strlen(buffer));
+  (void) CopyMagickString(fits_info+80,buffer,2880);
+  (void) CopyMagickString(buffer,"NAXIS   =                    2",
+    MaxTextExtent);
+  (void) CopyMagickString(fits_info+160,buffer,2880);
   (void) FormatMagickString(buffer,MaxTextExtent,"NAXIS1  =           %10lu",
     image->columns);
-  (void) strncpy(fits_info+240,buffer,strlen(buffer));
+  (void) CopyMagickString(fits_info+240,buffer,2880);
   (void) FormatMagickString(buffer,MaxTextExtent,"NAXIS2  =           %10lu",
     image->rows);
-  (void) strncpy(fits_info+320,buffer,strlen(buffer));
+  (void) CopyMagickString(fits_info+320,buffer,2880);
   (void) FormatMagickString(buffer,MaxTextExtent,"DATAMIN =           %10u",0);
-  (void) strncpy(fits_info+400,buffer,strlen(buffer));
+  (void) CopyMagickString(fits_info+400,buffer,2880);
   (void) FormatMagickString(buffer,MaxTextExtent,"DATAMAX =           %10u",
     (1 << image->depth)-1);
-  (void) strncpy(fits_info+480,buffer,strlen(buffer));
+  (void) CopyMagickString(fits_info+480,buffer,2880);
   (void) FormatMagickString(buffer,MaxTextExtent,"BZERO   =           %10u",
     image->depth > 8 ? 32768 : 0);
-  (void) strncpy(fits_info+560,buffer,strlen(buffer));
-  (void) strcpy(buffer,"HISTORY Created by ImageMagick.");
-  (void) strncpy(fits_info+640,buffer,strlen(buffer));
-  (void) strcpy(buffer,"END");
-  (void) strncpy(fits_info+720,buffer,strlen(buffer));
+  (void) CopyMagickString(fits_info+560,buffer,2880);
+  (void) CopyMagickString(buffer,"HISTORY Created by ImageMagick.",
+    MaxTextExtent);
+  (void) CopyMagickString(fits_info+640,buffer,2880);
+  (void) CopyMagickString(buffer,"END",MaxTextExtent);
+  (void) CopyMagickString(fits_info+720,buffer,2880);
   (void) WriteBlob(image,2880,(unsigned char *) fits_info);
   /*
     Convert image to fits scale PseudoColor class.
@@ -794,7 +804,8 @@ static MagickBooleanType WriteFITSImage(const ImageInfo *image_info,
   }
   (void) ResetMagickMemory(fits_info,0,2880*sizeof(*fits_info));
   length=packet_size*image->columns*image->rows;
-  (void) WriteBlob(image,2880-(length % 2880),(unsigned char *) fits_info);
+  (void) WriteBlob(image,2880-((size_t) length % 2880),(unsigned char *)
+    fits_info);
   fits_info=(char *) RelinquishMagickMemory(fits_info);
   pixels=(unsigned char *) RelinquishMagickMemory(pixels);
   CloseBlob(image);
