@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -57,6 +57,7 @@
 #include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/pixel.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #if defined(HasFPX)
@@ -207,7 +208,8 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
@@ -432,7 +434,7 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
             (long) tile_height,&fpx_info);
         if (fpx_status == FPX_LOW_MEMORY_ERROR)
           {
-            scanline=(char *) RelinquishMagickMemory(scanline);
+            scanline=(unsigned char *) RelinquishMagickMemory(scanline);
             (void) FPX_CloseImage(flashpix);
             FPX_ClearSystem();
             ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
@@ -480,7 +482,7 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
           break;
       }
   }
-  scanline=(char *) RelinquishMagickMemory(scanline);
+  scanline=(unsigned char *) RelinquishMagickMemory(scanline);
   (void) FPX_CloseImage(flashpix);
   FPX_ClearSystem();
   return(GetFirstImageInList(image));
@@ -524,8 +526,8 @@ ModuleExport void RegisterFPXImage(void)
   entry->seekable_stream=MagickTrue;
   entry->blob_support=MagickFalse;
   entry->magick=(MagickHandler *) IsFPX;
-  entry->description=AcquireString("FlashPix Format");
-  entry->module=AcquireString("FPX");
+  entry->description=ConstantString("FlashPix Format");
+  entry->module=ConstantString("FPX");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -826,7 +828,7 @@ static MagickBooleanType WriteFPXImage(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
@@ -835,7 +837,8 @@ static MagickBooleanType WriteFPXImage(const ImageInfo *image_info,Image *image)
     Initialize FPX toolkit.
   */
   image->depth=8;
-  (void) SetImageColorspace(image,RGBColorspace);
+  if (image_info->colorspace == UndefinedColorspace)
+    (void) SetImageColorspace(image,RGBColorspace);
   memory_limit=20000000;
   fpx_status=FPX_SetToolkitMemoryLimit(&memory_limit);
   if (fpx_status != FPX_OK)
@@ -926,12 +929,8 @@ static MagickBooleanType WriteFPXImage(const ImageInfo *image_info,Image *image)
         Note image comment.
       */
       summary_info.comments_valid=MagickTrue;
+      summary_info.comments.ptr=(unsigned char *) AcquireString(comment->value);
       summary_info.comments.length=strlen(comment->value);
-      summary_info.comments.ptr=(unsigned char *)
-        AcquireMagickMemory(strlen(comment->value)+MaxTextExtent);
-      if (summary_info.comments.ptr == (unsigned char *) NULL)
-        ThrowWriterException(DelegateError,"UnableToSetImageComment");
-      (void) strcpy((char *) summary_info.comments.ptr,comment->value);
     }
   fpx_status=FPX_SetSummaryInformation(flashpix,&summary_info);
   if (fpx_status != FPX_OK)
@@ -939,8 +938,8 @@ static MagickBooleanType WriteFPXImage(const ImageInfo *image_info,Image *image)
   /*
     Allocate pixels.
   */
-  pixels=(unsigned char *)
-    AcquireMagickMemory(colorspace.numberOfComponents*image->columns);
+  pixels=(unsigned char *) AcquireMagickMemory((colorspace.numberOfComponents*
+    image->columns)*sizeof(*pixels));
   if (pixels == (unsigned char *) NULL)
     {
       (void) FPX_CloseImage(flashpix);
