@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -55,6 +55,7 @@
 #include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/quantize.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 
@@ -156,12 +157,12 @@ static MagickBooleanType DecodeImage(Image *image,const Quantum opacity)
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  packet=(unsigned char *) AcquireMagickMemory(256);
+  packet=(unsigned char *) AcquireMagickMemory(256*sizeof(*packet));
   prefix=(short *) AcquireMagickMemory(MaxStackSize*sizeof(*prefix));
-  suffix=(unsigned char *) AcquireMagickMemory(MaxStackSize);
-  pixel_stack=(unsigned char *) AcquireMagickMemory(MaxStackSize+1);
-  if ((packet == (unsigned char *) NULL) ||
-      (prefix == (short *) NULL) ||
+  suffix=(unsigned char *) AcquireMagickMemory(MaxStackSize*sizeof(*suffix));
+  pixel_stack=(unsigned char *) AcquireMagickMemory((MaxStackSize+1)*
+    sizeof(*pixel_stack));
+  if ((packet == (unsigned char *) NULL) || (prefix == (short *) NULL) ||
       (suffix == (unsigned char *) NULL) ||
       (pixel_stack == (unsigned char *) NULL))
     {
@@ -303,8 +304,8 @@ static MagickBooleanType DecodeImage(Image *image,const Quantum opacity)
       index=ConstrainColormapIndex(image,*top_stack);
       indexes[x]=index;
       *q=image->colormap[index];
-      q->opacity=(Quantum)
-        (index == opacity ? TransparentOpacity : OpaqueOpacity);
+      q->opacity=(Quantum) (index == opacity ? TransparentOpacity :
+        OpaqueOpacity);
       x++;
       q++;
     }
@@ -492,10 +493,11 @@ static MagickBooleanType EncodeImage(const ImageInfo *image_info,Image *image,
     Allocate encoder tables.
   */
   assert(image != (Image *) NULL);
-  packet=(unsigned char *) AcquireMagickMemory(256);
+  packet=(unsigned char *) AcquireMagickMemory(256*sizeof(*packet));
   hash_code=(short *) AcquireMagickMemory(MaxHashTable*sizeof(*hash_code));
   hash_prefix=(short *) AcquireMagickMemory(MaxHashTable*sizeof(*hash_prefix));
-  hash_suffix=(unsigned char *) AcquireMagickMemory(MaxHashTable);
+  hash_suffix=(unsigned char *) AcquireMagickMemory(MaxHashTable*
+    sizeof(*hash_suffix));
   if ((packet == (unsigned char *) NULL) || (hash_code == (short *) NULL) ||
       (hash_prefix == (short *) NULL) ||
       (hash_suffix == (unsigned char *) NULL))
@@ -869,8 +871,8 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   background=(unsigned char) ReadBlobByte(image);
   c=(unsigned char) ReadBlobByte(image);  /* reserved */
   global_colors=1 << (((unsigned long) flag & 0x07)+1);
-  global_colormap=(unsigned char *)
-    AcquireMagickMemory((size_t) (3*Max(global_colors,256)));
+  global_colormap=(unsigned char *) AcquireMagickMemory((size_t)
+    (3*Max(global_colors,256))*sizeof(*global_colormap));
   if (global_colormap == (unsigned char *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   if (BitSet((int) flag,0x80) != 0)
@@ -894,7 +896,12 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         count=ReadBlob(image,1,&c);
         if (count != 1)
-          ThrowReaderException(CorruptImageError,"UnableToReadExtensionBlock");
+          {
+            global_colormap=(unsigned char *)
+              RelinquishMagickMemory(global_colormap);
+            ThrowReaderException(CorruptImageError,
+              "UnableToReadExtensionBlock");
+          }
         switch (c)
         {
           case 0xf9:
@@ -940,8 +947,8 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
             */
             loop=MagickFalse;
             if (ReadBlobBlock(image,header) != 0)
-              loop=(MagickBooleanType)
-                (LocaleNCompare((char *) header,"NETSCAPE2.0",11) == 0);
+              loop=LocaleNCompare((char *) header,"NETSCAPE2.0",11) == 0 ?
+                MagickTrue : MagickFalse;
             while (ReadBlobBlock(image,header) != 0)
             if (loop != MagickFalse)
               iterations=(unsigned long) ((header[2] << 8) | header[1]);
@@ -1001,19 +1008,28 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     image->page.y=page.y;
     image->page.x=page.x;
     image->delay=delay;
+    image->ticks_per_second=100;
     image->dispose=(DisposeType) dispose;
     image->iterations=iterations;
-    image->matte=(MagickBooleanType) (opacity >= 0);
+    image->matte=opacity >= 0 ? MagickTrue : MagickFalse;
     delay=0;
     dispose=0;
     iterations=1;
     if ((image->columns == 0) || (image->rows == 0))
-      ThrowReaderException(CorruptImageError,"NegativeOrZeroImageSize");
+      {
+        global_colormap=(unsigned char *)
+          RelinquishMagickMemory(global_colormap);
+        ThrowReaderException(CorruptImageError,"NegativeOrZeroImageSize");
+      }
     /*
       Inititialize colormap.
     */
     if (AllocateImageColormap(image,image->colors) == MagickFalse)
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+      {
+        global_colormap=(unsigned char *)
+          RelinquishMagickMemory(global_colormap);
+        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+      }
     if (BitSet((int) flag,0x80) == 0)
       {
         /*
@@ -1025,6 +1041,8 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
           image->colormap[i].red=ScaleCharToQuantum(*p++);
           image->colormap[i].green=ScaleCharToQuantum(*p++);
           image->colormap[i].blue=ScaleCharToQuantum(*p++);
+          if (i == opacity)
+            image->colormap[i].opacity=TransparentOpacity;
         }
         image->background_color=
           image->colormap[Min((unsigned long) background,image->colors-1)];
@@ -1038,18 +1056,30 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
           Read local colormap.
         */
         colormap=(unsigned char *)
-          AcquireMagickMemory((size_t) (3*image->colors));
+          AcquireMagickMemory((3*image->colors)*sizeof(*colormap));
         if (colormap == (unsigned char *) NULL)
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-        count=ReadBlob(image,(size_t) (3*image->colors),colormap);
+          {
+            global_colormap=(unsigned char *)
+              RelinquishMagickMemory(global_colormap);
+            ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+          }
+        count=ReadBlob(image,(3*image->colors)*sizeof(*colormap),colormap);
         if (count != (ssize_t) (3*image->colors))
-          ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
+          {
+            global_colormap=(unsigned char *)
+              RelinquishMagickMemory(global_colormap);
+            colormap=(unsigned char *) RelinquishMagickMemory(colormap);
+            ThrowReaderException(CorruptImageError,
+              "InsufficientImageDataInFile");
+          }
         p=colormap;
         for (i=0; i < (long) image->colors; i++)
         {
           image->colormap[i].red=ScaleCharToQuantum(*p++);
           image->colormap[i].green=ScaleCharToQuantum(*p++);
           image->colormap[i].blue=ScaleCharToQuantum(*p++);
+          if (i == opacity)
+            image->colormap[i].opacity=TransparentOpacity;
         }
         colormap=(unsigned char *) RelinquishMagickMemory(colormap);
       }
@@ -1110,17 +1140,17 @@ ModuleExport void RegisterGIFImage(void)
   entry->decoder=(DecoderHandler *) ReadGIFImage;
   entry->encoder=(EncoderHandler *) WriteGIFImage;
   entry->magick=(MagickHandler *) IsGIF;
-  entry->description=AcquireString("CompuServe graphics interchange format");
-  entry->module=AcquireString("GIF");
+  entry->description=ConstantString("CompuServe graphics interchange format");
+  entry->module=ConstantString("GIF");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("GIF87");
   entry->decoder=(DecoderHandler *) ReadGIFImage;
   entry->encoder=(EncoderHandler *) WriteGIFImage;
   entry->magick=(MagickHandler *) IsGIF;
   entry->adjoin=MagickFalse;
-  entry->description=AcquireString("CompuServe graphics interchange format");
-  entry->version=AcquireString("version 87a");
-  entry->module=AcquireString("GIF");
+  entry->description=ConstantString("CompuServe graphics interchange format");
+  entry->version=ConstantString("version 87a");
+  entry->module=ConstantString("GIF");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -1229,7 +1259,8 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
     *global_colormap;
 
   unsigned long
-    bits_per_pixel;
+    bits_per_pixel,
+    delay;
 
   /*
     Open output image file.
@@ -1243,28 +1274,12 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  /*
-    Determine image bounding box.
-  */
-  page.width=image->columns;
-  page.height=image->rows;
-  page.x=0;
-  page.y=0;
-  for (next_image=image; next_image != (Image *) NULL; )
-  {
-    page.x=next_image->page.x;
-    page.y=next_image->page.y;
-    if ((next_image->columns+page.x) > page.width)
-      page.width=next_image->columns+page.x;
-    if ((next_image->rows+page.y) > page.height)
-      page.height=next_image->rows+page.y;
-    next_image=GetNextImageInList(next_image);
-  }
   /*
     Allocate colormap.
   */
-  global_colormap=(unsigned char *) AcquireMagickMemory(768);
-  colormap=(unsigned char *) AcquireMagickMemory(768);
+  global_colormap=(unsigned char *)
+    AcquireMagickMemory(768*sizeof(*global_colormap));
+  colormap=(unsigned char *) AcquireMagickMemory(768*sizeof(*colormap));
   if ((global_colormap == (unsigned char *) NULL) ||
       (colormap == (unsigned char *) NULL))
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
@@ -1281,6 +1296,24 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
       (void) WriteBlob(image,6,(unsigned char *) "GIF87a");
       write_info->adjoin=MagickFalse;
     }
+  /*
+    Determine image bounding box.
+  */
+  page.width=image->columns;
+  page.height=image->rows;
+  page.x=0;
+  page.y=0;
+  if (write_info->adjoin != MagickFalse)
+    for (next_image=image; next_image != (Image *) NULL; )
+    {
+      page.x=next_image->page.x;
+      page.y=next_image->page.y;
+      if ((next_image->columns+page.x) > page.width)
+        page.width=next_image->columns+page.x;
+      if ((next_image->rows+page.y) > page.height)
+        page.height=next_image->rows+page.y;
+      next_image=GetNextImageInList(next_image);
+    }
   page.x=image->page.x;
   page.y=image->page.y;
   if ((image->page.width != 0) && (image->page.height != 0))
@@ -1291,13 +1324,58 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
     Write images to file.
   */
   interlace=write_info->interlace;
-  if ((write_info->adjoin != MagickFalse) && (GetNextImageInList(image) != (Image *) NULL))
+  if ((write_info->adjoin != MagickFalse) &&
+      (GetNextImageInList(image) != (Image *) NULL))
     interlace=NoInterlace;
   scene=0;
   do
   {
+    if (image_info->colorspace == UndefinedColorspace)
+      (void) SetImageColorspace(image,RGBColorspace);
     opacity=(-1);
-    (void) SetImageColorspace(image,RGBColorspace);
+    if ((image->storage_class == PseudoClass) &&
+        (image->matte != MagickFalse) && (image->colors <= 256))
+      {
+        /*
+          Identify transparent pixel index.
+        */
+        for (y=0; y < (long) image->rows; y++)
+        {
+          p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+          if (p == (const PixelPacket *) NULL)
+            break;
+          indexes=GetIndexes(image);
+          for (x=0; x < (long) image->columns; x++)
+          {
+            if (p->opacity != OpaqueOpacity)
+              {
+                opacity=(long) indexes[x];
+                break;
+              }
+            p++;
+          }
+          if (x < (long) image->columns)
+            break;
+        }
+        for ( ; y < (long) image->rows; y++)
+        {
+          p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+          if (p == (const PixelPacket *) NULL)
+            break;
+          indexes=GetIndexes(image);
+          for (x=0; x < (long) image->columns; x++)
+          {
+            if ((p->opacity != OpaqueOpacity) && (opacity != (long) indexes[x]))
+              {
+                opacity=(-1);
+                break;
+              }
+            p++;
+          }
+          if (x < (long) image->columns)
+            break;
+        }
+      }
     if ((image->storage_class == DirectClass) || (image->colors > 256))
       {
         /*
@@ -1308,76 +1386,52 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
         quantize_info.dither=IsPaletteImage(image,&image->exception) ==
           MagickFalse ? MagickTrue : MagickFalse;
         (void) QuantizeImage(&quantize_info,image);
-        if ((LocaleCompare(write_info->magick,"GIF87") != 0) &&
-            (image->matte != MagickFalse))
-          {
-            MagickSizeType
-              length;
-
-            /*
-              Set transparent pixel.
-            */
-            quantize_info.number_colors=255;
-            (void) QuantizeImage(&quantize_info,image);
-            opacity=(long) image->colors++;
-            length=(size_t) image->colors*sizeof(*image->colormap);
-            image->colormap=(PixelPacket *)
-              ResizeMagickMemory(image->colormap,length);
-            if (image->colormap == (PixelPacket *) NULL)
-              {
-                global_colormap=(unsigned char *)
-                  RelinquishMagickMemory(global_colormap);
-                colormap=(unsigned char *) RelinquishMagickMemory(colormap);
-                write_info=DestroyImageInfo(write_info);
-                ThrowWriterException(ResourceLimitError,
-                  "MemoryAllocationFailed");
-              }
-            (void) QueryColorDatabase("#cccccc",image->colormap+opacity,
-              &image->exception);
-            for (y=0; y < (long) image->rows; y++)
-            {
-              p=AcquireImagePixels(image,0,y,image->columns,1,
-                &image->exception);
-              if (p == (const PixelPacket *) NULL)
-                break;
-              indexes=GetIndexes(image);
-              for (x=0; x < (long) image->columns; x++)
-              {
-                if ((double) p->opacity >= (QuantumRange-image->fuzz))
-                  indexes[x]=(IndexPacket) opacity;
-                p++;
-              }
-              if (SyncImagePixels(image) == MagickFalse)
-                break;
-            }
-          }
       }
-    else
-      if ((LocaleCompare(write_info->magick,"GIF87") != 0) &&
-          (image->matte != MagickFalse))
-        {
-          /*
-            Identify transparent pixel index.
-          */
-          for (y=0; y < (long) image->rows; y++)
+    if ((LocaleCompare(write_info->magick,"GIF87") != 0) &&
+        (image->matte != MagickFalse) && (opacity == -1))
+      {
+        size_t
+          length;
+
+        /*
+          Set transparent pixel.
+        */
+        GetQuantizeInfo(&quantize_info);
+        quantize_info.number_colors=255;
+        quantize_info.dither=IsPaletteImage(image,&image->exception) ==
+          MagickFalse ? MagickTrue : MagickFalse;
+        (void) QuantizeImage(&quantize_info,image);
+        opacity=(long) image->colors++;
+        length=(size_t) image->colors*sizeof(*image->colormap);
+        image->colormap=(PixelPacket *)
+          ResizeMagickMemory(image->colormap,length);
+        if (image->colormap == (PixelPacket *) NULL)
           {
-            p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
-            if (p == (const PixelPacket *) NULL)
-              break;
-            indexes=GetIndexes(image);
-            for (x=0; x < (long) image->columns; x++)
-            {
-              if ((double) p->opacity >= (QuantumRange-image->fuzz))
-                {
-                  opacity=(long) indexes[x];
-                  break;
-                }
-              p++;
-            }
-            if (x < (long) image->columns)
-              break;
+            global_colormap=(unsigned char *)
+              RelinquishMagickMemory(global_colormap);
+            colormap=(unsigned char *) RelinquishMagickMemory(colormap);
+            write_info=DestroyImageInfo(write_info);
+            ThrowWriterException(ResourceLimitError,
+              "MemoryAllocationFailed");
           }
+        (void) QueryColorDatabase("#abacadab",image->colormap+opacity,
+          &image->exception);
+        for (y=0; y < (long) image->rows; y++)
+        {
+          p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+          if (p == (const PixelPacket *) NULL)
+            break;
+          indexes=GetIndexes(image);
+          for (x=0; x < (long) image->columns; x++)
+          {
+            if (p->opacity != OpaqueOpacity)
+              indexes[x]=(IndexPacket) opacity;
+            p++;
+          }
+          if (SyncImagePixels(image) == MagickFalse)
+            break;
         }
+      }
     if (image->colormap == (PixelPacket *) NULL)
       break;
     for (bits_per_pixel=1; bits_per_pixel < 8; bits_per_pixel++)
@@ -1410,7 +1464,7 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
           if (ColorMatch(&image->background_color,image->colormap+j))
             break;
         (void) WriteBlobByte(image,(unsigned char)
-          j == (long) image->colors ? 0 : j);  /* background color */
+          (j == (long) image->colors ? 0 : j));  /* background color */
         (void) WriteBlobByte(image,(unsigned char) 0x00);  /* reserved */
         length=(size_t) (3*(1 << bits_per_pixel));
         (void) WriteBlob(image,length,colormap);
@@ -1429,7 +1483,8 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
         if (opacity >= 0)
           c|=0x01;
         (void) WriteBlobByte(image,(unsigned char) c);
-        (void) WriteBlobLSBShort(image,(unsigned short) image->delay);
+        delay=100*image->delay/Max(image->ticks_per_second,1);
+        (void) WriteBlobLSBShort(image,(unsigned short) delay);
         (void) WriteBlobByte(image,(unsigned char)
           (opacity >= 0 ? opacity : 0));
         (void) WriteBlobByte(image,(unsigned char) 0x00);
@@ -1462,7 +1517,8 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
             (void) WriteBlobByte(image,(unsigned char) 0x00);
           }
         if ((GetPreviousImageInList(image) == (Image *) NULL) &&
-            (GetNextImageInList(image) != (Image *) NULL) && (image->iterations != 1))
+            (GetNextImageInList(image) != (Image *) NULL) &&
+            (image->iterations != 1))
           {
             /*
               Write Netscape Loop extension.
