@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -53,6 +53,7 @@
 #include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/pixel.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/statistic.h"
 #include "magick/string_.h"
@@ -121,7 +122,7 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
     packet_size;
 
   unsigned char
-    *scanline;
+    *pixels;
 
   unsigned long
     depth,
@@ -133,7 +134,8 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
@@ -148,13 +150,13 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
   for (i=0; i < image->offset; i++)
     (void) ReadBlobByte(image);
   /*
-    Allocate memory for a scanline.
+    Allocate memory for a pixels.
   */
-  depth=GetImageQuantumDepth(image,MagickTrue);
+  depth=GetImageQuantumDepth(image,MagickFalse);
   packet_size=(size_t) (depth/8);
-  scanline=(unsigned char *)
-    AcquireMagickMemory(packet_size*image->extract_info.width);
-  if (scanline == (unsigned char *) NULL)
+  pixels=(unsigned char *)
+    AcquireMagickMemory(packet_size*image->extract_info.width*sizeof(*pixels));
+  if (pixels == (unsigned char *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   if (image_info->number_scenes != 0)
     while (image->scene < image_info->scene)
@@ -165,7 +167,7 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
       image->scene++;
       for (y=0; y < (long) image->rows; y++)
       {
-        count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+        count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
         if (count != (ssize_t) (packet_size*image->extract_info.width))
           break;
       }
@@ -176,23 +178,20 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
     /*
       Convert raster image to pixel packets.
     */
-    if ((image->depth <= QuantumDepth) && (image->depth <= 16))
-      if (AllocateImageColormap(image,1UL << image->depth) == MagickFalse)
-        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     for (y=0; y < image->extract_info.y; y++)
     {
-      count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+      count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
       if (count != (ssize_t) (packet_size*image->extract_info.width))
         break;
     }
     for (y=0; y < (long) image->rows; y++)
     {
-      if ((y > 0) || (image->previous == (Image *) NULL))
+      if ((y > 0) || (GetPreviousImageInList(image) == (Image *) NULL))
         {
-          count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
           if (count != (ssize_t) (packet_size*image->extract_info.width))
             break;
         }
@@ -200,10 +199,10 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
       if (q == (PixelPacket *) NULL)
         break;
       if ((image->depth <= QuantumDepth) && (image->depth <= 16))
-        (void) ExportQuantumPixels(image,GrayQuantum,0,scanline+offset);
+        (void) ExportQuantumPixels(image,GrayQuantum,0,pixels+offset);
       else
         {
-          (void) ExportQuantumPixels(image,RedQuantum,0,scanline+offset);
+          (void) ExportQuantumPixels(image,RedQuantum,0,pixels+offset);
           for (x=0; x < (long) image->columns; x++)
           {
             q->green=q->red;
@@ -226,7 +225,7 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
     width=image->extract_info.height-image->rows-image->extract_info.y;
     for (j=0; j < (long) width; j++)
     {
-      count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+      count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
       if (count != (ssize_t) (packet_size*image->extract_info.width))
         break;
     }
@@ -242,14 +241,14 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
     if (image_info->number_scenes != 0)
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
-    count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+    count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
     if (count == (ssize_t) (packet_size*image->extract_info.width))
       {
         /*
           Allocate next image structure.
         */
         AllocateNextImage(image_info,image);
-        if (image->next == (Image *) NULL)
+        if (GetNextImageInList(image) == (Image *) NULL)
           {
             image=DestroyImageList(image);
             return((Image *) NULL);
@@ -264,7 +263,7 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
           }
       }
   } while (count == (ssize_t) (packet_size*image->extract_info.width));
-  scanline=(unsigned char *) RelinquishMagickMemory(scanline);
+  pixels=(unsigned char *) RelinquishMagickMemory(pixels);
   CloseBlob(image);
   return(GetFirstImageInList(image));
 }
@@ -302,8 +301,8 @@ ModuleExport void RegisterGRAYImage(void)
   entry->encoder=(EncoderHandler *) WriteGRAYImage;
   entry->raw=MagickTrue;
   entry->endian_support=MagickTrue;
-  entry->description=AcquireString("Raw gray samples");
-  entry->module=AcquireString("GRAY");
+  entry->description=ConstantString("Raw gray samples");
+  entry->module=ConstantString("GRAY");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -376,7 +375,7 @@ static MagickBooleanType WriteGRAYImage(const ImageInfo *image_info,
     packet_size;
 
   unsigned char
-    *scanline;
+    *pixels;
 
   unsigned long
     depth;
@@ -389,7 +388,7 @@ static MagickBooleanType WriteGRAYImage(const ImageInfo *image_info,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
@@ -400,24 +399,26 @@ static MagickBooleanType WriteGRAYImage(const ImageInfo *image_info,
   do
   {
     /*
-      Allocate memory for scanline.
+      Allocate memory for pixels.
     */
-    (void) SetImageColorspace(image,RGBColorspace);
-    depth=GetImageQuantumDepth(image,MagickTrue);
+    if (image_info->colorspace == UndefinedColorspace)
+      (void) SetImageColorspace(image,RGBColorspace);
+    depth=GetImageQuantumDepth(image,MagickFalse);
     packet_size=(size_t) (depth/8);
-    scanline=(unsigned char *) AcquireMagickMemory(packet_size*image->columns);
-    if (scanline == (unsigned char *) NULL)
+    pixels=(unsigned char *)
+      AcquireMagickMemory(packet_size*image->columns*sizeof(*pixels));
+    if (pixels == (unsigned char *) NULL)
       ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     /*
-      Convert MIFF to GRAY raster scanline.
+      Convert MIFF to GRAY raster pixels.
     */
     for (y=0; y < (long) image->rows; y++)
     {
       p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
       if (p == (const PixelPacket *) NULL)
         break;
-      (void) ImportQuantumPixels(image,GrayQuantum,0,scanline);
-      (void) WriteBlob(image,packet_size*image->columns,scanline);
+      (void) ImportQuantumPixels(image,GrayQuantum,0,pixels);
+      (void) WriteBlob(image,packet_size*image->columns,pixels);
       if (image->previous == (Image *) NULL)
         if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
             (QuantumTick(y,image->rows) != MagickFalse))
@@ -428,8 +429,8 @@ static MagickBooleanType WriteGRAYImage(const ImageInfo *image_info,
               break;
           }
     }
-    scanline=(unsigned char *) RelinquishMagickMemory(scanline);
-    if (image->next == (Image *) NULL)
+    pixels=(unsigned char *) RelinquishMagickMemory(pixels);
+    if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
