@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -54,6 +54,7 @@
 #include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/resource_.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/statistic.h"
 #include "magick/string_.h"
@@ -95,8 +96,8 @@ ModuleExport void RegisterHISTOGRAMImage(void)
   entry=SetMagickInfo("HISTOGRAM");
   entry->encoder=(EncoderHandler *) WriteHISTOGRAMImage;
   entry->adjoin=MagickFalse;
-  entry->description=AcquireString("Histogram of the image");
-  entry->module=AcquireString("HISTOGRAM");
+  entry->description=ConstantString("Histogram of the image");
+  entry->module=ConstantString("HISTOGRAM");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -163,6 +164,9 @@ static MagickBooleanType WriteHISTOGRAMImage(const ImageInfo *image_info,
 {
 #define HistogramDensity  "256x200"
 
+  ChannelType
+    channel;
+
   char
     filename[MaxTextExtent];
 
@@ -212,7 +216,8 @@ static MagickBooleanType WriteHISTOGRAMImage(const ImageInfo *image_info,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   (void) SetImageDepth(image,image->depth);
   SetGeometry(image,&geometry);
   if (image_info->density == (char *) NULL)
@@ -227,7 +232,7 @@ static MagickBooleanType WriteHISTOGRAMImage(const ImageInfo *image_info,
   /*
     Allocate histogram count arrays.
   */
-  length=(size_t) Max((unsigned long) ScaleQuantumToChar(MaxRGB)+1,
+  length=(size_t) Max((unsigned long) ScaleQuantumToChar(QuantumRange)+1,
     histogram_image->columns);
   histogram=(MagickPixelPacket *)
     AcquireMagickMemory(length*sizeof(*histogram));
@@ -239,6 +244,7 @@ static MagickBooleanType WriteHISTOGRAMImage(const ImageInfo *image_info,
   /*
     Initialize histogram count arrays.
   */
+  channel=image_info->channel;
   (void) ResetMagickMemory(histogram,0,length*sizeof(*histogram));
   for (y=0; y < (long) image->rows; y++)
   {
@@ -247,21 +253,27 @@ static MagickBooleanType WriteHISTOGRAMImage(const ImageInfo *image_info,
       break;
     for (x=0; x < (long) image->columns; x++)
     {
-      histogram[ScaleQuantumToChar(p->red)].red++;
-      histogram[ScaleQuantumToChar(p->green)].green++;
-      histogram[ScaleQuantumToChar(p->blue)].blue++;
+      if ((channel & RedChannel) != 0)
+        histogram[ScaleQuantumToChar(p->red)].red++;
+      if ((channel & GreenChannel) != 0)
+        histogram[ScaleQuantumToChar(p->green)].green++;
+      if ((channel & BlueChannel) != 0)
+        histogram[ScaleQuantumToChar(p->blue)].blue++;
       p++;
     }
   }
   maximum=histogram[0].red;
   for (x=0; x < (long) histogram_image->columns; x++)
   {
-    if (maximum < histogram[x].red)
-      maximum=histogram[x].red;
-    if (maximum < histogram[x].green)
-      maximum=histogram[x].green;
-    if (maximum < histogram[x].blue)
-      maximum=histogram[x].blue;
+    if ((channel & RedChannel) != 0)
+      if (maximum < histogram[x].red)
+        maximum=histogram[x].red;
+    if ((channel & GreenChannel) != 0)
+      if (maximum < histogram[x].green)
+        maximum=histogram[x].green;
+    if ((channel & BlueChannel) != 0)
+      if (maximum < histogram[x].blue)
+        maximum=histogram[x].blue;
   }
   scale=(MagickRealType) histogram_image->rows/maximum;
   /*
@@ -275,27 +287,36 @@ static MagickBooleanType WriteHISTOGRAMImage(const ImageInfo *image_info,
     q=GetImagePixels(histogram_image,x,0,1,histogram_image->rows);
     if (q == (PixelPacket *) NULL)
       break;
-    y=(long) (histogram_image->rows-scale*histogram[x].red+0.5);
-    r=q+y;
-    for ( ; y < (long) histogram_image->rows; y++)
-    {
-      r->red=MaxRGB;
-      r++;
-    }
-    y=(long) (histogram_image->rows-scale*histogram[x].green+0.5);
-    r=q+y;
-    for ( ; y < (long) histogram_image->rows; y++)
-    {
-      r->green=MaxRGB;
-      r++;
-    }
-    y=(long) (histogram_image->rows-scale*histogram[x].blue+0.5);
-    r=q+y;
-    for ( ; y < (long) histogram_image->rows; y++)
-    {
-      r->blue=MaxRGB;
-      r++;
-    }
+    if ((channel & RedChannel) != 0)
+      {
+        y=(long) (histogram_image->rows-scale*histogram[x].red+0.5);
+        r=q+y;
+        for ( ; y < (long) histogram_image->rows; y++)
+        {
+          r->red=QuantumRange;
+          r++;
+        }
+      }
+    if ((channel & GreenChannel) != 0)
+      {
+        y=(long) (histogram_image->rows-scale*histogram[x].green+0.5);
+        r=q+y;
+        for ( ; y < (long) histogram_image->rows; y++)
+        {
+          r->green=QuantumRange;
+          r++;
+        }
+      }
+    if ((channel & BlueChannel) != 0)
+      {
+        y=(long) (histogram_image->rows-scale*histogram[x].blue+0.5);
+        r=q+y;
+        for ( ; y < (long) histogram_image->rows; y++)
+        {
+          r->blue=QuantumRange;
+          r++;
+        }
+      }
     if (SyncImagePixels(histogram_image) == MagickFalse)
       break;
     if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
