@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -53,6 +53,7 @@
 #include "magick/monitor.h"
 #include "magick/nt-feature.h"
 #include "magick/quantize.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 
@@ -207,7 +208,8 @@ static Image *ReadICONImage(const ImageInfo *image_info,
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
@@ -253,7 +255,7 @@ static Image *ReadICONImage(const ImageInfo *image_info,
     icon_info.y_pixels=ReadBlobLSBLong(image);
     icon_info.number_colors=ReadBlobLSBLong(image);
     icon_info.colors_important=ReadBlobLSBLong(image);
-    image->matte=(MagickBooleanType) (icon_info.bits_per_pixel == 32);
+    image->matte=icon_info.bits_per_pixel == 32 ? MagickTrue : MagickFalse;
     image->columns=icon_info.width;
     image->rows=icon_info.height;
     image->depth=8;
@@ -272,10 +274,14 @@ static Image *ReadICONImage(const ImageInfo *image_info,
       unsigned char
         *icon_colormap;
 
+      unsigned long
+        number_colors;
+
       /*
         Read Icon raster colormap.
       */
-      if (AllocateImageColormap(image,1 << icon_info.bits_per_pixel) == MagickFalse)
+      number_colors=1UL << icon_info.bits_per_pixel;
+      if (AllocateImageColormap(image,number_colors) == MagickFalse)
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
       icon_colormap=(unsigned char *)
         AcquireMagickMemory((size_t) (4*image->colors));
@@ -454,11 +460,12 @@ static Image *ReadICONImage(const ImageInfo *image_info,
             break;
           for (x=0; x < (long) image->columns; x++)
           {
-            q->blue=(Quantum) ScaleCharToQuantum(ReadBlobByte(image));
-            q->green=(Quantum) ScaleCharToQuantum(ReadBlobByte(image));
-            q->red=(Quantum) ScaleCharToQuantum(ReadBlobByte(image));
+            q->blue=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
+            q->green=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
+            q->red=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
             if (image->matte != MagickFalse)
-              q->opacity=(MaxRGB-ScaleCharToQuantum(ReadBlobByte(image)));
+              q->opacity=ScaleCharToQuantum((unsigned char)
+                ReadBlobByte(image));
             q++;
           }
           if (SyncImagePixels(image) == MagickFalse)
@@ -530,7 +537,7 @@ static Image *ReadICONImage(const ImageInfo *image_info,
           Allocate next image structure.
         */
         AllocateNextImage(image_info,image);
-        if (image->next == (Image *) NULL)
+        if (GetNextImageInList(image) == (Image *) NULL)
           {
             image=DestroyImageList(image);
             return((Image *) NULL);
@@ -582,24 +589,24 @@ ModuleExport void RegisterICONImage(void)
   entry->encoder=(EncoderHandler *) WriteICONImage;
   entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
-  entry->description=AcquireString("Microsoft icon");
-  entry->module=AcquireString("CUR");
+  entry->description=ConstantString("Microsoft icon");
+  entry->module=ConstantString("CUR");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("ICO");
   entry->decoder=(DecoderHandler *) ReadICONImage;
   entry->encoder=(EncoderHandler *) WriteICONImage;
   entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
-  entry->description=AcquireString("Microsoft icon");
-  entry->module=AcquireString("ICON");
+  entry->description=ConstantString("Microsoft icon");
+  entry->module=ConstantString("ICON");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("ICON");
   entry->decoder=(DecoderHandler *) ReadICONImage;
   entry->encoder=(EncoderHandler *) WriteICONImage;
   entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
-  entry->description=AcquireString("Microsoft icon");
-  entry->module=AcquireString("ICON");
+  entry->description=ConstantString("Microsoft icon");
+  entry->module=ConstantString("ICON");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -646,7 +653,8 @@ ModuleExport void UnregisterICONImage(void)
 %
 %  The format of the WriteICONImage method is:
 %
-%      MagickBooleanType WriteICONImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteICONImage(const ImageInfo *image_info,
+%        Image *image)
 %
 %  A description of each parameter follows.
 %
@@ -707,7 +715,7 @@ static MagickBooleanType WriteICONImage(const ImageInfo *image_info,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
@@ -726,6 +734,7 @@ static MagickBooleanType WriteICONImage(const ImageInfo *image_info,
   (void) WriteBlobLSBShort(image,0);
   (void) WriteBlobLSBShort(image,1);
   (void) WriteBlobLSBShort(image,scene);
+  (void) ResetMagickMemory(&icon_info,0,sizeof(icon_info));
   scene=0;
   next=image;
   do
@@ -747,36 +756,13 @@ static MagickBooleanType WriteICONImage(const ImageInfo *image_info,
     /*
       Initialize ICON raster file header.
     */
-    (void) SetImageColorspace(image,RGBColorspace);
-    (void) ResetMagickMemory(&icon_info,0,sizeof(icon_info));
+    if (image_info->colorspace == UndefinedColorspace)
+      (void) SetImageColorspace(image,RGBColorspace);
     icon_info.file_size=14+12+28;
     icon_info.offset_bits=icon_info.file_size;
     icon_info.compression=BI_RGB;
     if ((image->storage_class != DirectClass) && (image->colors > 256))
-      image->storage_class=DirectClass;
-    if (image->storage_class != DirectClass)
-      {
-        /*
-          Colormapped ICON raster.
-        */
-        icon_info.bits_per_pixel=8;
-        if (image->colors <= 2)
-          icon_info.bits_per_pixel=1;
-        else if (image->colors <= 16)
-          icon_info.bits_per_pixel=4;
-        else if (image->colors <= 256)
-          icon_info.bits_per_pixel=8;
-        icon_info.number_colors=1 << icon_info.bits_per_pixel;
-        if (icon_info.number_colors < image->colors)
-          image->storage_class=DirectClass;
-        else
-          {
-            icon_info.file_size+=3*(1 << icon_info.bits_per_pixel);
-            icon_info.offset_bits+=3*(1 << icon_info.bits_per_pixel);
-            icon_info.file_size+=(1 << icon_info.bits_per_pixel);
-            icon_info.offset_bits+=(1 << icon_info.bits_per_pixel);
-          }
-      }
+      (void) SetImageStorageClass(image,DirectClass);
     if (image->storage_class == DirectClass)
       {
         /*
@@ -786,19 +772,43 @@ static MagickBooleanType WriteICONImage(const ImageInfo *image_info,
         icon_info.bits_per_pixel=(unsigned short) 24;
         icon_info.compression=(unsigned long) BI_RGB;
       }
+    else
+      {
+        /*
+          Colormapped ICON raster.
+        */
+        icon_info.bits_per_pixel=8;
+        if (image->colors <= 256)
+          icon_info.bits_per_pixel=8;
+        if (image->colors <= 16)
+          icon_info.bits_per_pixel=4;
+        if (image->colors <= 2)
+          icon_info.bits_per_pixel=1;
+        icon_info.number_colors=1 << icon_info.bits_per_pixel;
+        if (icon_info.number_colors < image->colors)
+          {
+            (void) SetImageStorageClass(image,DirectClass);
+            icon_info.number_colors=0;
+            icon_info.bits_per_pixel=(unsigned short) 24;
+            icon_info.compression=(unsigned long) BI_RGB;
+          }
+        else
+          {
+            icon_info.file_size+=3*(1UL << icon_info.bits_per_pixel);
+            icon_info.offset_bits+=3*(1UL << icon_info.bits_per_pixel);
+            icon_info.file_size+=(1UL << icon_info.bits_per_pixel);
+            icon_info.offset_bits+=(1UL << icon_info.bits_per_pixel);
+          }
+      }
     bytes_per_line=4*((image->columns*icon_info.bits_per_pixel+31)/32);
     icon_info.ba_offset=0;
     if (image->matte == MagickFalse)
       icon_info.size=40;
     else
       {
-        int
-         extra_size;
-
         icon_info.size=108;
-        extra_size=68;
-        icon_info.file_size+=extra_size;
-        icon_info.offset_bits+=extra_size;
+        icon_info.file_size+=68;
+        icon_info.offset_bits+=68;
       }
     icon_info.width=(long) image->columns;
     icon_info.height=(long) image->rows;
@@ -968,6 +978,9 @@ static MagickBooleanType WriteICONImage(const ImageInfo *image_info,
               *q++=ScaleQuantumToChar(p->opacity);
             p++;
           }
+          if (icon_info.bits_per_pixel == 24)
+            for (x=3L*(long) image->columns; x < (long) bytes_per_line; x++)
+              *q++=0x00;
           if (image->previous == (Image *) NULL)
             if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
                 (QuantumTick(y,image->rows) != MagickFalse))
@@ -1049,7 +1062,7 @@ static MagickBooleanType WriteICONImage(const ImageInfo *image_info,
       for (x=0; x < (long) image->columns; x++)
       {
         byte>>=1;
-        if (image->matte && (p->opacity == OpaqueOpacity))
+        if ((image->matte == MagickFalse) || (p->opacity != OpaqueOpacity))
           byte|=0x80;
         bit++;
         if (bit == 8)
@@ -1065,7 +1078,7 @@ static MagickBooleanType WriteICONImage(const ImageInfo *image_info,
       for (i=0; i < (long) scanline_pad; i++)
         (void) WriteBlobByte(image,(unsigned char) (byte >> (8-bit)));
     }
-    if (image->next == (Image *) NULL)
+    if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
@@ -1080,7 +1093,7 @@ static MagickBooleanType WriteICONImage(const ImageInfo *image_info,
   (void) SeekBlob(image,0,SEEK_SET);
   (void) WriteBlobLSBShort(image,0);
   (void) WriteBlobLSBShort(image,1);
-  (void) WriteBlobLSBShort(image,scene+1);
+  (void) WriteBlobLSBShort(image,(unsigned short) (scene+1));
   scene=0;
   next=image;
   do
