@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -65,6 +65,7 @@
 #include "magick/profile.h"
 #include "magick/resource_.h"
 #include "magick/splay-tree.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/utility.h"
@@ -388,7 +389,7 @@ static boolean ReadICCProfile(j_decompress_ptr jpeg_info)
   p=profile->datum;
   for (i=(long) profile->length-1; i >= 0; i--)
     *p++=(unsigned char) GetCharacter(jpeg_info);
-  icc_profile=GetImageProfile(image,"icc");
+  icc_profile=(StringInfo *) GetImageProfile(image,"icc");
   if (icc_profile != (StringInfo *) NULL)
     {
       ConcatenateStringInfo(icc_profile,profile);
@@ -455,7 +456,7 @@ static boolean ReadIPTCProfile(j_decompress_ptr jpeg_info)
 #endif
   error_manager=(ErrorManager *) jpeg_info->client_data;
   image=error_manager->image;
-  iptc_profile=GetImageProfile(image,"iptc");
+  iptc_profile=(StringInfo *) GetImageProfile(image,"iptc");
   if (iptc_profile == (StringInfo *) NULL)
     {
 #ifdef GET_ONLY_IPTC_DATA
@@ -510,7 +511,7 @@ static boolean ReadIPTCProfile(j_decompress_ptr jpeg_info)
   p=profile->datum;
   for (i=(long) profile->length-1; i >= 0; i--)
     *p++=(unsigned char) GetCharacter(jpeg_info);
-  iptc_profile=GetImageProfile(image,"iptc");
+  iptc_profile=(StringInfo *) GetImageProfile(image,"iptc");
   if (iptc_profile != (StringInfo *) NULL)
     {
       ConcatenateStringInfo(iptc_profile,profile);
@@ -739,7 +740,9 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
     if ((i != 2) && (i != 13) && (i != 14))
       jpeg_set_marker_processor(&jpeg_info,(int) i+JPEG_APP0,ReadProfile);
   i=jpeg_read_header(&jpeg_info,MagickTrue);
-  if (image_info->colorspace == YCbCrColorspace)
+  if ((image_info->colorspace == YCbCrColorspace) ||
+      (image_info->colorspace == Rec601YCbCrColorspace) ||
+      (image_info->colorspace == Rec709YCbCrColorspace))
     {
       jpeg_info.out_color_space=JCS_YCbCr;
       image->colorspace=YCbCrColorspace;
@@ -839,7 +842,7 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Geometry: %dx%d",
         (int) jpeg_info.output_width,(int) jpeg_info.output_height);
     }
-  image->quality=0;
+  image->quality=UndefinedCompressionQuality;
 #ifdef D_LOSSLESS_SUPPORTED
   if (image->compression == LosslessJPEGCompression)
     {
@@ -850,35 +853,30 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   else
 #endif
   {
-    int
-      hashval,
+    long
+      value,
       sum;
 
+    register long
+      j;
+
     /*
-      Log the JPEG quality that was used for compression.
+      Determine the JPEG compression quality from the quantization tables.
     */
     sum=0;
     for (i=0; i < NUM_QUANT_TBLS; i++)
     {
-      int
-        j;
-
       if (jpeg_info.quant_tbl_ptrs[i] != NULL)
         for (j=0; j < DCTSIZE2; j++)
-        {
-          UINT16
-            *c;
-
-          c=jpeg_info.quant_tbl_ptrs[i]->quantval;
-          sum+=c[j];
-        }
+          sum+=jpeg_info.quant_tbl_ptrs[i]->quantval[j];
      }
      if ((jpeg_info.quant_tbl_ptrs[0] != NULL) &&
          (jpeg_info.quant_tbl_ptrs[1] != NULL))
        {
-         int
-           hash[]=
-           { 1020, 1015,  932,  848,  780,  735,  702,  679,  660,  645,
+         long
+           hash[101] =
+           {
+             1020, 1015,  932,  848,  780,  735,  702,  679,  660,  645,
               632,  623,  613,  607,  600,  594,  589,  585,  581,  571,
               555,  542,  529,  514,  494,  474,  457,  439,  424,  410,
               397,  386,  373,  364,  351,  341,  334,  324,  317,  309,
@@ -890,48 +888,50 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
                45,   40,   34,   30,   25,   20,   15,   11,    6,    4,
                 0
            },
-           sums[]=
-           { 32640,32635,32266,31495,30665,29804,29146,28599,28104,27670,
-             27225,26725,26210,25716,25240,24789,24373,23946,23572,22846,
-             21801,20842,19949,19121,18386,17651,16998,16349,15800,15247,
-             14783,14321,13859,13535,13081,12702,12423,12056,11779,11513,
-             11135,10955,10676,10392,10208, 9928, 9747, 9564, 9369, 9193,
-              9017, 8822, 8639, 8458, 8270, 8084, 7896, 7710, 7527, 7347,
-              7156, 6977, 6788, 6607, 6422, 6236, 6054, 5867, 5684, 5495,
-              5305, 5128, 4945, 4751, 4638, 4442, 4248, 4065, 3888, 3698,
-              3509, 3326, 3139, 2957, 2775, 2586, 2405, 2216, 2037, 1846,
-              1666, 1483, 1297, 1109,  927,  735,  554,  375,  201,  128,
-                 0
+           sums[101] =
+           {
+             32640, 32635, 32266, 31495, 30665, 29804, 29146, 28599, 28104,
+             27670, 27225, 26725, 26210, 25716, 25240, 24789, 24373, 23946,
+             23572, 22846, 21801, 20842, 19949, 19121, 18386, 17651, 16998,
+             16349, 15800, 15247, 14783, 14321, 13859, 13535, 13081, 12702,
+             12423, 12056, 11779, 11513, 11135, 10955, 10676, 10392, 10208,
+              9928,  9747,  9564,  9369,  9193,  9017,  8822,  8639,  8458,
+              8270,  8084,  7896,  7710,  7527,  7347,  7156,  6977,  6788,
+              6607,  6422,  6236,  6054,  5867,  5684,  5495,  5305,  5128,
+              4945,  4751,  4638,  4442,  4248,  4065,  3888,  3698,  3509,
+              3326,  3139,  2957,  2775,  2586,  2405,  2216,  2037,  1846,
+              1666,  1483,  1297,  1109,   927,   735,   554,   375,   201,
+               128,     0
            };
 
-         hashval=(int) (jpeg_info.quant_tbl_ptrs[0]->quantval[2]+
+         value=(long) (jpeg_info.quant_tbl_ptrs[0]->quantval[2]+
            jpeg_info.quant_tbl_ptrs[0]->quantval[53]+
            jpeg_info.quant_tbl_ptrs[1]->quantval[0]+
            jpeg_info.quant_tbl_ptrs[1]->quantval[DCTSIZE2-1]);
          for (i=0; i < 100; i++)
          {
-           if ((hashval >= hash[i]) || (sum >= sums[i]))
+           if ((value < hash[i]) && (sum < sums[i]))
+             continue;
+           image->quality=(unsigned long) i+1;
+           if (image->debug != MagickFalse)
              {
-               image->quality=(unsigned long) i+1;
-               if (image->debug != MagickFalse)
-                 {
-                   if ((hashval > hash[i]) || (sum > sums[i]))
-                     (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                       "Quality: %ld (approximate)",image->quality);
-                   else
-                     (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                       "Quality: %ld",image->quality);
-                 }
-               break;
+               if ((value > hash[i]) || (sum > sums[i]))
+                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                   "Quality: %ld (approximate)",image->quality);
+               else
+                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                   "Quality: %ld",image->quality);
              }
+           break;
          }
        }
      else
        if (jpeg_info.quant_tbl_ptrs[0] != NULL)
          {
-           int
-             bwhash[]=
-             { 510,  505,  422,  380,  355,  338,  326,  318,  311,  305,
+           long
+             hash[101] =
+             {
+               510,  505,  422,  380,  355,  338,  326,  318,  311,  305,
                300,  297,  293,  291,  288,  286,  284,  283,  281,  280,
                279,  278,  277,  273,  262,  251,  243,  233,  225,  218,
                211,  205,  198,  193,  186,  181,  177,  172,  168,  164,
@@ -943,38 +943,39 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
                 24,   21,   18,   16,   13,   11,    8,    6,    3,    2,
                  0
              },
-             bwsum[]=
-             { 16320,16315,15946,15277,14655,14073,13623,13230,12859,12560,
-               12240,11861,11456,11081,10714,10360,10027, 9679, 9368, 9056,
-                8680, 8331, 7995, 7668, 7376, 7084, 6823, 6562, 6345, 6125,
-                5939, 5756, 5571, 5421, 5240, 5086, 4976, 4829, 4719, 4616,
-                4463, 4393, 4280, 4166, 4092, 3980, 3909, 3835, 3755, 3688,
-                3621, 3541, 3467, 3396, 3323, 3247, 3170, 3096, 3021, 2952,
-                2874, 2804, 2727, 2657, 2583, 2509, 2437, 2362, 2290, 2211,
-                2136, 2068, 1996, 1915, 1858, 1773, 1692, 1620, 1552, 1477,
-                1398, 1326, 1251, 1179, 1109, 1031,  961,  884,  814,  736,
-                 667,  592,  518,  441,  369,  292,  221,  151,   86,   64,
-                   0
+             sums[101] =
+             {
+               16320, 16315, 15946, 15277, 14655, 14073, 13623, 13230, 12859,
+               12560, 12240, 11861, 11456, 11081, 10714, 10360, 10027,  9679,
+                9368,  9056,  8680,  8331,  7995,  7668,  7376,  7084,  6823,
+                6562,  6345,  6125,  5939,  5756,  5571,  5421,  5240,  5086,
+                4976,  4829,  4719,  4616,  4463,  4393,  4280,  4166,  4092,
+                3980,  3909,  3835,  3755,  3688,  3621,  3541,  3467,  3396,
+                3323,  3247,  3170,  3096,  3021,  2952,  2874,  2804,  2727,
+                2657,  2583,  2509,  2437,  2362,  2290,  2211,  2136,  2068,
+                1996,  1915,  1858,  1773,  1692,  1620,  1552,  1477,  1398,
+                1326,  1251,  1179,  1109,  1031,   961,   884,   814,   736,
+                 667,   592,   518,   441,   369,   292,   221,   151,    86,
+                  64,     0
              };
 
-           hashval=(int) (jpeg_info.quant_tbl_ptrs[0]->quantval[2]+
+           value=(long) (jpeg_info.quant_tbl_ptrs[0]->quantval[2]+
              jpeg_info.quant_tbl_ptrs[0]->quantval[53]);
            for (i=0; i < 100; i++)
            {
-             if ((hashval >= bwhash[i]) || (sum >= bwsum[i]))
+             if ((value < hash[i]) && (sum < sums[i]))
+               continue;
+             image->quality=(unsigned long) i+1;
+             if (image->debug != MagickFalse)
                {
-                 image->quality=(unsigned long) i+1;
-                 if (image->debug != MagickFalse)
-                   {
-                     if ((hashval > bwhash[i]) || (sum > bwsum[i]))
-                       (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                         "Quality: %ld (approximate)",i+1);
-                     else
-                       (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                         "Quality: %ld",i+1);
-                   }
-                 break;
+                 if ((value > hash[i]) || (sum > sums[i]))
+                   (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                     "Quality: %ld (approximate)",i+1);
+                 else
+                   (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                     "Quality: %ld",i+1);
                }
+             break;
            }
          }
   }
@@ -1067,7 +1068,7 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
     }
   if (jpeg_info.quantize_colors != MagickFalse)
     {
-      image->colors=jpeg_info.actual_number_of_colors;
+      image->colors=(unsigned long) jpeg_info.actual_number_of_colors;
       if (jpeg_info.out_color_space == JCS_GRAYSCALE)
         for (i=0; i < (long) image->colors; i++)
         {
@@ -1087,7 +1088,11 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   for (y=0; y < (long) image->rows; y++)
   {
     if (jpeg_read_scanlines(&jpeg_info,scanline,1) != 1)
-      break;
+      {
+        (void) ThrowMagickException(&image->exception,GetMagickModule(),
+          CorruptImageWarning,"SkipToSyncByte","`%s'",image->filename);
+        continue;
+      }
     p=jpeg_pixels;
     q=SetImagePixels(image,0,y,image->columns,1);
     if (q == (PixelPacket *) NULL)
@@ -1099,7 +1104,7 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
           for (x=0; x < (long) image->columns; x++)
           {
             index=ConstrainColormapIndex(image,(unsigned long)
-              (16*GETJSAMPLE(*p)));
+              (GETJSAMPLE(*p) << 4));
             indexes[x]=(IndexPacket) index;
             *q++=image->colormap[index];
             p++;
@@ -1107,11 +1112,14 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
         else
           for (x=0; x < (long) image->columns; x++)
           {
-            q->red=ScaleShortToQuantum(16*GETJSAMPLE(*p++));
-            q->green=ScaleShortToQuantum(16*GETJSAMPLE(*p++));
-            q->blue=ScaleShortToQuantum(16*GETJSAMPLE(*p++));
+            q->red=ScaleShortToQuantum((unsigned char) (GETJSAMPLE(*p++) << 4));
+            q->green=ScaleShortToQuantum((unsigned char)
+              (GETJSAMPLE(*p++) << 4));
+            q->blue=ScaleShortToQuantum((unsigned char)
+              (GETJSAMPLE(*p++) << 4));
             if (image->colorspace == CMYKColorspace)
-              indexes[x]=ScaleShortToQuantum(16*GETJSAMPLE(*p++));
+              indexes[x]=ScaleShortToQuantum((unsigned char)
+                (GETJSAMPLE(*p++) << 4));
             q++;
           }
       }
@@ -1127,11 +1135,11 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
       else
         for (x=0; x < (long) image->columns; x++)
         {
-          q->red=ScaleCharToQuantum(GETJSAMPLE(*p++));
-          q->green=ScaleCharToQuantum(GETJSAMPLE(*p++));
-          q->blue=ScaleCharToQuantum(GETJSAMPLE(*p++));
+          q->red=ScaleCharToQuantum((unsigned char) GETJSAMPLE(*p++));
+          q->green=ScaleCharToQuantum((unsigned char) GETJSAMPLE(*p++));
+          q->blue=ScaleCharToQuantum((unsigned char) GETJSAMPLE(*p++));
           if (image->colorspace == CMYKColorspace)
-            indexes[x]=ScaleCharToQuantum(GETJSAMPLE(*p++));
+            indexes[x]=ScaleCharToQuantum((unsigned char) GETJSAMPLE(*p++));
           q++;
         }
     if (SyncImagePixels(image) == MagickFalse)
@@ -1216,18 +1224,18 @@ ModuleExport void RegisterJPEGImage(void)
 #endif
   entry->magick=(MagickHandler *) IsJPEG;
   entry->adjoin=MagickFalse;
-  entry->description=
-    AcquireString("Joint Photographic Experts Group JFIF format");
+  entry->description=ConstantString(
+    "Joint Photographic Experts Group JFIF format");
 #if defined(JPEG_LIB_VERSION)
   {
     char
       version[MaxTextExtent];
 
     (void) FormatMagickString(version,MaxTextExtent,"%d",JPEG_LIB_VERSION);
-    entry->version=AcquireString(version);
+    entry->version=ConstantString(version);
   }
 #endif
-  entry->module=AcquireString("JPEG");
+  entry->module=ConstantString("JPEG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("JPG");
   entry->thread_support=MagickFalse;
@@ -1236,9 +1244,9 @@ ModuleExport void RegisterJPEGImage(void)
   entry->encoder=(EncoderHandler *) WriteJPEGImage;
 #endif
   entry->adjoin=MagickFalse;
-  entry->description=
-    AcquireString("Joint Photographic Experts Group JFIF format");
-  entry->module=AcquireString("JPEG");
+  entry->description=ConstantString(
+    "Joint Photographic Experts Group JFIF format");
+  entry->module=ConstantString("JPEG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PJPEG");
   entry->thread_support=MagickFalse;
@@ -1247,9 +1255,9 @@ ModuleExport void RegisterJPEGImage(void)
   entry->encoder=(EncoderHandler *) WriteJPEGImage;
 #endif
   entry->adjoin=MagickFalse;
-  entry->description=
-    AcquireString("Progessive Joint Photographic Experts Group JFIF");
-  entry->module=AcquireString("JPEG");
+  entry->description=ConstantString(
+    "Progessive Joint Photographic Experts Group JFIF");
+  entry->module=ConstantString("JPEG");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -1333,30 +1341,6 @@ static void InitializeDestination(j_compress_ptr cinfo)
   destination->manager.free_in_buffer=MaxBufferExtent;
 }
 
-static MagickBooleanType JPEGWarningHandler(j_common_ptr jpeg_info,int level)
-{
-  char
-    message[JMSG_LENGTH_MAX];
-
-  Image
-    *image;
-
-  (jpeg_info->err->format_message)(jpeg_info,message);
-  image=(Image *) jpeg_info->client_data;
-  if (level < 0)
-    {
-      if ((jpeg_info->err->num_warnings == 0) ||
-          (jpeg_info->err->trace_level >= 3))
-        ThrowBinaryException(CorruptImageWarning,(char *) message,
-          image->filename);
-      jpeg_info->err->num_warnings++;
-    }
-  else
-    if (jpeg_info->err->trace_level >= level)
-      ThrowBinaryException(CoderError,(char *) message,image->filename);
-  return(MagickTrue);
-}
-
 static void TerminateDestination(j_compress_ptr cinfo)
 {
   DestinationManager
@@ -1529,6 +1513,9 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
   const ImageAttribute
     *attribute;
 
+  ErrorManager
+    error_manager;
+
   JSAMPLE
     *jpeg_pixels;
 
@@ -1580,7 +1567,19 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
   */
   jpeg_info.client_data=(void *) image;
   jpeg_info.err=jpeg_std_error(&jpeg_error);
-  jpeg_info.err->emit_message=(void (*)(j_common_ptr,int)) JPEGWarningHandler;
+  jpeg_info.err->emit_message=(void (*)(j_common_ptr,int)) EmitMessage;
+  jpeg_info.err->error_exit=(void (*)(j_common_ptr)) JPEGErrorHandler;
+  error_manager.image=image;
+  jpeg_pixels=(JSAMPLE *) NULL;
+  if (setjmp(error_manager.error_recovery) != 0)
+    {
+      jpeg_destroy_compress(&jpeg_info);
+      if (jpeg_pixels != (unsigned char *) NULL)
+        jpeg_pixels=(unsigned char *) RelinquishMagickMemory(jpeg_pixels);
+      CloseBlob(image);
+      return(MagickFalse);
+    }
+  jpeg_info.client_data=(void *) &error_manager;
   jpeg_create_compress(&jpeg_info);
   JPEGDestinationManager(&jpeg_info,image);
   jpeg_info.image_width=(unsigned int) image->columns;
@@ -1598,13 +1597,18 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
       break;
     }
     case YCbCrColorspace:
+    case Rec601YCbCrColorspace:
+    case Rec709YCbCrColorspace:
     {
       jpeg_info.in_color_space=JCS_YCbCr;
       (void) SetImageColorspace(image,YCbCrColorspace);
       break;
     }
     case GRAYColorspace:
+    case Rec601LumaColorspace:
+    case Rec709LumaColorspace:
     {
+      jpeg_info.input_components=1;
       jpeg_info.in_color_space=JCS_GRAYSCALE;
       (void) SetImageColorspace(image,GRAYColorspace);
       break;
@@ -1617,12 +1621,16 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
           jpeg_info.in_color_space=JCS_CMYK;
           break;
         }
-      if (image->colorspace == YCbCrColorspace)
+      if ((image->colorspace == YCbCrColorspace) ||
+          (image->colorspace == Rec601YCbCrColorspace) ||
+          (image->colorspace == Rec709YCbCrColorspace))
         {
           jpeg_info.in_color_space=JCS_YCbCr;
           break;
         }
-      (void) SetImageColorspace(image,RGBColorspace);
+      if (image_info->colorspace != LogColorspace)
+    if (image_info->colorspace == UndefinedColorspace)
+          (void) SetImageColorspace(image,RGBColorspace);
       break;
     }
   }
@@ -1656,7 +1664,6 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
       if (image->units == PixelsPerCentimeterResolution)
         jpeg_info.density_unit=(UINT8) 2;
     }
-  jpeg_info.dct_method=JDCT_FLOAT;
   number_pixels=(MagickSizeType) image->columns*image->rows;
   if (number_pixels == (MagickSizeType) ((size_t) number_pixels))
     {
@@ -1665,7 +1672,15 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
       */
       status=AcquireMagickResource(AreaResource,number_pixels);
       if (status != MagickFalse)
-        jpeg_info.optimize_coding=MagickTrue;
+        {
+          const char
+            *option;
+
+          option=GetImageOption(image_info,"jpeg:optimize-coding");
+          if ((option == (const char *) NULL) ||
+              (LocaleCompare(option,"true") == 0))
+            jpeg_info.optimize_coding=MagickTrue;
+        }
       RelinquishMagickResource(AreaResource,number_pixels);
     }
 #if (JPEG_LIB_VERSION >= 61) && defined(C_PROGRESSIVE_SUPPORTED)
@@ -1815,6 +1830,8 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
           break;
         }
         case YCbCrColorspace:
+        case Rec601YCbCrColorspace:
+        case Rec709YCbCrColorspace:
         {
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
             "Colorspace: YCbCr");
@@ -1842,6 +1859,8 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
           break;
         }
         case GRAYColorspace:
+        case Rec601LumaColorspace:
+        case Rec709LumaColorspace:
         {
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
             "Colorspace: GRAY");
@@ -1865,6 +1884,8 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
           break;
         }
         case YCbCrColorspace:
+        case Rec601YCbCrColorspace:
+        case Rec709YCbCrColorspace:
         {
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
             "Colorspace: YCbCr");
@@ -1926,7 +1947,8 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
           q=jpeg_pixels;
           for (x=0; x < (long) image->columns; x++)
           {
-            *q++=(JSAMPLE) (ScaleQuantumToShort(PixelIntensityToQuantum(p))/16);
+            *q++=(JSAMPLE)
+              (ScaleQuantumToShort(PixelIntensityToQuantum(p)) >> 4);
             p++;
           }
           (void) jpeg_write_scanlines(&jpeg_info,scanline,1);
@@ -1950,9 +1972,9 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
             q=jpeg_pixels;
             for (x=0; x < (long) image->columns; x++)
             {
-              *q++=(JSAMPLE) (ScaleQuantumToShort(p->red)/16);
-              *q++=(JSAMPLE) (ScaleQuantumToShort(p->green)/16);
-              *q++=(JSAMPLE) (ScaleQuantumToShort(p->blue)/16);
+              *q++=(JSAMPLE) (ScaleQuantumToShort(p->red) >> 4);
+              *q++=(JSAMPLE) (ScaleQuantumToShort(p->green) >> 4);
+              *q++=(JSAMPLE) (ScaleQuantumToShort(p->blue) >> 4);
               p++;
             }
             (void) jpeg_write_scanlines(&jpeg_info,scanline,1);
@@ -1978,10 +2000,10 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
               /*
                 Convert DirectClass packets to contiguous CMYK scanlines.
               */
-              *q++=(JSAMPLE) (4095-ScaleQuantumToShort(p->red)/16);
-              *q++=(JSAMPLE) (4095-ScaleQuantumToShort(p->green)/16);
-              *q++=(JSAMPLE) (4095-ScaleQuantumToShort(p->blue)/16);
-              *q++=(JSAMPLE) (4095-ScaleQuantumToShort(indexes[x])/16);
+              *q++=(JSAMPLE) (4095-(ScaleQuantumToShort(p->red) >> 4));
+              *q++=(JSAMPLE) (4095-(ScaleQuantumToShort(p->green) >> 4));
+              *q++=(JSAMPLE) (4095-(ScaleQuantumToShort(p->blue) >> 4));
+              *q++=(JSAMPLE) (4095-(ScaleQuantumToShort(indexes[x]) >> 4));
               p++;
             }
             (void) jpeg_write_scanlines(&jpeg_info,scanline,1);
@@ -2057,10 +2079,14 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
             /*
               Convert DirectClass packets to contiguous CMYK scanlines.
             */
-            *q++=(JSAMPLE) (ScaleQuantumToChar(QuantumRange-p->red));
-            *q++=(JSAMPLE) (ScaleQuantumToChar(QuantumRange-p->green));
-            *q++=(JSAMPLE) (ScaleQuantumToChar(QuantumRange-p->blue));
-            *q++=(JSAMPLE) (ScaleQuantumToChar(QuantumRange-indexes[x]));
+            *q++=(JSAMPLE) (ScaleQuantumToChar((Quantum)
+              (QuantumRange-p->red)));
+            *q++=(JSAMPLE) (ScaleQuantumToChar((Quantum)
+              (QuantumRange-p->green)));
+            *q++=(JSAMPLE) (ScaleQuantumToChar((Quantum)
+              (QuantumRange-p->blue)));
+            *q++=(JSAMPLE) (ScaleQuantumToChar((Quantum)
+              (QuantumRange-indexes[x])));
             p++;
           }
           (void) jpeg_write_scanlines(&jpeg_info,scanline,1);
