@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -51,6 +51,7 @@
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/utility.h"
@@ -99,13 +100,14 @@ static Image *ReadLABELImage(const ImageInfo *image_info,
   MagickBooleanType
     status;
 
-  size_t
-    length;
+  register char
+    *p;
 
   TypeMetric
     metrics;
 
   unsigned long
+    height,
     width;
 
   /*
@@ -114,19 +116,33 @@ static Image *ReadLABELImage(const ImageInfo *image_info,
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   if (*image_info->filename != '@')
     label=AcquireString(image_info->filename);
   else
-    label=(char *) FileToBlob(image_info->filename+1,&length,exception);
+    {
+      label=FileToString(image_info->filename+1,~0,exception);
+      if ((label != (char *) NULL) && (*label != '\0'))
+        if (label[strlen(label)-1] == '\n')
+          label[strlen(label)-1]='\0';
+    }
   if (label == (char *) NULL)
     return((Image *) NULL);
-  StripString(label);
+  for (p=label; *p != '\0'; p++)
+  {
+    if (*p == '\r')
+      *p=' ';
+    if ((*p == '\\') && (*(p+1) == 'n'))
+      {
+        (void) CopyMagickString(p,p+1,MaxTextExtent);
+        *p='\n';
+      }
+  }
   image=AllocateImage(image_info);
   draw_info=CloneDrawInfo(image_info,(DrawInfo *) NULL);
-  draw_info->fill=image_info->pen;
   draw_info->text=label;
   if ((image->columns != 0) || (image->rows != 0))
     {
@@ -137,18 +153,20 @@ static Image *ReadLABELImage(const ImageInfo *image_info,
       for ( ; status != MagickFalse; draw_info->pointsize*=2)
       {
         width=(unsigned long) (metrics.width+metrics.max_advance/2+0.5);
+        height=(unsigned long) (metrics.height+0.5);
         if (((image->columns != 0) && (width >= image->columns)) ||
-            ((image->rows != 0) && (metrics.height >= image->rows)))
+            ((image->rows != 0) && (height >= image->rows)))
           break;
         status=GetMultilineTypeMetrics(image,draw_info,&metrics);
       }
       for ( ; status != MagickFalse; draw_info->pointsize--)
       {
         width=(unsigned long) (metrics.width+metrics.max_advance/2+0.5);
+        height=(unsigned long) (metrics.height+0.5);
         if ((image->columns != 0) && (width <= image->columns) &&
-           ((image->rows == 0) || (metrics.height <= image->rows)))
+           ((image->rows == 0) || (height <= image->rows)))
           break;
-        if ((image->rows != 0) && (metrics.height <= image->rows) &&
+        if ((image->rows != 0) && (height <= image->rows) &&
            ((image->columns == 0) || (width <= image->columns)))
           break;
         if (draw_info->pointsize < 2.0)
@@ -158,18 +176,24 @@ static Image *ReadLABELImage(const ImageInfo *image_info,
     }
   status=GetMultilineTypeMetrics(image,draw_info,&metrics);
   if (status == MagickFalse)
-    ThrowReaderException(TypeError,"UnableToGetTypeMetrics");
-  (void) FormatMagickString(geometry,MaxTextExtent,"+%g+%g",
-    metrics.max_advance/4,metrics.ascent);
+    {
+      InheritException(exception,&image->exception);
+      image=DestroyImageList(image);
+      return((Image *) NULL);
+    }
   if (image->columns == 0)
     image->columns=(unsigned long) (metrics.width+metrics.max_advance/2+0.5);
-  if (image->rows == 0)
+  if (draw_info->gravity == UndefinedGravity)
     {
-      image->rows=(unsigned long) metrics.height;
       (void) FormatMagickString(geometry,MaxTextExtent,"+%g+%g",
-        image->columns/2.0-metrics.width/2.0,metrics.ascent);
+        metrics.max_advance/4,metrics.ascent);
+      if (image->rows == 0)
+        (void) FormatMagickString(geometry,MaxTextExtent,"+%g+%g",
+          image->columns/2.0-metrics.width/2.0,metrics.ascent);
+      draw_info->geometry=AcquireString(geometry);
     }
-  draw_info->geometry=AcquireString(geometry);
+  if (image->rows == 0)
+    image->rows=(unsigned long) metrics.height;
   SetImageBackgroundColor(image);
   (void) AnnotateImage(image,draw_info);
   draw_info=DestroyDrawInfo(draw_info);
@@ -207,8 +231,8 @@ ModuleExport void RegisterLABELImage(void)
   entry=SetMagickInfo("LABEL");
   entry->decoder=(DecoderHandler *) ReadLABELImage;
   entry->adjoin=MagickFalse;
-  entry->description=AcquireString("Image label");
-  entry->module=AcquireString("LABEL");
+  entry->description=ConstantString("Image label");
+  entry->module=ConstantString("LABEL");
   (void) RegisterMagickInfo(entry);
 }
 
