@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -52,6 +52,7 @@
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/statistic.h"
 #include "magick/string_.h"
@@ -137,7 +138,8 @@ static Image *ReadMAPImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
@@ -215,7 +217,7 @@ static Image *ReadMAPImage(const ImageInfo *image_info,ExceptionInfo *exception)
       p++;
       if (image->colors > 256)
         {
-          index=ConstrainColormapIndex(image,((int) (index << 8)+(int) (*p)));
+          index=ConstrainColormapIndex(image,(index << 8)+(*p));
           p++;
         }
       indexes[x]=(IndexPacket) index;
@@ -266,8 +268,8 @@ ModuleExport void RegisterMAPImage(void)
   entry->adjoin=MagickFalse;
   entry->raw=MagickTrue;
   entry->endian_support=MagickTrue;
-  entry->description=AcquireString("Colormap intensities and indices");
-  entry->module=AcquireString("MAP");
+  entry->description=ConstantString("Colormap intensities and indices");
+  entry->module=ConstantString("MAP");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -360,11 +362,12 @@ static MagickBooleanType WriteMAPImage(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  (void) SetImageColorspace(image,RGBColorspace);
+  if (image_info->colorspace == UndefinedColorspace)
+    (void) SetImageColorspace(image,RGBColorspace);
   /*
     Allocate colormap.
   */
@@ -392,11 +395,11 @@ static MagickBooleanType WriteMAPImage(const ImageInfo *image_info,Image *image)
   else
     for (i=0; i < (long) image->colors; i++)
     {
-      *q++=(unsigned char) (image->colormap[i].red >> 8);
+      *q++=(unsigned char) ((unsigned long) image->colormap[i].red >> 8);
       *q++=(unsigned char) image->colormap[i].red;
-      *q++=(unsigned char) (image->colormap[i].green >> 8);
+      *q++=(unsigned char) ((unsigned long) image->colormap[i].green >> 8);
       *q++=(unsigned char) image->colormap[i].green;
-      *q++=(unsigned char) (image->colormap[i].blue >> 8);
+      *q++=(unsigned char) ((unsigned long) image->colormap[i].blue >> 8);
       *q++=(unsigned char) image->colormap[i].blue;
     }
   (void) WriteBlob(image,packet_size*image->colors,colormap);
