@@ -59,6 +59,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/shear.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/transform.h"
@@ -106,7 +107,7 @@ static void InsertRow(unsigned char *p, int y, Image *image)
         for (x = 0; x < (long) image->columns; x++)
         {
           index = (IndexPacket) (*p);
-          ConstrainColormapIndex(image, index);
+          index = ConstrainColormapIndex(image, index);
           indexes[x] = index;
           *q++ = image->colormap[index];
           p++;
@@ -152,9 +153,9 @@ static void InsertFloatRow(double *p, int y, Image * image, double Min,
   for (x = 0; x < (long) image->columns; x++)
   {
     f = (double) QuantumRange *(*p - Min) / (Max - Min);
-    q->red = ScaleShortToQuantum(f);
-    q->green = ScaleShortToQuantum(f);
-    q->blue = ScaleShortToQuantum(f);
+    q->red=ScaleShortToQuantum((unsigned short) (f+0.5));
+    q->green=ScaleShortToQuantum((unsigned short) (f+0.5));
+    q->blue=ScaleShortToQuantum((unsigned short) (f+0.5));
     p++;
     q++;
   }
@@ -484,9 +485,9 @@ static Image *ReadMATImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
     for (i = 0; i < (long) image->colors; i++)
     {
-      image->colormap[i].red = ScaleCharToQuantum(i);
-      image->colormap[i].green = ScaleCharToQuantum(i);
-      image->colormap[i].blue = ScaleCharToQuantum(i);
+      image->colormap[i].red = ScaleCharToQuantum((unsigned char) i);
+      image->colormap[i].green = ScaleCharToQuantum((unsigned char) i);
+      image->colormap[i].blue = ScaleCharToQuantum((unsigned char) i);
     }
   }
 
@@ -644,8 +645,8 @@ ModuleExport void RegisterMATImage(void)
   entry=SetMagickInfo("MAT");
   entry->decoder=(DecoderHandler *) ReadMATImage;
   entry->seekable_stream=MagickTrue;
-  entry->description=AcquireString("MATLAB image format");
-  entry->module=AcquireString("MAT");
+  entry->description=ConstantString("MATLAB image format");
+  entry->module=ConstantString("MAT");
   (void) RegisterMagickInfo(entry);
 }
 
