@@ -17,7 +17,7 @@
 %                                 July 2001                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -51,6 +51,7 @@
 #include "magick/memory_.h"
 #include "magick/profile.h"
 #include "magick/splay-tree.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/token.h"
@@ -149,6 +150,7 @@ typedef struct _html_code
     val;
 } html_code;
 
+#include "magick/quantum.h"
 static html_code html_codes[] = {
 #ifdef HANDLE_GT_LT
   { 4,"&lt;",'<' },
@@ -211,7 +213,7 @@ static int convertHTMLcodes(char *s, int len)
             break;
         }
         if (o < 6)
-          (void) strcpy(s+1, s+1+o);
+          (void) CopyMagickString(s+1,s+1+o,strlen(s+1+o)+1);
         *s = val;
         return o;
       }
@@ -227,7 +229,7 @@ static int convertHTMLcodes(char *s, int len)
         if (html_codes[i].len <= len)
           if (stringnicmp(s, html_codes[i].code, html_codes[i].len) == 0)
             {
-              (void) strcpy(s+1, s+html_codes[i].len);
+              (void) CopyMagickString(s+1,s+html_codes[i].len,strlen(s+html_codes[i].len));
               *s = html_codes[i].val;
               return html_codes[i].len-1;
             }
@@ -366,7 +368,7 @@ static long parse8BIM(Image *ifile, Image *ofile)
               case 2:
                 name=(char *) AcquireMagickMemory(strlen(newstr)+MaxTextExtent);
                 if (name)
-                  (void) strcpy(name,newstr);
+                  (void) CopyMagickString(name,newstr,strlen(newstr)+MaxTextExtent);
                 break;
             }
             state++;
@@ -625,7 +627,7 @@ static long parse8BIMW(Image *ifile, Image *ofile)
               case 2:
                 name=(char *) AcquireMagickMemory(strlen(newstr)+MaxTextExtent);
                 if (name)
-                  (void) strcpy(name,newstr);
+                  (void) CopyMagickString(name,newstr,strlen(newstr)+MaxTextExtent);
                 break;
             }
             state++;
@@ -1039,12 +1041,12 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
   MagickBooleanType
     status;
 
-  size_t
-    length;
-
   StringInfo
     *profile;
 
+  size_t
+    length;
+
   void
     *blob;
 
@@ -1103,10 +1105,10 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
             c=ReadBlobByte(image);
             if (c == EOF)
               break;
-            WriteBlobByte(buff,c);
+            WriteBlobByte(buff,(unsigned char) c);
           }
         }
-      profile=AcquireStringInfo(GetBlobSize(buff));
+      profile=AcquireStringInfo((size_t) GetBlobSize(buff));
       SetStringInfoDatum(profile,GetBlobStreamData(buff));
       status=SetImageProfile(image,"iptc",profile);
       profile=DestroyStringInfo(profile);
@@ -1148,7 +1150,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
               blob=DetachBlob(buff->blob);
               blob=(char *) RelinquishMagickMemory(blob);
               buff=DestroyImage(buff);
-              ThrowReaderException(CoderError,"NoIPTCProfileIsAvailable");
+              ThrowReaderException(CoderError,"NoIPTCProfileAvailable");
             }
           iptc=AllocateImage((ImageInfo *) NULL);
           if (iptc == (Image *) NULL)
@@ -1216,7 +1218,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
             }
 #endif
         }
-      profile=AcquireStringInfo(GetBlobSize(buff));
+      profile=AcquireStringInfo((size_t) GetBlobSize(buff));
       SetStringInfoDatum(profile,GetBlobStreamData(buff));
       status=SetImageProfile(image,name,profile);
       profile=DestroyStringInfo(profile);
@@ -1244,9 +1246,9 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
         c=ReadBlobByte(image);
         if (c == EOF)
           break;
-        WriteBlobByte(buff,c);
+        WriteBlobByte(buff,(unsigned char) c);
       }
-      profile=AcquireStringInfo(GetBlobSize(buff));
+      profile=AcquireStringInfo((size_t) GetBlobSize(buff));
       SetStringInfoDatum(profile,GetBlobStreamData(buff));
       (void) SetImageProfile(image,"icc",profile);
       profile=DestroyStringInfo(profile);
@@ -1284,10 +1286,10 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
             c=ReadBlobByte(image);
             if (c == EOF)
               break;
-            WriteBlobByte(buff,c);
+            WriteBlobByte(buff,(unsigned char) c);
           }
         }
-      profile=AcquireStringInfo(GetBlobSize(buff));
+      profile=AcquireStringInfo((size_t) GetBlobSize(buff));
       /*
         subtract off the length of the 8BIM stuff.
       */
@@ -1339,8 +1341,8 @@ ModuleExport void RegisterMETAImage(void)
   entry->adjoin=MagickFalse;
   entry->stealth=MagickTrue;
   entry->seekable_stream=MagickTrue;
-  entry->description=AcquireString("Photoshop resource format");
-  entry->module=AcquireString("META");
+  entry->description=ConstantString("Photoshop resource format");
+  entry->module=ConstantString("META");
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("8BIMTEXT");
@@ -1349,8 +1351,8 @@ ModuleExport void RegisterMETAImage(void)
   entry->adjoin=MagickFalse;
   entry->stealth=MagickTrue;
   entry->seekable_stream=MagickTrue;
-  entry->description=AcquireString("Photoshop resource text format");
-  entry->module=AcquireString("META");
+  entry->description=ConstantString("Photoshop resource text format");
+  entry->module=ConstantString("META");
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("8BIMWTEXT");
@@ -1359,8 +1361,8 @@ ModuleExport void RegisterMETAImage(void)
   entry->adjoin=MagickFalse;
   entry->stealth=MagickTrue;
   entry->seekable_stream=MagickTrue;
-  entry->description=AcquireString("Photoshop resource wide text format");
-  entry->module=AcquireString("META");
+  entry->description=ConstantString("Photoshop resource wide text format");
+  entry->module=ConstantString("META");
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("APP1");
@@ -1369,8 +1371,8 @@ ModuleExport void RegisterMETAImage(void)
   entry->adjoin=MagickFalse;
   entry->stealth=MagickTrue;
   entry->seekable_stream=MagickTrue;
-  entry->description=AcquireString("Raw application information");
-  entry->module=AcquireString("META");
+  entry->description=ConstantString("Raw application information");
+  entry->module=ConstantString("META");
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("APP1JPEG");
@@ -1379,8 +1381,8 @@ ModuleExport void RegisterMETAImage(void)
   entry->adjoin=MagickFalse;
   entry->stealth=MagickTrue;
   entry->seekable_stream=MagickTrue;
-  entry->description=AcquireString("Raw JPEG binary data");
-  entry->module=AcquireString("META");
+  entry->description=ConstantString("Raw JPEG binary data");
+  entry->module=ConstantString("META");
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("EXIF");
@@ -1389,8 +1391,8 @@ ModuleExport void RegisterMETAImage(void)
   entry->adjoin=MagickFalse;
   entry->stealth=MagickTrue;
   entry->seekable_stream=MagickTrue;
-  entry->description=AcquireString("Exif digital camera binary data");
-  entry->module=AcquireString("META");
+  entry->description=ConstantString("Exif digital camera binary data");
+  entry->module=ConstantString("META");
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("XMP");
@@ -1399,8 +1401,8 @@ ModuleExport void RegisterMETAImage(void)
   entry->adjoin=MagickFalse;
   entry->stealth=MagickTrue;
   entry->seekable_stream=MagickTrue;
-  entry->description=AcquireString("Adobe XML metadata");
-  entry->module=AcquireString("META");
+  entry->description=ConstantString("Adobe XML metadata");
+  entry->module=ConstantString("META");
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("ICM");
@@ -1409,8 +1411,8 @@ ModuleExport void RegisterMETAImage(void)
   entry->adjoin=MagickFalse;
   entry->stealth=MagickTrue;
   entry->seekable_stream=MagickTrue;
-  entry->description=AcquireString("ICC Color Profile");
-  entry->module=AcquireString("META");
+  entry->description=ConstantString("ICC Color Profile");
+  entry->module=ConstantString("META");
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("ICC");
@@ -1419,8 +1421,8 @@ ModuleExport void RegisterMETAImage(void)
   entry->adjoin=MagickFalse;
   entry->stealth=MagickTrue;
   entry->seekable_stream=MagickTrue;
-  entry->description=AcquireString("ICC Color Profile");
-  entry->module=AcquireString("META");
+  entry->description=ConstantString("ICC Color Profile");
+  entry->module=ConstantString("META");
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("IPTC");
@@ -1429,8 +1431,8 @@ ModuleExport void RegisterMETAImage(void)
   entry->adjoin=MagickFalse;
   entry->stealth=MagickTrue;
   entry->seekable_stream=MagickTrue;
-  entry->description=AcquireString("IPTC Newsphoto");
-  entry->module=AcquireString("META");
+  entry->description=ConstantString("IPTC Newsphoto");
+  entry->module=ConstantString("META");
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("IPTCTEXT");
@@ -1439,8 +1441,8 @@ ModuleExport void RegisterMETAImage(void)
   entry->adjoin=MagickFalse;
   entry->stealth=MagickTrue;
   entry->seekable_stream=MagickTrue;
-  entry->description=AcquireString("IPTC Newsphoto text format");
-  entry->module=AcquireString("META");
+  entry->description=ConstantString("IPTC Newsphoto text format");
+  entry->module=ConstantString("META");
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("IPTCWTEXT");
@@ -1449,8 +1451,8 @@ ModuleExport void RegisterMETAImage(void)
   entry->adjoin=MagickFalse;
   entry->stealth=MagickTrue;
   entry->seekable_stream=MagickTrue;
-  entry->description=AcquireString("IPTC Newsphoto text format");
-  entry->module=AcquireString("META");
+  entry->description=ConstantString("IPTC Newsphoto text format");
+  entry->module=ConstantString("META");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -1519,20 +1521,20 @@ ModuleExport void UnregisterMETAImage(void)
 %
 */
 
-static long GetIPTCStream(unsigned char **info,unsigned long length)
+static size_t GetIPTCStream(unsigned char **info,unsigned long length)
 {
   int
     c;
 
-  long
-    info_length;
-
   register long
     i;
 
   register unsigned char
     *p;
 
+  size_t
+    info_length;
+
   unsigned char
     buffer[4];
 
@@ -1976,13 +1978,13 @@ static int formatIPTCfromBuffer(Image *ofile, char *s, long len)
     else
       (void) FormatMagickString(temp,MaxTextExtent,"%d#%d=",
         (unsigned int) dataset,(unsigned int) recnum);
-    WriteBlobString(ofile,temp);
+    (void) WriteBlobString(ofile,temp);
     formatString( ofile, (char *)str, taglen );
     str=(unsigned char *) RelinquishMagickMemory(str);
 
     tagsfound++;
   }
-  return tagsfound;
+  return ((int) tagsfound);
 }
 
 static int format8BIM(Image *ifile, Image *ofile)
@@ -1993,20 +1995,21 @@ static int format8BIM(Image *ifile, Image *ofile)
   unsigned int
     foundOSType;
 
-  MagickOffsetType
-    Size;
-
   int
     ID,
     resCount,
     i,
     c;
 
+  ssize_t
+    count;
+
   unsigned char
     *PString,
     *str;
 
-  resCount = foundOSType = 0; /* found the OSType */
+  resCount = 0;
+  foundOSType = 0; /* found the OSType */
 
   c =ReadBlobByte(ifile);
   while (c != EOF)
@@ -2037,7 +2040,7 @@ static int format8BIM(Image *ifile, Image *ofile)
     /*
       We found the OSType (8BIM) and now grab the ID, PString, and Size fields.
     */
-    ID = ReadBlobMSBShort(ifile);
+    ID = (int) ReadBlobMSBShort(ifile);
     if (ID < 0) return -1;
     {
       unsigned char
@@ -2066,16 +2069,16 @@ static int format8BIM(Image *ifile, Image *ofile)
         if (c == EOF) return -1;
       }
     }
-    Size = ReadBlobMSBLong(ifile);
-    if (Size < 0) return -1;
+    count = (ssize_t) ReadBlobMSBLong(ifile);
+    if (count < 0) return -1;
     /* make a buffer to hold the data and snag it from the input stream */
-    str=(unsigned char *) AcquireMagickMemory((size_t) Size);
+    str=(unsigned char *) AcquireMagickMemory((size_t) count);
     if (str == (unsigned char *) NULL)
       {
         printf("MemoryAllocationFailed");
         return 0;
       }
-    for (i=0; i<Size; i++)
+    for (i=0; i < (long) count; i++)
     {
       c=ReadBlobByte(ifile);
       if (c == EOF) return -1;
@@ -2095,20 +2098,18 @@ static int format8BIM(Image *ifile, Image *ofile)
             PString);
         else
           (void) FormatMagickString(temp,MaxTextExtent,"8BIM#%d=",ID);
-        WriteBlobString(ofile,temp);
+        (void) WriteBlobString(ofile,temp);
         if (ID == IPTC_ID)
           {
             formatString(ofile, "IPTC", 4);
-            formatIPTCfromBuffer(ofile, (char *)str, (long) Size);
+            formatIPTCfromBuffer(ofile, (char *)str, (long) count);
           }
         else
-          formatString(ofile, (char *)str, (long) Size);
+          formatString(ofile, (char *)str, (long) count);
       }
     str=(unsigned char *) RelinquishMagickMemory(str);
     PString=(unsigned char *) RelinquishMagickMemory(PString);
-
     resCount++;
-
     c=ReadBlobByte(ifile);
   }
   return resCount;
@@ -2117,12 +2118,12 @@ static int format8BIM(Image *ifile, Image *ofile)
 static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
   Image *image)
 {
+  const StringInfo
+    *profile;
+
   MagickBooleanType
     status;
 
-  StringInfo
-    *profile;
-
   size_t
     length;
 
@@ -2143,7 +2144,9 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
       /*
         Write 8BIM image.
       */
-      profile=GetImageProfile(image,"8BIM");
+      profile=GetImageProfile(image,"8bim");
+      if (profile == (StringInfo *) NULL)
+        profile=GetImageProfile(image,"iptc");
       if (profile == (StringInfo *) NULL)
         ThrowWriterException(CoderError,"No8BIMDataIsAvailable");
       status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
@@ -2169,7 +2172,7 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
       length=profile->length;
       length=GetIPTCStream(&info,length);
       if (length == 0)
-        ThrowWriterException(CoderError,"NoIPTCInfoWasFound");
+        ThrowWriterException(CoderError,"NoIPTCProfileAvailable");
       (void) WriteBlob(image,length,info);
       CloseBlob(image);
       return(MagickTrue);
@@ -2179,7 +2182,9 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
       Image
         *buff;
 
-      profile=GetImageProfile(image,"iptc");
+      profile=GetImageProfile(image,"8bim");
+      if (profile == (StringInfo *) NULL)
+        profile=GetImageProfile(image,"iptc");
       if (profile == (StringInfo *) NULL)
         ThrowWriterException(CoderError,"No8BIMDataIsAvailable");
       status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
@@ -2214,7 +2219,7 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
       length=profile->length;
       length=GetIPTCStream(&info,length);
       if (length == 0)
-        ThrowWriterException(CoderError,"NoIPTCInfoWasFound");
+        ThrowWriterException(CoderError,"NoIPTCProfileAvailable");
       status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
       if (status == MagickFalse)
         return(status);
