@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -60,6 +60,7 @@
 #include "magick/option.h"
 #include "magick/pixel.h"
 #include "magick/profile.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/statistic.h"
 #include "magick/string_.h"
@@ -91,7 +92,7 @@
 }
 #define PushCharPixel(pixel,p) \
 { \
-  pixel=(unsigned long) (*(p)); \
+  pixel=(unsigned char) (*(p)); \
   (p)++; \
 }
 #define PushLongPixel(pixel,p) \
@@ -102,7 +103,7 @@
 }
 #define PushShortPixel(pixel,p) \
 { \
-  pixel=(unsigned long) ((*(p) << 8) | *((p)+1)); \
+  pixel=(unsigned short) ((*(p) << 8) | *((p)+1)); \
   (p)+=2; \
 }
 
@@ -198,20 +199,17 @@ static voidpf AcquireZIPMemory(voidpf context,unsigned int items,
 }
 #endif
 
-static inline void PushRunlengthPacket(Image *image,const unsigned long depth,
-  unsigned char *pixels,size_t *length,PixelPacket *pixel,IndexPacket *index)
+static inline void PushRunlengthPacket(Image *image,const unsigned char *pixels,
+  size_t *length,PixelPacket *pixel,IndexPacket *index)
 {
-  register unsigned char
+  register const unsigned char
     *p;
 
-  unsigned long
-    quantum;
-
   p=pixels;
   if (image->storage_class == PseudoClass)
     {
       *index=0;
-      switch (depth)
+      switch (image->depth)
       {
         case 32:
         {
@@ -237,32 +235,41 @@ static inline void PushRunlengthPacket(Image *image,const unsigned long depth,
             CorruptImageError,"ImageDepthNotSupported","`%s'",image->filename);
       }
       *pixel=image->colormap[*index];
-      switch (depth)
+      switch (image->depth)
       {
-        case 32:
+        case 8:
         {
+          unsigned char
+            quantum;
+
           if (image->matte != MagickFalse)
             {
-              PushLongPixel(quantum,p);
-              pixel->opacity=(Quantum) (quantum >> (depth-QuantumDepth));
+              PushCharPixel(quantum,p);
+              pixel->opacity=ScaleCharToQuantum(quantum);
             }
           break;
         }
         case 16:
         {
+          unsigned short
+            quantum;
+
           if (image->matte != MagickFalse)
             {
               PushShortPixel(quantum,p);
-              pixel->opacity=(Quantum) (quantum >> (depth-QuantumDepth));
+              pixel->opacity=(Quantum) (quantum >> (image->depth-QuantumDepth));
             }
           break;
         }
-        case 8:
+        case 32:
         {
+          unsigned long
+            quantum;
+
           if (image->matte != MagickFalse)
             {
-              PushCharPixel(quantum,p);
-              pixel->opacity=ScaleCharToQuantum(quantum);
+              PushLongPixel(quantum,p);
+              pixel->opacity=(Quantum) (quantum >> (image->depth-QuantumDepth));
             }
           break;
         }
@@ -273,50 +280,13 @@ static inline void PushRunlengthPacket(Image *image,const unsigned long depth,
       *length=(size_t) (*p++)+1;
       return;
     }
-  switch (depth)
+  switch (image->depth)
   {
-    case 32:
-    {
-      PushLongPixel(quantum,p);
-      pixel->red=(Quantum) (quantum >> (depth-QuantumDepth));
-      PushLongPixel(quantum,p);
-      pixel->green=(Quantum) (quantum >> (depth-QuantumDepth));
-      PushLongPixel(quantum,p);
-      pixel->blue=(Quantum) (quantum >> (depth-QuantumDepth));
-      if (image->matte != MagickFalse)
-        {
-          PushLongPixel(quantum,p);
-          pixel->opacity=(Quantum) (quantum >> (depth-QuantumDepth));
-        }
-      if (image->colorspace == CMYKColorspace)
-        {
-          PushLongPixel(quantum,p);
-          *index=(IndexPacket) (quantum >> (depth-QuantumDepth));
-        }
-      break;
-    }
-    case 16:
-    {
-      PushShortPixel(quantum,p);
-      pixel->red=(Quantum) (quantum >> (depth-QuantumDepth));
-      PushShortPixel(quantum,p);
-      pixel->green=(Quantum) (quantum >> (depth-QuantumDepth));
-      PushShortPixel(quantum,p);
-      pixel->blue=(Quantum) (quantum >> (depth-QuantumDepth));
-      if (image->matte != MagickFalse)
-        {
-          PushShortPixel(quantum,p);
-          pixel->opacity=(Quantum) (quantum >> (depth-QuantumDepth));
-        }
-      if (image->colorspace == CMYKColorspace)
-        {
-          PushShortPixel(quantum,p);
-          *index=(IndexPacket) (quantum >> (depth-QuantumDepth));
-        }
-      break;
-    }
     case 8:
     {
+      unsigned char
+        quantum;
+
       PushCharPixel(quantum,p);
       pixel->red=ScaleCharToQuantum(quantum);
       PushCharPixel(quantum,p);
@@ -335,6 +305,52 @@ static inline void PushRunlengthPacket(Image *image,const unsigned long depth,
         }
       break;
     }
+    case 16:
+    {
+      unsigned short
+        quantum;
+
+      PushShortPixel(quantum,p);
+      pixel->red=(Quantum) (quantum >> (image->depth-QuantumDepth));
+      PushShortPixel(quantum,p);
+      pixel->green=(Quantum) (quantum >> (image->depth-QuantumDepth));
+      PushShortPixel(quantum,p);
+      pixel->blue=(Quantum) (quantum >> (image->depth-QuantumDepth));
+      if (image->matte != MagickFalse)
+        {
+          PushShortPixel(quantum,p);
+          pixel->opacity=(Quantum) (quantum >> (image->depth-QuantumDepth));
+        }
+      if (image->colorspace == CMYKColorspace)
+        {
+          PushShortPixel(quantum,p);
+          *index=(IndexPacket) (quantum >> (image->depth-QuantumDepth));
+        }
+      break;
+    }
+    case 32:
+    {
+      unsigned long
+        quantum;
+
+      PushLongPixel(quantum,p);
+      pixel->red=(Quantum) (quantum >> (image->depth-QuantumDepth));
+      PushLongPixel(quantum,p);
+      pixel->green=(Quantum) (quantum >> (image->depth-QuantumDepth));
+      PushLongPixel(quantum,p);
+      pixel->blue=(Quantum) (quantum >> (image->depth-QuantumDepth));
+      if (image->matte != MagickFalse)
+        {
+          PushLongPixel(quantum,p);
+          pixel->opacity=(Quantum) (quantum >> (image->depth-QuantumDepth));
+        }
+      if (image->colorspace == CMYKColorspace)
+        {
+          PushLongPixel(quantum,p);
+          *index=(IndexPacket) (quantum >> (image->depth-QuantumDepth));
+        }
+      break;
+    }
     default:
       (void) ThrowMagickException(&image->exception,GetMagickModule(),
         CorruptImageError,"ImageDepthNotSupported","`%s'",image->filename);
@@ -406,6 +422,9 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
   MagickStatusType
     flags;
 
+  register const unsigned char
+    *p;
+
   register IndexPacket
     *indexes;
 
@@ -416,9 +435,6 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
   register PixelPacket
     *q;
 
-  register unsigned char
-    *p;
-
   size_t
     length,
     packet_size;
@@ -426,16 +442,12 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
   ssize_t
     count;
 
-  StringInfo
-    *profile;
-
   unsigned char
     *compress_pixels,
     *pixels;
 
   unsigned long
-    colors,
-    depth;
+    colors;
 
 #if defined(HasZLIB)
   z_stream
@@ -467,6 +479,7 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   code=0;
   *id='\0';
+  (void) ResetMagickMemory(keyword,0,sizeof(keyword));
   version=0.0;
   do
   {
@@ -531,7 +544,7 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
                 break;
               if (c == (int) '=')
                 break;
-              if ((size_t) (p-keyword) < (MaxTextExtent-1))
+              if ((size_t) (p-keyword) < (MaxTextExtent/2))
                 *p++=(char) c;
               c=ReadBlobByte(image);
             } while (c != EOF);
@@ -623,12 +636,6 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
                       MagickColorspaceOptions,MagickFalse,options);
                     break;
                   }
-                if (LocaleCompare(keyword,"compose") == 0)
-                  {
-                    image->compose=(CompositeOperator) ParseMagickOption(
-                      MagickCompositeOptions,MagickFalse,options);
-                    break;
-                  }
                 if (LocaleCompare(keyword,"compression") == 0)
                   {
                     image->compression=(CompressionType) ParseMagickOption(
@@ -764,6 +771,9 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
                   }
                 if (LocaleNCompare(keyword,"profile-",8) == 0)
                   {
+                    StringInfo
+                      *profile;
+
                     if (profiles == (LinkedListInfo *) NULL)
                       profiles=NewLinkedList(0);
                     (void) AppendValueToLinkedList(profiles,
@@ -834,6 +844,17 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
                 (void) SetImageAttribute(image,keyword,options);
                 break;
               }
+              case 't':
+              case 'T':
+              {
+                if (LocaleCompare(keyword,"ticks-per-second") == 0)
+                  {
+                    image->ticks_per_second=(unsigned long) atol(options);
+                    break;
+                  }
+                (void) SetImageAttribute(image,keyword,options);
+                break;
+              }
               case 'u':
               case 'U':
               {
@@ -929,6 +950,9 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
         const char
           *name;
 
+        const StringInfo
+          *profile;
+
         /*
           Read image profiles.
         */
@@ -942,7 +966,7 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
         }
         profiles=DestroyLinkedList(profiles,RelinquishMagickMemory);
       }
-    depth=GetImageQuantumDepth(image,MagickFalse);
+    image->depth=GetImageQuantumDepth(image,MagickFalse);
     if (image->storage_class == PseudoClass)
       {
         /*
@@ -959,26 +983,26 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
             unsigned char
               *colormap;
 
-            unsigned long
-              pixel;
-
             /*
               Read image colormap from file.
             */
-            packet_size=(size_t) (3*depth/8);
+            packet_size=(size_t) (3*image->depth/8);
             colormap=(unsigned char *)
               AcquireMagickMemory(packet_size*image->colors);
             if (colormap == (unsigned char *) NULL)
               ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
             count=ReadBlob(image,packet_size*image->colors,colormap);
             p=colormap;
-            switch (depth)
+            switch (image->depth)
             {
               default:
                 ThrowReaderException(CorruptImageError,
                   "ImageDepthNotSupported");
               case 8:
               {
+                unsigned char
+                  pixel;
+
                 for (i=0; i < (long) image->colors; i++)
                 {
                   PushCharPixel(pixel,p);
@@ -992,6 +1016,9 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
               }
               case 16:
               {
+                unsigned short
+                  pixel;
+
                 for (i=0; i < (long) image->colors; i++)
                 {
                   PushShortPixel(pixel,p);
@@ -1005,6 +1032,9 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
               }
               case 32:
               {
+                unsigned long
+                  pixel;
+
                 for (i=0; i < (long) image->colors; i++)
                 {
                   PushLongPixel(pixel,p);
@@ -1026,13 +1056,13 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
     /*
       Allocate image pixels.
     */
-    packet_size=(size_t) (depth/8);
+    packet_size=(size_t) (image->depth/8);
     if (image->storage_class == DirectClass)
-      packet_size=(size_t) (3*depth/8);
+      packet_size=(size_t) (3*image->depth/8);
     if (image->matte != MagickFalse)
-      packet_size+=depth/8;
+      packet_size+=image->depth/8;
     if (image->colorspace == CMYKColorspace)
-      packet_size+=depth/8;
+      packet_size+=image->depth/8;
     if (image->compression == RLECompression)
       packet_size++;
     length=packet_size*image->columns;
@@ -1104,7 +1134,7 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
               if (code >= 0)
                 status=MagickTrue;
             }
-          (void) ExportQuantumPixels(image,quantum_type,0,pixels);
+          status=ExportQuantumPixels(image,quantum_type,0,pixels);
           break;
         }
 #endif
@@ -1147,7 +1177,7 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
               if (code >= 0)
                 status=MagickTrue;
             }
-          (void) ExportQuantumPixels(image,quantum_type,0,pixels);
+          status=ExportQuantumPixels(image,quantum_type,0,pixels);
           break;
         }
 #endif
@@ -1163,8 +1193,8 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
           {
             if (length == 0)
               {
-                count=ReadBlob(image,packet_size,pixels);
-                PushRunlengthPacket(image,depth,pixels,&length,&pixel,&index);
+                p=ReadBlobStream(image,packet_size,pixels,&count);
+                PushRunlengthPacket(image,p,&length,&pixel,&index);
               }
             length--;
             if ((image->storage_class == PseudoClass) ||
@@ -1176,8 +1206,8 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
         }
         default:
         {
-          count=ReadBlob(image,packet_size*image->columns,pixels);
-          (void) ExportQuantumPixels(image,quantum_type,0,pixels);
+          p=ReadBlobStream(image,packet_size*image->columns,pixels,&count);
+          status=ExportQuantumPixels(image,quantum_type,0,p);
           break;
         }
       }
@@ -1277,10 +1307,10 @@ ModuleExport void RegisterMIFFImage(void)
   entry->decoder=(DecoderHandler *) ReadMIFFImage;
   entry->encoder=(EncoderHandler *) WriteMIFFImage;
   entry->magick=(MagickHandler *) IsMIFF;
-  entry->description=AcquireString("Magick Image File Format");
+  entry->description=ConstantString("Magick Image File Format");
   if (*version != '\0')
-    entry->version=AcquireString(version);
-  entry->module=AcquireString("MIFF");
+    entry->version=ConstantString(version);
+  entry->module=ConstantString("MIFF");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -1323,7 +1353,8 @@ ModuleExport void UnregisterMIFFImage(void)
 %
 %  The format of the WriteMIFFImage method is:
 %
-%      MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
+%        Image *image)
 %
 %  Compression code contributed by Kyle Shorter.
 %
@@ -1336,8 +1367,8 @@ ModuleExport void UnregisterMIFFImage(void)
 %
 */
 
-static inline size_t PopRunlengthPacket(Image *image,const unsigned long depth,
-  unsigned char *pixels,size_t length,PixelPacket pixel,IndexPacket index)
+static inline size_t PopRunlengthPacket(Image *image,unsigned char *pixels,
+  size_t length,PixelPacket pixel,IndexPacket index)
 {
   register unsigned char
     *q;
@@ -1348,15 +1379,15 @@ static inline size_t PopRunlengthPacket(Image *image,const unsigned long depth,
   q=pixels;
   if (image->storage_class != DirectClass)
     {
-      switch (depth)
+      switch (image->depth)
       {
         case 32:
         {
-          *q++=(unsigned char) (index >> 24);
-          *q++=(unsigned char) (index >> 16);
+          *q++=(unsigned char) ((unsigned long) index >> 24);
+          *q++=(unsigned char) ((unsigned long) index >> 16);
         }
         case 16:
-          *q++=(unsigned char) (index >> 8);
+          *q++=(unsigned char) ((unsigned long) index >> 8);
         case 8:
         {
           *q++=(unsigned char) index;
@@ -1366,7 +1397,7 @@ static inline size_t PopRunlengthPacket(Image *image,const unsigned long depth,
           (void) ThrowMagickException(&image->exception,GetMagickModule(),
             CorruptImageError,"ImageDepthNotSupported","`%s'",image->filename);
       }
-      switch (depth)
+      switch (image->depth)
       {
         case 32:
         {
@@ -1402,7 +1433,7 @@ static inline size_t PopRunlengthPacket(Image *image,const unsigned long depth,
       *q++=(unsigned char) length;
       return((size_t) (q-pixels));
     }
-  switch (depth)
+  switch (image->depth)
   {
     case 32:
     {
@@ -1531,10 +1562,6 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
     *compress_pixels,
     *pixels;
 
-  unsigned long
-    depth,
-    quality;
-
 #if defined(HasZLIB)
   z_stream
     zip_info;
@@ -1559,13 +1586,11 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
     /*
       Allocate image pixels.
     */
-    depth=GetImageQuantumDepth(image,MagickTrue);
+    image->depth=GetImageQuantumDepth(image,MagickTrue);
     if ((image->storage_class == PseudoClass) &&
-        (image->colors > (1UL << depth)))
-      image->storage_class=DirectClass;
+        (image->colors > (1UL << image->depth)))
+      (void) SetImageStorageClass(image,DirectClass);
     compression=NoCompression;
-    quality=(unsigned long)
-      (image->quality == UndefinedCompressionQuality ? 75 : image->quality);
     switch (image->compression)
     {
 #if defined(HasZLIB)
@@ -1579,15 +1604,15 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
       default:
         break;
     }
-    packet_size=(size_t) (depth/8);
+    packet_size=(size_t) (image->depth/8);
     if (image->storage_class == DirectClass)
-      packet_size=(size_t) (3*depth/8);
+      packet_size=(size_t) (3*image->depth/8);
     if (image->matte != MagickFalse)
-      packet_size+=depth/8;
+      packet_size+=image->depth/8;
     if (image->colorspace == CMYKColorspace)
-      packet_size+=depth/8;
+      packet_size+=image->depth/8;
     if (compression == RLECompression)
-      packet_size+=depth/8;
+      packet_size+=image->depth/8;
     length=packet_size*image->columns;
     pixels=(unsigned char *) AcquireMagickMemory(length);
     length=(size_t) Max(BZipMaxExtent(packet_size*image->columns),
@@ -1609,7 +1634,8 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
       MagickOptionToMnemonic(MagickBooleanOptions,(long) image->matte));
     (void) WriteBlobString(image,buffer);
     (void) FormatMagickString(buffer,MaxTextExtent,
-      "columns=%lu  rows=%lu  depth=%lu\n",image->columns,image->rows,depth);
+      "columns=%lu  rows=%lu  depth=%lu\n",image->columns,image->rows,
+      image->depth);
     (void) WriteBlobString(image,buffer);
     if (image->colorspace != UndefinedColorspace)
       {
@@ -1650,15 +1676,18 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
             image->page.x,image->page.y);
           (void) WriteBlobString(image,buffer);
         }
-    if ((GetNextImageInList(image) != (Image *) NULL) || (GetPreviousImageInList(image) != (Image *) NULL))
+    if ((GetNextImageInList(image) != (Image *) NULL) ||
+        (GetPreviousImageInList(image) != (Image *) NULL))
       {
         if (image->scene == 0)
           (void) FormatMagickString(buffer,MaxTextExtent,
-            "iterations=%lu  delay=%lu\n",image->iterations,image->delay);
+            "iterations=%lu  delay=%lu  ticks-per-second=%lu\n",
+            image->iterations,image->delay,image->ticks_per_second);
         else
           (void) FormatMagickString(buffer,MaxTextExtent,
-            "scene=%lu  iterations=%lu  delay=%lu\n",image->scene,
-            image->iterations,image->delay);
+            "scene=%lu  iterations=%lu  delay=%lu  ticks-per-second=%lu\n",
+            image->scene,image->iterations,image->delay,
+            image->ticks_per_second);
         (void) WriteBlobString(image,buffer);
       }
     else
@@ -1681,6 +1710,12 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
               image->delay);
             (void) WriteBlobString(image,buffer);
           }
+        if (image->ticks_per_second != UndefinedTicksPerSecond)
+          {
+            (void) FormatMagickString(buffer,MaxTextExtent,
+              "ticks-per-second=%lu\n",image->ticks_per_second);
+            (void) WriteBlobString(image,buffer);
+          }
       }
     if (image->dispose != UndefinedDispose)
       {
@@ -1688,12 +1723,6 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
           MagickOptionToMnemonic(MagickDisposeOptions,image->dispose));
         (void) WriteBlobString(image,buffer);
       }
-    if (image->compose != UndefinedCompositeOp)
-      {
-        (void) FormatMagickString(buffer,MaxTextExtent,"compose=%s\n",
-          MagickOptionToMnemonic(MagickCompositeOptions,image->compose));
-        (void) WriteBlobString(image,buffer);
-      }
     if (image->rendering_intent != UndefinedIntent)
       {
         (void) FormatMagickString(buffer,MaxTextExtent,
@@ -1824,7 +1853,7 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
         /*
           Allocate colormap.
         */
-        packet_size=(size_t) (3*depth/8);
+        packet_size=(size_t) (3*image->depth/8);
         colormap=(unsigned char *)
           AcquireMagickMemory(packet_size*image->colors*sizeof(*colormap));
         if (colormap == (unsigned char *) NULL)
@@ -1835,7 +1864,7 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
         q=colormap;
         for (i=0; i < (long) image->colors; i++)
         {
-          switch (depth)
+          switch (image->depth)
           {
             default:
               ThrowWriterException(CorruptImageError,"ImageDepthNotSupported");
@@ -1905,7 +1934,8 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
               zip_info.zalloc=AcquireZIPMemory;
               zip_info.zfree=RelinquishZIPMemory;
               zip_info.opaque=(voidpf) NULL;
-              code=deflateInit(&zip_info,(int) Min(quality/10,9));
+              code=deflateInit(&zip_info,(int) (image->quality ==
+                UndefinedCompressionQuality ? 7 : Min(image->quality/10,9)));
               if (code >= 0)
                 status=MagickTrue;
             }
@@ -1943,7 +1973,7 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
                 if (code == Z_STREAM_END)
                   break;
               }
-              status=(MagickBooleanType) (deflateEnd(&zip_info) == MagickFalse);
+              status=deflateEnd(&zip_info) == 0 ? MagickTrue : MagickFalse;
             }
           break;
         }
@@ -1956,7 +1986,8 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
               bzip_info.bzalloc=AcquireBZIPMemory;
               bzip_info.bzfree=RelinquishBZIPMemory;
               bzip_info.opaque=(void *) NULL;
-              code=BZ2_bzCompressInit(&bzip_info,(int) Min(quality/10,9),
+              code=BZ2_bzCompressInit(&bzip_info,(int) (image->quality ==
+                UndefinedCompressionQuality ? 7 : Min(image->quality/10,9)),
                 (int) image_info->verbose,0);
               if (code >= 0)
                 status=MagickTrue;
@@ -1996,8 +2027,8 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
                 if (code == BZ_STREAM_END)
                   break;
               }
-              status=(MagickBooleanType)
-                (BZ2_bzCompressEnd(&bzip_info) == MagickFalse);
+              status=BZ2_bzCompressEnd(&bzip_info) == 0 ? MagickTrue :
+                MagickFalse;
             }
           break;
         }
@@ -2020,7 +2051,7 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
             else
               {
                 if (x > 0)
-                  q+=PopRunlengthPacket(image,depth,q,length,pixel,index);
+                  q+=PopRunlengthPacket(image,q,length,pixel,index);
                 length=0;
               }
             pixel=(*p);
@@ -2028,7 +2059,7 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
               index=indexes[x];
             p++;
           }
-          q+=PopRunlengthPacket(image,depth,q,length,pixel,index);
+          q+=PopRunlengthPacket(image,q,length,pixel,index);
           (void) WriteBlob(image,(size_t) (q-pixels),pixels);
           break;
         }
