@@ -17,7 +17,7 @@
 %                                 July 1999                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -53,6 +53,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/resource_.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/transform.h"
@@ -164,6 +165,7 @@ static Image *ReadMPEGImage(const ImageInfo *image_info,
       return((Image *) NULL);
     }
   CloseBlob(image);
+  (void) DestroyImageList(image);
   /*
     Convert MPEG to PPM with delegate.
   */
@@ -184,7 +186,7 @@ static Image *ReadMPEGImage(const ImageInfo *image_info,
     image=ReadImage(read_info,exception);
     if (image == (Image *) NULL)
       break;
-    (void) strcpy(image->magick,image_info->magick);
+    (void) CopyMagickString(image->magick,image_info->magick,MaxTextExtent);
     image->scene=(unsigned long) i;
     AppendImageToList(&images,image);
     if (read_info->number_scenes != 0)
@@ -239,24 +241,24 @@ ModuleExport void RegisterMPEGImage(void)
   entry->encoder=(EncoderHandler *) WriteMPEGImage;
   entry->magick=(MagickHandler *) IsMPEG;
   entry->blob_support=MagickFalse;
-  entry->description=AcquireString("MPEG Video Stream");
-  entry->module=AcquireString("MPEG");
+  entry->description=ConstantString("MPEG Video Stream");
+  entry->module=ConstantString("MPEG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("MPG");
   entry->decoder=(DecoderHandler *) ReadMPEGImage;
   entry->encoder=(EncoderHandler *) WriteMPEGImage;
   entry->magick=(MagickHandler *) IsMPEG;
   entry->blob_support=MagickFalse;
-  entry->description=AcquireString("MPEG Video Stream");
-  entry->module=AcquireString("MPEG");
+  entry->description=ConstantString("MPEG Video Stream");
+  entry->module=ConstantString("MPEG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("M2V");
   entry->decoder=(DecoderHandler *) ReadMPEGImage;
   entry->encoder=(EncoderHandler *) WriteMPEGImage;
   entry->magick=(MagickHandler *) IsMPEG;
   entry->blob_support=MagickFalse;
-  entry->description=AcquireString("MPEG Video Stream");
-  entry->module=AcquireString("MPEG");
+  entry->description=ConstantString("MPEG Video Stream");
+  entry->module=ConstantString("MPEG");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -356,6 +358,9 @@ static MagickBooleanType WriteMPEGParameterFiles(const ImageInfo *image_info,
       27, 29, 35, 38, 46, 56, 69, 83
     };
 
+  unsigned long
+    delay;
+
   /*
     Write parameter file (see mpeg2encode documentation for details).
   */
@@ -426,11 +431,14 @@ static MagickBooleanType WriteMPEGParameterFiles(const ImageInfo *image_info,
   (void) fprintf(file,"1\n");  /* input picture file format */
   count=0;
   for (p=image; p != (Image *) NULL; p=GetNextImageInList(p))
-    count+=Max((p->delay+1)/3,1);
+  {
+    delay=100*p->delay/Max(p->ticks_per_second,1);
+    count+=Max((delay+1)/3,1);
+  }
   (void) fprintf(file,"%lu\n",(unsigned long) count); /* number of frames */
   (void) fprintf(file,"0\n");  /* number of first frame */
   (void) fprintf(file,"00:00:00:00\n");  /* timecode of first frame */
-  mpeg=(MagickBooleanType) (LocaleCompare(image_info->magick,"M2V") != 0);
+  mpeg=LocaleCompare(image_info->magick,"M2V") != 0 ? MagickTrue : MagickFalse;
   if (image->quality > 98)
     (void) fprintf(file,"1\n");
   else
@@ -523,7 +531,8 @@ static MagickBooleanType WriteMPEGParameterFiles(const ImageInfo *image_info,
   return(MagickTrue);
 }
 
-static MagickBooleanType WriteMPEGImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteMPEGImage(const ImageInfo *image_info,
+  Image *image)
 {
   char
     basename[MaxTextExtent],
@@ -556,6 +565,7 @@ static MagickBooleanType WriteMPEGImage(const ImageInfo *image_info,Image *image
 
   unsigned long
     count,
+    delay,
     scene;
 
   /*
@@ -627,7 +637,8 @@ static MagickBooleanType WriteMPEGImage(const ImageInfo *image_info,Image *image
     blob=(unsigned char *) NULL;
     length=0;
     scene=p->scene;
-    for (i=0; i < (long) Max((p->delay+1)/3,1); i++)
+    delay=100*p->delay/Max(p->ticks_per_second,1);
+    for (i=0; i < (long) Max((delay+1)/3,1); i++)
     {
       p->scene=count;
       count++;
@@ -655,7 +666,7 @@ static MagickBooleanType WriteMPEGImage(const ImageInfo *image_info,Image *image
         case 1:
         {
           blob=(unsigned char *)
-            FileToBlob(previous_image,&length,&image->exception);
+            FileToBlob(previous_image,~0,&length,&image->exception);
         }
         default:
         {
@@ -697,7 +708,8 @@ static MagickBooleanType WriteMPEGImage(const ImageInfo *image_info,Image *image
   count=0;
   for (p=coalesce_image; p != (Image *) NULL; p=GetNextImageInList(p))
   {
-    for (i=0; i < (long) Max((p->delay+1)/3,1); i++)
+    delay=100*p->delay/Max(p->ticks_per_second,1);
+    for (i=0; i < (long) Max((delay+1)/3,1); i++)
     {
       (void) FormatMagickString(p->filename,MaxTextExtent,"%s.%lu.yuv",
         basename,count++);
