@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -44,6 +44,7 @@
 #include "magick/blob-private.h"
 #include "magick/color.h"
 #include "magick/color-private.h"
+#include "magick/colorspace-private.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
 #include "magick/image.h"
@@ -51,6 +52,8 @@
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
+#include "magick/pixel-private.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 
@@ -95,8 +98,20 @@ static Image *ReadNULLImage(const ImageInfo *image_info,
   Image
     *image;
 
-  MagickBooleanType
-    status;
+  long
+    y;
+
+  MagickPixelPacket
+    background;
+
+  register IndexPacket
+    *indexes;
+
+  register long
+    x;
+
+  register PixelPacket
+    *q;
 
   /*
     Initialize Image structure.
@@ -104,7 +119,8 @@ static Image *ReadNULLImage(const ImageInfo *image_info,
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
@@ -112,18 +128,26 @@ static Image *ReadNULLImage(const ImageInfo *image_info,
     image->columns=1;
   if (image->rows == 0)
     image->rows=1;
-  (void) CopyMagickString(image->filename,image_info->filename,MaxTextExtent);
-  status=QueryColorDatabase((char *) image_info->filename,
-    &image->background_color,exception);
-  if (status == MagickFalse)
+  image->matte=MagickTrue;
+  GetMagickPixelPacket(image,&background);
+  background.opacity=TransparentOpacity;
+  if (image->colorspace == CMYKColorspace)
+    RGBtoCMYK(&background);
+  for (y=0; y < (long) image->rows; y++)
+  {
+    q=SetImagePixels(image,0,y,image->columns,1);
+    if (q == (PixelPacket *) NULL)
+      break;
+    indexes=GetIndexes(image);
+    for (x=0; x < (long) image->columns; x++)
     {
-      image=DestroyImage(image);
-      return((Image *) NULL);
+      SetPixelPacket(&background,q,indexes);
+      q++;
+      indexes++;
     }
-  if (AllocateImageColormap(image,1) == MagickFalse)
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-  image->colormap[0]=image->background_color;
-  SetImageBackgroundColor(image);
+    if (SyncImagePixels(image) == MagickFalse)
+      break;
+  }
   return(GetFirstImageInList(image));
 }
 
@@ -159,8 +183,8 @@ ModuleExport void RegisterNULLImage(void)
   entry->decoder=(DecoderHandler *) ReadNULLImage;
   entry->encoder=(EncoderHandler *) WriteNULLImage;
   entry->adjoin=MagickFalse;
-  entry->description=AcquireString("Constant image of uniform color");
-  entry->module=AcquireString("NULL");
+  entry->description=ConstantString("Constant image of uniform color");
+  entry->module=ConstantString("NULL");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -223,6 +247,6 @@ static MagickBooleanType WriteNULLImage(const ImageInfo *image_info,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   return(MagickTrue);
 }
