@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -40,6 +40,7 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/attribute.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
 #include "magick/client.h"
@@ -55,11 +56,11 @@
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
-#include "magick/mogrify.h"
 #include "magick/monitor.h"
 #include "magick/montage.h"
 #include "magick/resize.h"
 #include "magick/shear.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/transform.h"
@@ -399,30 +400,23 @@ static MagickBooleanType IsPCD(const unsigned char *magick,const size_t length)
 static Image *OverviewImage(const ImageInfo *image_info,Image *image,
   ExceptionInfo *exception)
 {
-  char
-    *commands[3];
-
   Image
     *montage_image;
 
-  ImageInfo
-    *mogrify_info;
-
   MontageInfo
     *montage_info;
 
-  /*
-    Create image tiles.
-  */
-  mogrify_info=CloneImageInfo(image_info);
-  commands[0]=(char *) GetClientName();
-  commands[1]=(char *) "-label";
-  commands[2]=(char *) DefaultTileLabel;
-  (void) MogrifyImages(mogrify_info,3,commands,&image);
-  mogrify_info=DestroyImageInfo(mogrify_info);
+  register Image
+    *p;
+
   /*
     Create the PCD Overview image.
   */
+  for (p=image; p != (Image *) NULL; p=p->next)
+  {
+    (void) DeleteImageAttribute(p,"label");
+    (void) SetImageAttribute(p,"label",DefaultTileLabel);
+  }
   montage_info=CloneMontageInfo(image_info,(MontageInfo *) NULL);
   (void) CopyMagickString(montage_info->filename,image_info->filename,
     MaxTextExtent);
@@ -434,6 +428,61 @@ static Image *OverviewImage(const ImageInfo *image_info,Image *image,
   return(montage_image);
 }
 
+static void Upsample(const unsigned long width,const unsigned long height,
+  const unsigned long scaled_width,unsigned char *pixels)
+{
+  register long
+    x,
+    y;
+
+  register unsigned char
+    *p,
+    *q,
+    *r;
+
+  /*
+    Create a new image that is a integral size greater than an existing one.
+  */
+  assert(pixels != (unsigned char *) NULL);
+  for (y=0; y < (long) height; y++)
+  {
+    p=pixels+(height-1-y)*scaled_width+(width-1);
+    q=pixels+((height-1-y) << 1)*scaled_width+((width-1) << 1);
+    *q=(*p);
+    *(q+1)=(*(p));
+    for (x=1; x < (long) width; x++)
+    {
+      p--;
+      q-=2;
+      *q=(*p);
+      *(q+1)=(unsigned char) ((((unsigned long) *p)+
+        ((unsigned long) *(p+1))+1) >> 1);
+    }
+  }
+  for (y=0; y < (long) (height-1); y++)
+  {
+    p=pixels+((unsigned long) y << 1)*scaled_width;
+    q=p+scaled_width;
+    r=q+scaled_width;
+    for (x=0; x < (long) (width-1); x++)
+    {
+      *q=(unsigned char) ((((unsigned long) *p)+((unsigned long) *r)+1) >> 1);
+      *(q+1)=(unsigned char) ((((unsigned long) *p)+((unsigned long) *(p+2))+
+        ((unsigned long) *r)+((unsigned long) *(r+2))+2) >> 2);
+      q+=2;
+      p+=2;
+      r+=2;
+    }
+    *q++=(unsigned char) ((((unsigned long) *p++)+
+      ((unsigned long) *r++)+1) >> 1);
+    *q++=(unsigned char) ((((unsigned long) *p++)+
+      ((unsigned long) *r++)+1) >> 1);
+  }
+  p=pixels+(2*height-2)*scaled_width;
+  q=pixels+(2*height-1)*scaled_width;
+  (void) CopyMagickMemory(q,p,(size_t) (2*width));
+}
+
 static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
   Image
@@ -644,7 +693,6 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
         image->colorspace=YCCColorspace;
         if (LocaleCompare(image_info->magick,"PCDS") == 0)
           image->colorspace=sRGBColorspace;
-        SetImageColorspace(image,RGBColorspace);
         if (j < (long) number_images)
           {
             /*
@@ -802,7 +850,6 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   image->colorspace=YCCColorspace;
   if (LocaleCompare(image_info->magick,"PCDS") == 0)
     image->colorspace=sRGBColorspace;
-  SetImageColorspace(image,RGBColorspace);
   return(GetFirstImageInList(image));
 }
 
@@ -839,15 +886,15 @@ ModuleExport void RegisterPCDImage(void)
   entry->encoder=(EncoderHandler *) WritePCDImage;
   entry->magick=(MagickHandler *) IsPCD;
   entry->adjoin=MagickFalse;
-  entry->description=AcquireString("Photo CD");
-  entry->module=AcquireString("PCD");
+  entry->description=ConstantString("Photo CD");
+  entry->module=ConstantString("PCD");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PCDS");
   entry->decoder=(DecoderHandler *) ReadPCDImage;
   entry->encoder=(EncoderHandler *) WritePCDImage;
   entry->adjoin=MagickFalse;
-  entry->description=AcquireString("Photo CD");
-  entry->module=AcquireString("PCD");
+  entry->description=ConstantString("Photo CD");
+  entry->module=ConstantString("PCD");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -904,7 +951,7 @@ ModuleExport void UnregisterPCDImage(void)
 */
 
 static MagickBooleanType WritePCDTile(const ImageInfo *image_info,Image *image,
-  char *page_geometry,char *tile_geometry)
+  const char *page_geometry,const char *tile_geometry)
 {
   GeometryInfo
     geometry_info;
@@ -973,7 +1020,8 @@ static MagickBooleanType WritePCDTile(const ImageInfo *image_info,Image *image,
       tile_image=bordered_image;
     }
   (void) TransformImage(&tile_image,(char *) NULL,tile_geometry);
-  (void) SetImageColorspace(tile_image,YCCColorspace);
+  if (image_info->colorspace == UndefinedColorspace)
+    (void) SetImageColorspace(tile_image,YCCColorspace);
   downsample_image=ResizeImage(tile_image,tile_image->columns/2,
     tile_image->rows/2,TriangleFilter,1.0,&image->exception);
   if (downsample_image == (Image *) NULL)
@@ -1063,7 +1111,8 @@ static MagickBooleanType WritePCDImage(const ImageInfo *image_info,Image *image)
   status=OpenBlob(image_info,pcd_image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  (void) SetImageColorspace(pcd_image,RGBColorspace);
+  if (image_info->colorspace == UndefinedColorspace)
+    (void) SetImageColorspace(pcd_image,RGBColorspace);
   /*
     Write PCD image header.
   */
@@ -1100,12 +1149,9 @@ static MagickBooleanType WritePCDImage(const ImageInfo *image_info,Image *image)
   /*
     Write PCD tiles.
   */
-  status=WritePCDTile(image_info,pcd_image,(char *) "768x512>",
-    (char *) "192x128");
-  status=WritePCDTile(image_info,pcd_image,(char *) "768x512>",
-    (char *) "384x256");
-  status=WritePCDTile(image_info,pcd_image,(char *) "768x512>",
-    (char *) "768x512");
+  status=WritePCDTile(image_info,pcd_image,"768x512>","192x128");
+  status=WritePCDTile(image_info,pcd_image,"768x512>","384x256");
+  status=WritePCDTile(image_info,pcd_image,"768x512>","768x512");
   CloseBlob(pcd_image);
   if (pcd_image != image)
     pcd_image=DestroyImage(pcd_image);
