@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -48,6 +48,7 @@
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
 #include "magick/delegate.h"
+#include "magick/draw.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
 #include "magick/geometry.h"
@@ -59,6 +60,7 @@
 #include "magick/monitor.h"
 #include "magick/profile.h"
 #include "magick/resource_.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/token.h"
@@ -148,15 +150,12 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
     density[MaxTextExtent],
     filename[MaxTextExtent],
     geometry[MaxTextExtent],
-    options[MaxTextExtent];
+    options[MaxTextExtent],
+    input_filename[MaxTextExtent];
 
   const DelegateInfo
     *delegate_info;
 
-  double
-    dx_resolution,
-    dy_resolution;
-
   Image
     *image,
     *next_image;
@@ -168,6 +167,9 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
     cmyk,
     status;
 
+  PointInfo
+    delta;
+
   RectangleInfo
     bounding_box,
     page;
@@ -205,11 +207,19 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image=DestroyImageList(image);
       return((Image *) NULL);
     }
+  status=AcquireUniqueSymbolicLink(image_info->filename,input_filename);
+  if (status == MagickFalse)
+    {
+      ThrowFileException(exception,FileOpenError,"UnableToCreateTemporaryFile",
+        image_info->filename);
+      image=DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Set the page density.
   */
-  dx_resolution=DefaultResolution;
-  dy_resolution=DefaultResolution;
+  delta.x=DefaultResolution;
+  delta.y=DefaultResolution;
   if ((image->x_resolution == 0.0) || (image->y_resolution == 0.0))
     {
       GeometryInfo
@@ -234,7 +244,9 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
   (void) ResetMagickMemory(&bounding_box,0,sizeof(bounding_box));
   (void) ResetMagickMemory(&bounds,0,sizeof(bounds));
   (void) ResetMagickMemory(&page,0,sizeof(page));
-  for (p=command; (c=ReadBlobByte(image)) != EOF; )
+  (void) ResetMagickMemory(command,0,sizeof(command));
+  p=command;
+	for (c=ReadBlobByte(image); c != EOF; c=ReadBlobByte(image))
   {
     if (image_info->page != (char *) NULL)
       continue;
@@ -242,7 +254,8 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
       Note PCL elements.
     */
     *p++=(char) c;
-    if ((c != (int) '/') && ((size_t) (p-command) < (MaxTextExtent-1)))
+    if ((c != (int) '/') && (c != '\n') &&
+        ((size_t) (p-command) < (MaxTextExtent/2)))
       continue;
     *p='\0';
     p=command;
@@ -294,9 +307,9 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (image_info->page != (char *) NULL)
     (void) ParseAbsoluteGeometry(image_info->page,&page);
   page.width=(unsigned long)
-    (page.width*image->x_resolution/dx_resolution+0.5);
+    (page.width*image->x_resolution/delta.x+0.5);
   page.height=(unsigned long)
-    (page.height*image->y_resolution/dy_resolution+0.5);
+    (page.height*image->y_resolution/delta.y+0.5);
   image=DestroyImage(image);
   (void) FormatMagickString(geometry,MaxTextExtent,"%lux%lu",
     page.width,page.height);
@@ -334,10 +347,12 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
     GetDelegateCommands(delegate_info),
     read_info->antialias != MagickFalse ? 4 : 1,
     read_info->antialias != MagickFalse ? 4 : 1,geometry,density,options,
-    read_info->filename,image_info->filename);
-  status=(MagickBooleanType) SystemCommand(read_info->verbose,command);
+    read_info->filename,input_filename);
+  status=SystemCommand(read_info->verbose,command) != 0 ? MagickTrue :
+    MagickFalse;
   image=ReadImage(read_info,exception);
   (void) RelinquishUniqueFileResource(read_info->filename);
+  (void) RelinquishUniqueFileResource(input_filename);
   read_info=DestroyImageInfo(read_info);
   if (image == (Image *) NULL)
     ThrowReaderException(DelegateError,"PCLDelegateFailed");
@@ -397,8 +412,8 @@ ModuleExport void RegisterPCLImage(void)
   entry->encoder=(EncoderHandler *) WritePCLImage;
   entry->magick=(MagickHandler *) IsPCL;
   entry->adjoin=MagickFalse;
-  entry->description=AcquireString("Printer Control Language");
-  entry->module=AcquireString("PCL");
+  entry->description=ConstantString("Printer Control Language");
+  entry->module=ConstantString("PCL");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -491,7 +506,8 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  (void) SetImageColorspace(image,RGBColorspace);
+  if (image_info->colorspace == UndefinedColorspace)
+    (void) SetImageColorspace(image,RGBColorspace);
   /*
     Initialize the printer.
   */
@@ -508,7 +524,7 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
     }
   if (IsMonochromeImage(image,&image->exception))
     {
-      int
+      IndexPacket
         polarity;
 
       register unsigned char
@@ -550,7 +566,7 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
             }
         }
         if (bit != 0)
-          (void) WriteBlobByte(image,byte << (8-bit));
+          (void) WriteBlobByte(image,(unsigned char) (byte << (8-bit)));
         if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
             (QuantumTick(y,image->rows) != MagickFalse))
           {
