@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -55,6 +55,7 @@
 #include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/pixel.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 
@@ -298,7 +299,8 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
@@ -609,8 +611,8 @@ ModuleExport void RegisterPDBImage(void)
   entry->decoder=(DecoderHandler *) ReadPDBImage;
   entry->encoder=(EncoderHandler *) WritePDBImage;
   entry->magick=(MagickHandler *) IsPDB;
-  entry->description=AcquireString("Palm Database ImageViewer Format");
-  entry->module=AcquireString("PDB");
+  entry->description=ConstantString("Palm Database ImageViewer Format");
+  entry->module=ConstantString("PDB");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -669,7 +671,7 @@ static unsigned char *EncodeRLE(unsigned char *destination,
 {
   if (literal > 0)
     *destination++=literal-1;
-  CopyMagickMemory(destination,source,literal);
+  (void) CopyMagickMemory(destination,source,literal);
   destination+=literal;
   if (repeat > 0)
     {
@@ -730,11 +732,12 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  (void) SetImageColorspace(image,RGBColorspace);
+  if (image_info->colorspace == UndefinedColorspace)
+    (void) SetImageColorspace(image,RGBColorspace);
   bits_per_pixel=image->depth;
   if (GetImageType(image,&image->exception) == BilevelType)
     bits_per_pixel=1;
@@ -750,8 +753,8 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
   pdb_info.modify_number=0;
   pdb_info.application_info=0;
   pdb_info.sort_info=0;
-  CopyMagickMemory(pdb_info.type,"vIMG",4);
-  CopyMagickMemory(pdb_info.id,"View",4);
+  (void) CopyMagickMemory(pdb_info.type,"vIMG",4);
+  (void) CopyMagickMemory(pdb_info.id,"View",4);
   pdb_info.seed=0;
   pdb_info.next_record=0;
   comment=GetImageAttribute(image,"Comment");
@@ -800,7 +803,8 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
   scanline=(unsigned char *) AcquireMagickMemory(packet_size*image->columns);
   if (scanline == (unsigned char *) NULL)
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
-  (void) SetImageColorspace(image,RGBColorspace);
+    if (image_info->colorspace == UndefinedColorspace)
+    (void) SetImageColorspace(image,RGBColorspace);
   /*
     Convert to GRAY raster scanline.
   */
@@ -854,7 +858,7 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
               if (0x7f < literal)
                 {
                   q=EncodeRLE(q,buffer,(literal < 0x80 ? literal : 0x80),0);
-                  CopyMagickMemory(buffer,buffer+literal+repeat,0x80);
+                  (void) CopyMagickMemory(buffer,buffer+literal+repeat,0x80);
                   literal-=0x80;
                 }
             }
