@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -45,9 +45,11 @@
 #include "magick/blob-private.h"
 #include "magick/color.h"
 #include "magick/color-private.h"
+#include "magick/colorspace.h"
 #include "magick/compress.h"
 #include "magick/constitute.h"
 #include "magick/delegate.h"
+#include "magick/draw.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
 #include "magick/geometry.h"
@@ -59,6 +61,7 @@
 #include "magick/monitor.h"
 #include "magick/resource_.h"
 #include "magick/resize.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/transform.h"
@@ -152,15 +155,12 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     filename[MaxTextExtent],
     geometry[MaxTextExtent],
     options[MaxTextExtent],
+    input_filename[MaxTextExtent],
     postscript_filename[MaxTextExtent];
 
   const DelegateInfo
     *delegate_info;
 
-  double
-    dx_resolution,
-    dy_resolution;
-
   Image
     *image,
     *next_image;
@@ -175,6 +175,9 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     cmyk,
     status;
 
+  PointInfo
+    delta;
+
   RectangleInfo
     bounding_box,
     page;
@@ -199,7 +202,8 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   /*
@@ -212,11 +216,19 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image=DestroyImageList(image);
       return((Image *) NULL);
     }
+  status=AcquireUniqueSymbolicLink(image_info->filename,input_filename);
+  if (status == MagickFalse)
+    {
+      ThrowFileException(exception,FileOpenError,"UnableToCreateTemporaryFile",
+        image_info->filename);
+      image=DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Set the page density.
   */
-  dx_resolution=DefaultResolution;
-  dy_resolution=DefaultResolution;
+  delta.x=DefaultResolution;
+  delta.y=DefaultResolution;
   if ((image->x_resolution == 0.0) || (image->y_resolution == 0.0))
     {
       GeometryInfo
@@ -241,7 +253,9 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   (void) ResetMagickMemory(&bounding_box,0,sizeof(bounding_box));
   (void) ResetMagickMemory(&bounds,0,sizeof(bounds));
   (void) ResetMagickMemory(&page,0,sizeof(page));
-  for (p=command; (c=ReadBlobByte(image)) != EOF; )
+  (void) ResetMagickMemory(command,0,sizeof(command));
+  p=command;
+  for (c=ReadBlobByte(image); c != EOF; c=ReadBlobByte(image))
   {
     if (image_info->page != (char *) NULL)
       continue;
@@ -249,7 +263,8 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       Note PDF elements.
     */
     *p++=(char) c;
-    if ((c != (int) '/') && ((size_t) (p-command) < (MaxTextExtent-1)))
+    if ((c != (int) '/') && (c != '\n') &&
+        ((size_t) (p-command) < (MaxTextExtent/2)))
       continue;
     *p='\0';
     p=command;
@@ -293,6 +308,8 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       page.height=height;
   }
   CloseBlob(image);
+  if (image_info->colorspace == RGBColorspace)
+    cmyk=MagickFalse;
   /*
     Create Ghostscript control file.
   */
@@ -314,19 +331,20 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (image_info->page != (char *) NULL)
     (void) ParseAbsoluteGeometry(image_info->page,&page);
   page.width=(unsigned long)
-    (page.width*image->x_resolution/dx_resolution+0.5);
+    (page.width*image->x_resolution/delta.x+0.5);
   page.height=(unsigned long)
-    (page.height*image->y_resolution/dy_resolution+0.5);
+    (page.height*image->y_resolution/delta.y+0.5);
   image=DestroyImage(image);
   (void) FormatMagickString(geometry,MaxTextExtent,"%lux%lu",
     page.width,page.height);
-  if (image_info->monochrome != MagickFalse)
-    delegate_info=GetDelegateInfo("gs-mono",(char *) NULL,exception);
+  if ((image_info->ping != MagickFalse) ||
+      (image_info->monochrome != MagickFalse))
+    delegate_info=GetDelegateInfo("ps:mono",(char *) NULL,exception);
   else
      if (cmyk != MagickFalse)
-       delegate_info=GetDelegateInfo("gs-cmyk",(char *) NULL,exception);
+       delegate_info=GetDelegateInfo("ps:cmyk",(char *) NULL,exception);
      else
-       delegate_info=GetDelegateInfo("gs-color",(char *) NULL,exception);
+       delegate_info=GetDelegateInfo("ps:color",(char *) NULL,exception);
   if (delegate_info == (const DelegateInfo *) NULL)
     {
       (void) RelinquishUniqueFileResource(postscript_filename);
@@ -357,11 +375,12 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     GetDelegateCommands(delegate_info),
     read_info->antialias != MagickFalse ? 4 : 1,
     read_info->antialias != MagickFalse ? 4 : 1,geometry,density,options,
-    read_info->filename,postscript_filename,image_info->filename);
+    read_info->filename,postscript_filename,input_filename);
   status=InvokePostscriptDelegate(read_info->verbose,command);
   image=ReadImage(read_info,exception);
   (void) RelinquishUniqueFileResource(postscript_filename);
   (void) RelinquishUniqueFileResource(read_info->filename);
+  (void) RelinquishUniqueFileResource(input_filename);
   read_info=DestroyImageInfo(read_info);
   if (image == (Image *) NULL)
     ThrowReaderException(DelegateError,"PostscriptDelegateFailed");
@@ -424,8 +443,8 @@ ModuleExport void RegisterPDFImage(void)
   entry->adjoin=MagickFalse;
   entry->blob_support=MagickFalse;
   entry->seekable_stream=MagickTrue;
-  entry->description=AcquireString("Encapsulated Portable Document Format");
-  entry->module=AcquireString("PDF");
+  entry->description=ConstantString("Encapsulated Portable Document Format");
+  entry->module=ConstantString("PDF");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PDF");
   entry->decoder=(DecoderHandler *) ReadPDFImage;
@@ -433,8 +452,8 @@ ModuleExport void RegisterPDFImage(void)
   entry->magick=(MagickHandler *) IsPDF;
   entry->blob_support=MagickFalse;
   entry->seekable_stream=MagickTrue;
-  entry->description=AcquireString("Portable Document Format");
-  entry->module=AcquireString("PDF");
+  entry->description=ConstantString("Portable Document Format");
+  entry->module=ConstantString("PDF");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -536,14 +555,6 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
   const ImageAttribute
     *attribute;
 
-  double
-    dx_resolution,
-    dy_resolution,
-    x_resolution,
-    x_scale,
-    y_resolution,
-    y_scale;
-
   GeometryInfo
     geometry_info;
 
@@ -569,6 +580,11 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
   MagickStatusType
     flags;
 
+  PointInfo
+    delta,
+    resolution,
+    scale;
+
   RectangleInfo
     geometry,
     media_info,
@@ -603,7 +619,6 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     info_id,
     object,
     pages_id,
-    quality,
     root_id,
     text_size;
 
@@ -615,13 +630,11 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
   compression=image->compression;
-  quality=(unsigned long)
-    (image->quality == UndefinedCompressionQuality ? 75 : image->quality);
   switch (compression)
   {
 #if !defined(HasJPEG)
@@ -729,11 +742,11 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
         Predict page object id's.
       */
       kid_image=image;
-      for ( ; kid_image->next != (Image *) NULL; count+=ObjectsPerImage)
+      for ( ; GetNextImageInList(kid_image) != (Image *) NULL; count+=ObjectsPerImage)
       {
         (void) FormatMagickString(buffer,MaxTextExtent,"%ld 0 R ",count);
         (void) WriteBlobString(image,buffer);
-        kid_image=kid_image->next;
+        kid_image=GetNextImageInList(kid_image);
       }
       xref=(MagickOffsetType *)
         ResizeMagickMemory(xref,(size_t) (count+2048)*sizeof(*xref));
@@ -752,29 +765,29 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     /*
       Scale relative to dots-per-inch.
     */
-    dx_resolution=DefaultResolution;
-    dy_resolution=DefaultResolution;
+    delta.x=DefaultResolution;
+    delta.y=DefaultResolution;
     flags=ParseGeometry(PSDensityGeometry,&geometry_info);
-    x_resolution=geometry_info.rho;
-    y_resolution=geometry_info.sigma;
+    resolution.x=geometry_info.rho;
+    resolution.y=geometry_info.sigma;
     if ((flags & SigmaValue) == 0)
-      y_resolution=x_resolution;
+      resolution.y=resolution.x;
     if (image->x_resolution != 0.0)
-      x_resolution=image->x_resolution;
+      resolution.x=image->x_resolution;
     if (image->y_resolution != 0.0)
-      y_resolution=image->y_resolution;
+      resolution.y=image->y_resolution;
     if (image_info->density != (char *) NULL)
       {
         flags=ParseGeometry(image_info->density,&geometry_info);
-        x_resolution=geometry_info.rho;
-        y_resolution=geometry_info.sigma;
+        resolution.x=geometry_info.rho;
+        resolution.y=geometry_info.sigma;
         if ((flags & SigmaValue) == 0)
-          y_resolution=x_resolution;
+          resolution.y=resolution.x;
       }
     if (image->units == PixelsPerCentimeterResolution)
       {
-        x_resolution*=2.54;
-        y_resolution*=2.54;
+        resolution.x*=2.54;
+        resolution.y*=2.54;
       }
     SetGeometry(image,&geometry);
     (void) FormatMagickString(page_geometry,MaxTextExtent,"%lux%lu",
@@ -788,13 +801,13 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
       else
         if ((image->gravity != UndefinedGravity) &&
             (LocaleCompare(image_info->magick,"PDF") == 0))
-          (void) strcpy(page_geometry,PSPageGeometry);
+          (void) CopyMagickString(page_geometry,PSPageGeometry,MaxTextExtent);
     (void) ParseMetaGeometry(page_geometry,&geometry.x,&geometry.y,
       &geometry.width,&geometry.height);
-    x_scale=(double) (geometry.width*dx_resolution)/x_resolution;
-    geometry.width=(unsigned long) (x_scale+0.5);
-    y_scale=(double) (geometry.height*dy_resolution)/y_resolution;
-    geometry.height=(unsigned long) (y_scale+0.5);
+    scale.x=(double) (geometry.width*delta.x)/resolution.x;
+    geometry.width=(unsigned long) (scale.x+0.5);
+    scale.y=(double) (geometry.height*delta.y)/resolution.y;
+    geometry.height=(unsigned long) (scale.y+0.5);
     (void) ParseAbsoluteGeometry(page_geometry,&media_info);
     (void) ParseGravityGeometry(image,page_geometry,&page_info);
     if (image->gravity != UndefinedGravity)
@@ -825,6 +838,12 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     matte=image->matte;
     if (IsGrayImage(image,&image->exception) != MagickFalse)
       matte=MagickFalse;
+    if (compression == JPEG2000Compression)
+      {
+        if (image_info->colorspace == UndefinedColorspace)
+  	      (void) SetImageColorspace(image,RGBColorspace);
+        matte=MagickFalse;
+      }
     (void) FormatMagickString(buffer,MaxTextExtent,
       "/XObject << /Im%lu %lu 0 R >>\n",image->scene,object+
       (matte != MagickFalse ? 7 : 5));
@@ -886,7 +905,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
         labels=(char **) RelinquishMagickMemory(labels);
       }
     (void) FormatMagickString(buffer,MaxTextExtent,"%g 0 0 %g %ld %ld cm\n",
-      x_scale,y_scale,geometry.x,geometry.y);
+      scale.x,scale.y,geometry.x,geometry.y);
     (void) WriteBlobString(image,buffer);
     (void) FormatMagickString(buffer,MaxTextExtent,"/Im%lu Do\n",image->scene);
     (void) WriteBlobString(image,buffer);
@@ -911,12 +930,12 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     (void) FormatMagickString(buffer,MaxTextExtent,"%lu 0 obj\n",object);
     (void) WriteBlobString(image,buffer);
     if ((image->storage_class == DirectClass) || (image->colors > 256))
-      (void) strcpy(buffer,"[ /PDF /Text /ImageC");
+      (void) CopyMagickString(buffer,"[ /PDF /Text /ImageC",MaxTextExtent);
     else
       if (compression == FaxCompression)
-        (void) strcpy(buffer,"[ /PDF /Text /ImageB");
+        (void) CopyMagickString(buffer,"[ /PDF /Text /ImageB",MaxTextExtent);
       else
-        (void) strcpy(buffer,"[ /PDF /Text /ImageI");
+        (void) CopyMagickString(buffer,"[ /PDF /Text /ImageI",MaxTextExtent);
     (void) WriteBlobString(image,buffer);
     (void) WriteBlobString(image," ]\n");
     (void) WriteBlobString(image,"endobj\n");
@@ -1022,13 +1041,13 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
                 break;
               for (x=0; x < (long) image->columns; x++)
               {
-                *q++=ScaleQuantumToChar(MaxRGB-p->opacity);
+                *q++=ScaleQuantumToChar(QuantumRange-p->opacity);
                 p++;
               }
             }
 #if defined(HasZLIB)
             if (compression == ZipCompression)
-              status=ZLIBEncodeImage(image,length,quality,pixels);
+              status=ZLIBEncodeImage(image,length,pixels);
             else
 #endif
               if (compression == LZWCompression)
@@ -1057,7 +1076,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
                 break;
               for (x=0; x < (long) image->columns; x++)
               {
-                Ascii85Encode(image,ScaleQuantumToChar(MaxRGB-p->opacity));
+                Ascii85Encode(image,ScaleQuantumToChar(QuantumRange-p->opacity));
                 p++;
               }
             }
@@ -1104,7 +1123,8 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
         if (image->colorspace != CMYKColorspace)
           break;
         (void) WriteBlobString(image,buffer);
-        (void) strcpy(buffer,"/Decode [1 0 1 0 1 0 1 0]\n");
+        (void) CopyMagickString(buffer,"/Decode [1 0 1 0 1 0 1 0]\n",
+          MaxTextExtent);
         break;
       }
       case JPEG2000Compression:
@@ -1113,7 +1133,8 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
         if (image->colorspace != CMYKColorspace)
           break;
         (void) WriteBlobString(image,buffer);
-        (void) strcpy(buffer,"/Decode [1 0 1 0 1 0 1 0]\n");
+        (void) CopyMagickString(buffer,"/Decode [1 0 1 0 1 0 1 0]\n",
+          MaxTextExtent);
         break;
       }
       case LZWCompression:
@@ -1128,7 +1149,8 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
       }
       case FaxCompression:
       {
-        (void) strcpy(buffer,"/Filter [ /CCITTFaxDecode ]\n");
+        (void) CopyMagickString(buffer,"/Filter [ /CCITTFaxDecode ]\n",
+          MaxTextExtent);
         (void) WriteBlobString(image,buffer);
         (void) FormatMagickString(buffer,MaxTextExtent,
           "/DecodeParms [ << >> << /K %s /Columns %ld /Rows %ld >> ]\n",
@@ -1237,7 +1259,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
             }
 #if defined(HasZLIB)
             if (compression == ZipCompression)
-              status=ZLIBEncodeImage(image,length,quality,pixels);
+              status=ZLIBEncodeImage(image,length,pixels);
             else
 #endif
               if (compression == LZWCompression)
@@ -1348,7 +1370,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
             }
 #if defined(HasZLIB)
             if (compression == ZipCompression)
-              status=ZLIBEncodeImage(image,length,quality,pixels);
+              status=ZLIBEncodeImage(image,length,pixels);
             else
 #endif
               if (compression == LZWCompression)
@@ -1442,7 +1464,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
               }
 #if defined(HasZLIB)
               if (compression == ZipCompression)
-                status=ZLIBEncodeImage(image,length,quality,pixels);
+                status=ZLIBEncodeImage(image,length,pixels);
               else
 #endif
                 if (compression == LZWCompression)
@@ -1507,17 +1529,17 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     (void) FormatMagickString(buffer,MaxTextExtent,"%lu 0 obj\n",object);
     (void) WriteBlobString(image,buffer);
     if (image->colorspace == CMYKColorspace)
-      (void) strcpy(buffer,"/DeviceCMYK\n");
+      (void) CopyMagickString(buffer,"/DeviceCMYK\n",MaxTextExtent);
     else
       if ((compression == FaxCompression) ||
           ((image_info->type != TrueColorType) &&
            (IsGrayImage(image,&image->exception) != MagickFalse)))
-          (void) strcpy(buffer,"/DeviceGray\n");
+          (void) CopyMagickString(buffer,"/DeviceGray\n",MaxTextExtent);
       else
         if ((image->storage_class == DirectClass) || (image->colors > 256) ||
             (compression == JPEGCompression) ||
             (compression == JPEG2000Compression))
-          (void) strcpy(buffer,"/DeviceRGB\n");
+          (void) CopyMagickString(buffer,"/DeviceRGB\n",MaxTextExtent);
         else
           (void) FormatMagickString(buffer,MaxTextExtent,
             "[ /Indexed /DeviceRGB %lu %lu 0 R ]\n",
@@ -1551,7 +1573,8 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
         if (image->colorspace != CMYKColorspace)
           break;
         (void) WriteBlobString(image,buffer);
-        (void) strcpy(buffer,"/Decode [1 0 1 0 1 0 1 0]\n");
+        (void) CopyMagickString(buffer,"/Decode [1 0 1 0 1 0 1 0]\n",
+          MaxTextExtent);
         break;
       }
       case JPEG2000Compression:
@@ -1560,7 +1583,8 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
         if (image->colorspace != CMYKColorspace)
           break;
         (void) WriteBlobString(image,buffer);
-        (void) strcpy(buffer,"/Decode [1 0 1 0 1 0 1 0]\n");
+        (void) CopyMagickString(buffer,"/Decode [1 0 1 0 1 0 1 0]\n",
+          MaxTextExtent);
         break;
       }
       case LZWCompression:
@@ -1575,7 +1599,8 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
       }
       case FaxCompression:
       {
-        (void) strcpy(buffer,"/Filter [ /CCITTFaxDecode ]\n");
+        (void) CopyMagickString(buffer,"/Filter [ /CCITTFaxDecode ]\n",
+          MaxTextExtent);
         (void) WriteBlobString(image,buffer);
         (void) FormatMagickString(buffer,MaxTextExtent,
           "/DecodeParms [ << >> << /K %s /Columns %lu /Rows %lu >> ]\n",
@@ -1669,7 +1694,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
             }
 #if defined(HasZLIB)
             if (compression == ZipCompression)
-              status=ZLIBEncodeImage(image,length,quality,pixels);
+              status=ZLIBEncodeImage(image,length,pixels);
             else
 #endif
               if (compression == LZWCompression)
@@ -1766,7 +1791,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
             }
 #if defined(HasZLIB)
             if (compression == ZipCompression)
-              status=ZLIBEncodeImage(image,length,quality,pixels);
+              status=ZLIBEncodeImage(image,length,pixels);
             else
 #endif
               if (compression == LZWCompression)
@@ -1845,7 +1870,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
               }
 #if defined(HasZLIB)
               if (compression == ZipCompression)
-                status=ZLIBEncodeImage(image,length,quality,pixels);
+                status=ZLIBEncodeImage(image,length,pixels);
               else
 #endif
                 if (compression == LZWCompression)
@@ -1947,7 +1972,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
       (unsigned long) offset);
     (void) WriteBlobString(image,buffer);
     (void) WriteBlobString(image,"endobj\n");
-    if (image->next == (Image *) NULL)
+    if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
