@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -50,6 +50,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 
@@ -122,7 +123,8 @@ static Image *ReadPIXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
@@ -176,12 +178,12 @@ static Image *ReadPIXImage(const ImageInfo *image_info,ExceptionInfo *exception)
           {
             length=(size_t) ReadBlobByte(image);
             if (bits_per_pixel == 8)
-              index=ScaleCharToQuantum(ReadBlobByte(image));
+              index=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
             else
               {
-                blue=ScaleCharToQuantum(ReadBlobByte(image));
-                green=ScaleCharToQuantum(ReadBlobByte(image));
-                red=ScaleCharToQuantum(ReadBlobByte(image));
+                blue=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
+                green=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
+                red=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
               }
           }
         if (image->storage_class == PseudoClass)
@@ -223,16 +225,16 @@ static Image *ReadPIXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     (void) ReadBlobMSBShort(image);
     (void) ReadBlobMSBShort(image);
     bits_per_pixel=ReadBlobMSBShort(image);
-    status=(MagickBooleanType) ((width != (unsigned long) ~0) &&
+    status=(width != (unsigned long) ~0) &&
       (height == (unsigned long) ~0) && ((bits_per_pixel == 8) ||
-      (bits_per_pixel == 24)));
+      (bits_per_pixel == 24)) ? MagickTrue : MagickFalse;
     if (status == MagickTrue)
       {
         /*
           Allocate next image structure.
         */
         AllocateNextImage(image_info,image);
-        if (image->next == (Image *) NULL)
+        if (GetNextImageInList(image) == (Image *) NULL)
           {
             image=DestroyImageList(image);
             return((Image *) NULL);
@@ -281,8 +283,8 @@ ModuleExport void RegisterPIXImage(void)
 
   entry=SetMagickInfo("PIX");
   entry->decoder=(DecoderHandler *) ReadPIXImage;
-  entry->description=AcquireString("Alias/Wavefront RLE image format");
-  entry->module=AcquireString("PIX");
+  entry->description=ConstantString("Alias/Wavefront RLE image format");
+  entry->module=ConstantString("PIX");
   (void) RegisterMagickInfo(entry);
 }
 
