@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -53,6 +53,7 @@
 #include "magick/monitor.h"
 #include "magick/random_.h"
 #include "magick/signature.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 
@@ -92,9 +93,12 @@ static void inline PlasmaPixel(Image *image,double x,double y)
   q=GetImagePixels(image,(long) ceil(x-0.5),(long) ceil(y-0.5),1,1);
   if (q == (PixelPacket *) NULL)
     return;
-  q->red=ScaleXToQuantum(GetRandomValue(),1.0);
-  q->green=ScaleXToQuantum(GetRandomValue(),1.0);
-  q->blue=ScaleXToQuantum(GetRandomValue(),1.0);
+  q->red=ScaleAnyToQuantum((unsigned long)
+    (65535.0*GetRandomValue()+0.5),65535UL);
+  q->green=ScaleAnyToQuantum((unsigned long)
+    (65535.0*GetRandomValue()+0.5),65535UL);
+  q->blue=ScaleAnyToQuantum((unsigned long)
+    (65535.0*GetRandomValue()+0.5),65535UL);
   (void) SyncImagePixels(image);
 }
 
@@ -184,7 +188,7 @@ static Image *ReadPlasmaImage(const ImageInfo *image_info,
     if (PlasmaImage(image,&segment_info,0,depth) != MagickFalse)
       break;
     if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(depth,max_depth) != MagickFalse))
+        (QuantumTick((MagickOffsetType) depth,max_depth) != MagickFalse))
       {
         status=image->progress_monitor(PlasmaImageTag,(MagickOffsetType) depth,
           max_depth,image->client_data);
@@ -226,14 +230,14 @@ ModuleExport void RegisterPLASMAImage(void)
   entry=SetMagickInfo("PLASMA");
   entry->decoder=(DecoderHandler *) ReadPlasmaImage;
   entry->adjoin=MagickFalse;
-  entry->description=AcquireString("Plasma fractal image");
-  entry->module=AcquireString("PLASMA");
+  entry->description=ConstantString("Plasma fractal image");
+  entry->module=ConstantString("PLASMA");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("FRACTAL");
   entry->decoder=(DecoderHandler *) ReadPlasmaImage;
   entry->adjoin=MagickFalse;
-  entry->description=AcquireString("Plasma fractal image");
-  entry->module=AcquireString("PLASMA");
+  entry->description=ConstantString("Plasma fractal image");
+  entry->module=ConstantString("PLASMA");
   (void) RegisterMagickInfo(entry);
 }
 
