@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -54,7 +54,9 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
+#include "magick/statistic.h"
 #include "magick/string_.h"
 
 /*
@@ -132,15 +134,27 @@ static unsigned long PNMInteger(Image *image,const unsigned int base)
 {
 #define P7Comment  "END_OF_COMMENTS\n"
 
+  char
+    *comment;
+
   int
     c;
 
+  register char
+    *p;
+
+  size_t
+    length;
+
   unsigned long
     value;
 
   /*
     Skip any leading whitespace.
   */
+  length=MaxTextExtent;
+  comment=(char *) NULL;
+  p=comment;
   do
   {
     c=ReadBlobByte(image);
@@ -148,30 +162,16 @@ static unsigned long PNMInteger(Image *image,const unsigned int base)
       return(0);
     if (c == (int) '#')
       {
-        char
-          *comment;
-
-        MagickOffsetType
-          offset;
-
-        register char
-          *p,
-          *q;
-
-        size_t
-          length;
-
         /*
           Read comment.
         */
-        length=MaxTextExtent;
-        offset=0;
-        comment=AcquireString((char *) NULL);
-        for (p=comment; (c != EOF) && (c != (int) '\n'); p++)
+        if (comment == (char *) NULL)
+          comment=AcquireString((char *) NULL);
+        p=comment+strlen(comment);
+        for ( ; (c != EOF) && (c != (int) '\n'); p++)
         {
           if ((size_t) (p-comment+1) >= length)
             {
-              *p='\0';
               length<<=1;
               comment=(char *) ResizeMagickMemory(comment,
                 (length+MaxTextExtent)*sizeof(*comment));
@@ -181,18 +181,24 @@ static unsigned long PNMInteger(Image *image,const unsigned int base)
             }
           c=ReadBlobByte(image);
           *p=(char) c;
+          *(p+1)='\0';
         }
-        *p='\0';
         if (comment == (char *) NULL)
           return(0);
-        q=comment+offset;
-        if (LocaleCompare(q,P7Comment) == 0)
-          *q='\0';
-        (void) SetImageAttribute(image,"Comment",comment);
-        comment=(char *) RelinquishMagickMemory(comment);
         continue;
       }
   } while (isdigit(c) == MagickFalse);
+  if (comment != (char *) NULL)
+    {
+      if (strlen(comment) > strlen(P7Comment))
+        {
+          p-=strlen(P7Comment);
+          if (LocaleCompare(p,P7Comment) == 0)
+            *p='\0';
+        }
+      (void) SetImageAttribute(image,"Comment",comment);
+      comment=(char *) RelinquishMagickMemory(comment);
+    }
   if (base == 2)
     return((unsigned long) (c-(int) '0'));
   /*
@@ -233,10 +239,8 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   long
     y;
 
-  LongPixelPacket
-    pixel;
-
   MagickBooleanType
+    grayscale,
     status;
 
   Quantum
@@ -258,7 +262,8 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
     *p;
 
   size_t
-    packets;
+    length,
+    packet_size;
 
   ssize_t
     count;
@@ -298,49 +303,105 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
     */
     if ((count != 1) || (format != 'P'))
       ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+    max_value=1;
+    grayscale=MagickFalse;
     format=(char) ReadBlobByte(image);
-    if (format == '7')
-      (void) PNMInteger(image,10);
-    image->columns=PNMInteger(image,10);
-    image->rows=PNMInteger(image,10);
-    if ((format == '1') || (format == '4'))
-      max_value=1;  /* bitmap */
-    else
-      max_value=PNMInteger(image,10);
-    image->depth=max_value < 256 ? 8UL : QuantumDepth;
-    if ((format != '3') && (format != '6'))
+    if (format != '7')
       {
-        image->storage_class=PseudoClass;
-        image->colors=(unsigned long) (max_value >= MaxColormapSize ?
-          MaxColormapSize : max_value+1);
+        /*
+          PBM, PGM, PPM, and PNM.
+        */
+        image->columns=PNMInteger(image,10);
+        image->rows=PNMInteger(image,10);
+        if ((format == '1') || (format == '4'))
+          max_value=1;  /* bitmap */
+        else
+          max_value=PNMInteger(image,10);
+        if ((format != '3') && (format != '6'))
+          {
+            image->storage_class=PseudoClass;
+            image->colors=(unsigned long) (max_value >= MaxColormapSize ?
+              MaxColormapSize : max_value+1);
+          }
+      }
+    else
+      {
+        char
+          keyword[MaxTextExtent],
+          value[MaxTextExtent];
+
+        int
+          c;
+
+        register char
+          *p;
+
+        /*
+          PAM.
+        */
+        for (c=ReadBlobByte(image); c != EOF; c=ReadBlobByte(image))
+        {
+          while (isspace((int) ((unsigned char) c)) != 0)
+            c=ReadBlobByte(image);
+          p=keyword;
+          do
+          {
+            if ((size_t) (p-keyword) < (MaxTextExtent/2))
+              *p++=c;
+            c=ReadBlobByte(image);
+          } while (isalnum(c));
+          *p='\0';
+          if (LocaleCompare(keyword,"endhdr") == 0)
+            break;
+          while (isspace((int) ((unsigned char) c)) != 0)
+            c=ReadBlobByte(image);
+          p=value;
+          while (isalnum(c))
+          {
+            if ((size_t) (p-value) < (MaxTextExtent/2))
+              *p++=c;
+            c=ReadBlobByte(image);
+          }
+          *p='\0';
+          /*
+            Assign a value to the specified keyword.
+          */
+          if (LocaleCompare(keyword,"depth") == 0)
+            packet_size=(unsigned long) atol(value);
+          if (LocaleCompare(keyword,"height") == 0)
+            image->rows=(unsigned long) atol(value);
+          if (LocaleCompare(keyword,"maxval") == 0)
+            max_value=(unsigned long) atol(value);
+          if (LocaleCompare(keyword,"TUPLTYPE") == 0)
+            {
+              if (LocaleCompare(value,"BLACKANDWHITE") == 0)
+                grayscale=MagickTrue;
+              if (LocaleCompare(value,"BLACKANDWHITE_ALPHA") == 0)
+                {
+                  grayscale=MagickTrue;
+                  image->matte=MagickTrue;
+                }
+              if (LocaleCompare(value,"GRAYSCALE") == 0)
+                grayscale=MagickTrue;
+              if (LocaleCompare(value,"GRAYSCALE_ALPHA") == 0)
+                {
+                  grayscale=MagickTrue;
+                  image->matte=MagickTrue;
+                }
+              if (LocaleCompare(value,"RGB_ALPHA") == 0)
+                image->matte=MagickTrue;
+            }
+          if (LocaleCompare(keyword,"width") == 0)
+            image->columns=(unsigned long) atol(value);
+        }
       }
     if ((image->columns == 0) || (image->rows == 0))
       ThrowReaderException(CorruptImageError,"NegativeOrZeroImageSize");
+    for (image->depth=1; (1UL << image->depth) < max_value; image->depth++);
     scale=(Quantum *) NULL;
     if (image->storage_class == PseudoClass)
-      {
-        /*
-          Create colormap.
-        */
-        if (AllocateImageColormap(image,image->colors) == MagickFalse)
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-        if ((format == '7') && (image->colors == 256))
-          {
-            /*
-              Initialize 332 colormap.
-            */
-            i=0;
-            for (pixel.red=0; pixel.red < 8; pixel.red++)
-              for (pixel.green=0; pixel.green < 8; pixel.green++)
-                for (pixel.blue=0; pixel.blue < 4; pixel.blue++)
-                {
-                  image->colormap[i].red=ScaleXToQuantum(pixel.red,0x07);
-                  image->colormap[i].green=ScaleXToQuantum(pixel.green,0x07);
-                  image->colormap[i].blue=ScaleXToQuantum(pixel.blue,0x03);
-                  i++;
-                }
-          }
-      }
+      if (AllocateImageColormap(image,image->colors) == MagickFalse)
+        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     if ((image->storage_class != PseudoClass) || (max_value > QuantumRange))
       {
         /*
@@ -351,7 +412,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (scale == (Quantum *) NULL)
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         for (i=0; i <= (long) max_value; i++)
-          scale[i]=ScaleXToQuantum(i,max_value);
+          scale[i]=ScaleAnyToQuantum((unsigned long) i,max_value);
       }
     if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
@@ -445,6 +506,9 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
       }
       case '3':
       {
+        LongPixelPacket
+          pixel;
+
         /*
           Convert PNM image to pixel packets.
         */
@@ -530,20 +594,19 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
         break;
       }
       case '5':
-      case '7':
       {
         /*
           Convert PGM raw image to pixel packets.
         */
-        packets=(size_t) (image->depth <= 8 ? 1 : 2);
-        pixels=(unsigned char *)
-          AcquireMagickMemory((size_t) packets*image->columns);
+        packet_size=(size_t) (image->depth <= 8 ? 1 : 2);
+        length=packet_size*image->columns;
+        pixels=(unsigned char *) AcquireMagickMemory(length*sizeof(*pixels));
         if (pixels == (unsigned char *) NULL)
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         for (y=0; y < (long) image->rows; y++)
         {
-          count=ReadBlob(image,(size_t) packets*image->columns,pixels);
-          if (count != (ssize_t) (packets*image->columns))
+          count=ReadBlob(image,length,pixels);
+          if (count != (ssize_t) length)
             ThrowReaderException(CorruptImageError,"UnableToReadImageData");
           p=pixels;
           q=SetImagePixels(image,0,y,image->columns,1);
@@ -598,17 +661,21 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
       }
       case '6':
       {
+        LongPixelPacket
+          pixel;
+
         /*
           Convert PNM raster image to pixel packets.
         */
-        packets=(size_t) (image->depth <= 8 ? 3 : 6);
-        pixels=(unsigned char *) AcquireMagickMemory(packets*image->columns);
+        packet_size=(size_t) (image->depth <= 8 ? 3 : 6);
+        pixels=(unsigned char *)
+          AcquireMagickMemory(packet_size*image->columns*sizeof(*pixels));
         if (pixels == (unsigned char *) NULL)
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         for (y=0; y < (long) image->rows; y++)
         {
-          count=ReadBlob(image,packets*image->columns,pixels);
-          if (count != (ssize_t) (packets*image->columns))
+          count=ReadBlob(image,packet_size*image->columns,pixels);
+          if (count != (ssize_t) (packet_size*image->columns))
             ThrowReaderException(CorruptImageError,"UnableToReadImageData");
           p=pixels;
           q=SetImagePixels(image,0,y,image->columns,1);
@@ -666,6 +733,118 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
             CorruptImageError,"UnexpectedEndOfFile","`%s'",image->filename);
         break;
       }
+      case '7':
+      {
+        unsigned long
+          pixel;
+
+        /*
+          Convert PNM raster image to pixel packets.
+        */
+        packet_size=3;
+        if (grayscale != MagickFalse)
+          packet_size=1;
+        if (image->matte != MagickFalse)
+          packet_size++;
+        if (image->depth > 8)
+          packet_size*=2;
+        length=packet_size*image->columns*sizeof(*pixels);
+        pixels=(unsigned char *) AcquireMagickMemory(length);
+        if (pixels == (unsigned char *) NULL)
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+        for (y=0; y < (long) image->rows; y++)
+        {
+          count=ReadBlob(image,packet_size*image->columns,pixels);
+          if (count != (ssize_t) (packet_size*image->columns))
+            ThrowReaderException(CorruptImageError,"UnableToReadImageData");
+          p=pixels;
+          q=SetImagePixels(image,0,y,image->columns,1);
+          if (q == (PixelPacket *) NULL)
+            break;
+          if (grayscale != MagickFalse)
+            {
+              if (image->depth <= 8)
+                for (x=0; x < (long) image->columns; x++)
+                {
+                  PushCharPixel(pixel,p);
+                  q->red=ScaleAnyToQuantum(pixel,(1UL << image->depth)-1);
+                  q->green=q->red;
+                  q->blue=q->red;
+                  if (image->matte != MagickFalse)
+                    {
+                      PushCharPixel(pixel,p);
+                      q->opacity=ScaleAnyToQuantum(pixel,
+                        (1UL << image->depth)-1);
+                    }
+                  q++;
+                }
+              else
+                for (x=0; x < (long) image->columns; x++)
+                {
+                  PushShortPixel(pixel,p);
+                  q->red=ScaleAnyToQuantum(pixel,(1UL << image->depth)-1);
+                  q->green=q->red;
+                  q->blue=q->red;
+                  if (image->matte != MagickFalse)
+                    {
+                      PushShortPixel(pixel,p);
+                      q->opacity=ScaleAnyToQuantum(pixel,
+                        (1UL << image->depth)-1);
+                    }
+                  q++;
+                }
+            }
+          else
+            if (image->depth <= 8)
+              for (x=0; x < (long) image->columns; x++)
+              {
+                PushCharPixel(pixel,p);
+                q->red=ScaleAnyToQuantum(pixel,(1UL << image->depth)-1);
+                PushCharPixel(pixel,p);
+                q->green=ScaleAnyToQuantum(pixel,(1UL << image->depth)-1);
+                PushCharPixel(pixel,p);
+                q->blue=ScaleAnyToQuantum(pixel,(1UL << image->depth)-1);
+                if (image->matte != MagickFalse)
+                  {
+                    PushCharPixel(pixel,p);
+                    q->opacity=ScaleAnyToQuantum(pixel,(1UL << image->depth)-1);
+                  }
+                q++;
+              }
+            else
+              for (x=0; x < (long) image->columns; x++)
+              {
+                PushShortPixel(pixel,p);
+                q->red=ScaleAnyToQuantum(pixel,(1UL << image->depth)-1);
+                PushShortPixel(pixel,p);
+                q->green=ScaleAnyToQuantum(pixel,(1UL << image->depth)-1);
+                PushShortPixel(pixel,p);
+                q->blue=ScaleAnyToQuantum(pixel,(1UL << image->depth)-1);
+                if (image->matte != MagickFalse)
+                  {
+                    PushShortPixel(pixel,p);
+                    q->opacity=ScaleAnyToQuantum(pixel,(1UL << image->depth)-1);
+                  }
+                q++;
+              }
+          if (SyncImagePixels(image) == MagickFalse)
+            break;
+          if (image->previous == (Image *) NULL)
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
+              {
+                status=image->progress_monitor(LoadImageTag,y,image->rows,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
+        }
+        pixels=(unsigned char *) RelinquishMagickMemory(pixels);
+        if (EOFBlob(image) != MagickFalse)
+          (void) ThrowMagickException(exception,GetMagickModule(),
+            CorruptImageError,"UnexpectedEndOfFile","`%s'",image->filename);
+        break;
+      }
       default:
         ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     }
@@ -746,33 +925,38 @@ ModuleExport void RegisterPNMImage(void)
   entry=SetMagickInfo("P7");
   entry->decoder=(DecoderHandler *) ReadPNMImage;
   entry->encoder=(EncoderHandler *) WritePNMImage;
-  entry->description=AcquireString("Xv thumbnail format");
-  entry->module=AcquireString("PNM");
+  entry->description=ConstantString("Xv thumbnail format");
+  entry->module=ConstantString("PNM");
+  (void) RegisterMagickInfo(entry);
+  entry=SetMagickInfo("PAM");
+  entry->encoder=(EncoderHandler *) WritePNMImage;
+  entry->description=ConstantString("Common 2-dimensional bitmap format");
+  entry->module=ConstantString("PNM");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PBM");
   entry->decoder=(DecoderHandler *) ReadPNMImage;
   entry->encoder=(EncoderHandler *) WritePNMImage;
-  entry->description=AcquireString("Portable bitmap format (black and white)");
-  entry->module=AcquireString("PNM");
+  entry->description=ConstantString("Portable bitmap format (black and white)");
+  entry->module=ConstantString("PNM");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PGM");
   entry->decoder=(DecoderHandler *) ReadPNMImage;
   entry->encoder=(EncoderHandler *) WritePNMImage;
-  entry->description=AcquireString("Portable graymap format (gray scale)");
-  entry->module=AcquireString("PNM");
+  entry->description=ConstantString("Portable graymap format (gray scale)");
+  entry->module=ConstantString("PNM");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PNM");
   entry->decoder=(DecoderHandler *) ReadPNMImage;
   entry->encoder=(EncoderHandler *) WritePNMImage;
   entry->magick=(MagickHandler *) IsPNM;
-  entry->description=AcquireString("Portable anymap");
-  entry->module=AcquireString("PNM");
+  entry->description=ConstantString("Portable anymap");
+  entry->module=ConstantString("PNM");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PPM");
   entry->decoder=(DecoderHandler *) ReadPNMImage;
   entry->encoder=(EncoderHandler *) WritePNMImage;
-  entry->description=AcquireString("Portable pixmap format (color)");
-  entry->module=AcquireString("PNM");
+  entry->description=ConstantString("Portable pixmap format (color)");
+  entry->module=ConstantString("PNM");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -798,6 +982,7 @@ ModuleExport void RegisterPNMImage(void)
 ModuleExport void UnregisterPNMImage(void)
 {
   (void) UnregisterMagickInfo("P7");
+  (void) UnregisterMagickInfo("PAM");
   (void) UnregisterMagickInfo("PBM");
   (void) UnregisterMagickInfo("PGM");
   (void) UnregisterMagickInfo("PNM");
@@ -832,6 +1017,19 @@ ModuleExport void UnregisterPNMImage(void)
 */
 static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
 {
+#define PopCharPixel(pixel,p) \
+{ \
+  *(q)=(unsigned char) pixel; \
+  (q)++; \
+}
+#define PopShortPixel(pixel,p) \
+{ \
+  *(q)=(unsigned char) (pixel >> 8); \
+  (q)++; \
+  *(q)=(unsigned char) pixel; \
+  (q)++; \
+}
+
   char
     buffer[MaxTextExtent];
 
@@ -845,10 +1043,10 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
     format;
 
   long
-    j,
     y;
 
   MagickBooleanType
+    grayscale,
     status;
 
   MagickOffsetType
@@ -864,6 +1062,19 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
     i,
     x;
 
+  register unsigned char
+    *q;
+
+  size_t
+    length,
+    packet_size;
+
+  unsigned char
+    *pixels;
+
+  unsigned long
+    depth;
+
   /*
     Open output image file.
   */
@@ -882,31 +1093,30 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
     /*
       Write PNM file header.
     */
-    (void) SetImageColorspace(image,RGBColorspace);
+    if (image_info->colorspace == UndefinedColorspace)
+      (void) SetImageColorspace(image,RGBColorspace);
+    grayscale=IsGrayImage(image,&image->exception);
+    depth=image->depth;
     format=6;
-    if (LocaleCompare(image_info->magick,"PGM") == 0)
-      format=5;
+    if (LocaleCompare(image_info->magick,"PAM") == 0)
+      format=7;
     else
-      if (LocaleCompare(image_info->magick,"PBM") == 0)
-        format=4;
+      if (LocaleCompare(image_info->magick,"PGM") == 0)
+        format=5;
       else
-        if ((LocaleCompare(image_info->magick,"PNM") == 0) &&
-            (image_info->type != TrueColorType) &&
-            (IsGrayImage(image,&image->exception) != MagickFalse))
-          {
-            format=5;
-            if (IsMonochromeImage(image,&image->exception) != MagickFalse)
-              format=4;
-          }
-    if (image->compression == NoCompression)
+        if (LocaleCompare(image_info->magick,"PBM") == 0)
+          format=4;
+        else
+          if ((LocaleCompare(image_info->magick,"PNM") == 0) &&
+              (image_info->type != TrueColorType) && (grayscale != MagickFalse))
+            {
+              format=5;
+              if (IsMonochromeImage(image,&image->exception) != MagickFalse)
+                format=4;
+            }
+    if ((image->compression == NoCompression) && (format != 7))
       format-=3;
-    if (LocaleCompare(image_info->magick,"P7") != 0)
-      (void) FormatMagickString(buffer,MaxTextExtent,"P%d\n",format);
-    else
-      {
-        format=7;
-        (void) strcpy(buffer,"P7 332\n");
-      }
+    (void) FormatMagickString(buffer,MaxTextExtent,"P%d\n",format);
     (void) WriteBlobString(image,buffer);
     attribute=GetImageAttribute(image,"Comment");
     if (attribute != (const ImageAttribute *) NULL)
@@ -934,6 +1144,44 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
           image->columns,image->rows);
         (void) WriteBlobString(image,buffer);
       }
+    else
+      {
+        char
+          type[MaxTextExtent];
+
+        unsigned long
+          extent;
+
+        /*
+          PAM header.
+        */
+        (void) FormatMagickString(buffer,MaxTextExtent,
+          "WIDTH %lu\nHEIGHT %lu\n",image->columns,image->rows);
+        (void) WriteBlobString(image,buffer);
+        packet_size=3;
+        (void) CopyMagickString(type,"RGB",MaxTextExtent);
+        if (grayscale != MagickFalse)
+          {
+            packet_size=1;
+            (void) CopyMagickString(type,"GRAYSCALE",MaxTextExtent);
+            if (IsMonochromeImage(image,&image->exception))
+              (void) CopyMagickString(type,"BLACKANDWHITE",MaxTextExtent);
+          }
+        if (image->matte != MagickFalse)
+          {
+            packet_size++;
+            (void) ConcatenateMagickString(type,"_ALPHA",MaxTextExtent);
+          }
+        if (depth > 16)
+          depth=16;
+        extent=(1UL << depth)-1;
+        (void) FormatMagickString(buffer,MaxTextExtent,
+          "DEPTH %lu\nMAXVAL %lu\n",(unsigned long) packet_size,extent);
+        (void) WriteBlobString(image,buffer);
+        (void) FormatMagickString(buffer,MaxTextExtent,"TUPLTYPE %s\nENDHDR\n",
+          type);
+        (void) WriteBlobString(image,buffer);
+      }
     /*
       Convert runlength encoded to PNM raster pixels.
     */
@@ -1179,20 +1427,12 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
       }
       case 6:
       {
-        register unsigned char
-          *q;
-
-        size_t
-          packets;
-
-        unsigned char
-          *pixels;
-
         /*
           Allocate memory for pixels.
         */
-        packets=(size_t) (image->depth <= 8 ? 3 : 6);
-        pixels=(unsigned char *) AcquireMagickMemory(packets*image->columns);
+        packet_size=(size_t) (image->depth <= 8 ? 3 : 6);
+        length=packet_size*image->columns*sizeof(*pixels);
+        pixels=(unsigned char *) AcquireMagickMemory(length);
         if (pixels == (unsigned char *) NULL)
           ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
         /*
@@ -1243,137 +1483,103 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image)
       }
       case 7:
       {
-        static const long
-          dither_red[2][16]=
-          {
-            {-16,  4, -1, 11,-14,  6, -3,  9,-15,  5, -2, 10,-13,  7, -4,  8},
-            { 15, -5,  0,-12, 13, -7,  2,-10, 14, -6,  1,-11, 12, -8,  3, -9}
-          },
-          dither_green[2][16]=
-          {
-            { 11,-15,  7, -3,  8,-14,  4, -2, 10,-16,  6, -4,  9,-13,  5, -1},
-            {-12, 14, -8,  2, -9, 13, -5,  1,-11, 15, -7,  3,-10, 12, -6,  0}
-          },
-          dither_blue[2][16]=
-          {
-            { -3,  9,-13,  7, -1, 11,-15,  5, -4,  8,-14,  6, -2, 10,-16,  4},
-            {  2,-10, 12, -8,  0,-12, 14, -6,  3, -9, 13, -7,  1,-11, 15, -5}
-          };
-
-        long
-          value;
-
-        LongPixelPacket
+        unsigned long
           pixel;
 
-        Quantum
-          quantum;
-
-        unsigned long
-          *blue_map[2][16],
-          *green_map[2][16],
-          *red_map[2][16];
-
         /*
-          Allocate and initialize dither maps.
+          Convert image to a PAM.
         */
-        for (i=0; i < 2; i++)
-          for (j=0; j < 16; j++)
-          {
-            red_map[i][j]=(unsigned long *)
-              AcquireMagickMemory(256*sizeof(*red_map));
-            green_map[i][j]=(unsigned long *)
-              AcquireMagickMemory(256*sizeof(*green_map));
-            blue_map[i][j]=(unsigned long *)
-              AcquireMagickMemory(256*sizeof(*blue_map));
-            if ((red_map[i][j] == (unsigned long *) NULL) ||
-                (green_map[i][j] == (unsigned long *) NULL) ||
-                (blue_map[i][j] == (unsigned long *) NULL))
-              ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
-          }
-        /*
-          Initialize dither tables.
-        */
-        for (i=0; i < 2; i++)
-          for (j=0; j < 16; j++)
-            for (x=0; x < 256; x++)
-            {
-              value=x-16;
-              if (x < 48)
-                value=x/2+8;
-              value+=dither_red[i][j];
-              red_map[i][j][x]=(unsigned long)
-                ((value < 0) ? 0 : (value > 255) ? 255 : value);
-              value=x-16;
-              if (x < 48)
-                value=x/2+8;
-              value+=dither_green[i][j];
-              green_map[i][j][x]=(unsigned long)
-                ((value < 0) ? 0 : (value > 255) ? 255 : value);
-              value=x-32;
-              if (x < 112)
-                value=x/2+24;
-              value+=2*dither_blue[i][j];
-              blue_map[i][j][x]=(unsigned long)
-                ((value < 0) ? 0 : (value > 255) ? 255 : value);
-            }
-        /*
-          Convert image to a P7 image.
-        */
-        (void) WriteBlobString(image,"#END_OF_COMMENTS\n");
-        (void) FormatMagickString(buffer,MaxTextExtent,"%lu %lu 255\n",
-          image->columns,image->rows);
-        (void) WriteBlobString(image,buffer);
-        i=0;
-        j=0;
+        packet_size=3;
+        if (grayscale != MagickFalse)
+          packet_size=1;
+        if (image->matte != MagickFalse)
+          packet_size++;
+        if (depth > 8)
+          packet_size*=2;
+        length=packet_size*image->columns*sizeof(*pixels);
+        pixels=(unsigned char *) AcquireMagickMemory(length);
+        if (pixels == (unsigned char *) NULL)
+          ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
         for (y=0; y < (long) image->rows; y++)
         {
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) image->columns; x++)
-          {
-            pixel.red=(unsigned long) ScaleQuantumToChar(p->red);
-            pixel.green=(unsigned long) ScaleQuantumToChar(p->green);
-            pixel.blue=(unsigned long) ScaleQuantumToChar(p->blue);
-            if (image_info->dither == MagickFalse)
-              quantum=(Quantum) ((pixel.red & 0xe0) |
-                ((pixel.green & 0xe0) >> 3) | ((pixel.blue & 0xc0) >> 6));
-            else
-              quantum=(Quantum) ((red_map[i][j][pixel.red] & 0xe0) |
-                ((green_map[i][j][pixel.green] & 0xe0) >> 3) |
-                ((blue_map[i][j][pixel.blue] & 0xc0) >> 6));
-            (void) WriteBlobByte(image,(unsigned char) quantum);
-            p++;
-            j++;
-            if (j == 16)
-              j=0;
-          }
-          i++;
-          if (i == 2)
-            i=0;
-          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-              (QuantumTick(y,image->rows) != MagickFalse))
+          q=pixels;
+          if (grayscale != MagickFalse)
             {
-              status=image->progress_monitor(SaveImageTag,y,image->rows,
-                image->client_data);
-              if (status == MagickFalse)
-                break;
+              if (depth <= 8)
+                for (x=0; x < (long) image->columns; x++)
+                {
+                  pixel=ScaleQuantumToAny(PixelIntensityToQuantum(p),
+                    (1UL << depth)-1);
+                  PopCharPixel(pixel,q);
+                  if (image->matte != MagickFalse)
+                    {
+                      pixel=ScaleQuantumToAny(p->opacity,(1UL << depth)-1);
+                      PopCharPixel(pixel,q);
+                    }
+                  p++;
+                }
+              else
+                for (x=0; x < (long) image->columns; x++)
+                {
+                  pixel=ScaleQuantumToAny(PixelIntensityToQuantum(p),
+                    (1UL << depth)-1);
+                  PopShortPixel(pixel,q);
+                  if (image->matte != MagickFalse)
+                    {
+                      pixel=ScaleQuantumToAny(p->opacity,(1UL << depth)-1);
+                      PopShortPixel(pixel,q);
+                    }
+                  p++;
+                }
             }
+          else
+            if (depth <= 8)
+              for (x=0; x < (long) image->columns; x++)
+              {
+                pixel=ScaleQuantumToAny(p->red,(1UL << depth)-1);
+                PopCharPixel(pixel,q);
+                pixel=ScaleQuantumToAny(p->green,(1UL << depth)-1);
+                PopCharPixel(pixel,q);
+                pixel=ScaleQuantumToAny(p->blue,(1UL << depth)-1);
+                PopCharPixel(pixel,q);
+                if (image->matte != MagickFalse)
+                  {
+                    pixel=ScaleQuantumToAny(p->opacity,(1UL << depth)-1);
+                    PopCharPixel(pixel,q);
+                  }
+                p++;
+              }
+            else
+              for (x=0; x < (long) image->columns; x++)
+              {
+                pixel=ScaleQuantumToAny(p->red,(1UL << depth)-1);
+                PopShortPixel(pixel,q);
+                pixel=ScaleQuantumToAny(p->green,(1UL << depth)-1);
+                PopShortPixel(pixel,q);
+                pixel=ScaleQuantumToAny(p->blue,(1UL << depth)-1);
+                PopShortPixel(pixel,q);
+                if (image->matte != MagickFalse)
+                  {
+                    pixel=ScaleQuantumToAny(p->opacity,(1UL << depth)-1);
+                    PopShortPixel(pixel,q);
+                  }
+                p++;
+              }
+          (void) WriteBlob(image,(size_t) (q-pixels),pixels);
+          if (image->previous == (Image *) NULL)
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
+              {
+                status=image->progress_monitor(SaveImageTag,y,image->rows,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
         }
-        /*
-          Free allocated memory.
-        */
-        for (i=0; i < 2; i++)
-          for (j=0; j < 16; j++)
-          {
-            green_map[i][j]=(unsigned long *)
-              RelinquishMagickMemory(green_map[i][j]);
-            blue_map[i][j]=(unsigned long *)
-              RelinquishMagickMemory(blue_map[i][j]);
-            red_map[i][j]=(unsigned long *)
-              RelinquishMagickMemory(red_map[i][j]);
-          }
+        pixels=(unsigned char *) RelinquishMagickMemory(pixels);
         break;
       }
     }
