@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -48,6 +48,7 @@
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
 #include "magick/delegate.h"
+#include "magick/draw.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
 #include "magick/geometry.h"
@@ -59,6 +60,7 @@
 #include "magick/monitor.h"
 #include "magick/profile.h"
 #include "magick/resource_.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/token.h"
@@ -171,17 +173,22 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
 #define DocumentMedia  "DocumentMedia:"
 #define EndDocument  "EndDocument:"
 #define HiResBoundingBox  "HiResBoundingBox:"
+#define ImageData  "ImageData:"
 #define PageBoundingBox  "PageBoundingBox:"
+#define LanguageLevel  "LanguageLevel:"
 #define PageMedia  "PageMedia:"
+#define Pages  "Pages:"
 #define PhotoshopProfile  "BeginPhotoshop:"
-#define PostscriptLevel  "%!PS-"
+#define PostscriptLevel  "!PS-"
 #define RenderPostscriptText  "  Rendering Postscript...  "
+#define XMLProfile  "begin_xml_packet:"
 
   char
     command[MaxTextExtent],
     density[MaxTextExtent],
     filename[MaxTextExtent],
     geometry[MaxTextExtent],
+    input_filename[MaxTextExtent],
     options[MaxTextExtent],
     postscript_filename[MaxTextExtent],
     *q,
@@ -191,10 +198,6 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   const DelegateInfo
     *delegate_info;
 
-  double
-    dx_resolution,
-    dy_resolution;
-
   GeometryInfo
     geometry_info;
 
@@ -217,6 +220,9 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickStatusType
     flags;
 
+  PointInfo
+    delta;
+
   RectangleInfo
     page;
 
@@ -236,11 +242,16 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     count;
 
   StringInfo
-    *iptc_profile;
+    *iptc_profile,
+    *xml_profile;
 
   unsigned long
+    columns,
     height,
+    language_level,
     length,
+    pages,
+    rows,
     scene,
     width;
 
@@ -261,6 +272,14 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image=DestroyImageList(image);
       return((Image *) NULL);
     }
+  status=AcquireUniqueSymbolicLink(image_info->filename,input_filename);
+  if (status == MagickFalse)
+    {
+      ThrowFileException(exception,FileOpenError,"UnableToCreateTemporaryFile",
+        image_info->filename);
+      image=DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Initialize hex values.
   */
@@ -289,27 +308,34 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Set the page density.
   */
-  dx_resolution=DefaultResolution;
-  dy_resolution=DefaultResolution;
-  if ((image->x_resolution == 0.0) || (image->y_resolution == 0.0))
+  delta.x=DefaultResolution;
+  delta.y=DefaultResolution;
+  if ((image->y_resolution == 0.0) || (image->y_resolution == 0.0))
     {
       flags=ParseGeometry(PSDensityGeometry,&geometry_info);
-      image->x_resolution=geometry_info.rho;
+      image->y_resolution=geometry_info.rho;
       image->y_resolution=geometry_info.sigma;
       if ((flags & SigmaValue) == 0)
-        image->y_resolution=image->x_resolution;
+        image->y_resolution=image->y_resolution;
     }
-  (void) FormatMagickString(density,MaxTextExtent,"%gx%g",image->x_resolution,
+  (void) FormatMagickString(density,MaxTextExtent,"%gx%g",image->y_resolution,
     image->y_resolution);
   /*
     Determine page geometry from the Postscript bounding box.
   */
   (void) ResetMagickMemory(&bounds,0,sizeof(bounds));
   (void) ResetMagickMemory(&page,0,sizeof(page));
+  (void) ResetMagickMemory(command,0,sizeof(command));
+  columns=0;
+  rows=0;
   iptc_profile=(StringInfo *) NULL;
+  xml_profile=(StringInfo *) NULL;
+  language_level=1;
   skip=MagickFalse;
   cmyk=MagickFalse;
-  for (p=command; (c=ReadBlobByte(image)) != EOF; )
+  pages=(~0UL);
+  p=command;
+  for (c=ReadBlobByte(image); c != EOF; c=ReadBlobByte(image))
   {
     if (image_info->page != (char *) NULL)
       continue;
@@ -317,7 +343,8 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       Note document structuring comments.
     */
     *p++=(char) c;
-    if ((c != (int) '%') && ((size_t) (p-command) < (MaxTextExtent-1)))
+    if ((c != (int) '%') && (c != '\n') &&
+        ((size_t) (p-command) < (MaxTextExtent/2)))
       continue;
     *p='\0';
     p=command;
@@ -330,12 +357,21 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       skip=MagickFalse;
     if (skip != MagickFalse)
       continue;
+    if (LocaleNCompare(PostscriptLevel,command,strlen(PostscriptLevel)) == 0)
+      if (GlobExpression(command,"*EPSF-*") != MagickFalse)
+        pages=1;
+    if (LocaleNCompare(LanguageLevel,command,strlen(LanguageLevel)) == 0)
+      (void) sscanf(command,LanguageLevel " %lu",&language_level);
+    if (LocaleNCompare(Pages,command,strlen(Pages)) == 0)
+      (void) sscanf(command,Pages " %lu",&pages);
+    if (LocaleNCompare(ImageData,command,strlen(Pages)) == 0)
+      (void) sscanf(command,ImageData " %lu %lu",&columns,&rows);
     if (LocaleNCompare(PhotoshopProfile,command,strlen(PhotoshopProfile)) == 0)
       {
         /*
           Read image profile.
         */
-        count=(ssize_t) sscanf(command,"BeginPhotoshop: %lu",&length);
+        count=(ssize_t) sscanf(command,PhotoshopProfile " %lu",&length);
         if (count != 1)
           continue;
         iptc_profile=AcquireStringInfo(length);
@@ -343,6 +379,19 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
           iptc_profile->datum[i]=(unsigned char)
             ProfileInteger(image,hex_digits);
       }
+    if (LocaleNCompare(XMLProfile,command,strlen(XMLProfile)) == 0)
+      {
+        /*
+          Read image profile.
+        */
+        count=(ssize_t) sscanf(command,XMLProfile " %lu",&length);
+        if (count != 1)
+          continue;
+        xml_profile=AcquireStringInfo(length);
+        for (i=0; i < (long) length; i++)
+          xml_profile->datum[i]=(unsigned char)
+            ProfileInteger(image,hex_digits);
+      }
     /*
       Is this a CMYK document?
     */
@@ -362,19 +411,19 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       continue;
     count=0;
     if (LocaleNCompare(BoundingBox,command,strlen(BoundingBox)) == 0)
-      count=(ssize_t) sscanf(command,"BoundingBox: %lf %lf %lf %lf",&bounds.x1,
+      count=(ssize_t) sscanf(command,BoundingBox " %lf %lf %lf %lf",&bounds.x1,
         &bounds.y1,&bounds.x2,&bounds.y2);
     if (LocaleNCompare(DocumentMedia,command,strlen(DocumentMedia)) == 0)
-      count=(ssize_t) sscanf(command,"DocumentMedia: %*s %lf %lf",&bounds.x2,
+      count=(ssize_t) sscanf(command,DocumentMedia " %*s %lf %lf",&bounds.x2,
         &bounds.y2)+2;
     if (LocaleNCompare(HiResBoundingBox,command,strlen(HiResBoundingBox)) == 0)
-      count=(ssize_t) sscanf(command,"HiResBoundingBox: %lf %lf %lf %lf",
+      count=(ssize_t) sscanf(command,HiResBoundingBox " %lf %lf %lf %lf",
         &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
     if (LocaleNCompare(PageBoundingBox,command,strlen(PageBoundingBox)) == 0)
-      count=(ssize_t) sscanf(command,"PageBoundingBox: %lf %lf %lf %lf",
+      count=(ssize_t) sscanf(command,PageBoundingBox " %lf %lf %lf %lf",
         &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
     if (LocaleNCompare(PageMedia,command,strlen(PageMedia)) == 0)
-      count=(ssize_t) sscanf(command,"PageMedia: %*s %lf %lf",&bounds.x2,
+      count=(ssize_t) sscanf(command,PageMedia " %*s %lf %lf",&bounds.x2,
         &bounds.y2)+2;
     if (count != 4)
       continue;
@@ -389,6 +438,8 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       page.height=height;
   }
   CloseBlob(image);
+  if (image_info->colorspace == RGBColorspace)
+    cmyk=MagickFalse;
   /*
     Create Ghostscript control file.
   */
@@ -415,19 +466,22 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (image_info->page != (char *) NULL)
     (void) ParseAbsoluteGeometry(image_info->page,&page);
   page.width=(unsigned long)
-    (page.width*image->x_resolution/dx_resolution+0.5);
+    (page.width*image->y_resolution/delta.x+0.5);
   page.height=(unsigned long)
-    (page.height*image->y_resolution/dy_resolution+0.5);
+    (page.height*image->y_resolution/delta.y+0.5);
   image=DestroyImageList(image);
   (void) FormatMagickString(geometry,MaxTextExtent,"%lux%lu",
     page.width,page.height);
   if (image_info->monochrome != MagickFalse)
-    delegate_info=GetDelegateInfo("gs-mono",(char *) NULL,exception);
+    delegate_info=GetDelegateInfo("ps:mono",(char *) NULL,exception);
   else
     if (cmyk != MagickFalse)
-      delegate_info=GetDelegateInfo("gs-cmyk",(char *) NULL,exception);
+      delegate_info=GetDelegateInfo("ps:cmyk",(char *) NULL,exception);
     else
-      delegate_info=GetDelegateInfo("gs-color",(char *) NULL,exception);
+      if ((pages == 1) && (language_level > 2))
+        delegate_info=GetDelegateInfo("ps:alpha",(char *) NULL,exception);
+      else
+        delegate_info=GetDelegateInfo("ps:color",(char *) NULL,exception);
   if (delegate_info == (const DelegateInfo *) NULL)
     {
       (void) RelinquishUniqueFileResource(postscript_filename);
@@ -455,7 +509,7 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     GetDelegateCommands(delegate_info),
     read_info->antialias != MagickFalse ? 4 : 1,
     read_info->antialias != MagickFalse ? 4 : 1,geometry,density,options,
-    read_info->filename,postscript_filename,image_info->filename);
+    read_info->filename,postscript_filename,input_filename);
   status=InvokePostscriptDelegate(read_info->verbose,command);
   image=ReadImage(read_info,exception);
   if (image == (Image *) NULL)
@@ -467,6 +521,7 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
   (void) RelinquishUniqueFileResource(postscript_filename);
   (void) RelinquishUniqueFileResource(read_info->filename);
+  (void) RelinquishUniqueFileResource(input_filename);
   if (image == (Image *) NULL)
     {
       /*
@@ -498,8 +553,14 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   do
   {
     (void) CopyMagickString(image->filename,filename,MaxTextExtent);
+    if (columns != 0)
+      image->magick_columns=columns;
+    if (rows != 0)
+      image->magick_rows=rows;
     if (iptc_profile != (StringInfo *) NULL)
       (void) SetImageProfile(image,"iptc",iptc_profile);
+    if (xml_profile != (StringInfo *) NULL)
+      (void) SetImageProfile(image,"xml",xml_profile);
     image->page=page;
     image->scene=scene++;
     next_image=SyncNextImageInList(image);
@@ -508,6 +569,8 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   } while (next_image != (Image *) NULL);
   if (iptc_profile != (StringInfo *) NULL)
     iptc_profile=DestroyStringInfo(iptc_profile);
+  if (xml_profile != (StringInfo *) NULL)
+    xml_profile=DestroyStringInfo(xml_profile);
   return(GetFirstImageInList(image));
 }
 
@@ -546,9 +609,9 @@ ModuleExport void RegisterPSImage(void)
   entry->adjoin=MagickFalse;
   entry->blob_support=MagickFalse;
   entry->seekable_stream=MagickTrue;
-  entry->description=
-    AcquireString("Encapsulated PostScript Interchange format");
-  entry->module=AcquireString("PS");
+  entry->description=ConstantString(
+   "Encapsulated PostScript Interchange format");
+  entry->module=ConstantString("PS");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("EPS");
   entry->decoder=(DecoderHandler *) ReadPSImage;
@@ -557,8 +620,8 @@ ModuleExport void RegisterPSImage(void)
   entry->adjoin=MagickFalse;
   entry->blob_support=MagickFalse;
   entry->seekable_stream=MagickTrue;
-  entry->description=AcquireString("Encapsulated PostScript");
-  entry->module=AcquireString("PS");
+  entry->description=ConstantString("Encapsulated PostScript");
+  entry->module=ConstantString("PS");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("EPSF");
   entry->decoder=(DecoderHandler *) ReadPSImage;
@@ -567,8 +630,8 @@ ModuleExport void RegisterPSImage(void)
   entry->adjoin=MagickFalse;
   entry->blob_support=MagickFalse;
   entry->seekable_stream=MagickTrue;
-  entry->description=AcquireString("Encapsulated PostScript");
-  entry->module=AcquireString("PS");
+  entry->description=ConstantString("Encapsulated PostScript");
+  entry->module=ConstantString("PS");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("EPSI");
   entry->decoder=(DecoderHandler *) ReadPSImage;
@@ -577,18 +640,18 @@ ModuleExport void RegisterPSImage(void)
   entry->adjoin=MagickFalse;
   entry->blob_support=MagickFalse;
   entry->seekable_stream=MagickTrue;
-  entry->description=
-    AcquireString("Encapsulated PostScript Interchange format");
-  entry->module=AcquireString("PS");
+  entry->description=ConstantString(
+    "Encapsulated PostScript Interchange format");
+  entry->module=ConstantString("PS");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PS");
   entry->decoder=(DecoderHandler *) ReadPSImage;
   entry->encoder=(EncoderHandler *) WritePSImage;
   entry->magick=(MagickHandler *) IsPS;
-  entry->module=AcquireString("PS");
+  entry->module=ConstantString("PS");
   entry->blob_support=MagickFalse;
   entry->seekable_stream=MagickTrue;
-  entry->description=AcquireString("PostScript");
+  entry->description=ConstantString("PostScript");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -932,14 +995,6 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
   const ImageAttribute
     *attribute;
 
-  double
-    dx_resolution,
-    dy_resolution,
-    x_resolution,
-    x_scale,
-    y_resolution,
-    y_scale;
-
   GeometryInfo
     geometry_info;
 
@@ -963,6 +1018,11 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
   PixelPacket
     pixel;
 
+  PointInfo
+    delta,
+    resolution,
+    scale;
+
   RectangleInfo
     geometry,
     media_info,
@@ -1016,30 +1076,31 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
     /*
       Scale relative to dots-per-inch.
     */
-    (void) SetImageColorspace(image,RGBColorspace);
-    dx_resolution=DefaultResolution;
-    dy_resolution=DefaultResolution;
+    if (image_info->colorspace == UndefinedColorspace)
+      (void) SetImageColorspace(image,RGBColorspace);
+    delta.x=DefaultResolution;
+    delta.y=DefaultResolution;
     flags=ParseGeometry(PSDensityGeometry,&geometry_info);
-    x_resolution=geometry_info.rho;
-    y_resolution=geometry_info.sigma;
+    resolution.x=geometry_info.rho;
+    resolution.y=geometry_info.sigma;
     if ((flags & SigmaValue) == 0)
-      y_resolution=x_resolution;
+      resolution.y=resolution.x;
     if (image->x_resolution != 0.0)
-      x_resolution=image->x_resolution;
+      resolution.x=image->x_resolution;
     if (image->y_resolution != 0.0)
-      y_resolution=image->y_resolution;
+      resolution.y=image->y_resolution;
     if (image_info->density != (char *) NULL)
       {
         flags=ParseGeometry(image_info->density,&geometry_info);
-        x_resolution=geometry_info.rho;
-        y_resolution=geometry_info.sigma;
+        resolution.x=geometry_info.rho;
+        resolution.y=geometry_info.sigma;
         if ((flags & SigmaValue) == 0)
-          y_resolution=x_resolution;
+          resolution.y=resolution.x;
       }
     if (image->units == PixelsPerCentimeterResolution)
       {
-        x_resolution*=2.54;
-        y_resolution*=2.54;
+        resolution.x*=2.54;
+        resolution.y*=2.54;
       }
     SetGeometry(image,&geometry);
     (void) FormatMagickString(page_geometry,MaxTextExtent,"%lux%lu",
@@ -1053,13 +1114,13 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
       else
         if ((image->gravity != UndefinedGravity) &&
             (LocaleCompare(image_info->magick,"PS") == 0))
-          (void) strcpy(page_geometry,PSPageGeometry);
+          (void) CopyMagickString(page_geometry,PSPageGeometry,MaxTextExtent);
     (void) ParseMetaGeometry(page_geometry,&geometry.x,&geometry.y,
       &geometry.width,&geometry.height);
-    x_scale=(double) (geometry.width*dx_resolution)/x_resolution;
-    geometry.width=(unsigned long) (x_scale+0.5);
-    y_scale=(double) (geometry.height*dy_resolution)/y_resolution;
-    geometry.height=(unsigned long) (y_scale+0.5);
+    scale.x=(double) (geometry.width*delta.x)/resolution.x;
+    geometry.width=(unsigned long) (scale.x+0.5);
+    scale.y=(double) (geometry.height*delta.y)/resolution.y;
+    geometry.height=(unsigned long) (scale.y+0.5);
     (void) ParseAbsoluteGeometry(page_geometry,&media_info);
     (void) ParseGravityGeometry(image,page_geometry,&page_info);
     if (image->gravity != UndefinedGravity)
@@ -1079,9 +1140,10 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
           Output Postscript header.
         */
         if (LocaleCompare(image_info->magick,"PS") == 0)
-          (void) strcpy(buffer,"%!PS-Adobe-3.0\n");
+          (void) CopyMagickString(buffer,"%!PS-Adobe-3.0\n",MaxTextExtent);
         else
-          (void) strcpy(buffer,"%!PS-Adobe-3.0 EPSF-3.0\n");
+          (void) CopyMagickString(buffer,"%!PS-Adobe-3.0 EPSF-3.0\n",
+            MaxTextExtent);
         (void) WriteBlobString(image,buffer);
         (void) WriteBlobString(image,"%%Creator: (ImageMagick)\n");
         (void) FormatMagickString(buffer,MaxTextExtent,"%%%%Title: (%s)\n",
@@ -1096,11 +1158,12 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
         (void) WriteBlobString(image,buffer);
         bounds.x1=(double) geometry.x;
         bounds.y1=(double) geometry.y;
-        bounds.x2=(double) geometry.x+x_scale;
+        bounds.x2=(double) geometry.x+scale.x;
         bounds.y2=(double) geometry.y+(geometry.height+text_size);
         if ((image_info->adjoin != MagickFalse) &&
             (GetNextImageInList(image) != (Image *) NULL))
-          (void) strcpy(buffer,"%%%%BoundingBox: (atend)\n");
+          (void) CopyMagickString(buffer,"%%%%BoundingBox: (atend)\n",
+            MaxTextExtent);
         else
           {
             (void) FormatMagickString(buffer,MaxTextExtent,
@@ -1271,7 +1334,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
       Output image data.
     */
     (void) FormatMagickString(buffer,MaxTextExtent,"%ld %ld\n%g %g\n%f\n",
-      geometry.x,geometry.y,x_scale,y_scale,image_info->pointsize);
+      geometry.x,geometry.y,scale.x,scale.y,image_info->pointsize);
     (void) WriteBlobString(image,buffer);
     labels=(char **) NULL;
     attribute=GetImageAttribute(image,"Label");
@@ -1489,7 +1552,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                 {
                   if ((image->matte != MagickFalse) &&
                       (p->opacity == TransparentOpacity))
-                    (void) strcpy(buffer,"ffffff");
+                    (void) CopyMagickString(buffer,"ffffff",MaxTextExtent);
                   else
                     (void) FormatMagickString(buffer,MaxTextExtent,
                       "%02X%02X%02X",ScaleQuantumToChar(p->red),
@@ -1568,7 +1631,8 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                       if (x > 0)
                         {
                           (void) FormatMagickString(buffer,MaxTextExtent,
-                            "%02X%02lX",index,(unsigned long) Min(length,0xff));
+                            "%02lX%02lX",(unsigned long) index,(unsigned long)
+                            Min(length,0xff));
                           (void) WriteBlobString(image,buffer);
                           i++;
                           if (i == 18)
@@ -1583,8 +1647,8 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                   pixel=(*p);
                   p++;
                 }
-                (void) FormatMagickString(buffer,MaxTextExtent,"%02X%02lX",
-                  index,(unsigned long) Min(length,0xff));
+                (void) FormatMagickString(buffer,MaxTextExtent,"%02lX%02lX",
+                  (unsigned long) index,(unsigned long) Min(length,0xff));
                 (void) WriteBlobString(image,buffer);
                 if (image->previous == (Image *) NULL)
                 if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
@@ -1614,8 +1678,8 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                 indexes=GetIndexes(image);
                 for (x=0; x < (long) image->columns; x++)
                 {
-                  (void) FormatMagickString(buffer,MaxTextExtent,"%02X",
-                    indexes[x]);
+                  (void) FormatMagickString(buffer,MaxTextExtent,"%02lX",
+                    (unsigned long) indexes[x]);
                   (void) WriteBlobString(image,buffer);
                   i++;
                   if (i == 36)
