@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -48,6 +48,7 @@
 #include "magick/color-private.h"
 #include "magick/compress.h"
 #include "magick/constitute.h"
+#include "magick/draw.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
 #include "magick/geometry.h"
@@ -59,6 +60,7 @@
 #include "magick/monitor.h"
 #include "magick/option.h"
 #include "magick/resource_.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/token.h"
@@ -121,13 +123,13 @@ ModuleExport void RegisterPS3Image(void)
 
   entry=SetMagickInfo("EPS3");
   entry->encoder=(EncoderHandler *) WritePS3Image;
-  entry->description=AcquireString("Level III Encapsulated PostScript");
-  entry->module=AcquireString("PS3");
+  entry->description=ConstantString("Level III Encapsulated PostScript");
+  entry->module=ConstantString("PS3");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PS3");
   entry->encoder=(EncoderHandler *) WritePS3Image;
-  entry->description=AcquireString("Level III PostScript");
-  entry->module=AcquireString("PS3");
+  entry->description=ConstantString("Level III PostScript");
+  entry->module=ConstantString("PS3");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -209,7 +211,7 @@ static MagickBooleanType SerializeImage(const ImageInfo *image_info,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=MagickTrue;
   *length=(image->colorspace == CMYKColorspace ? 4 : 3)*
     (size_t) image->columns*image->rows;
@@ -284,7 +286,7 @@ static MagickBooleanType SerializeImageChannel(const ImageInfo *image_info,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=MagickTrue;
   pack=(unsigned long)
     (IsMonochromeImage(image,&image->exception) != MagickFalse ? 8 : 1);
@@ -362,7 +364,7 @@ static MagickBooleanType SerializeImageIndexes(const ImageInfo *image_info,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=MagickTrue;
   *length=(size_t) image->columns*image->rows;
   *pixels=(unsigned char *) AcquireMagickMemory(*length);
@@ -420,20 +422,15 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,
   unsigned char
     *pixels;
 
-  unsigned long
-    quality;
-
   assert(image_info != (ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(image->matte != MagickFalse);
   status=MagickTrue;
   compression=image->compression;
-  quality=(unsigned long)
-    (image->quality == UndefinedCompressionQuality ? 75 : image->quality);
   /*
     Note BeginData DSC comment for update later.
   */
@@ -559,7 +556,7 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,
       status=SerializeImageChannel(image_info,mask_image,&pixels,&length);
       if (status == MagickFalse)
         break;
-      status=ZLIBEncodeImage(image,length,quality,pixels);
+      status=ZLIBEncodeImage(image,length,pixels);
       pixels=(unsigned char *) RelinquishMagickMemory(pixels);
       break;
     }
@@ -791,14 +788,6 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
   const ImageAttribute
     *attribute;
 
-  double
-    dx_resolution,
-    dy_resolution,
-    x_resolution,
-    x_scale,
-    y_resolution,
-    y_scale;
-
   GeometryInfo
     geometry_info;
 
@@ -816,6 +805,11 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
   MagickStatusType
     flags;
 
+  PointInfo
+    delta,
+    resolution,
+    scale;
+
   RectangleInfo
     geometry,
     media_info,
@@ -849,7 +843,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(MagickFalse);
@@ -886,29 +880,29 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     /*
       Scale relative to dots-per-inch.
     */
-    dx_resolution=DefaultResolution;
-    dy_resolution=DefaultResolution;
+    delta.x=DefaultResolution;
+    delta.y=DefaultResolution;
     flags=ParseGeometry(PSDensityGeometry,&geometry_info);
-    x_resolution=geometry_info.rho;
-    y_resolution=geometry_info.sigma;
+    resolution.x=geometry_info.rho;
+    resolution.y=geometry_info.sigma;
     if ((flags & SigmaValue) == 0)
-      y_resolution=x_resolution;
+      resolution.y=resolution.x;
     if (image->x_resolution != 0.0)
-      x_resolution=image->x_resolution;
+      resolution.x=image->x_resolution;
     if (image->y_resolution != 0.0)
-      y_resolution=image->y_resolution;
+      resolution.y=image->y_resolution;
     if (image_info->density != (char *) NULL)
       {
         flags=ParseGeometry(image_info->density,&geometry_info);
-        x_resolution=geometry_info.rho;
-        y_resolution=geometry_info.sigma;
+        resolution.x=geometry_info.rho;
+        resolution.y=geometry_info.sigma;
         if ((flags & SigmaValue) == 0)
-          y_resolution=x_resolution;
+          resolution.y=resolution.x;
       }
     if (image->units == PixelsPerCentimeterResolution)
       {
-        x_resolution*=2.54;
-        y_resolution*=2.54;
+        resolution.x*=2.54;
+        resolution.y*=2.54;
       }
     SetGeometry(image,&geometry);
     (void) FormatMagickString(page_geometry,MaxTextExtent,"%lux%lu",
@@ -922,13 +916,13 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
       else
         if ((image->gravity != UndefinedGravity) &&
             (LocaleCompare(image_info->magick,"PS") == 0))
-          (void) strcpy(page_geometry,PSPageGeometry);
+          (void) CopyMagickString(page_geometry,PSPageGeometry,MaxTextExtent);
     (void) ParseMetaGeometry(page_geometry,&geometry.x,&geometry.y,
       &geometry.width,&geometry.height);
-    x_scale=(double) (geometry.width*dx_resolution)/x_resolution;
-    geometry.width=(unsigned long) (x_scale+0.5);
-    y_scale=(double) (geometry.height*dy_resolution)/y_resolution;
-    geometry.height=(unsigned long) (y_scale+0.5);
+    scale.x=(double) (geometry.width*delta.x)/resolution.x;
+    geometry.width=(unsigned long) (scale.x+0.5);
+    scale.y=(double) (geometry.height*delta.y)/resolution.y;
+    geometry.height=(unsigned long) (scale.y+0.5);
     (void) ParseAbsoluteGeometry(page_geometry,&media_info);
     (void) ParseGravityGeometry(image,page_geometry,&page_info);
     if (image->gravity != UndefinedGravity)
@@ -948,9 +942,10 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
           Postscript header on the first page.
         */
         if (LocaleCompare(image_info->magick,"PS3") == 0)
-          (void) strcpy(buffer,"%!PS-Adobe-3.0\n");
+          (void) CopyMagickString(buffer,"%!PS-Adobe-3.0\n",MaxTextExtent);
         else
-          (void) strcpy(buffer,"%!PS-Adobe-3.0 EPSF-3.0\n");
+          (void) CopyMagickString(buffer,"%!PS-Adobe-3.0 EPSF-3.0\n",
+            MaxTextExtent);
         (void) WriteBlobString(image,buffer);
         (void) FormatMagickString(buffer,MaxTextExtent,
           "%%%%Creator: ImageMagick %s\n",MagickLibVersionText);
@@ -967,10 +962,10 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
         (void) WriteBlobString(image,buffer);
         bounds.x1=(double) geometry.x;
         bounds.y1=(double) geometry.y;
-        bounds.x2=(double) geometry.x+x_scale;
-        bounds.y2=(double) geometry.y+y_scale+text_size;
+        bounds.x2=(double) geometry.x+scale.x;
+        bounds.y2=(double) geometry.y+scale.y+text_size;
         if ((image_info->adjoin != MagickFalse) &&
-            (image->next != (Image *) NULL))
+            (GetNextImageInList(image) != (Image *) NULL))
           {
             (void) WriteBlobString(image,"%%BoundingBox: (atend)\n");
             (void) WriteBlobString(image,"%%HiResBoundingBox: (atend)\n");
@@ -1011,7 +1006,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
             (void) WriteBlobString(image,"%%Orientation: Portrait\n");
             (void) WriteBlobString(image,"%%PageOrder: Ascend\n");
             if (image_info->adjoin == MagickFalse)
-              (void) strcpy(buffer,"%%Pages: 1\n");
+              (void) CopyMagickString(buffer,"%%Pages: 1\n",MaxTextExtent);
             else
               (void) FormatMagickString(buffer,MaxTextExtent,"%%%%Pages: %lu\n",
                 (unsigned long) GetImageListLength(image));
@@ -1079,10 +1074,10 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
       bounds.x1=(double) geometry.x;
     if ((double) geometry.y < bounds.y1)
       bounds.y1=(double) geometry.y;
-    if ((double) (geometry.x+x_scale) > bounds.x2)
-      bounds.x2=(double) geometry.x+x_scale;
-    if ((double) (geometry.y+y_scale+text_size) > bounds.y2)
-      bounds.y2=(double) geometry.y+y_scale+text_size;
+    if ((double) (geometry.x+scale.x) > bounds.x2)
+      bounds.x2=(double) geometry.x+scale.x;
+    if ((double) (geometry.y+scale.y+text_size) > bounds.y2)
+      bounds.y2=(double) geometry.y+scale.y+text_size;
     /*
       Page font resource if there's a label.
     */
@@ -1114,12 +1109,12 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     /*
       Image mask.
     */
-    if ((LocaleCompare(image_info->magick,"EPS3") != 0) && image->matte)
-      if (WritePS3MaskImage(image_info,image) == MagickFalse)
-        {
-          CloseBlob(image);
-          return(MagickFalse);
-        }
+    if ((image->matte != MagickFalse) &&
+        (WritePS3MaskImage(image_info,image) == MagickFalse))
+      {
+        CloseBlob(image);
+        return(MagickFalse);
+      }
     /*
       Remember position of BeginData comment so we can update it.
     */
@@ -1134,7 +1129,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
       Translate, scale, and font point size.
     */
     (void) FormatMagickString(buffer,MaxTextExtent,"%ld %ld\n%g %g\n%f\n",
-      geometry.x,geometry.y,x_scale,y_scale,image_info->pointsize);
+      geometry.x,geometry.y,scale.x,scale.y,image_info->pointsize);
     (void) WriteBlobString(image,buffer);
     /*
       Output labels.
@@ -1190,15 +1185,15 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     /*
       Masked image?
     */
-    (void) WriteBlobString(image,
-      (LocaleCompare(image_info->magick,"EPS3") != 0) && image->matte ?
+    (void) WriteBlobString(image,image->matte != MagickFalse ?
       "true\n" : "false\n");
     /*
       Render with imagemask operator?
     */
     option=GetImageOption(image_info,"ps3:imagemask");
-    (void) WriteBlobString(image,
-      option != (const char *) NULL ? "true\n" : "false\n");
+    (void) WriteBlobString(image,(option != (const char *) NULL) &&
+      (IsMonochromeImage(image,&image->exception) != MagickFalse) ?
+      "true\n" : "false\n");
     /*
       Output pixel data.
     */
@@ -1304,7 +1299,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
               }
               case ZipCompression:
               {
-                status=ZLIBEncodeImage(image,length,image->quality,pixels);
+                status=ZLIBEncodeImage(image,length,pixels);
                 break;
               }
             }
@@ -1388,7 +1383,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
                 }
                 case ZipCompression:
                 {
-                  status=ZLIBEncodeImage(image,length,image->quality,pixels);
+                  status=ZLIBEncodeImage(image,length,pixels);
                   break;
                 }
               }
@@ -1500,7 +1495,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
               }
               case ZipCompression:
               {
-                status=ZLIBEncodeImage(image,length,image->quality,pixels);
+                status=ZLIBEncodeImage(image,length,pixels);
                 break;
               }
             }
@@ -1530,7 +1525,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     if (LocaleCompare(image_info->magick,"PS3") != 0)
       (void) WriteBlobString(image,"end\n");
     (void) WriteBlobString(image,"%%PageTrailer\n");
-    if (image->next == (Image *) NULL)
+    if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
