@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -53,6 +53,7 @@
 #include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/pixel.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/statistic.h"
 #include "magick/string_.h"
@@ -118,7 +119,7 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
     packet_size;
 
   unsigned char
-    *scanline;
+    *pixels;
 
   unsigned long
     depth,
@@ -149,9 +150,9 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         (void) ReadBlobByte(image);
     }
   /*
-    Allocate memory for a scanline.
+    Allocate memory for a pixels.
   */
-  depth=GetImageQuantumDepth(image,MagickTrue);
+  depth=GetImageQuantumDepth(image,MagickFalse);
   packet_size=(size_t) (3*depth/8);
   if ((LocaleCompare(image_info->magick,"RGBA") == 0) ||
       (LocaleCompare(image_info->magick,"RGBO") == 0))
@@ -159,9 +160,9 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       packet_size+=depth/8;
       image->matte=MagickTrue;
     }
-  scanline=(unsigned char *)
-    AcquireMagickMemory(packet_size*image->extract_info.width);
-  if (scanline == (unsigned char *) NULL)
+  pixels=(unsigned char *)
+    AcquireMagickMemory(packet_size*image->extract_info.width*sizeof(*pixels));
+  if (pixels == (unsigned char *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   if (image_info->number_scenes != 0)
     while (image->scene < image_info->scene)
@@ -172,7 +173,7 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image->scene++;
       for (y=0; y < (long) image->rows; y++)
       {
-        count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+        count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
         if (count != (ssize_t) (packet_size*image->extract_info.width))
           break;
       }
@@ -196,7 +197,7 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         for (y=0; y < image->extract_info.y; y++)
         {
-          count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
           if (count != (ssize_t) (packet_size*image->extract_info.width))
             break;
         }
@@ -205,7 +206,7 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if ((y > 0) || (GetPreviousImageInList(image) == (Image *) NULL))
             {
               count=ReadBlob(image,packet_size*image->extract_info.width,
-                scanline);
+                pixels);
               if (count != (ssize_t) (packet_size*image->extract_info.width))
                 break;
             }
@@ -213,12 +214,12 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if (q == (PixelPacket *) NULL)
             break;
           if (image->matte == MagickFalse)
-            (void) ExportQuantumPixels(image,RGBQuantum,0,scanline+offset);
+            (void) ExportQuantumPixels(image,RGBQuantum,0,pixels+offset);
           else
             if (LocaleCompare(image_info->magick,"RGBA") == 0)
-              (void) ExportQuantumPixels(image,RGBAQuantum,0,scanline+offset);
+              (void) ExportQuantumPixels(image,RGBAQuantum,0,pixels+offset);
             else
-              (void) ExportQuantumPixels(image,RGBOQuantum,0,scanline+offset);
+              (void) ExportQuantumPixels(image,RGBOQuantum,0,pixels+offset);
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
@@ -234,7 +235,7 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         width=image->extract_info.height-image->rows-image->extract_info.y;
         for (i=0; i < (long) width; i++)
         {
-          count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
           if (count != (ssize_t) (packet_size*image->extract_info.width))
             break;
         }
@@ -248,7 +249,7 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         packet_size=(size_t) (image->depth > 8 ? 2 : 1);
         for (y=0; y < image->extract_info.y; y++)
         {
-          count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
           if (count != (ssize_t) (packet_size*image->extract_info.width))
             break;
         }
@@ -257,34 +258,34 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if ((y > 0) || (GetPreviousImageInList(image) == (Image *) NULL))
             {
               count=ReadBlob(image,packet_size*image->extract_info.width,
-                scanline);
+                pixels);
               if (count != (ssize_t) (packet_size*image->extract_info.width))
                 break;
             }
           q=SetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          (void) ExportQuantumPixels(image,RedQuantum,0,scanline+offset);
-          count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          (void) ExportQuantumPixels(image,RedQuantum,0,pixels+offset);
+          count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
           if (count != (ssize_t) (packet_size*image->extract_info.width))
             break;
-          (void) ExportQuantumPixels(image,GreenQuantum,0,scanline+offset);
-          count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          (void) ExportQuantumPixels(image,GreenQuantum,0,pixels+offset);
+          count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
           if (count != (ssize_t) (packet_size*image->extract_info.width))
             break;
-          (void) ExportQuantumPixels(image,BlueQuantum,0,scanline+offset);
+          (void) ExportQuantumPixels(image,BlueQuantum,0,pixels+offset);
           if (image->matte != MagickFalse)
             {
               count=ReadBlob(image,packet_size*image->extract_info.width,
-                scanline);
+                pixels);
               if (count != (ssize_t) (packet_size*image->extract_info.width))
                 break;
               if (LocaleCompare(image_info->magick,"RGBA") == 0)
                 (void) ExportQuantumPixels(image,AlphaQuantum,0,
-                  scanline+offset);
+                  pixels+offset);
               else
                 (void) ExportQuantumPixels(image,OpacityQuantum,0,
-                  scanline+offset);
+                  pixels+offset);
             }
           if (SyncImagePixels(image) == MagickFalse)
             break;
@@ -301,7 +302,7 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         width=image->extract_info.height-image->rows-image->extract_info.y;
         for (i=0; i < (long) width; i++)
         {
-          count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
           if (count != (ssize_t) (packet_size*image->extract_info.width))
             break;
         }
@@ -329,7 +330,7 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         packet_size=(size_t) (image->depth > 8 ? 2 : 1);
         for (y=0; y < image->extract_info.y; y++)
         {
-          count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
           if (count != (ssize_t) (packet_size*image->extract_info.width))
             break;
         }
@@ -340,14 +341,14 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if ((y > 0) || (GetPreviousImageInList(image) == (Image *) NULL))
             {
               count=ReadBlob(image,packet_size*image->extract_info.width,
-                scanline);
+                pixels);
               if (count != (ssize_t) (packet_size*image->extract_info.width))
                 break;
             }
           q=SetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          (void) ExportQuantumPixels(image,RedQuantum,0,scanline+offset);
+          (void) ExportQuantumPixels(image,RedQuantum,0,pixels+offset);
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
@@ -364,7 +365,7 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         width=image->extract_info.height-image->rows-image->extract_info.y;
         for (i=0; i < (long) width; i++)
         {
-          count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
           if (count != (ssize_t) (packet_size*image->extract_info.width))
             break;
         }
@@ -381,19 +382,19 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           }
         for (y=0; y < image->extract_info.y; y++)
         {
-          count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
           if (count != (ssize_t) (packet_size*image->extract_info.width))
             break;
         }
         for (y=0; y < (long) image->rows; y++)
         {
-          count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
           if (count != (ssize_t) (packet_size*image->extract_info.width))
             break;
           q=GetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          (void) ExportQuantumPixels(image,GreenQuantum,0,scanline+offset);
+          (void) ExportQuantumPixels(image,GreenQuantum,0,pixels+offset);
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
@@ -410,7 +411,7 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         width=image->extract_info.height-image->rows-image->extract_info.y;
         for (i=0; i < (long) width; i++)
         {
-          count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
           if (count != (ssize_t) (packet_size*image->extract_info.width))
             break;
         }
@@ -427,19 +428,19 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           }
         for (y=0; y < image->extract_info.y; y++)
         {
-          count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
           if (count != (ssize_t) (packet_size*image->extract_info.width))
             break;
         }
         for (y=0; y < (long) image->rows; y++)
         {
-          count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
           if (count != (ssize_t) (packet_size*image->extract_info.width))
             break;
           q=GetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          (void) ExportQuantumPixels(image,BlueQuantum,0,scanline+offset);
+          (void) ExportQuantumPixels(image,BlueQuantum,0,pixels+offset);
           if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
@@ -456,7 +457,7 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         width=image->extract_info.height-image->rows-image->extract_info.y;
         for (i=0; i < (long) width; i++)
         {
-          count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
           if (count != (ssize_t) (packet_size*image->extract_info.width))
             break;
         }
@@ -479,14 +480,14 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
             for (y=0; y < image->extract_info.y; y++)
             {
               count=ReadBlob(image,packet_size*image->extract_info.width,
-                scanline);
+                pixels);
               if (count != (ssize_t) (packet_size*image->extract_info.width))
                 break;
             }
             for (y=0; y < (long) image->rows; y++)
             {
               count=ReadBlob(image,packet_size*image->extract_info.width,
-                scanline);
+                pixels);
               if (count != (ssize_t) (packet_size*image->extract_info.width))
                 break;
               q=GetImagePixels(image,0,y,image->columns,1);
@@ -494,10 +495,10 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 break;
               if (LocaleCompare(image_info->magick,"RGBA") == 0)
                 (void) ExportQuantumPixels(image,AlphaQuantum,0,
-                  scanline+offset);
+                  pixels+offset);
               else
                 (void) ExportQuantumPixels(image,OpacityQuantum,0,
-                  scanline+offset);
+                  pixels+offset);
               if (SyncImagePixels(image) == MagickFalse)
                 break;
               if (image->previous == (Image *) NULL)
@@ -515,7 +516,7 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
             for (i=0; i < (long) width; i++)
             {
               count=ReadBlob(image,packet_size*image->extract_info.width,
-                scanline);
+                pixels);
               if (count != (ssize_t) (packet_size*image->extract_info.width))
                 break;
             }
@@ -540,7 +541,7 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         break;
     if (image_info->interlace == PartitionInterlace)
       break;
-    count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+    count=ReadBlob(image,packet_size*image->extract_info.width,pixels);
     if (count == (ssize_t) (packet_size*image->extract_info.width))
       {
         /*
@@ -561,8 +562,8 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
               break;
           }
       }
-  } while (count == (packet_size*image->extract_info.width));
-  scanline=(unsigned char *) RelinquishMagickMemory(scanline);
+  } while ((size_t) count == (packet_size*image->extract_info.width));
+  pixels=(unsigned char *) RelinquishMagickMemory(pixels);
   CloseBlob(image);
   return(GetFirstImageInList(image));
 }
@@ -600,24 +601,24 @@ ModuleExport void RegisterRGBImage(void)
   entry->encoder=(EncoderHandler *) WriteRGBImage;
   entry->raw=MagickTrue;
   entry->endian_support=MagickTrue;
-  entry->description=AcquireString("Raw red, green, and blue samples");
-  entry->module=AcquireString("RGB");
+  entry->description=ConstantString("Raw red, green, and blue samples");
+  entry->module=ConstantString("RGB");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("RGBA");
   entry->decoder=(DecoderHandler *) ReadRGBImage;
   entry->encoder=(EncoderHandler *) WriteRGBImage;
   entry->raw=MagickTrue;
   entry->endian_support=MagickTrue;
-  entry->description=AcquireString("Raw red, green, blue, and alpha samples");
-  entry->module=AcquireString("RGB");
+  entry->description=ConstantString("Raw red, green, blue, and alpha samples");
+  entry->module=ConstantString("RGB");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("RGBO");
   entry->decoder=(DecoderHandler *) ReadRGBImage;
   entry->encoder=(EncoderHandler *) WriteRGBImage;
   entry->raw=MagickTrue;
   entry->endian_support=MagickTrue;
-  entry->description=AcquireString("Raw red, green, blue, and opacity samples");
-  entry->module=AcquireString("RGB");
+  entry->description=ConstantString("Raw red, green, blue, and opacity samples");
+  entry->module=ConstantString("RGB");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -704,7 +705,7 @@ static MagickBooleanType WriteRGBImage(const ImageInfo *image_info,Image *image)
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  depth=GetImageQuantumDepth(image,MagickTrue);
+  depth=GetImageQuantumDepth(image,MagickFalse);
   packet_size=(size_t) (3*depth/8);
   if ((LocaleCompare(image_info->magick,"RGBA") == 0) ||
       (LocaleCompare(image_info->magick,"RGBO") == 0))
@@ -728,7 +729,8 @@ static MagickBooleanType WriteRGBImage(const ImageInfo *image_info,Image *image)
     /*
       Convert MIFF to RGB raster pixels.
     */
-    (void) SetImageColorspace(image,RGBColorspace);
+    if (image_info->colorspace == UndefinedColorspace)
+      (void) SetImageColorspace(image,RGBColorspace);
     if (LocaleCompare(image_info->magick,"RGBA") == 0)
       if (image->matte == MagickFalse)
         SetImageOpacity(image,OpaqueOpacity);
