@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -51,6 +51,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 
@@ -183,7 +184,8 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
@@ -440,7 +442,7 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
             q->green=ScaleCharToQuantum(*p++);
             q->blue=ScaleCharToQuantum(*p++);
             if (image->matte != MagickFalse)
-              q->opacity=(Quantum) (MaxRGB-ScaleCharToQuantum(*p++));
+              q->opacity=(Quantum) (QuantumRange-ScaleCharToQuantum(*p++));
             q++;
           }
           if (SyncImagePixels(image) == MagickFalse)
@@ -472,9 +474,9 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
             /*
               Pseudocolor.
             */
-            image->colormap[i].red=ScaleCharToQuantum(i);
-            image->colormap[i].green=ScaleCharToQuantum(i);
-            image->colormap[i].blue=ScaleCharToQuantum(i);
+            image->colormap[i].red=ScaleCharToQuantum((unsigned char) i);
+            image->colormap[i].green=ScaleCharToQuantum((unsigned char) i);
+            image->colormap[i].blue=ScaleCharToQuantum((unsigned char) i);
           }
         else
           if (number_colormaps > 1)
@@ -528,7 +530,7 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 q->red=image->colormap[*p++].red;
                 q->green=image->colormap[*p++].green;
                 q->blue=image->colormap[*p++].blue;
-                q->opacity=(Quantum) (MaxRGB-ScaleCharToQuantum(*p++));
+                q->opacity=(Quantum) (QuantumRange-ScaleCharToQuantum(*p++));
                 q++;
               }
               if (SyncImagePixels(image) == MagickFalse)
@@ -572,7 +574,7 @@ static Image *ReadRLEImage(const ImageInfo *image_info,ExceptionInfo *exception)
           Allocate next image structure.
         */
         AllocateNextImage(image_info,image);
-        if (image->next == (Image *) NULL)
+        if (GetNextImageInList(image) == (Image *) NULL)
           {
             image=DestroyImageList(image);
             return((Image *) NULL);
@@ -623,8 +625,8 @@ ModuleExport void RegisterRLEImage(void)
   entry->decoder=(DecoderHandler *) ReadRLEImage;
   entry->magick=(MagickHandler *) IsRLE;
   entry->adjoin=MagickFalse;
-  entry->description=AcquireString("Utah Run length encoded image");
-  entry->module=AcquireString("RLE");
+  entry->description=ConstantString("Utah Run length encoded image");
+  entry->module=ConstantString("RLE");
   (void) RegisterMagickInfo(entry);
 }
 
