@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -53,6 +53,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 
@@ -271,7 +272,8 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
@@ -468,7 +470,7 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
     /*
       Initialize image structure.
     */
-    image->matte=(MagickBooleanType) (iris_info.depth == 4);
+    image->matte=iris_info.depth == 4 ? MagickTrue : MagickFalse;
     image->columns=iris_info.columns;
     image->rows=iris_info.rows;
     /*
@@ -489,11 +491,16 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 break;
               for (x=0; x < (long) image->columns; x++)
               {
-                q->red=ScaleShortToQuantum((*(p+0) << 8) | (*(p+1)));
-                q->green=ScaleShortToQuantum((*(p+2) << 8) | (*(p+3)));
-                q->blue=ScaleShortToQuantum((*(p+4) << 8) | (*(p+5)));
-                q->opacity=(Quantum)
-                  (MaxRGB-ScaleShortToQuantum((*(p+6) << 8) | (*(p+7))));
+                q->red=ScaleShortToQuantum((unsigned short)
+                  ((*(p+0) << 8) | (*(p+1))));
+                q->green=ScaleShortToQuantum((unsigned short)
+                  ((*(p+2) << 8) | (*(p+3))));
+                q->blue=ScaleShortToQuantum((unsigned short)
+                  ((*(p+4) << 8) | (*(p+5))));
+                q->opacity=OpaqueOpacity;
+                if (image->matte != MagickFalse)
+                  q->opacity=(Quantum) (QuantumRange-ScaleShortToQuantum(
+                    (unsigned short) ((*(p+6) << 8) | (*(p+7)))));
                 p+=8;
                 q++;
               }
@@ -522,7 +529,9 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
               q->red=ScaleCharToQuantum(*p);
               q->green=ScaleCharToQuantum(*(p+1));
               q->blue=ScaleCharToQuantum(*(p+2));
-              q->opacity=(Quantum) (MaxRGB-ScaleCharToQuantum(*(p+3)));
+              q->opacity=OpaqueOpacity;
+              if (image->matte != MagickFalse)
+                q->opacity=(Quantum) (QuantumRange-ScaleCharToQuantum(*(p+3)));
               p+=4;
               q++;
             }
@@ -626,7 +635,7 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
           Allocate next image structure.
         */
         AllocateNextImage(image_info,image);
-        if (image->next == (Image *) NULL)
+        if (GetNextImageInList(image) == (Image *) NULL)
           {
             image=DestroyImageList(image);
             return((Image *) NULL);
@@ -677,8 +686,8 @@ ModuleExport void RegisterSGIImage(void)
   entry->decoder=(DecoderHandler *) ReadSGIImage;
   entry->encoder=(EncoderHandler *) WriteSGIImage;
   entry->magick=(MagickHandler *) IsSGI;
-  entry->description=AcquireString("Irix RGB image");
-  entry->module=AcquireString("SGI");
+  entry->description=ConstantString("Irix RGB image");
+  entry->module=ConstantString("SGI");
   entry->seekable_stream=MagickTrue;
   (void) RegisterMagickInfo(entry);
 }
@@ -826,7 +835,7 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if ((image->columns > 65535UL) || (image->rows > 65535UL))
     ThrowWriterException(ImageError,"WidthOrHeightExceedsLimit");
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
@@ -838,7 +847,8 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
     /*
       Initialize SGI raster file header.
     */
-    (void) SetImageColorspace(image,RGBColorspace);
+    if (image_info->colorspace == UndefinedColorspace)
+      (void) SetImageColorspace(image,RGBColorspace);
     iris_info.magic=0x01DA;
     if (image->compression == NoCompression)
       iris_info.storage=(unsigned char) 0x00;
@@ -862,7 +872,7 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
           iris_info.depth=3;
       }
     iris_info.minimum_value=0;
-    iris_info.maximum_value=(unsigned long) ScaleQuantumToChar(MaxRGB);
+    iris_info.maximum_value=(unsigned long) ScaleQuantumToChar(QuantumRange);
     for (i=0; i < (int) sizeof(iris_info.filler); i++)
       iris_info.filler[i]=(unsigned char) 0;
     /*
@@ -902,7 +912,7 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
         *q++=ScaleQuantumToChar(p->red);
         *q++=ScaleQuantumToChar(p->green);
         *q++=ScaleQuantumToChar(p->blue);
-        *q++=ScaleQuantumToChar(MaxRGB-p->opacity);
+        *q++=ScaleQuantumToChar((Quantum) (QuantumRange-p->opacity));
         p++;
       }
       if (image->previous == (Image *) NULL)
@@ -1000,7 +1010,7 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
         offsets=(ssize_t *) RelinquishMagickMemory(offsets);
       }
     iris_pixels=(unsigned char *) RelinquishMagickMemory(iris_pixels);
-    if (image->next == (Image *) NULL)
+    if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
@@ -1012,9 +1022,6 @@ static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
       }
     scene++;
   } while (image_info->adjoin != MagickFalse);
-  if (image_info->adjoin != MagickFalse)
-    while (image->previous != (Image *) NULL)
-      image=image->previous;
   CloseBlob(image);
   return(MagickTrue);
 }
