@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -51,6 +51,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 
@@ -124,7 +125,8 @@ static Image *ReadSTEGANOImage(const ImageInfo *image_info,
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
@@ -203,7 +205,7 @@ static Image *ReadSTEGANOImage(const ImageInfo *image_info,
       }
   }
   watermark=DestroyImage(watermark);
-  SyncImage(image);
+  (void) SyncImage(image);
   return(GetFirstImageInList(image));
 }
 
@@ -237,8 +239,8 @@ ModuleExport void RegisterSTEGANOImage(void)
 
   entry=SetMagickInfo("STEGANO");
   entry->decoder=(DecoderHandler *) ReadSTEGANOImage;
-  entry->description=AcquireString("Steganographic image");
-  entry->module=AcquireString("STEGANO");
+  entry->description=ConstantString("Steganographic image");
+  entry->module=ConstantString("STEGANO");
   (void) RegisterMagickInfo(entry);
 }
 
