@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -53,6 +53,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 
@@ -270,7 +271,8 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
@@ -283,7 +285,7 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Read SUN raster header.
   */
-  ResetMagickMemory(&sun_info,0,sizeof(sun_info));
+  (void) ResetMagickMemory(&sun_info,0,sizeof(sun_info));
   sun_info.magic=ReadBlobMSBLong(image);
   do
   {
@@ -299,8 +301,8 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
     sun_info.type=ReadBlobMSBLong(image);
     sun_info.maptype=ReadBlobMSBLong(image);
     sun_info.maplength=ReadBlobMSBLong(image);
-    image->columns= sun_info.width;
-    image->rows= sun_info.height;
+    image->columns=sun_info.width;
+    image->rows=sun_info.height;
     image->depth=sun_info.depth <= 8 ? 8UL : QuantumDepth;
     if (sun_info.depth < 24)
       {
@@ -471,7 +473,7 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
           for (x=0; x < (long) image->columns; x++)
           {
             if (image->matte != MagickFalse)
-              q->opacity=(Quantum) (MaxRGB-ScaleCharToQuantum(*p++));
+              q->opacity=(Quantum) (QuantumRange-ScaleCharToQuantum(*p++));
             if (sun_info.type == RT_STANDARD)
               {
                 q->blue=ScaleCharToQuantum(*p++);
@@ -528,7 +530,7 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
           Allocate next image structure.
         */
         AllocateNextImage(image_info,image);
-        if (image->next == (Image *) NULL)
+        if (GetNextImageInList(image) == (Image *) NULL)
           {
             image=DestroyImageList(image);
             return((Image *) NULL);
@@ -579,14 +581,14 @@ ModuleExport void RegisterSUNImage(void)
   entry->decoder=(DecoderHandler *) ReadSUNImage;
   entry->encoder=(EncoderHandler *) WriteSUNImage;
   entry->magick=(MagickHandler *) IsSUN;
-  entry->description=AcquireString("SUN Rasterfile");
-  entry->module=AcquireString("SUN");
+  entry->description=ConstantString("SUN Rasterfile");
+  entry->module=ConstantString("SUN");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("SUN");
   entry->decoder=(DecoderHandler *) ReadSUNImage;
   entry->encoder=(EncoderHandler *) WriteSUNImage;
-  entry->description=AcquireString("SUN Rasterfile");
-  entry->module=AcquireString("SUN");
+  entry->description=ConstantString("SUN Rasterfile");
+  entry->module=ConstantString("SUN");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -696,7 +698,7 @@ static MagickBooleanType WriteSUNImage(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
@@ -706,7 +708,8 @@ static MagickBooleanType WriteSUNImage(const ImageInfo *image_info,Image *image)
     /*
       Initialize SUN raster file header.
     */
-    (void) SetImageColorspace(image,RGBColorspace);
+    if (image_info->colorspace == UndefinedColorspace)
+      (void) SetImageColorspace(image,RGBColorspace);
     sun_info.magic=0x59a66a95;
     sun_info.width=image->columns;
     sun_info.height=image->rows;
@@ -794,7 +797,7 @@ static MagickBooleanType WriteSUNImage(const ImageInfo *image_info,Image *image)
           for (x=0; x < (long) image->columns; x++)
           {
             if (image->matte != MagickFalse)
-              *q++=ScaleQuantumToChar(MaxRGB-p->opacity);
+              *q++=ScaleQuantumToChar((Quantum) (QuantumRange-p->opacity));
             *q++=ScaleQuantumToChar(p->red);
             *q++=ScaleQuantumToChar(p->green);
             *q++=ScaleQuantumToChar(p->blue);
@@ -827,7 +830,7 @@ static MagickBooleanType WriteSUNImage(const ImageInfo *image_info,Image *image)
             Convert PseudoClass image to a SUN monochrome image.
           */
           (void) SetImageType(image,BilevelType);
-          polarity=PixelIntensityToQuantum(&image->colormap[0]) < (MaxRGB/2);
+          polarity=PixelIntensityToQuantum(&image->colormap[0]) < (QuantumRange/2);
           if (image->colors == 2)
             polarity=PixelIntensityToQuantum(&image->colormap[0]) <
               PixelIntensityToQuantum(&image->colormap[1]);
@@ -910,7 +913,7 @@ static MagickBooleanType WriteSUNImage(const ImageInfo *image_info,Image *image)
                 }
           }
         }
-    if (image->next == (Image *) NULL)
+    if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
