@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -53,6 +53,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 
@@ -283,7 +284,7 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               /*
                 Gray scale.
               */
-              pixel.red=ScaleCharToQuantum(ReadBlobByte(image));
+              pixel.red=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
               pixel.green=pixel.red;
               pixel.blue=pixel.red;
               break;
@@ -296,10 +297,10 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               */
               j=(unsigned char) ReadBlobByte(image);
               k=(unsigned char) ReadBlobByte(image);
-              pixel.red=ScaleXToQuantum((int) (k & 0x7c) >> 2,31);
-              pixel.green=ScaleXToQuantum(((int) (k & 0x03) << 3)+
+              pixel.red=ScaleAnyToQuantum((int) (k & 0x7c) >> 2,31);
+              pixel.green=ScaleAnyToQuantum(((int) (k & 0x03) << 3)+
                 ((int) (j & 0xe0) >> 5),31);
-              pixel.blue=ScaleXToQuantum((int) (j & 0x1f),31);
+              pixel.blue=ScaleAnyToQuantum((int) (j & 0x1f),31);
               break;
             }
             case 24:
@@ -308,9 +309,12 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               /*
                 8 bits each of blue, green and red.
               */
-              pixel.blue=ScaleCharToQuantum(ReadBlobByte(image));
-              pixel.green=ScaleCharToQuantum(ReadBlobByte(image));
-              pixel.red=ScaleCharToQuantum(ReadBlobByte(image));
+              pixel.blue=ScaleCharToQuantum((unsigned char)
+                ReadBlobByte(image));
+              pixel.green=ScaleCharToQuantum((unsigned char)
+                ReadBlobByte(image));
+              pixel.red=ScaleCharToQuantum((unsigned char)
+                ReadBlobByte(image));
               break;
             }
           }
@@ -376,9 +380,9 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 pixel=image->colormap[ConstrainColormapIndex(image,index)];
               else
                 {
-                  pixel.red=ScaleCharToQuantum(index);
-                  pixel.green=ScaleCharToQuantum(index);
-                  pixel.blue=ScaleCharToQuantum(index);
+                  pixel.red=ScaleCharToQuantum((unsigned char) index);
+                  pixel.green=ScaleCharToQuantum((unsigned char) index);
+                  pixel.blue=ScaleCharToQuantum((unsigned char) index);
                 }
               break;
             }
@@ -390,10 +394,10 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               */
               j=(unsigned char) ReadBlobByte(image);
               k=(unsigned char) ReadBlobByte(image);
-              pixel.red=ScaleXToQuantum((int) (k & 0x7c) >> 2,31);
-              pixel.red=ScaleXToQuantum(((int) (k & 0x03) << 3)+
+              pixel.red=ScaleAnyToQuantum((int) (k & 0x7c) >> 2,31);
+              pixel.green=ScaleAnyToQuantum(((int) (k & 0x03) << 3)+
                 ((int) (j & 0xe0) >> 5),31);
-              pixel.blue=ScaleXToQuantum((int) (j & 0x1f),31);
+              pixel.blue=ScaleAnyToQuantum((int) (j & 0x1f),31);
               if (image->storage_class == PseudoClass)
                 index=ConstrainColormapIndex(image,((unsigned long) k << 8)+j);
               break;
@@ -404,11 +408,14 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               /*
                 8 bits each of blue green and red.
               */
-              pixel.blue=ScaleCharToQuantum(ReadBlobByte(image));
-              pixel.green=ScaleCharToQuantum(ReadBlobByte(image));
-              pixel.red=ScaleCharToQuantum(ReadBlobByte(image));
+              pixel.blue=ScaleCharToQuantum((unsigned char)
+                ReadBlobByte(image));
+              pixel.green=ScaleCharToQuantum((unsigned char)
+                ReadBlobByte(image));
+              pixel.red=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
               if (tga_info.bits_per_pixel == 32)
-                pixel.opacity=QuantumRange-ScaleCharToQuantum(ReadBlobByte(image));
+                pixel.opacity=(Quantum) (QuantumRange-ScaleCharToQuantum(
+                  (unsigned char) ReadBlobByte(image)));
               break;
             }
           }
@@ -519,26 +526,26 @@ ModuleExport void RegisterTGAImage(void)
   entry=SetMagickInfo("ICB");
   entry->decoder=(DecoderHandler *) ReadTGAImage;
   entry->encoder=(EncoderHandler *) WriteTGAImage;
-  entry->description=AcquireString("Truevision Targa image");
-  entry->module=AcquireString("TGA");
+  entry->description=ConstantString("Truevision Targa image");
+  entry->module=ConstantString("TGA");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("TGA");
   entry->decoder=(DecoderHandler *) ReadTGAImage;
   entry->encoder=(EncoderHandler *) WriteTGAImage;
-  entry->description=AcquireString("Truevision Targa image");
-  entry->module=AcquireString("TGA");
+  entry->description=ConstantString("Truevision Targa image");
+  entry->module=ConstantString("TGA");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("VDA");
   entry->decoder=(DecoderHandler *) ReadTGAImage;
   entry->encoder=(EncoderHandler *) WriteTGAImage;
-  entry->description=AcquireString("Truevision Targa image");
-  entry->module=AcquireString("TGA");
+  entry->description=ConstantString("Truevision Targa image");
+  entry->module=ConstantString("TGA");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("VST");
   entry->decoder=(DecoderHandler *) ReadTGAImage;
   entry->encoder=(EncoderHandler *) WriteTGAImage;
-  entry->description=AcquireString("Truevision Targa image");
-  entry->module=AcquireString("TGA");
+  entry->description=ConstantString("Truevision Targa image");
+  entry->module=ConstantString("TGA");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -683,7 +690,10 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
     /*
       Initialize TGA raster file header.
     */
-    (void) SetImageColorspace(image,RGBColorspace);
+    if ((image->columns > 65535L) || (image->rows > 65535L))
+      ThrowWriterException(ImageError,"WidthOrHeightExceedsLimit");
+    if (image_info->colorspace == UndefinedColorspace)
+      (void) SetImageColorspace(image,RGBColorspace);
     targa_info.id_length=0;
     attribute=GetImageAttribute(image,"Comment");
     if (attribute != (const ImageAttribute *) NULL)
