@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -47,6 +47,7 @@
 #include "magick/color-private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
+#include "magick/enhance.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
 #include "magick/geometry.h"
@@ -57,10 +58,12 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/option.h"
 #include "magick/pixel.h"
 #include "magick/profile.h"
 #include "magick/resize.h"
 #include "magick/splay-tree.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/statistic.h"
 #include "magick/string_.h"
@@ -159,12 +162,12 @@ static MagickBooleanType IsTIFF(const unsigned char *magick,const size_t length)
 */
 
 static MagickBooleanType ReadProfile(Image *image,const char *name,
-  unsigned char *datum,size_t length)
+  unsigned char *datum,long length)
 {
   MagickBooleanType
     status;
 
-  register size_t
+  register long
     i;
 
   StringInfo
@@ -298,7 +301,7 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
 {
   typedef enum
   {
-    ReadColormapMethod,
+    ReadSingleSampleMethod,
     ReadRGBAMethod,
     ReadStripMethod,
     ReadTileMethod,
@@ -331,11 +334,9 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
     quantum_type;
 
   register long
+    i,
     x;
 
-  register long
-    i;
-
   register PixelPacket
     *q;
 
@@ -413,7 +414,7 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
         Skip to next image.
       */
       image->scene++;
-      status=(MagickBooleanType) TIFFReadDirectory(tiff);
+      status=TIFFReadDirectory(tiff) != 0 ? MagickTrue : MagickFalse;
       if (status == MagickFalse)
         {
           TIFFClose(tiff);
@@ -497,8 +498,6 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
     image->endian=MSBEndian;
     if ((int) (*(char *) &lsb_first) != 0)
       image->endian=LSBEndian;
-    if (photometric == PHOTOMETRIC_CIELAB)
-      image->colorspace=LABColorspace;
     if (photometric == PHOTOMETRIC_SEPARATED)
       image->colorspace=CMYKColorspace;
     (void) TIFFGetFieldDefaulted(tiff,TIFFTAG_SAMPLESPERPIXEL,
@@ -532,24 +531,26 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
     length=0;
 #if defined(TIFFTAG_ICCPROFILE)
     if (TIFFGetField(tiff,TIFFTAG_ICCPROFILE,&length,&profile) == 1)
-      (void) ReadProfile(image,"icc",profile,length);
+      (void) ReadProfile(image,"icc",profile,(long) length);
 #endif
 #if defined(TIFFTAG_PHOTOSHOP)
     if (TIFFGetField(tiff,TIFFTAG_PHOTOSHOP,&length,&profile) == 1)
-      (void) ReadProfile(image,"iptc",profile,length);
+      (void) ReadProfile(image,"iptc",profile,(long) length);
 #endif
 #if defined(TIFFTAG_RICHTIFFIPTC)
     if (TIFFGetField(tiff,TIFFTAG_RICHTIFFIPTC,&length,&profile) == 1)
       {
         if (TIFFIsByteSwapped(tiff) != 0)
           TIFFSwabArrayOfLong((uint32 *) profile,(unsigned long) length);
-        (void) ReadProfile(image,"iptc",profile,4*length);
+        (void) ReadProfile(image,"iptc",profile,4L*length);
       }
 #endif
 #if defined(TIFFTAG_XMLPACKET)
     if (TIFFGetField(tiff,TIFFTAG_XMLPACKET,&length,&profile) == 1)
-      (void) ReadProfile(image,"xmp",profile,length);
+      (void) ReadProfile(image,"xmp",profile,(long) length);
 #endif
+    if (TIFFGetField(tiff,37724,&length,&profile) == 1)
+      (void) ReadProfile(image,"tiff:37724",profile,(long) length);
     /*
       Allocate memory for the image and pixel buffer.
     */
@@ -635,31 +636,24 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     method=ReadGenericMethod;
+    if (TIFFGetField(tiff,TIFFTAG_ROWSPERSTRIP,&rows_per_strip) != 0)
+      method=ReadStripMethod;
+    if ((samples_per_pixel >= 2) && (interlace == PLANARCONFIG_CONTIG))
+      method=ReadRGBAMethod;
+    if (TIFFIsTiled(tiff) != MagickFalse)
+      method=ReadTileMethod;
+    if (photometric != PHOTOMETRIC_RGB)
+      method=ReadGenericMethod;
+    if (image->colorspace == CMYKColorspace)
+      method=ReadRGBAMethod;
     if (image->storage_class == PseudoClass)
-      method=ReadColormapMethod;
-    else
-      {
-        if (TIFFGetField(tiff,TIFFTAG_ROWSPERSTRIP,&rows_per_strip) != 0)
-          method=ReadStripMethod;
-        if ((samples_per_pixel >= 2) && (interlace == PLANARCONFIG_CONTIG))
-          method=ReadRGBAMethod;
-        if (TIFFIsTiled(tiff) != MagickFalse)
-          method=ReadTileMethod;
-        if (photometric != PHOTOMETRIC_RGB)
-          method=ReadGenericMethod;
-        if (image->colorspace == CMYKColorspace)
-          method=ReadRGBAMethod;
-        if (image->colorspace == LABColorspace)
-          method=ReadRGBAMethod;
-      }
+      method=ReadSingleSampleMethod;
     switch (method)
     {
-      case ReadColormapMethod:
+      case ReadSingleSampleMethod:
       {
-        Quantum
-          quantum;
-
         size_t
+          length,
           packet_size;
 
         /*
@@ -668,91 +662,99 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
         packet_size=(size_t) bits_per_sample/8;
         if (image->matte != MagickFalse)
           packet_size*=2;
-        scanline=(unsigned char *) AcquireMagickMemory(Max((size_t)
-          TIFFScanlineSize(tiff),packet_size*samples_per_pixel*width));
+        length=Max((size_t) TIFFScanlineSize(tiff),packet_size*
+          samples_per_pixel*width);
+        scanline=(unsigned char *) AcquireMagickMemory(length*
+          sizeof(*scanline));
         if (scanline == (unsigned char *) NULL)
           {
             TIFFClose(tiff);
             ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
           }
-        /*
-          Create colormap.
-        */
-        switch (photometric)
-        {
-          case PHOTOMETRIC_MINISBLACK:
+        (void) ResetMagickMemory(scanline,0,length);
+        if (image->storage_class == PseudoClass)
           {
-            for (i=0; i < (long) image->colors; i++)
+            /*
+              Initialize colormap.
+            */
+            switch (photometric)
             {
-              quantum=ScaleXToQuantum(i,Max(image->colors-1,1));
-              image->colormap[i].red=quantum;
-              image->colormap[i].green=quantum;
-              image->colormap[i].blue=quantum;
-              image->colormap[i].opacity=OpaqueOpacity;
-            }
-            break;
-          }
-          case PHOTOMETRIC_MINISWHITE:
-          default:
-          {
-            for (i=0; i < (long) image->colors; i++)
-            {
-              quantum=QuantumRange-ScaleXToQuantum(i,Max(image->colors-1,1));
-              image->colormap[i].red=quantum;
-              image->colormap[i].green=quantum;
-              image->colormap[i].blue=quantum;
-              image->colormap[i].opacity=OpaqueOpacity;
-            }
-            break;
-          }
-          case PHOTOMETRIC_PALETTE:
-          {
-            long
-              range;
-
-            uint16
-              *blue_colormap,
-              *green_colormap,
-              *red_colormap;
-
-            (void) TIFFGetField(tiff,TIFFTAG_COLORMAP,&red_colormap,
-              &green_colormap,&blue_colormap);
-            range=256L;  /* might be old style 8-bit colormap */
-            for (i=0; i < (long) image->colors; i++)
-              if ((red_colormap[i] >= 256) || (green_colormap[i] >= 256) ||
-                  (blue_colormap[i] >= 256))
+              case PHOTOMETRIC_MINISBLACK:
+              {
+                for (i=0; i < image->colors; i++)
                 {
-                  range=65535L;
-                  break;
+                  image->colormap[i].red=ScaleAnyToQuantum(i,image->colors-1);
+                  image->colormap[i].green=image->colormap[i].red;
+                  image->colormap[i].blue=image->colormap[i].red;
+                  image->colormap[i].opacity=OpaqueOpacity;
                 }
-            for (i=0; i < (long) image->colors; i++)
-            {
-              image->colormap[i].red=ScaleXToQuantum(red_colormap[i],range);
-              image->colormap[i].green=ScaleXToQuantum(green_colormap[i],range);
-              image->colormap[i].blue=ScaleXToQuantum(blue_colormap[i],range);
+                break;
+              }
+              case PHOTOMETRIC_MINISWHITE:
+              default:
+              {
+                for (i=0; i < image->colors; i++)
+                {
+                  image->colormap[i].red=
+                    ScaleAnyToQuantum(image->colors-i-1,image->colors-1);
+                  image->colormap[i].green=image->colormap[i].red;
+                  image->colormap[i].blue=image->colormap[i].red;
+                  image->colormap[i].opacity=OpaqueOpacity;
+                }
+                break;
+              }
+              case PHOTOMETRIC_PALETTE:
+              {
+                long
+                  range;
+
+                uint16
+                  *blue_colormap,
+                  *green_colormap,
+                  *red_colormap;
+
+                (void) TIFFGetField(tiff,TIFFTAG_COLORMAP,&red_colormap,
+                  &green_colormap,&blue_colormap);
+                range=256L;  /* might be old style 8-bit colormap */
+                for (i=0; i < (long) image->colors; i++)
+                  if ((red_colormap[i] >= 256) || (green_colormap[i] >= 256) ||
+                      (blue_colormap[i] >= 256))
+                    {
+                      range=65535L;
+                      break;
+                    }
+                for (i=0; i < (long) image->colors; i++)
+                {
+                  image->colormap[i].red=ScaleAnyToQuantum(red_colormap[i],
+                    range);
+                  image->colormap[i].green=ScaleAnyToQuantum(green_colormap[i],
+                    range);
+                  image->colormap[i].blue=ScaleAnyToQuantum(blue_colormap[i],
+                    range);
+                }
+                break;
+              }
             }
-            break;
           }
-        }
-        quantum_type=IndexQuark;
+        quantum_type=IndexQuantum;
         pad=samples_per_pixel-1;
         if (image->matte != MagickFalse)
           {
-            if (photometric != PHOTOMETRIC_PALETTE)
+            if (image->storage_class != PseudoClass)
               {
-                quantum_type=GrayAlphaQuark;
+                quantum_type=GrayAlphaQuantum;
                 pad=samples_per_pixel-2;
               }
             else
               {
-                quantum_type=IndexAlphaQuark;
+                quantum_type=IndexAlphaQuantum;
                 pad=samples_per_pixel-2;
               }
           }
         else
-          if (photometric != PHOTOMETRIC_PALETTE)
+          if (image->storage_class != PseudoClass)
             {
-              quantum_type=GrayQuark;
+              quantum_type=GrayQuantum;
               pad=samples_per_pixel-1;
             }
         for (y=0; y < (long) image->rows; y++)
@@ -826,25 +828,6 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
           status=TIFFReadPixels(tiff,bits_per_sample,0,y,(char *) scanline);
           if (status == -1)
             break;
-          if (bits_per_sample == 4)
-            {
-              register unsigned char
-                *p,
-                *r;
-
-              uint32
-                width;
-
-              width=(uint32) TIFFScanlineSize(tiff);
-              p=scanline+width-1;
-              r=scanline+(width << 1)-1;
-              for (x=0; x < (long) width; x++)
-              {
-                *r--=((*p) & 0xf) << 4;
-                *r--=((*p >> 4) & 0xf) << 4;
-                p--;
-              }
-            }
           q=SetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
@@ -906,11 +889,12 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
           p=pixels+image->columns*i;
           for (x=0; x < (long) image->columns; x++)
           {
-            q->red=ScaleCharToQuantum(TIFFGetR(*p));
-            q->green=ScaleCharToQuantum(TIFFGetG(*p));
-            q->blue=ScaleCharToQuantum(TIFFGetB(*p));
+            q->red=ScaleCharToQuantum((unsigned char) TIFFGetR(*p));
+            q->green=ScaleCharToQuantum((unsigned char) TIFFGetG(*p));
+            q->blue=ScaleCharToQuantum((unsigned char) TIFFGetB(*p));
             if (image->matte != MagickFalse)
-              q->opacity=(Quantum) ScaleCharToQuantum(TIFFGetA(*p));
+              q->opacity=(Quantum)
+                ScaleCharToQuantum((unsigned char) TIFFGetA(*p));
             p++;
             q++;
           }
@@ -946,7 +930,7 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
           Convert tiled TIFF image to DirectClass MIFF image.
         */
         if ((TIFFGetField(tiff,TIFFTAG_TILEWIDTH,&columns) == 0) ||
-           (TIFFGetField(tiff,TIFFTAG_TILELENGTH,&rows) == 0))
+            (TIFFGetField(tiff,TIFFTAG_TILELENGTH,&rows) == 0))
           {
             TIFFClose(tiff);
             ThrowReaderException(CoderError,"ImageIsNotTiled");
@@ -994,19 +978,19 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
               if (image->matte != MagickFalse)
                 for (column=columns_remaining; column > 0; column--)
                 {
-                  q->red=ScaleCharToQuantum(TIFFGetR(*p));
-                  q->green=ScaleCharToQuantum(TIFFGetG(*p));
-                  q->blue=ScaleCharToQuantum(TIFFGetB(*p));
-                  q->opacity=ScaleCharToQuantum(TIFFGetA(*p));
+                  q->red=ScaleCharToQuantum((unsigned char) TIFFGetR(*p));
+                  q->green=ScaleCharToQuantum((unsigned char) TIFFGetG(*p));
+                  q->blue=ScaleCharToQuantum((unsigned char) TIFFGetB(*p));
+                  q->opacity=ScaleCharToQuantum((unsigned char) TIFFGetA(*p));
                   q++;
                   p++;
                 }
               else
                 for (column=columns_remaining; column > 0; column--)
                 {
-                  q->red=ScaleCharToQuantum(TIFFGetR(*p));
-                  q->green=ScaleCharToQuantum(TIFFGetG(*p));
-                  q->blue=ScaleCharToQuantum(TIFFGetB(*p));
+                  q->red=ScaleCharToQuantum((unsigned char) TIFFGetR(*p));
+                  q->green=ScaleCharToQuantum((unsigned char) TIFFGetG(*p));
+                  q->blue=ScaleCharToQuantum((unsigned char) TIFFGetB(*p));
                   q++;
                   p++;
                 }
@@ -1064,13 +1048,14 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
           if (q == (PixelPacket *) NULL)
             break;
           q+=image->columns-1;
-          for (x=(long) image->columns-1; x >= 0; x--)
+          for (x=0; x < (long) image->columns; x++)
           {
-            q->red=ScaleCharToQuantum(TIFFGetR(*p));
-            q->green=ScaleCharToQuantum(TIFFGetG(*p));
-            q->blue=ScaleCharToQuantum(TIFFGetB(*p));
+            q->red=ScaleCharToQuantum((unsigned char) TIFFGetR(*p));
+            q->green=ScaleCharToQuantum((unsigned char) TIFFGetG(*p));
+            q->blue=ScaleCharToQuantum((unsigned char) TIFFGetB(*p));
             if (image->matte != MagickFalse)
-              q->opacity=(Quantum) ScaleCharToQuantum(TIFFGetA(*p));
+              q->opacity=(Quantum)
+                ScaleCharToQuantum((unsigned char) TIFFGetA(*p));
             p--;
             q--;
           }
@@ -1096,7 +1081,7 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
           gamma;
 
         /*
-          Unpremultiply alpha.
+          Disassociate alpha.
         */
         for (y=0; y < (long) image->rows; y++)
         {
@@ -1125,7 +1110,7 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
     if (image_info->number_scenes != 0)
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
-    status=(MagickBooleanType) TIFFReadDirectory(tiff);
+    status=TIFFReadDirectory(tiff) != 0 ? MagickTrue : MagickFalse;
     if (status == MagickTrue)
       {
         /*
@@ -1189,6 +1174,30 @@ ModuleExport void RegisterTIFFImage(void)
 #if defined(TIFF_VERSION)
   (void) FormatMagickString(version,MaxTextExtent,"%d",TIFF_VERSION);
 #endif
+#if defined(HasTIFF)
+  {
+    const char
+      *p;
+
+    register long int
+      i;
+
+    p=TIFFGetVersion();
+    for (i=0; (i < MaxTextExtent-1) && (*p != 0) && (*p != '\n'); i++)
+      version[i]=(*p++);
+    version[i]='\0';
+  }
+#endif
+  entry=SetMagickInfo("DNG");
+#if defined(HasTIFF)
+  entry->decoder=(DecoderHandler *) ReadTIFFImage;
+#endif
+  entry->adjoin=MagickFalse;
+  entry->endian_support=MagickTrue;
+  entry->seekable_stream=MagickTrue;
+  entry->description=ConstantString("Digital Negative");
+  entry->module=ConstantString("DNG");
+  (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PTIF");
 #if defined(HasTIFF)
   entry->decoder=(DecoderHandler *) ReadTIFFImage;
@@ -1197,20 +1206,21 @@ ModuleExport void RegisterTIFFImage(void)
   entry->adjoin=MagickFalse;
   entry->endian_support=MagickTrue;
   entry->seekable_stream=MagickTrue;
-  entry->description=AcquireString("Pyramid encoded TIFF");
-  entry->module=AcquireString("TIFF");
+  entry->description=ConstantString("Pyramid encoded TIFF");
+  entry->module=ConstantString("TIFF");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("TIF");
 #if defined(HasTIFF)
   entry->decoder=(DecoderHandler *) ReadTIFFImage;
   entry->encoder=(EncoderHandler *) WriteTIFFImage;
 #endif
-  entry->description=AcquireString(TIFFDescription);
+  entry->description=ConstantString(TIFFDescription);
   if (*version != '\0')
-    entry->version=AcquireString(version);
+    entry->version=ConstantString(version);
   entry->endian_support=MagickTrue;
   entry->seekable_stream=MagickTrue;
-  entry->module=AcquireString("TIFF");
+  entry->stealth=MagickTrue;
+  entry->module=ConstantString("TIFF");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("TIFF");
 #if defined(HasTIFF)
@@ -1218,12 +1228,12 @@ ModuleExport void RegisterTIFFImage(void)
   entry->encoder=(EncoderHandler *) WriteTIFFImage;
 #endif
   entry->magick=(MagickHandler *) IsTIFF;
-  entry->description=AcquireString(TIFFDescription);
+  entry->description=ConstantString(TIFFDescription);
   if (*version != '\0')
-    entry->version=AcquireString(version);
+    entry->version=ConstantString(version);
   entry->endian_support=MagickTrue;
   entry->seekable_stream=MagickTrue;
-  entry->module=AcquireString("TIFF");
+  entry->module=ConstantString("TIFF");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -1248,6 +1258,7 @@ ModuleExport void RegisterTIFFImage(void)
 */
 ModuleExport void UnregisterTIFFImage(void)
 {
+  (void) UnregisterMagickInfo("DNG");
   (void) UnregisterMagickInfo("PTIF");
   (void) UnregisterMagickInfo("TIF");
   (void) UnregisterMagickInfo("TIFF");
@@ -1445,6 +1456,9 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
 #define TIFFDefaultStripSize(tiff,request)  ((8*1024)/TIFFScanlineSize(tiff))
 #endif
 
+  const char
+    *option;
+
   const ImageAttribute
     *attribute;
 
@@ -1478,7 +1492,7 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
 
   unsigned long
     lsb_first,
-    strip_size;
+    rows_per_strip;
 
   /*
     Open TIFF file.
@@ -1520,55 +1534,49 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
     compress_tag=COMPRESSION_NONE;
     switch (image->compression)
     {
-#if defined(CCITT_SUPPORT)
       case FaxCompression:
       {
-        if (IsMonochromeImage(image,&image->exception) != MagickFalse)
+        if ((IsMonochromeImage(image,&image->exception) != MagickFalse) &&
+            (image->matte == MagickFalse))
           compress_tag=COMPRESSION_CCITTFAX3;
         break;
       }
       case Group4Compression:
       {
-        if (IsMonochromeImage(image,&image->exception) != MagickFalse)
+        if ((IsMonochromeImage(image,&image->exception) != MagickFalse) &&
+            (image->matte == MagickFalse))
           compress_tag=COMPRESSION_CCITTFAX4;
         break;
       }
-#endif
-#if defined(YCBCR_SUPPORT)
       case JPEGCompression:
       {
         compress_tag=COMPRESSION_JPEG;
         (void) SetImageDepth(image,8);
         break;
       }
-#endif
-#if defined(LZW_SUPPORT)
       case LZWCompression:
       {
         if (image_info->compression == LZWCompression)
           compress_tag=COMPRESSION_LZW;  /* LZW compression must be explicit */
         break;
       }
-#endif
       case RLECompression:
       {
         compress_tag=COMPRESSION_PACKBITS;
         break;
       }
-#if defined(ZIP_SUPPORT)
       case ZipCompression:
       {
         compress_tag=COMPRESSION_ADOBE_DEFLATE;
         break;
       }
-#endif
       default:
       {
         compress_tag=COMPRESSION_NONE;
         break;
       }
     }
-    (void) TIFFSetField(tiff,TIFFTAG_BITSPERSAMPLE,image->depth > 8 ? 16 : 8);
+    (void) TIFFSetField(tiff,TIFFTAG_BITSPERSAMPLE,image->depth);
     if (((image_info->colorspace == UndefinedColorspace) &&
          (image->colorspace == CMYKColorspace)) ||
          (image_info->colorspace == CMYKColorspace))
@@ -1586,7 +1594,8 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
           photometric=PHOTOMETRIC_CIELAB;
         else
           {
-            (void) SetImageColorspace(image,RGBColorspace);
+            if (image_info->colorspace == UndefinedColorspace)
+              (void) SetImageColorspace(image,RGBColorspace);
             photometric=PHOTOMETRIC_RGB;
           }
         (void) TIFFSetField(tiff,TIFFTAG_SAMPLESPERPIXEL,3);
@@ -1600,10 +1609,12 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
                 photometric=PHOTOMETRIC_MINISBLACK;
                 if ((image_info->depth == 0) &&
                     (IsMonochromeImage(image,&image->exception) != MagickFalse))
-                  image->depth=1;
-                for (i=1; i < (long) image->depth; i*=2);
-                (void) TIFFSetField(tiff,TIFFTAG_BITSPERSAMPLE,i);
-                if (i == 1)
+                  {
+                    image->depth=1;
+                    (void) TIFFSetField(tiff,TIFFTAG_BITSPERSAMPLE,
+                      image->depth);
+                  }
+                if (image->depth == 1)
                   photometric=PHOTOMETRIC_MINISWHITE;
               }
             else
@@ -1614,11 +1625,7 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
                   */
                   (void) TIFFSetField(tiff,TIFFTAG_SAMPLESPERPIXEL,1);
                   photometric=PHOTOMETRIC_PALETTE;
-                  bits_per_sample=1;
-                  while ((1UL << bits_per_sample) < image->colors)
-                    bits_per_sample*=2;
-                  (void) TIFFSetField(tiff,TIFFTAG_BITSPERSAMPLE,
-                    bits_per_sample);
+                  (void) TIFFSetField(tiff,TIFFTAG_BITSPERSAMPLE,image->depth);
                 }
           }
       }
@@ -1652,7 +1659,8 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
       }
       case Group4Compression:
       {
-        if (IsMonochromeImage(image,&image->exception) != MagickFalse)
+        if ((IsMonochromeImage(image,&image->exception) != MagickFalse) &&
+            (image->matte == MagickFalse))
           compress_tag=COMPRESSION_CCITTFAX4;
         break;
       }
@@ -1663,6 +1671,10 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
         compress_tag=COMPRESSION_JPEG;
         (void) TIFFSetField(tiff,TIFFTAG_JPEGQUALITY,
           image->quality == UndefinedCompressionQuality ? 75 : image->quality);
+        if (image->colorspace == RGBColorspace)
+          (void) TIFFSetField(tiff,TIFFTAG_JPEGCOLORMODE,JPEGCOLORMODE_RGB);
+        if (bits_per_sample == 12)
+          (void) TIFFSetField(tiff,TIFFTAG_JPEGTABLESMODE,JPEGTABLESMODE_QUANT);
         break;
       }
 #endif
@@ -1704,7 +1716,7 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
         break;
       }
     }
-    lsb_first=1;                                                                
+    lsb_first=1;
     image->endian=MSBEndian;
     if ((int) (*(char *) &lsb_first) != 0)
       image->endian=LSBEndian;
@@ -1715,13 +1727,18 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
       if ((image_info->interlace == PlaneInterlace) ||
           (image_info->interlace == PartitionInterlace))
         (void) TIFFSetField(tiff,TIFFTAG_PLANARCONFIG,PLANARCONFIG_SEPARATE);
-    strip_size=(unsigned long) Max(TIFFDefaultStripSize(tiff,-1),1);
+    rows_per_strip=1;
+    if (TIFFScanlineSize(tiff) != 0)
+      rows_per_strip=(unsigned long) Max(TIFFDefaultStripSize(tiff,-1),1);
+    option=GetImageOption(image_info,"tiff:rows-per-strip");
+    if (option != (const char *) NULL)
+      rows_per_strip=strtol(option,(char **) NULL,10);
     switch (compress_tag)
     {
       case COMPRESSION_JPEG:
       {
         (void) TIFFSetField(tiff,TIFFTAG_ROWSPERSTRIP,
-          strip_size+(16-(strip_size % 16)));
+          rows_per_strip+(16-(rows_per_strip % 16)));
         break;
       }
       case COMPRESSION_ADOBE_DEFLATE:
@@ -1733,7 +1750,8 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
              (photometric == PHOTOMETRIC_MINISBLACK)) &&
             ((bits_per_sample == 8) || (bits_per_sample == 16)))
           (void) TIFFSetField(tiff,TIFFTAG_PREDICTOR,2);
-        (void) TIFFSetField(tiff,TIFFTAG_ZIPQUALITY,9);
+        (void) TIFFSetField(tiff,TIFFTAG_ZIPQUALITY,image_info->quality ==
+          UndefinedCompressionQuality ? 7 : Min(image_info->quality/10,9));
         break;
       }
       case COMPRESSION_CCITTFAX4:
@@ -1743,7 +1761,7 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
       }
       case COMPRESSION_LZW:
       {
-        (void) TIFFSetField(tiff,TIFFTAG_ROWSPERSTRIP,strip_size);
+        (void) TIFFSetField(tiff,TIFFTAG_ROWSPERSTRIP,rows_per_strip);
         (void) TIFFGetFieldDefaulted(tiff,TIFFTAG_BITSPERSAMPLE,
           &bits_per_sample);
         if (((photometric == PHOTOMETRIC_RGB) ||
@@ -1754,7 +1772,7 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
       }
       default:
       {
-        (void) TIFFSetField(tiff,TIFFTAG_ROWSPERSTRIP,strip_size);
+        (void) TIFFSetField(tiff,TIFFTAG_ROWSPERSTRIP,rows_per_strip);
         break;
       }
     }
@@ -1841,6 +1859,9 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
               iptc_profile=DestroyStringInfo(iptc_profile);
 #endif
             }
+          if (LocaleCompare(name,"tiff:37724") == 0)
+            (void) TIFFSetField(tiff,37724,(uint32) profile->length,
+              profile->datum);
           name=GetNextImageProfile(image);
         }
       }
@@ -1848,12 +1869,11 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
       {
         (void) TIFFSetField(tiff,TIFFTAG_SUBFILETYPE,FILETYPE_PAGE);
         if (image->scene != 0)
-          (void) TIFFSetField(tiff,TIFFTAG_PAGENUMBER,(unsigned short)
-            image->scene,GetImageListLength(image));
+          (void) TIFFSetField(tiff,TIFFTAG_PAGENUMBER,(uint16) image->scene,
+            GetImageListLength(image));
       }
     if (image->orientation != UndefinedOrientation)
-      (void) TIFFSetField(tiff,TIFFTAG_ORIENTATION,(unsigned short)
-        image->orientation);
+      (void) TIFFSetField(tiff,TIFFTAG_ORIENTATION,(uint16) image->orientation);
     attribute=GetImageAttribute(image,"Artist");
     if (attribute != (const ImageAttribute *) NULL)
       (void) TIFFSetField(tiff,TIFFTAG_ARTIST,attribute->value);
@@ -2085,15 +2105,15 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
           if (image->matte != MagickFalse)
              {
                if (photometric != PHOTOMETRIC_PALETTE)
-                 (void) ImportQuantumPixels(image,GrayAlphaQuark,0,scanline);
+                 (void) ImportQuantumPixels(image,GrayAlphaQuantum,0,scanline);
                else
-                 (void) ImportQuantumPixels(image,IndexAlphaQuark,0,scanline);
+                 (void) ImportQuantumPixels(image,IndexAlphaQuantum,0,scanline);
              }
            else
              if (photometric != PHOTOMETRIC_PALETTE)
-               (void) ImportQuantumPixels(image,GrayQuark,0,scanline);
+               (void) ImportQuantumPixels(image,GrayQuantum,0,scanline);
              else
-               (void) ImportQuantumPixels(image,IndexQuark,0,scanline);
+               (void) ImportQuantumPixels(image,IndexQuantum,0,scanline);
           if (TIFFWritePixels(tiff,(char *) scanline,y,0,image) < 0)
             break;
           if (image->previous == (Image *) NULL)
@@ -2128,7 +2148,7 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
       }
     scene++;
   } while (image_info->adjoin != MagickFalse);
-  CloseBlob(image);
+  TIFFClose(tiff);
   return(MagickTrue);
 }
 #endif
