@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -56,6 +56,7 @@
 #include "magick/monitor.h"
 #include "magick/montage.h"
 #include "magick/resize.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/utility.h"
@@ -131,7 +132,8 @@ static Image *ReadVIDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
@@ -186,8 +188,8 @@ static Image *ReadVIDImage(const ImageInfo *image_info,ExceptionInfo *exception)
     AppendImageToList(&images,next_image);
     if (images->progress_monitor != (MagickProgressMonitor) NULL)
       {
-        status=images->progress_monitor(LoadImagesTag,i,number_files,
-          images->client_data);
+        status=images->progress_monitor(LoadImagesTag,i,(MagickSizeType)
+          number_files,images->client_data);
         if (status == MagickFalse)
           break;
       }
@@ -241,8 +243,8 @@ ModuleExport void RegisterVIDImage(void)
   entry=SetMagickInfo("VID");
   entry->decoder=(DecoderHandler *) ReadVIDImage;
   entry->encoder=(EncoderHandler *) WriteVIDImage;
-  entry->description=AcquireString("Visual Image Directory");
-  entry->module=AcquireString("VID");
+  entry->description=ConstantString("Visual Image Directory");
+  entry->module=ConstantString("VID");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -312,7 +314,7 @@ static MagickBooleanType WriteVIDImage(const ImageInfo *image_info,Image *image)
   /*
     Create the visual image directory.
   */
-  for (p=image; p != (Image *) NULL; p=p->next)
+  for (p=image; p != (Image *) NULL; p=GetNextImageInList(p))
     (void) SetImageAttribute(p,"Label",DefaultTileLabel);
   montage_info=CloneMontageInfo(image_info,(MontageInfo *) NULL);
   montage_image=MontageImages(image,montage_info,&image->exception);
@@ -321,6 +323,6 @@ static MagickBooleanType WriteVIDImage(const ImageInfo *image_info,Image *image)
   (void) FormatMagickString(montage_image->filename,MaxTextExtent,
     "miff:%s",image->filename);
   status=WriteImage(image_info,montage_image);
-  DestroyImageList(montage_image);
+  (void) DestroyImageList(montage_image);
   return(status);
 }
