@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -54,6 +54,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 
@@ -256,7 +257,8 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
@@ -457,19 +459,19 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
           }
           if (i < (long) image->colors)
             {
-              image->colormap[i].red=ScaleCharToQuantum((unsigned long) value);
+              image->colormap[i].red=ScaleCharToQuantum((unsigned char) value);
               image->colormap[i].green=
-                ScaleCharToQuantum((unsigned long) value);
-              image->colormap[i].blue=ScaleCharToQuantum((unsigned long) value);
+                ScaleCharToQuantum((unsigned char) value);
+              image->colormap[i].blue=ScaleCharToQuantum((unsigned char) value);
             }
           else
             if (i < (long) (2*image->colors))
               image->colormap[i % image->colors].green=
-                ScaleCharToQuantum((unsigned long) value);
+                ScaleCharToQuantum((unsigned char) value);
             else
               if (i < (long) (3*image->colors))
                 image->colormap[i % image->colors].blue=
-                  ScaleCharToQuantum((unsigned long) value);
+                  ScaleCharToQuantum((unsigned char) value);
         }
         viff_colormap=(unsigned char *) RelinquishMagickMemory(viff_colormap);
         break;
@@ -569,11 +571,11 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
         else
           if (min_value == max_value)
             {
-              scale_factor=MaxRGB/min_value;
+              scale_factor=QuantumRange/min_value;
               min_value=0;
             }
           else
-            scale_factor=MaxRGB/(max_value-min_value);
+            scale_factor=QuantumRange/(max_value-min_value);
       }
     /*
       Convert pixels to Quantum size.
@@ -592,8 +594,8 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
       if (viff_info.map_scheme == VFF_MS_NONE)
         {
           value=(value-min_value)*scale_factor;
-          if (value > MaxRGB)
-            value=MaxRGB;
+          if (value > QuantumRange)
+            value=QuantumRange;
           else
             if (value < 0)
               value=0;
@@ -614,7 +616,7 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
           Convert bitmap scanline.
         */
         SetImageType(image,BilevelType);
-        polarity=PixelIntensityToQuantum(&image->colormap[0]) < (MaxRGB/2);
+        polarity=PixelIntensityToQuantum(&image->colormap[0]) < (QuantumRange/2);
         if (image->colors >= 2)
           polarity=PixelIntensityToQuantum(&image->colormap[0]) >
             PixelIntensityToQuantum(&image->colormap[1]);
@@ -695,7 +697,7 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
                   q->green=image->colormap[q->green].green;
                   q->blue=image->colormap[q->blue].blue;
                 }
-              q->opacity=(Quantum) (image->matte ? MaxRGB-
+              q->opacity=(Quantum) (image->matte ? QuantumRange-
                 ScaleCharToQuantum(*(p+number_pixels*3)) : OpaqueOpacity);
               p++;
               q++;
@@ -735,7 +737,7 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
           Allocate next image structure.
         */
         AllocateNextImage(image_info,image);
-        if (image->next == (Image *) NULL)
+        if (GetNextImageInList(image) == (Image *) NULL)
           {
             image=DestroyImageList(image);
             return((Image *) NULL);
@@ -786,14 +788,14 @@ ModuleExport void RegisterVIFFImage(void)
   entry->decoder=(DecoderHandler *) ReadVIFFImage;
   entry->encoder=(EncoderHandler *) WriteVIFFImage;
   entry->magick=(MagickHandler *) IsVIFF;
-  entry->description=AcquireString("Khoros Visualization image");
-  entry->module=AcquireString("VIFF");
+  entry->description=ConstantString("Khoros Visualization image");
+  entry->module=ConstantString("VIFF");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("XV");
   entry->decoder=(DecoderHandler *) ReadVIFFImage;
   entry->encoder=(EncoderHandler *) WriteVIFFImage;
-  entry->description=AcquireString("Khoros Visualization image");
-  entry->module=AcquireString("VIFF");
+  entry->description=ConstantString("Khoros Visualization image");
+  entry->module=ConstantString("VIFF");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -950,7 +952,7 @@ static MagickBooleanType WriteVIFFImage(const ImageInfo *image_info,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
@@ -961,7 +963,8 @@ static MagickBooleanType WriteVIFFImage(const ImageInfo *image_info,
     /*
       Initialize VIFF image structure.
     */
-    (void) SetImageColorspace(image,RGBColorspace);
+    if (image_info->colorspace == UndefinedColorspace)
+      (void) SetImageColorspace(image,RGBColorspace);
     viff_info.identifier=(char) 0xab;
     viff_info.file_type=1;
     viff_info.release=1;
@@ -1068,7 +1071,8 @@ static MagickBooleanType WriteVIFFImage(const ImageInfo *image_info,
     /*
       Convert MIFF to VIFF raster pixels.
     */
-    viff_pixels=(unsigned char *) AcquireMagickMemory(packets);
+    viff_pixels=(unsigned char *)
+      AcquireMagickMemory((size_t) packets*sizeof(*viff_pixels));
     if (viff_pixels == (unsigned char *) NULL)
       ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     q=viff_pixels;
@@ -1089,7 +1093,8 @@ static MagickBooleanType WriteVIFFImage(const ImageInfo *image_info,
             *(q+number_pixels)=ScaleQuantumToChar(p->green);
             *(q+number_pixels*2)=ScaleQuantumToChar(p->blue);
             if (image->matte != MagickFalse)
-              *(q+number_pixels*3)=ScaleQuantumToChar(MaxRGB-p->opacity);
+              *(q+number_pixels*3)=ScaleQuantumToChar((Quantum)
+                (QuantumRange-p->opacity));
             p++;
             q++;
           }
@@ -1164,7 +1169,7 @@ static MagickBooleanType WriteVIFFImage(const ImageInfo *image_info,
               Convert PseudoClass image to a VIFF monochrome image.
             */
             SetImageType(image,BilevelType);
-            polarity=PixelIntensityToQuantum(&image->colormap[0]) < (MaxRGB/2);
+            polarity=PixelIntensityToQuantum(&image->colormap[0]) < (QuantumRange/2);
             if (image->colors == 2)
               polarity=PixelIntensityToQuantum(&image->colormap[0]) <
                 PixelIntensityToQuantum(&image->colormap[1]);
@@ -1230,9 +1235,9 @@ static MagickBooleanType WriteVIFFImage(const ImageInfo *image_info,
                   }
             }
           }
-    (void) WriteBlob(image,packets,viff_pixels);
+    (void) WriteBlob(image,(size_t) packets,viff_pixels);
     viff_pixels=(unsigned char *) RelinquishMagickMemory(viff_pixels);
-    if (image->next == (Image *) NULL)
+    if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
