@@ -17,7 +17,7 @@
 %                               January 2000                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,6 +41,7 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/color-private.h"
 #include "magick/colorspace.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
@@ -50,6 +51,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 
@@ -146,7 +148,8 @@ static Image *ReadWBMPImage(const ImageInfo *image_info,
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
@@ -213,7 +216,7 @@ static Image *ReadWBMPImage(const ImageInfo *image_info,
           break;
       }
   }
-  SyncImage(image);
+  (void) SyncImage(image);
   if (EOFBlob(image) != MagickFalse)
     ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
       image->filename);
@@ -253,8 +256,8 @@ ModuleExport void RegisterWBMPImage(void)
   entry->decoder=(DecoderHandler *) ReadWBMPImage;
   entry->encoder=(EncoderHandler *) WriteWBMPImage;
   entry->adjoin=MagickFalse;
-  entry->description=AcquireString("Wireless Bitmap (level 0) image");
-  entry->module=AcquireString("WBMP");
+  entry->description=ConstantString("Wireless Bitmap (level 0) image");
+  entry->module=ConstantString("WBMP");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -374,16 +377,17 @@ static MagickBooleanType WriteWBMPImage(const ImageInfo *image_info,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  (void) SetImageColorspace(image,RGBColorspace);
+  if (image_info->colorspace == UndefinedColorspace)
+    (void) SetImageColorspace(image,RGBColorspace);
   /*
     Convert image to a bi-level image.
   */
   SetImageType(image,BilevelType);
-  polarity=PixelIntensityToQuantum(&image->colormap[0]) < (MaxRGB/2);
+  polarity=PixelIntensityToQuantum(&image->colormap[0]) < (QuantumRange/2);
   if (image->colors == 2)
     polarity=PixelIntensityToQuantum(&image->colormap[0]) <
       PixelIntensityToQuantum(&image->colormap[1]);
