@@ -22,7 +22,7 @@
 %                            May 2001 - Oct 2001                              %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -40,13 +40,14 @@
 */
 
 /*
- * Include declarations.
- */
+  Include declarations.
+*/
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
 #include "magick/color.h"
+#include "magick/color-private.h"
 #include "magick/constitute.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
@@ -58,6 +59,7 @@
 #include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/paint.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/type.h"
@@ -486,7 +488,7 @@ static void ipa_bmp_draw(wmfAPI *API, wmfBMP_Draw_t *bmp_draw)
     height,
     width;
 
-  MagickPixelPacket
+  PixelPacket
     white;
 
   if (bmp_draw->bmp.data == 0)
@@ -527,17 +529,21 @@ static void ipa_bmp_draw(wmfAPI *API, wmfBMP_Draw_t *bmp_draw)
         InheritException(&ddata->image->exception,&exception);
     }
 
-  QueryMagickColor( "white", &white, &exception );
+  QueryColorDatabase( "white", &white, &exception );
 
   if ( ddata->image_info->texture ||
-       !(ColorMatch(&ddata->image_info->background_color, &white)) ||
+       !(ColorMatch(&ddata->image_info->background_color,&white)) ||
        ddata->image_info->background_color.opacity != OpaqueOpacity )
   {
+    MagickPixelPacket
+      white;
+
     /*
       Set image white background to transparent so that it may be
       overlaid over non-white backgrounds.
     */
-    PaintTransparentImage( image, &white, MaxRGB );
+    QueryMagickColor( "white", &white, &exception );
+    PaintTransparentImage( image, &white, QuantumRange );
   }
 
   width = AbsoluteValue(bmp_draw->pixel_width * (double) bmp_draw->crop.w);
@@ -577,8 +583,8 @@ static void ipa_bmp_read(wmfAPI * API, wmfBMP_Read_t * bmp_read) {
 
   GetExceptionInfo(&exception);
 
-  image_info = CloneImageInfo((ImageInfo *) 0);
-  strcpy(image_info->magick, "DIB");
+  image_info=CloneImageInfo((ImageInfo *) 0);
+  (void) CopyMagickString(image_info->magick,"DIB",MaxTextExtent);
   if (bmp_read->width || bmp_read->height)
     {
       char
@@ -718,8 +724,8 @@ static void ipa_device_begin(wmfAPI * API)
       GetExceptionInfo(&exception);
 
       image_info = CloneImageInfo((ImageInfo *) 0);
-      strcpy(image_info->filename, ddata->image_info->texture);
-
+      (void) CopyMagickString(image_info->filename,ddata->image_info->texture,
+        MaxTextExtent);
       if ( ddata->image_info->size )
         CloneString(&image_info->size,ddata->image_info->size);
 
@@ -733,7 +739,7 @@ static void ipa_device_begin(wmfAPI * API)
           MagickWand
             *magick_wand;
 
-          (void) strcpy(image->magick,"MIFF");
+          (void) CopyMagickString(image->magick,"MIFF",MaxTextExtent);
           DrawPushDefs(WmfDrawingWand);
           draw_pattern_push(API,ddata->pattern_id,image->columns,image->rows);
           magick_wand=NewMagickWandFromImage(image);
@@ -754,7 +760,8 @@ static void ipa_device_begin(wmfAPI * API)
         }
       else
         {
-          LogMagickEvent(CoderEvent,GetMagickModule(),"reading texture image failed!");
+          LogMagickEvent(CoderEvent,GetMagickModule(),
+            "reading texture image failed!");
           InheritException(&ddata->image->exception,&exception);
         }
     }
@@ -2254,12 +2261,13 @@ static void lite_font_map( wmfAPI* API, wmfFont* font)
                                        strstr(wmf_font_name,"Oblique"))) )
         want_italic = MagickTrue;
 
-      strcpy(target,"Times");
+      (void) CopyMagickString(target,"Times",MaxTextExtent);
       for( i=0; SubFontMap[i].name != NULL; i++ )
         {
           if (LocaleCompare(wmf_font_name, SubFontMap[i].name) == 0)
             {
-              strcpy(target,SubFontMap[i].mapping);
+              (void) CopyMagickString(target,SubFontMap[i].mapping,
+                MaxTextExtent);
               break;
             }
         }
@@ -2411,8 +2419,10 @@ static Image *ReadWMFImage(const ImageInfo * image_info, ExceptionInfo * excepti
         wmf_api_destroy(API);
       if (image->debug != MagickFalse)
         {
-          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  wmf_api_create failed");
-          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"leave ReadWMFImage()");
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+            "  wmf_api_create failed");
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+            "leave ReadWMFImage()");
         }
       ThrowReaderException(DelegateError,"UnableToInitializeWMFLibrary");
     }
@@ -2442,8 +2452,10 @@ static Image *ReadWMFImage(const ImageInfo * image_info, ExceptionInfo * excepti
       wmf_api_destroy(API);
       if (image->debug != MagickFalse)
         {
-          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  wmf_bbuf_input failed");
-          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"leave ReadWMFImage()");
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+            "  wmf_bbuf_input failed");
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+            "leave ReadWMFImage()");
         }
       ThrowFileException(exception,FileOpenError,"UnableToOpenFile",
         image->filename);
@@ -2456,15 +2468,18 @@ static Image *ReadWMFImage(const ImageInfo * image_info, ExceptionInfo * excepti
    *
    */
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  Scanning WMF to obtain bounding box");
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+      "  Scanning WMF to obtain bounding box");
   wmf_error = wmf_scan(API, 0, &bbox);
   if (wmf_error != wmf_E_None)
     {
       wmf_api_destroy(API);
       if (image->debug != MagickFalse)
         {
-          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  wmf_scan failed with wmf_error %d", wmf_error);
-          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"leave ReadWMFImage()");
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+            "  wmf_scan failed with wmf_error %d", wmf_error);
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+            "leave ReadWMFImage()");
         }
       ThrowReaderException(DelegateError,"FailedToScanFile");
     }
@@ -2500,8 +2515,10 @@ static Image *ReadWMFImage(const ImageInfo * image_info, ExceptionInfo * excepti
       wmf_api_destroy(API);
       if (image->debug != MagickFalse)
         {
-          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  wmf_size failed with wmf_error %d", wmf_error);
-          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"leave ReadWMFImage()");
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+            "  wmf_size failed with wmf_error %d", wmf_error);
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+            "leave ReadWMFImage()");
         }
       ThrowReaderException(DelegateError,"FailedToComputeOutputSize");
     }
@@ -2554,27 +2571,31 @@ static Image *ReadWMFImage(const ImageInfo * image_info, ExceptionInfo * excepti
 
   if (image->debug != MagickFalse)
     {
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  Placeable metafile:          %s",
-                     (API)->File->placeable ? "Yes" : "No");
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+         "  Placeable metafile:          %s",
+         (API)->File->placeable ? "Yes" : "No");
 
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  Size in metafile units:      %gx%g",
-                     wmf_width, wmf_height);
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  Metafile units/inch:         %g",
-                     units_per_inch);
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  Size in inches:              %gx%g",
-                     image_width_inch,image_height_inch);
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  Bounding Box:                %g,%g %g,%g",
-                     bbox.TL.x, bbox.TL.y, bbox.BR.x, bbox.BR.y);
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  Bounding width x height:     %gx%g",
-                     bounding_width, bounding_height);
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  Output resolution:           %gx%g",
-                     resolution_x, resolution_y);
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  Image size:                  %gx%g",
-                     image_width, image_height);
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  Bounding box scale factor:   %g,%g",
-                     ddata->scale_x, ddata->scale_y);
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  Translation:                 %g,%g",
-                     ddata->translate_x, ddata->translate_y);
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+        "  Size in metafile units:      %gx%g",wmf_width,wmf_height);
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+        "  Metafile units/inch:         %g",units_per_inch);
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+        "  Size in inches:              %gx%g",
+        image_width_inch,image_height_inch);
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+        "  Bounding Box:                %g,%g %g,%g",
+        bbox.TL.x, bbox.TL.y, bbox.BR.x, bbox.BR.y);
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+        "  Bounding width x height:     %gx%g",bounding_width,bounding_height);
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+        "  Output resolution:           %gx%g",resolution_x,resolution_y);
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+        "  Image size:                  %gx%g",image_width,image_height);
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+        "  Bounding box scale factor:   %g,%g",ddata->scale_x,ddata->scale_y);
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+        "  Translation:                 %g,%g",
+        ddata->translate_x, ddata->translate_y);
     }
 
 #if 0
@@ -2619,22 +2640,23 @@ static Image *ReadWMFImage(const ImageInfo * image_info, ExceptionInfo * excepti
    * Create canvas image
    *
    */
-  image->rows = (unsigned long) (image_height+0.5);
-  image->columns = (unsigned long) (image_width+0.5);
+  image->rows=(unsigned long) ceil(image_height);
+  image->columns=(unsigned long) ceil(image_width);
 
   if (image_info->ping != MagickFalse)
     {
       wmf_api_destroy(API);
       CloseBlob(image);
       if (image->debug != MagickFalse)
-        (void) LogMagickEvent(CoderEvent,GetMagickModule(),"leave ReadWMFImage()");
+        (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+          "leave ReadWMFImage()");
       return(GetFirstImageInList(image));
     }
 
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                         "  Creating canvas image with size %ldx%ld",
-                        image->rows, image->columns);
+       "  Creating canvas image with size %ldx%ld",
+       image->rows,image->columns);
 
   /*
    * Set solid background color
@@ -2670,7 +2692,8 @@ static Image *ReadWMFImage(const ImageInfo * image_info, ExceptionInfo * excepti
    */
 
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  Playing WMF to prepare vectors");
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+      "  Playing WMF to prepare vectors");
 
   wmf_error = wmf_play(API, 0, &bbox);
   if (wmf_error != wmf_E_None)
@@ -2678,8 +2701,10 @@ static Image *ReadWMFImage(const ImageInfo * image_info, ExceptionInfo * excepti
       wmf_api_destroy(API);
       if (image->debug != MagickFalse)
         {
-          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  Playing WMF failed with wmf_error %d", wmf_error);
-          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"leave ReadWMFImage()");
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+            "  Playing WMF failed with wmf_error %d", wmf_error);
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+            "leave ReadWMFImage()");
         }
       ThrowReaderException(DelegateError,"FailedToRenderFile");
     }
@@ -2690,7 +2715,8 @@ static Image *ReadWMFImage(const ImageInfo * image_info, ExceptionInfo * excepti
    */
 
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  Rendering WMF vectors");
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+      "  Rendering WMF vectors");
   DrawRender(ddata->draw_wand);
 
   /* Cleanup allocated data */
@@ -2743,19 +2769,19 @@ ModuleExport void RegisterWMFImage(void)
 #if defined(HasWMF) || defined(HasWMFlite)
   entry->decoder=ReadWMFImage;
 #endif
-  entry->description=AcquireString("Compressed Windows Meta File");
+  entry->description=ConstantString("Compressed Windows Meta File");
   entry->blob_support=MagickTrue;
   entry->seekable_stream=MagickTrue;
-  entry->module=AcquireString("WMZ");
+  entry->module=ConstantString("WMZ");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("WMF");
 #if defined(HasWMF) || defined(HasWMFlite)
   entry->decoder=ReadWMFImage;
 #endif
-  entry->description=AcquireString("Windows Meta File");
+  entry->description=ConstantString("Windows Meta File");
   entry->blob_support=MagickTrue;
   entry->seekable_stream=MagickTrue;
-  entry->module=AcquireString("WMF");
+  entry->module=ConstantString("WMF");
   (void) RegisterMagickInfo(entry);
 }
 
