@@ -16,7 +16,7 @@
 %                                 June 2000                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -53,6 +53,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/resource_.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 
@@ -588,10 +589,10 @@ static Image *ExtractPostscript(Image *image,const ImageInfo *image_info,
   image2=ReadImage(clone_info,exception);
   if (image2 == (Image *) NULL)
     goto FINISH_UNL;
-  CopyMagickString(image2->filename,image->filename,MaxTextExtent);
-  CopyMagickString(image2->magick_filename,image->magick_filename,
+  (void) CopyMagickString(image2->filename,image->filename,MaxTextExtent);
+  (void) CopyMagickString(image2->magick_filename,image->magick_filename,
     MaxTextExtent);
-  CopyMagickString(image2->magick,image->magick,MaxTextExtent);
+  (void) CopyMagickString(image2->magick,image->magick,MaxTextExtent);
   image2->depth=image->depth;
   DestroyBlob(image2);
   image2->blob=ReferenceBlob(image->blob);
@@ -725,7 +726,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
     *image,
     *p;
 
-  long
+  unsigned long
     scene;
 
   static const ColorPalette
@@ -844,7 +845,8 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
@@ -925,11 +927,11 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
                    i < (int)WPG_Palette.NumOfEntries; i++)
                 {
                   image->colormap[i].red=
-                    ScaleCharToQuantum(ReadBlobByte(image));
+                    ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
                   image->colormap[i].green=
-                    ScaleCharToQuantum(ReadBlobByte(image));
+                    ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
                   image->colormap[i].blue=
-                    ScaleCharToQuantum(ReadBlobByte(image));
+                    ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
                 }
               break;
 
@@ -1005,7 +1007,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
                     {  /*fix crippled monochrome palette*/
                       image->colormap[1].red =
                         image->colormap[1].green =
-                        image->colormap[1].blue = MaxRGB;
+                        image->colormap[1].blue = QuantumRange;
                     }
                 }
 
@@ -1019,7 +1021,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
               /* Allocate next image structure. */
               AllocateNextImage(image_info,image);
               image->depth=8;
-              if (image->next == (Image *) NULL)
+              if (GetNextImageInList(image) == (Image *) NULL)
                 goto Finish;
               image=SyncNextImageInList(image);
               image->columns=image->rows=0;
@@ -1036,7 +1038,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
       break;
 
     case 2:  /*WPG level 2*/
-      ResetMagickMemory(&StartWPG,0,sizeof(StartWPG));
+      (void) ResetMagickMemory(&StartWPG,0,sizeof(StartWPG));
       while(!EOFBlob(image)) /* object parser loop */
 
         {
@@ -1076,11 +1078,11 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
                    i < (int)WPG_Palette.NumOfEntries; i++)
                 {
                   image->colormap[i].red=
-                    ScaleCharToQuantum(ReadBlobByte(image));
+                    ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
                   image->colormap[i].green=
-                    ScaleCharToQuantum(ReadBlobByte(image));
+                    ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
                   image->colormap[i].blue=
-                    ScaleCharToQuantum(ReadBlobByte(image));
+                    ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
                   (void) ReadBlobByte(image);   /*Opacity??*/
                 }
               break;
@@ -1158,7 +1160,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
               /* Allocate next image structure. */
               AllocateNextImage(image_info,image);
               image->depth=8;
-              if (image->next == (Image *) NULL)
+              if (GetNextImageInList(image) == (Image *) NULL)
                 goto Finish;
               image=SyncNextImageInList(image);
               image->columns=image->rows=0;
@@ -1189,14 +1191,14 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
   /*
     Rewind list, removing any empty images while rewinding.
   */
-  for (p=image; p != (Image *) NULL; p=p->previous)
+  for (p=image; p != (Image *) NULL; p=GetPreviousImageInList(p))
   {
     if ((p->rows == 0) || (p->columns == 0))
       DeleteImageFromList(&p);
     image=p;
   }
   scene=0;
-  for (p=image; p != (Image *) NULL; p=p->next)
+  for (p=image; p != (Image *) NULL; p=GetNextImageInList(p))
     p->scene=scene++;
   return(GetFirstImageInList(image));
 }
@@ -1232,8 +1234,8 @@ ModuleExport void RegisterWPGImage(void)
   entry=SetMagickInfo("WPG");
   entry->decoder=(DecoderHandler *) ReadWPGImage;
   entry->magick=(MagickHandler *) IsWPG;
-  entry->description=AcquireString("Word Perfect Graphics");
-  entry->module=AcquireString("WPG");
+  entry->description=ConstantString("Word Perfect Graphics");
+  entry->module=ConstantString("WPG");
   entry->seekable_stream=MagickTrue;
   (void) RegisterMagickInfo(entry);
 }
