@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -57,6 +57,7 @@
 #include "magick/quantize.h"
 #include "magick/resize.h"
 #include "magick/resource_.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/utility.h"
@@ -224,7 +225,8 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
@@ -333,7 +335,7 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
     /*
       Parse color.
     */
-    (void) strcpy(target,"gray");
+    (void) CopyMagickString(target,"gray",MaxTextExtent);
     q=ParseColor(p+width);
     if (q != (char *) NULL)
       {
@@ -350,7 +352,7 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
         image->storage_class=DirectClass;
         image->matte=MagickTrue;
         none=j;
-        (void) strcpy(target,"black");
+        (void) CopyMagickString(target,"black",MaxTextExtent);
       }
     if (QueryColorDatabase(target,&image->colormap[j],exception) == MagickFalse)
       break;
@@ -444,24 +446,24 @@ ModuleExport void RegisterXPMImage(void)
   entry->decoder=(DecoderHandler *) ReadXPMImage;
   entry->encoder=(EncoderHandler *) WritePICONImage;
   entry->adjoin=MagickFalse;
-  entry->description=AcquireString("Personal Icon");
-  entry->module=AcquireString("XPM");
+  entry->description=ConstantString("Personal Icon");
+  entry->module=ConstantString("XPM");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PM");
   entry->decoder=(DecoderHandler *) ReadXPMImage;
   entry->encoder=(EncoderHandler *) WriteXPMImage;
   entry->adjoin=MagickFalse;
   entry->stealth=MagickTrue;
-  entry->description=AcquireString("X Windows system pixmap (color)");
-  entry->module=AcquireString("XPM");
+  entry->description=ConstantString("X Windows system pixmap (color)");
+  entry->module=ConstantString("XPM");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("XPM");
   entry->decoder=(DecoderHandler *) ReadXPMImage;
   entry->encoder=(EncoderHandler *) WriteXPMImage;
   entry->magick=(MagickHandler *) IsXPM;
   entry->adjoin=MagickFalse;
-  entry->description=AcquireString("X Windows system pixmap (color)");
-  entry->module=AcquireString("XPM");
+  entry->description=ConstantString("X Windows system pixmap (color)");
+  entry->module=ConstantString("XPM");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -609,11 +611,12 @@ static MagickBooleanType WritePICONImage(const ImageInfo *image_info,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  (void) SetImageColorspace(image,RGBColorspace);
+  if (image_info->colorspace == UndefinedColorspace)
+    (void) SetImageColorspace(image,RGBColorspace);
   SetGeometry(image,&geometry);
   (void) ParseMetaGeometry(PiconGeometry,&geometry.x,&geometry.y,
     &geometry.width,&geometry.height);
@@ -673,7 +676,7 @@ static MagickBooleanType WritePICONImage(const ImageInfo *image_info,
     {
       colors++;
       picon->colormap=(PixelPacket *) ResizeMagickMemory((void **)
-        &picon->colormap,(size_t) colors*sizeof(*picon->colormap));
+        picon->colormap,(size_t) colors*sizeof(*picon->colormap));
       if (picon->colormap == (PixelPacket *) NULL)
         ThrowWriterException(ResourceLimitError,"MemoryAllocationError");
       for (y=0; y < (long) picon->rows; y++)
@@ -721,7 +724,7 @@ static MagickBooleanType WritePICONImage(const ImageInfo *image_info,
     if (transparent != MagickFalse)
       {
         if (i == (long) (colors-1))
-          (void) strcpy(name,"grey75");
+          (void) CopyMagickString(name,"grey75",MaxTextExtent);
       }
     /*
       Write XPM color.
@@ -854,11 +857,12 @@ static MagickBooleanType WriteXPMImage(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  (void) SetImageColorspace(image,RGBColorspace);
+    if (image_info->colorspace == UndefinedColorspace)
+    (void) SetImageColorspace(image,RGBColorspace);
   transparent=MagickFalse;
   if (image->storage_class == PseudoClass)
     {
@@ -948,7 +952,7 @@ static MagickBooleanType WriteXPMImage(const ImageInfo *image_info,Image *image)
     if (transparent != MagickFalse)
       {
         if (i == (long) (colors-1))
-          (void) strcpy(name,"None");
+          (void) CopyMagickString(name,"None",MaxTextExtent);
       }
     /*
       Write XPM color.
