@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -53,6 +53,7 @@
 #include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/pixel.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/statistic.h"
 #include "magick/string_.h"
@@ -129,7 +130,8 @@ static Image *ReadYCBCRImage(const ImageInfo *image_info,
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
@@ -195,7 +197,7 @@ static Image *ReadYCBCRImage(const ImageInfo *image_info,
           count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
         for (y=0; y < (long) image->rows; y++)
         {
-          if ((y > 0) || (image->previous == (Image *) NULL))
+          if ((y > 0) || (GetPreviousImageInList(image) == (Image *) NULL))
             count=ReadBlob(image,packet_size*image->extract_info.width,
               scanline);
           q=SetImagePixels(image,0,y,image->columns,1);
@@ -232,7 +234,7 @@ static Image *ReadYCBCRImage(const ImageInfo *image_info,
           count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
         for (y=0; y < (long) image->rows; y++)
         {
-          if ((y > 0) || (image->previous == (Image *) NULL))
+          if ((y > 0) || (GetPreviousImageInList(image) == (Image *) NULL))
             count=ReadBlob(image,packet_size*image->extract_info.width,
               scanline);
           q=SetImagePixels(image,0,y,image->columns,1);
@@ -292,7 +294,7 @@ static Image *ReadYCBCRImage(const ImageInfo *image_info,
         span=image->rows*(image->matte != MagickFalse ? 5 : 4);
         for (y=0; y < (long) image->rows; y++)
         {
-          if ((y > 0) || (image->previous == (Image *) NULL))
+          if ((y > 0) || (GetPreviousImageInList(image) == (Image *) NULL))
             count=ReadBlob(image,packet_size*image->extract_info.width,
               scanline);
           q=SetImagePixels(image,0,y,image->columns,1);
@@ -459,7 +461,7 @@ static Image *ReadYCBCRImage(const ImageInfo *image_info,
           Allocate next image structure.
         */
         AllocateNextImage(image_info,image);
-        if (image->next == (Image *) NULL)
+        if (GetNextImageInList(image) == (Image *) NULL)
           {
             image=DestroyImageList(image);
             return((Image *) NULL);
@@ -511,15 +513,15 @@ ModuleExport void RegisterYCBCRImage(void)
   entry->decoder=(DecoderHandler *) ReadYCBCRImage;
   entry->encoder=(EncoderHandler *) WriteYCBCRImage;
   entry->raw=MagickTrue;
-  entry->description=AcquireString("Raw Y, Cb, and Cr samples");
-  entry->module=AcquireString("YCbCr");
+  entry->description=ConstantString("Raw Y, Cb, and Cr samples");
+  entry->module=ConstantString("YCbCr");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("YCbCrA");
   entry->decoder=(DecoderHandler *) ReadYCBCRImage;
   entry->encoder=(EncoderHandler *) WriteYCBCRImage;
   entry->raw=MagickTrue;
-  entry->description=AcquireString("Raw Y, Cb, Cr, and opacity samples");
-  entry->module=AcquireString("YCbCr");
+  entry->description=ConstantString("Raw Y, Cb, Cr, and opacity samples");
+  entry->module=ConstantString("YCbCr");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -607,7 +609,7 @@ static MagickBooleanType WriteYCBCRImage(const ImageInfo *image_info,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   depth=GetImageQuantumDepth(image,MagickTrue);
   packet_size=(size_t) (3*depth/8);
   if (LocaleCompare(image_info->magick,"YCbCrA") == 0)
@@ -630,7 +632,8 @@ static MagickBooleanType WriteYCBCRImage(const ImageInfo *image_info,
     /*
       Convert to YCbCr pixels.
     */
-    (void) SetImageColorspace(image,YCbCrColorspace);
+    if (image_info->colorspace == UndefinedColorspace)
+      (void) SetImageColorspace(image,YCbCrColorspace);
     if (LocaleCompare(image_info->magick,"YCbCrA") == 0)
       if (image->matte == MagickFalse)
         SetImageOpacity(image,OpaqueOpacity);
@@ -812,7 +815,7 @@ static MagickBooleanType WriteYCBCRImage(const ImageInfo *image_info,
         break;
       }
     }
-    if (image->next == (Image *) NULL)
+    if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
