@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -13,10 +13,10 @@
   See the License for the specific language governing permissions and
   limitations under the License.
 
-  ImageMagick private image colorspace methods.
+  MagickCore image colorspace private methods.
 */
-#ifndef _MAGICK_COLORSPACE_PRIVATE_H
-#define _MAGICK_COLORSPACE_PRIVATE_H
+#ifndef _MAGICKCORE_COLORSPACE_PRIVATE_H
+#define _MAGICKCORE_COLORSPACE_PRIVATE_H
 
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
@@ -30,10 +30,12 @@ static inline void RGBtoCMYK(MagickPixelPacket *pixel)
     magenta,
     yellow;
                                                                                 
-  cyan=MaxRGB-pixel->red;
-  magenta=MaxRGB-pixel->green;
-  yellow=MaxRGB-pixel->blue;
-  black=cyan < magenta ? Min(cyan,yellow) : Min(magenta,yellow);
+  black=(MagickRealType) Min(Min(QuantumRange-pixel->red,QuantumRange-
+    pixel->green),QuantumRange-pixel->blue);
+  cyan=(MagickRealType) (QuantumRange-pixel->red-black)/(QuantumRange-black);
+  magenta=(MagickRealType) (QuantumRange-pixel->green-black)/
+    (QuantumRange-black);
+  yellow=(MagickRealType) (QuantumRange-pixel->blue-black)/(QuantumRange-black);
   pixel->colorspace=CMYKColorspace;
   pixel->red=cyan;
   pixel->green=magenta;
