@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -40,15 +40,20 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/attribute.h"
+#include "magick/cache.h"
 #include "magick/color.h"
+#include "magick/color-private.h"
 #include "magick/colorspace.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
 #include "magick/image.h"
+#include "magick/image-private.h"
 #include "magick/gem.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/quantize.h"
+#include "magick/quantum.h"
 #include "magick/string_.h"
 #include "magick/utility.h"
 
@@ -131,13 +136,13 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
       (void) SetImageColorspace(image,image->colorspace);
       break;
     }
-	}
+  }
   image->colorspace=colorspace;
   switch (colorspace)
   {
     case CMYKColorspace:
     {
-      Quantum
+      MagickRealType
         black,
         cyan,
         magenta,
@@ -148,9 +153,13 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
       */
       if (image->storage_class == PseudoClass)
         {
-          (void) SyncImage(image);
-          image->storage_class=DirectClass;
+          if (SyncImage(image) == MagickFalse)
+            return(MagickFalse);
+          if (SetImageStorageClass(image,DirectClass) == MagickFalse)
+            return(MagickFalse);
         }
+      if (SyncImageCache(image) == MagickFalse)
+        return(MagickFalse);
       for (y=0; y < (long) image->rows; y++)
       {
         q=GetImagePixels(image,0,y,image->columns,1);
@@ -159,15 +168,18 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
         indexes=GetIndexes(image);
         for (x=0; x < (long) image->columns; x++)
         {
-          cyan=(Quantum) (QuantumRange-q->red);
-          magenta=(Quantum) (QuantumRange-q->green);
-          yellow=(Quantum) (QuantumRange-q->blue);
-          black=(Quantum)
-            (cyan < magenta ? Min(cyan,yellow) : Min(magenta,yellow));
-          q->red=cyan;
-          q->green=magenta;
-          q->blue=yellow;
-          indexes[x]=black;
+          black=(MagickRealType) Min(Min(QuantumRange-q->red,QuantumRange-
+            q->green),QuantumRange-q->blue);
+          cyan=(MagickRealType) QuantumRange*(QuantumRange-q->red-black)/
+            (QuantumRange-black);
+          magenta=(MagickRealType) QuantumRange*(QuantumRange-q->green-black)/
+            (QuantumRange-black);
+          yellow=(MagickRealType) QuantumRange*(QuantumRange-q->blue-black)/
+            (QuantumRange-black);
+          q->red=RoundToQuantum(cyan);
+          q->green=RoundToQuantum(magenta);
+          q->blue=RoundToQuantum(yellow);
+          indexes[x]=(IndexPacket) RoundToQuantum(black);
           q++;
         }
         if (SyncImagePixels(image) == MagickFalse)
@@ -220,8 +232,10 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
       */
       if (image->storage_class == PseudoClass)
         {
-          (void) SyncImage(image);
-          image->storage_class=DirectClass;
+          if (SyncImage(image) == MagickFalse)
+            return(MagickFalse);
+          if (SetImageStorageClass(image,DirectClass) == MagickFalse)
+            return(MagickFalse);
         }
       hue=0.0;
       saturation=0.0;
@@ -231,12 +245,12 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
         q=GetImagePixels(image,0,y,image->columns,1);
         if (q == (PixelPacket *) NULL)
           break;
-        for (x=(long) image->columns-1; x >= 0; x--)
+        for (x=0; x < (long) image->columns; x++)
         {
           TransformHSB(q->red,q->green,q->blue,&hue,&saturation,&brightness);
-          q->red=RoundToQuantum(QuantumRange*hue);
-          q->green=RoundToQuantum(QuantumRange*saturation);
-          q->blue=RoundToQuantum(QuantumRange*brightness);
+          q->red=RoundToQuantum((MagickRealType) QuantumRange*hue);
+          q->green=RoundToQuantum((MagickRealType) QuantumRange*saturation);
+          q->blue=RoundToQuantum((MagickRealType) QuantumRange*brightness);
           q++;
         }
         if (SyncImagePixels(image) == MagickFalse)
@@ -257,8 +271,10 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
       */
       if (image->storage_class == PseudoClass)
         {
-          (void) SyncImage(image);
-          image->storage_class=DirectClass;
+          if (SyncImage(image) == MagickFalse)
+            return(MagickFalse);
+          if (SetImageStorageClass(image,DirectClass) == MagickFalse)
+            return(MagickFalse);
         }
       hue=0.0;
       saturation=0.0;
@@ -268,12 +284,12 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
         q=GetImagePixels(image,0,y,image->columns,1);
         if (q == (PixelPacket *) NULL)
           break;
-        for (x=(long) image->columns-1; x >= 0; x--)
+        for (x=0; x < (long) image->columns; x++)
         {
           TransformHSL(q->red,q->green,q->blue,&hue,&saturation,&luminosity);
-          q->red=RoundToQuantum(QuantumRange*hue);
-          q->green=RoundToQuantum(QuantumRange*saturation);
-          q->blue=RoundToQuantum(QuantumRange*luminosity);
+          q->red=RoundToQuantum((MagickRealType) QuantumRange*hue);
+          q->green=RoundToQuantum((MagickRealType) QuantumRange*saturation);
+          q->blue=RoundToQuantum((MagickRealType) QuantumRange*luminosity);
           q++;
         }
         if (SyncImagePixels(image) == MagickFalse)
@@ -294,8 +310,10 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
       */
       if (image->storage_class == PseudoClass)
         {
-          (void) SyncImage(image);
-          image->storage_class=DirectClass;
+          if (SyncImage(image) == MagickFalse)
+            return(MagickFalse);
+          if (SetImageStorageClass(image,DirectClass) == MagickFalse)
+            return(MagickFalse);
         }
       hue=0.0;
       whiteness=0.0;
@@ -305,12 +323,12 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
         q=GetImagePixels(image,0,y,image->columns,1);
         if (q == (PixelPacket *) NULL)
           break;
-        for (x=(long) image->columns-1; x >= 0; x--)
+        for (x=0; x < (long) image->columns; x++)
         {
           TransformHWB(q->red,q->green,q->blue,&hue,&whiteness,&blackness);
-          q->red=RoundToQuantum(QuantumRange*hue);
-          q->green=RoundToQuantum(QuantumRange*whiteness);
-          q->blue=RoundToQuantum(QuantumRange*blackness);
+          q->red=RoundToQuantum((MagickRealType) QuantumRange*hue);
+          q->green=RoundToQuantum((MagickRealType) QuantumRange*whiteness);
+          q->blue=RoundToQuantum((MagickRealType) QuantumRange*blackness);
           q++;
         }
         if (SyncImagePixels(image) == MagickFalse)
@@ -319,6 +337,70 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
       image->colorspace=RGBColorspace;
       return(MagickTrue);
     }
+    case LogColorspace:
+    {
+#define ReferenceBlack  95.0
+#define ReferenceWhite  685.0
+#define DisplayGamma  (1.0/1.7)
+
+      const ImageAttribute
+        *attribute;
+
+      double
+        black,
+        density,
+        gamma,
+        reference_black,
+        reference_white;
+
+      Quantum
+        *logmap;
+
+      /*
+        Transform RGB to Log colorspace.
+      */
+      density=2.03728;
+      gamma=DisplayGamma;
+      attribute=GetImageAttribute(image,"Gamma");
+      if (attribute != (const ImageAttribute *) NULL)
+        gamma=1.0/atof(attribute->value) != 0.0 ? atof(attribute->value) : 1.0;
+      reference_black=ReferenceBlack;
+      attribute=GetImageAttribute(image,"reference-black");
+      if (attribute != (const ImageAttribute *) NULL)
+        reference_black=atof(attribute->value);
+      reference_white=ReferenceWhite;
+      attribute=GetImageAttribute(image,"reference-white");
+      if (attribute != (const ImageAttribute *) NULL)
+        reference_white=atof(attribute->value);
+      logmap=(Quantum *)
+        AcquireMagickMemory((size_t) (MaxMap+1)*sizeof(*logmap));
+      if (logmap == (Quantum *) NULL)
+        ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+          image->filename);
+      black=pow(10.0,(reference_black-reference_white)*(gamma/density)*
+        0.002/0.6);
+      for (i=0; i <= (long) MaxMap; i++)
+        logmap[i]=ScaleMapToQuantum((unsigned long) (MaxMap*(reference_white+
+          log10(black+((double) i/MaxMap)*(1.0-black))/((gamma/density)*
+          0.002/0.6))/1024.0+0.5));
+      for (y=0; y < (long) image->rows; y++)
+      {
+        q=GetImagePixels(image,0,y,image->columns,1);
+        if (q == (PixelPacket *) NULL)
+          break;
+        for (x=(long) image->columns; x > 0; x--)
+        {
+          q->red=logmap[ScaleQuantumToMap(q->red)];
+          q->green=logmap[ScaleQuantumToMap(q->green)];
+          q->blue=logmap[ScaleQuantumToMap(q->blue)];
+          q++;
+        }
+        if (SyncImagePixels(image) == MagickFalse)
+          break;
+      }
+      logmap=(Quantum *) RelinquishMagickMemory(logmap);
+      return(y == (long) image->rows ? MagickTrue : MagickFalse);
+    }
     default:
       break;
   }
@@ -335,6 +417,34 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
   (void) ResetMagickMemory(&primary_info,0,sizeof(primary_info));
   switch (colorspace)
   {
+    case OHTAColorspace:
+    {
+      /*
+        Initialize OHTA tables:
+
+          I1 = 0.33333*R+0.33334*G+0.33333*B
+          I2 = 0.50000*R+0.00000*G-0.50000*B
+          I3 =-0.25000*R+0.50000*G-0.25000*B
+
+        I and Q, normally -0.5 through 0.5, are normalized to the range 0
+        through QuantumRange.
+      */
+      primary_info.y=(double) (MaxMap+1)/2.0;
+      primary_info.z=(double) (MaxMap+1)/2.0;
+      for (i=0; i <= (long) MaxMap; i++)
+      {
+        x_map[i].x=0.33333*i;
+        x_map[i].y=0.5*i;
+        x_map[i].z=(-0.25)*i;
+        y_map[i].x=0.33334*i;
+        y_map[i].y=0.0;
+        y_map[i].z=0.5*i;
+        z_map[i].x=0.33333*i;
+        z_map[i].y=(-0.5)*i;
+        z_map[i].z=(-0.25)*i;
+      }
+      break;
+    }
     case Rec601LumaColorspace:
     case GRAYColorspace:
     {
@@ -357,6 +467,35 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
       }
       break;
     }
+    case Rec601YCbCrColorspace:
+    case YCbCrColorspace:
+    {
+      /*
+        Initialize YCbCr tables (ITU-R BT.601):
+
+          Y =  0.299000*R+0.587000*G+0.114000*B
+          Cb= -0.168736*R-0.331264*G+0.500000*B
+          Cr=  0.500000*R-0.418688*G-0.081316*B
+
+        Cb and Cr, normally -0.5 through 0.5, are normalized to the range 0
+        through QuantumRange.
+      */
+      primary_info.y=(double) (MaxMap+1)/2.0;
+      primary_info.z=(double) (MaxMap+1)/2.0;
+      for (i=0; i <= (long) MaxMap; i++)
+      {
+        x_map[i].x=0.299000*i;
+        x_map[i].y=(-0.168730)*i;
+        x_map[i].z=0.500000*i;
+        y_map[i].x=0.587000*i;
+        y_map[i].y=(-0.331264)*i;
+        y_map[i].z=(-0.418688)*i;
+        z_map[i].x=0.114000*i;
+        z_map[i].y=0.500000*i;
+        z_map[i].z=(-0.081316)*i;
+      }
+      break;
+    }
     case Rec709LumaColorspace:
     {
       /*
@@ -378,31 +517,31 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
       }
       break;
     }
-    case OHTAColorspace:
+    case Rec709YCbCrColorspace:
     {
       /*
-        Initialize OHTA tables:
+        Initialize YCbCr tables (ITU-R BT.709):
 
-          I1 = 0.33333*R+0.33334*G+0.33333*B
-          I2 = 0.50000*R+0.00000*G-0.50000*B
-          I3 =-0.25000*R+0.50000*G-0.25000*B
+          Y =  0.212600*R+0.715200*G+0.072200*B
+          Cb= -0.114572*R-0.385428*G+0.500000*B
+          Cr=  0.500000*R-0.454153*G-0.045847*B
 
-        I and Q, normally -0.5 through 0.5, are normalized to the range 0
+        Cb and Cr, normally -0.5 through 0.5, are normalized to the range 0
         through QuantumRange.
       */
-      primary_info.y=(MaxMap+1)/2.0;
-      primary_info.z=(MaxMap+1)/2.0;
+      primary_info.y=(double) (MaxMap+1)/2.0;
+      primary_info.z=(double) (MaxMap+1)/2.0;
       for (i=0; i <= (long) MaxMap; i++)
       {
-        x_map[i].x=0.33333*i;
-        x_map[i].y=0.5*i;
-        x_map[i].z=(-0.25)*i;
-        y_map[i].x=0.33334*i;
-        y_map[i].y=0.0;
-        y_map[i].z=0.5*i;
-        z_map[i].x=0.33333*i;
-        z_map[i].y=(-0.5)*i;
-        z_map[i].z=(-0.25)*i;
+        x_map[i].x=0.212600*i;
+        x_map[i].y=(-0.114572)*i;
+        x_map[i].z=0.500000*i;
+        y_map[i].x=0.715200*i;
+        y_map[i].y=(-0.385428)*i;
+        y_map[i].z=(-0.454153)*i;
+        z_map[i].x=0.072200*i;
+        z_map[i].y=0.500000*i;
+        z_map[i].z=(-0.045847)*i;
       }
       break;
     }
@@ -448,7 +587,7 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
     case XYZColorspace:
     {
       /*
-        Initialize CIE XYZ tables:
+        Initialize CIE XYZ tables (ITU-R 709 RGB):
 
           X = 0.412453*X+0.357580*Y+0.180423*Z
           Y = 0.212671*X+0.715160*Y+0.072169*Z
@@ -468,34 +607,6 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
       }
       break;
     }
-    case YCbCrColorspace:
-    {
-      /*
-        Initialize YCbCr tables:
-
-          Y =  0.299000*R+0.587000*G+0.114000*B
-          Cb= -0.168736*R-0.331264*G+0.500000*B
-          Cr=  0.500000*R-0.418688*G-0.081316*B
-
-        Cb and Cr, normally -0.5 through 0.5, are normalized to the range 0
-        through QuantumRange.
-      */
-      primary_info.y=(MaxMap+1)/2.0;
-      primary_info.z=(MaxMap+1)/2.0;
-      for (i=0; i <= (long) MaxMap; i++)
-      {
-        x_map[i].x=0.299*i;
-        x_map[i].y=(-0.16873)*i;
-        x_map[i].z=0.500000*i;
-        y_map[i].x=0.587*i;
-        y_map[i].y=(-0.331264)*i;
-        y_map[i].z=(-0.418688)*i;
-        z_map[i].x=0.114*i;
-        z_map[i].y=0.500000*i;
-        z_map[i].z=(-0.081316)*i;
-      }
-      break;
-    }
     case YCCColorspace:
     {
       /*
@@ -547,8 +658,8 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
         I and Q, normally -0.5 through 0.5, are normalized to the range 0
         through QuantumRange.
       */
-      primary_info.y=(MaxMap+1)/2.0;
-      primary_info.z=(MaxMap+1)/2.0;
+      primary_info.y=(double) (MaxMap+1)/2.0;
+      primary_info.z=(double) (MaxMap+1)/2.0;
       for (i=0; i <= (long) MaxMap; i++)
       {
         x_map[i].x=0.299*i;
@@ -566,7 +677,7 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
     case YPbPrColorspace:
     {
       /*
-        Initialize YPbPr tables:
+        Initialize YPbPr tables (ITU-R BT.601):
 
           Y =  0.299000*R+0.587000*G+0.114000*B
           Pb= -0.168736*R-0.331264*G+0.500000*B
@@ -575,8 +686,8 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
         Pb and Pr, normally -0.5 through 0.5, are normalized to the range 0
         through QuantumRange.
       */
-      primary_info.y=(MaxMap+1)/2.0;
-      primary_info.z=(MaxMap+1)/2.0;
+      primary_info.y=(double) (MaxMap+1)/2.0;
+      primary_info.z=(double) (MaxMap+1)/2.0;
       for (i=0; i <= (long) MaxMap; i++)
       {
         x_map[i].x=0.299*i;
@@ -604,8 +715,8 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
         U and V, normally -0.5 through 0.5, are normalized to the range 0
         through QuantumRange.  Note that U = 0.493*(B-Y), V = 0.877*(R-Y).
       */
-      primary_info.y=(MaxMap+1)/2.0;
-      primary_info.z=(MaxMap+1)/2.0;
+      primary_info.y=(double) (MaxMap+1)/2.0;
+      primary_info.z=(double) (MaxMap+1)/2.0;
       for (i=0; i <= (long) MaxMap; i++)
       {
         x_map[i].x=0.299*i;
@@ -787,67 +898,70 @@ MagickExport MagickBooleanType SetImageColorspace(Image *image,
 MagickExport MagickBooleanType TransformRGBImage(Image *image,
   const ColorspaceType colorspace)
 {
-#define MaxYCC  ((unsigned long) (1.3584*MaxMap+0.5))
+#define D50X  (0.9642)
+#define D50Y  (1.0)
+#define D50Z  (0.8249)
+#define MaxYCC  ((MagickRealType) (1.3584*MaxMap+0.5))
 #define TransformRGBImageTag  "Transform/Image"
 
   static const unsigned long
     sRGBMap[351] =
     {
-      0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18,
-      19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 29, 30, 31, 32, 33,
-      34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49,
-      50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 65, 66,
-      67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82,
-      83, 84, 85, 86, 87, 88, 89, 90, 91, 92, 93, 95, 96, 97, 98, 99,
-      100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 112,
-      114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125, 126,
-      127, 128, 129, 130, 131, 132, 133, 135, 136, 137, 138, 139, 140,
-      141, 142, 143, 144, 145, 146, 147, 148, 149, 150, 151, 152, 153,
-      154, 155, 156, 157, 158, 159, 160, 161, 162, 163, 164, 165, 166,
-      167, 168, 169, 170, 171, 172, 173, 174, 175, 175, 176, 177, 178,
-      179, 180, 181, 182, 183, 184, 185, 186, 187, 187, 188, 189, 190,
-      191, 192, 193, 194, 194, 195, 196, 197, 198, 199, 199, 200, 201,
-      202, 203, 203, 204, 205, 206, 207, 207, 208, 209, 210, 210, 211,
-      212, 213, 213, 214, 215, 215, 216, 217, 218, 218, 219, 220, 220,
-      221, 222, 222, 223, 223, 224, 225, 225, 226, 227, 227, 228, 228,
-      229, 229, 230, 230, 231, 232, 232, 233, 233, 234, 234, 235, 235,
-      235, 236, 236, 237, 237, 238, 238, 238, 239, 239, 240, 240, 240,
-      241, 241, 242, 242, 242, 243, 243, 243, 243, 244, 244, 244, 245,
-      245, 245, 245, 246, 246, 246, 247, 247, 247, 247, 247, 248, 248,
-      248, 248, 249, 249, 249, 249, 249, 249, 250, 250, 250, 250, 250,
-      250, 251, 251, 251, 251, 251, 251, 252, 252, 252, 252, 252, 252,
-      252, 252, 252, 253, 253, 253, 253, 253, 253, 253, 253, 253, 254,
-      254, 254, 254, 254, 254, 254, 254, 254, 254, 254, 254, 255, 255,
-      255, 255, 255, 255, 255
+        0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,
+       14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,
+       28,  29,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,
+       41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,
+       55,  56,  57,  58,  59,  60,  61,  62,  63,  65,  66,  67,  68,  69,
+       70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,
+       84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  95,  96,  97,  98,
+       99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 112,
+      114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125, 126, 127,
+      128, 129, 130, 131, 132, 133, 135, 136, 137, 138, 139, 140, 141, 142,
+      143, 144, 145, 146, 147, 148, 149, 150, 151, 152, 153, 154, 155, 156,
+      157, 158, 159, 160, 161, 162, 163, 164, 165, 166, 167, 168, 169, 170,
+      171, 172, 173, 174, 175, 175, 176, 177, 178, 179, 180, 181, 182, 183,
+      184, 185, 186, 187, 187, 188, 189, 190, 191, 192, 193, 194, 194, 195,
+      196, 197, 198, 199, 199, 200, 201, 202, 203, 203, 204, 205, 206, 207,
+      207, 208, 209, 210, 210, 211, 212, 213, 213, 214, 215, 215, 216, 217,
+      218, 218, 219, 220, 220, 221, 222, 222, 223, 223, 224, 225, 225, 226,
+      227, 227, 228, 228, 229, 229, 230, 230, 231, 232, 232, 233, 233, 234,
+      234, 235, 235, 235, 236, 236, 237, 237, 238, 238, 238, 239, 239, 240,
+      240, 240, 241, 241, 242, 242, 242, 243, 243, 243, 243, 244, 244, 244,
+      245, 245, 245, 245, 246, 246, 246, 247, 247, 247, 247, 247, 248, 248,
+      248, 248, 249, 249, 249, 249, 249, 249, 250, 250, 250, 250, 250, 250,
+      251, 251, 251, 251, 251, 251, 252, 252, 252, 252, 252, 252, 252, 252,
+      252, 253, 253, 253, 253, 253, 253, 253, 253, 253, 254, 254, 254, 254,
+      254, 254, 254, 254, 254, 254, 254, 254, 255, 255, 255, 255, 255, 255,
+      255
     },
     YCCMap[351] =  /* Photo CD information beyond 100% white, Gamma 2.2 */
     {
-      0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18,
-      19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 32, 33, 34, 35,
-      36, 37, 38, 39, 40, 41, 42, 43, 45, 46, 47, 48, 49, 50, 51, 52,
-      53, 54, 56, 57, 58, 59, 60, 61, 62, 63, 64, 66, 67, 68, 69, 70,
-      71, 72, 73, 74, 76, 77, 78, 79, 80, 81, 82, 83, 84, 86, 87, 88,
-      89, 90, 91, 92, 93, 94, 95, 97, 98, 99, 100, 101, 102, 103, 104,
-      105, 106, 107, 108, 110, 111, 112, 113, 114, 115, 116, 117, 118,
-      119, 120, 121, 122, 123, 124, 125, 126, 127, 129, 130, 131, 132,
-      133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 144, 145,
-      146, 147, 148, 149, 150, 151, 152, 153, 154, 155, 156, 157, 158,
-      159, 160, 161, 162, 163, 164, 165, 166, 167, 168, 169, 170, 171,
-      172, 173, 174, 175, 176, 176, 177, 178, 179, 180, 181, 182, 183,
-      184, 185, 186, 187, 188, 189, 190, 191, 192, 193, 193, 194, 195,
-      196, 197, 198, 199, 200, 201, 201, 202, 203, 204, 205, 206, 207,
-      207, 208, 209, 210, 211, 211, 212, 213, 214, 215, 215, 216, 217,
-      218, 218, 219, 220, 221, 221, 222, 223, 224, 224, 225, 226, 226,
-      227, 228, 228, 229, 230, 230, 231, 232, 232, 233, 234, 234, 235,
-      236, 236, 237, 237, 238, 238, 239, 240, 240, 241, 241, 242, 242,
-      243, 243, 244, 244, 245, 245, 245, 246, 246, 247, 247, 247, 248,
-      248, 248, 249, 249, 249, 249, 250, 250, 250, 250, 251, 251, 251,
-      251, 251, 252, 252, 252, 252, 252, 253, 253, 253, 253, 253, 253,
-      253, 253, 253, 253, 253, 253, 253, 254, 254, 254, 254, 254, 254,
-      254, 254, 254, 254, 254, 254, 254, 254, 254, 254, 254, 254, 254,
-      255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
-      255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
-      255, 255, 255, 255, 255, 255
+        0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,
+       14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,
+       28,  29,  30,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,
+       43,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  56,  57,  58,
+       59,  60,  61,  62,  63,  64,  66,  67,  68,  69,  70,  71,  72,  73,
+       74,  76,  77,  78,  79,  80,  81,  82,  83,  84,  86,  87,  88,  89,
+       90,  91,  92,  93,  94,  95,  97,  98,  99, 100, 101, 102, 103, 104,
+      105, 106, 107, 108, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119,
+      120, 121, 122, 123, 124, 125, 126, 127, 129, 130, 131, 132, 133, 134,
+      135, 136, 137, 138, 139, 140, 141, 142, 143, 144, 145, 146, 147, 148,
+      149, 150, 151, 152, 153, 154, 155, 156, 157, 158, 159, 160, 161, 162,
+      163, 164, 165, 166, 167, 168, 169, 170, 171, 172, 173, 174, 175, 176,
+      176, 177, 178, 179, 180, 181, 182, 183, 184, 185, 186, 187, 188, 189,
+      190, 191, 192, 193, 193, 194, 195, 196, 197, 198, 199, 200, 201, 201,
+      202, 203, 204, 205, 206, 207, 207, 208, 209, 210, 211, 211, 212, 213,
+      214, 215, 215, 216, 217, 218, 218, 219, 220, 221, 221, 222, 223, 224,
+      224, 225, 226, 226, 227, 228, 228, 229, 230, 230, 231, 232, 232, 233,
+      234, 234, 235, 236, 236, 237, 237, 238, 238, 239, 240, 240, 241, 241,
+      242, 242, 243, 243, 244, 244, 245, 245, 245, 246, 246, 247, 247, 247,
+      248, 248, 248, 249, 249, 249, 249, 250, 250, 250, 250, 251, 251, 251,
+      251, 251, 252, 252, 252, 252, 252, 253, 253, 253, 253, 253, 253, 253,
+      253, 253, 253, 253, 253, 253, 254, 254, 254, 254, 254, 254, 254, 254,
+      254, 254, 254, 254, 254, 254, 254, 254, 254, 254, 254, 255, 255, 255,
+      255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
+      255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
+      255
     };
 
   long
@@ -891,7 +1005,7 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
       return(MagickTrue);
     default:
       break;
-	}
+  }
   switch (colorspace)
   {
     case CMYKColorspace:
@@ -904,8 +1018,10 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
       */
       if (image->storage_class == PseudoClass)
         {
-          (void) SyncImage(image);
-          image->storage_class=DirectClass;
+          if (SyncImage(image) == MagickFalse)
+            return(MagickFalse);
+          if (SetImageStorageClass(image,DirectClass) == MagickFalse)
+            return(MagickFalse);
         }
       for (y=0; y < (long) image->rows; y++)
       {
@@ -915,19 +1031,24 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
         indexes=GetIndexes(image);
         for (x=0; x < (long) image->columns; x++)
         {
-          q->red=RoundToQuantum(QuantumScale* ((MagickRealType)
-            (QuantumRange-q->red)*(QuantumRange-indexes[x])));
-          q->green=RoundToQuantum(QuantumScale* ((MagickRealType)
-            (QuantumRange-q->green)*(QuantumRange-indexes[x])));
-          q->blue=RoundToQuantum(QuantumScale* ((MagickRealType)
-            (QuantumRange-q->blue)*(QuantumRange-indexes[x])));
+          q->red=RoundToQuantum((MagickRealType) (QuantumRange-
+            Min(QuantumRange,QuantumScale*q->red*(QuantumRange-indexes[x])+
+            indexes[x])));
+          q->green=RoundToQuantum((MagickRealType) (QuantumRange-
+            Min(QuantumRange,QuantumScale*q->green*(QuantumRange-indexes[x])+
+            indexes[x])));
+          q->blue=RoundToQuantum((MagickRealType) (QuantumRange-
+            Min(QuantumRange,QuantumScale*q->blue*(QuantumRange-indexes[x])+
+            indexes[x])));
           q++;
         }
         if (SyncImagePixels(image) == MagickFalse)
           break;
       }
       image->colorspace=RGBColorspace;
-      return(MagickTrue);
+      if (SyncImageCache(image) == MagickFalse)
+        return(MagickFalse);
+      return(y == (long) image->rows ? MagickTrue : MagickFalse);
     }
     case HSBColorspace:
     {
@@ -941,15 +1062,17 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
       */
       if (image->storage_class == PseudoClass)
         {
-          (void) SyncImage(image);
-          image->storage_class=DirectClass;
+          if (SyncImage(image) == MagickFalse)
+            return(MagickFalse);
+          if (SetImageStorageClass(image,DirectClass) == MagickFalse)
+            return(MagickFalse);
         }
       for (y=0; y < (long) image->rows; y++)
       {
         q=GetImagePixels(image,0,y,image->columns,1);
         if (q == (PixelPacket *) NULL)
           break;
-        for (x=(long) image->columns-1; x >= 0; x--)
+        for (x=0; x < (long) image->columns; x++)
         {
           hue=(double) (QuantumScale*q->red);
           saturation=(double) (QuantumScale*q->green);
@@ -961,7 +1084,7 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           break;
       }
       image->colorspace=RGBColorspace;
-      return(MagickTrue);
+      return(y == (long) image->rows ? MagickTrue : MagickFalse);
     }
     case HSLColorspace:
     {
@@ -975,15 +1098,17 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
       */
       if (image->storage_class == PseudoClass)
         {
-          (void) SyncImage(image);
-          image->storage_class=DirectClass;
+          if (SyncImage(image) == MagickFalse)
+            return(MagickFalse);
+          if (SetImageStorageClass(image,DirectClass) == MagickFalse)
+            return(MagickFalse);
         }
       for (y=0; y < (long) image->rows; y++)
       {
         q=GetImagePixels(image,0,y,image->columns,1);
         if (q == (PixelPacket *) NULL)
           break;
-        for (x=(long) image->columns-1; x >= 0; x--)
+        for (x=0; x < (long) image->columns; x++)
         {
           hue=(double) (QuantumScale*q->red);
           saturation=(double) (QuantumScale*q->green);
@@ -995,7 +1120,7 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           break;
       }
       image->colorspace=RGBColorspace;
-      return(MagickTrue);
+      return(y == (long) image->rows ? MagickTrue : MagickFalse);
     }
     case HWBColorspace:
     {
@@ -1009,15 +1134,17 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
       */
       if (image->storage_class == PseudoClass)
         {
-          (void) SyncImage(image);
-          image->storage_class=DirectClass;
+          if (SyncImage(image) == MagickFalse)
+            return(MagickFalse);
+          if (SetImageStorageClass(image,DirectClass) == MagickFalse)
+            return(MagickFalse);
         }
       for (y=0; y < (long) image->rows; y++)
       {
         q=GetImagePixels(image,0,y,image->columns,1);
         if (q == (PixelPacket *) NULL)
           break;
-        for (x=(long) image->columns-1; x >= 0; x--)
+        for (x=0; x < (long) image->columns; x++)
         {
           hue=(double) (QuantumScale*q->red);
           whiteness=(double) (QuantumScale*q->green);
@@ -1029,7 +1156,101 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           break;
       }
       image->colorspace=RGBColorspace;
-      return(MagickTrue);
+      return(y == (long) image->rows ? MagickTrue : MagickFalse);
+    }
+    case LABColorspace:
+    {
+      /*
+        Transform image from LAB to XYZ.
+          * * * not implemented * * *
+      */
+      if (image->storage_class == PseudoClass)
+        {
+          if (SyncImage(image) == MagickFalse)
+            return(MagickFalse);
+          if (SetImageStorageClass(image,DirectClass) == MagickFalse)
+            return(MagickFalse);
+        }
+      for (y=0; y < (long) image->rows; y++)
+      {
+        q=GetImagePixels(image,0,y,image->columns,1);
+        if (q == (PixelPacket *) NULL)
+          break;
+        for (x=0; x < (long) image->columns; x++)
+        {
+          q++;
+        }
+        if (SyncImagePixels(image) == MagickFalse)
+          break;
+      }
+      break;
+    }
+    case LogColorspace:
+    {
+      const ImageAttribute
+        *attribute;
+
+      double
+        black,
+        density,
+        gamma,
+        reference_black,
+        reference_white;
+
+      Quantum
+        *logmap;
+
+      /*
+        Transform Log to RGB colorspace.
+      */
+      density=2.03728;
+      gamma=DisplayGamma;
+      attribute=GetImageAttribute(image,"Gamma");
+      if (attribute != (const ImageAttribute *) NULL)
+        gamma=1.0/atof(attribute->value) != 0.0 ? atof(attribute->value) : 1.0;
+      reference_black=ReferenceBlack;
+      attribute=GetImageAttribute(image,"reference-black");
+      if (attribute != (const ImageAttribute *) NULL)
+        reference_black=atof(attribute->value);
+      reference_white=ReferenceWhite;
+      attribute=GetImageAttribute(image,"reference-white");
+      if (attribute != (const ImageAttribute *) NULL)
+        reference_white=atof(attribute->value);
+      logmap=(Quantum *)
+        AcquireMagickMemory((size_t) (MaxMap+1)*sizeof(*logmap));
+      if (logmap == (Quantum *) NULL)
+        ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+          image->filename);
+      black=pow(10.0,(reference_black-reference_white)*(gamma/density)*
+        0.002/0.6);
+      for (i=0; i <= (long) (reference_black*MaxMap/1024.0); i++)
+        logmap[i]=0;
+      for ( ; i < (long) (reference_white*MaxMap/1024.0); i++)
+        logmap[i]=RoundToQuantum((MagickRealType) QuantumRange/(1.0-black)*
+          (pow(10.0,(1024.0*i/MaxMap-reference_white)*
+          (gamma/density)*0.002/0.6)-black));
+      for ( ; i <= (long) MaxMap; i++)
+        logmap[i]=QuantumRange;
+      if (SetImageStorageClass(image,DirectClass) == MagickFalse)
+        return(MagickFalse);
+      for (y=0; y < (long) image->rows; y++)
+      {
+        q=GetImagePixels(image,0,y,image->columns,1);
+        if (q == (PixelPacket *) NULL)
+          break;
+        for (x=(long) image->columns; x > 0; x--)
+        {
+          q->red=logmap[ScaleQuantumToMap(q->red)];
+          q->green=logmap[ScaleQuantumToMap(q->green)];
+          q->blue=logmap[ScaleQuantumToMap(q->blue)];
+          q++;
+        }
+        if (SyncImagePixels(image) == MagickFalse)
+          break;
+      }
+      logmap=(Quantum *) RelinquishMagickMemory(logmap);
+      image->colorspace=RGBColorspace;
+      return(y == (long) image->rows ? MagickTrue : MagickFalse);
     }
     default:
       break;
@@ -1072,6 +1293,59 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
       }
       break;
     }
+    case Rec601YCbCrColorspace:
+    case YCbCrColorspace:
+    {
+      /*
+        Initialize YCbCr tables:
+
+          R = Y            +1.402000*Cr
+          G = Y-0.344136*Cb-0.714136*Cr
+          B = Y+1.772000*Cb
+
+        Cb and Cr, normally -0.5 through 0.5, must be normalized to the range 0
+        through QuantumRange.
+      */
+      for (i=0; i <= (long) MaxMap; i++)
+      {
+        x_map[i].x=(double) i;
+        x_map[i].y=(double) i;
+        x_map[i].z=(double) i;
+        y_map[i].x=0.0;
+        y_map[i].y=(-0.344136*0.5)*(2.0*i-MaxMap);
+        y_map[i].z=(1.772000*0.5)*(2.0*i-MaxMap);
+        z_map[i].x=(1.402000*0.5)*(2.0*i-MaxMap);
+        z_map[i].y=(-0.714136*0.5)*(2.0*i-MaxMap);
+        z_map[i].z=0.0;
+      }
+      break;
+    }
+    case Rec709YCbCrColorspace:
+    {
+      /*
+        Initialize YCbCr tables:
+
+          R = Y            +1.574800*Cr
+          G = Y-0.187324*Cb-0.468124*Cr
+          B = Y+1.855600*Cb
+
+        Cb and Cr, normally -0.5 through 0.5, must be normalized to the range 0
+        through QuantumRange.
+      */
+      for (i=0; i <= (long) MaxMap; i++)
+      {
+        x_map[i].x=(double) i;
+        x_map[i].y=(double) i;
+        x_map[i].z=(double) i;
+        y_map[i].x=0.0;
+        y_map[i].y=(-0.187324*0.5)*(2.0*i-MaxMap);
+        y_map[i].z=(1.855600*0.5)*(2.0*i-MaxMap);
+        z_map[i].x=(1.574800*0.5)*(2.0*i-MaxMap);
+        z_map[i].y=(-0.468124*0.5)*(2.0*i-MaxMap);
+        z_map[i].z=0.0;
+      }
+      break;
+    }
     case sRGBColorspace:
     {
       /*
@@ -1101,10 +1375,11 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
       }
       break;
     }
+    case LABColorspace:
     case XYZColorspace:
     {
       /*
-        Initialize CIE XYZ tables:
+        Initialize CIE XYZ tables (ITU R-709 RGB):
 
           R =  3.240479*R-1.537150*G-0.498535*B
           G = -0.969256*R+1.875992*G+0.041556*B
@@ -1124,32 +1399,6 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
       }
       break;
     }
-    case YCbCrColorspace:
-    {
-      /*
-        Initialize YCbCr tables:
-
-          R = Y            +1.402000*Cr
-          G = Y-0.344136*Cb-0.714136*Cr
-          B = Y+1.772000*Cb
-
-        Cb and Cr, normally -0.5 through 0.5, must be normalized to the range 0
-        through QuantumRange.
-      */
-      for (i=0; i <= (long) MaxMap; i++)
-      {
-        x_map[i].x=(double) i;
-        x_map[i].y=(double) i;
-        x_map[i].z=(double) i;
-        y_map[i].x=0.0;
-        y_map[i].y=(-0.344136*0.5)*(2.0*i-MaxMap);
-        y_map[i].z=(1.772000*0.5)*(2.0*i-MaxMap);
-        z_map[i].x=(1.402000*0.5)*(2.0*i-MaxMap);
-        z_map[i].y=(-0.714136*0.5)*(2.0*i-MaxMap);
-        z_map[i].z=0.0;
-      }
-      break;
-    }
     case YCCColorspace:
     {
       /*
@@ -1296,14 +1545,11 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
             case YCCColorspace:
             {
               quantum.red=(unsigned long) ((pixel.red < 0.0) ? 0.0 :
-                (pixel.red > (double) MaxYCC) ? (double)
-                MaxYCC : pixel.red+0.5);
+                (pixel.red > MaxYCC) ? MaxYCC : pixel.red+0.5);
               quantum.green=(unsigned long) ((pixel.green < 0.0) ? 0.0 :
-                (pixel.green > (double) MaxYCC) ? (double)
-                MaxYCC : pixel.green+0.5);
+                (pixel.green > MaxYCC) ?  MaxYCC : pixel.green+0.5);
               quantum.blue=(unsigned long) ((pixel.blue < 0.0) ? 0.0 :
-                (pixel.blue > (double) MaxYCC) ? (double)
-                MaxYCC : pixel.blue+0.5);
+                (pixel.blue > MaxYCC) ? MaxYCC : pixel.blue+0.5);
               if (colorspace == sRGBColorspace)
                 {
                   q->red=(Quantum)
@@ -1370,14 +1616,11 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           case YCCColorspace:
           {
             quantum.red=(unsigned long) ((pixel.red < 0.0) ? 0.0 :
-              (pixel.red > (double) MaxYCC) ? (double) MaxYCC :
-              pixel.red+0.5);
+              (pixel.red > MaxYCC) ? MaxYCC : pixel.red+0.5);
             quantum.green=(unsigned long) ((pixel.green < 0.0) ? 0.0 :
-              (pixel.green > (double) MaxYCC) ? (double) MaxYCC :
-              pixel.green+0.5);
+              (pixel.green > MaxYCC) ? MaxYCC : pixel.green+0.5);
             quantum.blue=(unsigned long) ((pixel.blue < 0.0) ? 0.0 :
-              (pixel.blue > (double) MaxYCC) ? (double) MaxYCC :
-              pixel.blue+0.5);
+              (pixel.blue > MaxYCC) ? MaxYCC : pixel.blue+0.5);
             if (colorspace == sRGBColorspace)
               {
                 image->colormap[i].red=(Quantum)
