@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -13,10 +13,10 @@
   See the License for the specific language governing permissions and
   limitations under the License.
 
-  ImageMagick image composite private methods.
+  MagickCore image composite private methods.
 */
-#ifndef _MAGICK_COMPOSITE_PRIVATE_H
-#define _MAGICK_COMPOSITE_PRIVATE_H
+#ifndef _MAGICKCORE_COMPOSITE_PRIVATE_H
+#define _MAGICKCORE_COMPOSITE_PRIVATE_H
 
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
@@ -27,9 +27,8 @@ extern "C" {
 */
 
 #include "magick/color.h"
-
-#define RoundToUnity(value) \
-  ((value) < 0.0 ? 0.0 : ((value) > 1.0) ? 1.0 : (value))
+#include "magick/image.h"
+#include "magick/image-private.h"
 
 static inline MagickRealType MagickOver_(const MagickRealType p,
   const MagickRealType alpha,const MagickRealType q,const MagickRealType beta)
@@ -47,7 +46,12 @@ static inline void MagickCompositeOver(const PixelPacket *p,
 {
   MagickRealType
     gamma;
-                                                                                
+
+  if (alpha == TransparentOpacity)
+    {
+      *composite=(*q);
+      return;
+    }
   gamma=1.0-QuantumScale*QuantumScale*alpha*beta;
   composite->opacity=(Quantum) (QuantumRange*(1.0-gamma)+0.5);
   gamma=1.0/(gamma <= MagickEpsilon ? 1.0 : gamma);
@@ -66,8 +70,13 @@ static inline void MagickPixelCompositeOver(const MagickPixelPacket *p,
   MagickRealType
     gamma;
 
+  if (alpha == TransparentOpacity)
+    {
+      *composite=(*q);
+      return;
+    }
   gamma=1.0-QuantumScale*QuantumScale*alpha*beta;
-  composite->opacity=QuantumRange*(1.0-gamma);
+  composite->opacity=(MagickRealType) QuantumRange*(1.0-gamma);
   gamma=1.0/(gamma <= MagickEpsilon ? 1.0 : gamma);
   composite->red=gamma*MagickOver_(p->red,alpha,q->red,beta);
   composite->green=gamma*MagickOver_(p->green,alpha,q->green,beta);
@@ -76,6 +85,11 @@ static inline void MagickPixelCompositeOver(const MagickPixelPacket *p,
     composite->index=gamma*MagickOver_(p->index,alpha,q->index,beta);
 }
 
+static inline MagickRealType RoundToUnity(const MagickRealType value)
+{
+  return(value < 0.0 ? 0.0 : (value > 1.0) ? 1.0 : value);
+}
+
 #if defined(__cplusplus) || defined(c_plusplus)
 }
 #endif
