@@ -17,7 +17,7 @@
 %                              May  1993                                      %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -43,11 +43,13 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/color-private.h"
 #include "magick/compress.h"
 #include "magick/constitute.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
 #include "magick/image-private.h"
+#include "magick/list.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/resource_.h"
@@ -286,7 +288,7 @@ MagickExport void Ascii85Flush(Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(image->ascii85 != (Ascii85Info *) NULL);
   if (image->ascii85->offset > 0)
     {
@@ -295,7 +297,7 @@ MagickExport void Ascii85Flush(Image *image)
       image->ascii85->buffer[image->ascii85->offset+2]='\0';
       tuple=Ascii85Tuple(image->ascii85->buffer);
       (void) WriteBlob(image,(size_t) image->ascii85->offset+1,
-        (unsigned char *) (*tuple == 'z' ? "!!!!" : tuple));
+        (const unsigned char *) (*tuple == 'z' ? "!!!!" : tuple));
     }
   (void) WriteBlobByte(image,'~');
   (void) WriteBlobByte(image,'>');
@@ -452,7 +454,7 @@ MagickExport MagickBooleanType HuffmanDecodeImage(Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   mb_hash=(HuffmanTable **) AcquireMagickMemory(HashSize*sizeof(*mb_hash));
   mw_hash=(HuffmanTable **) AcquireMagickMemory(HashSize*sizeof(*mw_hash));
   scanline=(unsigned char *) AcquireMagickMemory((size_t) image->columns);
@@ -736,7 +738,7 @@ MagickExport MagickBooleanType HuffmanEncodeImage(const ImageInfo *image_info,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   width=image->columns;
   if (LocaleCompare(image_info->magick,"FAX") == 0)
     width=Max(image->columns,1728);
@@ -765,7 +767,7 @@ MagickExport MagickBooleanType HuffmanEncodeImage(const ImageInfo *image_info,
     Compress runlength encoded to 1D Huffman pixels.
   */
   polarity=(IndexPacket) (PixelIntensityToQuantum(
-    &huffman_image->colormap[0]) >= (MaxRGB/2) ? 0x00 : 0x01);
+    &huffman_image->colormap[0]) >= (QuantumRange/2) ? 0x00 : 0x01);
   if (huffman_image->colors == 2)
     polarity=(IndexPacket)
       (PixelIntensityToQuantum(&huffman_image->colormap[0]) <
@@ -841,7 +843,7 @@ MagickExport MagickBooleanType HuffmanEncodeImage(const ImageInfo *image_info,
       OutputBit(0);
     OutputBit(1);
     q=scanline;
-    if (huffman_image->previous == (Image *) NULL)
+    if (GetPreviousImageInList(huffman_image) == (Image *) NULL)
       if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
           (QuantumTick(y,huffman_image->rows) != MagickFalse))
         {
@@ -950,7 +952,7 @@ MagickExport MagickBooleanType Huffman2DEncodeImage(const ImageInfo *image_info,
   assert(image_info != (ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   huffman_image=CloneImage(image,0,0,MagickTrue,&image->exception);
@@ -1021,7 +1023,7 @@ MagickExport MagickBooleanType Huffman2DEncodeImage(const ImageInfo *image_info,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   ThrowBinaryException(MissingDelegateError,"TIFFLibraryIsNotAvailable",
     image->filename);
 }
@@ -1078,7 +1080,7 @@ MagickExport MagickBooleanType JPEGEncodeImage(const ImageInfo *image_info,
   jpeg_image->blob=CloneBlobInfo((BlobInfo *) NULL);
   jpeg_info=CloneImageInfo(image_info);
   *jpeg_info->filename='\0';
-  (void) strcpy(jpeg_info->magick,"JPEG");
+  (void) CopyMagickString(jpeg_info->magick,"JPEG",MaxTextExtent);
   length=0;
   blob=ImageToBlob(jpeg_info,jpeg_image,&length,&image->exception);
   jpeg_info=DestroyImageInfo(jpeg_info);
@@ -1096,7 +1098,7 @@ MagickExport MagickBooleanType JPEGEncodeImage(const ImageInfo *image_info,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   ThrowBinaryException(MissingDelegateError,"JPEGLibraryIsNotAvailable",
     image->filename);
 }
@@ -1153,7 +1155,7 @@ MagickExport MagickBooleanType JP2EncodeImage(const ImageInfo *image_info,
   jp2_image->blob=CloneBlobInfo((BlobInfo *) NULL);
   jp2_info=CloneImageInfo(image_info);
   *jp2_info->filename='\0';
-  (void) strcpy(jp2_info->magick,"JP2");
+  (void) CopyMagickString(jp2_info->magick,"JP2",MaxTextExtent);
   length=0;
   blob=ImageToBlob(jp2_info,jp2_image,&length,&image->exception);
   jp2_info=DestroyImageInfo(jp2_info);
@@ -1165,13 +1167,13 @@ MagickExport MagickBooleanType JP2EncodeImage(const ImageInfo *image_info,
   return(MagickTrue);
 }
 #else
-MagickExport MagickBooleanType JP2EncodeImage(const ImageInfo *image_info,
-  Image *image)
+MagickExport MagickBooleanType JP2EncodeImage(
+  const ImageInfo *magick_unused(image_info),Image *image)
 {
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   ThrowBinaryException(MissingDelegateError,"JP2LibraryIsNotAvailable",
     image->filename);
 }
@@ -1254,7 +1256,7 @@ MagickExport MagickBooleanType LZWEncodeImage(Image *image,const size_t length,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(pixels != (unsigned char *) NULL);
   table=(TableType *) AcquireMagickMemory((1 << 12)*sizeof(*table));
   if (table == (TableType *) NULL)
@@ -1389,7 +1391,7 @@ MagickExport MagickBooleanType PackbitsEncodeImage(Image *image,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(pixels != (unsigned char *) NULL);
   packbits=(unsigned char *) AcquireMagickMemory(128);
   if (packbits == (unsigned char *) NULL)
@@ -1494,7 +1496,7 @@ MagickExport MagickBooleanType PackbitsEncodeImage(Image *image,
 %  The format of the ZLIBEncodeImage method is:
 %
 %      MagickBooleanType ZLIBEncodeImage(Image *image,const size_t length,
-%        const unsigned long quality,unsigned char *pixels)
+%        unsigned char *pixels)
 %
 %  A description of each parameter follows:
 %
@@ -1503,15 +1505,12 @@ MagickExport MagickBooleanType PackbitsEncodeImage(Image *image,
 %
 %    o length:  A value that specifies the number of pixels to compress.
 %
-%    o quality: the compression level (0-100).
-%
 %    o pixels: The address of an unsigned array of characters containing the
 %      pixels to compress.
 %
-%
 */
 MagickExport MagickBooleanType ZLIBEncodeImage(Image *image,const size_t length,
-  const unsigned long quality,unsigned char *pixels)
+  unsigned char *pixels)
 {
   int
     status;
@@ -1531,7 +1530,7 @@ MagickExport MagickBooleanType ZLIBEncodeImage(Image *image,const size_t length,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   compressed_packets=(size_t) (1.001*length+12);
   compressed_pixels=(unsigned char *) AcquireMagickMemory(compressed_packets);
   if (compressed_pixels == (unsigned char *) NULL)
@@ -1544,7 +1543,8 @@ MagickExport MagickBooleanType ZLIBEncodeImage(Image *image,const size_t length,
   stream.zalloc=(alloc_func) NULL;
   stream.zfree=(free_func) NULL;
   stream.opaque=(voidpf) NULL;
-  status=deflateInit(&stream,(int) Min(quality/10,9));
+  status=deflateInit(&stream,(int) (image->quality ==
+    UndefinedCompressionQuality ? 7 : Min(image->quality/10,9)));
   if (status == Z_OK)
     {
       status=deflate(&stream,Z_FINISH);
@@ -1563,13 +1563,13 @@ MagickExport MagickBooleanType ZLIBEncodeImage(Image *image,const size_t length,
   return((MagickBooleanType) (status == Z_OK));
 }
 #else
-MagickExport MagickBooleanType ZLIBEncodeImage(Image *image,const size_t length,
-  const unsigned long quality,unsigned char *pixels)
+MagickExport MagickBooleanType ZLIBEncodeImage(Image *image,
+  const size_t magick_unused(length),unsigned char *magick_unused(pixels))
 {
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   ThrowBinaryException(MissingDelegateError,"ZLIBLibraryIsNotAvailable",
     image->filename);
 }
