@@ -17,7 +17,7 @@
 %                               October 1998                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -46,6 +46,7 @@
 #include "magick/exception.h"
 #include "magick/exception-private.h"
 #include "magick/cache.h"
+#include "magick/client.h"
 #include "magick/constitute.h"
 #include "magick/delegate.h"
 #include "magick/geometry.h"
@@ -57,6 +58,7 @@
 #include "magick/monitor.h"
 #include "magick/option.h"
 #include "magick/pixel.h"
+#include "magick/resize.h"
 #include "magick/resource_.h"
 #include "magick/semaphore.h"
 #include "magick/stream.h"
@@ -141,7 +143,6 @@ MagickExport Image *ConstituteImage(const unsigned long columns,
   assert(pixels != (void *) NULL);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  (void) SetExceptionInfo(exception,UndefinedException);
   image=AllocateImage((ImageInfo *) NULL);
   if (image == (Image *) NULL)
     return((Image *) NULL);
@@ -164,7 +165,7 @@ MagickExport Image *ConstituteImage(const unsigned long columns,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   D e s t r o y C o n s t i t u t e                                         %
++   D e s t r o y C o n s t i t u t e                                         %
 %                                                                             %
 %                                                                             %
 %                                                                             %
@@ -218,10 +219,10 @@ MagickExport void DestroyConstitute(void)
 extern "C" {
 #endif
 
-static MagickBooleanType PingStream(const Image *image,const void *pixels,
-  const size_t columns)
+static size_t PingStream(const Image *magick_unused(image),
+  const void *magick_unused(pixels),const size_t columns)
 {
-  return(MagickTrue);
+  return(columns);
 }
 
 #if defined(__cplusplus) || defined(c_plusplus)
@@ -243,17 +244,16 @@ MagickExport Image *PingImage(const ImageInfo *image_info,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  (void) SetExceptionInfo(exception,UndefinedException);
   ping_info=CloneImageInfo(image_info);
   ping_info->ping=MagickTrue;
   image=ReadStream(ping_info,&PingStream,exception);
-  ping_info=DestroyImageInfo(ping_info);
   if (image != (Image *) NULL)
     {
       ResetTimer(&image->timer);
-	    if (image_info->verbose != MagickFalse)
+      if (ping_info->verbose != MagickFalse)
         (void) IdentifyImage(image,stdout,MagickFalse);
     }
+  ping_info=DestroyImageInfo(ping_info);
   return(image);
 }
 
@@ -291,14 +291,11 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
 {
   char
     filename[MaxTextExtent],
-    magick[MaxTextExtent];
+    magick[MaxTextExtent],
+    magick_filename[MaxTextExtent];
 
   const char
-    *comment,
-    *delay,
-    *dispose,
-    *label,
-    *page;
+    *option;
 
   const DelegateInfo
     *delegate_info;
@@ -309,6 +306,9 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
   const MagickInfo
     *magick_info;
 
+  ExceptionInfo
+    sans_exception;
+
   GeometryInfo
     geometry_info;
 
@@ -319,6 +319,9 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
   ImageInfo
     *read_info;
 
+  MagickStatusType
+    flags;
+
   /*
     Determine image type from filename prefix or suffix (e.g. image.jpg).
   */
@@ -329,17 +332,19 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  (void) SetExceptionInfo(exception,UndefinedException);
   if (*image_info->filename == '@')
     return(ReadImages(image_info,exception));
   read_info=CloneImageInfo(image_info);
+  (void) CopyMagickString(magick_filename,read_info->filename,MaxTextExtent);
   (void) SetImageInfo(read_info,MagickFalse,exception);
   (void) CopyMagickString(filename,read_info->filename,MaxTextExtent);
   (void) CopyMagickString(magick,read_info->magick,MaxTextExtent);
   /*
     Call appropriate image reader based on image type.
   */
-  magick_info=GetMagickInfo(read_info->magick,exception);
+  GetExceptionInfo(&sans_exception);
+  magick_info=GetMagickInfo(read_info->magick,&sans_exception);
+  (void) DestroyExceptionInfo(&sans_exception);
   if ((magick_info != (const MagickInfo *) NULL) &&
       (GetMagickSeekableStream(magick_info) != MagickFalse))
     {
@@ -382,7 +387,7 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
       (GetMagickDecoder(magick_info) != (DecoderHandler *) NULL))
     {
       if (GetMagickThreadSupport(magick_info) == MagickFalse)
-        constitute_semaphore=AcquireSemaphoreInfo(constitute_semaphore);
+        AcquireSemaphoreInfo(&constitute_semaphore);
       image=GetMagickDecoder(magick_info)(read_info,exception);
       if (GetMagickThreadSupport(magick_info) == MagickFalse)
         RelinquishSemaphoreInfo(constitute_semaphore);
@@ -392,11 +397,7 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
       delegate_info=GetDelegateInfo(read_info->magick,(char *) NULL,exception);
       if (delegate_info == (const DelegateInfo *) NULL)
         {
-          if (IsAccessible(read_info->filename) == MagickFalse)
-            (void) ThrowMagickException(exception,GetMagickModule(),
-              FileOpenError,"UnableToOpenFile","`%s': %s",read_info->filename,
-              strerror(errno));
-          else
+          if (IsAccessible(read_info->filename) != MagickFalse)
             (void) ThrowMagickException(exception,GetMagickModule(),
               MissingDelegateError,"NoDecodeDelegateForThisImageFormat","`%s'",
               read_info->filename);
@@ -414,7 +415,6 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
           read_info=DestroyImageInfo(read_info);
           return((Image *) NULL);
         }
-      (void) SetExceptionInfo(exception,UndefinedException);
       (void) CopyMagickString(image->filename,read_info->filename,
         MaxTextExtent);
       *read_info->filename='\0';
@@ -427,19 +427,26 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
       if ((magick_info == (const MagickInfo *) NULL) ||
           (GetMagickDecoder(magick_info) == (DecoderHandler *) NULL))
         {
-          if (IsAccessible(read_info->filename) == MagickFalse)
-            (void) ThrowMagickException(exception,GetMagickModule(),
-              FileOpenError,"UnableToOpenFile","`%s': %s",read_info->filename,
-              strerror(errno));
-          else
+          if (IsAccessible(read_info->filename) != MagickFalse)
             (void) ThrowMagickException(exception,GetMagickModule(),
               MissingDelegateError,"NoDecodeDelegateForThisImageFormat","`%s'",
               read_info->filename);
+          else
+            {
+              char
+                *message;
+
+              message=GetExceptionMessage(errno);
+              (void) ThrowMagickException(exception,GetMagickModule(),
+                FileOpenError,"UnableToOpenFile","`%s': %s",read_info->filename,
+                message);
+              message=(char *) RelinquishMagickMemory(message);
+            }
           read_info=DestroyImageInfo(read_info);
           return((Image *) NULL);
         }
       if (GetMagickThreadSupport(magick_info) == MagickFalse)
-        constitute_semaphore=AcquireSemaphoreInfo(constitute_semaphore);
+        AcquireSemaphoreInfo(&constitute_semaphore);
       image=(Image *) (GetMagickDecoder(magick_info))(read_info,exception);
       if (GetMagickThreadSupport(magick_info) == MagickFalse)
         RelinquishSemaphoreInfo(constitute_semaphore);
@@ -480,9 +487,14 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
     }
   if (GetBlobError(image) != MagickFalse)
     {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
       (void) ThrowMagickException(exception,GetMagickModule(),CorruptImageError,
         "AnErrorHasOccurredReadingFromFile","`%s': %s",read_info->filename,
-        strerror(errno));
+        message);
+      message=(char *) RelinquishMagickMemory(message);
       image=DestroyImageList(image);
       read_info=DestroyImageInfo(read_info);
       return((Image *) NULL);
@@ -491,7 +503,8 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
   {
     next->taint=MagickFalse;
     (void) CopyMagickString(next->magick,magick,MaxTextExtent);
-    (void) CopyMagickString(next->magick_filename,filename,MaxTextExtent);
+    (void) CopyMagickString(next->magick_filename,magick_filename,
+      MaxTextExtent);
     if (IsBlobTemporary(image) != MagickFalse)
       (void) CopyMagickString(next->filename,filename,MaxTextExtent);
     attribute=GetImageAttribute(next,"EXIF:XResolution");
@@ -500,7 +513,7 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
       {
         geometry_info.rho=next->x_resolution;
         geometry_info.sigma=1.0;
-        (void) ParseGeometry(attribute->value,&geometry_info);
+        flags=ParseGeometry(attribute->value,&geometry_info);
         if (geometry_info.sigma != 0)
           next->x_resolution=geometry_info.rho/geometry_info.sigma;
         (void) DeleteImageAttribute(next,"EXIF:XResolution");
@@ -511,7 +524,7 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
       {
         geometry_info.rho=next->y_resolution;
         geometry_info.sigma=1.0;
-        (void) ParseGeometry(attribute->value,&geometry_info);
+        flags=ParseGeometry(attribute->value,&geometry_info);
         if (geometry_info.sigma != 0)
           next->y_resolution=geometry_info.rho/geometry_info.sigma;
         (void) DeleteImageAttribute(next,"EXIF:YResolution");
@@ -528,58 +541,64 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
         }
         (void) DeleteImageAttribute(next,"EXIF:ResolutionUnit");
       }
-    page=GetImageOption(image_info,"page");
-    if ((page != (char *) NULL) && (LocaleCompare(next->magick,"TEXT") != 0))
+    (void) SyncImageOptions(image_info,next);
+    option=GetImageOption(read_info,"comment");
+    if (option != (const char *) NULL)
+      {
+        (void) DeleteImageAttribute(next,"comment");
+        (void) SetImageAttribute(next,"comment",option);
+      }
+    option=GetImageOption(read_info,"delay");
+    if (option != (const char *) NULL)
+      {
+        flags=ParseGeometry(option,&geometry_info);
+        if ((flags & GreaterValue) != 0)
+          {
+            if (next->delay > (unsigned long) (geometry_info.rho+0.5))
+              next->delay=(unsigned long) (geometry_info.rho+0.5);
+          }
+        else
+          if ((flags & LessValue) != 0)
+            {
+              if (next->delay < (unsigned long) (geometry_info.rho+0.5))
+                next->ticks_per_second=(unsigned long)
+                  (geometry_info.sigma+0.5);
+            }
+          else
+            next->delay=(unsigned long) (geometry_info.rho+0.5);
+        if ((flags & SigmaValue) != 0)
+          next->ticks_per_second=(unsigned long) (geometry_info.sigma+0.5);
+      }
+    option=GetImageOption(read_info,"dispose");
+    if (option != (const char *) NULL)
+      next->dispose=(DisposeType) ParseMagickOption(MagickDisposeOptions,
+        MagickFalse,option);
+    option=GetImageOption(read_info,"label");
+    if (option != (const char *) NULL)
+      {
+        (void) DeleteImageAttribute(next,"label");
+        (void) SetImageAttribute(next,"label",option);
+      }
+    if (next->page.width == 0)
+      next->page.width=next->columns;
+    if (next->page.height == 0)
+      next->page.height=next->rows;
+    option=GetImageOption(read_info,"page");
+    if ((option != (const char *) NULL) &&
+        (LocaleCompare(next->magick,"TEXT") != 0))
       {
         char
           *geometry;
 
-        geometry=GetPageGeometry(page);
-        (void) ParseAbsoluteGeometry(geometry,&next->page);
+        geometry=GetPageGeometry(option);
+        flags=ParseAbsoluteGeometry(geometry,&next->page);
         geometry=(char *) RelinquishMagickMemory(geometry);
       }
-    delay=GetImageOption(image_info,"delay");
-    if (delay != (char *) NULL)
-      {
-        double
-          maximum_delay,
-          minimum_delay;
-
-        int
-          count;
-
-        count=sscanf(delay,"%lf-%lf",&minimum_delay,&maximum_delay);
-        if (count == 1)
-          next->delay=(unsigned long) minimum_delay;
-        else
-          {
-            if ((double) next->delay < minimum_delay)
-              next->delay=(unsigned long) (minimum_delay+0.5);
-            if ((double) next->delay > maximum_delay)
-              next->delay=(unsigned long) (maximum_delay+0.5);
-          }
-      }
-    dispose=GetImageOption(image_info,"dispose");
-    if (dispose != (char *) NULL)
-      next->dispose=(DisposeType)
-        ParseMagickOption(MagickDisposeOptions,MagickFalse,dispose);
-    comment=GetImageOption(image_info,"Comment");
-    if (comment != (char *) NULL)
-      {
-        (void) DeleteImageAttribute(next,"Comment");
-        (void) SetImageAttribute(next,"Comment",comment);
-      }
-    label=GetImageOption(image_info,"Label");
-    if (label != (char *) NULL)
-      {
-        (void) DeleteImageAttribute(next,"Label");
-        (void) SetImageAttribute(next,"Label",label);
-      }
     if (next->magick_columns == 0)
       next->magick_columns=next->columns;
     if (next->magick_rows == 0)
       next->magick_rows=next->rows;
-    if (image_info->verbose != MagickFalse)
+    if (read_info->verbose != MagickFalse)
       (void) IdentifyImage(next,stdout,MagickFalse);
   }
   read_info=DestroyImageInfo(read_info);
@@ -638,9 +657,6 @@ static Image *ReadImages(const ImageInfo *image_info,ExceptionInfo *exception)
   register long
     i;
 
-  size_t
-    length;
-
   /*
     Read image list from a file.
   */
@@ -650,7 +666,7 @@ static Image *ReadImages(const ImageInfo *image_info,ExceptionInfo *exception)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  command=(char *) FileToBlob(image_info->filename+1,&length,exception);
+  command=FileToString(image_info->filename+1,~0,exception);
   if (command == (char *) NULL)
     return((Image *) NULL);
   StripString(command);
@@ -681,7 +697,7 @@ static Image *ReadImages(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   R e a d I n l i n e I m a g e                                             %
++   R e a d I n l i n e I m a g e                                             %
 %                                                                             %
 %                                                                             %
 %                                                                             %
@@ -725,7 +741,6 @@ MagickExport Image *ReadInlineImage(const ImageInfo *image_info,
   register const char
     *p;
 
-  (void) SetExceptionInfo(exception,UndefinedException);
   image=NewImageList();
   for (p=content; (*p != ',') && (*p != '\0'); p++);
   if (*p == '\0')
@@ -833,7 +848,6 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
       image_info->filename);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  (void) SetExceptionInfo(&image->exception,UndefinedException);
   GetExceptionInfo(&sans_exception);
   write_info=CloneImageInfo(image_info);
   (void) CopyMagickString(write_info->filename,image->filename,MaxTextExtent);
@@ -853,8 +867,8 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
     }
   (void) CopyMagickString(filename,image->filename,MaxTextExtent);
   SyncImageInfo(write_info,image,&sans_exception);
-  if ((GetNextImageInList(image) == (Image *) NULL) &&
-      (GetPreviousImageInList(image) == (Image *) NULL) &&
+  if ((GetPreviousImageInList(image) == (Image *) NULL) &&
+      (GetNextImageInList(image) == (Image *) NULL) &&
       (write_info->page == (char *) NULL) &&
       (IsTaintImage(image) == MagickFalse))
     {
@@ -873,8 +887,8 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
             write_info->magick,&image->exception);
           write_info=DestroyImageInfo(write_info);
           (void) CopyMagickString(image->filename,filename,MaxTextExtent);
-          DestroyExceptionInfo(&sans_exception);
-          return((MagickBooleanType) (status == MagickFalse));
+          (void) DestroyExceptionInfo(&sans_exception);
+          return(status);
         }
     }
   status=MagickFalse;
@@ -911,7 +925,7 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
         Call appropriate image writer based on image type.
       */
       if (GetMagickThreadSupport(magick_info) == MagickFalse)
-        constitute_semaphore=AcquireSemaphoreInfo(constitute_semaphore);
+        AcquireSemaphoreInfo(&constitute_semaphore);
       status=GetMagickEncoder(magick_info)(write_info,image);
       if (GetMagickThreadSupport(magick_info) == MagickFalse)
         RelinquishSemaphoreInfo(constitute_semaphore);
@@ -938,10 +952,9 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
             {
               (void) CopyMagickString(write_info->magick,image->magick,
                 MaxTextExtent);
-              magick_info=(MagickInfo *) GetMagickInfo(write_info->magick,
-                &image->exception);
+              magick_info=GetMagickInfo(write_info->magick,&image->exception);
             }
-          if ((magick_info == (MagickInfo *) NULL) ||
+          if ((magick_info == (const MagickInfo *) NULL) ||
               (GetMagickEncoder(magick_info) == (EncoderHandler *) NULL))
             (void) ThrowMagickException(&image->exception,GetMagickModule(),
               MissingDelegateError,"NoEncodeDelegateForThisImageFormat","`%s'",
@@ -952,7 +965,7 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
                 Call appropriate image writer based on image type.
               */
               if (GetMagickThreadSupport(magick_info) == MagickFalse)
-                constitute_semaphore=AcquireSemaphoreInfo(constitute_semaphore);
+                AcquireSemaphoreInfo(&constitute_semaphore);
               status=GetMagickEncoder(magick_info)(write_info,image);
               if (GetMagickThreadSupport(magick_info) == MagickFalse)
                 RelinquishSemaphoreInfo(constitute_semaphore);
@@ -960,9 +973,16 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
         }
     }
   if (GetBlobError(image) != MagickFalse)
-    (void) ThrowMagickException(&image->exception,GetMagickModule(),
-      CorruptImageError,"AnErrorHasOccurredWritingToFile","`%s': %s",
-        image->filename,strerror(errno));
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      (void) ThrowMagickException(&image->exception,GetMagickModule(),
+        CorruptImageError,"AnErrorHasOccurredWritingToFile","`%s': %s",
+          image->filename,message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
   if (temporary == MagickTrue)
     {
       /*
@@ -976,8 +996,11 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
         MaxTextExtent);
       CloseBlob(image);
     }
+  if ((LocaleCompare(write_info->magick,"info") != 0) &&
+      (write_info->verbose != MagickFalse))
+    (void) IdentifyImage(image,stdout,MagickFalse);
   write_info=DestroyImageInfo(write_info);
-  DestroyExceptionInfo(&sans_exception);
+  (void) DestroyExceptionInfo(&sans_exception);
   return(status);
 }
 
@@ -1042,7 +1065,6 @@ MagickExport MagickBooleanType WriteImages(const ImageInfo *image_info,
   if (*write_info->magick == '\0')
     (void) CopyMagickString(write_info->magick,images->magick,MaxTextExtent);
   (void) SetImageInfo(write_info,MagickTrue,exception);
-  (void) SetExceptionInfo(exception,UndefinedException);
   p=images;
   for ( ; GetNextImageInList(p) != (Image *) NULL; p=GetNextImageInList(p))
     if (p->scene >= GetNextImageInList(p)->scene)
@@ -1062,11 +1084,7 @@ MagickExport MagickBooleanType WriteImages(const ImageInfo *image_info,
   for (p=images; p != (Image *) NULL; p=GetNextImageInList(p))
   {
     status&=WriteImage(write_info,p);
-    if (p->exception.severity >= ErrorException)
-      InheritException(exception,&p->exception);
     GetImageException(p,exception);
-    if (write_info->verbose != MagickFalse)
-      (void) IdentifyImage(p,stdout,MagickFalse);
     if (write_info->adjoin != MagickFalse)
       break;
   }
