@@ -16,7 +16,7 @@
 %                               October 1998                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -46,6 +46,7 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/attribute.h"
 #include "magick/blob.h"
 #include "magick/client.h"
 #include "magick/configure.h"
@@ -71,10 +72,67 @@
   Declare delegate map.
 */
 static const char
-  *DelegateMap = (char *)
+  *DelegateMap = (const char *)
     "<?xml version=\"1.0\"?>"
     "<delegatemap>"
-    "  <delegate stealth=\"True\" />"
+    "  <delegate decode=\"autotrace\" stealth=\"True\" command='\"autotrace\" -output-format svg -output-file \"%o\" \"%i\"' />"
+    "  <delegate decode=\"browse\" stealth=\"True\" spawn=\"True\" command='\"htmlview\" http://www.imagemagick.org/'  />"
+    "  <delegate decode=\"cgm\" command='\"ralcgm\" -d ps -oC < \"%i\" > \"%o\" 2>/dev/null' />"
+    "  <delegate decode=\"crw\" command='\"dcraw\" -3 -w -c \"%i\" > \"%o\"' />"
+    "  <delegate decode=\"dcr\" command='\"dcraw\" -3 -w -c \"%i\" > \"%o\"' />"
+    "  <delegate decode=\"mrw\" command='\"dcraw\" -3 -w -c \"%i\" > \"%o\"' />"
+    "  <delegate decode=\"nef\" command='\"dcraw\" -3 -w -c \"%i\" > \"%o\"' />"
+    "  <delegate decode=\"orf\" command='\"dcraw\" -3 -w -c \"%i\" > \"%o\"' />"
+    "  <delegate decode=\"raf\" command='\"dcraw\" -3 -w -c \"%i\" > \"%o\"' />"
+    "  <delegate decode=\"x3f\" command='\"dcraw\" -3 -w -c \"%i\" > \"%o\"' />"
+    "  <delegate decode=\"dvi\" command='\"dvips\" -q -o \"%o\" \"%i\"' />"
+    "  <delegate decode=\"edit\" stealth=\"True\" command='\"xterm\" -title \"Edit Image Comment\" -e vi \"%o\"' />"
+    "  <delegate decode=\"emf\" command='\"wmf2eps\" -o \"%o\" \"%i\"' />"
+    "  <delegate decode=\"eps\" encode=\"pdf\" mode=\"bi\" command='\"gs\" -q -dBATCH -dSAFER -dMaxBitmap=500000000 -dNOPAUSE -dAlignToPixels=0 -sDEVICE=\"pdfwrite\" -sOutputFile=\"%o\" -f\"%i\"' />"
+    "  <delegate decode=\"eps\" encode=\"ps\" mode=\"bi\" command='\"gs\" -q -dBATCH -dSAFER -dMaxBitmap=500000000 -dNOPAUSE -dAlignToPixels=0 -sDEVICE=\"pswrite\" -sOutputFile=\"%o\" -f\"%i\"' />"
+    "  <delegate decode=\"fig\" command='\"fig2dev\" -L ps \"%i\" \"%o\"' />"
+    "  <delegate decode=\"gplt\" command='\"echo\" \"set size 1.25,0.62"
+    "    set terminal postscript portrait color solid; set output \"%o\"; load \"%i\"\" > \"%u\";\"gnuplot\" \"%u\"' />"
+    "  <delegate decode=\"gs-color\" stealth=\"True\" command='\"gs\" -q -dBATCH -dSAFER -dMaxBitmap=500000000 -dNOPAUSE -dAlignToPixels=0 \"-sDEVICE=pnmraw\" -dTextAlphaBits=%u -dGraphicsAlphaBits=%u \"-g%s\" \"-r%s\" %s \"-sOutputFile=%s\" \"-f%s\" \"-f%s\"' />"
+    "  <delegate decode=\"gs-cmyk\" stealth=\"True\" command='\"gs\" -q -dBATCH -dSAFER -dMaxBitmap=500000000 -dNOPAUSE -dAlignToPixels=0 \"-sDEVICE=bmpsep8\" -dTextAlphaBits=%u -dGraphicsAlphaBits=%u \"-g%s\" \"-r%s\" %s \"-sOutputFile=%s\" \"-f%s\" \"-f%s\"' />"
+    "  <delegate decode=\"gs-mono\" stealth=\"True\" command='\"gs\" -q -dBATCH -dSAFER -dMaxBitmap=500000000 -dNOPAUSE -dAlignToPixels=0 \"-sDEVICE=pbmraw\" -dTextAlphaBits=%u -dGraphicsAlphaBits=%u \"-g%s\" \"-r%s\" %s \"-sOutputFile=%s\" \"-f%s\" \"-f%s\"' />"
+    "  <delegate decode=\"hpg\" command='\"hp2xx\" -q -m eps -f `basename \"%o\"` \"%i\""
+    "    mv -f `basename \"%o\"` \"%o\"' />"
+    "  <delegate decode=\"hpgl\" command='if [ -e hp2xx -o -e /usr/bin/hp2xx ]; then"
+    "    hp2xx -q -m eps -f `basename \"%o\"` \"%i\""
+    "    mv -f `basename \"%o\"` \"%o"
+    "  else"
+    "    echo \"You need to install hp2xx to use HPGL files with ImageMagick.\""
+    "    exit 1"
+    "  fi' />"
+    "  <delegate decode=\"htm\" command='\"html2ps\" -U -o \"%o\" \"%i\"' />"
+    "  <delegate decode=\"html\" command='\"html2ps\" -U -o \"%o\" \"%i\"' />"
+    "  <delegate decode=\"https\" command='\"@WWWDecodeDelegateDefault@\" -q -O \"%o\" \"https:%i\"' />"
+    "  <delegate decode=\"ilbm\" command='\"ilbmtoppm\" \"%i\" > \"%o\"' />"
+    "  <delegate decode=\"man\" command='\"groff\" -man -Tps \"%i\" > \"%o\"' />"
+    "  <delegate decode=\"mpeg-decode\" stealth=\"True\" command='\"mpeg2decode\" -q -b \"%i\" -f -r -o3 \"%u%%d\"' />"
+    "  <delegate encode=\"mpeg-encode\" stealth=\"True\" command='\"mpeg2encode\" \"%i\" \"%o\"' />"
+    "  <delegate decode=\"pcl-color\" stealth=\"True\" command='\"pcl6\" -dBATCH -dSAFER -dMaxBitmap=500000000 -dNOPAUSE -dAlignToPixels=0 \"-sDEVICE=ppmraw\" -dTextAlphaBits=%u -dGraphicsAlphaBits=%u \"-g%s\" \"-r%s\" %s \"-sOutputFile=%s\" \"%s\"' />"
+    "  <delegate decode=\"pcl-cmyk\" stealth=\"True\" command='\"pcl6\" -dBATCH -dSAFER -dMaxBitmap=500000000 -dNOPAUSE -dAlignToPixels=0 \"-sDEVICE=@PCLCMYKDevice@\" -dTextAlphaBits=%u -dGraphicsAlphaBits=%u \"-g%s\" \"-r%s\" %s \"-sOutputFile=%s\" \"%s\"' />"
+    "  <delegate decode=\"pcl-mono\" stealth=\"True\" command='\"pcl6\" -dBATCH -dSAFER -dMaxBitmap=500000000 -dNOPAUSE -dAlignToPixels=0 \"-sDEVICE=pbmraw\" -dTextAlphaBits=%u -dGraphicsAlphaBits=%u \"-g%s\" \"-r%s\" %s \"-sOutputFile=%s\" \"%s\"' />"
+    "  <delegate decode=\"pdf\" encode=\"eps\" mode=\"bi\" command='\"gs\" -q -dBATCH -dSAFER -dMaxBitmap=500000000 -dNOPAUSE -dAlignToPixels=0 -sDEVICE=\"epswrite\" -sOutputFile=\"%o\" -f\"%i\"' />"
+    "  <delegate decode=\"pdf\" encode=\"ps\" mode=\"bi\" command='\"gs\" -q -dBATCH -dSAFER -dMaxBitmap=500000000 -dNOPAUSE -dAlignToPixels=0 -sDEVICE=\"pswrite\" -sOutputFile=\"%o\" -f\"%i\"' />"
+    "  <delegate decode=\"pnm\" encode=\"ilbm\" mode=\"encode\" command='\"ppmtoilbm\" -24if \"%i\" > \"%o\"' />"
+    "  <delegate decode=\"pnm\" encode=\"launch\" mode=\"encode\" command='\"gimp\" \"%i\"' />"
+    "  <delegate decode=\"miff\" encode=\"win\" mode=\"encode\" command='\"display\" -immutable \"%i\"' />"
+    "  <delegate decode=\"pov\" command='\"povray\" \"+i\"%i\"\" +o\"%o\" +fn%q +w%w +h%h +a -q9 -kfi\"%s\" -kff\"%n\""
+    "    \"convert\" -concatenate \"%o*.png\" \"%o\"' />"
+    "  <delegate decode=\"ps\" encode=\"eps\" mode=\"bi\" command='\"gs\" -q -dBATCH -dSAFER -dMaxBitmap=500000000 -dNOPAUSE -dAlignToPixels=0 -sDEVICE=\"epswrite\" -sOutputFile=\"%o\" -f\"%i\"' />"
+    "  <delegate decode=\"ps\" encode=\"pdf\" mode=\"bi\" command='\"gs\" -q -dBATCH -dSAFER -dMaxBitmap=500000000 -dNOPAUSE -dAlignToPixels=0 -sDEVICE=\"pdfwrite\" -sOutputFile=\"%o\" -f\"%i\"' />"
+    "  <delegate decode=\"ps\" encode=\"print\" mode=\"encode\" command='lpr \"%i\"' />"
+    "  <delegate decode=\"rad\" command='\"ra_ppm\" -g 1.0 \"%i\" \"%o\"' />"
+    "  <delegate decode=\"rgba\" encode=\"rle\" mode=\"encode\" command='\"modify\" -flip -size %wx%h \"rgba:%i\""
+    "    \"rawtorle\" -w %w -h %h -n 4 -o \"%o\" \"%i\"' />"
+    "  <delegate decode=\"scan\" command='\"scanimage\" -d \"%i\" > \"%o\"' />"
+    "  <delegate decode=\"shtml\" command='\"html2ps\" -U -o \"%o\" \"%i\"' />"
+    "  <delegate decode=\"txt\" encode=\"ps\" mode=\"bi\" command='\"enscript\" -o \"%o\" \"%i\"' />"
+    "  <delegate decode=\"wmf\" command='\"wmf2eps\" -o \"%o\" \"%i\"' />"
+    "  <delegate encode=\"show\" stealth=\"True\" spawn=\"True\" command='\"display\" -immutable -delay 0 -window_group %g -title \"%l of %f\" \"tmp:%i\"' />"
     "</delegatemap>";
 
 /*
@@ -136,7 +194,7 @@ static void *DestroyDelegate(void *delegate_info)
 
 MagickExport void DestroyDelegateList(void)
 {
-  delegate_semaphore=AcquireSemaphoreInfo(delegate_semaphore);
+  AcquireSemaphoreInfo(&delegate_semaphore);
   if (delegate_list != (LinkedListInfo *) NULL)
     delegate_list=DestroyLinkedList(delegate_list,DestroyDelegate);
   instantiate_delegate=MagickFalse;
@@ -216,7 +274,7 @@ MagickExport char *GetDelegateCommand(const ImageInfo *image_info,Image *image,
         decode ? decode : encode);
       return((char *) NULL);
     }
-  command=TranslateText(image_info,image,commands[0]);
+  command=InterpretImageAttributes(image_info,image,commands[0]);
   if (command == (char *) NULL)
     (void) ThrowMagickException(exception,GetMagickModule(),ResourceLimitError,
       "MemoryAllocationFailed","`%s'",commands[0]);
@@ -308,7 +366,7 @@ MagickExport const DelegateInfo *GetDelegateInfo(const char *decode,
   /*
     Search for named delegate.
   */
-  delegate_semaphore=AcquireSemaphoreInfo(delegate_semaphore);
+  AcquireSemaphoreInfo(&delegate_semaphore);
   ResetLinkedListIterator(delegate_list);
   p=(const DelegateInfo *) GetNextValueInLinkedList(delegate_list);
   while (p != (const DelegateInfo *) NULL)
@@ -429,7 +487,7 @@ MagickExport const DelegateInfo **GetDelegateInfoList(const char *pattern,
   /*
     Generate delegate list.
   */
-  delegate_semaphore=AcquireSemaphoreInfo(delegate_semaphore);
+  AcquireSemaphoreInfo(&delegate_semaphore);
   ResetLinkedListIterator(delegate_list);
   p=(const DelegateInfo *) GetNextValueInLinkedList(delegate_list);
   for (i=0; p != (const DelegateInfo *) NULL; )
@@ -483,12 +541,12 @@ extern "C" {
 
 static int DelegateCompare(const void *x,const void *y)
 {
-  register char
+  register const char
     **p,
     **q;
 
-  p=(char **) x;
-  q=(char **) y;
+  p=(const char **) x;
+  q=(const char **) y;
   return(LocaleCompare(*p,*q));
 }
 
@@ -522,17 +580,17 @@ MagickExport char **GetDelegateList(const char *pattern,
     (GetNumberOfElementsInLinkedList(delegate_list)+1)*sizeof(*delegates));
   if (delegates == (char **) NULL)
     return((char **) NULL);
-  delegate_semaphore=AcquireSemaphoreInfo(delegate_semaphore);
+  AcquireSemaphoreInfo(&delegate_semaphore);
   ResetLinkedListIterator(delegate_list);
   p=(const DelegateInfo *) GetNextValueInLinkedList(delegate_list);
   for (i=0; p != (const DelegateInfo *) NULL; )
   {
     if ((p->stealth == MagickFalse) &&
         (GlobExpression(p->decode,pattern) != MagickFalse))
-      delegates[i++]=ConstantString(AcquireString(p->decode));
+      delegates[i++]=ConstantString(p->decode);
     if ((p->stealth == MagickFalse) &&
         (GlobExpression(p->encode,pattern) != MagickFalse))
-      delegates[i++]=ConstantString(AcquireString(p->encode));
+      delegates[i++]=ConstantString(p->encode);
     p=(const DelegateInfo *) GetNextValueInLinkedList(delegate_list);
   }
   RelinquishSemaphoreInfo(delegate_semaphore);
@@ -599,7 +657,7 @@ static MagickBooleanType InitializeDelegateList(ExceptionInfo *exception)
   if ((delegate_list == (LinkedListInfo *) NULL) &&
       (instantiate_delegate == MagickFalse))
     {
-      delegate_semaphore=AcquireSemaphoreInfo(delegate_semaphore);
+      AcquireSemaphoreInfo(&delegate_semaphore);
       if ((delegate_list == (LinkedListInfo *) NULL) &&
           (instantiate_delegate == MagickFalse))
         {
@@ -647,7 +705,8 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
   char
     *command,
     **commands,
-    filename[MaxTextExtent];
+    input_filename[MaxTextExtent],
+    output_filename[MaxTextExtent];
 
   const DelegateInfo
     *delegate_info;
@@ -668,7 +727,7 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  temporary=(MagickBooleanType) (*image->filename == '\0');
+  temporary=(*image->filename == '\0') ? MagickTrue : MagickFalse;
   if (temporary != MagickFalse)
     if (AcquireUniqueFilename(image->filename) == MagickFalse)
       {
@@ -676,7 +735,6 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
           "UnableToCreateTemporaryFile",image->filename);
         return(MagickFalse);
       }
-  (void) CopyMagickString(filename,image->filename,MaxTextExtent);
   delegate_info=GetDelegateInfo(decode,encode,exception);
   if (delegate_info == (DelegateInfo *) NULL)
     {
@@ -698,89 +756,88 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
         }
       image_info->temporary=MagickTrue;
     }
-  if (delegate_info->mode != 0)
-    if (((decode != (const char *) NULL) &&
-         (delegate_info->encode != (char *) NULL)) ||
-        ((encode != (const char *) NULL) &&
-         (delegate_info->decode != (char *) NULL)))
+  if ((delegate_info->mode != 0) &&
+      (((decode != (const char *) NULL) &&
+       (delegate_info->encode != (char *) NULL)) ||
+      ((encode != (const char *) NULL) &&
+       (delegate_info->decode != (char *) NULL))))
+    {
+      char
+        *magick;
+
+      ImageInfo
+        *clone_info;
+
+      register Image
+        *p;
+
+      /*
+        Delegate requires a particular image format.
+      */
+      if (AcquireUniqueFilename(image_info->unique) == MagickFalse)
+        {
+          ThrowFileException(exception,FileOpenError,
+            "UnableToCreateTemporaryFile",image_info->unique);
+          return(MagickFalse);
+        }
+      if (AcquireUniqueFilename(image_info->zero) == MagickFalse)
+        {
+          (void) RelinquishUniqueFileResource(image_info->zero);
+          ThrowFileException(exception,FileOpenError,
+            "UnableToCreateTemporaryFile",image_info->zero);
+          return(MagickFalse);
+        }
+      magick=InterpretImageAttributes(image_info,image,decode != (char *) NULL ?
+        delegate_info->encode : delegate_info->decode);
+      if (magick == (char *) NULL)
+        {
+          (void) RelinquishUniqueFileResource(image_info->unique);
+          (void) RelinquishUniqueFileResource(image_info->zero);
+          if (temporary != MagickFalse)
+            (void) RelinquishUniqueFileResource(image->filename);
+          (void) ThrowMagickException(exception,GetMagickModule(),
+            DelegateError,"DelegateFailed","`%s'",decode ? decode : encode);
+          return(MagickFalse);
+        }
+      LocaleUpper(magick);
+      clone_info=CloneImageInfo(image_info);
+      (void) CopyMagickString((char *) clone_info->magick,magick,
+        MaxTextExtent);
+      (void) CopyMagickString(image->magick,magick,MaxTextExtent);
+      magick=(char *) RelinquishMagickMemory(magick);
+      (void) FormatMagickString(clone_info->filename,MaxTextExtent,"%s:",
+        delegate_info->decode);
+      (void) SetImageInfo(clone_info,MagickTrue,exception);
+      (void) CopyMagickString(clone_info->filename,image_info->filename,
+        MaxTextExtent);
+      (void) CopyMagickString(image_info->filename,image->filename,
+        MaxTextExtent);
+      for (p=image; p != (Image *) NULL; p=GetNextImageInList(p))
       {
-        char
-          filename[MaxTextExtent],
-          *magick;
-
-        ImageInfo
-          *clone_info;
-
-        register Image
-          *p;
-
-        /*
-          Delegate requires a particular image format.
-        */
-        if (AcquireUniqueFilename(image_info->unique) == MagickFalse)
-          {
-            ThrowFileException(exception,FileOpenError,
-              "UnableToCreateTemporaryFile",image_info->unique);
-            return(MagickFalse);
-          }
-        if (AcquireUniqueFilename(image_info->zero) == MagickFalse)
-          {
-            (void) RelinquishUniqueFileResource(image_info->zero);
-            ThrowFileException(exception,FileOpenError,
-              "UnableToCreateTemporaryFile",image_info->zero);
-            return(MagickFalse);
-          }
-        magick=TranslateText(image_info,image,decode != (char *) NULL ?
-          delegate_info->encode : delegate_info->decode);
-        if (magick == (char *) NULL)
+        (void) FormatMagickString(p->filename,MaxTextExtent,"%s:%s",
+          delegate_info->decode,clone_info->filename);
+        status=WriteImage(clone_info,p);
+        if (status == MagickFalse)
           {
             (void) RelinquishUniqueFileResource(image_info->unique);
             (void) RelinquishUniqueFileResource(image_info->zero);
             if (temporary != MagickFalse)
               (void) RelinquishUniqueFileResource(image->filename);
+            clone_info=DestroyImageInfo(clone_info);
             (void) ThrowMagickException(exception,GetMagickModule(),
               DelegateError,"DelegateFailed","`%s'",decode ? decode : encode);
             return(MagickFalse);
           }
-        LocaleUpper(magick);
-        clone_info=CloneImageInfo(image_info);
-        (void) CopyMagickString((char *) clone_info->magick,magick,
-          MaxTextExtent);
-        (void) CopyMagickString(image->magick,magick,MaxTextExtent);
-        magick=(char *) RelinquishMagickMemory(magick);
-        (void) CopyMagickString(filename,image->filename,MaxTextExtent);
-        (void) FormatMagickString(clone_info->filename,MaxTextExtent,"%s:",
-          delegate_info->decode);
-        (void) SetImageInfo(clone_info,MagickTrue,exception);
-        (void) CopyMagickString(clone_info->filename,image_info->filename,
-          MaxTextExtent);
-        for (p=image; p != (Image *) NULL; p=GetNextImageInList(p))
-        {
-          (void) FormatMagickString(p->filename,MaxTextExtent,"%s:%s",
-            delegate_info->decode,filename);
-          status=WriteImage(clone_info,p);
-          if (status == MagickFalse)
-            {
-              (void) RelinquishUniqueFileResource(image_info->unique);
-              (void) RelinquishUniqueFileResource(image_info->zero);
-              if (temporary != MagickFalse)
-                (void) RelinquishUniqueFileResource(image->filename);
-              clone_info=DestroyImageInfo(clone_info);
-              (void) ThrowMagickException(exception,GetMagickModule(),
-                DelegateError,"DelegateFailed","`%s'",decode ? decode : encode);
-              return(MagickFalse);
-            }
-          if (clone_info->adjoin != MagickFalse)
-            break;
-        }
-        (void) RelinquishUniqueFileResource(image_info->unique);
-        (void) RelinquishUniqueFileResource(image_info->zero);
-        clone_info=DestroyImageInfo(clone_info);
+        if (clone_info->adjoin != MagickFalse)
+          break;
       }
+      (void) RelinquishUniqueFileResource(image_info->unique);
+      (void) RelinquishUniqueFileResource(image_info->zero);
+      clone_info=DestroyImageInfo(clone_info);
+    }
   /*
     Invoke delegate.
   */
-  (void) CopyMagickString(image->filename,filename,MaxTextExtent);
   commands=StringToList(delegate_info->commands);
   if (commands == (char **) NULL)
     {
@@ -793,23 +850,38 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
     }
   command=(char *) NULL;
   status=MagickFalse;
+  (void) CopyMagickString(output_filename,image_info->filename,MaxTextExtent);
+  (void) CopyMagickString(input_filename,image->filename,MaxTextExtent);
   for (i=0; commands[i] != (char *) NULL; i++)
   {
-    status=MagickFalse;
+    status=AcquireUniqueSymbolicLink(output_filename,image_info->filename);
+    if (status == MagickFalse)
+      {
+        ThrowFileException(exception,FileOpenError,
+          "UnableToCreateTemporaryFile",output_filename);
+        break;
+      }
     if (AcquireUniqueFilename(image_info->unique) == MagickFalse)
       {
         ThrowFileException(exception,FileOpenError,
           "UnableToCreateTemporaryFile",image_info->unique);
-        return(MagickFalse);
+        break;
       }
     if (AcquireUniqueFilename(image_info->zero) == MagickFalse)
       {
         (void) RelinquishUniqueFileResource(image_info->unique);
         ThrowFileException(exception,FileOpenError,
           "UnableToCreateTemporaryFile",image_info->zero);
-        return(MagickFalse);
+        break;
       }
-    command=TranslateText(image_info,image,commands[i]);
+    status=AcquireUniqueSymbolicLink(input_filename,image->filename);
+    if (status == MagickFalse)
+      {
+        ThrowFileException(exception,FileOpenError,
+          "UnableToCreateTemporaryFile",input_filename);
+        break;
+      }
+    command=InterpretImageAttributes(image_info,image,commands[i]);
     if (command == (char *) NULL)
       break;
     /*
@@ -817,10 +889,13 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
     */
     if (delegate_info->spawn != MagickFalse)
       (void) ConcatenateString(&command," &");
-    status=(MagickBooleanType) SystemCommand(image_info->verbose,command);
+    status=SystemCommand(image_info->verbose,command) != 0 ?  MagickTrue :
+      MagickFalse;
     command=(char *) RelinquishMagickMemory(command);
+    (void) RelinquishUniqueFileResource(image->filename);
     (void) RelinquishUniqueFileResource(image_info->unique);
     (void) RelinquishUniqueFileResource(image_info->zero);
+    (void) RelinquishUniqueFileResource(image_info->filename);
     if (status != MagickFalse)
       {
         (void) ThrowMagickException(exception,GetMagickModule(),DelegateError,
@@ -829,6 +904,8 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
       }
     commands[i]=(char *) RelinquishMagickMemory(commands[i]);
   }
+  (void) CopyMagickString(image_info->filename,output_filename,MaxTextExtent);
+  (void) CopyMagickString(image->filename,input_filename,MaxTextExtent);
   /*
     Free resources.
   */
@@ -837,7 +914,7 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
   commands=(char **) RelinquishMagickMemory(commands);
   if (temporary != MagickFalse)
     (void) RelinquishUniqueFileResource(image->filename);
-  return((MagickBooleanType) (status == MagickFalse));
+  return(status == MagickFalse ? MagickTrue : MagickFalse);
 }
 
 /*
@@ -934,7 +1011,7 @@ MagickExport MagickBooleanType InvokePostscriptDelegate(
     }
   return(MagickTrue);
 #else
-  return((MagickBooleanType) SystemCommand(verbose,command));
+  return(SystemCommand(verbose,command) != 0 ? MagickTrue : MagickFalse);
 #endif
 }
 
@@ -1024,7 +1101,8 @@ MagickExport MagickBooleanType ListDelegateInfo(FILE *file,
       (void) fprintf(file,"                     %s\n",commands[j]);
     }
     for (j=0; commands[j] != (char *) NULL; j++)
-      commands[i]=(char *) RelinquishMagickMemory(commands[j]);
+      commands[j]=(char *) RelinquishMagickMemory(commands[j]);
+    commands=(char **) RelinquishMagickMemory(commands);
   }
   (void) fflush(file);
   delegate_info=(const DelegateInfo **)
@@ -1061,7 +1139,6 @@ MagickExport MagickBooleanType ListDelegateInfo(FILE *file,
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 static MagickBooleanType LoadDelegateList(const char *xml,const char *filename,
   const unsigned long depth,ExceptionInfo *exception)
@@ -1095,7 +1172,7 @@ static MagickBooleanType LoadDelegateList(const char *xml,const char *filename,
         }
     }
   status=MagickTrue;
-  token=AcquireString((char *) xml);
+  token=AcquireString(xml);
   for (q=(char *) xml; *q != '\0'; )
   {
     /*
@@ -1105,6 +1182,15 @@ static MagickBooleanType LoadDelegateList(const char *xml,const char *filename,
     if (*token == '\0')
       break;
     (void) CopyMagickString(keyword,token,MaxTextExtent);
+    if (LocaleNCompare(keyword,"<!DOCTYPE",9) == 0)
+      {
+        /*
+          Doctype element.
+        */
+        while ((LocaleNCompare(q,"]>",2) != 0) && (*q != '\0'))
+          GetMagickToken(q,&q,token);
+        continue;
+      }
     if (LocaleNCompare(keyword,"<!--",4) == 0)
       {
         /*
@@ -1142,7 +1228,7 @@ static MagickBooleanType LoadDelegateList(const char *xml,const char *filename,
                     (void) ConcatenateMagickString(path,DirectorySeparator,
                       MaxTextExtent);
                   (void) ConcatenateMagickString(path,token,MaxTextExtent);
-                  xml=FileToString(path);
+                  xml=FileToString(path,~0,exception);
                   status|=LoadDelegateList(xml,path,depth+1,exception);
                   xml=(char *) RelinquishMagickMemory(xml);
                 }
@@ -1161,7 +1247,7 @@ static MagickBooleanType LoadDelegateList(const char *xml,const char *filename,
           ThrowMagickFatalException(ResourceLimitFatalError,
             "MemoryAllocationFailed",filename);
         (void) ResetMagickMemory(delegate_info,0,sizeof(*delegate_info));
-        delegate_info->path=ConstantString(AcquireString(filename));
+        delegate_info->path=ConstantString(filename);
         delegate_info->signature=MagickSignature;
         continue;
       }
@@ -1188,18 +1274,21 @@ static MagickBooleanType LoadDelegateList(const char *xml,const char *filename,
       {
         if (LocaleCompare((char *) keyword,"command") == 0)
           {
-            delegate_info->commands=ConstantString(AcquireString(token));
+            char
+              *commands;
+
+            commands=AcquireString(token);
 #if defined(__WINDOWS__)
-            if (strchr(delegate_info->commands,'@') != (char *) NULL)
+            if (strchr(commands,'@') != (char *) NULL)
               {
                 char
                   path[MaxTextExtent];
 
                 NTGhostscriptEXE(path,MaxTextExtent);
-                SubstituteString((char **) &delegate_info->commands,
-                  "@PSDelegate@",path);
+                SubstituteString((char **) &commands,"@PSDelegate@",path);
               }
 #endif
+            delegate_info->commands=commands;
             break;
           }
         break;
@@ -1209,7 +1298,7 @@ static MagickBooleanType LoadDelegateList(const char *xml,const char *filename,
       {
         if (LocaleCompare((char *) keyword,"decode") == 0)
           {
-            delegate_info->decode=ConstantString(AcquireString(token));
+            delegate_info->decode=ConstantString(token);
             delegate_info->mode=1;
             break;
           }
@@ -1220,7 +1309,7 @@ static MagickBooleanType LoadDelegateList(const char *xml,const char *filename,
       {
         if (LocaleCompare((char *) keyword,"encode") == 0)
           {
-            delegate_info->encode=ConstantString(AcquireString(token));
+            delegate_info->encode=ConstantString(token);
             delegate_info->mode=(-1);
             break;
           }
@@ -1246,14 +1335,14 @@ static MagickBooleanType LoadDelegateList(const char *xml,const char *filename,
       {
         if (LocaleCompare((char *) keyword,"spawn") == 0)
           {
-            delegate_info->spawn=(MagickBooleanType)
-              (LocaleCompare(token,"True") == 0);
+            delegate_info->spawn=LocaleCompare(token,"True") == 0 ? MagickTrue :
+              MagickFalse;
             break;
           }
         if (LocaleCompare((char *) keyword,"stealth") == 0)
           {
-            delegate_info->stealth=(MagickBooleanType)
-              (LocaleCompare(token,"True") == 0);
+            delegate_info->stealth=LocaleCompare(token,"True") == 0 ?
+              MagickTrue : MagickFalse;
             break;
           }
         break;
@@ -1298,9 +1387,9 @@ static MagickBooleanType LoadDelegateList(const char *xml,const char *filename,
 static MagickBooleanType LoadDelegateLists(const char *filename,
   ExceptionInfo *exception)
 {
-  char
-    path[MaxTextExtent];
-
+#if defined(UseEmbeddableMagick)
+  return(LoadDelegateList(DelegateMap,"built-in",0,exception));
+#else
   const StringInfo
     *option;
 
@@ -1311,12 +1400,10 @@ static MagickBooleanType LoadDelegateLists(const char *filename,
     status;
 
   status=MagickFalse;
-  *path='\0';
   options=GetConfigureOptions(filename,exception);
   option=(const StringInfo *) GetNextValueInLinkedList(options);
   while (option != (const StringInfo *) NULL)
   {
-    (void) CopyMagickString(path,option->path,MaxTextExtent);
     status|=LoadDelegateList((const char *) option->datum,option->path,0,
       exception);
     option=(const StringInfo *) GetNextValueInLinkedList(options);
@@ -1324,10 +1411,7 @@ static MagickBooleanType LoadDelegateLists(const char *filename,
   options=DestroyConfigureOptions(options);
   if ((delegate_list == (LinkedListInfo *) NULL) || 
       (IsLinkedListEmpty(delegate_list) != MagickFalse))
-    {
-      (void) ThrowMagickException(exception,GetMagickModule(),ConfigureWarning,
-        "UnableToOpenConfigureFile","`%s'",path);
-      status|=LoadDelegateList(DelegateMap,"built-in",0,exception);
-    }
+    status|=LoadDelegateList(DelegateMap,"built-in",0,exception);
   return(status != 0 ? MagickTrue : MagickFalse);
+#endif
 }
