@@ -44,6 +44,7 @@
 #include "magick/client.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
+#include "magick/hashmap.h"
 #include "magick/locale_.h"
 #include "magick/log.h"
 #include "magick/magick.h"
@@ -84,6 +85,107 @@ static WarningHandler
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   A c q u i r e E x c e p t i o n I n f o                                   %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  AcquireExceptionInfo() allocates the ExceptionInfo structure.
+%
+%  The format of the AcquireExceptionInfo method is:
+%
+%      ExceptionInfo *AcquireExceptionInfo(void)
+%
+*/
+MagickExport ExceptionInfo *AcquireExceptionInfo(void)
+{
+  ExceptionInfo
+    *exception;
+
+  exception=(ExceptionInfo *) AcquireMagickMemory(sizeof(*exception));
+  if (exception == (ExceptionInfo *) NULL)
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowMagickFatalException(ResourceLimitFatalError,
+        "MemoryAllocationFailed",message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
+  (void) ResetMagickMemory(exception,0,sizeof(*exception));
+  GetExceptionInfo(exception);
+  exception->relinquish=MagickTrue;
+  exception->signature=MagickSignature;
+  return(exception);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   C l e a r M a g i c k E x c e p t i o n                                   %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ClearMagickException() clears any exception that may not have been caught
+%  yet.
+%
+%  The format of the ClearMagickException method is:
+%
+%      ClearMagickException(ExceptionInfo *exception)
+%
+%  A description of each parameter follows:
+%
+%    o exception: The exception info.
+%
+*/
+
+static void *DestroyExceptionElement(void *exception)
+{
+  register ExceptionInfo
+    *p;
+
+  p=(ExceptionInfo *) exception;
+  if (p->reason != (char *) NULL)
+    p->reason=(char *) RelinquishMagickMemory(p->reason);
+  if (p->description != (char *) NULL)
+    p->description=(char *) RelinquishMagickMemory(p->description);
+  p=(ExceptionInfo *) RelinquishMagickMemory(p);
+  return((void *) NULL);
+}
+
+MagickExport void ClearMagickException(ExceptionInfo *exception)
+{
+  register ExceptionInfo
+    *p;
+
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickSignature);
+  AcquireSemaphoreInfo(&exception->semaphore);
+  p=(ExceptionInfo *) RemoveLastElementFromLinkedList((LinkedListInfo *)
+    exception->exceptions);
+  while (p != (ExceptionInfo *) NULL)
+  {
+    DestroyExceptionElement(p);
+    p=(ExceptionInfo *) RemoveLastElementFromLinkedList((LinkedListInfo *)
+      exception->exceptions);
+	}
+  exception->severity=UndefinedException;
+  exception->reason=(char *) NULL;
+  exception->description=(char *) NULL;
+  RelinquishSemaphoreInfo(exception->semaphore);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   C a t c h E x c e p t i o n                                               %
 %                                                                             %
 %                                                                             %
@@ -105,32 +207,28 @@ static WarningHandler
 */
 MagickExport void CatchException(ExceptionInfo *exception)
 {
+  register const ExceptionInfo
+    *p;
+
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  if (exception->severity == UndefinedException)
-    return;
-  if ((exception->severity >= WarningException) &&
-      (exception->severity < ErrorException))
-    {
-      MagickWarning(exception->severity,exception->reason,
-        exception->description);
-      (void) SetExceptionInfo(exception,UndefinedException);
-      return;
-    }
-  if ((exception->severity >= ErrorException) &&
-      (exception->severity < FatalErrorException))
-    {
-      MagickError(exception->severity,exception->reason,exception->description);
-      (void) SetExceptionInfo(exception,UndefinedException);
-      return;
-    }
-  if (exception->severity >= FatalErrorException)
-    {
-      MagickFatalError(exception->severity,exception->reason,
-        exception->description);
-      (void) SetExceptionInfo(exception,UndefinedException);
-      return;
-    }
+  AcquireSemaphoreInfo(&exception->semaphore);
+  ResetLinkedListIterator((LinkedListInfo *) exception->exceptions);
+  p=(const ExceptionInfo *) GetNextValueInLinkedList((LinkedListInfo *)
+    exception->exceptions);
+  while (p != (const ExceptionInfo *) NULL)
+  {
+    if ((p->severity >= WarningException) && (p->severity < ErrorException))
+      MagickWarning(p->severity,p->reason,p->description);
+    if ((p->severity >= ErrorException) && (p->severity < FatalErrorException))
+      MagickError(p->severity,p->reason,p->description);
+    if (exception->severity >= FatalErrorException)
+      MagickFatalError(p->severity,p->reason,p->description);
+    p=(const ExceptionInfo *) GetNextValueInLinkedList((LinkedListInfo *)
+      exception->exceptions);
+  }
+  RelinquishSemaphoreInfo(exception->semaphore);
+	ClearMagickException(exception);
 }
 
 /*
@@ -162,8 +260,8 @@ MagickExport void CatchException(ExceptionInfo *exception)
 %
 %
 */
-static void DefaultErrorHandler(const ExceptionType severity,const char *reason,
-  const char *description)
+static void DefaultErrorHandler(const ExceptionType magick_unused(severity),
+  const char *reason,const char *description)
 {
   if (reason == (char *) NULL)
     return;
@@ -215,7 +313,7 @@ static void DefaultFatalErrorHandler(const ExceptionType severity,
   (void) fprintf(stderr,".\n");
   (void) fflush(stderr);
   DestroyMagick();
-  Exit(severity);
+  exit(severity);
 }
 
 /*
@@ -247,7 +345,7 @@ static void DefaultFatalErrorHandler(const ExceptionType severity,
 %
 %
 */
-static void DefaultWarningHandler(const ExceptionType severity,
+static void DefaultWarningHandler(const ExceptionType magick_unused(severity),
   const char *reason,const char *description)
 {
   if (reason == (char *) NULL)
@@ -274,25 +372,28 @@ static void DefaultWarningHandler(const ExceptionType severity,
 %
 %  The format of the DestroyExceptionInfo method is:
 %
-%      void DestroyExceptionInfo(ExceptionInfo *exception)
+%      ExceptionInfo *DestroyExceptionInfo(ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
 %    o exception: The exception info.
 %
-%
 */
-MagickExport void DestroyExceptionInfo(ExceptionInfo *exception)
+MagickExport ExceptionInfo *DestroyExceptionInfo(ExceptionInfo *exception)
 {
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  if (exception->reason != (char *) NULL)
-    exception->reason=(char *) RelinquishMagickMemory(exception->reason);
-  if (exception->description != (char *) NULL)
-    exception->description=(char *)
-      RelinquishMagickMemory(exception->description);
+  AcquireSemaphoreInfo(&exception->semaphore);
   exception->severity=UndefinedException;
+  if (exception->exceptions != (void *) NULL)
+    exception->exceptions=(void *) DestroyLinkedList((LinkedListInfo *)
+      exception->exceptions,DestroyExceptionElement);
   exception->signature=(~MagickSignature);
+  RelinquishSemaphoreInfo(exception->semaphore);
+  exception->semaphore=DestroySemaphoreInfo(exception->semaphore);
+  if (exception->relinquish != MagickFalse)
+    exception=(ExceptionInfo *) RelinquishMagickMemory(exception);
+  return(exception);
 }
 
 /*
@@ -322,7 +423,8 @@ MagickExport void GetExceptionInfo(ExceptionInfo *exception)
 {
   assert(exception != (ExceptionInfo *) NULL);
   (void) ResetMagickMemory(exception,0,sizeof(*exception));
-  (void) SetExceptionInfo(exception,UndefinedException);
+  exception->severity=UndefinedException;
+  exception->exceptions=(void *) NewLinkedList(0);
   exception->signature=MagickSignature;
 }
 
@@ -331,6 +433,34 @@ MagickExport void GetExceptionInfo(ExceptionInfo *exception)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   G e t E x c e p t i o n M e s s a g e                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetExceptionMessage() returns the error message defined by the specified
+%  error code.
+%
+%  The format of the GetExceptionMessage method is:
+%
+%      char *GetExceptionMessage(const int error_code)
+%
+%  A description of each parameter follows:
+%
+%    o error_code: the error code.
+%
+*/
+MagickExport char *GetExceptionMessage(const int error_code)
+{
+  return(ConstantString(strerror(error_code)));
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   G e t L o c a l e E x c e p t i o n M e s s a g e                         %
 %                                                                             %
 %                                                                             %
@@ -351,8 +481,6 @@ MagickExport void GetExceptionInfo(ExceptionInfo *exception)
 %
 %    o tag: the message tag.
 %
-%
-%
 */
 
 static const char *ExceptionSeverityToTag(const ExceptionType severity)
@@ -469,22 +597,24 @@ MagickExport const char *GetLocaleExceptionMessage(const ExceptionType severity,
 MagickExport void InheritException(ExceptionInfo *exception,
   const ExceptionInfo *relative)
 {
+  register const ExceptionInfo
+    *p;
+
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   assert(relative != (ExceptionInfo *) NULL);
   assert(relative->signature == MagickSignature);
-  if (relative->severity < exception->severity)
-    return;
-  exception->severity=relative->severity;
-  if (exception->reason != (char *) NULL)
-    exception->reason=(char *) RelinquishMagickMemory(exception->reason);
-  if (relative->reason != (char *) NULL)
-    (void) CloneString(&exception->reason,relative->reason);
-  if (exception->description != (char *) NULL)
-    exception->description=(char *)
-      RelinquishMagickMemory(exception->description);
-  if (relative->description != (char *) NULL)
-    (void) CloneString(&exception->description,relative->description);
+  AcquireSemaphoreInfo(&exception->semaphore);
+  ResetLinkedListIterator((LinkedListInfo *) relative->exceptions);
+  p=(const ExceptionInfo *) GetNextValueInLinkedList((LinkedListInfo *)
+    relative->exceptions);
+  while (p != (const ExceptionInfo *) NULL)
+  {
+    ThrowException(exception,p->severity,p->reason,p->description);
+    p=(const ExceptionInfo *) GetNextValueInLinkedList((LinkedListInfo *)
+      relative->exceptions);
+  }
+  RelinquishSemaphoreInfo(exception->semaphore);
 }
 
 /*
@@ -634,40 +764,6 @@ MagickExport ErrorHandler SetErrorHandler(ErrorHandler handler)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   S e t E x c e p t i o n I n f o                                           %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  SetExceptionInfo() set the exception severity.
-%
-%  The format of the SetExceptionInfo method is:
-%
-%      MagickBooleanType SetExceptionInfo(ExceptionInfo *exception,
-%        ExceptionType severity)
-%
-%  A description of each parameter follows:
-%
-%    o exception: The exception info.
-%
-%    o severity: The exception severity.
-%
-%
-*/
-MagickExport MagickBooleanType SetExceptionInfo(ExceptionInfo *exception,
-  ExceptionType severity)
-{
-  assert(exception != (ExceptionInfo *) NULL);
-  exception->severity=severity;
-  return(MagickTrue);
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
 %   S e t F a t a l E r r o r H a n d l e r                                   %
 %                                                                             %
 %                                                                             %
@@ -747,13 +843,13 @@ MagickExport WarningHandler SetWarningHandler(WarningHandler handler)
 %
 %  The format of the ThrowException method is:
 %
-%      MagickBooleanType ThrowException(ExceptionInfo *exception_info,
+%      MagickBooleanType ThrowException(ExceptionInfo *exception,
 %        const ExceptionType severity,const char *reason,
 %        const char *description)
 %
 %  A description of each parameter follows:
 %
-%    o exception_info: The exception info.
+%    o exception: The exception info.
 %
 %    o severity: The severity of the exception.
 %
@@ -766,20 +862,39 @@ MagickExport WarningHandler SetWarningHandler(WarningHandler handler)
 MagickExport MagickBooleanType ThrowException(ExceptionInfo *exception,
   const ExceptionType severity,const char *reason,const char *description)
 {
+  register ExceptionInfo
+    *p;
+
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  exception->severity=(ExceptionType) severity;
-  if (reason != (char *) NULL)
-    (void) CloneString(&exception->reason,reason);
-  else
-    if (exception->reason != (char *) NULL)
-      exception->reason=(char *) RelinquishMagickMemory(exception->reason);
-  if (description != (char *) NULL)
-    (void) CloneString(&exception->description,description);
-  else
-    if (exception->description != (char *) NULL)
-      exception->description=(char *)
-        RelinquishMagickMemory(exception->description);
+  p=(ExceptionInfo *) GetLastValueInLinkedList((LinkedListInfo *)
+    exception->exceptions);
+  if ((p != (ExceptionInfo *) NULL) && (p->severity == severity) &&
+      (LocaleCompare(exception->reason,reason) == 0) &&
+      (LocaleCompare(exception->description,description) == 0))
+    return(MagickTrue);
+  p=(ExceptionInfo *) AcquireMagickMemory(sizeof(*p));
+  if (p == (ExceptionInfo *) NULL)
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowMagickFatalException(ResourceLimitFatalError,
+        "MemoryAllocationFailed",message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
+  (void) ResetMagickMemory(p,0,sizeof(*p));
+  p->severity=severity;
+  if (reason != (const char *) NULL)
+    p->reason=ConstantString(reason);
+  if (description != (const char *) NULL)
+    p->description=ConstantString(description);
+  p->signature=MagickSignature;
+  (void) AppendValueToLinkedList((LinkedListInfo *) exception->exceptions,p);
+  exception->severity=p->severity;
+  exception->reason=p->reason;
+  exception->description=p->description;
   return(MagickTrue);
 }
 
@@ -805,7 +920,7 @@ MagickExport MagickBooleanType ThrowException(ExceptionInfo *exception,
 %
 %  A description of each parameter follows:
 %
-%    o exception_info: The exception info.
+%    o exception: The exception info.
 %
 %    o filename: The source module filename.
 %
@@ -820,9 +935,11 @@ MagickExport MagickBooleanType ThrowException(ExceptionInfo *exception,
 %    o format: The output format.
 %
 */
-MagickExport MagickBooleanType ThrowMagickException(ExceptionInfo *exception,
-  const char *module,const char *function,const unsigned long line,
-  const ExceptionType severity,const char *tag,const char *format,...)
+
+MagickExport MagickBooleanType ThrowMagickExceptionList(
+  ExceptionInfo *exception,const char *module,const char *function,
+  const unsigned long line,const ExceptionType severity,const char *tag,
+  const char *format,va_list operands)
 {
   char
     reason[MaxTextExtent];
@@ -839,19 +956,12 @@ MagickExport MagickBooleanType ThrowMagickException(ExceptionInfo *exception,
   size_t
     length;
 
-  va_list
-    operands;
-
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  if (severity < exception->severity)
-    return(MagickTrue);
-  exception->severity=(ExceptionType) severity;
   locale=GetLocaleExceptionMessage(severity,tag);
   (void) CopyMagickString(reason,locale,MaxTextExtent);
   (void) ConcatenateMagickString(reason," ",MaxTextExtent);
   length=strlen(reason);
-  va_start(operands,format);
 #if defined(HAVE_VSNPRINTF)
   n=vsnprintf(reason+length,MaxTextExtent-length,format,operands);
 #else
@@ -859,11 +969,24 @@ MagickExport MagickBooleanType ThrowMagickException(ExceptionInfo *exception,
 #endif
   if (n < 0)
     reason[MaxTextExtent-1]='\0';
-  va_end(operands);
-#if defined(win32)
-  (void) NTReportEvent(reason,severity >= ErrorException);
-#endif
   status=LogMagickEvent(ExceptionEvent,module,function,line,"%s",reason);
-  (void) CloneString(&exception->reason,reason);
+  ThrowException(exception,severity,reason,(char *) NULL);
+  return(status);
+}
+
+MagickExport MagickBooleanType ThrowMagickException(ExceptionInfo *exception,
+  const char *module,const char *function,const unsigned long line,
+  const ExceptionType severity,const char *tag,const char *format,...)
+{
+  MagickBooleanType
+    status;
+
+  va_list
+    operands;
+
+  va_start(operands,format);
+  status=ThrowMagickExceptionList(exception,module,function,line,severity,tag,
+    format,operands);
+  va_end(operands);
   return(status);
 }
