@@ -17,7 +17,7 @@
 %                                 August 1996                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,10 +41,14 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/draw.h"
 #include "magick/gem.h"
 #include "magick/image.h"
+#include "magick/image-private.h"
 #include "magick/log.h"
 #include "magick/memory_.h"
+#include "magick/pixel-private.h"
+#include "magick/quantum.h"
 #include "magick/random_.h"
 #include "magick/signature.h"
 
@@ -192,7 +196,7 @@ MagickExport unsigned long GetOptimalKernelWidth1D(const double radius,
       normalize+=exp(-((double) u*u)/(2.0*sigma*sigma))/(MagickSQ2PI*sigma);
     u=width/2;
     value=exp(-((double) u*u)/(2.0*sigma*sigma))/(MagickSQ2PI*sigma)/normalize;
-    if ((long) (MaxRGB*value) <= 0)
+    if ((long) (QuantumRange*value) <= 0)
       break;
     width+=2;
   }
@@ -231,7 +235,7 @@ MagickExport unsigned long GetOptimalKernelWidth2D(const double radius,
     }
     v=width/2;
     value=exp(-((double) v*v)/(2.0*sigma*sigma))/(MagickSQ2PI*sigma)/normalize;
-    if ((long) (MaxRGB*value) <= 0)
+    if ((long) (QuantumRange*value) <= 0)
       break;
     width+=2;
   }
@@ -290,7 +294,7 @@ MagickExport void HSBTransform(const double hue,const double saturation,
   assert(blue != (Quantum *) NULL);
   if (saturation == 0.0)
     {
-      *red=RoundToQuantum(MaxRGB*brightness);
+      *red=RoundToQuantum((MagickRealType) QuantumRange*brightness);
       *green=(*red);
       *blue=(*red);
       return;
@@ -305,44 +309,44 @@ MagickExport void HSBTransform(const double hue,const double saturation,
     case 0:
     default:
     {
-      *red=RoundToQuantum(MaxRGB*brightness);
-      *green=RoundToQuantum(MaxRGB*t);
-      *blue=RoundToQuantum(MaxRGB*p);
+      *red=RoundToQuantum((MagickRealType) QuantumRange*brightness);
+      *green=RoundToQuantum((MagickRealType) QuantumRange*t);
+      *blue=RoundToQuantum((MagickRealType) QuantumRange*p);
       break;
     }
     case 1:
     {
-      *red=RoundToQuantum(MaxRGB*q);
-      *green=RoundToQuantum(MaxRGB*brightness);
-      *blue=RoundToQuantum(MaxRGB*p);
+      *red=RoundToQuantum((MagickRealType) QuantumRange*q);
+      *green=RoundToQuantum((MagickRealType) QuantumRange*brightness);
+      *blue=RoundToQuantum((MagickRealType) QuantumRange*p);
       break;
     }
     case 2:
     {
-      *red=RoundToQuantum(MaxRGB*p);
-      *green=RoundToQuantum(MaxRGB*brightness);
-      *blue=RoundToQuantum(MaxRGB*t);
+      *red=RoundToQuantum((MagickRealType) QuantumRange*p);
+      *green=RoundToQuantum((MagickRealType) QuantumRange*brightness);
+      *blue=RoundToQuantum((MagickRealType) QuantumRange*t);
       break;
     }
     case 3:
     {
-      *red=RoundToQuantum(MaxRGB*p);
-      *green=RoundToQuantum(MaxRGB*q);
-      *blue=RoundToQuantum(MaxRGB*brightness);
+      *red=RoundToQuantum((MagickRealType) QuantumRange*p);
+      *green=RoundToQuantum((MagickRealType) QuantumRange*q);
+      *blue=RoundToQuantum((MagickRealType) QuantumRange*brightness);
       break;
     }
     case 4:
     {
-      *red=RoundToQuantum(MaxRGB*t);
-      *green=RoundToQuantum(MaxRGB*p);
-      *blue=RoundToQuantum(MaxRGB*brightness);
+      *red=RoundToQuantum((MagickRealType) QuantumRange*t);
+      *green=RoundToQuantum((MagickRealType) QuantumRange*p);
+      *blue=RoundToQuantum((MagickRealType) QuantumRange*brightness);
       break;
     }
     case 5:
     {
-      *red=RoundToQuantum(MaxRGB*brightness);
-      *green=RoundToQuantum(MaxRGB*p);
-      *blue=RoundToQuantum(MaxRGB*q);
+      *red=RoundToQuantum((MagickRealType) QuantumRange*brightness);
+      *green=RoundToQuantum((MagickRealType) QuantumRange*p);
+      *blue=RoundToQuantum((MagickRealType) QuantumRange*q);
       break;
     }
   }
@@ -376,6 +380,23 @@ MagickExport void HSBTransform(const double hue,const double saturation,
 %
 %
 */
+
+static inline MagickRealType HueToRGB(MagickRealType m1,MagickRealType m2,
+  MagickRealType hue)
+{
+  if (hue < 0.0)
+    hue+=1.0;
+  if (hue > 1.0)
+    hue-=1.0;
+  if ((6.0*hue) < 1.0)
+    return(m1+6.0*(m2-m1)*hue);
+  if ((2.0*hue) < 1.0)
+    return(m2);
+  if ((3.0*hue) < 2.0)
+    return(m1+6.0*(m2-m1)*(2.0/3.0-hue));
+  return(m1);
+}
+
 MagickExport void HSLTransform(const double hue,const double saturation,
   const double luminosity,Quantum *red,Quantum *green,Quantum *blue)
 {
@@ -383,10 +404,8 @@ MagickExport void HSLTransform(const double hue,const double saturation,
     b,
     g,
     r,
-    v,
-    x,
-    y,
-    z;
+    m1,
+    m2;
 
   /*
     Convert HSL to RGB colorspace.
@@ -394,31 +413,17 @@ MagickExport void HSLTransform(const double hue,const double saturation,
   assert(red != (Quantum *) NULL);
   assert(green != (Quantum *) NULL);
   assert(blue != (Quantum *) NULL);
-  if (saturation == 0.0)
-    {
-      *red=(Quantum) (MaxRGB*luminosity+0.5);
-      *green=(Quantum) (MaxRGB*luminosity+0.5);
-      *blue=(Quantum) (MaxRGB*luminosity+0.5);
-      return;
-    }
-  v=(luminosity <= 0.5) ? (luminosity*(1.0+saturation)) :
-    (luminosity+saturation-luminosity*saturation);
-  y=2.0*luminosity-v;
-  x=y+(v-y)*(6.0*hue-floor(6.0*hue));
-  z=v-(v-y)*(6.0*hue-floor(6.0*hue));
-  switch ((int) (6.0*hue))
-  {
-    case 0: r=v; g=x; b=y; break;
-    case 1: r=z; g=v; b=y; break;
-    case 2: r=y; g=v; b=x; break;
-    case 3: r=y; g=z; b=v; break;
-    case 4: r=x; g=y; b=v; break;
-    case 5: r=v; g=y; b=z; break;
-    default: r=v; g=x; b=y; break;
-  }
-  *red=RoundToQuantum(MaxRGB*r);
-  *green=RoundToQuantum(MaxRGB*g);
-  *blue=RoundToQuantum(MaxRGB*b);
+  if (luminosity <= 0.5)
+    m2=luminosity*(saturation+1.0);
+  else
+    m2=luminosity+saturation-luminosity*saturation;
+  m1=2.0*luminosity-m2;
+  r=HueToRGB(m1,m2,hue+1.0/3.0);
+  g=HueToRGB(m1,m2,hue);
+  b=HueToRGB(m1,m2,hue-1.0/3.0);
+  *red=RoundToQuantum(QuantumRange*r);
+  *green=RoundToQuantum(QuantumRange*g);
+  *blue=RoundToQuantum(QuantumRange*b);
 }
 
 /*
@@ -472,9 +477,9 @@ MagickExport void HWBTransform(const double hue,const double whiteness,
   v=1.0-blackness;
   if (hue == 0.0)
     {
-      *red=(Quantum) (MaxRGB*v+0.5);
-      *green=(Quantum) (MaxRGB*v+0.5);
-      *blue=(Quantum) (MaxRGB*v+0.5);
+      *red=(Quantum) (QuantumRange*v+0.5);
+      *green=(Quantum) (QuantumRange*v+0.5);
+      *blue=(Quantum) (QuantumRange*v+0.5);
       return;
     }
   i=(long) floor(hue);
@@ -493,9 +498,9 @@ MagickExport void HWBTransform(const double hue,const double whiteness,
     case 4: r=n; g=whiteness; b=v; break;
     case 5: r=v; g=whiteness; b=n; break;
   }
-  *red=(Quantum) (MaxRGB*r+0.5);
-  *green=(Quantum) (MaxRGB*g+0.5);
-  *blue=(Quantum) (MaxRGB*b+0.5);
+  *red=(Quantum) (QuantumRange*r+0.5);
+  *green=(Quantum) (QuantumRange*g+0.5);
+  *blue=(Quantum) (QuantumRange*b+0.5);
 }
 
 /*
@@ -649,8 +654,8 @@ MagickExport void Hull(const long x_offset,const long y_offset,
 %
 %  The format of the InterpolateColor method is:
 %
-%      PixelPacket InterpolateColor(const Image *image,const double x_offset,
-%        const double y_offset,ExceptionInfo *exception)
+%      MagickPixelPacket InterpolateColor(const Image *image,
+%        const double x_offset,const double y_offset,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -661,38 +666,74 @@ MagickExport void Hull(const long x_offset,const long y_offset,
 %
 %
 */
-MagickExport PixelPacket InterpolateColor(const Image *image,
+
+MagickExport MagickPixelPacket InterpolateColor(const Image *image,
   const double x_offset,const double y_offset,ExceptionInfo *exception)
 {
-  MagickRealType
-    alpha,
-    beta;
+  MagickPixelPacket
+    pixel,
+    pixels[4];
 
-  PixelPacket
-    color;
+  MagickRealType
+    alpha[4],
+    gamma;
+
+  PointInfo
+    delta;
 
   register const PixelPacket
     *p;
 
+  register IndexPacket
+    *indexes;
+
+  register long
+    i;
+
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  GetMagickPixelPacket(image,&pixel);
   p=AcquireImagePixels(image,(long) x_offset,(long) y_offset,2,2,exception);
   if (p == (const PixelPacket *) NULL)
-    return(AcquireOnePixel(image,(long) x_offset,(long) y_offset,exception));
-  alpha=x_offset-floor(x_offset);
-  beta=y_offset-floor(y_offset);
-  color.red=(Quantum) ((1.0-beta)*((1.0-alpha)*p[0].red+
-    alpha*p[1].red)+beta*((1.0-alpha)*p[2].red+alpha*p[3].red)+0.5);
-  color.green=(Quantum) ((1.0-beta)*((1.0-alpha)*p[0].green+
-    alpha*p[1].green)+beta*((1.0-alpha)*p[2].green+alpha*p[3].green)+0.5);
-  color.blue=(Quantum) ((1.0-beta)*((1.0-alpha)*p[0].blue+
-    alpha*p[1].blue)+beta*((1.0-alpha)*p[2].blue+alpha*p[3].blue)+0.5);
-  if (image->matte == MagickFalse)
-    color.opacity=p[0].opacity;
-  else
-    color.opacity=(Quantum) ((1.0-beta)*((1.0-alpha)*p[0].opacity+alpha*
-      p[1].opacity)+beta*((1.0-alpha)*p[2].opacity+alpha*p[3].opacity)+0.5);
-  return(color);
+    return(pixel);
+  for (i=0; i < 4L; i++)
+  {
+    indexes=GetIndexes(image);
+    alpha[i]=1.0;
+    if (image->matte != MagickFalse)
+      alpha[i]=((MagickRealType) QuantumRange-p->opacity)/QuantumRange;
+    GetMagickPixelPacket(image,pixels+i);
+    SetMagickPixelPacket(p,indexes,pixels+i);
+    pixels[i].red*=alpha[i];
+    pixels[i].green*=alpha[i];
+    pixels[i].blue*=alpha[i];
+    if (image->colorspace == CMYKColorspace)
+      pixels[i].index*=alpha[i];
+    p++;
+  }
+  delta.x=x_offset-floor(x_offset);
+  delta.y=y_offset-floor(y_offset);
+  gamma=(((1.0-delta.y)*((1.0-delta.x)*alpha[0]+delta.x*alpha[1])+
+    delta.y*((1.0-delta.x)*alpha[2]+delta.x*alpha[3])));
+  gamma=1.0/(fabs((double) gamma) <= MagickEpsilon ? 1.0 : gamma);
+  pixel.red=(gamma*((1.0-delta.y)*((1.0-delta.x)*pixels[0].red+delta.x*
+    pixels[1].red)+delta.y*((1.0-delta.x)*pixels[2].red+delta.x*
+    pixels[3].red)));
+  pixel.green=(gamma*((1.0-delta.y)*((1.0-delta.x)*pixels[0].green+delta.x*
+    pixels[1].green)+delta.y*((1.0-delta.x)*pixels[2].green+delta.x*
+    pixels[3].green)));
+  pixel.blue=(gamma*((1.0-delta.y)*((1.0-delta.x)*pixels[0].blue+delta.x*
+    pixels[1].blue)+delta.y*((1.0-delta.x)*pixels[2].blue+delta.x*
+    pixels[3].blue)));
+  if (image->matte != MagickFalse)
+    pixel.opacity=((1.0-delta.y)*((1.0-delta.x)*pixels[0].opacity+delta.x*
+      pixels[1].opacity)+delta.y*((1.0-delta.x)*pixels[2].opacity+delta.x*
+      pixels[3].opacity));
+  if (image->colorspace == CMYKColorspace)
+    pixel.index=(gamma*((1.0-delta.y)*((1.0-delta.x)*pixels[0].index+delta.x*
+      pixels[1].index)+delta.y*((1.0-delta.x)*pixels[2].index+delta.x*
+      pixels[3].index)));
+  return(pixel);
 }
 
 /*
@@ -906,15 +947,15 @@ MagickExport void TransformHSB(const Quantum red,const Quantum green,
   min=(MagickRealType) (red < green ? red : green);
   if ((MagickRealType) blue < min)
     min=(MagickRealType) blue;
+  *hue=0.0;
+  *saturation=0.0;
   *brightness=(double) (QuantumScale*max);
   if (max == 0.0)
-    {
-      *saturation=0.0;
-      *hue=0.0;
-      return;
-    }
+    return;
   *saturation=(double) (1.0-min/max);
   delta=max-min;
+  if (delta == 0.0)
+    return;
   if ((MagickRealType) red == max)
     *hue=(double) ((green-(MagickRealType) blue)/delta);
   else
@@ -973,9 +1014,9 @@ MagickExport void TransformHSL(const Quantum red,const Quantum green,
   assert(hue != (double *) NULL);
   assert(saturation != (double *) NULL);
   assert(luminosity != (double *) NULL);
-  r=(MagickRealType) red/MaxRGB;
-  g=(MagickRealType) green/MaxRGB;
-  b=(MagickRealType) blue/MaxRGB;
+  r=(MagickRealType) red/QuantumRange;
+  g=(MagickRealType) green/QuantumRange;
+  b=(MagickRealType) blue/QuantumRange;
   max=Max(r,Max(g,b));
   min=Min(r,Min(g,b));
   *hue=0.0;
@@ -1046,7 +1087,7 @@ MagickExport void TransformHWB(const Quantum red,const Quantum green,
   assert(blackness != (double *) NULL);
   w=Min(red,Min(green,blue));
   v=Max(red,Max(green,blue));
-  *blackness=(double) (MaxRGB-v)/MaxRGB;
+  *blackness=(double) (QuantumRange-v)/(double) QuantumRange;
   if (v == w)
     {
       *hue=0.0;
@@ -1057,90 +1098,5 @@ MagickExport void TransformHWB(const Quantum red,const Quantum green,
     (MagickRealType) blue-red : (MagickRealType) red-green);
   i=(red == w) ? 3 : ((green == w) ? 5 : 1);
   *hue=(double) (i-f/(v-w));
-  *whiteness=(double) w/MaxRGB;
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%   U p s a m p l e                                                           %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  Upsample() doubles the size of the image.
-%
-%  The format of the Upsample method is:
-%
-%      void Upsample(const unsigned long width,const unsigned long height,
-%        const unsigned long scaled_width,unsigned char *pixels)
-%
-%  A description of each parameter follows:
-%
-%    o width,height:  Unsigned values representing the width and height of
-%      the image pixel array.
-%
-%    o scaled_width:  Specifies the final width of the upsampled pixel array.
-%
-%    o pixels:  An unsigned char containing the pixel data.  On output the
-%      upsampled pixels are returns here.
-%
-%
-*/
-MagickExport void Upsample(const unsigned long width,const unsigned long height,
-  const unsigned long scaled_width,unsigned char *pixels)
-{
-  register long
-    x,
-    y;
-
-  register unsigned char
-    *p,
-    *q,
-    *r;
-
-  /*
-    Create a new image that is a integral size greater than an existing one.
-  */
-  assert(pixels != (unsigned char *) NULL);
-  for (y=0; y < (long) height; y++)
-  {
-    p=pixels+(height-1-y)*scaled_width+(width-1);
-    q=pixels+((height-1-y) << 1)*scaled_width+((width-1) << 1);
-    *q=(*p);
-    *(q+1)=(*(p));
-    for (x=1; x < (long) width; x++)
-    {
-      p--;
-      q-=2;
-      *q=(*p);
-      *(q+1)=(unsigned char) ((((unsigned long) *p)+
-        ((unsigned long) *(p+1))+1) >> 1);
-    }
-  }
-  for (y=0; y < (long) (height-1); y++)
-  {
-    p=pixels+((unsigned long) y << 1)*scaled_width;
-    q=p+scaled_width;
-    r=q+scaled_width;
-    for (x=0; x < (long) (width-1); x++)
-    {
-      *q=(unsigned char) ((((unsigned long) *p)+((unsigned long) *r)+1) >> 1);
-      *(q+1)=(unsigned char) ((((unsigned long) *p)+((unsigned long) *(p+2))+
-        ((unsigned long) *r)+((unsigned long) *(r+2))+2) >> 2);
-      q+=2;
-      p+=2;
-      r+=2;
-    }
-    *q++=(unsigned char) ((((unsigned long) *p++)+
-      ((unsigned long) *r++)+1) >> 1);
-    *q++=(unsigned char) ((((unsigned long) *p++)+
-      ((unsigned long) *r++)+1) >> 1);
-  }
-  p=pixels+(2*height-2)*scaled_width;
-  q=pixels+(2*height-1)*scaled_width;
-  (void) CopyMagickMemory(q,p,(size_t) (2*width));
+  *whiteness=(double) w/(double) QuantumRange;
 }
