@@ -41,6 +41,7 @@
 */
 #include "magick/studio.h"
 #include "magick/constitute.h"
+#include "magick/draw.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
 #include "magick/geometry.h"
@@ -341,7 +342,7 @@ MagickExport char *GetPageGeometry(const char *page_geometry)
     i;
 
   assert(page_geometry != (char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),page_geometry);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",page_geometry);
   page=AcquireString(page_geometry);
   for (i=0; *PageSizes[i] != (char *) NULL; i++)
     if (LocaleNCompare(PageSizes[i][0],page,strlen(PageSizes[i][0])) == 0)
@@ -526,6 +527,9 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
     pedantic_geometry[MaxTextExtent],
     *q;
 
+  double
+    value;
+
   MagickStatusType
     flags;
 
@@ -585,7 +589,6 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
         break;
       }
       case '-':
-      case '.':
       case '+':
       case '0':
       case '1':
@@ -605,12 +608,18 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
         p++;
         break;
       }
+      case '.':
+      {
+        p++;
+        flags|=DecimalValue;
+        break;
+      }
       default:
         return(flags);
     }
   }
   /*
-    Parse rho, sigma, xi, and psi.
+    Parse rho, sigma, xi, psi, and optionally chi.
   */
   p=pedantic_geometry;
   if (*p == '\0')
@@ -619,31 +628,46 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
   (void) strtod(p,&q);
   if (LocaleNCompare(p,"0x",2) == 0)
     (void) strtol(p,&q,10);
-  if ((*q == 'x') || (*q == 'X') || (*q == '/') || (*q == ',') || (*q =='\0'))
+  if ((*q == 'x') || (*q == 'X') || (*q == ',') || (*q == '/') ||
+      (*q == ':') || (*q =='\0'))
     {
       /*
         Parse rho.
       */
       q=p;
       if (LocaleNCompare(p,"0x",2) == 0)
-        geometry_info->rho=(double) strtol(p,&p,10);
+        value=(double) strtol(p,&p,10);
       else
-        geometry_info->rho=strtod(p,&p);
+        value=strtod(p,&p);
       if (p != q)
-        flags|=RhoValue;
+        {
+          flags|=RhoValue;
+          geometry_info->rho=value;
+        }
     }
-  if ((*p == 'x') || (*p == 'X') || (*p == '/') || (*p == ','))
+  q=p;
+  if ((*p == 'x') || (*p == 'X') || (*p == ',') || (*p == '/') || (*p == ':'))
     {
       /*
         Parse sigma.
       */
       p++;
-      q=p;
-      geometry_info->sigma=strtod(p,&p);
-      if (p != q)
-        flags|=SigmaValue;
+      while (isspace((int) ((unsigned char) *p)) != 0)
+        p++;
+      if (((*q != 'x') && (*q != 'X')) || ((*p != '+') && (*p != '-')))
+        {
+          q=p;
+          value=strtod(p,&p);
+          if (p != q)
+            {
+              flags|=SigmaValue;
+              geometry_info->sigma=value;
+            }
+        }
     }
-  if ((*p == '+') || (*p == '-') || (*p == ',') || (*p == '/'))
+  while (isspace((int) ((unsigned char) *p)) != 0)
+    p++;
+  if ((*p == '+') || (*p == '-') || (*p == ',') || (*p == '/') || (*p == ':'))
     {
       /*
         Parse xi value.
@@ -651,14 +675,18 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
       if ((*p == ',') || (*p == '/'))
         p++;
       q=p;
-      geometry_info->xi=strtod(p,&p);
+      value=strtod(p,&p);
       if (p != q)
         {
           flags|=XiValue;
           if (*q == '-')
             flags|=XiNegative;
+          geometry_info->xi=value;
         }
-      if ((*p == '+') || (*p == '-') || (*p == ',') || (*p == '/'))
+      while (isspace((int) ((unsigned char) *p)) != 0)
+        p++;
+      if ((*p == '+') || (*p == '-') || (*p == ',') || (*p == '/') ||
+          (*p == ':'))
         {
           /*
             Parse psi value.
@@ -666,15 +694,19 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
           if ((*p == ',') || (*p == '/'))
             p++;
           q=p;
-          geometry_info->psi=strtod(p,&p);
+          value=strtod(p,&p);
           if (p != q)
             {
               flags|=PsiValue;
               if (*q == '-')
                 flags|=PsiNegative;
+              geometry_info->psi=value;
             }
         }
-      if ((*p == '+') || (*p == '-') || (*p == ',') || (*p == '/'))
+      while (isspace((int) ((unsigned char) *p)) != 0)
+        p++;
+      if ((*p == '+') || (*p == '-') || (*p == ',') || (*p == '/') ||
+          (*p == ':'))
         {
           /*
             Parse chi value.
@@ -682,15 +714,26 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
           if ((*p == ',') || (*p == '/'))
             p++;
           q=p;
-          geometry_info->chi=strtod(p,&p);
+          value=strtod(p,&p);
           if (p != q)
             {
               flags|=ChiValue;
               if (*q == '-')
                 flags|=ChiNegative;
+              geometry_info->chi=value;
             }
         }
     }
+  if (strchr(pedantic_geometry,':') != (char *) NULL)
+    {
+      /*
+        Normalize sampling factor (e.g. 4:2:2 => 2x1).
+      */
+      geometry_info->rho*=geometry_info->sigma;
+      geometry_info->sigma=1.0;
+      if (geometry_info->xi == 0.0)
+        geometry_info->sigma=2.0;
+    }
   return(flags);
 }
 
@@ -720,14 +763,10 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
 %    o region_info: The region as defined by the geometry string with
 %      respect to the image dimensions and its gravity.
 %
-%
 */
 MagickExport MagickStatusType ParseGravityGeometry(Image *image,
   const char *geometry,RectangleInfo *region_info)
 {
-  char
-    region_geometry[MaxTextExtent];
-
   MagickStatusType
     flags;
 
@@ -736,14 +775,37 @@ MagickExport MagickStatusType ParseGravityGeometry(Image *image,
     width;
 
   SetGeometry(image,region_info);
-  (void) FormatMagickString(region_geometry,MaxTextExtent,"%s!",geometry);
-  flags=ParseAbsoluteGeometry(region_geometry,region_info);
+  flags=ParseAbsoluteGeometry(geometry,region_info);
   if (flags == NoValue)
     {
       (void) ThrowMagickException(&image->exception,GetMagickModule(),
         OptionError,"InvalidGeometry","`%s'",geometry);
       return(flags);
     }
+  if ((flags & PercentValue) != 0)
+    {
+      GeometryInfo
+        geometry_info;
+
+      MagickStatusType
+        flags;
+
+      PointInfo
+        scale;
+
+      /*
+        Geometry is a percentage of the image size.
+      */
+      flags=ParseGeometry(geometry,&geometry_info);
+      scale.x=geometry_info.rho;
+      if ((flags & RhoValue) == 0)
+        scale.x=100.0;
+      scale.y=geometry_info.sigma;
+      if ((flags & SigmaValue) == 0)
+        scale.y=scale.x;
+      region_info->width=(unsigned long) ((scale.x*image->columns/100.0)+0.5);
+      region_info->height=(unsigned long) ((scale.y*image->rows/100.0)+0.5);
+    }
   width=region_info->width;
   height=region_info->height;
   if (width == 0)
@@ -760,31 +822,6 @@ MagickExport MagickStatusType ParseGravityGeometry(Image *image,
       else
         height=image->rows;
     }
-  if ((flags & PercentValue) != 0)
-    {
-      GeometryInfo
-        geometry_info;
-
-      MagickRealType
-        x_scale,
-        y_scale;
-
-      MagickStatusType
-        flags;
-
-      /*
-        Geometry is a percentage of the image size.
-      */
-      flags=ParseGeometry(geometry,&geometry_info);
-      x_scale=geometry_info.rho;
-      if ((flags & RhoValue) == 0)
-        x_scale=100.0;
-      y_scale=geometry_info.sigma;
-      if ((flags & SigmaValue) == 0)
-        y_scale=x_scale;
-      region_info->width=(unsigned long) ((x_scale*image->columns/100.0)+0.5);
-      region_info->height=(unsigned long) ((y_scale*image->rows/100.0)+0.5);
-    }
   switch (image->gravity)
   {
     case ForgetGravity:
@@ -869,7 +906,6 @@ MagickExport MagickStatusType ParseGravityGeometry(Image *image,
 %    o width,height:  The width and height as determined by the geometry
 %      specification.
 %
-%
 */
 MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,long *x,
   long *y,unsigned long *width,unsigned long *height)
@@ -893,7 +929,7 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,long *x,
   assert(height != (unsigned long *) NULL);
   if ((geometry == (char *) NULL) || (*geometry == '\0'))
     return(NoValue);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),geometry);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",geometry);
   /*
     Parse geometry using GetGeometry.
   */
@@ -903,25 +939,24 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,long *x,
   flags=GetGeometry(geometry,x,y,width,height);
   if ((flags & PercentValue) != 0)
     {
-      MagickRealType
-        x_scale,
-        y_scale;
-
       MagickStatusType
         flags;
 
+      PointInfo
+        scale;
+
       /*
         Geometry is a percentage of the image size.
       */
       flags=ParseGeometry(geometry,&geometry_info);
-      x_scale=geometry_info.rho;
+      scale.x=geometry_info.rho;
       if ((flags & RhoValue) == 0)
-        x_scale=100.0;
-      y_scale=geometry_info.sigma;
+        scale.x=100.0;
+      scale.y=geometry_info.sigma;
       if ((flags & SigmaValue) == 0)
-        y_scale=x_scale;
-      *width=(unsigned long) ((x_scale*former_width/100.0)+0.5);
-      *height=(unsigned long) ((y_scale*former_height/100.0)+0.5);
+        scale.y=scale.x;
+      *width=(unsigned long) ((scale.x*former_width/100.0)+0.5);
+      *height=(unsigned long) ((scale.y*former_height/100.0)+0.5);
       former_width=(*width);
       former_height=(*height);
     }
@@ -972,9 +1007,10 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,long *x,
     {
       MagickRealType
         area,
-        distance,
-        x_scale,
-        y_scale;
+        distance;
+
+      PointInfo
+        scale;
 
       /*
         Geometry is a maximum area in pixels.
@@ -982,10 +1018,9 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,long *x,
       (void) ParseGeometry(geometry,&geometry_info);
       area=geometry_info.rho;
       distance=sqrt((double) former_width*former_height);
-      x_scale=(MagickRealType) former_width/(distance/sqrt((double) area));
-      y_scale=(MagickRealType) former_height/(distance/sqrt((double) area));
-      if ((x_scale < (MagickRealType) *width) ||
-          (y_scale < (MagickRealType) *height))
+      scale.x=(double) former_width/(double) (distance/sqrt((double) area));
+      scale.y=(double) former_height/(double) (distance/sqrt((double) area));
+      if ((scale.x < (double) *width) || (scale.y < (double) *height))
         {
           *width=(unsigned long) (former_width/(distance/sqrt((double) area)));
           *height=(unsigned long) (former_height/(distance/
@@ -1023,24 +1058,23 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,long *x,
 %    o region_info: The region as defined by the geometry string with
 %      respect to the image and its gravity.
 %
-%
 */
 MagickExport MagickStatusType ParsePageGeometry(Image *image,
   const char *geometry,RectangleInfo *region_info)
 {
-  char
-    region_geometry[MaxTextExtent];
-
   MagickStatusType
     flags;
 
   SetGeometry(image,region_info);
-  (void) FormatMagickString(region_geometry,MaxTextExtent,"%s!",geometry);
-  flags=ParseMetaGeometry(region_geometry,&region_info->x,&region_info->y,
-    &region_info->width,&region_info->height);
+  flags=ParseAbsoluteGeometry(geometry,region_info);
   if (flags == NoValue)
-    (void) ThrowMagickException(&image->exception,GetMagickModule(),OptionError,
-      "InvalidGeometry","`%s'",geometry);
+    {
+      (void) ThrowMagickException(&image->exception,GetMagickModule(),
+        OptionError,"InvalidGeometry","`%s'",geometry);
+      return(flags);
+    }
+  flags=ParseMetaGeometry(geometry,&region_info->x,&region_info->y,
+    &region_info->width,&region_info->height);
   return(flags);
 }
 
@@ -1116,11 +1150,15 @@ MagickExport void SetGeometry(const Image *image,RectangleInfo *geometry)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(geometry != (RectangleInfo *) NULL);
   (void) ResetMagickMemory(geometry,0,sizeof(*geometry));
   geometry->width=image->columns;
+  if (image->page.width != 0)
+    geometry->width=image->page.width;
   geometry->height=image->rows;
+  if (image->page.height != 0)
+    geometry->height=image->page.height;
 }
 
 /*
