@@ -17,13 +17,13 @@
 %                               December 2002                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -160,6 +160,7 @@ MagickExport MagickBooleanType AppendValueToLinkedList(
 
   assert(list_info != (LinkedListInfo *) NULL);
   assert(list_info->signature == MagickSignature);
+  list_info->debug=IsEventLogging();
   if (list_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   if (list_info->elements == list_info->capacity)
@@ -169,7 +170,7 @@ MagickExport MagickBooleanType AppendValueToLinkedList(
     return(MagickFalse);
   next->value=(void *) value;
   next->next=(ElementInfo *) NULL;
-  list_info->semaphore=AcquireSemaphoreInfo(list_info->semaphore);
+  AcquireSemaphoreInfo(&list_info->semaphore);
   if (list_info->next == (ElementInfo *) NULL)
     list_info->next=next;
   if (list_info->elements == 0)
@@ -221,7 +222,7 @@ MagickExport void ClearLinkedList(LinkedListInfo *list_info,
   assert(list_info->signature == MagickSignature);
   if (list_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
-  list_info->semaphore=AcquireSemaphoreInfo(list_info->semaphore);
+  AcquireSemaphoreInfo(&list_info->semaphore);
   next=list_info->head;
   while (next != (ElementInfo *) NULL)
   {
@@ -267,12 +268,12 @@ MagickExport void ClearLinkedList(LinkedListInfo *list_info,
 MagickExport MagickBooleanType CompareHashmapString(const void *target,
   const void *source)
 {
-  char
+  const char
     *p,
     *q;
 
-  p=(char *) target;
-  q=(char *) source;
+  p=(const char *) target;
+  q=(const char *) source;
   return(LocaleCompare(p,q) == 0 ? MagickTrue : MagickFalse);
 }
 
@@ -305,12 +306,12 @@ MagickExport MagickBooleanType CompareHashmapString(const void *target,
 MagickExport MagickBooleanType CompareHashmapStringInfo(const void *target,
   const void *source)
 {
-  StringInfo
+  const StringInfo
     *p,
     *q;
 
-  p=(StringInfo *) target;
-  q=(StringInfo *) source;
+  p=(const StringInfo *) target;
+  q=(const StringInfo *) source;
   return(CompareStringInfo(p,q) == 0 ? MagickTrue : MagickFalse);
 }
 
@@ -351,7 +352,7 @@ MagickExport HashmapInfo *DestroyHashmap(HashmapInfo *hashmap_info)
   assert(hashmap_info->signature == MagickSignature);
   if (hashmap_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
-  hashmap_info->semaphore=AcquireSemaphoreInfo(hashmap_info->semaphore);
+  AcquireSemaphoreInfo(&hashmap_info->semaphore);
   for (i=0; i < (long) hashmap_info->capacity; i++)
   {
     list_info=hashmap_info->map[i];
@@ -419,7 +420,7 @@ MagickExport LinkedListInfo *DestroyLinkedList(LinkedListInfo *list_info,
   assert(list_info->signature == MagickSignature);
   if (list_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
-  list_info->semaphore=AcquireSemaphoreInfo(list_info->semaphore);
+  AcquireSemaphoreInfo(&list_info->semaphore);
   next=list_info->head;
   while (next != (ElementInfo *) NULL)
   {
@@ -721,7 +722,7 @@ MagickExport void *GetValueFromHashmap(const HashmapInfo *hashmap_info,
   assert(hashmap_info->signature == MagickSignature);
   if (hashmap_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
-  if (key == NULL)
+  if (key == (const void *) NULL)
     return((void *) NULL);
   hash=hashmap_info->hash(key);
   list_info=hashmap_info->map[hash % hashmap_info->capacity];
@@ -872,7 +873,7 @@ MagickExport size_t HashStringType(const void *string)
 
   GetSignatureInfo(&signature_info);
   UpdateSignature(&signature_info,(const unsigned char *) string,
-    strlen((char *) string));
+    strlen((const char *) string));
   FinalizeSignature(&signature_info);
   hash=0;
   for (i=0; i < 8; i++)
@@ -915,8 +916,8 @@ MagickExport size_t HashStringInfoType(const void *string)
     hash;
 
   GetSignatureInfo(&signature_info);
-  UpdateSignature(&signature_info,((StringInfo *) string)->datum,
-    ((StringInfo *) string)->length);
+  UpdateSignature(&signature_info,((const StringInfo *) string)->datum,
+    ((const StringInfo *) string)->length);
   FinalizeSignature(&signature_info);
   hash=0;
   for (i=0; i < 8; i++)
@@ -965,7 +966,7 @@ MagickExport MagickBooleanType InsertValueInLinkedList(
   assert(list_info->signature == MagickSignature);
   if (list_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
-  if (value == NULL)
+  if (value == (const void *) NULL)
     return(MagickFalse);
   if ((index > list_info->elements) ||
       (list_info->elements == list_info->capacity))
@@ -975,7 +976,7 @@ MagickExport MagickBooleanType InsertValueInLinkedList(
     return(MagickFalse);
   next->value=(void *) value;
   next->next=(ElementInfo *) NULL;
-  list_info->semaphore=AcquireSemaphoreInfo(list_info->semaphore);
+  AcquireSemaphoreInfo(&list_info->semaphore);
   if (list_info->elements == 0)
     {
       if (list_info->next == (ElementInfo *) NULL)
@@ -1082,7 +1083,7 @@ MagickExport MagickBooleanType InsertValueInSortedLinkedList(
     return(MagickFalse);
   next->value=(void *) value;
   element=(ElementInfo *) NULL;
-  list_info->semaphore=AcquireSemaphoreInfo(list_info->semaphore);
+  AcquireSemaphoreInfo(&list_info->semaphore);
   next->next=list_info->head;
   while (next->next != (ElementInfo *) NULL)
   {
@@ -1285,8 +1286,15 @@ MagickExport HashmapInfo *NewHashmap(const unsigned long capacity,
 
   hashmap_info=(HashmapInfo *) AcquireMagickMemory(sizeof(*hashmap_info));
   if (hashmap_info == (HashmapInfo *) NULL)
-    ThrowMagickFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-      strerror(errno));
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowMagickFatalException(ResourceLimitFatalError,
+        "MemoryAllocationFailed",message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
   (void) ResetMagickMemory(hashmap_info,0,sizeof(*hashmap_info));
   hashmap_info->hash=HashPointerType;
   if (hash != (size_t (*)(const void *)) NULL)
@@ -1298,11 +1306,18 @@ MagickExport HashmapInfo *NewHashmap(const unsigned long capacity,
   hashmap_info->relinquish_value=relinquish_value;
   hashmap_info->entries=0;
   hashmap_info->capacity=capacity;
-  hashmap_info->map=(LinkedListInfo **)
-    AcquireMagickMemory((size_t) (capacity+1)*sizeof(*hashmap_info->map));
+  hashmap_info->map=(LinkedListInfo **) AcquireMagickMemory((size_t)
+    (capacity+1)*sizeof(*hashmap_info->map));
   if (hashmap_info->map == (LinkedListInfo **) NULL)
-    ThrowMagickFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-      strerror(errno));
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowMagickFatalException(ResourceLimitFatalError,
+        "MemoryAllocationFailed",message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
   (void) ResetMagickMemory(hashmap_info->map,0,(size_t) capacity*
     sizeof(*hashmap_info->map));
   hashmap_info->debug=IsEventLogging();
@@ -1341,15 +1356,22 @@ MagickExport LinkedListInfo *NewLinkedList(const unsigned long capacity)
 
   list_info=(LinkedListInfo *) AcquireMagickMemory(sizeof(*list_info));
   if (list_info == (LinkedListInfo *) NULL)
-    ThrowMagickFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-      strerror(errno));
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowMagickFatalException(ResourceLimitFatalError,
+        "MemoryAllocationFailed",message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
   (void) ResetMagickMemory(list_info,0,sizeof(*list_info));
   list_info->capacity=capacity == 0 ? (unsigned long) (~0) : capacity;
   list_info->elements=0;
   list_info->head=(ElementInfo *) NULL;
   list_info->tail=(ElementInfo *) NULL;
   list_info->next=(ElementInfo *) NULL;
-  list_info->debug=IsEventLogging();
+  list_info->debug=MagickFalse;
   list_info->semaphore=(SemaphoreInfo *) NULL;
   list_info->signature=MagickSignature;
   return(list_info);
@@ -1437,7 +1459,7 @@ static MagickBooleanType IncreaseHashmapCapacity(HashmapInfo *hashmap_info)
     list_info=hashmap_info->map[i];
     if (list_info == (LinkedListInfo *) NULL)
       continue;
-    list_info->semaphore=AcquireSemaphoreInfo(list_info->semaphore);
+    AcquireSemaphoreInfo(&list_info->semaphore);
     for (next=list_info->head; next != (ElementInfo *) NULL; )
     {
       element=next;
@@ -1488,7 +1510,7 @@ MagickExport MagickBooleanType PutEntryInHashmap(HashmapInfo *hashmap_info,
   next=(EntryInfo *) AcquireMagickMemory(sizeof(*next));
   if (next == (EntryInfo *) NULL)
     return(MagickFalse);
-  hashmap_info->semaphore=AcquireSemaphoreInfo(hashmap_info->semaphore);
+  AcquireSemaphoreInfo(&hashmap_info->semaphore);
   next->hash=hashmap_info->hash(key);
   next->key=(void *) key;
   next->value=(void *) value;
@@ -1580,9 +1602,9 @@ MagickExport void *RemoveElementByValueFromLinkedList(LinkedListInfo *list_info,
   assert(list_info->signature == MagickSignature);
   if (list_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
-  if ((list_info->elements == 0) || (value == NULL))
+  if ((list_info->elements == 0) || (value == (const void *) NULL))
     return((void *) NULL);
-  list_info->semaphore=AcquireSemaphoreInfo(list_info->semaphore);
+  AcquireSemaphoreInfo(&list_info->semaphore);
   if (value == list_info->head->value)
     {
       if (list_info->next == list_info->head)
@@ -1662,7 +1684,7 @@ MagickExport void *RemoveElementFromLinkedList(LinkedListInfo *list_info,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   if (index >= list_info->elements)
     return((void *) NULL);
-  list_info->semaphore=AcquireSemaphoreInfo(list_info->semaphore);
+  AcquireSemaphoreInfo(&list_info->semaphore);
   if (index == 0)
     {
       if (list_info->next == list_info->head)
@@ -1740,9 +1762,9 @@ MagickExport void *RemoveEntryFromHashmap(HashmapInfo *hashmap_info,
   assert(hashmap_info->signature == MagickSignature);
   if (hashmap_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
-  if (key == NULL)
+  if (key == (const void *) NULL)
     return((void *) NULL);
-  hashmap_info->semaphore=AcquireSemaphoreInfo(hashmap_info->semaphore);
+  AcquireSemaphoreInfo(&hashmap_info->semaphore);
   hash=hashmap_info->hash(key);
   list_info=hashmap_info->map[hash % hashmap_info->capacity];
   if (list_info != (LinkedListInfo *) NULL)
@@ -1818,7 +1840,7 @@ MagickExport void *RemoveLastElementFromLinkedList(LinkedListInfo *list_info)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   if (list_info->elements == 0)
     return((void *) NULL);
-  list_info->semaphore=AcquireSemaphoreInfo(list_info->semaphore);
+  AcquireSemaphoreInfo(&list_info->semaphore);
   if (list_info->next == list_info->tail)
     list_info->next=(ElementInfo *) NULL;
   if (list_info->elements == 1UL)
