@@ -17,7 +17,7 @@
 %                                 July 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -49,7 +49,6 @@
 #include "magick/locale_.h"
 #include "magick/log.h"
 #include "magick/memory_.h"
-#include "magick/nt-feature.h"
 #include "magick/semaphore.h"
 #include "magick/splay-tree.h"
 #include "magick/string_.h"
@@ -113,7 +112,7 @@ static MagickBooleanType
 */
 MagickExport void DestroyLocaleList(void)
 {
-  locale_semaphore=AcquireSemaphoreInfo(locale_semaphore);
+  AcquireSemaphoreInfo(&locale_semaphore);
   if (locale_list != (SplayTreeInfo *) NULL)
     locale_list=DestroySplayTree(locale_list);
   instantiate_locale=MagickFalse;
@@ -207,7 +206,7 @@ MagickExport const LocaleInfo *GetLocaleInfo_(const char *tag,
   /*
     Search for named tag.
   */
-  locale_semaphore=AcquireSemaphoreInfo(locale_semaphore);
+  AcquireSemaphoreInfo(&locale_semaphore);
   ResetSplayTreeIterator(locale_list);
   p=(const LocaleInfo *) GetNextValueInSplayTree(locale_list);
   while (p != (const LocaleInfo *) NULL)
@@ -300,7 +299,7 @@ MagickExport const LocaleInfo **GetLocaleInfoList(const char *pattern,
   /*
     Generate locale list.
   */
-  locale_semaphore=AcquireSemaphoreInfo(locale_semaphore);
+  AcquireSemaphoreInfo(&locale_semaphore);
   ResetSplayTreeIterator(locale_list);
   p=(const LocaleInfo *) GetNextValueInSplayTree(locale_list);
   for (i=0; p != (const LocaleInfo *) NULL; )
@@ -388,19 +387,19 @@ MagickExport char **GetLocaleList(const char *pattern,
   p=GetLocaleInfo_("*",exception);
   if (p == (const LocaleInfo *) NULL)
     return((char **) NULL);
-  locale_semaphore=AcquireSemaphoreInfo(locale_semaphore);
+  AcquireSemaphoreInfo(&locale_semaphore);
   RelinquishSemaphoreInfo(locale_semaphore);
   messages=(char **) AcquireMagickMemory((size_t)
     (GetNumberOfNodesInSplayTree(locale_list)+1)*sizeof(*messages));
   if (messages == (char **) NULL)
     return((char **) NULL);
-  locale_semaphore=AcquireSemaphoreInfo(locale_semaphore);
+  AcquireSemaphoreInfo(&locale_semaphore);
   p=(const LocaleInfo *) GetNextValueInSplayTree(locale_list);
   for (i=0; p != (const LocaleInfo *) NULL; )
   {
     if ((p->stealth == MagickFalse) &&
         (GlobExpression(p->tag,pattern) != MagickFalse))
-      messages[i++]=ConstantString(AcquireString(p->tag));
+      messages[i++]=ConstantString(p->tag);
     p=(const LocaleInfo *) GetNextValueInSplayTree(locale_list);
   }
   RelinquishSemaphoreInfo(locale_semaphore);
@@ -450,7 +449,7 @@ MagickExport const char *GetLocaleMessage(const char *tag)
   GetExceptionInfo(&exception);
   (void) FormatMagickString(name,MaxTextExtent,"%s/",tag);
   locale_info=GetLocaleInfo_(name,&exception);
-  DestroyExceptionInfo(&exception);
+  (void) DestroyExceptionInfo(&exception);
   if (locale_info != (const LocaleInfo *) NULL)
     return(locale_info->message);
   return(tag);
@@ -517,7 +516,7 @@ MagickExport LinkedListInfo *GetLocaleOptions(const char *filename,
         (void) FormatMagickString(path,MaxTextExtent,"%s%s",element,filename);
         (void) LogMagickEvent(LocaleEvent,GetMagickModule(),
           "Searching for locale file: \"%s\"",path);
-        xml=FileToStringInfo(path);
+        xml=ConfigureFileToStringInfo(path);
         if (xml != (StringInfo *) NULL)
           (void) AppendValueToLinkedList(messages,xml);
         element=(const char *) GetNextValueInLinkedList(paths);
@@ -599,25 +598,32 @@ static MagickBooleanType InitializeLocaleList(ExceptionInfo *exception)
   if ((locale_list == (SplayTreeInfo *) NULL) &&
       (instantiate_locale == MagickFalse))
     {
-      locale_semaphore=AcquireSemaphoreInfo(locale_semaphore);
+      AcquireSemaphoreInfo(&locale_semaphore);
       if ((locale_list == (SplayTreeInfo *) NULL) &&
           (instantiate_locale == MagickFalse))
         {
           char
             *locale;
 
-          locale=setlocale(LC_CTYPE,0);
-          if ((locale == (char *) NULL) || (*locale == '\0'))
-            locale=getenv("LC_ALL");
-          if ((locale == (char *) NULL) || (*locale == '\0'))
-            locale=getenv("LC_MESSAGES");
-          if ((locale == (char *) NULL) || (*locale == '\0'))
-            locale=getenv("LC_CTYPE");
-          if ((locale == (char *) NULL) || (*locale == '\0'))
-            locale=getenv("LANG");
-          if ((locale == (char *) NULL) || (*locale == '\0'))
-            locale="C";
+          register const char
+            *p;
+
+          locale=(char *) NULL;
+          p=setlocale(LC_CTYPE,0);
+          if (p != (const char *) NULL)
+            locale=ConstantString(p);
+          if (locale == (char *) NULL)
+            locale=GetEnvironmentValue("LC_ALL");
+          if (locale == (char *) NULL)
+            locale=GetEnvironmentValue("LC_MESSAGES");
+          if (locale == (char *) NULL)
+            locale=GetEnvironmentValue("LC_CTYPE");
+          if (locale == (char *) NULL)
+            locale=GetEnvironmentValue("LANG");
+          if (locale == (char *) NULL)
+            locale=ConstantString("C");
           (void) LoadLocaleLists(LocaleFilename,locale,exception);
+          locale=(char *) RelinquishMagickMemory(locale);
           instantiate_locale=MagickTrue;
         }
       RelinquishSemaphoreInfo(locale_semaphore);
@@ -816,13 +822,30 @@ static MagickBooleanType LoadLocaleList(const char *xml,const char *filename,
     if (*token == '\0')
       break;
     (void) CopyMagickString(keyword,token,MaxTextExtent);
+    if (LocaleNCompare(keyword,"<!DOCTYPE",9) == 0)
+      {
+        /*
+          Doctype element.
+        */
+        while ((LocaleNCompare(q,"]>",2) != 0) && (*q != '\0'))
+        {
+          GetMagickToken(q,&q,token);
+          while (isspace((int) ((unsigned char) *q)) != 0)
+            q++;
+        }
+        continue;
+      }
     if (LocaleNCompare(keyword,"<!--",4) == 0)
       {
         /*
           Comment element.
         */
         while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
+        {
           GetMagickToken(q,&q,token);
+          while (isspace((int) ((unsigned char) *q)) != 0)
+            q++;
+        }
         continue;
       }
     if (LocaleCompare(keyword,"<include") == 0)
@@ -860,7 +883,7 @@ static MagickBooleanType LoadLocaleList(const char *xml,const char *filename,
                     (void) ConcatenateMagickString(path,DirectorySeparator,
                       MaxTextExtent);
                   (void) ConcatenateMagickString(path,token,MaxTextExtent);
-                  xml=FileToString(path);
+                  xml=FileToString(path,~0,exception);
                   status|=LoadLocaleList(xml,path,locale,depth+1,exception);
                   xml=(char *) RelinquishMagickMemory(xml);
                 }
@@ -923,12 +946,12 @@ static MagickBooleanType LoadLocaleList(const char *xml,const char *filename,
           ThrowMagickFatalException(ResourceLimitFatalError,
             "MemoryAllocationFailed",filename);
         (void) ResetMagickMemory(locale_info,0,sizeof(*locale_info));
-        locale_info->path=ConstantString(AcquireString(filename));
-        locale_info->tag=ConstantString(AcquireString(tag));
-        locale_info->message=ConstantString(AcquireString(message));
+        locale_info->path=ConstantString(filename);
+        locale_info->tag=ConstantString(tag);
+        locale_info->message=ConstantString(message);
         locale_info->signature=MagickSignature;
-        status=AddValueToSplayTree(locale_list,
-          ConstantString(AcquireString(locale_info->tag)),locale_info);
+        status=AddValueToSplayTree(locale_list,ConstantString(locale_info->tag),
+          locale_info);
         if (status == MagickFalse)
           (void) ThrowMagickException(exception,GetMagickModule(),
             ResourceLimitError,"MemoryAllocationFailed","`%s'",
@@ -1006,6 +1029,9 @@ static MagickBooleanType LoadLocaleList(const char *xml,const char *filename,
 static MagickBooleanType LoadLocaleLists(const char *filename,
   const char *locale,ExceptionInfo *exception)
 {
+#if defined(UseEmbeddableMagick)
+  return(LoadLocaleList(LocaleMap,"built-in",locale,0,exception));
+#else
   const StringInfo
     *option;
 
@@ -1042,4 +1068,5 @@ static MagickBooleanType LoadLocaleLists(const char *filename,
       (GetNumberOfNodesInSplayTree(locale_list) == 0))
     status|=LoadLocaleList(LocaleMap,"built-in",locale,0,exception);
   return(status != 0 ? MagickTrue : MagickFalse);
+#endif
 }
