@@ -55,6 +55,12 @@
 #include "magick/token.h"
 #include "magick/utility.h"
 #include "magick/version.h"
+#if defined(HAVE_PTHREAD)
+#include <pthread.h>
+#endif
+#if defined(__WINDOWS__)
+#include <windows.h>
+#endif
 
 /*
   Define declarations.
@@ -64,6 +70,18 @@
 /*
   Typedef declarations.
 */
+typedef enum
+{
+  UndefinedHandler = 0x0000,
+  NoHandler = 0x0000,
+  ConsoleHandler = 0x0001,
+  StdoutHandler = 0x0002,
+  StderrHandler = 0x0004,
+  FileHandler = 0x0008,
+  DebugHandler = 0x0010,
+  EventHandler = 0x0020
+} LogHandlerType;
+
 typedef struct _EventInfo
 {
   char
@@ -75,12 +93,54 @@ typedef struct _EventInfo
 
 typedef struct _HandlerInfo
 {
-  char
+  const char
     *name;
 
   LogHandlerType
     handler;
 } HandlerInfo;
+
+struct _LogInfo
+{
+  LogEventType
+    event_mask;
+
+  LogHandlerType
+    handler_mask;
+
+  char
+    *path,
+    *name,
+    *filename,
+    *format;
+
+  unsigned long
+    generations,
+    limit;
+
+  FILE
+    *file;
+
+  MagickBooleanType
+    append;
+
+  unsigned long
+    generation,
+    count;
+
+  MagickBooleanType
+    stealth;
+
+  TimerInfo
+    timer;
+
+  struct _LogInfo
+    *previous,
+    *next;  /* deprecated, use GetLogInfoList() */
+
+  unsigned long
+    signature;
+};
 
 /*
   Declare log map.
@@ -89,15 +149,17 @@ static const HandlerInfo
   LogHandlers[] =
   {
     { "console", ConsoleHandler },
+    { "debug", DebugHandler },
+    { "event", EventHandler },
     { "file", FileHandler },
     { "none", NoHandler },
-    { "stdout", StdoutHandler },
     { "stderr", StderrHandler },
+    { "stdout", StdoutHandler },
     { (char *) NULL, UndefinedHandler }
   };
 
 static const char
-  *LogMap = (char *)
+  *LogMap = (const char *)
     "<?xml version=\"1.0\"?>"
     "<logmap>"
     "  <log events=\"None\" />"
@@ -105,12 +167,15 @@ static const char
     "  <log filename=\"Magick-%d.log\" />"
     "  <log generations=\"3\" />"
     "  <log limit=\"2000\" />"
-    "  <log format=\"%t %r %u %v %c[%p]: %m/%f/%l/%d\n  %e\" />"
+    "  <log format=\"%t %r %u %v %d %c[%p]: %m/%f/%l/%d\n  %e\" />"
     "</logmap>";
 
 /*
   Static declarations.
 */
+static char
+  log_name[MaxTextExtent] = "Magick";
+
 static LinkedListInfo
   *log_list = (LinkedListInfo *) NULL;
 
@@ -157,11 +222,11 @@ static void *DestroyLogElement(void *log_info)
 
   p=(LogInfo *) log_info;
   if (p->file != (FILE *) NULL)
-    if (LocaleCompare(p->format,"xml") == 0)
-      {
+    {
+      if (p->append == MagickFalse)
         (void) fprintf(p->file,"</log>\n");
-        (void) fclose(p->file);
-      }
+      (void) fclose(p->file);
+    }
   if (p->filename != (char *) NULL)
     p->filename=(char *) RelinquishMagickMemory(p->filename);
   if (p->path != (char *) NULL)
@@ -174,10 +239,9 @@ static void *DestroyLogElement(void *log_info)
 
 MagickExport void DestroyLogList(void)
 {
-  log_semaphore=AcquireSemaphoreInfo(log_semaphore);
+  AcquireSemaphoreInfo(&log_semaphore);
   if (log_list != (LinkedListInfo *) NULL)
     log_list=DestroyLinkedList(log_list,DestroyLogElement);
-  log_list=(LinkedListInfo *) NULL;
   instantiate_log=MagickFalse;
   RelinquishSemaphoreInfo(log_semaphore);
   log_semaphore=DestroySemaphoreInfo(log_semaphore);
@@ -227,7 +291,7 @@ MagickExport const LogInfo *GetLogInfo(const char *name,
   /*
     Search for named log.
   */
-  log_semaphore=AcquireSemaphoreInfo(log_semaphore);
+  AcquireSemaphoreInfo(&log_semaphore);
   ResetLinkedListIterator(log_list);
   p=(const LogInfo *) GetNextValueInLinkedList(log_list);
   while (p != (const LogInfo *) NULL)
@@ -238,7 +302,7 @@ MagickExport const LogInfo *GetLogInfo(const char *name,
   }
   if (p == (LogInfo *) NULL)
     (void) ThrowMagickException(exception,GetMagickModule(),OptionWarning,
-      "UnrecognizedLog","`%s'",name);
+      "NoSuchElement","`%s'",name);
   RelinquishSemaphoreInfo(log_semaphore);
   return(p);
 }
@@ -320,7 +384,7 @@ MagickExport const LogInfo **GetLogInfoList(const char *pattern,
   /*
     Generate log list.
   */
-  log_semaphore=AcquireSemaphoreInfo(log_semaphore);
+  AcquireSemaphoreInfo(&log_semaphore);
   ResetLinkedListIterator(log_list);
   p=(const LogInfo *) GetNextValueInLinkedList(log_list);
   for (i=0; p != (const LogInfo *) NULL; )
@@ -371,12 +435,12 @@ extern "C" {
 
 static int LogCompare(const void *x,const void *y)
 {
-  register char
+  register const char
     **p,
     **q;
 
-  p=(char **) x;
-  q=(char **) y;
+  p=(const char **) x;
+  q=(const char **) y;
   return(LocaleCompare(*p,*q));
 }
 
@@ -413,14 +477,14 @@ MagickExport char **GetLogList(const char *pattern,
   /*
     Generate log list.
   */
-  log_semaphore=AcquireSemaphoreInfo(log_semaphore);
+  AcquireSemaphoreInfo(&log_semaphore);
   ResetLinkedListIterator(log_list);
   p=(const LogInfo *) GetNextValueInLinkedList(log_list);
   for (i=0; p != (const LogInfo *) NULL; )
   {
     if ((p->stealth == MagickFalse) &&
         (GlobExpression(p->name,pattern) != MagickFalse))
-      preferences[i++]=ConstantString(AcquireString(p->name));
+      preferences[i++]=ConstantString(p->name);
     p=(const LogInfo *) GetNextValueInLinkedList(log_list);
   }
   RelinquishSemaphoreInfo(log_semaphore);
@@ -435,6 +499,29 @@ MagickExport char **GetLogList(const char *pattern,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   G e t L o g N a m e                                                       %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetLogName() returns the current log name.
+%
+%  The format of the GetLogName method is:
+%
+%      const char *GetLogName(void)
+%
+*/
+MagickExport const char *GetLogName(void)
+{
+  return(log_name);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 +   I n i t i a l i z e L o g L i s t                                         %
 %                                                                             %
 %                                                                             %
@@ -456,7 +543,7 @@ static MagickBooleanType InitializeLogList(ExceptionInfo *exception)
 {
   if ((log_list == (LinkedListInfo *) NULL) && (instantiate_log == MagickFalse))
     {
-      log_semaphore=AcquireSemaphoreInfo(log_semaphore);
+      AcquireSemaphoreInfo(&log_semaphore);
       if ((log_list == (LinkedListInfo *) NULL) &&
           (instantiate_log == MagickFalse))
         {
@@ -501,7 +588,7 @@ MagickExport MagickBooleanType IsEventLogging(void)
     return(MagickFalse);
   GetExceptionInfo(&exception);
   log_info=GetLogInfo("*",&exception);
-  DestroyExceptionInfo(&exception);
+  (void) DestroyExceptionInfo(&exception);
   return((MagickBooleanType) (log_info->event_mask != NoEvents));
 }
 /*
@@ -619,6 +706,17 @@ MagickExport MagickBooleanType ListLogInfo(FILE *file,ExceptionInfo *exception)
 %
 */
 
+static unsigned long GetLogThreadId(void)
+{
+#if defined(HAVE_PTHREAD)
+  return((unsigned long) pthread_self());
+#endif
+#if defined(__WINDOWS__)
+  return((unsigned long) GetCurrentThreadId());
+#endif
+  return(0);
+}
+
 static char *TranslateEvent(const LogEventType type,const char *module,
   const char *function,const unsigned long line,const char *domain,
   const char *event)
@@ -636,12 +734,12 @@ static char *TranslateEvent(const LogEventType type,const char *module,
   ExceptionInfo
     exception;
 
-  register const char
-    *p;
-
   register char
     *q;
 
+  register const char
+    *p;
+
   size_t
     length;
 
@@ -653,13 +751,12 @@ static char *TranslateEvent(const LogEventType type,const char *module,
 
   GetExceptionInfo(&exception);
   log_info=(LogInfo *) GetLogInfo("*",&exception);
-  DestroyExceptionInfo(&exception);
+  (void) DestroyExceptionInfo(&exception);
   seconds=time((time_t *) NULL);
   time_meridian=localtime(&seconds);
   elapsed_time=GetElapsedTime(&log_info->timer);
   user_time=GetUserTime(&log_info->timer);
   text=AcquireString((char *) NULL);
-  length=MaxTextExtent;
   if (log_info->format == (char *) NULL)
     return(text);
   if (LocaleCompare(log_info->format,"xml") == 0)
@@ -670,29 +767,28 @@ static char *TranslateEvent(const LogEventType type,const char *module,
       /*
         Translate event in "XML" format.
       */
-      (void) FormatMagickString(timestamp,MaxTextExtent,
-        "%04d%02d%02d%02d%02d%02d",time_meridian->tm_year+1900,
-        time_meridian->tm_mon+1,time_meridian->tm_mday,time_meridian->tm_hour,
-        time_meridian->tm_min,time_meridian->tm_sec);
-      (void) FormatMagickString(text,length,
-        "<record>\n"
+      (void) FormatMagickTime(seconds,MaxTextExtent,timestamp);
+      (void) FormatMagickString(text,MaxTextExtent,
+        "<entry>\n"
         "  <timestamp>%s</timestamp>\n"
         "  <elapsed-time>%ld:%02ld</elapsed-time>\n"
         "  <user-time>%0.3f</user-time>\n"
-        "  <pid>%ld</pid>\n"
+        "  <process-id>%ld</process-id>\n"
+        "  <thread-id>%lu</thread-id>\n"
         "  <module>%s</module>\n"
         "  <function>%s</function>\n"
         "  <line>%lu</line>\n"
         "  <domain>%s</domain>\n"
         "  <event>%s</event>\n"
-        "</record>\n",timestamp,(long) (elapsed_time/60.0),
-        (long) ceil(fmod(elapsed_time,60.0)),user_time,(long) getpid(),module,
-        function,line,domain,event);
+        "</entry>",timestamp,(long) (elapsed_time/60.0),
+        (long) ceil(fmod(elapsed_time,60.0)),user_time,(long) getpid(),
+        GetLogThreadId(),module,function,line,domain,event);
       return(text);
     }
   /*
     Translate event in "human readable" format.
   */
+  length=MaxTextExtent;
   q=text;
   for (p=log_info->format; *p != '\0'; p++)
   {
@@ -709,11 +805,15 @@ static char *TranslateEvent(const LogEventType type,const char *module,
     /*
       The format of the log is defined by embedding special format characters:
 
+        %c   client name
         %d   domain
         %e   event
         %f   function
+				%g   generation
+        %i   thread id
         %l   line
         %m   module
+        %n   log name
         %p   process id
         %r   real CPU time
         %t   wall clock time
@@ -767,6 +867,21 @@ static char *TranslateEvent(const LogEventType type,const char *module,
         q+=strlen(function);
         break;
       }
+      case 'g':
+      {
+        (void) FormatMagickString(q,MaxTextExtent,"%lu",
+          log_info->generation % log_info->generations);
+        while (*q != '\0')
+          q++;
+        break;
+      }
+      case 'i':
+      {
+        (void) FormatMagickString(q,MaxTextExtent,"%lu",GetLogThreadId());
+        while (*q != '\0')
+          q++;
+        break;
+      }
       case 'l':
       {
         (void) FormatMagickString(q,MaxTextExtent,"%lu",line);
@@ -789,6 +904,12 @@ static char *TranslateEvent(const LogEventType type,const char *module,
         q+=strlen(p);
         break;
       }
+      case 'n':
+      {
+        (void) CopyMagickString(q,GetLogName(),MaxTextExtent);
+        q+=strlen(GetLogName());
+        break;
+      }
       case 'p':
       {
         (void) FormatMagickString(q,MaxTextExtent,"%ld",(long) getpid());
@@ -836,9 +957,113 @@ static char *TranslateEvent(const LogEventType type,const char *module,
   return(text);
 }
 
-MagickExport MagickBooleanType LogMagickEvent(const LogEventType type,
-  const char *module,const char *function,const unsigned long line,
-  const char *format,...)
+static char *TranslateFilename(const LogInfo *log_info)
+{
+  char
+    *filename;
+
+  size_t
+    length;
+
+  register char
+    *q;
+
+  register const char
+    *p;
+
+  /*
+    Translate event in "human readable" format.
+  */
+  assert(log_info != (LogInfo *) NULL);
+  assert(log_info->filename != (char *) NULL);
+  filename=AcquireString((char *) NULL);
+  length=MaxTextExtent;
+  q=filename;
+  for (p=log_info->filename; *p != '\0'; p++)
+  {
+    *q='\0';
+    if ((size_t) (q-filename+MaxTextExtent) >= length)
+      {
+        length<<=1;
+        filename=(char *) ResizeMagickMemory(filename,(length+MaxTextExtent)*
+          sizeof(*filename));
+        if (filename == (char *) NULL)
+          return((char *) NULL);
+        q=filename+strlen(filename);
+      }
+    /*
+      The format of the filename is defined by embedding special format
+      characters:
+
+        %c   client name
+        %n   log name
+        %i   thread id
+        %p   process id
+        %v   version
+        %%   percent sign
+    */
+    if (*p != '%')
+      {
+        *q++=(*p);
+        continue;
+      }
+    p++;
+    switch (*p)
+    {
+      case 'c':
+      {
+        (void) CopyMagickString(q,GetClientName(),MaxTextExtent);
+        q+=strlen(GetClientName());
+        break;
+      }
+      case 'g':
+      {
+        (void) FormatMagickString(q,MaxTextExtent,"%lu",
+          log_info->generation % log_info->generations);
+        while (*q != '\0')
+          q++;
+        break;
+      }
+      case 'i':
+      {
+        (void) FormatMagickString(q,MaxTextExtent,"%lu",GetLogThreadId());
+        while (*q != '\0')
+          q++;
+        break;
+      }
+      case 'n':
+      {
+        (void) CopyMagickString(q,GetLogName(),MaxTextExtent);
+        q+=strlen(GetLogName());
+        break;
+      }
+      case 'p':
+      {
+        (void) FormatMagickString(q,MaxTextExtent,"%ld",(long) getpid());
+        while (*q != '\0')
+          q++;
+        break;
+      }
+      case 'v':
+      {
+        (void) CopyMagickString(q,MagickLibVersionText,MaxTextExtent);
+        q+=strlen(MagickLibVersionText);
+        break;
+      }
+      default:
+      {
+        *q++=(*p);
+        break;
+      }
+    }
+  }
+  *q='\0';
+  return(filename);
+}
+
+MagickBooleanType LogMagickEventList(const LogEventType type,const char *module,
+  const char *function,const unsigned long line,const char *format,
+  va_list operands)
 {
   char
     event[MaxTextExtent],
@@ -856,30 +1081,25 @@ MagickExport MagickBooleanType LogMagickEvent(const LogEventType type,
   LogInfo
     *log_info;
 
-  va_list
-    operands;
-
   if (IsEventLogging() == MagickFalse)
     return(MagickFalse);
   GetExceptionInfo(&exception);
   log_info=(LogInfo *) GetLogInfo("*",&exception);
-  DestroyExceptionInfo(&exception);
-  log_semaphore=AcquireSemaphoreInfo(log_semaphore);
+  (void) DestroyExceptionInfo(&exception);
+  AcquireSemaphoreInfo(&log_semaphore);
   if ((log_info->event_mask & type) == 0)
     {
       RelinquishSemaphoreInfo(log_semaphore);
       return(MagickTrue);
     }
   domain=MagickOptionToMnemonic(MagickLogEventOptions,type);
-  va_start(operands,format);
 #if defined(HAVE_VSNPRINTF)
   n=vsnprintf(event,MaxTextExtent,format,operands);
 #else
   n=vsprintf(event,format,operands);
 #endif
-  if (n < 0)
+  if  (n < 0)
     event[MaxTextExtent-1]='\0';
-  va_end(operands);
   text=TranslateEvent(type,module,function,line,domain,event);
   if (text == (char *) NULL)
     {
@@ -891,10 +1111,60 @@ MagickExport MagickBooleanType LogMagickEvent(const LogEventType type,
     {
       (void) fprintf(stdout,"%s\n",text);
       (void) fflush(stdout);
+    }
+  if ((log_info->handler_mask & DebugHandler) != 0)
+    {
 #if defined(__WINDOWS__)
       OutputDebugString(text);
 #endif
     }
+  if ((log_info->handler_mask & EventHandler) != 0)
+    {
+#if defined(__WINDOWS__)
+      (void) NTReportEvent(text,MagickFalse);
+#endif
+    }
+  if ((log_info->handler_mask & FileHandler) != 0)
+    {
+      log_info->count++;
+      if (log_info->count == log_info->limit)
+        {
+          (void) fprintf(log_info->file,"</log>\n");
+          (void) fclose(log_info->file);
+          log_info->file=(FILE *) NULL;
+          log_info->count=0;
+        }
+      if (log_info->file == (FILE *) NULL)
+        {
+          char
+            *filename;
+
+          filename=TranslateFilename(log_info);
+          if (filename == (char *) NULL)
+            {
+              (void) ContinueTimer((TimerInfo *) &log_info->timer);
+              RelinquishSemaphoreInfo(log_semaphore);
+              return(MagickFalse);
+            }
+          log_info->append=IsAccessible(filename);
+          log_info->file=fopen(filename,"ab");
+          filename=(char  *) RelinquishMagickMemory(filename);
+          if (log_info->file == (FILE *) NULL)
+            {
+              RelinquishSemaphoreInfo(log_semaphore);
+              return(MagickFalse);
+            }
+          log_info->generation++;
+          if (log_info->append == MagickFalse)
+            {
+              (void) fprintf(log_info->file,"<?xml version=\"1.0\" "
+                "encoding=\"UTF-8\" standalone=\"yes\"?>\n");
+              (void) fprintf(log_info->file,"<log>\n");
+            }
+        }
+      (void) fprintf(log_info->file,"%s\n",text);
+      (void) fflush(log_info->file);
+    }
   if ((log_info->handler_mask & StdoutHandler) != 0)
     {
       (void) fprintf(stdout,"%s\n",text);
@@ -905,40 +1175,26 @@ MagickExport MagickBooleanType LogMagickEvent(const LogEventType type,
       (void) fprintf(stderr,"%s\n",text);
       (void) fflush(stderr);
     }
-  if ((log_info->handler_mask & FileHandler) != 0)
-    {
-      log_info->count++;
-      if (log_info->count == log_info->limit)
-        {
-          (void) fprintf(log_info->file,"</log>\n");
-          (void) fclose(log_info->file);
-          log_info->file=(FILE *) NULL;
-        }
-      if (log_info->file == (FILE *) NULL)
-        {
-          char
-            filename[MaxTextExtent];
-
-          (void) FormatMagickString(filename,MaxTextExtent,log_info->filename,
-            log_info->generation % log_info->generations);
-          log_info->file=fopen(filename,"w");
-          if (log_info->file == (FILE *) NULL)
-            {
-              RelinquishSemaphoreInfo(log_semaphore);
-              return(MagickFalse);
-            }
-          log_info->generation++;
-          (void) fprintf(log_info->file,"<?xml version=\"1.0\"?>\n");
-          (void) fprintf(log_info->file,"<log>\n");
-        }
-      (void) fprintf(log_info->file,"%s\n",text);
-      (void) fflush(log_info->file);
-    }
-  text=(char *) RelinquishMagickMemory(text);
+  text=(char  *) RelinquishMagickMemory(text);
   (void) ContinueTimer((TimerInfo *) &log_info->timer);
   RelinquishSemaphoreInfo(log_semaphore);
   return(MagickTrue);
 }
+
+MagickBooleanType LogMagickEvent(const LogEventType type,const char *module,
+  const char *function,const unsigned long line,const char *format,...)
+{
+  va_list
+    operands;
+
+  MagickBooleanType
+    status;
+
+  va_start(operands,format);
+  status=LogMagickEventList(type,module,function,line,format,operands);
+  va_end(operands);
+  return(status);
+}
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -1011,6 +1267,15 @@ static MagickBooleanType LoadLogList(const char *xml,const char *filename,
     if (*token == '\0')
       break;
     (void) CopyMagickString(keyword,token,MaxTextExtent);
+    if (LocaleNCompare(keyword,"<!DOCTYPE",9) == 0)
+      {
+        /*
+          Doctype element.
+        */
+        while ((LocaleNCompare(q,"]>",2) != 0) && (*q != '\0'))
+          GetMagickToken(q,&q,token);
+        continue;
+      }
     if (LocaleNCompare(keyword,"<!--",4) == 0)
       {
         /*
@@ -1048,7 +1313,7 @@ static MagickBooleanType LoadLogList(const char *xml,const char *filename,
                     (void) ConcatenateMagickString(path,DirectorySeparator,
                       MaxTextExtent);
                   (void) ConcatenateMagickString(path,token,MaxTextExtent);
-                  xml=FileToString(path);
+                  xml=FileToString(path,~0,exception);
                   status|=LoadLogList(xml,path,depth+1,exception);
                   xml=(char *) RelinquishMagickMemory(xml);
                 }
@@ -1066,7 +1331,7 @@ static MagickBooleanType LoadLogList(const char *xml,const char *filename,
           ThrowMagickFatalException(ResourceLimitFatalError,
             "MemoryAllocationFailed",filename);
         (void) ResetMagickMemory(log_info,0,sizeof(*log_info));
-        log_info->path=ConstantString(AcquireString(filename));
+        log_info->path=ConstantString(filename);
         GetTimerInfo((TimerInfo *) &log_info->timer);
         log_info->signature=MagickSignature;
         continue;
@@ -1107,7 +1372,7 @@ static MagickBooleanType LoadLogList(const char *xml,const char *filename,
             if (log_info->filename != (char *) NULL)
               log_info->filename=(char *)
                 RelinquishMagickMemory(log_info->filename);
-            log_info->filename=ConstantString(AcquireString(token));
+            log_info->filename=ConstantString(token);
             break;
           }
         if (LocaleCompare((char *) keyword,"format") == 0)
@@ -1115,7 +1380,7 @@ static MagickBooleanType LoadLogList(const char *xml,const char *filename,
             if (log_info->format != (char *) NULL)
               log_info->format=(char *)
                 RelinquishMagickMemory(log_info->format);
-            log_info->format=ConstantString(AcquireString(token));
+            log_info->format=ConstantString(token);
             break;
           }
         break;
@@ -1182,7 +1447,7 @@ static MagickBooleanType LoadLogList(const char *xml,const char *filename,
 %
 %  A description of each parameter follows:
 %
-%    o filename: The font file name.
+%    o filename: The log configuration filename.
 %
 %    o exception: Return any errors or warnings in this structure.
 %
@@ -1191,9 +1456,9 @@ static MagickBooleanType LoadLogList(const char *xml,const char *filename,
 static MagickBooleanType LoadLogLists(const char *filename,
   ExceptionInfo *exception)
 {
-  char
-    path[MaxTextExtent];
-
+#if defined(UseEmbeddableMagick)
+  return(LoadLogList(LogMap,"built-in",0,exception));
+#else
   const StringInfo
     *option;
 
@@ -1204,24 +1469,19 @@ static MagickBooleanType LoadLogLists(const char *filename,
     status;
 
   status=MagickFalse;
-  *path='\0';
   options=GetConfigureOptions(filename,exception);
   option=(const StringInfo *) GetNextValueInLinkedList(options);
   while (option != (const StringInfo *) NULL)
   {
-    (void) CopyMagickString(path,option->path,MaxTextExtent);
     status|=LoadLogList((const char *) option->datum,option->path,0,exception);
     option=(const StringInfo *) GetNextValueInLinkedList(options);
   }
   options=DestroyConfigureOptions(options);
-  if ((log_list == (LinkedListInfo *) NULL) || 
+  if ((log_list == (LinkedListInfo *) NULL) ||
       (IsLinkedListEmpty(log_list) != MagickFalse))
-    {
-      (void) ThrowMagickException(exception,GetMagickModule(),ConfigureWarning,
-        "UnableToOpenConfigureFile","`%s'",path);
-      status|=LoadLogList(LogMap,"built-in",0,exception);
-    }
+    status|=LoadLogList(LogMap,"built-in",0,exception);
   return(status != 0 ? MagickTrue : MagickFalse);
+#endif
 }
 
 /*
@@ -1313,20 +1573,21 @@ MagickExport LogEventType SetLogEventMask(const char *events)
   ExceptionInfo
     exception;
 
-  LogEventType
-    event_mask;
-
   LogInfo
     *log_info;
 
+  long
+    option;
+
   GetExceptionInfo(&exception);
   log_info=(LogInfo *) GetLogInfo("*",&exception);
-  DestroyExceptionInfo(&exception);
-  log_semaphore=AcquireSemaphoreInfo(log_semaphore);
-  event_mask=(LogEventType)
-    ParseMagickOption(MagickLogEventOptions,MagickTrue,events);
+  (void) DestroyExceptionInfo(&exception);
+  option=ParseMagickOption(MagickLogEventOptions,MagickTrue,events);
+  AcquireSemaphoreInfo(&log_semaphore);
   log_info=(LogInfo *) GetValueFromLinkedList(log_list,0);
-  log_info->event_mask=event_mask;
+  log_info->event_mask=(LogEventType) option;
+  if (option == -1)
+    log_info->event_mask=UndefinedEvents;
   RelinquishSemaphoreInfo(log_semaphore);
   return(log_info->event_mask);
 }
@@ -1365,10 +1626,41 @@ MagickExport void SetLogFormat(const char *format)
 
   GetExceptionInfo(&exception);
   log_info=(LogInfo *) GetLogInfo("*",&exception);
-  DestroyExceptionInfo(&exception);
-  log_semaphore=AcquireSemaphoreInfo(log_semaphore);
+  (void) DestroyExceptionInfo(&exception);
+  AcquireSemaphoreInfo(&log_semaphore);
   if (log_info->format != (char *) NULL)
     log_info->format=(char *) RelinquishMagickMemory(log_info->format);
-  log_info->format=ConstantString(AcquireString(format));
+  log_info->format=ConstantString(format);
   RelinquishSemaphoreInfo(log_semaphore);
 }
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   S e t L o g N a m e                                                       %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  SetLogName() sets the log name and returns it.
+%
+%  The format of the SetLogName method is:
+%
+%      const char *SetLogName(const char *name)
+%
+%  A description of each parameter follows:
+%
+%    o log_name: SetLogName() returns the current client name.
+%
+%    o name: Specifies the new client name.
+%
+*/
+MagickExport const char *SetLogName(const char *name)
+{
+  if ((name != (char *) NULL) && (*name != '\0'))
+    (void) CopyMagickString(log_name,name,MaxTextExtent);
+  return(log_name);
+}
