@@ -18,7 +18,7 @@
 %                             November 1998                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -58,7 +58,9 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/module.h"
-#include "magick/nt-feature.h"
+#if defined(__WINDOWS__)
+# include "magick/nt-feature.h"
+#endif
 #include "magick/random_.h"
 #include "magick/registry.h"
 #include "magick/resource_.h"
@@ -150,6 +152,7 @@ MagickExport void DestroyMagick(void)
   DestroyCoderList();
   DestroyMagickResources();
   DestroyMagickRegistry();
+  DestroyCacheResources();
   DestroyRandomReservoir();
   DestroyLocaleList();
   DestroyLogList();
@@ -182,7 +185,7 @@ MagickExport void DestroyMagickList(void)
 #if !defined(BuildMagickModules)
   UnregisterStaticModules();
 #endif
-  magick_semaphore=AcquireSemaphoreInfo(magick_semaphore);
+  AcquireSemaphoreInfo(&magick_semaphore);
   if (magick_list != (SplayTreeInfo *) NULL)
     magick_list=DestroySplayTree(magick_list);
   RelinquishSemaphoreInfo(magick_semaphore);
@@ -229,10 +232,10 @@ MagickExport const char *GetImageMagick(const unsigned char *magick,
   assert(magick != (const unsigned char *) NULL);
   GetExceptionInfo(&exception);
   p=GetMagickInfo("*",&exception);
-  DestroyExceptionInfo(&exception);
+  (void) DestroyExceptionInfo(&exception);
   if (p == (const MagickInfo *) NULL)
     return((const char *) NULL);
-  magick_semaphore=AcquireSemaphoreInfo(magick_semaphore);
+  AcquireSemaphoreInfo(&magick_semaphore);
   ResetSplayTreeIterator(magick_list);
   p=(const MagickInfo *) GetNextValueInSplayTree(magick_list);
   while (p != (const MagickInfo *) NULL)
@@ -245,7 +248,7 @@ MagickExport const char *GetImageMagick(const unsigned char *magick,
   RelinquishSemaphoreInfo(magick_semaphore);
   if (p != (MagickInfo *) NULL)
     return(p->name);
-  return((char *) NULL);
+  return((const char *) NULL);
 }
 
 /*
@@ -482,7 +485,7 @@ MagickExport const MagickInfo *GetMagickInfo(const char *name,
   /*
     Find name in list.
   */
-  magick_semaphore=AcquireSemaphoreInfo(magick_semaphore);
+  AcquireSemaphoreInfo(&magick_semaphore);
   ResetSplayTreeIterator(magick_list);
   p=(const MagickInfo *) GetNextValueInSplayTree(magick_list);
   while (p != (const MagickInfo *) NULL)
@@ -587,7 +590,7 @@ MagickExport const MagickInfo **GetMagickInfoList(const char *pattern,
   /*
     Generate magick list.
   */
-  magick_semaphore=AcquireSemaphoreInfo(magick_semaphore);
+  AcquireSemaphoreInfo(&magick_semaphore);
   ResetSplayTreeIterator(magick_list);
   p=(const MagickInfo *) GetNextValueInSplayTree(magick_list);
   for (i=0; p != (const MagickInfo *) NULL; )
@@ -638,12 +641,12 @@ extern "C" {
 
 static int MagickCompare(const void *x,const void *y)
 {
-  register char
+  register const char
     **p,
     **q;
 
-  p=(char **) x;
-  q=(char **) y;
+  p=(const char **) x;
+  q=(const char **) y;
   return(LocaleCompare(*p,*q));
 }
 
@@ -677,14 +680,14 @@ MagickExport char **GetMagickList(const char *pattern,
     (GetNumberOfNodesInSplayTree(magick_list)+1)*sizeof(*formats));
   if (formats == (char **) NULL)
     return((char **) NULL);
-  magick_semaphore=AcquireSemaphoreInfo(magick_semaphore);
+  AcquireSemaphoreInfo(&magick_semaphore);
   ResetSplayTreeIterator(magick_list);
   p=(const MagickInfo *) GetNextValueInSplayTree(magick_list);
   for (i=0; p != (const MagickInfo *) NULL; )
   {
     if ((p->stealth == MagickFalse) &&
         (GlobExpression(p->name,pattern) != MagickFalse))
-      formats[i++]=AcquireString(p->name);
+      formats[i++]=ConstantString(p->name);
     p=(const MagickInfo *) GetNextValueInSplayTree(magick_list);
   }
   RelinquishSemaphoreInfo(magick_semaphore);
@@ -831,7 +834,27 @@ static void MagickSignalHandler(int signal_number)
   if (signal_handlers[signal_number] != MagickSignalHandler)
     raise(signal_number);
 #endif
-  Exit(signal_number);
+#if !defined(HAVE__EXIT)
+  exit(signal_number);
+#else
+#if defined(SIGHUP)
+  if (signal_number == SIGHUP)
+    exit(signal_number);
+#endif
+#if defined(SIGBREAK)
+  if (signal_number == SIGBREAK)
+    exit(signal_number);
+#endif
+#if defined(SIGINT) && !defined(__WINDOWS__)
+  if (signal_number == SIGINT)
+    exit(signal_number);
+#endif
+#if defined(SIGTERM)
+  if (signal_number == SIGTERM)
+    exit(signal_number);
+#endif
+  _exit(signal_number);
+#endif
 }
 
 static SignalHandler *RegisterMagickSignalHandler(int signal_number)
@@ -853,11 +876,12 @@ static SignalHandler *RegisterMagickSignalHandler(int signal_number)
 MagickExport void InitializeMagick(const char *path)
 {
   char
+    *events,
     execution_path[MaxTextExtent],
     filename[MaxTextExtent];
 
-  const char
-    *p;
+  ExceptionInfo
+    exception;
 
   time_t
     seconds;
@@ -870,10 +894,14 @@ MagickExport void InitializeMagick(const char *path)
   seconds=time((time_t *) NULL);
   srand((unsigned int) (seconds+getpid()));
   InitializeSemaphore();
-  p=getenv("MAGICK_DEBUG");
-  if (p != (const char *) NULL)
-    (void) SetLogEventMask(p);
+  events=GetEnvironmentValue("MAGICK_DEBUG");
+  if (events != (char *) NULL)
+    {
+      (void) SetLogEventMask(events);
+      events=(char *) RelinquishMagickMemory(events);
+    }
 #if defined(__WINDOWS__)
+  (void) NTControlHandler();
 #if defined(_DEBUG) && !defined(__BORLANDC__)
   if (IsEventLogging() != MagickFalse)
     {
@@ -908,6 +936,10 @@ MagickExport void InitializeMagick(const char *path)
   if (signal_handlers[SIGABRT] == (SignalHandler *) NULL)
     signal_handlers[SIGABRT]=RegisterMagickSignalHandler(SIGABRT);
 #endif
+#if defined(SIGBREAK)
+  if (signal_handlers[SIGBREAK] == (SignalHandler *) NULL)
+    signal_handlers[SIGBREAK]=RegisterMagickSignalHandler(SIGBREAK);
+#endif
 #if defined(SIGFPE)
   if (signal_handlers[SIGFPE] == (SignalHandler *) NULL)
     signal_handlers[SIGFPE]=RegisterMagickSignalHandler(SIGFPE);
@@ -916,7 +948,7 @@ MagickExport void InitializeMagick(const char *path)
   if (signal_handlers[SIGHUP] == (SignalHandler *) NULL)
     signal_handlers[SIGHUP]=RegisterMagickSignalHandler(SIGHUP);
 #endif
-#if defined(SIGINT)
+#if defined(SIGINT) && !defined(__WINDOWS__)
   if (signal_handlers[SIGINT] == (SignalHandler *) NULL)
     signal_handlers[SIGINT]=RegisterMagickSignalHandler(SIGINT);
 #endif
@@ -941,6 +973,9 @@ MagickExport void InitializeMagick(const char *path)
     Initialize magick resources.
   */
   InitializeMagickResources();
+  GetExceptionInfo(&exception);
+  (void) GetMagickInfo((char *) NULL,&exception);
+  (void) DestroyExceptionInfo(&exception);
 }
 
 /*
@@ -991,7 +1026,7 @@ static MagickBooleanType InitializeMagickList(ExceptionInfo *exception)
   if ((magick_list == (SplayTreeInfo *) NULL) &&
       (instantiate_magick == MagickFalse))
     {
-      magick_semaphore=AcquireSemaphoreInfo(magick_semaphore);
+      AcquireSemaphoreInfo(&magick_semaphore);
       if ((magick_list == (SplayTreeInfo *) NULL) &&
           (instantiate_magick == MagickFalse))
         {
@@ -1004,22 +1039,43 @@ static MagickBooleanType InitializeMagickList(ExceptionInfo *exception)
           magick_list=NewSplayTree(CompareSplayTreeString,
             RelinquishMagickMemory,DestroyMagickNode);
           if (magick_list == (SplayTreeInfo *) NULL)
-            ThrowMagickFatalException(ResourceLimitFatalError,
-              "MemoryAllocationFailed",strerror(errno));
+            {
+              char
+                *message;
+
+              message=GetExceptionMessage(errno);
+              ThrowMagickFatalException(ResourceLimitFatalError,
+                "MemoryAllocationFailed",message);
+              message=(char *) RelinquishMagickMemory(message);
+            }
           magick_info=SetMagickInfo("tmp");
           magick_info->stealth=MagickTrue;
-          status=AddValueToSplayTree(magick_list,
-            ConstantString(AcquireString(magick_info->name)),magick_info);
+          status=AddValueToSplayTree(magick_list,ConstantString(
+            magick_info->name),magick_info);
           if (status == MagickFalse)
-            ThrowMagickFatalException(ResourceLimitFatalError,
-              "UnableToAllocateMagickInfo",magick_info->name);
+            {
+              char
+                *message;
+
+              message=GetExceptionMessage(errno);
+              ThrowMagickFatalException(ResourceLimitFatalError,
+                "MemoryAllocationFailed",message);
+              message=(char *) RelinquishMagickMemory(message);
+            }
           magick_info=SetMagickInfo("clipmask");
           magick_info->stealth=MagickTrue;
-          status=AddValueToSplayTree(magick_list,
-            ConstantString(AcquireString(magick_info->name)),magick_info);
+          status=AddValueToSplayTree(magick_list,ConstantString(
+            magick_info->name),magick_info);
           if (status == MagickFalse)
-            ThrowMagickFatalException(ResourceLimitFatalError,
-              "UnableToAllocateMagickInfo",magick_info->name);
+            {
+              char
+                *message;
+
+              message=GetExceptionMessage(errno);
+              ThrowMagickFatalException(ResourceLimitFatalError,
+                "MemoryAllocationFailed",message);
+              message=(char *) RelinquishMagickMemory(message);
+            }
 #if defined(SupportMagickModules)
           (void) GetModuleInfo((char *) NULL,exception);
 #endif
@@ -1073,7 +1129,7 @@ MagickExport MagickBooleanType IsMagickConflict(const char *magick)
   delegate_info=GetDelegateInfo(magick,(char *) NULL,&exception);
   if (delegate_info == (const DelegateInfo *) NULL)
     delegate_info=GetDelegateInfo((char *) NULL,magick,&exception);
-  DestroyExceptionInfo(&exception);
+  (void) DestroyExceptionInfo(&exception);
   if ((magick_info == (const MagickInfo *) NULL) &&
       (delegate_info == (const DelegateInfo *) NULL))
     return(MagickTrue);
@@ -1155,7 +1211,7 @@ MagickExport MagickBooleanType ListMagickInfo(FILE *file,
 
       *module='\0';
       if (magick_info[i]->module != (char *) NULL)
-        (void) strcpy(module,magick_info[i]->module);
+        (void) CopyMagickString(module,magick_info[i]->module,MaxTextExtent);
       (void) ConcatenateMagickString(module,"          ",MaxTextExtent);
       module[9]='\0';
       (void) fprintf(file,"%9s ",module);
@@ -1340,8 +1396,8 @@ MagickExport MagickInfo *RegisterMagickInfo(MagickInfo *magick_info)
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",magick_info->name);
   if (magick_list == (SplayTreeInfo *) NULL)
     return((MagickInfo *) NULL);
-  status=AddValueToSplayTree(magick_list,
-    ConstantString(AcquireString(magick_info->name)),magick_info);
+  status=AddValueToSplayTree(magick_list,ConstantString(magick_info->name),
+    magick_info);
   if (status == MagickFalse)
     ThrowMagickFatalException(ResourceLimitFatalError,
       "UnableToAllocateMagickInfo",magick_info->name);
@@ -1388,7 +1444,7 @@ MagickExport MagickInfo *SetMagickInfo(const char *name)
     ThrowMagickFatalException(ResourceLimitFatalError,
       "UnableToAllocateMagickInfo",name);
   (void) ResetMagickMemory(magick_info,0,sizeof(*magick_info));
-  magick_info->name=ConstantString(AcquireString(name));
+  magick_info->name=ConstantString(name);
   magick_info->adjoin=MagickTrue;
   magick_info->blob_support=MagickTrue;
   magick_info->thread_support=MagickTrue;
@@ -1434,7 +1490,7 @@ MagickExport MagickBooleanType UnregisterMagickInfo(const char *name)
   if (GetNumberOfNodesInSplayTree(magick_list) == 0)
     return(MagickFalse);
   status=MagickFalse;
-  magick_semaphore=AcquireSemaphoreInfo(magick_semaphore);
+  AcquireSemaphoreInfo(&magick_semaphore);
   ResetSplayTreeIterator(magick_list);
   p=(const MagickInfo *) GetNextValueInSplayTree(magick_list);
   while (p != (const MagickInfo *) NULL)
