@@ -17,7 +17,7 @@
 %                                March 2000                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -59,9 +59,6 @@
 #include "magick/string_.h"
 #include "magick/token.h"
 #include "magick/utility.h"
-#if defined(__WINDOWS__)
-# include "magick/nt-feature.h"
-#endif
 #if defined(SupportMagickModules)
 #if defined(HasLTDL)
 #include "ltdl.h"
@@ -135,7 +132,7 @@ MagickExport void DestroyModuleList(void)
   /*
     Deestroy magick modules.
   */
-  module_semaphore=AcquireSemaphoreInfo(module_semaphore);
+  AcquireSemaphoreInfo(&module_semaphore);
 #if defined(SupportMagickModules)
   if (module_list != (SplayTreeInfo *) NULL)
     module_list=DestroySplayTree(module_list);
@@ -193,6 +190,9 @@ MagickExport MagickBooleanType ExecuteModuleProcess(const char *tag,
   ModuleHandle
     handle;
 
+  size_t
+    length;
+
   /*
     Find the module.
   */
@@ -207,9 +207,9 @@ MagickExport MagickBooleanType ExecuteModuleProcess(const char *tag,
 #endif
   status=MagickFalse;
   TagToFilterModuleName(tag,name);
-  status=GetMagickModulePath(name,MagickFilterModule,path,&(*image)->exception);
-  if (status == MagickFalse)
-    return(status);
+  length=GetMagickModulePath(name,MagickFilterModule,path,&(*image)->exception);
+  if (length == 0)
+    return(MagickFalse);
   /*
     Open the module.
   */
@@ -302,7 +302,7 @@ MagickExport const ModuleInfo *GetModuleInfo(const char *tag,
       ResetSplayTreeIterator(module_list);
       return((const ModuleInfo *) GetNextValueInSplayTree(module_list));
     }
-  module_semaphore=AcquireSemaphoreInfo(module_semaphore);
+  AcquireSemaphoreInfo(&module_semaphore);
   ResetSplayTreeIterator(module_list);
   p=(const ModuleInfo *) GetNextValueInSplayTree(module_list);
   while (p != (const ModuleInfo *) NULL)
@@ -311,9 +311,6 @@ MagickExport const ModuleInfo *GetModuleInfo(const char *tag,
       break;
     p=(const ModuleInfo *) GetNextValueInSplayTree(module_list);
   }
-  if (p == (ModuleInfo *) NULL)
-    (void) ThrowMagickException(exception,GetMagickModule(),ModuleError,
-      "UnrecognizedModule","`%s'",tag);
   RelinquishSemaphoreInfo(module_semaphore);
   return((const ModuleInfo *) p);
 }
@@ -396,7 +393,7 @@ MagickExport const ModuleInfo **GetModuleInfoList(const char *pattern,
   /*
     Generate module list.
   */
-  module_semaphore=AcquireSemaphoreInfo(module_semaphore);
+  AcquireSemaphoreInfo(&module_semaphore);
   ResetSplayTreeIterator(module_list);
   p=(const ModuleInfo *) GetNextValueInSplayTree(module_list);
   for (i=0; p != (const ModuleInfo *) NULL; )
@@ -449,12 +446,12 @@ extern "C" {
 
 static int ModuleCompare(const void *x,const void *y)
 {
-  register char
+  register const char
     **p,
     **q;
 
-  p=(char **) x;
-  q=(char **) y;
+   p=(const char **) x;
+  q=(const char **) y;
   return(LocaleCompare(*p,*q));
 }
 
@@ -474,12 +471,12 @@ MagickExport char **GetModuleList(const char *pattern,
   DIR
     *directory;
 
-  MagickBooleanType
-    status;
-
   register long
     i;
 
+  size_t
+    length;
+
   struct dirent
     *entry;
 
@@ -490,9 +487,9 @@ MagickExport char **GetModuleList(const char *pattern,
     Locate all modules in the coder path.
   */
   TagToCoderModuleName("magick",filename);
-  status=GetMagickModulePath(filename,MagickCoderModule,module_path,
+  length=GetMagickModulePath(filename,MagickCoderModule,module_path,
     exception);
-  if (status == MagickFalse)
+  if (length == 0)
     return((char **) NULL);
   GetPathComponent(module_path,HeadPath,path);
   max_entries=255;
@@ -578,21 +575,15 @@ MagickExport char **GetModuleList(const char *pattern,
 static MagickBooleanType GetMagickModulePath(const char *filename,
   MagickModuleType module_type,char *path,ExceptionInfo *exception)
 {
-  const char
+  char
     *module_path;
 
-  register char
-    *q;
-
-  register const char
-    *p;
-
   assert(filename != (const char *) NULL);
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",filename);
   assert(path != (char *) NULL);
   assert(exception != (ExceptionInfo *) NULL);
   (void) CopyMagickString(path,filename,MaxTextExtent);
-  module_path=(const char *) NULL;
+  module_path=(char *) NULL;
   switch (module_type)
   {
     case MagickCoderModule:
@@ -600,31 +591,38 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
     {
       (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
         "Searching for coder module file \"%s\" ...",filename);
-      module_path=getenv("MAGICK_CODER_MODULE_PATH");
+      module_path=GetEnvironmentValue("MAGICK_CODER_MODULE_PATH");
       break;
     }
     case MagickFilterModule:
     {
       (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
         "Searching for filter module file \"%s\" ...",filename);
-      module_path=getenv("MAGICK_FILTER_MODULE_PATH");
+      module_path=GetEnvironmentValue("MAGICK_CODER_FILTER_PATH");
       break;
     }
   }
-  if (module_path != (const char *) NULL)
-    for (p=module_path-1; p != (char *) NULL; )
+  if (module_path != (char *) NULL)
     {
-      (void) CopyMagickString(path,p+1,MaxTextExtent);
-      q=strchr(path,DirectoryListSeparator);
-      if (q != (char *) NULL)
-        *q='\0';
-      q=path+strlen(path)-1;
-      if ((q >= path) && (*q != *DirectorySeparator))
-        (void) ConcatenateMagickString(path,DirectorySeparator,MaxTextExtent);
-      (void) ConcatenateMagickString(path,filename,MaxTextExtent);
-      if (IsAccessible(path) != MagickFalse)
-        return(MagickTrue);
-      p=strchr(p+1,DirectoryListSeparator);
+      register char
+        *p,
+        *q;
+
+      for (p=module_path-1; p != (char *) NULL; )
+      {
+        (void) CopyMagickString(path,p+1,MaxTextExtent);
+        q=strchr(path,DirectoryListSeparator);
+        if (q != (char *) NULL)
+          *q='\0';
+        q=path+strlen(path)-1;
+        if ((q >= path) && (*q != *DirectorySeparator))
+          (void) ConcatenateMagickString(path,DirectorySeparator,MaxTextExtent);
+        (void) ConcatenateMagickString(path,filename,MaxTextExtent);
+        if (IsAccessible(path) != MagickFalse)
+          return(MagickTrue);
+        p=strchr(p+1,DirectoryListSeparator);
+      }
+      module_path=(char *) RelinquishMagickMemory(module_path);
     }
 #if defined(UseInstalledMagick)
 #if defined(MagickCoderModulesPath)
@@ -702,15 +700,15 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
 #endif
 #endif
 #if !defined(MagickCoderModulesPath) && !defined(__WINDOWS__)
-# error MagickCoderModulesPath or WIN32 must be defined when UseInstalledMagick is defined
+# error MagickCoderModulesPath or __WINDOWS__ must be defined when UseInstalledMagick is defined
 #endif
 #else
   {
-    const char
+    char
       *home;
 
-    home=getenv("MAGICK_HOME");
-    if (home != (char *) NULL)
+    home=GetEnvironmentValue("MAGICK_HOME");
+		if (home != (char *) NULL)
       {
         /*
           Search MAGICK_HOME.
@@ -739,6 +737,7 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
         (void) FormatMagickString(path,MaxTextExtent,"%s/lib/%s/%s",home,
           directory,filename);
 #endif
+        home=(char *) RelinquishMagickMemory(home);
         if (IsAccessible(path) != MagickFalse)
           return(MagickTrue);
       }
@@ -795,10 +794,12 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
   }
 #endif
   {
-    const char
+    char
       *home;
 
-    home=getenv("HOME");
+    home=GetEnvironmentValue("HOME");
+    if (home == (char *) NULL)
+      home=GetEnvironmentValue("USERPROFILE");
     if (home != (char *) NULL)
       {
         /*
@@ -808,6 +809,7 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
           *home == '/' ? "/.magick" : "",DirectorySeparator,filename);
         if (IsAccessible(path) != MagickFalse)
           return(MagickTrue);
+        home=(char *) RelinquishMagickMemory(home);
       }
   }
   /*
@@ -862,16 +864,17 @@ static void *DestroyModuleNode(void *module_info)
     p->tag=(char *) RelinquishMagickMemory(p->tag);
   if (p->path != (char *) NULL)
     p->path=(char *) RelinquishMagickMemory(p->path);
-  DestroyExceptionInfo(&exception);
+  (void) DestroyExceptionInfo(&exception);
   return(RelinquishMagickMemory(p));
 }
 
-static MagickBooleanType InitializeModuleList(ExceptionInfo *exception)
+static MagickBooleanType InitializeModuleList(
+  ExceptionInfo *magick_unused(exception))
 {
   if ((module_list == (SplayTreeInfo *) NULL) &&
       (instantiate_module == MagickFalse))
     {
-      module_semaphore=AcquireSemaphoreInfo(module_semaphore);
+      AcquireSemaphoreInfo(&module_semaphore);
       if ((module_list == (SplayTreeInfo *) NULL) &&
           (instantiate_module == MagickFalse))
         {
@@ -884,22 +887,43 @@ static MagickBooleanType InitializeModuleList(ExceptionInfo *exception)
           module_list=NewSplayTree(CompareSplayTreeString,
             RelinquishMagickMemory,DestroyModuleNode);
           if (module_list == (SplayTreeInfo *) NULL)
-            ThrowMagickFatalException(ResourceLimitFatalError,
-              "MemoryAllocationFailed",strerror(errno));
+            {
+              char
+                *message;
+
+              message=GetExceptionMessage(errno);
+              ThrowMagickFatalException(ResourceLimitFatalError,
+                "MemoryAllocationFailed",message);
+              message=(char *) RelinquishMagickMemory(message);
+            }
           module_info=(ModuleInfo *) AcquireMagickMemory(sizeof(*module_info));
           if (module_info == (ModuleInfo *) NULL)
-            ThrowMagickFatalException(ResourceLimitFatalError,
-              "MemoryAllocationFailed",strerror(errno));
+            {
+              char
+                *message;
+
+              message=GetExceptionMessage(errno);
+              ThrowMagickFatalException(ResourceLimitFatalError,
+                "MemoryAllocationFailed",message);
+              message=(char *) RelinquishMagickMemory(message);
+            }
           (void) ResetMagickMemory(module_info,0,sizeof(*module_info));
-          module_info->tag=ConstantString(AcquireString("[boot-strap]"));
+          module_info->tag=ConstantString("[boot-strap]");
           module_info->stealth=MagickTrue;
           (void) time(&module_info->load_time);
           module_info->signature=MagickSignature;
           status=AddValueToSplayTree(module_list,
-            ConstantString(AcquireString(module_info->tag)),module_info);
+            ConstantString(module_info->tag),module_info);
           if (status == MagickFalse)
-            ThrowMagickFatalException(ResourceLimitFatalError,
-              "MemoryAllocationFailed",strerror(errno));
+            {
+              char
+                *message;
+
+              message=GetExceptionMessage(errno);
+              ThrowMagickFatalException(ResourceLimitFatalError,
+                "MemoryAllocationFailed",message);
+              message=(char *) RelinquishMagickMemory(message);
+            }
           if (lt_dlinit() != 0)
             ThrowMagickFatalException(ModuleFatalError,
               "UnableToInitializeModuleLoader",lt_dlerror());
@@ -1011,9 +1035,6 @@ MagickExport MagickBooleanType OpenModule(const char *module,
     name[MaxTextExtent],
     path[MaxTextExtent];
 
-  MagickBooleanType
-    status;
-
   ModuleHandle
     handle;
 
@@ -1023,6 +1044,9 @@ MagickExport MagickBooleanType OpenModule(const char *module,
   register const CoderInfo
     *p;
 
+  size_t
+    length;
+
   /*
     Assign module name from alias.
   */
@@ -1030,8 +1054,6 @@ MagickExport MagickBooleanType OpenModule(const char *module,
   module_info=(ModuleInfo *) GetModuleInfo(module,exception);
   if (module_info != (ModuleInfo *) NULL)
     return(MagickTrue);
-  if (exception->severity == ModuleError)
-    (void) SetExceptionInfo(exception,UndefinedException);
   (void) CopyMagickString(module_name,module,MaxTextExtent);
   p=GetCoderInfo(module,exception);
   if (p != (CoderInfo *) NULL)
@@ -1044,9 +1066,9 @@ MagickExport MagickBooleanType OpenModule(const char *module,
   (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
     "Searching for module \"%s\" using filename \"%s\"",module_name,filename);
   *path='\0';
-  status=GetMagickModulePath(filename,MagickCoderModule,path,exception);
-  if (status == MagickFalse)
-    return(status);
+  length=GetMagickModulePath(filename,MagickCoderModule,path,exception);
+  if (length == 0)
+    return(MagickFalse);
   /*
     Load module
   */
@@ -1067,8 +1089,8 @@ MagickExport MagickBooleanType OpenModule(const char *module,
     ThrowMagickFatalException(ResourceLimitFatalError,
       "UnableToAllocateModuleInfo",module_name);
   (void) ResetMagickMemory(module_info,0,sizeof(*module_info));
-  module_info->path=ConstantString(AcquireString(path));
-  module_info->tag=ConstantString(AcquireString(module_name));
+  module_info->path=ConstantString(path);
+  module_info->tag=ConstantString(module_name);
   module_info->signature=MagickSignature;
   module_info->handle=handle;
   (void) time(&module_info->load_time);
@@ -1087,7 +1109,7 @@ MagickExport MagickBooleanType OpenModule(const char *module,
     }
   (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
     "Method \"%s\" in module \"%s\" at address %p",name,module_name,
-    module_info->register_module);
+    (void *) module_info->register_module);
   /*
     Locate and record UnregisterFORMATImage function
   */
@@ -1101,7 +1123,7 @@ MagickExport MagickBooleanType OpenModule(const char *module,
     }
   (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
     "Method \"%s\" in module \"%s\" at address %p",name,module_name,
-    module_info->unregister_module);
+    (void *) module_info->unregister_module);
   /*
     Execute RegisterFORMATImage module.
   */
@@ -1201,8 +1223,8 @@ static const ModuleInfo *RegisterModule(const ModuleInfo *module_info,
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",module_info->tag);
   if (module_list == (SplayTreeInfo *) NULL)
     return((const ModuleInfo *) NULL);
-  status=AddValueToSplayTree(module_list,
-    ConstantString(AcquireString(module_info->tag)),module_info);
+  status=AddValueToSplayTree(module_list,ConstantString(module_info->tag),
+    module_info);
   if (status == MagickFalse)
     (void) ThrowMagickException(exception,GetMagickModule(),ResourceLimitError,
       "MemoryAllocationFailed","`%s'",module_info->tag);
