@@ -17,7 +17,7 @@
 %                                December 1996                                %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -44,6 +44,7 @@
 #include "magick/log.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
+#include "magick/resource_.h"
 #include "magick/timer.h"
 #include "magick/string_.h"
 #include "magick/utility.h"
@@ -67,11 +68,12 @@
 static char
   *lt_slsearchpath = (char *) NULL;
 #endif
-static void
-  *gs_dll_handle = (void *) NULL;
 
 static GhostscriptVectors
   gs_vectors;
+
+static void
+  *gs_dll_handle = (void *) NULL;
 
 /*
   External declarations.
@@ -132,36 +134,45 @@ BOOL WINAPI DllMain(HINSTANCE handle,DWORD reason,LPVOID lpvReserved)
     case DLL_PROCESS_ATTACH:
     {
       char
-        module_path[MaxTextExtent],
-        new_path[16*MaxTextExtent],
-        path[16*MaxTextExtent];
+        *module_path,
+        *new_path,
+        *path;
 
-      long
+      ssize_t
         count;
 
-      count=GetModuleFileName(handle,module_path,MaxTextExtent);
-      if (count == 0)
+      module_path=(char *) AcquireMagickMemory(MaxTextExtent*
+        sizeof(*module_path));
+      new_path=(char *) AcquireMagickMemory(16*MaxTextExtent*sizeof(*new_path));
+      path=(char *) AcquireMagickMemory(16*MaxTextExtent*sizeof(*path));
+      if ((module_path == (char *) NULL) || (new_path == (char *) NULL) ||
+         (path == (char *) NULL))
+        return(FALSE);
+      count=(ssize_t) GetModuleFileName(handle,module_path,MaxTextExtent);
+      if (count != 0)
         {
-          InitializeMagick((char *) NULL);
-          break;
+          for ( ; count > 0; count--)
+            if (module_path[count] == '\\')
+              {
+                module_path[count+1]='\0';
+                break;
+              }
+          InitializeMagick(module_path);
+          count=GetEnvironmentVariable("PATH",path,16*MaxTextExtent);
+          if ((count != 0) && (strstr(path,module_path) == (char *) NULL))
+            {
+              if ((strlen(module_path)+count+1) < (16*MaxTextExtent-1))
+                {
+                  (void) FormatMagickString(new_path,16*MaxTextExtent,
+                    "%s;%s",module_path,path);
+                  SetEnvironmentVariable("PATH",new_path);
+                }
+              path=(char *) RelinquishMagickMemory(path);
+            }
         }
-      for ( ; count > 0; count--)
-        if (module_path[count] == '\\')
-          {
-            module_path[count+1]='\0';
-            break;
-          }
-      InitializeMagick(module_path);
-      count=GetEnvironmentVariable("PATH",path,16*MaxTextExtent);
-      if (count == 0)
-        break;
-      if (strstr(path,module_path) != (char *) NULL)
-        break;
-      if ((strlen(module_path)+count+1) >= (16*MaxTextExtent-1))
-        break;
-      (void) FormatMagickString(new_path,16*MaxTextExtent,"%.*s;%.*s",
-        MaxTextExtent,module_path,15*MaxTextExtent-1,path);
-      SetEnvironmentVariable("PATH",new_path);
+      path=(char *) RelinquishMagickMemory(path);
+      new_path=(char *) RelinquishMagickMemory(new_path);
+      module_path=(char *) RelinquishMagickMemory(module_path);
       break;
     }
     case DLL_PROCESS_DETACH:
@@ -189,7 +200,7 @@ BOOL WINAPI DllMain(HINSTANCE handle,DWORD reason,LPVOID lpvReserved)
 %
 %  Exit() calls TerminateProcess for Win95.
 %
-%  The format of the Exit method is:
+%  The format of the exit method is:
 %
 %      int Exit(int status)
 %
@@ -305,22 +316,30 @@ MagickExport int NTCloseLibrary(void *handle)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   N T E x i t L i b r a r y                                                 %
+%   N T C o n t r o l H a n d l e r                                           %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  NTExitLibrary() exits the dynamic module loading subsystem.
+%  NTControlHandler() registers a control handler that is activated when, for
+%  example, a ctrl-c is received.
 %
-%  The format of the NTExitLibrary method is:
+%  The format of the NTControlHandler method is:
 %
-%      int NTExitLibrary(void)
+%      int NTControlHandler(void)
 %
 */
-MagickExport int NTExitLibrary(void)
+
+static BOOL ControlHandler(DWORD type)
 {
-  return(0);
+  AsynchronousDestroyMagickResources();
+  return(FALSE);
+}
+
+MagickExport int NTControlHandler(void)
+{
+  return(SetConsoleCtrlHandler((PHANDLER_ROUTINE) ControlHandler,TRUE));
 }
 
 /*
@@ -394,31 +413,57 @@ MagickExport void NTErrorHandler(const ExceptionType error,const char *reason,
   const char *description)
 {
   char
-    buffer[3*MaxTextExtent];
+    buffer[3*MaxTextExtent],
+    *message;
 
   if (reason == (char *) NULL)
     {
       DestroyMagick();
-      Exit(0);
+      exit(0);
     }
+  message=GetExceptionMessage(errno);
   if ((description != (char *) NULL) && errno)
     (void) FormatMagickString(buffer,MaxTextExtent,"%s: %s (%s) [%s].\n",
-      GetClientName(),reason,description,strerror(errno));
+      GetClientName(),reason,description,message);
   else
     if (description != (char *) NULL)
       (void) FormatMagickString(buffer,MaxTextExtent,"%s: %s (%s).\n",
         GetClientName(),reason,description);
     else
-      if (errno)
+      if (errno != 0)
         (void) FormatMagickString(buffer,MaxTextExtent,"%s: %s [%s].\n",
-          GetClientName(),reason,strerror(errno));
+          GetClientName(),reason,message);
       else
         (void) FormatMagickString(buffer,MaxTextExtent,"%s: %s.\n",
           GetClientName(),reason);
+  message=(char *) RelinquishMagickMemory(message);
   (void) MessageBox(NULL,buffer,"ImageMagick Exception",MB_OK | MB_TASKMODAL |
     MB_SETFOREGROUND | MB_ICONEXCLAMATION);
   DestroyMagick();
-  Exit(0);
+  exit(0);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   N T E x i t L i b r a r y                                                 %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  NTExitLibrary() exits the dynamic module loading subsystem.
+%
+%  The format of the NTExitLibrary method is:
+%
+%      int NTExitLibrary(void)
+%
+*/
+MagickExport int NTExitLibrary(void)
+{
+  return(0);
 }
 
 /*
@@ -712,14 +757,14 @@ static int NTGhostscriptProductVersions(int *pver,int offset,
     cbData;
 
   char
-    key[256],
+    key[MaxTextExtent],
     *p;
 
   int
     n = 0,
     ver;
 
-  (void) sprintf(key, "Software\\%s",gs_productfamily);
+  (void) FormatMagickString(key,MaxTextExtent,"Software\\%s",gs_productfamily);
   hkeyroot = HKEY_LOCAL_MACHINE;
   if (RegOpenKeyExA(hkeyroot,key,0,KEY_READ,&hkey) == ERROR_SUCCESS)
     {
@@ -850,14 +895,14 @@ static int NTGhostscriptGetProductString(int gs_revision,const char *name,
    * and N.NN is obtained from gs_revision.
    */
 
+  char
+    dotversion[MaxTextExtent],
+    key[MaxTextExtent];
+
   int
     code,
     length;
 
-  char
-    dotversion[16],
-    key[256];
-
   DWORD version = GetVersion();
 
   if (((HIWORD(version) & 0x8000) != 0) && ((HIWORD(version) & 0x4000) == 0))
@@ -865,9 +910,10 @@ static int NTGhostscriptGetProductString(int gs_revision,const char *name,
       /* Win32s */
       return FALSE;
     }
-  (void) sprintf(dotversion,"%d.%02d",(int)(gs_revision/100),(int)
-    (gs_revision % 100));
-  (void) sprintf(key, "Software\\%s\\%s",gs_productfamily, dotversion);
+  (void) FormatMagickString(dotversion,MaxTextExtent,"%d.%02d",(int)
+    (gs_revision/100),(int) (gs_revision % 100));
+  (void) FormatMagickString(key,MaxTextExtent,"Software\\%s\\%s",
+    gs_productfamily,dotversion);
   length = len;
   code = NTGetRegistryValue(HKEY_CURRENT_USER, key, name, ptr, &length);
   if ( code == 0 )
@@ -1035,7 +1081,7 @@ MagickExport int NTGhostscriptEXE(char *path, int path_length)
     buf[256],
     *p;
 
-  *path='\0';
+  (void) CopyMagickString(path,"gswin32c.exe",path_length);
   gsver = NTGetLatestGhostscript();
   if ((gsver == FALSE) || (gsver < GS_MINIMUM_VERSION))
     return FALSE;
@@ -1380,6 +1426,16 @@ MagickExport DIR *NTOpenDirectory(const char *path)
 %      is to be loaded.
 %
 */
+
+static const char *GetSearchPath( void )
+{
+#ifdef HasLTDL
+  return(lt_dlGetSearchPath());
+#else
+  return(lt_slsearchpath);
+#endif
+}
+
 MagickExport void *NTOpenLibrary(const char *filename)
 {
 #define MaxPathElements  31
@@ -1390,7 +1446,7 @@ MagickExport void *NTOpenLibrary(const char *filename)
   int
     index;
 
-  register char
+  register const char
     *p,
     *q;
 
@@ -1405,12 +1461,12 @@ MagickExport void *NTOpenLibrary(const char *filename)
 
   mode=SetErrorMode(SEM_FAILCRITICALERRORS | SEM_NOOPENFILEERRORBOX);
   handle=(void *) LoadLibraryEx(filename,NULL,LOAD_WITH_ALTERED_SEARCH_PATH);
-  if ((handle != (void *) NULL) || (lt_slsearchpath == (char *) NULL))
+  if ((handle != (void *) NULL) || (GetSearchPath() == (char *) NULL))
     {
       SetErrorMode(mode);
       return(handle);
     }
-  p=lt_slsearchpath;
+  p=(char *) GetSearchPath();
   index=0;
   while (index < MaxPathElements)
   {
@@ -1482,7 +1538,7 @@ MagickExport struct dirent *NTReadDirectory(DIR *entry)
   if (length >= sizeof(entry->file_info.d_name))
     return((struct dirent *) NULL);
   entry->firsttime=FALSE;
-  entry->file_info.d_namlen=strlen(entry->file_info.d_name);
+  entry->file_info.d_namlen=(int) strlen(entry->file_info.d_name);
   return(&entry->file_info);
 }
 
@@ -1608,12 +1664,15 @@ MagickExport MagickBooleanType NTReportEvent(const char *event,
   HANDLE
     handle;
 
+  WORD
+    type;
+
   handle=RegisterEventSource(NULL,"ImageMagick");
   if (handle == NULL)
     return(MagickFalse);
   events[0]=event;
-  ReportEvent(handle,error ? EVENTLOG_ERROR_TYPE : EVENTLOG_WARNING_TYPE,0,0,
-    NULL,1,0,events,NULL);
+  type=error ? EVENTLOG_ERROR_TYPE : EVENTLOG_WARNING_TYPE;
+  ReportEvent(handle,type,0,0,NULL,1,0,events,NULL);
   DeregisterEventSource(handle);
   return(MagickTrue);
 }
@@ -1756,10 +1815,14 @@ MagickExport void NTSeekDirectory(DIR *entry,long position)
 */
 MagickExport int NTSetSearchPath(const char *path)
 {
-  if (lt_slsearchpath)
+#ifdef HasLTDL
+  lt_dlsetsearchpath(path);
+#else /*!HasLTDL*/
+  if (lt_slsearchpath != (char *) NULL)
     lt_slsearchpath=(char *) RelinquishMagickMemory(lt_slsearchpath);
   if (path != (char *) NULL)
     lt_slsearchpath=AcquireString(path);
+#endif /*HasLTDL*/
   return(0);
 }
 
@@ -1885,7 +1948,7 @@ MagickExport int NTSystemCommand(const char *command)
 %  Sysconf() provides  a  way for the application to determine values for
 %  system limits or options at runtime.
 %
-%  The format of the Exit method is:
+%  The format of the exit method is:
 %
 %      long NTSystemConfiguration(int name)
 %
