@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -13,10 +13,10 @@
   See the License for the specific language governing permissions and
   limitations under the License.
 
-  ImageMagick image constitute methods.
+  MagickCore image constitute methods.
 */
-#ifndef _MAGICK_PIXEL_H
-#define _MAGICK_PIXEL_H
+#ifndef _MAGICKCORE_PIXEL_H
+#define _MAGICKCORE_PIXEL_H
 
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
@@ -34,21 +34,19 @@ typedef enum
   CMYKQuantum,
   CyanQuantum,
   GrayAlphaQuantum,
-  GrayAlphaQuark,
   GrayQuantum,
-  GrayQuark,
   GreenQuantum,
   IndexAlphaQuantum,
-  IndexAlphaQuark,
   IndexQuantum,
-  IndexQuark,
   MagentaQuantum,
   OpacityQuantum,
   RedQuantum,
   RGBAQuantum,
   RGBOQuantum,
   RGBQuantum,
-  YellowQuantum
+  YellowQuantum,
+  GrayPadQuantum,
+  RGBPadQuantum
 } QuantumType;
 
 typedef struct _LongPixelPacket
