@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -47,6 +47,7 @@
 #include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/profile.h"
+#include "magick/quantum.h"
 #include "magick/splay-tree.h"
 #include "magick/string_.h"
 #include "magick/utility.h"
@@ -138,7 +139,7 @@ MagickExport MagickBooleanType CloneImageProfiles(Image *image,
 MagickExport void DestroyImageProfiles(Image *image)
 {
   if (image->profiles != (SplayTreeInfo *) NULL)
-    image->profiles=DestroySplayTree(image->profiles);
+    image->profiles=DestroySplayTree((SplayTreeInfo *) image->profiles);
 }
 
 /*
@@ -156,7 +157,7 @@ MagickExport void DestroyImageProfiles(Image *image)
 %
 %  The format of the GetImageProfile method is:
 %
-%      StringInfo *GetImageProfile(const Image *image,const char *name)
+%      const StringInfo *GetImageProfile(const Image *image,const char *name)
 %
 %  A description of each parameter follows:
 %
@@ -165,11 +166,15 @@ MagickExport void DestroyImageProfiles(Image *image)
 %    o name: The profile name.
 %
 */
-MagickExport StringInfo *GetImageProfile(const Image *image,const char *name)
+MagickExport const StringInfo *GetImageProfile(const Image *image,
+  const char *name)
 {
   char
     key[MaxTextExtent];
 
+  const StringInfo
+    *profile;
+
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
@@ -178,7 +183,9 @@ MagickExport StringInfo *GetImageProfile(const Image *image,const char *name)
     return((StringInfo *) NULL);
   (void) CopyMagickString(key,name,MaxTextExtent);
   LocaleLower(key);
-  return((StringInfo *) GetValueFromSplayTree(image->profiles,key));
+  profile=(const StringInfo *) GetValueFromSplayTree((SplayTreeInfo *)
+    image->profiles,key);
+  return(profile);
 }
 
 /*
@@ -211,7 +218,7 @@ MagickExport char *GetNextImageProfile(const Image *image)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (image->profiles == (SplayTreeInfo *) NULL)
     return((char *) NULL);
-  return((char *) GetNextKeyInSplayTree(image->profiles));
+  return((char *) GetNextKeyInSplayTree((SplayTreeInfo *) image->profiles));
 }
 
 /*
@@ -296,14 +303,14 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
     Add a ICC, IPTC, or generic profile to the image.
   */
   profile=AcquireStringInfo((size_t) length);
-  SetStringInfoDatum(profile,datum);
+  SetStringInfoDatum(profile,(unsigned char *) datum);
   if ((LocaleCompare("icc",name) == 0) || (LocaleCompare("icm",name) == 0))
     {
-      StringInfo
+      const StringInfo
         *icc_profile;
 
       icc_profile=GetImageProfile(image,"icc");
-      if ((icc_profile != (StringInfo *) NULL) &&
+      if ((icc_profile != (const StringInfo *) NULL) &&
           (CompareStringInfo(icc_profile,profile) == 0))
         {
           profile=DestroyStringInfo(profile);
@@ -521,7 +528,8 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
               ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
                 image->filename);
             }
-          image->storage_class=DirectClass;
+          if (SetImageStorageClass(image,DirectClass) == MagickFalse)
+            return(MagickFalse);
           if (target_colorspace == CMYKColorspace)
             image->colorspace=target_colorspace;
           for (y=0; y < (long) image->rows; y++)
@@ -631,7 +639,7 @@ MagickExport MagickBooleanType RemoveImageProfile(Image *image,
       image->iptc_profile.length=0;
       image->iptc_profile.info=(unsigned char *) NULL;
     }
-  return(RemoveNodeFromSplayTree(image->profiles,name));
+  return(RemoveNodeFromSplayTree((SplayTreeInfo *) image->profiles,name));
 }
 
 /*
@@ -666,7 +674,7 @@ MagickExport void ResetImageProfileIterator(const Image *image)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (image->profiles == (SplayTreeInfo *) NULL)
     return;
-  ResetSplayTreeIterator(image->profiles);
+  ResetSplayTreeIterator((SplayTreeInfo *) image->profiles);
 }
 
 /*
@@ -701,8 +709,7 @@ MagickExport void ResetImageProfileIterator(const Image *image)
 
 static void *DestroyProfile(void *profile)
 {
-  profile=(void *) DestroyStringInfo((StringInfo *) profile);
-  return(profile);
+  return((void *) DestroyStringInfo((StringInfo *) profile));
 }
 
 MagickExport MagickBooleanType SetImageProfile(Image *image,const char *name,
@@ -723,8 +730,8 @@ MagickExport MagickBooleanType SetImageProfile(Image *image,const char *name,
       DestroyProfile);
   (void) CopyMagickString(key,name,MaxTextExtent);
   LocaleLower(key);
-  status=AddValueToSplayTree(image->profiles,ConstantString(AcquireString(key)),
-    CloneStringInfo(profile));
+  status=AddValueToSplayTree((SplayTreeInfo *) image->profiles,
+    ConstantString(key),CloneStringInfo(profile));
   if ((status != MagickFalse) &&
       ((LocaleCompare(name,"icc") == 0) || (LocaleCompare(name,"icm") == 0)))
     {
