@@ -17,7 +17,7 @@
 %                              July 1992                                      %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -177,14 +177,17 @@
 */
 #include "magick/studio.h"
 #include "magick/color.h"
+#include "magick/color-private.h"
 #include "magick/colorspace.h"
 #include "magick/enhance.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
 #include "magick/image.h"
+#include "magick/image-private.h"
 #include "magick/list.h"
 #include "magick/memory_.h"
 #include "magick/quantize.h"
+#include "magick/quantum.h"
 #include "magick/monitor.h"
 #include "magick/string_.h"
 
@@ -275,6 +278,12 @@ typedef struct _CubeInfo
 
   unsigned long
     depth;
+
+  MagickOffsetType
+    offset;
+
+  MagickSizeType
+    span;
 } CubeInfo;
 
 /*
@@ -356,7 +365,6 @@ static MagickBooleanType AssignImageColors(CubeInfo *cube_info,Image *image)
 #define AssignImageTag  "Assign/Image"
 
   long
-    count,
     y;
 
   MagickBooleanType
@@ -376,6 +384,9 @@ static MagickBooleanType AssignImageColors(CubeInfo *cube_info,Image *image)
   register PixelPacket
     *q;
 
+  ssize_t
+    count;
+
   unsigned long
     id,
     index;
@@ -428,7 +439,7 @@ static MagickBooleanType AssignImageColors(CubeInfo *cube_info,Image *image)
           ((MagickRealType) QuantumRange+1.0);
         ClosestColor(image,cube_info,node_info->parent);
         index=(IndexPacket) cube_info->color_number;
-        for (i=0; i < count; i++)
+        for (i=0; i < (long) count; i++)
         {
           if (image->storage_class == PseudoClass)
             indexes[x+i]=(IndexPacket) index;
@@ -464,8 +475,8 @@ static MagickBooleanType AssignImageColors(CubeInfo *cube_info,Image *image)
       q=image->colormap;
       for (i=0; i < (long) image->colors; i++)
       {
-        intensity=(Quantum) ((unsigned long) PixelIntensityToQuantum(q) <
-          (QuantumRange/2) ? 0 : QuantumRange);
+        intensity=(Quantum)
+          (PixelIntensityToQuantum(q) < (QuantumRange/2) ? 0 : QuantumRange);
         q->red=intensity;
         q->green=intensity;
         q->blue=intensity;
@@ -547,7 +558,6 @@ static MagickBooleanType ClassifyImageColors(CubeInfo *cube_info,
 #define ClassifyImageTag  "Classify/Image"
 
   long
-    count,
     y;
 
   MagickBooleanType
@@ -569,6 +579,9 @@ static MagickBooleanType ClassifyImageColors(CubeInfo *cube_info,
   register const PixelPacket
     *p;
 
+  ssize_t
+    count;
+
   unsigned long
     index,
     id,
@@ -773,8 +786,15 @@ MagickExport QuantizeInfo *CloneQuantizeInfo(const QuantizeInfo *quantize_info)
 
   clone_info=(QuantizeInfo *) AcquireMagickMemory(sizeof(*clone_info));
   if (clone_info == (QuantizeInfo *) NULL)
-    ThrowMagickFatalException(ResourceLimitFatalError,
-      "UnableToAllocateQuantizeInfo",strerror(errno));
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowMagickFatalException(ResourceLimitFatalError,
+        "MemoryAllocationFailed",message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
   GetQuantizeInfo(clone_info);
   if (quantize_info == (QuantizeInfo *) NULL)
     return(clone_info);
@@ -1075,6 +1095,8 @@ MagickExport QuantizeInfo *DestroyQuantizeInfo(QuantizeInfo *quantize_info)
 static MagickBooleanType Dither(CubeInfo *cube_info,Image *image,
   const unsigned int direction)
 {
+#define DitherImageTag  "Dither/Image"
+
   MagickPixelPacket
     error;
 
@@ -1173,6 +1195,18 @@ static MagickBooleanType Dither(CubeInfo *cube_info,Image *image,
       p->error[i].red=pixel.red-(double) image->colormap[index].red;
       p->error[i].green=pixel.green-(double) image->colormap[index].green;
       p->error[i].blue=pixel.blue-(double) image->colormap[index].blue;
+      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+          (QuantumTick(p->offset,p->span) != MagickFalse))
+        {
+          MagickBooleanType
+            status;
+
+          status=image->progress_monitor(DitherImageTag,p->offset,p->span,
+            image->client_data);
+          if (status == MagickFalse)
+            return(MagickFalse);
+        }
+      p->offset++;
     }
   switch (direction)
   {
@@ -1241,6 +1275,8 @@ static MagickBooleanType DitherImage(CubeInfo *cube_info,Image *image)
   i=image->columns > image->rows ? image->columns : image->rows;
   for (depth=1; i != 0; depth++)
     i>>=1;
+  cube_info->offset=0;
+  cube_info->span=(MagickSizeType) image->columns*image->rows;
   HilbertCurve(cube_info,image,depth-1,NorthGravity);
   (void) Dither(cube_info,image,ForgetGravity);
   return(MagickTrue);
@@ -1743,7 +1779,8 @@ MagickExport MagickBooleanType MapImage(Image *image,const Image *map_image,
   if (cube_info == (CubeInfo *) NULL)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
-  if (image->colorspace == CMYKColorspace)
+  if ((image->colorspace != GRAYColorspace) &&
+      (image->colorspace != RGBColorspace))
     (void) SetImageColorspace(image,RGBColorspace);
   status=ClassifyImageColors(cube_info,map_image,&image->exception);
   if (status != MagickFalse)
@@ -1754,6 +1791,8 @@ MagickExport MagickBooleanType MapImage(Image *image,const Image *map_image,
       quantize_info.number_colors=cube_info->colors;
       status=AssignImageColors(cube_info,image);
     }
+  if (image->colorspace != RGBColorspace)
+    (void) SetImageColorspace(image,RGBColorspace);
   DestroyCubeInfo(cube_info);
   return(status);
 }
@@ -1781,13 +1820,12 @@ MagickExport MagickBooleanType MapImage(Image *image,const Image *map_image,
 %
 %    o image: Specifies a pointer to a set of Image structures.
 %
-%    o map_image: The image.  Reduce
-%      image to a set of colors represented by this image.
+%    o map_image: The image.  Reduce image to a set of colors represented by
+%      this image.
 %
 %    o dither: Set this integer value to something other than zero to
 %      dither the quantized image.
 %
-%
 */
 MagickExport MagickBooleanType MapImages(Image *images,const Image *map_image,
   const MagickBooleanType dither)
@@ -1829,7 +1867,8 @@ MagickExport MagickBooleanType MapImages(Image *images,const Image *map_image,
   if (cube_info == (CubeInfo *) NULL)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
-  if (image->colorspace == CMYKColorspace)
+  if ((image->colorspace != GRAYColorspace) &&
+      (image->colorspace != RGBColorspace))
     (void) SetImageColorspace(image,RGBColorspace);
   status=ClassifyImageColors(cube_info,map_image,&image->exception);
   if (status != MagickFalse)
@@ -1846,6 +1885,8 @@ MagickExport MagickBooleanType MapImages(Image *images,const Image *map_image,
         status=AssignImageColors(cube_info,image);
         if (status == MagickFalse)
           break;
+        if (image->colorspace != RGBColorspace)
+          (void) SetImageColorspace(image,RGBColorspace);
       }
     }
   DestroyCubeInfo(cube_info);
@@ -1879,8 +1920,6 @@ MagickExport MagickBooleanType MapImages(Image *images,const Image *map_image,
 */
 MagickExport MagickBooleanType OrderedDitherImage(Image *image)
 {
-#define DitherImageTag  "Dither/Image"
-
   static Quantum
     DitherMatrix[8][8] =
     {
@@ -1930,8 +1969,8 @@ MagickExport MagickBooleanType OrderedDitherImage(Image *image)
     indexes=GetIndexes(image);
     for (x=0; x < (long) image->columns; x++)
     {
-      index=(Quantum) (PixelIntensityToQuantum(q) >
-        ScaleCharToQuantum(DitherMatrix[y & 0x07][x & 0x07]) ? 1 : 0);
+      index=(Quantum) (PixelIntensityToQuantum(q) > ScaleCharToQuantum(
+        (unsigned char) (DitherMatrix[y & 0x07][x & 0x07] != 0 ? 1 : 0)));
       indexes[x]=index;
       q->red=image->colormap[index].red;
       q->green=image->colormap[index].green;
@@ -2242,10 +2281,6 @@ MagickExport MagickBooleanType QuantizeImage(const QuantizeInfo *quantize_info,
     number_colors=MaxColormapSize;
   if (number_colors > MaxColormapSize)
     number_colors=MaxColormapSize;
-  if (quantize_info->colorspace != RGBColorspace)
-    (void) SetImageColorspace(image,quantize_info->colorspace);
-  if ((image->storage_class == PseudoClass) && (image->colors <= number_colors))
-    return(MagickTrue);
   depth=quantize_info->tree_depth;
   if (depth == 0)
     {
@@ -2270,7 +2305,8 @@ MagickExport MagickBooleanType QuantizeImage(const QuantizeInfo *quantize_info,
   if (cube_info == (CubeInfo *) NULL)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
-  if (image->colorspace == CMYKColorspace)
+  if ((image->colorspace != GRAYColorspace) &&
+      (image->colorspace != RGBColorspace))
     (void) SetImageColorspace(image,RGBColorspace);
   status=ClassifyImageColors(cube_info,image,&image->exception);
   if (status != MagickFalse)
@@ -2280,10 +2316,9 @@ MagickExport MagickBooleanType QuantizeImage(const QuantizeInfo *quantize_info,
       */
       ReduceImageColors(cube_info,image,number_colors,&image->exception);
       status=AssignImageColors(cube_info,image);
-      if ((quantize_info->colorspace != GRAYColorspace) &&
-          (quantize_info->colorspace != RGBColorspace))
-        (void) SetImageColorspace(image,quantize_info->colorspace);
     }
+  if (quantize_info->colorspace != RGBColorspace)
+    (void) SetImageColorspace(image,quantize_info->colorspace);
   DestroyCubeInfo(cube_info);
   return(status);
 }
@@ -2393,20 +2428,14 @@ MagickExport MagickBooleanType QuantizeImages(const QuantizeInfo *quantize_info,
   if (cube_info == (CubeInfo *) NULL)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
-  image=images;
-  for (i=0; image != (Image *) NULL; i++)
-  {
-    if (quantize_info->colorspace != RGBColorspace)
-      (void) SetImageColorspace(image,quantize_info->colorspace);
-    image=GetNextImageInList(image);
-  }
-  number_images=(unsigned long) i;
+  number_images=GetImageListLength(images);
   image=images;
   for (i=0; image != (Image *) NULL; i++)
   {
     progress_monitor=SetImageProgressMonitor(image,(MagickProgressMonitor) NULL,
       image->client_data);
-    if (image->colorspace == CMYKColorspace)
+    if ((image->colorspace != GRAYColorspace) &&
+        (image->colorspace != RGBColorspace))
       (void) SetImageColorspace(image,RGBColorspace);
     status=ClassifyImageColors(cube_info,image,&image->exception);
     if (status == MagickFalse)
@@ -2436,8 +2465,7 @@ MagickExport MagickBooleanType QuantizeImages(const QuantizeInfo *quantize_info,
         status=AssignImageColors(cube_info,image);
         if (status == MagickFalse)
           break;
-        if ((quantize_info->colorspace != GRAYColorspace) &&
-            (quantize_info->colorspace != RGBColorspace))
+        if (quantize_info->colorspace != RGBColorspace)
           (void) SetImageColorspace(image,quantize_info->colorspace);
         (void) SetImageProgressMonitor(image,progress_monitor,
           image->client_data);
@@ -2572,11 +2600,14 @@ static void Reduce(CubeInfo *cube_info,const NodeInfo *node_info)
 static void ReduceImageColors(CubeInfo *cube_info,const Image *image,
   const unsigned long number_colors,ExceptionInfo *exception)
 {
-#define ReduceImageTag  " Reduce/Image"
+#define ReduceImageTag  "Reduce/Image"
 
   MagickBooleanType
     status;
 
+  MagickOffsetType
+    offset;
+
   unsigned long
     span;
 
@@ -2588,11 +2619,12 @@ static void ReduceImageColors(CubeInfo *cube_info,const Image *image,
     cube_info->next_threshold=cube_info->root->quantize_error-1;
     cube_info->colors=0;
     Reduce(cube_info,cube_info->root);
+    offset=(MagickOffsetType) span-cube_info->colors;
     if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(span-cube_info->colors,span-number_colors+1) != MagickFalse))
+        (QuantumTick(offset,span-number_colors+1) != MagickFalse))
       {
-        status=image->progress_monitor(ReduceImageTag,(MagickOffsetType) span-
-          cube_info->colors,span-number_colors+1,image->client_data);
+        status=image->progress_monitor(ReduceImageTag,offset,span-
+          number_colors+1,image->client_data);
         if (status == MagickFalse)
           break;
       }
