@@ -16,13 +16,13 @@
 %                              December 2001                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -95,7 +95,7 @@ static void
 */
 MagickExport void DestroyRandomReservoir(void)
 {
-  random_semaphore=AcquireSemaphoreInfo(random_semaphore);
+  AcquireSemaphoreInfo(&random_semaphore);
   if (reservoir != (SignatureInfo *) NULL)
     {
       (void) ResetMagickMemory(reservoir,0,sizeof(*reservoir));
@@ -152,7 +152,7 @@ MagickExport void DistillRandomEvent(const unsigned char *event,
   if ((reservoir == (SignatureInfo *) NULL) ||
       (roulette == (unsigned long *) NULL))
     {
-      random_semaphore=AcquireSemaphoreInfo(random_semaphore);
+      AcquireSemaphoreInfo(&random_semaphore);
       if (reservoir == (SignatureInfo *) NULL)
         reservoir=(SignatureInfo *) AcquireMagickMemory(sizeof(*reservoir));
       if (roulette == (unsigned long *) NULL)
@@ -160,12 +160,19 @@ MagickExport void DistillRandomEvent(const unsigned char *event,
       RelinquishSemaphoreInfo(random_semaphore);
       if ((reservoir == (SignatureInfo *) NULL) ||
           (roulette == (unsigned long *) NULL))
-        ThrowMagickFatalException(ResourceLimitFatalError,
-          "UnableToDistillRandomEvent",strerror(errno));
+        {
+          char
+            *message;
+
+          message=GetExceptionMessage(errno);
+          ThrowMagickFatalException(ResourceLimitFatalError,
+            "MemoryAllocationFailed",message);
+          message=(char *) RelinquishMagickMemory(message);
+        }
       (void) ResetMagickMemory(reservoir,0,sizeof(*reservoir));
       (void) ResetMagickMemory(roulette,0,sizeof(*roulette));
     }
-  random_semaphore=AcquireSemaphoreInfo(random_semaphore);
+  AcquireSemaphoreInfo(&random_semaphore);
   GetSignatureInfo(&digest_info);
   UpdateSignature(&digest_info,(const unsigned char *) reservoir->digest,
     sizeof(reservoir->digest));
@@ -213,7 +220,7 @@ MagickExport void GetRandomKey(unsigned char *key,const size_t length)
   if ((roulette == (unsigned long *) NULL) ||
       (reservoir == (SignatureInfo *) NULL))
     InitializeRandomReservoir();
-  random_semaphore=AcquireSemaphoreInfo(random_semaphore);
+  AcquireSemaphoreInfo(&random_semaphore);
   for (n=(long) length; n > 0; n-=sizeof(reservoir->digest))
   {
     GetSignatureInfo(&digest_info);
