@@ -17,7 +17,7 @@
 %                                October 2001                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -101,8 +101,8 @@ static void
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  DeleteMagickRegistry() deletes an entry in the registry as defined by the id.
-%  It returns MagickTrue if the entry is deleted otherwise MagickFalse if no entry is found
-%  in the registry that matches the id.
+%  It returns MagickTrue if the entry is deleted otherwise MagickFalse if no
+%  entry is found in the registry that matches the id.
 %
 %  The format of the DeleteMagickRegistry method is:
 %
@@ -123,7 +123,7 @@ MagickExport MagickBooleanType DeleteMagickRegistry(const long id)
     return(MagickFalse);
   if (GetNumberOfNodesInSplayTree(registry_list) == 0)
     return(MagickFalse);
-  registry_semaphore=AcquireSemaphoreInfo(registry_semaphore);
+  AcquireSemaphoreInfo(&registry_semaphore);
   ResetSplayTreeIterator(registry_list);
   p=(const RegistryInfo *) GetNextValueInSplayTree(registry_list);
   while (p != (const RegistryInfo *) NULL)
@@ -157,7 +157,7 @@ MagickExport MagickBooleanType DeleteMagickRegistry(const long id)
 */
 MagickExport void DestroyMagickRegistry(void)
 {
-  registry_semaphore=AcquireSemaphoreInfo(registry_semaphore);
+  AcquireSemaphoreInfo(&registry_semaphore);
   if (registry_list != (SplayTreeInfo *) NULL)
     registry_list=DestroySplayTree(registry_list);
   RelinquishSemaphoreInfo(registry_semaphore);
@@ -205,13 +205,13 @@ MagickExport Image *GetImageFromMagickRegistry(const char *name,long *id,
   if ((registry_list == (SplayTreeInfo *) NULL) ||
       (GetNumberOfNodesInSplayTree(registry_list) == 0))
     {
-      (void) ThrowMagickException(exception,GetMagickModule(),RegistryError,
+      (void) ThrowMagickException(exception,GetMagickModule(),RegistryWarning,
         "UnableToLocateRegistryImage","`%s'",name);
       return((Image *) NULL);
     }
   *id=(-1);
   image=NewImageList();
-  registry_semaphore=AcquireSemaphoreInfo(registry_semaphore);
+  AcquireSemaphoreInfo(&registry_semaphore);
   ResetSplayTreeIterator(registry_list);
   p=(const RegistryInfo *) GetNextValueInSplayTree(registry_list);
   while (p != (const RegistryInfo *) NULL)
@@ -227,7 +227,7 @@ MagickExport Image *GetImageFromMagickRegistry(const char *name,long *id,
   }
   RelinquishSemaphoreInfo(registry_semaphore);
   if (image == (Image *) NULL)
-    (void) ThrowMagickException(exception,GetMagickModule(),RegistryError,
+    (void) ThrowMagickException(exception,GetMagickModule(),RegistryWarning,
       "UnableToLocateRegistryImage","`%s'",name);
   return(image);
 }
@@ -289,7 +289,7 @@ MagickExport void *GetMagickRegistry(const long id,RegistryType *type,
   blob=(void *) NULL;
   *type=UndefinedRegistryType;
   *length=0;
-  registry_semaphore=AcquireSemaphoreInfo(registry_semaphore);
+  AcquireSemaphoreInfo(&registry_semaphore);
   ResetSplayTreeIterator(registry_list);
   p=(const RegistryInfo *) GetNextValueInSplayTree(registry_list);
   while (p != (const RegistryInfo *) NULL)
@@ -325,9 +325,13 @@ MagickExport void *GetMagickRegistry(const long id,RegistryType *type,
           blob=(void *) AcquireMagickMemory(p->length);
           if (blob == (void *) NULL)
             {
+              char
+                *message;
+
+              message=GetExceptionMessage(errno);
               (void) ThrowMagickException(exception,GetMagickModule(),
-                ResourceLimitError,"MemoryAllocationFailed","`%s'",
-                strerror(errno));
+                ResourceLimitError,"MemoryAllocationFailed","`%s'",message);
+              message=(char *) RelinquishMagickMemory(message);
               break;
             }
           (void) CopyMagickMemory(blob,p->blob,p->length);
@@ -432,14 +436,24 @@ MagickExport long SetMagickRegistry(const RegistryType type,const void *blob,
       image=(Image *) blob;
       if (length != sizeof(Image))
         {
+          char
+            *message;
+
+          message=GetExceptionMessage(errno);
           (void) ThrowMagickException(exception,GetMagickModule(),RegistryError,
-            "UnableToSetRegistry","`%s'",strerror(errno));
+            "UnableToSetRegistry","`%s'",message);
+          message=(char *) RelinquishMagickMemory(message);
           return(-1);
         }
       if (image->signature != MagickSignature)
         {
+          char
+            *message;
+
+          message=GetExceptionMessage(errno);
           (void) ThrowMagickException(exception,GetMagickModule(),RegistryError,
-            "UnableToSetRegistry","`%s'",strerror(errno));
+            "UnableToSetRegistry","`%s'",message);
+          message=(char *) RelinquishMagickMemory(message);
           return(-1);
         }
       clone_blob=(void *) CloneImageList(image,exception);
@@ -455,14 +469,24 @@ MagickExport long SetMagickRegistry(const RegistryType type,const void *blob,
       image_info=(ImageInfo *) blob;
       if (length != sizeof(ImageInfo))
         {
+          char
+            *message;
+
+          message=GetExceptionMessage(errno);
           (void) ThrowMagickException(exception,GetMagickModule(),RegistryError,
-            "UnableToSetRegistry","`%s'",strerror(errno));
+            "UnableToSetRegistry","`%s'",message);
+          message=(char *) RelinquishMagickMemory(message);
           return(-1);
         }
       if (image_info->signature != MagickSignature)
         {
+          char
+            *message;
+
+          message=GetExceptionMessage(errno);
           (void) ThrowMagickException(exception,GetMagickModule(),RegistryError,
-            "UnableToSetRegistry","`%s'",strerror(errno));
+            "UnableToSetRegistry","`%s'",message);
+          message=(char *) RelinquishMagickMemory(message);
           return(-1);
         }
       clone_blob=(void *) CloneImageInfo(image_info);
@@ -480,27 +504,48 @@ MagickExport long SetMagickRegistry(const RegistryType type,const void *blob,
   }
   registry_info=(RegistryInfo *) AcquireMagickMemory(sizeof(*registry_info));
   if (registry_info == (RegistryInfo *) NULL)
-    ThrowMagickFatalException(ResourceLimitFatalError,
-      "UnableToAllocateRegistryInfo",strerror(errno));
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowMagickFatalException(ResourceLimitFatalError,
+        "MemoryAllocationFailed",message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
   (void) ResetMagickMemory(registry_info,0,sizeof(*registry_info));
   registry_info->type=type;
   registry_info->blob=clone_blob;
   registry_info->length=length;
   registry_info->signature=MagickSignature;
-  registry_semaphore=AcquireSemaphoreInfo(registry_semaphore);
+  AcquireSemaphoreInfo(&registry_semaphore);
   registry_info->id=id++;
   if (registry_list == (SplayTreeInfo *) NULL)
     {
       registry_list=NewSplayTree((int (*)(const void *,const void *)) NULL,
         (void *(*)(void *)) NULL,DestroyRegistryNode);
       if (registry_list == (SplayTreeInfo *) NULL)
-        ThrowMagickFatalException(ResourceLimitFatalError,
-          "UnableToAllocateRegistryInfo",strerror(errno));
+        {
+          char
+            *message;
+
+          message=GetExceptionMessage(errno);
+          ThrowMagickFatalException(ResourceLimitFatalError,
+            "MemoryAllocationFailed",message);
+          message=(char *) RelinquishMagickMemory(message);
+        }
     }
   status=AddValueToSplayTree(registry_list,(const void *) id,registry_info);
   if (status == MagickFalse)
-    (void) ThrowMagickException(exception,GetMagickModule(),ResourceLimitError,
-      "MemoryAllocationFailed","`%s'",strerror(errno));
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      (void) ThrowMagickException(exception,GetMagickModule(),
+        ResourceLimitError,"MemoryAllocationFailed","`%s'",message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
   RelinquishSemaphoreInfo(registry_semaphore);
   return(registry_info->id);
 }
