@@ -17,7 +17,7 @@
 %                               September 2002                                %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -46,6 +46,7 @@
 #include "magick/log.h"
 #include "magick/image.h"
 #include "magick/memory_.h"
+#include "magick/option.h"
 #include "magick/random_.h"
 #include "magick/resource_.h"
 #include "magick/semaphore.h"
@@ -131,8 +132,7 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
   char
     resource_current[MaxTextExtent],
     resource_limit[MaxTextExtent],
-    resource_request[MaxTextExtent],
-    resource_type[MaxTextExtent];
+    resource_request[MaxTextExtent];
 
   MagickBooleanType
     status;
@@ -142,16 +142,15 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
 
   status=MagickFalse;
   FormatSize(size,resource_request);
-  resource_semaphore=AcquireSemaphoreInfo(resource_semaphore);
+  AcquireSemaphoreInfo(&resource_semaphore);
   switch (type)
   {
     case AreaResource:
     {
-      resource_info.area+=size;
+      resource_info.area=(MagickOffsetType) size;
       limit=MegabytesToBytes(resource_info.area_limit);
       status=(resource_info.area_limit == ResourceInfinity) || (size < limit) ?
         MagickTrue : MagickFalse;
-      (void) FormatMagickString(resource_type,MaxTextExtent,"area");
       FormatSize((MagickSizeType) resource_info.area,resource_current);
       FormatSize(MegabytesToBytes(resource_info.area_limit),resource_limit);
       break;
@@ -163,7 +162,6 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
       status=(resource_info.memory_limit == ResourceInfinity) ||
         ((MagickSizeType) resource_info.memory < limit) ?
         MagickTrue : MagickFalse;
-      (void) FormatMagickString(resource_type,MaxTextExtent,"memory");
       FormatSize((MagickSizeType) resource_info.memory,resource_current);
       FormatSize(MegabytesToBytes(resource_info.memory_limit),resource_limit);
       break;
@@ -175,7 +173,6 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
       status=(resource_info.map_limit == ResourceInfinity) ||
         ((MagickSizeType) resource_info.map < limit) ?
         MagickTrue : MagickFalse;
-      (void) FormatMagickString(resource_type,MaxTextExtent,"map");
       FormatSize((MagickSizeType) resource_info.map,resource_current);
       FormatSize(MegabytesToBytes(resource_info.map_limit),resource_limit);
       break;
@@ -187,7 +184,6 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
       status=(resource_info.disk_limit == ResourceInfinity) ||
         ((MagickSizeType) resource_info.disk < limit) ?
         MagickTrue : MagickFalse;
-      (void) FormatMagickString(resource_type,MaxTextExtent,"disk");
       FormatSize((MagickSizeType) resource_info.disk,resource_current);
       FormatSize(GigabytesToBytes(resource_info.disk_limit),resource_limit);
       break;
@@ -199,7 +195,6 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
       status=(resource_info.file_limit == ResourceInfinity) ||
         ((MagickSizeType) resource_info.file < limit) ?
         MagickTrue : MagickFalse;
-      (void) FormatMagickString(resource_type,MaxTextExtent,"file");
       FormatSize((MagickSizeType) resource_info.file,resource_current);
       FormatSize((MagickSizeType) resource_info.file_limit,resource_limit);
       break;
@@ -209,7 +204,8 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
   }
   RelinquishSemaphoreInfo(resource_semaphore);
   (void) LogMagickEvent(ResourceEvent,GetMagickModule(),"%s: %s/%s/%s",
-    resource_type,resource_request,resource_current,resource_limit);
+    MagickOptionToMnemonic(MagickResourceOptions,(long) type),resource_request,
+    resource_current,resource_limit);
   return(status);
 }
 
@@ -279,6 +275,12 @@ MagickExport void AsynchronousDestroyMagickResources(void)
 %
 */
 
+static void *DestroyTemporaryResources(void *temporary_resource)
+{
+  remove((char *) temporary_resource);
+  return((void *) NULL);
+}
+
 static MagickBooleanType GetPathTemplate(char *path)
 {
   char
@@ -290,35 +292,43 @@ static MagickBooleanType GetPathTemplate(char *path)
   struct stat
     file_info;
 
-  (void) strcpy(path,"magick-XXXXXXXX");
-  directory=getenv("MAGICK_TMPDIR");
+  (void) CopyMagickString(path,"magick-XXXXXXXX",MaxTextExtent);
+  directory=GetEnvironmentValue("MAGICK_TMPDIR");
   if (directory == (char *) NULL)
-    directory=getenv("TMPDIR");
+    directory=GetEnvironmentValue("TMPDIR");
 #if defined(__WINDOWS__)
   if (directory == (char *) NULL)
-    directory=getenv("TMP");
+    directory=GetEnvironmentValue("TMP");
   if (directory == (char *) NULL)
-    directory=getenv("TEMP");
+    directory=GetEnvironmentValue("TEMP");
 #endif
 #if defined(__VMS)
-   directory=getenv("MTMPDIR");
+  if (directory == (char *) NULL)
+    directory=GetEnvironmentValue("MTMPDIR");
 #endif
 #if defined(P_tmpdir)
   if (directory == (char *) NULL)
-    directory=P_tmpdir;
+    directory=ConstantString(P_tmpdir);
 #endif
   if (directory == (char *) NULL)
     return(MagickTrue);
   if (strlen(directory) > (MaxTextExtent-15))
-    return(MagickTrue);
+    {
+      directory=(char *) RelinquishMagickMemory(directory);
+      return(MagickTrue);
+    }
   status=stat(directory,&file_info);
   if ((status != 0) || !S_ISDIR(file_info.st_mode))
-    return(MagickTrue);
+    {
+      directory=(char *) RelinquishMagickMemory(directory);
+      return(MagickTrue);
+    }
   if (directory[strlen(directory)-1] == *DirectorySeparator)
     (void) FormatMagickString(path,MaxTextExtent,"%smagick-XXXXXXXX",directory);
   else
     (void) FormatMagickString(path,MaxTextExtent,"%s%smagick-XXXXXXXX",
       directory,DirectorySeparator);
+  directory=(char *) RelinquishMagickMemory(directory);
   return(MagickTrue);
 }
 
@@ -375,12 +385,13 @@ MagickExport int AcquireUniqueFileResource(char *path)
   (void) LogMagickEvent(ResourceEvent,GetMagickModule(),"%s",path);
   if (file == -1)
     return(file);
-  (void) AcquireMagickResource(FileResource,1);
+  AcquireSemaphoreInfo(&resource_semaphore);
   if (temporary_resources == (SplayTreeInfo *) NULL)
     temporary_resources=NewSplayTree(CompareSplayTreeString,
-      RelinquishMagickMemory,(void *(*)(void *)) NULL);
-  resource=ConstantString(AcquireString(path));
-  (void) AddValueToSplayTree(temporary_resources,resource,(void *) NULL);
+      RelinquishMagickMemory,DestroyTemporaryResources);
+  RelinquishSemaphoreInfo(resource_semaphore);
+  resource=ConstantString(path);
+  (void) AddValueToSplayTree(temporary_resources,resource,resource);
   return(file);
 }
 
@@ -405,24 +416,9 @@ MagickExport int AcquireUniqueFileResource(char *path)
 */
 MagickExport void DestroyMagickResources(void)
 {
-  resource_semaphore=AcquireSemaphoreInfo(resource_semaphore);
+  AcquireSemaphoreInfo(&resource_semaphore);
   if (temporary_resources != (SplayTreeInfo *) NULL)
-    {
-      const char
-        *path;
-
-      /*
-        Remove any lingering temporary files.
-      */
-      ResetSplayTreeIterator(temporary_resources);
-      path=(const char *) GetNextKeyInSplayTree(temporary_resources);
-      while (path != (const char *) NULL)
-      {
-        (void) remove(path);
-        path=(const char *) GetNextKeyInSplayTree(temporary_resources);
-      }
-      temporary_resources=DestroySplayTree(temporary_resources);
-    }
+    temporary_resources=DestroySplayTree(temporary_resources);
   RelinquishSemaphoreInfo(resource_semaphore);
   resource_semaphore=DestroySemaphoreInfo(resource_semaphore);
 }
@@ -456,7 +452,7 @@ MagickExport unsigned long GetMagickResource(const ResourceType type)
     resource;
 
   resource=0;
-  resource_semaphore=AcquireSemaphoreInfo(resource_semaphore);
+  AcquireSemaphoreInfo(&resource_semaphore);
   switch (type)
   {
     case AreaResource:
@@ -496,6 +492,69 @@ MagickExport unsigned long GetMagickResource(const ResourceType type)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   G e t M a g i c k R e s o u r c e L i m i t                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetMagickResource() returns the the specified resource limit in megabytes.
+%
+%  The format of the GetMagickResourceLimit() method is:
+%
+%      unsigned long GetMagickResourceLimit(const ResourceType type)
+%
+%  A description of each parameter follows:
+%
+%    o type: The type of resource.
+%
+*/
+MagickExport unsigned long GetMagickResourceLimit(const ResourceType type)
+{
+  unsigned long
+    resource;
+
+  resource=0;
+  AcquireSemaphoreInfo(&resource_semaphore);
+  switch (type)
+  {
+    case AreaResource:
+    {
+      resource=resource_info.area_limit;
+      break;
+    }
+    case MemoryResource:
+    {
+      resource=resource_info.memory_limit;
+      break;
+    }
+    case MapResource:
+    {
+      resource=resource_info.map_limit;
+      break;
+    }
+    case DiskResource:
+    {
+      resource=resource_info.disk_limit;
+      break;
+    }
+    case FileResource:
+    {
+      resource=resource_info.file_limit;
+      break;
+    }
+    default:
+      break;
+  }
+  RelinquishSemaphoreInfo(resource_semaphore);
+  return(resource);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 +   I n i t i a l i z e M a g i c k R e s o u r c e s                         %
 %                                                                             %
 %                                                                             %
@@ -512,8 +571,8 @@ MagickExport unsigned long GetMagickResource(const ResourceType type)
 */
 MagickExport void InitializeMagickResources(void)
 {
-  const char
-    *p;
+  char
+    *limit;
 
   long
     files,
@@ -544,33 +603,46 @@ MagickExport void InitializeMagickResources(void)
 #if defined(PixelCacheThreshold)
   memory=PixelCacheThreshold;
 #endif
-  (void) SetMagickResourceLimit(AreaResource,memory/2);
-  (void) SetMagickResourceLimit(MemoryResource,memory);
-  (void) SetMagickResourceLimit(MapResource,2*memory);
-  p=getenv("MAGICK_AREA_LIMIT");
-  if (p != (const char *) NULL)
-    (void) SetMagickResourceLimit(AreaResource,(unsigned long) atol(p));
-  p=getenv("MAGICK_MEMORY_LIMIT");
-  if (p != (const char *) NULL)
-    (void) SetMagickResourceLimit(MemoryResource,(unsigned long) atol(p));
-  p=getenv("MAGICK_MAP_LIMIT");
-  if (p != (const char *) NULL)
-    (void) SetMagickResourceLimit(MapResource,(unsigned long) atol(p));
-  p=getenv("MAGICK_DISK_LIMIT");
-  if (p != (const char *) NULL)
-    (void) SetMagickResourceLimit(DiskResource,(unsigned long) atol(p));
+  (void) SetMagickResourceLimit(AreaResource,memory);
+  (void) SetMagickResourceLimit(MemoryResource,2*memory);
+  (void) SetMagickResourceLimit(MapResource,4*memory);
+  limit=GetEnvironmentValue("MAGICK_AREA_LIMIT");
+  if (limit != (char *) NULL)
+    {
+      (void) SetMagickResourceLimit(AreaResource,(unsigned long) atol(limit));
+      limit=(char *) RelinquishMagickMemory(limit);
+    }
+  limit=GetEnvironmentValue("MAGICK_MEMORY_LIMIT");
+  if (limit != (char *) NULL)
+    {
+      (void) SetMagickResourceLimit(MemoryResource,(unsigned long) atol(limit));
+      limit=(char *) RelinquishMagickMemory(limit);
+    }
+  limit=GetEnvironmentValue("MAGICK_MAP_LIMIT");
+  if (limit != (char *) NULL)
+    {
+      (void) SetMagickResourceLimit(MapResource,(unsigned long) atol(limit));
+      limit=(char *) RelinquishMagickMemory(limit);
+    }
+  limit=GetEnvironmentValue("MAGICK_DISK_LIMIT");
+  if (limit != (char *) NULL)
+    {
+      (void) SetMagickResourceLimit(DiskResource,(unsigned long) atol(limit));
+      limit=(char *) RelinquishMagickMemory(limit);
+    }
   files=(-1);
 #if defined(HAVE_SYSCONF) && defined(_SC_OPEN_MAX)
   files=sysconf(_SC_OPEN_MAX);
 #elif defined(HAVE_GETDTABLESIZE) && defined(POSIX)
   files=getdtablesize();
 #endif
-  if (files <= 0)
-    files=64;
-  (void) SetMagickResourceLimit(FileResource,(unsigned long) Max(3*files/4,8));
-  p=getenv("MAGICK_FILES_LIMIT");
-  if (p != (const char *) NULL)
-    (void) SetMagickResourceLimit(FileResource,(unsigned long) atol(p));
+  (void) SetMagickResourceLimit(FileResource,(unsigned long) Max(files-64,64));
+  limit=GetEnvironmentValue("MAGICK_FILES_LIMIT");
+  if (limit != (char *) NULL)
+    {
+      (void) SetMagickResourceLimit(FileResource,(unsigned long) atol(limit));
+      limit=(char *) RelinquishMagickMemory(limit);
+    }
 }
 
 /*
@@ -611,7 +683,7 @@ MagickExport MagickBooleanType ListMagickResourceInfo(FILE *file,
 
   if (file == (const FILE *) NULL)
     file=stdout;
-  resource_semaphore=AcquireSemaphoreInfo(resource_semaphore);
+  AcquireSemaphoreInfo(&resource_semaphore);
   FormatSize(MegabytesToBytes(resource_info.area_limit),area_limit);
   FormatSize(GigabytesToBytes(resource_info.disk_limit),disk_limit);
   FormatSize((MagickSizeType) resource_info.file_limit,file_limit);
@@ -658,17 +730,15 @@ MagickExport void RelinquishMagickResource(const ResourceType type,
   char
     resource_current[MaxTextExtent],
     resource_limit[MaxTextExtent],
-    resource_request[MaxTextExtent],
-    resource_type[MaxTextExtent];
+    resource_request[MaxTextExtent];
 
   FormatSize(size,resource_request);
-  resource_semaphore=AcquireSemaphoreInfo(resource_semaphore);
+  AcquireSemaphoreInfo(&resource_semaphore);
   switch (type)
   {
     case AreaResource:
     {
-      resource_info.area-=size;
-      (void) FormatMagickString(resource_type,MaxTextExtent,"area");
+      resource_info.area=(MagickOffsetType) size;
       FormatSize((MagickSizeType) resource_info.area,resource_current);
       FormatSize(MegabytesToBytes(resource_info.area_limit),resource_limit);
       break;
@@ -676,7 +746,6 @@ MagickExport void RelinquishMagickResource(const ResourceType type,
     case MemoryResource:
     {
       resource_info.memory-=size;
-      (void) FormatMagickString(resource_type,MaxTextExtent,"memory");
       FormatSize((MagickSizeType) resource_info.memory,resource_current);
       FormatSize(MegabytesToBytes(resource_info.memory_limit),resource_limit);
       break;
@@ -684,7 +753,6 @@ MagickExport void RelinquishMagickResource(const ResourceType type,
     case MapResource:
     {
       resource_info.map-=size;
-      (void) FormatMagickString(resource_type,MaxTextExtent,"map");
       FormatSize((MagickSizeType) resource_info.map,resource_current);
       FormatSize(MegabytesToBytes(resource_info.map_limit),resource_limit);
       break;
@@ -692,7 +760,6 @@ MagickExport void RelinquishMagickResource(const ResourceType type,
     case DiskResource:
     {
       resource_info.disk-=size;
-      (void) FormatMagickString(resource_type,MaxTextExtent,"disk");
       FormatSize((MagickSizeType) resource_info.disk,resource_current);
       FormatSize(GigabytesToBytes(resource_info.disk_limit),resource_limit);
       break;
@@ -700,7 +767,6 @@ MagickExport void RelinquishMagickResource(const ResourceType type,
     case FileResource:
     {
       resource_info.file-=size;
-      (void) FormatMagickString(resource_type,MaxTextExtent,"file");
       FormatSize((MagickSizeType) resource_info.file,resource_current);
       FormatSize((MagickSizeType) resource_info.file_limit,resource_limit);
       break;
@@ -710,7 +776,8 @@ MagickExport void RelinquishMagickResource(const ResourceType type,
   }
   RelinquishSemaphoreInfo(resource_semaphore);
   (void) LogMagickEvent(ResourceEvent,GetMagickModule(),"%s: %s/%s/%s",
-    resource_type,resource_request,resource_current,resource_limit);
+    MagickOptionToMnemonic(MagickResourceOptions,(long) type),resource_request,
+    resource_current,resource_limit);
 }
 
 /*
@@ -757,15 +824,12 @@ MagickExport MagickBooleanType RelinquishUniqueFileResource(const char *path)
         p=(char *) GetNextKeyInSplayTree(temporary_resources);
       }
       if (p != (char *) NULL)
-        {
-          (void) RemoveNodeFromSplayTree(temporary_resources,p);
-          RelinquishMagickResource(FileResource,1);
-        }
+        (void) RemoveNodeFromSplayTree(temporary_resources,p);
     }
   (void) CopyMagickString(cache_path,path,MaxTextExtent);
   AppendImageFormat("cache",cache_path);
   (void) remove(cache_path);
-  return((MagickBooleanType) (remove(path) == 0));
+  return(remove(path) == 0 ? MagickTrue : MagickFalse);
 }
 
 /*
@@ -798,7 +862,7 @@ MagickExport MagickBooleanType RelinquishUniqueFileResource(const char *path)
 MagickExport MagickBooleanType SetMagickResourceLimit(const ResourceType type,
   const unsigned long limit)
 {
-  resource_semaphore=AcquireSemaphoreInfo(resource_semaphore);
+  AcquireSemaphoreInfo(&resource_semaphore);
   switch (type)
   {
     case AreaResource:
