@@ -17,7 +17,7 @@
 %                                April 1993                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -88,9 +88,11 @@
 #include "magick/exception.h"
 #include "magick/exception-private.h"
 #include "magick/image.h"
+#include "magick/image-private.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/quantize.h"
+#include "magick/quantum.h"
 #include "magick/segment.h"
 #include "magick/string_.h"
 
@@ -234,7 +236,8 @@ static void
 */
 static MagickBooleanType Classify(Image *image,short **extrema,
   const MagickRealType cluster_threshold,
-  const MagickRealType weighting_exponent,const MagickBooleanType verbose)
+  const MagickRealType magick_unused(weighting_exponent),
+  const MagickBooleanType verbose)
 {
 #define SegmentImageTag  "Segment/Image"
 
@@ -410,7 +413,7 @@ static MagickBooleanType Classify(Image *image,short **extrema,
   {
     next_cluster=cluster->next;
     if ((cluster->count > 0) &&
-        ((MagickRealType) cluster->count >= (count*cluster_threshold/100.0)))
+        (cluster->count >= (count*cluster_threshold/100.0)))
       {
         /*
           Initialize cluster.
@@ -498,9 +501,12 @@ static MagickBooleanType Classify(Image *image,short **extrema,
   cluster=head;
   for (i=0; i < (long) number_clusters; i++)
   {
-    image->colormap[i].red=ScaleCharToQuantum(cluster->red.center);
-    image->colormap[i].green=ScaleCharToQuantum(cluster->green.center);
-    image->colormap[i].blue=ScaleCharToQuantum(cluster->blue.center);
+    image->colormap[i].red=ScaleCharToQuantum((unsigned char)
+      (cluster->red.center+0.5));
+    image->colormap[i].green=ScaleCharToQuantum((unsigned char)
+      (cluster->green.center+0.5));
+    image->colormap[i].blue=ScaleCharToQuantum((unsigned char)
+       (cluster->blue.center+0.5));
     cluster=cluster->next;
   }
   /*
@@ -913,7 +919,7 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   (void) ResetMagickMemory(&pixel,0,sizeof(pixel));
   for (i=0; i < MaxDimension; i++)
   {
@@ -1112,11 +1118,14 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
       }
     }
   threshold=(background->red.center+object->red.center)/2.0;
-  pixel.red=(MagickRealType) ScaleCharToQuantum(threshold);
+  pixel.red=(MagickRealType) ScaleCharToQuantum((unsigned char)
+    (threshold+0.5));
   threshold=(background->green.center+object->green.center)/2.0;
-  pixel.green=(MagickRealType) ScaleCharToQuantum(threshold);
+  pixel.green=(MagickRealType) ScaleCharToQuantum((unsigned char)
+    (threshold+0.5));
   threshold=(background->blue.center+object->blue.center)/2.0;
-  pixel.blue=(MagickRealType) ScaleCharToQuantum(threshold);
+  pixel.blue=(MagickRealType) ScaleCharToQuantum((unsigned char)
+    (threshold+0.5));
   /*
     Free resources.
   */
@@ -1565,7 +1574,8 @@ static MagickRealType OptimalTau(const long *histogram,const double max_tau,
     /*
       Find the value of the peak.
     */
-    peak=(MagickBooleanType) (zero_crossing[k].crossings[node->right] == -1);
+    peak=zero_crossing[k].crossings[node->right] == -1 ? MagickTrue :
+      MagickFalse;
     index=node->left;
     value=zero_crossing[k].histogram[index];
     for (x=node->left; x <= node->right; x++)
@@ -1735,7 +1745,7 @@ MagickExport MagickBooleanType SegmentImage(Image *image,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   for (i=0; i < MaxDimension; i++)
   {
     histogram[i]=(long *) AcquireMagickMemory(256*sizeof(**histogram));
