@@ -3,11 +3,17 @@
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%                       SSSSS  PPPP   L       AAA   Y   Y                     %
-%                       SS     P   P  L      A   A   Y Y                      %
-%                        SSS   PPPP   L      AAAAA    Y                       %
-%                          SS  P      L      A   A    Y                       %
-%                       SSSSS  P      LLLLL  A   A    Y                       %
+%                      SSSSS  PPPP   L       AAA   Y   Y                      %
+%                      SS     P   P  L      A   A   Y Y                       %
+%                       SSS   PPPP   L      AAAAA    Y                        %
+%                         SS  P      L      A   A    Y                        %
+%                      SSSSS  P      LLLLL  A   A    Y                        %
+%                                                                             %
+%                         TTTTT  RRRR   EEEEE  EEEEE                          %
+%                           T    R   R  E      E                              %
+%                           T    RRRR   EEE    EEE                            %
+%                           T    R R    E      E                              %
+%                           T    R  R   EEEEE  EEEEE                          %
 %                                                                             %
 %                                                                             %
 %                 See Self-adjusting Binary Search Tree Methods               %
@@ -17,7 +23,7 @@
 %                               December 2002                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -153,7 +159,7 @@ MagickExport MagickBooleanType AddValueToSplayTree(SplayTreeInfo *splay_info,
   register NodeInfo
     *node;
 
-  splay_info->semaphore=AcquireSemaphoreInfo(splay_info->semaphore);
+  AcquireSemaphoreInfo(&splay_info->semaphore);
   SplaySplayTree(splay_info,key);
   compare=0;
   if (splay_info->root != (NodeInfo *) NULL)
@@ -165,14 +171,14 @@ MagickExport MagickBooleanType AddValueToSplayTree(SplayTreeInfo *splay_info,
           ((splay_info->root->key < key) ? -1 : 0);
       if (compare == 0)
         {
-          if ((splay_info->relinquish_key != (void *(*)(void *)) NULL) &&
-              (splay_info->root->key != (void *) NULL))
-            splay_info->root->key=
-              splay_info->relinquish_key(splay_info->root->key);
           if ((splay_info->relinquish_value != (void *(*)(void *)) NULL) &&
               (splay_info->root->value != (void *) NULL))
-            splay_info->root->value=
-              splay_info->relinquish_value(splay_info->root->value);
+            splay_info->root->value=splay_info->relinquish_value(
+              splay_info->root->value);
+          if ((splay_info->relinquish_key != (void *(*)(void *)) NULL) &&
+              (splay_info->root->key != (void *) NULL))
+            splay_info->root->key=splay_info->relinquish_key(
+              splay_info->root->key);
           splay_info->root->key=(void *) key;
           splay_info->root->value=(void *) value;
           RelinquishSemaphoreInfo(splay_info->semaphore);
@@ -181,8 +187,7 @@ MagickExport MagickBooleanType AddValueToSplayTree(SplayTreeInfo *splay_info,
     }
   node=(NodeInfo *) AcquireMagickMemory(sizeof(*node));
   if (node == (NodeInfo *) NULL)
-    ThrowMagickFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-      strerror(errno));
+    return(MagickFalse);
   node->key=(void *) key;
   node->value=(void *) value;
   if (splay_info->root == (NodeInfo *) NULL)
@@ -259,10 +264,10 @@ static NodeInfo *LinkSplayTreeNodes(NodeInfo **nodes,unsigned long low,
 
 static int SplayTreeToNodeArray(NodeInfo *node,const void *nodes)
 {
-  register NodeInfo
+  register const NodeInfo
     ***p;
 
-  p=(NodeInfo ***) nodes;
+  p=(const NodeInfo ***) nodes;
   *(*p)=node;
   (*p)++;
   return(0);
@@ -281,8 +286,15 @@ static void BalanceSplayTree(SplayTreeInfo *splay_info)
     }
   nodes=(NodeInfo **) AcquireMagickMemory(splay_info->nodes*sizeof(*nodes));
   if (nodes == (NodeInfo **) NULL)
-    ThrowMagickFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-      strerror(errno));
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowMagickFatalException(ResourceLimitFatalError,
+        "MemoryAllocationFailed",message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
   node=nodes;
   (void) IterateOverSplayTree(splay_info,SplayTreeToNodeArray,
     (const void *) &node);
@@ -318,12 +330,12 @@ static void BalanceSplayTree(SplayTreeInfo *splay_info)
 */
 MagickExport int CompareSplayTreeString(const void *target,const void *source)
 {
-  char
+  const char
     *p,
     *q;
 
-  p=(char *) target;
-  q=(char *) source;
+  p=(const char *) target;
+  q=(const char *) source;
   return(LocaleCompare(p,q));
 }
 
@@ -355,12 +367,12 @@ MagickExport int CompareSplayTreeString(const void *target,const void *source)
 MagickExport int CompareSplayTreeStringInfo(const void *target,
   const void *source)
 {
-  StringInfo
+  const StringInfo
     *p,
     *q;
 
-  p=(StringInfo *) target;
-  q=(StringInfo *) source;
+  p=(const StringInfo *) target;
+  q=(const StringInfo *) source;
   return(CompareStringInfo(p,q));
 }
 
@@ -395,51 +407,53 @@ MagickExport SplayTreeInfo *DestroySplayTree(SplayTreeInfo *splay_info)
     *active,
     *pend;
 
-  if (splay_info->root == (NodeInfo *) NULL)
-    return((SplayTreeInfo *) NULL);
-  splay_info->semaphore=AcquireSemaphoreInfo(splay_info->semaphore);
-  if ((splay_info->relinquish_key != (void *(*)(void *)) NULL) &&
-      (splay_info->root->key != (void *) NULL))
-    splay_info->root->key=splay_info->relinquish_key(splay_info->root->key);
-  if ((splay_info->relinquish_value != (void *(*)(void *)) NULL) &&
-      (splay_info->root->value != (void *) NULL))
-    splay_info->root->value=
-      splay_info->relinquish_value(splay_info->root->value);
-  splay_info->root->key=(void *) NULL;
-  for (pend=splay_info->root; pend != (NodeInfo *) NULL; )
-  {
-    active=pend;
-    for (pend=(NodeInfo *) NULL; active != (NodeInfo *) NULL; )
+  AcquireSemaphoreInfo(&splay_info->semaphore);
+  if (splay_info->root != (NodeInfo *) NULL)
     {
-      if (active->left != (NodeInfo *) NULL)
+      if ((splay_info->relinquish_value != (void *(*)(void *)) NULL) &&
+          (splay_info->root->value != (void *) NULL))
+        splay_info->root->value=splay_info->relinquish_value(
+          splay_info->root->value);
+      if ((splay_info->relinquish_key != (void *(*)(void *)) NULL) &&
+          (splay_info->root->key != (void *) NULL))
+        splay_info->root->key=splay_info->relinquish_key(splay_info->root->key);
+      splay_info->root->key=(void *) NULL;
+      for (pend=splay_info->root; pend != (NodeInfo *) NULL; )
+      {
+        active=pend;
+        for (pend=(NodeInfo *) NULL; active != (NodeInfo *) NULL; )
         {
-          if ((splay_info->relinquish_key != (void *(*)(void *)) NULL) &&
-              (active->left->key != (void *) NULL))
-            active->left->key=splay_info->relinquish_key(active->left->key);
-          if ((splay_info->relinquish_value != (void *(*)(void *)) NULL) &&
-              (active->left->value != (void *) NULL))
-            active->left->value=
-              splay_info->relinquish_value(active->left->value);
-          active->left->key=(void *) pend;
-          pend=active->left;
+          if (active->left != (NodeInfo *) NULL)
+            {
+              if ((splay_info->relinquish_value != (void *(*)(void *)) NULL) &&
+                  (active->left->value != (void *) NULL))
+                active->left->value=splay_info->relinquish_value(
+                  active->left->value);
+              if ((splay_info->relinquish_key != (void *(*)(void *)) NULL) &&
+                  (active->left->key != (void *) NULL))
+                active->left->key=splay_info->relinquish_key(active->left->key);
+              active->left->key=(void *) pend;
+              pend=active->left;
+            }
+          if (active->right != (NodeInfo *) NULL)
+            {
+              if ((splay_info->relinquish_value != (void *(*)(void *)) NULL) &&
+                  (active->right->value != (void *) NULL))
+                active->right->value=splay_info->relinquish_value(
+                  active->right->value);
+              if ((splay_info->relinquish_key != (void *(*)(void *)) NULL) &&
+                  (active->right->key != (void *) NULL))
+                active->right->key=splay_info->relinquish_key(
+                  active->right->key);
+              active->right->key=(void *) pend;
+              pend=active->right;
+            }
+          node=active;
+          active=(NodeInfo *) node->key;
+          node=(NodeInfo *) RelinquishMagickMemory(node);
         }
-      if (active->right != (NodeInfo *) NULL)
-        {
-          if ((splay_info->relinquish_key != (void *(*)(void *)) NULL) &&
-              (active->right->key != (void *) NULL))
-            active->right->key=splay_info->relinquish_key(active->right->key);
-          if ((splay_info->relinquish_value != (void *(*)(void *)) NULL) &&
-              (active->right->value != (void *) NULL))
-            active->right->value=
-              splay_info->relinquish_value(active->right->value);
-          active->right->key=(void *) pend;
-          pend=active->right;
-        }
-      node=active;
-      active=(NodeInfo *) node->key;
-      node=(NodeInfo *) RelinquishMagickMemory(node);
+      }
     }
-  }
   splay_info->signature=(~MagickSignature);
   RelinquishSemaphoreInfo(splay_info->semaphore);
   splay_info->semaphore=DestroySemaphoreInfo(splay_info->semaphore);
@@ -483,7 +497,7 @@ MagickExport void *GetNextKeyInSplayTree(SplayTreeInfo *splay_info)
   if ((splay_info->root == (NodeInfo *) NULL) ||
       (splay_info->next == (void *) NULL))
     return((void *) NULL);
-  splay_info->semaphore=AcquireSemaphoreInfo(splay_info->semaphore);
+  AcquireSemaphoreInfo(&splay_info->semaphore);
   SplaySplayTree(splay_info,splay_info->next);
   splay_info->next=(void *) NULL;
   node=splay_info->root->right;
@@ -533,7 +547,7 @@ MagickExport void *GetNextValueInSplayTree(SplayTreeInfo *splay_info)
   if ((splay_info->root == (NodeInfo *) NULL) ||
       (splay_info->next == (void *) NULL))
     return((void *) NULL);
-  splay_info->semaphore=AcquireSemaphoreInfo(splay_info->semaphore);
+  AcquireSemaphoreInfo(&splay_info->semaphore);
   SplaySplayTree(splay_info,splay_info->next);
   splay_info->next=(void *) NULL;
   node=splay_info->root->right;
@@ -583,7 +597,7 @@ MagickExport void *GetValueFromSplayTree(SplayTreeInfo *splay_info,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   if (splay_info->root == (NodeInfo *) NULL)
     return((void *) NULL);
-  splay_info->semaphore=AcquireSemaphoreInfo(splay_info->semaphore);
+  AcquireSemaphoreInfo(&splay_info->semaphore);
   SplaySplayTree(splay_info,key);
   if (splay_info->compare != (int (*)(const void *,const void *)) NULL)
     compare=splay_info->compare(splay_info->root->key,key);
@@ -670,9 +684,6 @@ static int IterateOverSplayTree(SplayTreeInfo *splay_info,
     UpTransition
   } TransitionType;
 
-  char
-    *transitions;
-
   int
     status;
 
@@ -691,48 +702,58 @@ static int IterateOverSplayTree(SplayTreeInfo *splay_info,
   TransitionType
     transition;
 
+  unsigned char
+    *transitions;
+
   if (splay_info->root == (NodeInfo *) NULL)
     return(0);
   nodes=(NodeInfo **) AcquireMagickMemory(splay_info->nodes*sizeof(*nodes));
-  transitions=(char *)
+  transitions=(unsigned char *)
     AcquireMagickMemory(splay_info->nodes*sizeof(*transitions));
-  if ((nodes == (NodeInfo **) NULL) || (transitions == (char *) NULL))
-    ThrowMagickFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-      strerror(errno));
+  if ((nodes == (NodeInfo **) NULL) || (transitions == (unsigned char *) NULL))
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowMagickFatalException(ResourceLimitFatalError,
+        "MemoryAllocationFailed",message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
   status=0;
   final_transition=MagickFalse;
   nodes[0]=splay_info->root;
-  transitions[0]=LeftTransition;
+  transitions[0]=(unsigned char) LeftTransition;
   for (i=0; final_transition == MagickFalse; )
   {
     node=nodes[i];
-    transition=transitions[i];
+    transition=(TransitionType) transitions[i];
     switch (transition)
     {
       case LeftTransition:
       {
-        transitions[i]=DownTransition;
+        transitions[i]=(unsigned char) DownTransition;
         if (node->left == (NodeInfo *) NULL)
           break;
         i++;
         nodes[i]=node->left;
-        transitions[i]=LeftTransition;
+        transitions[i]=(unsigned char) LeftTransition;
         break;
       }
       case RightTransition:
       {
-        transitions[i]=UpTransition;
+        transitions[i]=(unsigned char) UpTransition;
         if (node->right == (NodeInfo *) NULL)
           break;
         i++;
         nodes[i]=node->right;
-        transitions[i]=LeftTransition;
+        transitions[i]=(unsigned char) LeftTransition;
         break;
       }
       case DownTransition:
       default:
       {
-        transitions[i]=RightTransition;
+        transitions[i]=(unsigned char) RightTransition;
         status=(*method)(node,value);
         if (status != 0)
           final_transition=MagickTrue;
@@ -751,7 +772,7 @@ static int IterateOverSplayTree(SplayTreeInfo *splay_info,
     }
   }
   nodes=(NodeInfo **) RelinquishMagickMemory(nodes);
-  transitions=(char *) RelinquishMagickMemory(transitions);
+  transitions=(unsigned char *) RelinquishMagickMemory(transitions);
   return(status);
 }
 
@@ -796,8 +817,15 @@ MagickExport SplayTreeInfo *NewSplayTree(
 
   splay_info=(SplayTreeInfo *) AcquireMagickMemory(sizeof(*splay_info));
   if (splay_info == (SplayTreeInfo *) NULL)
-    ThrowMagickFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-      strerror(errno));
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowMagickFatalException(ResourceLimitFatalError,
+        "MemoryAllocationFailed",message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
   (void) ResetMagickMemory(splay_info,0,sizeof(*splay_info));
   splay_info->root=(NodeInfo *) NULL;
   splay_info->compare=compare;
@@ -866,17 +894,18 @@ MagickExport MagickBooleanType RemoveNodeByValueFromSplayTree(
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   if (splay_info->root == (NodeInfo *) NULL)
     return(MagickFalse);
-  splay_info->semaphore=AcquireSemaphoreInfo(splay_info->semaphore);
-  for (next=GetFirstSplayTreeNode(splay_info); next != (void *) NULL; )
+  AcquireSemaphoreInfo(&splay_info->semaphore);
+  next=(NodeInfo *) GetFirstSplayTreeNode(splay_info);
+  while (next != (NodeInfo *) NULL)
   {
     SplaySplayTree(splay_info,next);
-    next=(void *) NULL;
+    next=(NodeInfo *) NULL;
     node=splay_info->root->right;
     if (node != (NodeInfo *) NULL)
       {
         while (node->left != (NodeInfo *) NULL)
           node=node->left;
-        next=node->key;
+        next=(NodeInfo *) node->key;
       }
     if (splay_info->root->value == value)
       {
@@ -929,7 +958,7 @@ MagickExport MagickBooleanType RemoveNodeFromSplayTree(
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   if (splay_info->root == (NodeInfo *) NULL)
     return(MagickFalse);
-  splay_info->semaphore=AcquireSemaphoreInfo(splay_info->semaphore);
+  AcquireSemaphoreInfo(&splay_info->semaphore);
   SplaySplayTree(splay_info,key);
   splay_info->key=(void *) NULL;
   if (splay_info->compare != (int (*)(const void *,const void *)) NULL)
@@ -944,13 +973,13 @@ MagickExport MagickBooleanType RemoveNodeFromSplayTree(
     }
   left=splay_info->root->left;
   right=splay_info->root->right;
+  if ((splay_info->relinquish_value != (void *(*)(void *)) NULL) &&
+      (splay_info->root->value != (void *) NULL))
+    splay_info->root->value=splay_info->relinquish_value(
+      splay_info->root->value);
   if ((splay_info->relinquish_key != (void *(*)(void *)) NULL) &&
       (splay_info->root->key != (void *) NULL))
     splay_info->root->key=splay_info->relinquish_key(splay_info->root->key);
-  if ((splay_info->relinquish_value != (void *(*)(void *)) NULL) &&
-      (splay_info->root->value != (void *) NULL))
-    splay_info->root->value=
-      splay_info->relinquish_value(splay_info->root->value);
   splay_info->root=(NodeInfo *) RelinquishMagickMemory(splay_info->root);
   splay_info->nodes--;
   if (left == (NodeInfo *) NULL)
@@ -1155,8 +1184,15 @@ static void SplaySplayTree(SplayTreeInfo *splay_info,const void *key)
       (void) Splay(splay_info,key,&splay_info->root,(NodeInfo **) NULL,
         (NodeInfo **) NULL);
       if (splay_info->balance != MagickFalse)
-        ThrowMagickFatalException(ResourceLimitFatalError,
-          "MemoryAllocationFailed",strerror(errno));
+        {
+          char
+            *message;
+
+          message=GetExceptionMessage(errno);
+          ThrowMagickFatalException(ResourceLimitFatalError,
+            "MemoryAllocationFailed",message);
+          message=(char *) RelinquishMagickMemory(message);
+        }
     }
   splay_info->key=(void *) key;
 }
