@@ -113,8 +113,15 @@ static inline size_t CheckOverflowException(const size_t length,
   const size_t extend)
 {
   if ((length+extend) < length)
-    ThrowMagickFatalException(ResourceLimitFatalError,"UnableToAcquireString",
-      strerror(errno));
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowMagickFatalException(ResourceLimitFatalError,
+        "MemoryAllocationFailed",message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
   return(length+extend);
 }
 
@@ -136,7 +143,7 @@ MagickExport char *AcquireString(const char *source)
       source);
   *destination='\0';
   if (source != (char *) NULL)
-    (void) strcpy(destination,source);
+    (void) CopyMagickString(destination,source,length*sizeof(*destination));
   return(destination);
 }
 
@@ -169,8 +176,15 @@ MagickExport StringInfo *AcquireStringInfo(const size_t length)
 
   string_info=(StringInfo *) AcquireMagickMemory(sizeof(*string_info));
   if (string_info == (StringInfo *) NULL)
-    ThrowMagickFatalException(ResourceLimitFatalError,"UnableToAcquireString",
-      strerror(errno));
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowMagickFatalException(ResourceLimitFatalError,
+        "MemoryAllocationFailed",message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
   (void) ResetMagickMemory(string_info,0,sizeof(*string_info));
   string_info->signature=MagickSignature;
   string_info->length=length;
@@ -183,8 +197,15 @@ MagickExport StringInfo *AcquireStringInfo(const size_t length)
       string_info->datum=(unsigned char *)
         AcquireMagickMemory(length*sizeof(*string_info->datum));
       if (string_info->datum == (unsigned char *) NULL)
-        ThrowMagickFatalException(ResourceLimitFatalError,
-          "UnableToAcquireString",strerror(errno));
+        {
+          char
+            *message;
+
+          message=GetExceptionMessage(errno);
+          ThrowMagickFatalException(ResourceLimitFatalError,
+            "MemoryAllocationFailed",message);
+          message=(char *) RelinquishMagickMemory(message);
+        }
     }
   return(string_info);
 }
@@ -234,12 +255,12 @@ MagickExport char *CloneString(char **destination,const char *source)
       return(*destination);
     }
   length=CheckOverflowException(strlen(source),MaxTextExtent);
-  *destination=(char *)
-    ResizeMagickMemory(*destination,length*sizeof(*destination));
+  *destination=(char *) ResizeMagickMemory(*destination,length*
+    sizeof(*destination));
   if (*destination == (char *) NULL)
     ThrowMagickFatalException(ResourceLimitFatalError,"UnableToAcquireString",
       source);
-  (void) strcpy(*destination,source);
+  (void) CopyMagickString(*destination,source,length*sizeof(*destination));
   return(*destination);
 }
 
@@ -362,7 +383,6 @@ MagickExport int CompareStringInfo(const StringInfo *target,
 %
 %    o length: The length of the destination string.
 %
-%
 */
 MagickExport size_t ConcatenateMagickString(char *destination,
   const char *source,const size_t length)
@@ -380,6 +400,9 @@ MagickExport size_t ConcatenateMagickString(char *destination,
   size_t
     count;
 
+  assert(destination != (char *) NULL);
+  assert(source != (const char *) NULL);
+  assert(length >= 1);
   p=source;
   q=destination;
   i=length;
@@ -453,7 +476,8 @@ MagickExport MagickBooleanType ConcatenateString(char **destination,
   if (*destination == (char *) NULL)
     ThrowMagickFatalException(ResourceLimitFatalError,
       "UnableToConcatenateString",source);
-  (void) strcat(*destination,source);
+  (void) ConcatenateMagickString(*destination,source,length*
+    sizeof(*destination));
   return(MagickTrue);
 }
 
@@ -505,6 +529,155 @@ MagickExport void ConcatenateStringInfo(StringInfo *string_info,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   C o n f i g u r e F i l e T o S t r i n g I n f o                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ConfigureFileToStringInfo() returns the contents of a configure file as a
+%  string.
+%
+%  The format of the ConfigureFileToStringInfo method is:
+%
+%      StringInfo *ConfigureFileToStringInfo(const char *filename)
+%        ExceptionInfo *exception)
+%
+%  A description of each parameter follows:
+%
+%    o filename: The filename.
+%
+*/
+MagickExport StringInfo *ConfigureFileToStringInfo(const char *filename)
+{
+  char
+    *string;
+
+  int
+    file;
+
+  MagickOffsetType
+    offset;
+
+  size_t
+    length;
+
+  StringInfo
+    *string_info;
+
+  void
+    *map;
+
+  assert(filename != (const char *) NULL);
+  file=open(filename,O_RDONLY | O_BINARY);
+  if (file == -1)
+    return((StringInfo *) NULL);
+  offset=(MagickOffsetType) MagickSeek(file,0,SEEK_END);
+  if ((offset < 0) || (offset != (MagickOffsetType) ((ssize_t) offset)))
+    {
+      file=close(file)-1;
+      return((StringInfo *) NULL);
+    }
+  length=(size_t) offset;
+  string=(char *) AcquireMagickMemory((length+1)*sizeof(*string));
+  if (string == (char *) NULL)
+    {
+      file=close(file)-1;
+      return((StringInfo *) NULL);
+    }
+  map=MapBlob(file,ReadMode,0,length);
+  if (map != (void *) NULL)
+    {
+      (void) CopyMagickMemory(string,map,length);
+      (void) UnmapBlob(map,length);
+    }
+  else
+    {
+      register size_t
+        i;
+
+      ssize_t
+        count;
+
+      (void) MagickSeek(file,0,SEEK_SET);
+      for (i=0; i < length; i+=count)
+      {
+        count=read(file,string+i,(size_t) Min(length-i,(size_t)
+          MagickMaxBufferSize));
+        if (count <= 0)
+          {
+            count=0;
+            if (errno != EINTR)
+              break;
+          }
+      }
+      if (i < length)
+        {
+          file=close(file)-1;
+          string=(char *) RelinquishMagickMemory(string);
+          return((StringInfo *) NULL);
+        }
+    }
+  string[length]='\0';
+  file=close(file)-1;
+  string_info=AcquireStringInfo(0);
+  (void) CopyMagickString(string_info->path,filename,MaxTextExtent);
+  string_info->length=strlen(string)+1;
+  string_info->datum=(unsigned char *) string;
+  return(string_info);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   C o n s t a n t S t r i n g                                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ConstantString() allocates memory for a string and copies the source string
+%  to that memory location (and returns it).  Use it for strings that you do
+%  do not expect to change over its lifetime.
+%
+%  The format of the ConstantString method is:
+%
+%      char *ConstantString(const char *source)
+%
+%  A description of each parameter follows:
+%
+%    o source: A character string.
+%
+*/
+MagickExport char *ConstantString(const char *source)
+{
+  char
+    *destination;
+
+  size_t
+    length;
+
+  length=0;
+  if (source != (char *) NULL)
+    length+=strlen(source);
+  length++;
+  destination=(char *) AcquireMagickMemory(length*sizeof(*destination));
+  if (destination == (char *) NULL)
+    ThrowMagickFatalException(ResourceLimitFatalError,"UnableToAcquireString",
+      source);
+  *destination='\0';
+  if (source != (char *) NULL)
+    (void) CopyMagickString(destination,source,length*sizeof(*destination));
+  return(destination);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   C o p y M a g i c k S t r i n g                                           %
 %                                                                             %
 %                                                                             %
@@ -528,7 +701,6 @@ MagickExport void ConcatenateStringInfo(StringInfo *string_info,
 %
 %    o length: The length of the destination string.
 %
-%
 */
 MagickExport size_t CopyMagickString(char *destination,const char *source,
   const size_t length)
@@ -543,6 +715,9 @@ MagickExport size_t CopyMagickString(char *destination,const char *source,
   register size_t
     i;
 
+  assert(destination != (char *) NULL);
+  assert(source != (const char *) NULL);
+  assert(length >= 1);
   p=source;
   q=destination;
   i=length;
@@ -569,41 +744,6 @@ MagickExport size_t CopyMagickString(char *destination,const char *source,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   C o n s t a n t S t r i n g                                               %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  ConstantString() trims a string of any excess memory.  This method is useful
-%  for strings that will not change over the life of the string.
-%
-%  The format of the ConstantString method is:
-%
-%      char *ConstantString(char *source)
-%
-%  A description of each parameter follows:
-%
-%    o source: A character string.
-%
-*/
-MagickExport char *ConstantString(char *source)
-{
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
-  if (source == (char *) NULL)
-    return((char *) NULL);
-  source=(char *) ResizeMagickMemory(source,(strlen(source)+1)*sizeof(*source));
-  if (source == (char *) NULL)
-    ThrowMagickFatalException(ResourceLimitFatalError,"UnableToAcquireString",
-      strerror(errno));
-  return(source);
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
 %   D e s t r o y S t r i n g                                                 %
 %                                                                             %
 %                                                                             %
@@ -781,83 +921,28 @@ MagickExport char *EscapeString(const char *source,const char escape)
 %
 %  The format of the FileToString method is:
 %
-%      char *FileToString(const char *filename)
+%      char *FileToString(const char *filename,const size_t extent,
+%        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
 %    o filename: The filename.
 %
+%    o extent: Maximum length of the string.
+%
+%    o exception: Return any errors or warnings in this structure.
 %
 */
-MagickExport char *FileToString(const char *filename)
+MagickExport char *FileToString(const char *filename,const size_t extent,
+  ExceptionInfo *exception)
 {
-  char
-    *string;
-
-  int
-    file;
-
-  MagickOffsetType
-    offset;
-
   size_t
     length;
 
-  void
-    *map;
-
   assert(filename != (const char *) NULL);
-  file=open(filename,O_RDONLY | O_BINARY);
-  if (file == -1)
-    return((char *) NULL);
-  offset=(MagickOffsetType) MagickSeek(file,0,SEEK_END);
-  if ((offset < 0) || (offset != (MagickOffsetType) ((ssize_t) offset)))
-    {
-      file=close(file)-1;
-      return((char *) NULL);
-    }
-  length=(size_t) offset;
-  string=(char *) AcquireMagickMemory((length+1)*sizeof(*string));
-  if (string == (char *) NULL)
-    {
-      file=close(file)-1;
-      return((char *) NULL);
-    }
-  map=MapBlob(file,ReadMode,0,length);
-  if (map != (void *) NULL)
-    {
-      (void) CopyMagickMemory(string,map,length);
-      (void) UnmapBlob(map,length);
-    }
-  else
-    {
-      register size_t
-        i;
-
-      ssize_t
-        count;
-
-      (void) MagickSeek(file,0,SEEK_SET);
-      for (i=0; i < length; i+=count)
-      {
-        count=read(file,string+i,Min(length-i,(size_t) MagickMaxBufferSize));
-        if (count <= 0)
-          {
-            count=0;
-            if (errno != EINTR)
-              break;
-          }
-      }
-      if (i < length)
-        {
-          file=close(file)-1;
-          string=(char *) RelinquishMagickMemory(string);
-          return((char *) NULL);
-        }
-    }
-  string[length]='\0';
-  file=close(file)-1;
-  return(string);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",filename);
+  assert(exception != (ExceptionInfo *) NULL);
+  return((char *) FileToBlob(filename,extent,&length,exception));
 }
 
 /*
@@ -875,33 +960,35 @@ MagickExport char *FileToString(const char *filename)
 %
 %  The format of the FileToStringInfo method is:
 %
-%      void *FileToStringInfo(const char *filename,ExceptionInfo *exception)
+%      StringInfo *FileToStringInfo(const char *filename,const size_t extent,
+%        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
 %    o filename: The filename.
 %
+%    o extent: Maximum length of the string.
+%
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
-MagickExport StringInfo *FileToStringInfo(const char *filename)
+MagickExport StringInfo *FileToStringInfo(const char *filename,
+  const size_t extent,ExceptionInfo *exception)
 {
-  char
-    *string;
-
   StringInfo
     *string_info;
 
   assert(filename != (const char *) NULL);
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",filename);
-  string=FileToString(filename);
-  if (string == (char *) NULL)
-    return((StringInfo *) NULL);
-  string_info=AcquireStringInfo(strlen(string)+1);
+  assert(exception != (ExceptionInfo *) NULL);
+  string_info=AcquireStringInfo(0);
   (void) CopyMagickString(string_info->path,filename,MaxTextExtent);
-  SetStringInfoDatum(string_info,(unsigned char *) string);
-  string=(char *) RelinquishMagickMemory(string);
+  string_info->datum=FileToBlob(filename,extent,&string_info->length,exception);
+  if (string_info->datum == (unsigned char *) NULL)
+    {
+      string_info=DestroyStringInfo(string_info);
+      return((StringInfo *) NULL);
+    }
   return(string_info);
 }
 
@@ -933,7 +1020,6 @@ MagickExport StringInfo *FileToStringInfo(const char *filename)
 %   o format:  A string describing the format to use to write the remaining
 %     arguments.
 %
-%
 */
 
 MagickExport long FormatMagickStringList(char *string,const size_t length,
@@ -972,6 +1058,96 @@ MagickExport long FormatMagickString(char *string,const size_t length,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%  F o r m a t M a g i c k T i m e                                            %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  FormatMagickTime() returns the specified time in the Internet date/time
+%  format and the length of the timestamp.
+%
+%  The format of the FormatMagickTime method is:
+%
+%      size_t FormatMagickTime(const time_t time,const size_t length,
+%        char *timestamp)
+%
+%  A description of each parameter follows.
+%
+%   o time:  the time since the Epoch (00:00:00 UTC, January 1, 1970),
+%     measured in seconds.
+%
+%   o length: The maximum length of the string.
+%
+%   o timestamp:  Return the Internet date/time here.
+%
+*/
+MagickExport long FormatMagickTime(const time_t time,const size_t length,
+  char *timestamp)
+{
+  long
+    count;
+
+  struct tm
+    local_meridian,
+    utc_meridian;
+
+  time_t
+    offset;
+
+  assert(timestamp != (char *) NULL);
+  local_meridian=(*localtime(&time));
+  utc_meridian=(*gmtime(&time));
+  offset=(time_t) ((local_meridian.tm_min-utc_meridian.tm_min)/60+
+    local_meridian.tm_hour-utc_meridian.tm_hour+24*((local_meridian.tm_year-
+    utc_meridian.tm_year) != 0 ? (local_meridian.tm_year-utc_meridian.tm_year) :
+    (local_meridian.tm_yday-utc_meridian.tm_yday)));
+  count=FormatMagickString(timestamp,length,
+    "%04d-%02d-%02dT%02d:%02d:%02d%+03ld:00",local_meridian.tm_year+1900,
+    local_meridian.tm_mon+1,local_meridian.tm_mday,local_meridian.tm_hour,
+    local_meridian.tm_min,local_meridian.tm_sec,offset);
+  return(count);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   G e t E n v i r o n m e n t V a l u e                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetEnvironmentValue() returns the environment string that matches the
+%  specified name.
+%
+%  The format of the GetEnvironmentValue method is:
+%
+%      char *GetEnvironmentValue(const char *name)
+%
+%  A description of each parameter follows:
+%
+%    o name: the environment name.
+%
+*/
+MagickExport char *GetEnvironmentValue(const char *name)
+{
+  const char
+    *environment;
+
+  environment=getenv(name);
+  if (environment == (const char *) NULL)
+    return((char *) NULL);
+  return(ConstantString(environment));
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   L o c a l e C o m p a r e                                                 %
 %                                                                             %
 %                                                                             %
@@ -999,6 +1175,8 @@ MagickExport long FormatMagickString(char *string,const size_t length,
 */
 MagickExport long LocaleCompare(const char *p,const char *q)
 {
+  if ((p == (char *) NULL) && (q == (char *) NULL))
+    return(0);
   if (p == (char *) NULL)
     return(-1);
   if (q == (char *) NULL)
@@ -1393,8 +1571,15 @@ MagickExport void SetStringInfoLength(StringInfo *string_info,
     string_info->datum=(unsigned char *) ResizeMagickMemory(string_info->datum,
       (length+MaxTextExtent)*sizeof(*string_info->datum));
   if (string_info->datum == (unsigned char *) NULL)
-    ThrowMagickFatalException(ResourceLimitFatalError,"UnableToAcquireString",
-      strerror(errno));
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowMagickFatalException(ResourceLimitFatalError,
+        "MemoryAllocationFailed",message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
 }
 
 /*
@@ -1467,7 +1652,7 @@ MagickExport StringInfo *StringToStringInfo(const char *string)
 
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(string != (const char *) NULL);
-  string_info=AcquireStringInfo(strlen(string));
+  string_info=AcquireStringInfo(strlen(string)+1);
   SetStringInfoDatum(string_info,(const unsigned char *) string);
   return(string_info);
 }
@@ -1505,10 +1690,17 @@ MagickExport char *StringInfoToString(const StringInfo *string_info)
   length=CheckOverflowException(string_info->length,MaxTextExtent);
   string=(char *) AcquireMagickMemory(length*sizeof(*string));
   if (string == (char *) NULL)
-    ThrowMagickFatalException(ResourceLimitFatalError,"UnableToAcquireString",
-      strerror(errno));
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowMagickFatalException(ResourceLimitFatalError,
+        "MemoryAllocationFailed",message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
   (void) CopyMagickString(string,(char *) string_info->datum,
-    string_info->length+1);
+    length*sizeof(*string));
   return(string);
 }
 
@@ -1546,7 +1738,7 @@ MagickExport char **StringToArgv(const char *text,int *argc)
   char
     **argv;
 
-  register char
+  register const char
     *p,
     *q;
 
@@ -1559,7 +1751,7 @@ MagickExport char **StringToArgv(const char *text,int *argc)
   /*
     Determine the number of arguments.
   */
-  for (p=(char *) text; *p != '\0'; )
+  for (p=text; *p != '\0'; )
   {
     while (isspace((int) ((unsigned char) *p)) != 0)
       p++;
@@ -1580,7 +1772,7 @@ MagickExport char **StringToArgv(const char *text,int *argc)
     Convert string to an ASCII list.
   */
   argv[0]=AcquireString("magick");
-  p=(char *) text;
+  p=text;
   for (i=1; i < (long) *argc; i++)
   {
     while (isspace((int) ((unsigned char) *p)) != 0)
@@ -1836,8 +2028,6 @@ MagickExport void StripString(char *message)
   if (*message == '\0')
     return;
   length=strlen(message);
-  if (length == 1)
-    return;
   p=message;
   while (isspace((int) ((unsigned char) *p)) != 0)
     p++;
@@ -1895,7 +2085,7 @@ MagickExport MagickBooleanType SubstituteString(char **buffer,
     *match,
     *source;
 
-  ssize_t
+  MagickOffsetType
     destination_offset;
 
   size_t
@@ -1910,6 +2100,8 @@ MagickExport MagickBooleanType SubstituteString(char **buffer,
   assert(*buffer != (char *) NULL);
   assert(search != (const char*) NULL);
   assert(replace != (const char*) NULL);
+  if (strcmp(search,replace) == 0)
+    return(MagickTrue);
   source=(*buffer);
   match=strstr(source,search);
   if (match == (char *) NULL)
@@ -1945,28 +2137,29 @@ MagickExport MagickBooleanType SubstituteString(char **buffer,
         (void) CopyMagickString(result+destination_offset,source,copy_length+1);
         destination_offset+=copy_length;
       }
-      /*
-        Copy replacement.
-      */
-      result_length+=replace_length;
-      if ((result_length+MaxTextExtent) >= allocate_length)
-        {
-          allocate_length+=replace_length;
-          result=(char *) ResizeMagickMemory(result,
-            (allocate_length+MaxTextExtent)*sizeof(*result));
-          if (result == (char *) NULL)
-            ThrowMagickFatalException(ResourceLimitFatalError,
-              "UnableToAcquireString",search);
-        }
-      (void) strcat(result+destination_offset,replace);
-      destination_offset+=replace_length;
-      /*
-        Find next match.
-      */
-      source=match;
-      source+=search_length;
-      match=strstr(source,search);
-    }
+    /*
+      Copy replacement.
+    */
+    result_length+=replace_length;
+    if ((result_length+MaxTextExtent) >= allocate_length)
+      {
+        allocate_length+=replace_length;
+        result=(char *) ResizeMagickMemory(result,(allocate_length+
+          MaxTextExtent)*sizeof(*result));
+        if (result == (char *) NULL)
+          ThrowMagickFatalException(ResourceLimitFatalError,
+            "UnableToAcquireString",search);
+      }
+    (void) ConcatenateMagickString(result+destination_offset,replace,
+      (allocate_length+MaxTextExtent)*sizeof(*result));
+    destination_offset+=replace_length;
+    /*
+      Find next match.
+    */
+    source=match;
+    source+=search_length;
+    match=strstr(source,search);
+  }
   /*
     Copy remaining string.
   */
@@ -1981,7 +2174,8 @@ MagickExport MagickBooleanType SubstituteString(char **buffer,
         ThrowMagickFatalException(ResourceLimitFatalError,
           "UnableToAcquireString",search);
     }
-  (void) strcat(result+destination_offset,source);
+  (void) ConcatenateMagickString(result+destination_offset,source,
+    (allocate_length+MaxTextExtent-destination_offset)*sizeof(*result));
   (void) RelinquishMagickMemory(*buffer);
   *buffer=result;
   return(MagickTrue);
