@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -13,15 +13,17 @@
   See the License for the specific language governing permissions and
   limitations under the License.
 
-  ImageMagick string methods.
+  MagickCore string methods.
 */
-#ifndef _MAGICK_STRING_H_
-#define _MAGICK_STRING_H_
+#ifndef _MAGICKCORE_STRING_H_
+#define _MAGICKCORE_STRING_H_
 
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
 #endif
 
+#include <stdarg.h>
+#include <time.h>
 #include "magick/exception.h"
 
 typedef struct _StringInfo
@@ -42,11 +44,12 @@ typedef struct _StringInfo
 extern MagickExport char
   *AcquireString(const char *),
   *CloneString(char **,const char *),
-  *ConstantString(char *),
+  *ConstantString(const char *),
   *DestroyString(char *),
   **DestroyStringList(char **),
   *EscapeString(const char *,const char),
-  *FileToString(const char *),
+  *FileToString(const char *,const size_t,ExceptionInfo *),
+  *GetEnvironmentValue(const char *),
   **StringToArgv(const char *,int *),
   **StringToList(const char *),
   *StringInfoToString(const StringInfo *);
@@ -54,9 +57,12 @@ extern MagickExport char
 extern MagickExport double
   StringToDouble(const char *,const double);
 
-extern MagickExport int
+extern MagickExport long
   FormatMagickString(char *,const size_t,const char *,...)
     magick_attribute((format (printf,3,4))),
+  FormatMagickStringList(char *,const size_t,const char *,va_list)
+    magick_attribute((format (printf,3,0))),
+  FormatMagickTime(const time_t,const size_t,char *),
   LocaleCompare(const char *,const char *),
   LocaleNCompare(const char *,const char *,const size_t);
 
@@ -74,8 +80,9 @@ extern MagickExport size_t
 extern MagickExport StringInfo
   *AcquireStringInfo(const size_t),
   *CloneStringInfo(const StringInfo *),
+  *ConfigureFileToStringInfo(const char *),
   *DestroyStringInfo(StringInfo *),
-  *FileToStringInfo(const char *),
+  *FileToStringInfo(const char *,const size_t,ExceptionInfo *),
   *SplitStringInfo(StringInfo *,const size_t),
   *StringToStringInfo(const char *);
 
