@@ -17,7 +17,7 @@
 %                              January 1993                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -81,14 +81,14 @@
 */
 MagickExport void GetMagickToken(const char *start,char **end,char *token)
 {
-  register char
+  register const char
     *p;
 
   register long
     i;
 
   i=0;
-  for (p=(char *) start; *p != '\0'; )
+  for (p=start; *p != '\0'; )
   {
     while ((isspace((int) ((unsigned char) *p)) != 0) && (*p != '\0'))
       p++;
@@ -128,12 +128,10 @@ MagickExport void GetMagickToken(const char *start,char **end,char *token)
       }
       case '/':
       {
-        if (*(p+1) == '>')
-          {
-            token[i++]=(*p++);
-            token[i++]=(*p++);
-            break;
-          }
+        token[i++]=(*p++);
+        if ((*p == '>') || (*p == '/'))
+          token[i++]=(*p++);
+        break;
       }
       default:
       {
@@ -158,11 +156,13 @@ MagickExport void GetMagickToken(const char *start,char **end,char *token)
         for ( ; *p != '\0'; p++)
         {
           if (((isspace((int) ((unsigned char) *p)) != 0) || (*p == '=') ||
-               (*p == ',') || (*p == ':')) && (*(p-1) != '\\'))
+              (*p == ',') || (*p == ':')) && (*(p-1) != '\\'))
             break;
           if ((i > 0) && (*p == '<'))
             break;
           token[i++]=(*p);
+          if (*p == '>')
+            break;
           if (*p == '(')
             for (p++; *p != '\0'; p++)
             {
@@ -183,8 +183,16 @@ MagickExport void GetMagickToken(const char *start,char **end,char *token)
       (void) CopyMagickString(token,token+5,MaxTextExtent);
       token[i-6]='\0';
     }
+  if (LocaleNCompare(token,"url(",4) == 0)
+    {
+      i=(long) strlen(token);
+      (void) CopyMagickString(token,token+4,MaxTextExtent);
+      token[i-5]='\0';
+    }
+  while (isspace((int) ((unsigned char) *p)) != 0)
+    p++;
   if (end != (char **) NULL)
-    *end=p;
+    *end=(char *) p;
 }
 
 /*
@@ -247,7 +255,7 @@ MagickExport MagickBooleanType GlobExpression(const char *expression,
       (void) CopyMagickString(image_info->filename,pattern,MaxTextExtent);
       GetExceptionInfo(&exception);
       (void) SetImageInfo(image_info,MagickTrue,&exception);
-      DestroyExceptionInfo(&exception);
+      (void) DestroyExceptionInfo(&exception);
       if (LocaleCompare(image_info->filename,pattern) != 0)
         {
           image_info=DestroyImageInfo(image_info);
@@ -281,7 +289,7 @@ MagickExport MagickBooleanType GlobExpression(const char *expression,
         pattern++;
         status=MagickFalse;
         while ((*expression != '\0') && (status == MagickFalse))
-          status=GlobExpression((char *) expression++,pattern);
+          status=GlobExpression(expression++,pattern);
         if (status != MagickFalse)
           {
             while (*expression != '\0')
@@ -384,7 +392,7 @@ MagickExport MagickBooleanType GlobExpression(const char *expression,
           {
             if (*pattern == '\\')
               pattern++;
-            match=(MagickBooleanType) (*pattern == *p);
+            match=(*pattern == *p) ? MagickTrue : MagickFalse;
             p++;
             pattern++;
           }
@@ -473,12 +481,12 @@ MagickExport MagickBooleanType IsGlob(const char *path)
   MagickBooleanType
     status;
 
-  status=(MagickBooleanType) ((strchr(path,'*') != (char *) NULL) ||
+  status=(strchr(path,'*') != (char *) NULL) ||
     (strchr(path,'?') != (char *) NULL) ||
     (strchr(path,'{') != (char *) NULL) ||
     (strchr(path,'}') != (char *) NULL) ||
     (strchr(path,'[') != (char *) NULL) ||
-    (strchr(path,']') != (char *) NULL));
+    (strchr(path,']') != (char *) NULL) ? MagickTrue : MagickFalse;
   return(status);
 }
 
@@ -569,9 +577,10 @@ MagickExport MagickBooleanType IsGlob(const char *path)
 %
 %  The format of the Tokenizer method is:
 %
-%      int Tokenizer(TokenInfo *token_info,unsigned flag,char *token,
-%        size_t max_token_length,char *line,char *white,char *break_set,
-%        char *quote,char escape,char *breaker,int *next,char *quoted)
+%      int Tokenizer(TokenInfo *token_info,const unsigned flag,char *token,
+%        const size_t max_token_length,const char *line,const char *white,
+%        const char *break_set,const char *quote,const char escape,
+%        char *breaker,int *next,char *quoted)
 %
 %  A description of each parameter follows:
 %
@@ -648,9 +657,9 @@ MagickExport MagickBooleanType IsGlob(const char *path)
 #define IN_QUOTE 2
 #define IN_OZONE 3
 
-static long sindex(int c,char *string)
+static long sindex(int c,const char *string)
 {
-  register char
+  register const char
     *p;
 
   for (p=string; *p != '\0'; p++)
@@ -689,9 +698,10 @@ static void StoreToken(TokenInfo *token_info,char *string,
   }
 }
 
-MagickExport int Tokenizer(TokenInfo *token_info,unsigned flag,char *token,
-  size_t max_token_length,char *line,char *white,char *break_set,char *quote,
-  char escape,char *breaker,int *next,char *quoted)
+MagickExport int Tokenizer(TokenInfo *token_info,const unsigned flag,
+  char *token,const size_t max_token_length,const char *line,const char *white,
+	const char *break_set,const char *quote,const char escape,char *breaker,
+  int *next,char *quoted)
 {
   int
     c;
@@ -699,8 +709,8 @@ MagickExport int Tokenizer(TokenInfo *token_info,unsigned flag,char *token,
   register long
     i;
 
-  *breaker=(char) MagickFalse;
-  *quoted=(char) MagickFalse;
+  *breaker='\0';
+  *quoted='\0';
   if (line[*next] == '\0')
     return(1);
   token_info->state=IN_WHITE;
