@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -125,7 +125,7 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   assert(chop_info != (RectangleInfo *) NULL);
@@ -250,7 +250,7 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
 %
 %  The format of the CoalesceImages method is:
 %
-%      Image *CoalesceImages(const Image *image,ExceptionInfo *exception)
+%      Image *CoalesceImages(Image *image,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -259,78 +259,148 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
 %    o exception: Return any errors or warnings in this structure.
 %
 */
-MagickExport Image *CoalesceImages(const Image *image,ExceptionInfo *exception)
+MagickExport Image *CoalesceImages(Image *image,ExceptionInfo *exception)
 {
   Image
-    *coalesce_image,
-    *previous_image;
+    *coalesce_image;
 
-  register const Image
+  Image
+    *dispose_image;
+
+  long
+    y;
+
+  register const PixelPacket
+    *p;
+
+  register Image
     *next;
 
+  register long
+    x;
+
+  register PixelPacket
+    *q;
+
+  RectangleInfo
+    region;
+
   /*
     Coalesce the image sequence.
   */
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   coalesce_image=CloneImage(image,image->page.width,image->page.height,
     MagickTrue,exception);
   if (coalesce_image == (Image *) NULL)
     return((Image *) NULL);
+  coalesce_image->page=image->page;
+  coalesce_image->page.x=0;
+  coalesce_image->page.y=0;
+  coalesce_image->dispose=NoneDispose;
   coalesce_image->background_color.opacity=TransparentOpacity;
   SetImageBackgroundColor(coalesce_image);
-  (void) CompositeImage(coalesce_image,image->matte != MagickFalse ?
-    OverCompositeOp : CopyCompositeOp,image,image->page.x,image->page.y);
-  previous_image=coalesce_image;
-  for (next=image->next; next != (Image *) NULL; next=next->next)
+  dispose_image=CloneImage(coalesce_image,0,0,MagickTrue,exception);
+  (void) CompositeImage(coalesce_image,CopyCompositeOp,image,image->page.x,
+    image->page.y);
+  next=GetNextImageInList(image);
+  for ( ; next != (Image *) NULL; next=GetNextImageInList(next))
   {
-    switch (next->dispose)
-    {
-      case UndefinedDispose:
-      case NoneDispose:
+    /*
+      Determine the region that was overlaid in the previous image.
+    */
+    region=GetPreviousImageInList(next)->page;
+    region.width=GetPreviousImageInList(next)->columns;
+    region.height=GetPreviousImageInList(next)->rows;
+    if (region.x < 0)
       {
-        coalesce_image->next=CloneImage(coalesce_image,0,0,MagickTrue,
-          exception);
-        if (coalesce_image->next != (Image *) NULL)
-          previous_image=coalesce_image->next;
-        break;
+        region.width+=region.x;
+        region.x=0;
       }
-      case BackgroundDispose:
+    if ((region.x+region.width) > (long) coalesce_image->columns)
+      region.width=coalesce_image->columns-region.x;
+    if (region.y < 0)
       {
-        coalesce_image->next=CloneImage(coalesce_image,0,0,MagickTrue,
-          exception);
-        if (coalesce_image->next != (Image *) NULL)
+        region.height+=region.y;
+        region.y=0;
+      }
+    if ((region.y+region.height) > (long) coalesce_image->rows)
+      region.height=coalesce_image->rows-region.y;
+    if (GetPreviousImageInList(next)->dispose != PreviousDispose)
+      {
+        /*
+          Replace the dispose image with the new coalesced image.
+        */
+        dispose_image=DestroyImage(dispose_image);
+        dispose_image=CloneImage(coalesce_image,0,0,MagickTrue,exception);
+        if (dispose_image == (Image *) NULL)
           {
-            coalesce_image->next->background_color.opacity=TransparentOpacity;
-            SetImageBackgroundColor(coalesce_image->next);
+            coalesce_image=DestroyImageList(coalesce_image);
+            return((Image *) NULL);
           }
-        break;
       }
-      case PreviousDispose:
-      default:
+    /*
+      Background: Clear the overlaid area of the coalesced region,
+    */
+    if (next->previous->dispose == BackgroundDispose)
+      for (y=0; y < (long) region.height; y++)
       {
-        coalesce_image->next=CloneImage(previous_image,0,0,MagickTrue,
-          exception);
-        break;
-      }
-    }
-    if (coalesce_image->next == (Image *) NULL)
-      {
-        coalesce_image=DestroyImageList(coalesce_image);
-        return((Image *) NULL);
+        q=GetImagePixels(dispose_image,region.x,region.y+y,region.width,1);
+        if (q == (PixelPacket *) NULL)
+          break;
+        for (x=0; x < (long) region.width; x++)
+        {
+          q->opacity=TransparentOpacity;
+          q++;
+        }
+        if (SyncImagePixels(image) == MagickFalse)
+          break;
       }
+    /*
+      Next image is the dispose image, overlay with next image in sequence.
+    */
+    coalesce_image->next=CloneImage(dispose_image,0,0,MagickTrue,exception);
     coalesce_image->next->previous=coalesce_image;
-    coalesce_image=coalesce_image->next;
+    coalesce_image=GetNextImageInList(coalesce_image);
+    coalesce_image->matte=MagickTrue;
     coalesce_image->delay=next->delay;
+    coalesce_image->ticks_per_second=next->ticks_per_second;
     coalesce_image->start_loop=next->start_loop;
     (void) CompositeImage(coalesce_image,next->matte != MagickFalse ?
       OverCompositeOp : CopyCompositeOp,next,next->page.x,next->page.y);
-    coalesce_image->matte=next->matte;
+    /*
+      Fix disposal setting of previous image, for correct animation.
+    */
+    for (y=0; y < (long) region.height; y++)
+    {
+      p=AcquireImagePixels(GetPreviousImageInList(coalesce_image),region.x,
+        region.y+y,region.width,1,exception);
+      q=GetImagePixels(coalesce_image,region.x,region.y+y,region.width,1);
+      if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
+        break;
+      for (x=0; x < (long) region.width; x++)
+      {
+        if (((double) p->opacity <= (QuantumRange/2.0)) &&
+            ((double) q->opacity > (QuantumRange/2.0)))
+          break;
+        p++;
+        q++;
+      }
+      if (x < (long) region.width)
+        break;
+    }
+    /*
+      If pixel goes opaque to transparent, use background dispose.
+    */
+    GetPreviousImageInList(coalesce_image)->dispose=y < (long) region.height ?
+      BackgroundDispose : NoneDispose;
+    coalesce_image->dispose=GetPreviousImageInList(coalesce_image)->dispose;
   }
+  dispose_image=DestroyImage(dispose_image);
   return(GetFirstImageInList(coalesce_image));
 }
 
@@ -388,7 +458,7 @@ MagickExport Image *ConsolidateCMYKImages(const Image *images,
   assert(images != (Image *) NULL);
   assert(images->signature == MagickSignature);
   if (images->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),images->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",images->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   cmyk_images=NewImageList();
@@ -398,7 +468,8 @@ MagickExport Image *ConsolidateCMYKImages(const Image *images,
       exception);
     if (cmyk_image == (Image *) NULL)
       break;
-    cmyk_image->storage_class=DirectClass;
+    if (SetImageStorageClass(cmyk_image,DirectClass) == MagickFalse)
+      break;
     cmyk_image->colorspace=CMYKColorspace;
     for (y=0; y < (long) images->rows; y++)
     {
@@ -408,14 +479,14 @@ MagickExport Image *ConsolidateCMYKImages(const Image *images,
         break;
       for (x=0; x < (long) images->columns; x++)
       {
-        q->red=MaxRGB-PixelIntensityToQuantum(p);
+        q->red=QuantumRange-PixelIntensityToQuantum(p);
         p++;
         q++;
       }
       if (SyncImagePixels(cmyk_image) == MagickFalse)
         break;
     }
-    images=images->next;
+    images=GetNextImageInList(images);
     if (images == (Image *) NULL)
       break;
     for (y=0; y < (long) images->rows; y++)
@@ -426,14 +497,14 @@ MagickExport Image *ConsolidateCMYKImages(const Image *images,
         break;
       for (x=0; x < (long) images->columns; x++)
       {
-        q->green=MaxRGB-PixelIntensityToQuantum(p);
+        q->green=QuantumRange-PixelIntensityToQuantum(p);
         p++;
         q++;
       }
       if (SyncImagePixels(cmyk_image) == MagickFalse)
         break;
     }
-    images=images->next;
+    images=GetNextImageInList(images);
     if (images == (Image *) NULL)
       break;
     for (y=0; y < (long) images->rows; y++)
@@ -444,14 +515,14 @@ MagickExport Image *ConsolidateCMYKImages(const Image *images,
         break;
       for (x=0; x < (long) images->columns; x++)
       {
-        q->blue=MaxRGB-PixelIntensityToQuantum(p);
+        q->blue=QuantumRange-PixelIntensityToQuantum(p);
         p++;
         q++;
       }
       if (SyncImagePixels(cmyk_image) == MagickFalse)
         break;
     }
-    images=images->next;
+    images=GetNextImageInList(images);
     if (images == (Image *) NULL)
       break;
     for (y=0; y < (long) images->rows; y++)
@@ -463,7 +534,7 @@ MagickExport Image *ConsolidateCMYKImages(const Image *images,
       indexes=GetIndexes(cmyk_image);
       for (x=0; x < (long) images->columns; x++)
       {
-        indexes[x]=MaxRGB-PixelIntensityToQuantum(p);
+        indexes[x]=QuantumRange-PixelIntensityToQuantum(p);
         p++;
       }
       if (SyncImagePixels(cmyk_image) == MagickFalse)
@@ -541,7 +612,7 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(geometry != (const RectangleInfo *) NULL);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
@@ -575,7 +646,7 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
     }
   if (page.x < 0)
     {
-      page.width+=page.x;
+      page.width+=page.x-bounding_box.x;
       page.x=0;
     }
   else
@@ -587,7 +658,7 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
     }
   if (page.y < 0)
     {
-      page.height+=page.y;
+      page.height+=page.y-bounding_box.y;
       page.y=0;
     }
   else
@@ -708,13 +779,13 @@ MagickExport Image *DeconstructImages(const Image *image,
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   /*
     Ensure the image are the same size.
   */
-  for (next=image; next != (Image *) NULL; next=next->next)
+  for (next=image; next != (Image *) NULL; next=GetNextImageInList(next))
   {
     if ((next->columns != image->columns) || (next->rows != image->rows))
       ThrowImageException(OptionError,"ImagesAreNotTheSameSize");
@@ -876,12 +947,12 @@ MagickExport Image *DeconstructImages(const Image *image,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  FlattenImage() merges a sequence of images.  This is useful for combining
-%  Photoshop layers into a single image.
+%  FlattenImage() composites all images from the current image pointer to the
+%  end of the image list and returns a single flattened image.
 %
 %  The format of the FlattenImage method is:
 %
-%      Image *FlattenImage(const Image *image,ExceptionInfo *exception)
+%      Image *FlattenImage(Image *image,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -890,7 +961,7 @@ MagickExport Image *DeconstructImages(const Image *image,
 %    o exception: Return any errors or warnings in this structure.
 %
 */
-MagickExport Image *FlattenImages(const Image *image,ExceptionInfo *exception)
+MagickExport Image *FlattenImages(Image *image,ExceptionInfo *exception)
 {
 #define FlattenImageTag  "Flatten/Image"
 
@@ -915,7 +986,7 @@ MagickExport Image *FlattenImages(const Image *image,ExceptionInfo *exception)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   page=image->page;
@@ -936,8 +1007,8 @@ MagickExport Image *FlattenImages(const Image *image,ExceptionInfo *exception)
   /*
     Flatten the images.
   */
-  number_images=GetImageListLength(image);
-  for (scene=0; scene < (long) number_images; scene++)
+  number_images=GetImageListLength(image)-GetImageIndexInList(image);
+  for (scene=0; image != (Image *) NULL; scene++)
   {
     (void) CompositeImage(flatten_image,image->compose,image,image->page.x,
       image->page.y);
@@ -1009,7 +1080,7 @@ MagickExport Image *FlipImage(const Image *image,ExceptionInfo *exception)
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   flip_image=CloneImage(image,image->columns,image->rows,MagickTrue,exception);
@@ -1104,7 +1175,7 @@ MagickExport Image *FlopImage(const Image *image,ExceptionInfo *exception)
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   flop_image=CloneImage(image,image->columns,image->rows,MagickTrue,exception);
@@ -1171,7 +1242,7 @@ MagickExport Image *FlopImage(const Image *image,ExceptionInfo *exception)
 %
 %
 */
-MagickExport Image *MosaicImages(const Image *image,ExceptionInfo *exception)
+MagickExport Image *MosaicImages(Image *image,ExceptionInfo *exception)
 {
 #define MosaicImageTag  "Mosaic/Image"
 
@@ -1199,13 +1270,13 @@ MagickExport Image *MosaicImages(const Image *image,ExceptionInfo *exception)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   (void) ResetMagickMemory(&page,0,sizeof(page));
   page.width=image->columns;
   page.height=image->rows;
-  for (next=image; next != (Image *) NULL; next=next->next)
+  for (next=image; next != (Image *) NULL; next=GetNextImageInList(next))
   {
     page.x=next->page.x;
     page.y=next->page.y;
@@ -1315,7 +1386,7 @@ MagickExport Image *RollImage(const Image *image,const long x_offset,
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   roll_image=CloneImage(image,image->columns,image->rows,MagickTrue,exception);
@@ -1414,7 +1485,7 @@ MagickExport Image *ShaveImage(const Image *image,
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (((2*shave_info->width) >= image->columns) ||
       ((2*shave_info->height) >= image->rows))
     ThrowImageException(OptionError,"GeometryDoesNotContainImage");
@@ -1490,7 +1561,7 @@ MagickExport Image *SpliceImage(const Image *image,
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(geometry != (const RectangleInfo *) NULL);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
@@ -1501,7 +1572,8 @@ MagickExport Image *SpliceImage(const Image *image,
     geometry->height,MagickTrue,exception);
   if (splice_image == (Image *) NULL)
     return((Image *) NULL);
-  splice_image->storage_class=DirectClass;
+  if (SetImageStorageClass(splice_image,DirectClass) == MagickFalse)
+    return((Image *) NULL);
   /*
     Splice image.
   */
@@ -1667,12 +1739,21 @@ MagickExport MagickBooleanType TransformImage(Image **image,
       /*
         Crop image to a user specified size.
       */
-      crop_image=(Image *) NULL;
+      crop_image=NewImageList();
       flags=ParseGravityGeometry(transform_image,crop_geometry,&geometry);
       if (((geometry.width == 0) && (geometry.height == 0)) ||
           ((flags & XValue) != 0) || ((flags & YValue) != 0) ||
           ((flags & PercentValue) != 0))
-        crop_image=CropImage(transform_image,&geometry,&(*image)->exception);
+        {
+          crop_image=CropImage(transform_image,&geometry,&(*image)->exception);
+          if ((crop_image != (Image *) NULL) && ((flags & AspectValue) != 0))
+            {
+              crop_image->page.width=geometry.width;
+              crop_image->page.height=geometry.height;
+              crop_image->page.x-=geometry.x;
+              crop_image->page.y-=geometry.y;
+            }
+        }
       else
         if ((transform_image->columns > geometry.width) ||
             (transform_image->rows > geometry.height))
@@ -1844,7 +1925,9 @@ MagickExport Image *TrimImage(const Image *image,ExceptionInfo *exception)
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   geometry=GetImageBoundingBox(image,exception);
+  geometry.x+=image->page.x;
+  geometry.y+=image->page.y;
   return(CropImage(image,&geometry,exception));
 }
