@@ -17,7 +17,7 @@
 %                                 May 2001                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -49,7 +49,9 @@
 #include "magick/hashmap.h"
 #include "magick/log.h"
 #include "magick/memory_.h"
-#include "magick/nt-feature.h"
+#if defined(__WINDOWS__)
+# include "magick/nt-feature.h"
+#endif
 #include "magick/option.h"
 #include "magick/semaphore.h"
 #include "magick/splay-tree.h"
@@ -67,7 +69,7 @@
   Declare type map.
 */
 static const char
-  *TypeMap = (char *)
+  *TypeMap = (const char *)
     "<?xml version=\"1.0\"?>"
     "<typemap>"
     "  <type stealth=\"True\" />"
@@ -113,7 +115,7 @@ static MagickBooleanType
 */
 MagickExport void DestroyTypeList(void)
 {
-  type_semaphore=AcquireSemaphoreInfo(type_semaphore);
+  AcquireSemaphoreInfo(&type_semaphore);
   if (type_list != (SplayTreeInfo *) NULL)
     type_list=DestroySplayTree(type_list);
   instantiate_type=MagickFalse;
@@ -169,7 +171,7 @@ MagickExport const TypeInfo *GetTypeInfo(const char *name,
   /*
     Search for requested type.
   */
-  type_semaphore=AcquireSemaphoreInfo(type_semaphore);
+  AcquireSemaphoreInfo(&type_semaphore);
   ResetSplayTreeIterator(type_list);
   p=(const TypeInfo *) GetNextValueInSplayTree(type_list);
   while (p != (const TypeInfo *) NULL)
@@ -224,7 +226,7 @@ MagickExport const TypeInfo *GetTypeInfoByFamily(const char *family,
 {
   typedef struct _Fontmap
   {
-    char
+    const char
       *name,
       *substitute;
   } Fontmap;
@@ -244,14 +246,14 @@ MagickExport const TypeInfo *GetTypeInfoByFamily(const char *family,
   static Fontmap
     fontmap[] =
     {
-      { (char *) "fixed", (char *) "courier" },
-      { (char *) "modern",(char *) "courier" },
-      { (char *) "monotype corsiva", (char *) "courier" },
-      { (char *) "news gothic", (char *) "helvetica" },
-      { (char *) "system", (char *) "courier" },
-      { (char *) "terminal", (char *) "courier" },
-      { (char *) "wingdings", (char *) "symbol" },
-      { (char *) NULL, (char *) NULL }
+      { "fixed", "courier" },
+      { "modern","courier" },
+      { "monotype corsiva", "courier" },
+      { "news gothic", "helvetica" },
+      { "system", "courier" },
+      { "terminal", "courier" },
+      { "wingdings", "symbol" },
+      { NULL, NULL }
     };
 
   unsigned long
@@ -264,7 +266,7 @@ MagickExport const TypeInfo *GetTypeInfoByFamily(const char *family,
   (void) GetTypeInfo("*",exception);
   if (type_list == (SplayTreeInfo *) NULL)
     return((TypeInfo *) NULL);
-  type_semaphore=AcquireSemaphoreInfo(type_semaphore);
+  AcquireSemaphoreInfo(&type_semaphore);
   ResetSplayTreeIterator(type_list);
   type_info=(const TypeInfo *) NULL;
   p=(const TypeInfo *) GetNextValueInSplayTree(type_list);
@@ -316,7 +318,7 @@ MagickExport const TypeInfo *GetTypeInfoByFamily(const char *family,
     Check for types in the same family.
   */
   max_score=0;
-  type_semaphore=AcquireSemaphoreInfo(type_semaphore);
+  AcquireSemaphoreInfo(&type_semaphore);
   ResetSplayTreeIterator(type_list);
   p=(const TypeInfo *) GetNextValueInSplayTree(type_list);
   while (p != (const TypeInfo *) NULL)
@@ -479,7 +481,7 @@ MagickExport const TypeInfo **GetTypeInfoList(const char *pattern,
   /*
     Generate type list.
   */
-  type_semaphore=AcquireSemaphoreInfo(type_semaphore);
+  AcquireSemaphoreInfo(&type_semaphore);
   ResetSplayTreeIterator(type_list);
   p=(const TypeInfo *) GetNextValueInSplayTree(type_list);
   for (i=0; p != (const TypeInfo *) NULL; )
@@ -530,12 +532,12 @@ extern "C" {
 
 static int TypeCompare(const void *x,const void *y)
 {
-  register char
+  register const char
     **p,
     **q;
 
-  p=(char **) x;
-  q=(char **) y;
+  p=(const char **) x;
+  q=(const char **) y;
   return(LocaleCompare(*p,*q));
 }
 
@@ -572,14 +574,14 @@ MagickExport char **GetTypeList(const char *pattern,unsigned long *number_fonts,
   /*
     Generate type list.
   */
-  type_semaphore=AcquireSemaphoreInfo(type_semaphore);
+  AcquireSemaphoreInfo(&type_semaphore);
   ResetSplayTreeIterator(type_list);
   p=(const TypeInfo *) GetNextValueInSplayTree(type_list);
   for (i=0; p != (const TypeInfo *) NULL; )
   {
     if ((p->stealth == MagickFalse) &&
         (GlobExpression(p->name,pattern) != MagickFalse))
-      fonts[i++]=ConstantString(AcquireString(p->name));
+      fonts[i++]=ConstantString(p->name);
     p=(const TypeInfo *) GetNextValueInSplayTree(type_list);
   }
   RelinquishSemaphoreInfo(type_semaphore);
@@ -616,7 +618,7 @@ static MagickBooleanType InitializeTypeList(ExceptionInfo *exception)
   if ((type_list == (SplayTreeInfo *) NULL) &&
       (instantiate_type == MagickFalse))
     {
-      type_semaphore=AcquireSemaphoreInfo(type_semaphore);
+      AcquireSemaphoreInfo(&type_semaphore);
       if ((type_list == (SplayTreeInfo *) NULL) &&
           (instantiate_type == MagickFalse))
         {
@@ -812,7 +814,7 @@ static MagickBooleanType LoadTypeList(const char *xml,const char *filename,
         }
     }
   status=MagickTrue;
-  token=AcquireString((char *) xml);
+  token=AcquireString(xml);
 #if defined(__WINDOWS__)
   /*
     Determine the Ghostscript font path.
@@ -830,6 +832,15 @@ static MagickBooleanType LoadTypeList(const char *xml,const char *filename,
     if (*token == '\0')
       break;
     (void) CopyMagickString(keyword,token,MaxTextExtent);
+    if (LocaleNCompare(keyword,"<!DOCTYPE",9) == 0)
+      {
+        /*
+          Doctype element.
+        */
+        while ((LocaleNCompare(q,"]>",2) != 0) && (*q != '\0'))
+          GetMagickToken(q,&q,token);
+        continue;
+      }
     if (LocaleNCompare(keyword,"<!--",4) == 0)
       {
         /*
@@ -868,7 +879,7 @@ static MagickBooleanType LoadTypeList(const char *xml,const char *filename,
                     (void) ConcatenateMagickString(path,DirectorySeparator,
                       MaxTextExtent);
                   (void) ConcatenateMagickString(path,token,MaxTextExtent);
-                  xml=FileToString(path);
+                  xml=FileToString(path,~0,exception);
                   status|=LoadTypeList(xml,path,depth+1,exception);
                   xml=(char *) RelinquishMagickMemory(xml);
                 }
@@ -886,7 +897,7 @@ static MagickBooleanType LoadTypeList(const char *xml,const char *filename,
           ThrowMagickFatalException(ResourceLimitFatalError,
             "MemoryAllocationFailed",filename);
         (void) ResetMagickMemory(type_info,0,sizeof(*type_info));
-        type_info->path=ConstantString(AcquireString(filename));
+        type_info->path=ConstantString(filename);
         type_info->signature=MagickSignature;
         continue;
       }
@@ -894,8 +905,8 @@ static MagickBooleanType LoadTypeList(const char *xml,const char *filename,
       continue;
     if (LocaleCompare(keyword,"/>") == 0)
       {
-        status=AddValueToSplayTree(type_list,
-          ConstantString(AcquireString(type_info->name)),type_info);
+        status=AddValueToSplayTree(type_list,ConstantString(type_info->name),
+          type_info);
         if (status == MagickFalse)
           (void) ThrowMagickException(exception,GetMagickModule(),
             ResourceLimitError,"MemoryAllocationFailed","`%s'",type_info->name);
@@ -913,7 +924,7 @@ static MagickBooleanType LoadTypeList(const char *xml,const char *filename,
       {
         if (LocaleCompare((char *) keyword,"encoding") == 0)
           {
-            type_info->encoding=ConstantString(AcquireString(token));
+            type_info->encoding=ConstantString(token);
             break;
           }
         break;
@@ -928,22 +939,22 @@ static MagickBooleanType LoadTypeList(const char *xml,const char *filename,
           }
         if (LocaleCompare((char *) keyword,"family") == 0)
           {
-            type_info->family=ConstantString(AcquireString(token));
+            type_info->family=ConstantString(token);
             break;
           }
         if (LocaleCompare((char *) keyword,"format") == 0)
           {
-            type_info->format=ConstantString(AcquireString(token));
+            type_info->format=ConstantString(token);
             break;
           }
         if (LocaleCompare((char *) keyword,"foundry") == 0)
           {
-            type_info->foundry=ConstantString(AcquireString(token));
+            type_info->foundry=ConstantString(token);
             break;
           }
         if (LocaleCompare((char *) keyword,"fullname") == 0)
           {
-            type_info->description=ConstantString(AcquireString(token));
+            type_info->description=ConstantString(token);
             break;
           }
         break;
@@ -959,9 +970,9 @@ static MagickBooleanType LoadTypeList(const char *xml,const char *filename,
             glyphs=AcquireString(token);
 #if defined(__WINDOWS__)
             if (strchr(glyphs,'@') != (char *) NULL)
-              SubstituteString(&glyphs,"@ghostscript_font_dir@",font_path);
+              SubstituteString(&glyphs,"@ghostscript_font_path@",font_path);
 #endif
-            type_info->glyphs=ConstantString(glyphs);
+            type_info->glyphs=glyphs;
             break;
           }
         break;
@@ -977,9 +988,9 @@ static MagickBooleanType LoadTypeList(const char *xml,const char *filename,
             metrics=AcquireString(token);
 #if defined(__WINDOWS__)
             if (strchr(metrics,'@') != (char *) NULL)
-              SubstituteString(&metrics,"@ghostscript_font_dir@",font_path);
+              SubstituteString(&metrics,"@ghostscript_font_path@",font_path);
 #endif
-            type_info->metrics=ConstantString(metrics);
+            type_info->metrics=metrics;
             break;
           }
         break;
@@ -989,7 +1000,7 @@ static MagickBooleanType LoadTypeList(const char *xml,const char *filename,
       {
         if (LocaleCompare((char *) keyword,"name") == 0)
           {
-            type_info->name=ConstantString(AcquireString(token));
+            type_info->name=ConstantString(token);
             break;
           }
         break;
@@ -999,8 +1010,8 @@ static MagickBooleanType LoadTypeList(const char *xml,const char *filename,
       {
         if (LocaleCompare((char *) keyword,"stealth") == 0)
           {
-            type_info->stealth=(MagickBooleanType)
-              (LocaleCompare(token,"True") == 0);
+            type_info->stealth=LocaleCompare(token,"True") == 0 ? MagickTrue :
+              MagickFalse;
             break;
           }
         if (LocaleCompare((char *) keyword,"stretch") == 0)
@@ -1053,7 +1064,7 @@ static MagickBooleanType LoadTypeList(const char *xml,const char *filename,
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  LoadTypeList() loads one or more type configuration files which provides a
-%  mapping between type attributes a type name.
+%  mapping between type attributes and a type name.
 %
 %  The format of the LoadTypeLists method is:
 %
@@ -1071,11 +1082,11 @@ static MagickBooleanType LoadTypeList(const char *xml,const char *filename,
 static MagickBooleanType LoadTypeLists(const char *filename,
   ExceptionInfo *exception)
 {
+#if defined(UseEmbeddableMagick)
+  return(LoadTypeList(TypeMap,"built-in",0,exception));
+#else
   char
-    path[MaxTextExtent];
-
-  const char
-    *p;
+    *font_path;
 
   const StringInfo
     *option;
@@ -1087,46 +1098,37 @@ static MagickBooleanType LoadTypeLists(const char *filename,
     status;
 
   status=MagickFalse;
-  *path='\0';
   options=GetConfigureOptions(filename,exception);
   option=(const StringInfo *) GetNextValueInLinkedList(options);
   while (option != (const StringInfo *) NULL)
   {
-    (void) CopyMagickString(path,option->path,MaxTextExtent);
     status|=LoadTypeList((const char *) option->datum,option->path,0,exception);
     option=(const StringInfo *) GetNextValueInLinkedList(options);
   }
   options=DestroyConfigureOptions(options);
-  p=getenv("MAGICK_FONT_PATH");
-  if (p != (const char *) NULL)
+  font_path=GetEnvironmentValue("MAGICK_FONT_PATH");
+  if (font_path != (char *) NULL)
     {
+      char
+        path[MaxTextExtent],
+        *option;
+
       /*
         Search MAGICK_FONT_PATH.
       */
-      (void) FormatMagickString(path,MaxTextExtent,"%s%s%s",p,
+      (void) FormatMagickString(path,MaxTextExtent,"%s%s%s",font_path,
         DirectorySeparator,filename);
-      if (IsAccessible(path) != MagickFalse)
+      option=FileToString(path,~0,exception);
+      if (option != (void *) NULL)
         {
-          char
-            *option;
-
-          size_t
-            length;
-
-          option=(char *) FileToBlob(path,&length,exception);
-          if (option != (void *) NULL)
-            {
-              status|=LoadTypeList(option,path,0,exception);
-              option=(char *) RelinquishMagickMemory(option);
-            }
+          status|=LoadTypeList(option,path,0,exception);
+          option=(char *) RelinquishMagickMemory(option);
         }
+      font_path=(char *) RelinquishMagickMemory(font_path);
     }
   if ((type_list == (SplayTreeInfo *) NULL) || 
       (GetNumberOfNodesInSplayTree(type_list) == 0))
-    {
-      (void) ThrowMagickException(exception,GetMagickModule(),ConfigureWarning,
-        "UnableToOpenConfigureFile","`%s'",path);
-      status|=LoadTypeList(TypeMap,"built-in",0,exception);
-    }
+    status|=LoadTypeList(TypeMap,"built-in",0,exception);
   return(status != 0 ? MagickTrue : MagickFalse);
+#endif
 }
