@@ -17,7 +17,7 @@
 %                              January 1993                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -109,6 +109,116 @@ MagickExport MagickBooleanType AcquireUniqueFilename(char *path)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   A c q u i r e U n i q u e S ym b o l i c L i n k                          %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  AcquireUniqueSymbolicLink() creates a unique symbolic link to the specified
+%  source path and returns MagickTrue on success otherwise MagickFalse.  If the
+%  symlink() method fails or is not available, a unique file name is generated
+%  and the source file copied to it.  When you are finished with the file, use
+%  RelinquishUniqueFilename() to destroy it.
+%
+%  The format of the AcquireUniqueSymbolicLink method is:
+%
+%      MagickBooleanType AcquireUniqueSymbolicLink(const char *source,
+%        char destination)
+%
+%  A description of each parameter follows.
+%
+%   o  source:  the source path.
+%
+%   o  destination:  the destination path.
+%
+*/
+MagickExport MagickBooleanType AcquireUniqueSymbolicLink(const char *source,
+  char *destination)
+{
+  int
+    destination_file,
+    source_file;
+
+  size_t
+    length;
+
+  ssize_t
+    count;
+
+  unsigned char
+    *buffer;
+
+  assert(source != (const char *) NULL);
+  assert(destination != (char *) NULL);
+#if defined(HAVE_SYMLINK)
+  AcquireUniqueFilename(destination);
+  (void) RelinquishUniqueFileResource(destination);
+  if (*source == *DirectorySeparator)
+    {
+      if (symlink(source,destination) == 0)
+        return(MagickTrue);
+    }
+  else
+    {
+      char
+        path[MaxTextExtent];
+
+      *path='\0';
+      if (getcwd(path,MaxTextExtent) == (char *) NULL)
+        return(MagickFalse);
+      (void) ConcatenateMagickString(path,DirectorySeparator,MaxTextExtent);
+      (void) ConcatenateMagickString(path,source,MaxTextExtent);
+      if (symlink(path,destination) == 0)
+        return(MagickTrue);
+    }
+#endif
+  destination_file=AcquireUniqueFileResource(destination);
+  if (destination_file == -1)
+    return(MagickFalse);
+  source_file=open(source,O_RDONLY | O_BINARY);
+  if (source_file == -1)
+    {
+      (void) close(destination_file);
+      (void) RelinquishUniqueFileResource(destination);
+      return(MagickFalse);
+    }
+  buffer=(unsigned char *) AcquireMagickMemory(MagickMaxBufferSize*
+    sizeof(*buffer));
+  if (buffer == (unsigned char *) NULL)
+    {
+      (void) close(source_file);
+      (void) close(destination_file);
+      (void) RelinquishUniqueFileResource(destination);
+      return(MagickFalse);
+    }
+  for (length=0; ; )
+  {
+    count=read(source_file,buffer,MagickMaxBufferSize);
+    if (count <= 0)
+      break;
+    length=(size_t) count;
+    count=write(destination_file,buffer,length);
+    if ((size_t) count != length)
+      {
+        (void) close(destination_file);
+        (void) close(source_file);
+        buffer=(unsigned char *) RelinquishMagickMemory(buffer);
+        (void) RelinquishUniqueFileResource(destination);
+        return(MagickFalse);
+      }
+  }
+  (void) close(destination_file);
+  (void) close(source_file);
+  buffer=(unsigned char *) RelinquishMagickMemory(buffer);
+  return(MagickTrue);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %  A p p e n d I m a g e F o r m a t                                          %
 %                                                                             %
 %                                                                             %
@@ -465,9 +575,6 @@ MagickExport void ExpandFilename(char *path)
   char
     expanded_path[MaxTextExtent];
 
-  register char
-    *p;
-
   if (path == (char *) NULL)
     return;
   if (*path != '~')
@@ -475,14 +582,23 @@ MagickExport void ExpandFilename(char *path)
   (void) CopyMagickString(expanded_path,path,MaxTextExtent);
   if ((*(path+1) == *DirectorySeparator) || (*(path+1) == '\0'))
     {
+      char
+        *home;
+
       /*
         Substitute ~ with $HOME.
       */
-      p=(char *) getenv("HOME");
-      if (p == (char *) NULL)
-        p=(char *) ".";
-      (void) CopyMagickString(expanded_path,p,MaxTextExtent);
+      (void) CopyMagickString(expanded_path,".",MaxTextExtent);
       (void) ConcatenateMagickString(expanded_path,path+1,MaxTextExtent);
+      home=GetEnvironmentValue("HOME");
+      if (home == (char *) NULL)
+        home=GetEnvironmentValue("USERPROFILE");
+      if (home != (char *) NULL)
+        {
+          (void) CopyMagickString(expanded_path,home,MaxTextExtent);
+          (void) ConcatenateMagickString(expanded_path,path+1,MaxTextExtent);
+          home=(char *) RelinquishMagickMemory(home);
+        }
     }
   else
     {
@@ -490,6 +606,9 @@ MagickExport void ExpandFilename(char *path)
       char
         username[MaxTextExtent];
 
+      register char
+        *p;
+
       struct passwd
         *entry;
 
@@ -582,12 +701,13 @@ MagickExport MagickBooleanType ExpandFilenames(int *argc,char ***argv)
   /*
     Expand any wildcard filenames.
   */
-  (void) getcwd(home_directory,MaxTextExtent);
+  if (getcwd(home_directory,MaxTextExtent) == (char *) NULL)
+    return(MagickFalse);
   count=0;
   for (i=0; i < (long) *argc; i++)
   {
     option=(*argv)[i];
-    vector[count++]=AcquireString(option);
+    vector[count++]=ConstantString(option);
     parameters=ParseMagickOption(MagickCommandOptions,MagickFalse,option);
     if (parameters > 0)
       {
@@ -600,7 +720,7 @@ MagickExport MagickBooleanType ExpandFilenames(int *argc,char ***argv)
           if (i == (long) *argc)
             break;
           option=(*argv)[i];
-          vector[count++]=AcquireString(option);
+          vector[count++]=ConstantString(option);
         }
         continue;
       }
@@ -663,7 +783,7 @@ MagickExport MagickBooleanType ExpandFilenames(int *argc,char ***argv)
               (void) ConcatenateMagickString(path,subimage,MaxTextExtent);
               (void) ConcatenateMagickString(path,"]",MaxTextExtent);
             }
-          vector[count++]=AcquireString(path);
+          vector[count++]=ConstantString(path);
         }
     }
     filelist=(char **) RelinquishMagickMemory(filelist);
@@ -709,7 +829,7 @@ MagickExport void FormatSize(const MagickSizeType size,char *format)
   register long
     i;
 
-  length=(double) ((MagickOffsetType) size);
+  length=(double) size;
   for (i=0; length > 1024.0; i++)
     length/=1024.0;
   switch (i)
@@ -721,32 +841,32 @@ MagickExport void FormatSize(const MagickSizeType size,char *format)
     }
     case 1:
     {
-      (void) FormatMagickString(format,MaxTextExtent,"%.0fkb",length);
+      (void) FormatMagickString(format,MaxTextExtent,"%.0gkb",length);
       break;
     }
     case 2:
     {
-      (void) FormatMagickString(format,MaxTextExtent,"%.1fmb",length);
+      (void) FormatMagickString(format,MaxTextExtent,"%.1gmb",length);
       break;
     }
     case 3:
     {
-      (void) FormatMagickString(format,MaxTextExtent,"%.2fgb",length);
+      (void) FormatMagickString(format,MaxTextExtent,"%.2ggb",length);
       break;
     }
     case 4:
     {
-      (void) FormatMagickString(format,MaxTextExtent,"%.2ftb",length);
+      (void) FormatMagickString(format,MaxTextExtent,"%.3gtb",length);
       break;
     }
     case 5:
     {
-      (void) FormatMagickString(format,MaxTextExtent,"%.2fpb",length);
+      (void) FormatMagickString(format,MaxTextExtent,"%.4gpb",length);
       break;
     }
     default:
     {
-      (void) FormatMagickString(format,MaxTextExtent,"%.3feb",length);
+      (void) FormatMagickString(format,MaxTextExtent,"%.5feb",length);
       break;
     }
   }
@@ -778,7 +898,8 @@ MagickExport void FormatSize(const MagickSizeType size,char *format)
 MagickExport MagickBooleanType GetExecutionPath(char *path)
 {
   *path='\0';
-  (void) getcwd(path,MaxTextExtent);
+  if (getcwd(path,MaxTextExtent) == (char *) NULL)
+    return(MagickFalse);
 #if defined(__WINDOWS__)
   return(NTGetExecutionPath(path));
 #endif
@@ -791,7 +912,7 @@ MagickExport MagickBooleanType GetExecutionPath(char *path)
     if (execution_path != (const char *) NULL)
       {
         if (*execution_path != *DirectorySeparator)
-          (void) ConcatenateMagickString(path,"/",MaxTextExtent);
+          (void) ConcatenateMagickString(path,DirectorySeparator,MaxTextExtent);
         (void) ConcatenateMagickString(path,execution_path,MaxTextExtent);
       }
   }
@@ -889,7 +1010,7 @@ MagickExport void GetPathComponent(const char *path,PathType type,
   p=component;
   if (*p != '\0')
     for (p=component+strlen(component)-1; p > component; p--)
-      if (IsBasenameSeparator(*p))
+      if (IsBasenameSeparator(*p) != MagickFalse)
         break;
   switch (type)
   {
@@ -902,7 +1023,7 @@ MagickExport void GetPathComponent(const char *path,PathType type,
     {
       for (p=component+(strlen(component)-1); p > component; p--)
       {
-        if (IsBasenameSeparator(*p))
+        if (IsBasenameSeparator(*p) != MagickFalse)
           break;
         if (*p == '.')
           break;
@@ -918,14 +1039,14 @@ MagickExport void GetPathComponent(const char *path,PathType type,
     }
     case TailPath:
     {
-      if (IsBasenameSeparator(*p))
+      if (IsBasenameSeparator(*p) != MagickFalse)
         (void) CopyMagickMemory((unsigned char *) component,
           (const unsigned char *) (p+1),strlen(p+1)+1);
       break;
     }
     case BasePath:
     {
-      if (IsBasenameSeparator(*p))
+      if (IsBasenameSeparator(*p) != MagickFalse)
         (void) CopyMagickString(component,p+1,MaxTextExtent);
       for (p=component+(strlen(component)-1); p > component; p--)
         if (*p == '.')
@@ -937,7 +1058,7 @@ MagickExport void GetPathComponent(const char *path,PathType type,
     }
     case ExtensionPath:
     {
-      if (IsBasenameSeparator(*p))
+      if (IsBasenameSeparator(*p) != MagickFalse)
         (void) CopyMagickString(component,p+1,MaxTextExtent);
       p=component;
       if (*p != '\0')
@@ -955,7 +1076,7 @@ MagickExport void GetPathComponent(const char *path,PathType type,
       break;
     }
     case CanonicalPath:
-    default:
+    case UndefinedPath:
       break;
   }
 }
@@ -1094,12 +1215,12 @@ extern "C" {
 
 static int FileCompare(const void *x,const void *y)
 {
-  register char
+  register const char
     **p,
     **q;
 
-  p=(char **) x;
-  q=(char **) y;
+  p=(const char **) x;
+  q=(const char **) y;
   return(LocaleCompare(*p,*q));
 }
 
@@ -1130,7 +1251,7 @@ MagickExport char **ListFiles(const char *directory,const char *pattern,
   assert(pattern != (const char *) NULL);
   assert(number_entries != (unsigned long *) NULL);
   *number_entries=0;
-  current_directory=opendir((char *) directory);
+  current_directory=opendir(directory);
   if (current_directory == (DIR *) NULL)
     return((char **) NULL);
   /*
@@ -1279,7 +1400,7 @@ MagickExport int SystemCommand(const MagickBooleanType verbose,
     status;
 
   if (verbose != MagickFalse)
-    (void) fputs(command,stdout);
+    (void) fprintf(stdout,"%s\n",command);
 #if defined(POSIX)
   status=system(command);
 #elif defined(__WINDOWS__)
@@ -1293,429 +1414,19 @@ MagickExport int SystemCommand(const MagickBooleanType verbose,
 #endif
   if (status < 0)
     {
+      char
+        *message;
+
       ExceptionInfo
         exception;
 
       GetExceptionInfo(&exception);
+      message=GetExceptionMessage(errno);
       (void) ThrowMagickException(&exception,GetMagickModule(),DelegateError,
-        "`%s': %s",command,strerror(errno));
+        "`%s': %s",command,message);
+      message=(char *) RelinquishMagickMemory(message);
       CatchException(&exception);
-      DestroyExceptionInfo(&exception);
+      (void) DestroyExceptionInfo(&exception);
     }
   return(status);
 }
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%   T r a n s l a t e T e x t                                                 %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  TranslateText() replaces any embedded formatting characters with the
-%  appropriate image attribute and returns the translated text.
-%
-%  The format of the TranslateText method is:
-%
-%      char *TranslateText(const ImageInfo *image_info,Image *image,
-%        const char *embed_text)
-%
-%  A description of each parameter follows:
-%
-%    o image_info: The image info.
-%
-%    o image: The image.
-%
-%    o embed_text: The address of a character string containing the embedded
-%      formatting characters.
-%
-%
-*/
-MagickExport char *TranslateText(const ImageInfo *image_info,Image *image,
-  const char *embed_text)
-{
-  char
-    filename[MaxTextExtent],
-    *text,
-    *translate_text;
-
-  const ImageAttribute
-    *attribute;
-
-  ImageInfo
-    *text_info;
-
-  register char
-    *p,
-    *q;
-
-  register long
-    i;
-
-  size_t
-    length;
-
-  assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
-  if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  if ((embed_text == (const char *) NULL) || (*embed_text == '\0'))
-    return((char *) NULL);
-  text=(char *) embed_text;
-  if ((*text == '@') && (IsAccessible(text+1) != MagickFalse))
-    {
-      text=(char *) FileToBlob(text+1,&length,&image->exception);
-      if (text == (char *) NULL)
-        return((char *) NULL);
-    }
-  /*
-    Translate any embedded format characters.
-  */
-  text_info=CloneImageInfo(image_info);
-  translate_text=AcquireString(text);
-  length=strlen(text)+MaxTextExtent;
-  p=text;
-  for (q=translate_text; *p != '\0'; p++)
-  {
-    *q='\0';
-    if ((size_t) (q-translate_text+MaxTextExtent) >= length)
-      {
-        length<<=1;
-        translate_text=(char *) ResizeMagickMemory(translate_text,
-          (length+MaxTextExtent)*sizeof(*translate_text));
-        if (translate_text == (char *) NULL)
-          break;
-        q=translate_text+strlen(translate_text);
-      }
-    /*
-      Process formatting characters in text.
-    */
-    if ((*p == '\\') && (*(p+1) == 'r'))
-      {
-        *q++='\r';
-        p++;
-        continue;
-      }
-    if ((*p == '\\') && (*(p+1) == 'n'))
-      {
-        *q++='\n';
-        p++;
-        continue;
-      }
-    if (*p != '%')
-      {
-        *q++=(*p);
-        continue;
-      }
-    p++;
-    switch (*p)
-    {
-      case 'b':
-      {
-        char
-          format[MaxTextExtent];
-
-        MagickSizeType
-          length;
-
-        length=GetBlobSize(image);
-        (void) FormatMagickString(format,MaxTextExtent,"%lu",
-          (unsigned long) length);
-        if (length != (MagickSizeType) ((size_t) length))
-          FormatSize(length,format);
-        (void) ConcatenateMagickString(q,format,MaxTextExtent);
-        while (*q != '\0')
-          q++;
-        break;
-      }
-      case 'c':
-      {
-        attribute=GetImageAttribute(image,"Comment");
-        if (attribute == (ImageAttribute *) NULL)
-          break;
-        (void) CopyMagickString(q,attribute->value,MaxTextExtent);
-        q+=strlen(attribute->value);
-        break;
-      }
-      case 'd':
-      case 'e':
-      case 'f':
-      case 't':
-      {
-        /*
-          Label segment is the base of the filename.
-        */
-        if (strlen(image->magick_filename) == 0)
-          break;
-        switch (*p)
-        {
-          case 'd':
-          {
-            GetPathComponent(image->magick_filename,HeadPath,filename);
-            (void) CopyMagickString(q,filename,MaxTextExtent);
-            q+=strlen(filename);
-            break;
-          }
-          case 'e':
-          {
-            GetPathComponent(image->magick_filename,ExtensionPath,filename);
-            (void) CopyMagickString(q,filename,MaxTextExtent);
-            q+=strlen(filename);
-            break;
-          }
-          case 'f':
-          {
-            GetPathComponent(image->magick_filename,TailPath,filename);
-            (void) CopyMagickString(q,filename,MaxTextExtent);
-            q+=strlen(filename);
-            break;
-          }
-          case 't':
-          {
-            GetPathComponent(image->magick_filename,BasePath,filename);
-            (void) CopyMagickString(q,filename,MaxTextExtent);
-            q+=strlen(filename);
-            break;
-          }
-        }
-        break;
-      }
-      case 'g':
-      {
-        (void) FormatMagickString(q,MaxTextExtent,"%lux%lu%+ld%+ld",
-          image->page.width,image->page.height,image->page.x,image->page.y);
-        while (*q != '\0')
-          q++;
-        break;
-      }
-      case 'h':
-      {
-        (void) FormatMagickString(q,MaxTextExtent,"%lu",
-          image->magick_rows != 0 ? image->magick_rows : image->rows);
-        while (*q != '\0')
-          q++;
-        break;
-      }
-      case 'i':
-      {
-        (void) CopyMagickString(q,image->filename,MaxTextExtent);
-        q+=strlen(image->filename);
-        break;
-      }
-      case 'k':
-      {
-        (void) FormatMagickString(q,MaxTextExtent,"%lu",
-          GetNumberColors(image,(FILE *) NULL,&image->exception));
-        while (*q != '\0')
-          q++;
-        break;
-      }
-      case 'l':
-      {
-        attribute=GetImageAttribute(image,"label");
-        if (attribute == (ImageAttribute *) NULL)
-          break;
-        (void) CopyMagickString(q,attribute->value,MaxTextExtent);
-        q+=strlen(attribute->value);
-        break;
-      }
-      case 'm':
-      {
-        (void) CopyMagickString(q,image->magick,MaxTextExtent);
-        q+=strlen(image->magick);
-        break;
-      }
-      case 'n':
-      {
-        (void) FormatMagickString(q,MaxTextExtent,"%lu",(unsigned long)
-          GetImageListLength(image));
-        while (*q != '\0')
-          q++;
-        break;
-      }
-      case 'o':
-      {
-        (void) CopyMagickString(q,text_info->filename,MaxTextExtent);
-        q+=strlen(text_info->filename);
-        break;
-      }
-      case 'p':
-      {
-        register const Image
-          *p;
-
-        unsigned long
-          page;
-
-        p=image;
-        for (page=1; GetPreviousImageInList(p) != (Image *) NULL; page++)
-          p=GetPreviousImageInList(p);
-        (void) FormatMagickString(q,MaxTextExtent,"%lu",page);
-        while (*q != '\0')
-          q++;
-        break;
-      }
-      case 'q':
-      {
-        (void) FormatMagickString(q,MaxTextExtent,"%lu",image->depth);
-        while (*q != '\0')
-          q++;
-        break;
-      }
-      case 'r':
-      {
-        ColorspaceType
-          colorspace;
-
-        colorspace=image->colorspace;
-        if (IsGrayImage(image,&image->exception) != MagickFalse)
-          colorspace=GRAYColorspace;
-        (void) FormatMagickString(q,MaxTextExtent,"%s%s%s",
-          MagickOptionToMnemonic(MagickClassOptions,(long) image->storage_class),
-          MagickOptionToMnemonic(MagickColorspaceOptions,(long) colorspace),
-          image->matte != MagickFalse ? "Matte" : "");
-        while (*q != '\0')
-          q++;
-        break;
-      }
-      case 's':
-      {
-        (void) FormatMagickString(q,MaxTextExtent,"%lu",image->scene);
-        if (text_info->number_scenes != 0)
-          (void) FormatMagickString(q,MaxTextExtent,"%lu",text_info->scene);
-        while (*q != '\0')
-          q++;
-        break;
-      }
-      case 'u':
-      {
-        (void) CopyMagickString(filename,text_info->unique,MaxTextExtent);
-        (void) CopyMagickString(q,filename,MaxTextExtent);
-        q+=strlen(filename);
-        break;
-      }
-      case 'w':
-      {
-        (void) FormatMagickString(q,MaxTextExtent,"%lu",
-          image->magick_columns != 0 ? image->magick_columns : image->columns);
-        while (*q != '\0')
-          q++;
-        break;
-      }
-      case 'x':
-      {
-        (void) FormatMagickString(q,MaxTextExtent,"%g %s",image->x_resolution,
-          MagickOptionToMnemonic(MagickResolutionOptions,(long) image->units));
-        while (*q != '\0')
-          q++;
-        break;
-      }
-      case 'y':
-      {
-        (void) FormatMagickString(q,MaxTextExtent,"%g %s",image->y_resolution,
-          MagickOptionToMnemonic(MagickResolutionOptions,(long) image->units));
-        while (*q != '\0')
-          q++;
-        break;
-      }
-      case 'z':
-      {
-        (void) FormatMagickString(q,MaxTextExtent,"%lu",
-          GetImageDepth(image,&image->exception));
-        while (*q != '\0')
-          q++;
-        break;
-      }
-      case '[':
-      {
-        char
-          key[MaxTextExtent];
-
-        MagickOffsetType
-          offset;
-
-        if (strchr(p,']') == (char *) NULL)
-          break;
-        p++;
-        for (i=0; (i < (long) MaxTextExtent) && (*p != ']'); i++)
-          key[i]=(*p++);
-        key[i]='\0';
-        attribute=GetImageAttribute(image,key);
-        if (attribute != (const ImageAttribute *) NULL)
-          {
-            offset=(MagickOffsetType) strlen(attribute->value);
-            if ((size_t) (q-translate_text+offset+1) >= length)
-              {
-                length+=offset;
-                translate_text=(char *) ResizeMagickMemory(translate_text,
-                  (length+MaxTextExtent)*sizeof(*translate_text));
-                if (translate_text == (char *) NULL)
-                  break;
-                q=translate_text+strlen(translate_text);
-              }
-            (void) strcpy(q,attribute->value);
-            q+=offset;
-            break;
-          }
-        attribute=GetImageInfoAttribute(text_info,image,key);
-        if (attribute == (ImageAttribute *) NULL)
-          break;
-        offset=(MagickOffsetType) strlen(attribute->value);
-        if ((size_t) (q-translate_text+offset+1) >= length)
-          {
-            length+=offset;
-            translate_text=(char *) ResizeMagickMemory(translate_text,
-              (length+MaxTextExtent)*sizeof(*translate_text));
-            if (translate_text == (char *) NULL)
-              break;
-            q=translate_text+strlen(translate_text);
-          }
-        (void) strcpy(q,attribute->value);
-        q+=offset;
-        break;
-      }
-      case '@':
-      {
-        RectangleInfo
-          page;
-
-        page=GetImageBoundingBox(image,&image->exception);
-        (void) FormatMagickString(q,MaxTextExtent,"%lux%lu%+ld%+ld",
-          page.width,page.height,page.x,page.y);
-        while (*q != '\0')
-          q++;
-        break;
-      }
-      case '#':
-      {
-        (void) SignatureImage(image);
-        attribute=GetImageAttribute(image,"Signature");
-        if (attribute == (ImageAttribute *) NULL)
-          break;
-        (void) CopyMagickString(q,attribute->value,MaxTextExtent);
-        q+=strlen(attribute->value);
-        break;
-      }
-      case '%':
-      {
-        *q++=(*p);
-        break;
-      }
-      default:
-      {
-        *q++='%';
-        *q++=(*p);
-        break;
-      }
-    }
-  }
-  *q='\0';
-  text_info=DestroyImageInfo(text_info);
-  if (text != (char *) embed_text)
-    text=(char *) RelinquishMagickMemory(text);
-  return(translate_text);
-}
