@@ -17,7 +17,7 @@
 %                              July 1992                                      %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -47,17 +47,14 @@
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/client.h"
-#include "magick/exception.h"
-#include "magick/import.h"
-#include "magick/image.h"
-#include "magick/log.h"
-#include "magick/magick.h"
-#include "magick/memory_.h"
-#include "magick/string_.h"
-#include "magick/utility.h"
-#include "magick/version.h"
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <time.h>
+#include "wand/MagickWand.h"
+#if defined(__WINDOWS__)
+#include <windows.h>
+#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -98,21 +95,23 @@ int main(int argc,char **argv)
       continue;
     if ((LocaleCompare("debug",option+1) == 0) && (i < (long) (argc-1)))
       (void) SetLogEventMask(argv[++i]);
-    if (LocaleCompare("version",option+1) == 0)
+    if ((LocaleCompare("version",option+1) == 0) ||
+        (LocaleCompare("-version",option+1) == 0))
       {
         (void) fprintf(stdout,"Version: %s\n",
           GetMagickVersion((unsigned long *) NULL));
         (void) fprintf(stdout,"Copyright: %s\n\n",GetMagickCopyright());
-        Exit(0);
+        exit(0);
       }
   }
   image_info=CloneImageInfo((ImageInfo *) NULL);
   status=ImportImageCommand(image_info,argc,argv,(char **) NULL,&exception);
-  if (exception.severity >= ErrorException)
+  if ((status == MagickFalse) && (exception.severity != UndefinedException))
     CatchException(&exception);
   image_info=DestroyImageInfo(image_info);
-  DestroyExceptionInfo(&exception);
+  (void) DestroyExceptionInfo(&exception);
   DestroyMagick();
-  Exit(!status);
-  return(MagickFalse);
+  if ((status == MagickFalse) && (exception.severity > ErrorException))
+    return(1);
+  return(0);
 }
