@@ -6,7 +6,7 @@
  */
 
 #if !defined(_VISUALC_)
-#include <magick/magick-config.h>
+#include <wand/wand-config.h>
 #endif
 #include <stdio.h>
 #include <stdlib.h>
@@ -17,7 +17,7 @@
 #include <sys\types.h>
 #endif
 #include <time.h>
-#include <wand/magick-wand.h>
+#include <wand/MagickWand.h>
 
 #if !defined(True)
 # define True 1
@@ -412,7 +412,7 @@ int main ( int argc, char **argv )
   GetExceptionInfo( &exception );
   FormatMagickString(size, MaxTextExtent, "%dx%d", columns, rows);
   (void) CloneString(&image_info->size,size);
-  strcpy( image_info->filename, "xc:white");
+  (void) CopyMagickString(image_info->filename,"xc:white",MaxTextExtent);
   canvas = ReadImage ( image_info, &exception );
   if (exception.severity >= ErrorException)
     CatchException(&exception);
@@ -434,7 +434,7 @@ int main ( int argc, char **argv )
     sizeof(image_info->filename));
   (void) WriteImage ( image_info, canvas );
 
-  DestroyExceptionInfo( &exception );
+  (void) DestroyExceptionInfo( &exception );
    canvas =DestroyImage( canvas );
    image_info =DestroyImageInfo( image_info );
   DestroyMagick();
