@@ -23,7 +23,7 @@
 %                                 August 2003                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -46,23 +46,8 @@
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/attribute.h"
-#include "magick/blob.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/image.h"
-#include "magick/list.h"
-#include "magick/memory_.h"
-#include "magick/option.h"
-#include "magick/paint.h"
-#include "magick/quantize.h"
-#include "magick/resize.h"
-#include "magick/resource_.h"
-#include "magick/splay-tree.h"
-#include "magick/string_.h"
-#include "magick/version.h"
-#include "wand/magick-wand.h"
+#include "wand/studio.h"
+#include "wand/MagickWand.h"
 #include "wand/magick-wand-private.h"
 #include "wand/wand.h"
 
@@ -101,7 +86,7 @@
 WandExport CompressionType MagickGetCompression(MagickWand *wand)
 {
   assert(wand != (MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
   return(wand->image_info->compression);
@@ -132,7 +117,7 @@ WandExport CompressionType MagickGetCompression(MagickWand *wand)
 WandExport unsigned long MagickGetCompressionQuality(MagickWand *wand)
 {
   assert(wand != (MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
   return(wand->image_info->quality);
@@ -193,7 +178,7 @@ WandExport char *MagickGetException(MagickWand *wand,ExceptionType *severity)
     *description;
 
   assert(wand != (const MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
   assert(severity != (ExceptionType *) NULL);
@@ -245,7 +230,7 @@ WandExport char *MagickGetException(MagickWand *wand,ExceptionType *severity)
 WandExport char *MagickGetFilename(const MagickWand *wand)
 {
   assert(wand != (const MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
   return(AcquireString(wand->image_info->filename));
@@ -276,7 +261,7 @@ WandExport char *MagickGetFilename(const MagickWand *wand)
 WandExport char *MagickGetFormat(MagickWand *wand)
 {
   assert(wand != (MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
   return(AcquireString(wand->image_info->magick));
@@ -330,7 +315,7 @@ WandExport char *MagickGetHomeURL(void)
 WandExport InterlaceType MagickGetInterlaceScheme(MagickWand *wand)
 {
   assert(wand != (MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
   return(wand->image_info->interlace);
@@ -341,6 +326,45 @@ WandExport InterlaceType MagickGetInterlaceScheme(MagickWand *wand)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   M a g i c k G e t O p t i o n                                             %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  MagickGetOption() returns a value associated with a wand and the specified
+%  key.  Use MagickRelinquishMemory() to free the value when you are finished
+%  with it.
+%
+%  The format of the MagickGetOption method is:
+%
+%      char *MagickGetOption(MagickWand *wand,const char *key)
+%
+%  A description of each parameter follows:
+%
+%    o wand: The magick wand.
+%
+%    o key: The key.
+%
+*/
+WandExport char *MagickGetOption(MagickWand *wand,const char *key)
+{
+  const char
+    *option;
+
+  assert(wand != (MagickWand *) NULL);
+  assert(wand->signature == WandSignature);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
+  option=GetImageOption(wand->image_info,key);
+  return(ConstantString(option));
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   M a g i c k G e t P a c k a g e N a m e                                   %
 %                                                                             %
 %                                                                             %
@@ -398,7 +422,7 @@ WandExport MagickBooleanType MagickGetPage(MagickWand *wand,
     geometry;
 
   assert(wand != (const MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
   (void) ResetMagickMemory(&geometry,0,sizeof(geometry));
@@ -491,6 +515,34 @@ WandExport const char *MagickGetReleaseDate(void)
 {
   return(GetMagickReleaseDate());
 }
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   M a g i c k G e t R e s o u r c e                                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  MagickGetResourceLimit() returns the specified resource in megabytes.
+%
+%  The format of the MagickGetResource method is:
+%
+%      unsigned long MagickGetResource(const ResourceType type)
+%
+%  A description of each parameter follows:
+%
+%    o wand: The magick wand.
+%
+*/
+WandExport unsigned long MagickGetResource(const ResourceType type)
+{
+  return(GetMagickResource(type));
+}
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -502,7 +554,7 @@ WandExport const char *MagickGetReleaseDate(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  MagickGetResourceLimit() returns the the specified resource in megabytes.
+%  MagickGetResourceLimit() returns the specified resource limit in megabytes.
 %
 %  The format of the MagickGetResourceLimit method is:
 %
@@ -515,7 +567,7 @@ WandExport const char *MagickGetReleaseDate(void)
 */
 WandExport unsigned long MagickGetResourceLimit(const ResourceType type)
 {
-  return(GetMagickResource(type));
+  return(GetMagickResourceLimit(type));
 }
 
 /*
@@ -556,7 +608,7 @@ WandExport double *MagickGetSamplingFactors(MagickWand *wand,
     i;
 
   assert(wand != (MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
   *number_factors=0;
@@ -574,7 +626,7 @@ WandExport double *MagickGetSamplingFactors(MagickWand *wand,
   sampling_factors=(double *) AcquireMagickMemory((size_t)
     i*sizeof(*sampling_factors));
   if (sampling_factors == (double *) NULL)
-    ThrowMagickFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
+    ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
       wand->image_info->filename);
   i=0;
   for (p=wand->image_info->sampling_factor; p != (char *) NULL; p=strchr(p,','))
@@ -623,7 +675,7 @@ WandExport MagickBooleanType MagickGetSize(const MagickWand *wand,
     geometry;
 
   assert(wand != (const MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
   (void) ResetMagickMemory(&geometry,0,sizeof(geometry));
@@ -690,7 +742,7 @@ WandExport MagickBooleanType MagickSetBackgroundColor(MagickWand *wand,
   const PixelWand *background)
 {
   assert(wand != (MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
   PixelGetQuantumColor(background,&wand->image_info->background_color);
@@ -726,7 +778,7 @@ WandExport MagickBooleanType MagickSetCompression(MagickWand *wand,
   const CompressionType compression)
 {
   assert(wand != (MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
   wand->image_info->compression=compression;
@@ -762,7 +814,7 @@ WandExport MagickBooleanType MagickSetCompressionQuality(MagickWand *wand,
   const unsigned long quality)
 {
   assert(wand != (MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
   wand->image_info->quality=quality;
@@ -798,7 +850,7 @@ WandExport MagickBooleanType MagickSetFilename(MagickWand *wand,
   const char *filename)
 {
   assert(wand != (MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
   if (filename != (const char *) NULL)
@@ -837,7 +889,7 @@ WandExport MagickBooleanType MagickSetFormat(MagickWand *wand,
     *magick_info;
 
   assert(wand != (MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
   if ((format == (char *) NULL) || (*format == '\0'))
@@ -848,7 +900,7 @@ WandExport MagickBooleanType MagickSetFormat(MagickWand *wand,
   magick_info=GetMagickInfo(format,&wand->exception);
   if (magick_info == (const MagickInfo *) NULL)
     return(MagickFalse);
-  (void) SetExceptionInfo(&wand->exception,UndefinedException);
+  ClearMagickException(&wand->exception);
   (void) CopyMagickString(wand->image_info->magick,format,MaxTextExtent);
   return(MagickTrue);
 }
@@ -883,7 +935,7 @@ WandExport MagickBooleanType MagickSetInterlaceScheme(MagickWand *wand,
   const InterlaceType interlace_scheme)
 {
   assert(wand != (MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
   wand->image_info->interlace=interlace_scheme;
@@ -895,6 +947,44 @@ WandExport MagickBooleanType MagickSetInterlaceScheme(MagickWand *wand,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   M a g i c k S e t O p t i o n                                             %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  MagickSetOption() associates one or options with the wand (.e.g
+%  MagickSetOption(wand,"jpeg:perserve","yes")).
+%
+%  The format of the MagickSetOption method is:
+%
+%      MagickBooleanType MagickSetOption(MagickWand *wand,const char *key,
+%        const char *value)
+%
+%  A description of each parameter follows:
+%
+%    o wand: The magick wand.
+%
+%    o key:  The key.
+%
+%    o value:  The value.
+%
+*/
+WandExport MagickBooleanType MagickSetOption(MagickWand *wand,const char *key,
+  const char *value)
+{
+  assert(wand != (MagickWand *) NULL);
+  assert(wand->signature == WandSignature);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
+  return(SetImageOption(wand->image_info,key,value));
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   M a g i c k S e t P a g e                                                 %
 %                                                                             %
 %                                                                             %
@@ -930,7 +1020,7 @@ WandExport MagickBooleanType MagickSetPage(MagickWand *wand,
     geometry[MaxTextExtent];
 
   assert(wand != (MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
   (void) FormatMagickString(geometry,MaxTextExtent,"%lux%lu%+ld%+ld",
@@ -944,6 +1034,42 @@ WandExport MagickBooleanType MagickSetPage(MagickWand *wand,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   M a g i c k S e t P a s s p h r a s e                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  MagickSetPassphrase() sets the passphrase.
+%
+%  The format of the MagickSetPassphrase method is:
+%
+%      MagickBooleanType MagickSetPassphrase(MagickWand *wand,
+%        const char *passphrase)
+%
+%  A description of each parameter follows:
+%
+%    o wand: The magick wand.
+%
+%    o passphrase: The passphrase.
+%
+*/
+WandExport MagickBooleanType MagickSetPassphrase(MagickWand *wand,
+  const char *passphrase)
+{
+  assert(wand != (MagickWand *) NULL);
+  assert(wand->signature == WandSignature);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
+  (void) CloneString(&wand->image_info->authenticate,passphrase);
+  return(MagickTrue);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   M a g i c k S e t P r o g r e s s M o n i t o r                           %
 %                                                                             %
 %                                                                             %
@@ -975,7 +1101,6 @@ WandExport MagickBooleanType MagickSetPage(MagickWand *wand,
 %
 %    o client_data: Specifies a pointer to any client data.
 %
-%
 */
 WandExport MagickProgressMonitor MagickSetProgressMonitor(MagickWand *wand,
   const MagickProgressMonitor progress_monitor,void *client_data)
@@ -984,7 +1109,7 @@ WandExport MagickProgressMonitor MagickSetProgressMonitor(MagickWand *wand,
     previous_monitor;
 
   assert(wand != (MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
   previous_monitor=SetImageInfoProgressMonitor(wand->image_info,
@@ -1059,7 +1184,7 @@ WandExport MagickBooleanType MagickSetResolution(MagickWand *wand,
     density[MaxTextExtent];
 
   assert(wand != (MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
   (void) FormatMagickString(density,MaxTextExtent,"%gx%g",x_resolution,
@@ -1106,7 +1231,7 @@ WandExport MagickBooleanType MagickSetSamplingFactors(MagickWand *wand,
     i;
 
   assert(wand != (MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
   if (wand->image_info->sampling_factor != (char *) NULL)
@@ -1152,7 +1277,7 @@ WandExport MagickBooleanType MagickSetSamplingFactors(MagickWand *wand,
 %
 %    o columns: The width in pixels.
 %
-%    o height: The height in pixels.
+%    o rows: The rows in pixels.
 %
 %
 */
@@ -1163,7 +1288,7 @@ WandExport MagickBooleanType MagickSetSize(MagickWand *wand,
     geometry[MaxTextExtent];
 
   assert(wand != (MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
   (void) FormatMagickString(geometry,MaxTextExtent,"%lux%lu",columns,rows);
@@ -1176,33 +1301,85 @@ WandExport MagickBooleanType MagickSetSize(MagickWand *wand,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   M a g i c k S e t P a s s p h r a s e                                     %
+%   M a g i c k S e t S i z e O f f s e t                                     %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  MagickSetPassphrase() sets the passphrase.
+%  MagickSetSizeOffset() sets the size and offset of the magick wand.  Set it
+%  before you read a raw image format such as RGB, GRAY, or CMYK.
 %
-%  The format of the MagickSetPassphrase method is:
+%  The format of the MagickSetSizeOffset method is:
 %
-%      MagickBooleanType MagickSetPassphrase(MagickWand *wand,
-%        const char *passphrase)
+%      MagickBooleanType MagickSetSizeOffset(MagickWand *wand,
+%        const unsigned long columns,const unsigned long rows,
+%        const long offset)
 %
 %  A description of each parameter follows:
 %
 %    o wand: The magick wand.
 %
-%    o passphrase: The passphrase.
+%    o columns: The width in pixels.
+%
+%    o rows: The rows in pixels.
+%
+%    o height: The height in pixels.
 %
 */
-WandExport MagickBooleanType MagickSetPassphrase(MagickWand *wand,
-  const char *passphrase)
+WandExport MagickBooleanType MagickSetSizeOffset(MagickWand *wand,
+  const unsigned long columns,const unsigned long rows,const long offset)
 {
+  char
+    geometry[MaxTextExtent];
+
   assert(wand != (MagickWand *) NULL);
-  assert(wand->signature == MagickSignature);
+  assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
-  (void) CloneString(&wand->image_info->authenticate,passphrase);
+  (void) FormatMagickString(geometry,MaxTextExtent,"%lux%lu%+ld",columns,rows,
+    offset);
+  (void) CloneString(&wand->image_info->size,geometry);
+  return(MagickTrue);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   M a g i c k S e t T y p e                                                 %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  MagickSetType() sets the image type attribute.
+%
+%  The format of the MagickSetType method is:
+%
+%      MagickBooleanType MagickSetType(MagickWand *wand,
+%        const ImageType image_type)
+%
+%  A description of each parameter follows:
+%
+%    o wand: The magick wand.
+%
+%    o image_type: The image type:   UndefinedType, BilevelType, GrayscaleType,
+%      GrayscaleMatteType, PaletteType, PaletteMatteType, TrueColorType,
+%      TrueColorMatteType, ColorSeparationType, ColorSeparationMatteType,
+%      or OptimizeType.
+%
+*/
+WandExport MagickBooleanType MagickSetType(MagickWand *wand,
+  const ImageType image_type)
+{
+  assert(wand != (MagickWand *) NULL);
+  assert(wand->signature == WandSignature);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
+  if (wand->images == (Image *) NULL)
+    ThrowWandException(WandError,"ContainsNoImages",wand->name);
+  wand->image_info->type=image_type;
   return(MagickTrue);
 }
