@@ -46,13 +46,9 @@
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/cache-view.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/memory_.h"
-#include "magick/string_.h"
-#include "wand/magick-wand.h"
+#include "wand/studio.h"
+#include "wand/MagickWand.h"
+#include "wand/magick-wand-private.h"
 #include "wand/pixel-iterator.h"
 #include "wand/pixel-wand.h"
 #include "wand/wand.h"
@@ -82,6 +78,9 @@ struct _PixelIterator
   RectangleInfo
     region;
 
+  MagickBooleanType
+    active;
+
   long
     y;
 
@@ -116,21 +115,20 @@ struct _PixelIterator
 %
 %    o iterator: The pixel iterator.
 %
-%
 */
 WandExport void ClearPixelIterator(PixelIterator *iterator)
 {
   assert(iterator != (const PixelIterator *) NULL);
-  assert(iterator->signature == MagickSignature);
+  assert(iterator->signature == WandSignature);
   if (iterator->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
-  CloseCacheView(iterator->view);
   iterator->pixel_wand=DestroyPixelWands(iterator->pixel_wand,
     iterator->region.width);
-  DestroyExceptionInfo(&iterator->exception);
+  (void) DestroyExceptionInfo(&iterator->exception);
   GetExceptionInfo(&iterator->exception);
   iterator->pixel_wand=NewPixelWands(iterator->region.width);
-  iterator->y=(-1);
+  iterator->active=MagickFalse;
+  iterator->y=0;
   iterator->debug=IsEventLogging();
 }
 
@@ -155,19 +153,18 @@ WandExport void ClearPixelIterator(PixelIterator *iterator)
 %
 %    o iterator: The pixel iterator.
 %
-%
 */
 WandExport PixelIterator *DestroyPixelIterator(PixelIterator *iterator)
 {
   assert(iterator != (const PixelIterator *) NULL);
-  assert(iterator->signature == MagickSignature);
+  assert(iterator->signature == WandSignature);
   if (iterator->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
   CloseCacheView(iterator->view);
   iterator->pixel_wand=DestroyPixelWands(iterator->pixel_wand,
     iterator->region.width);
-  DestroyExceptionInfo(&iterator->exception);
-  iterator->signature=(~MagickSignature);
+  (void) DestroyExceptionInfo(&iterator->exception);
+  iterator->signature=(~WandSignature);
   RelinquishWandId(iterator->id);
   iterator=(PixelIterator *) RelinquishMagickMemory(iterator);
   return(iterator);
@@ -196,14 +193,14 @@ WandExport PixelIterator *DestroyPixelIterator(PixelIterator *iterator)
 %    o iterator: The magick iterator.
 %
 */
-MagickExport MagickBooleanType IsPixelIterator(const PixelIterator *iterator)
+WandExport MagickBooleanType IsPixelIterator(const PixelIterator *iterator)
 {
   size_t
     length;
 
   if (iterator == (const PixelIterator *) NULL)
     return(MagickFalse);
-  if (iterator->signature != MagickSignature)
+  if (iterator->signature != WandSignature)
     return(MagickFalse);
   length=strlen(PixelIteratorId);
   if (LocaleNCompare(iterator->name,PixelIteratorId,length) != 0)
@@ -247,31 +244,48 @@ WandExport PixelIterator *NewPixelIterator(MagickWand *wand)
   unsigned long
     depth;
 
+  ViewInfo
+    *view;
+
   depth=QuantumDepth;
   quantum=GetMagickQuantumDepth(&depth);
   if (depth != QuantumDepth)
-    ThrowMagickFatalException(WandError,"WandQuantumDepthMismatch",quantum);
+    ThrowWandFatalException(WandError,"QuantumDepthMismatch",quantum);
   assert(wand != (MagickWand *) NULL);
   image=GetImageFromMagickWand(wand);
   if (image == (Image *) NULL)
     return((PixelIterator *) NULL);
+  view=OpenCacheView(image);
+  if (view == (ViewInfo *) NULL)
+    return((PixelIterator *) NULL);
   iterator=(PixelIterator *) AcquireMagickMemory(sizeof(*iterator));
   if (iterator == (PixelIterator *) NULL)
-    ThrowMagickFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-      strerror(errno));
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
+        message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
   (void) ResetMagickMemory(iterator,0,sizeof(*iterator));
   iterator->id=AcquireWandId();
   (void) FormatMagickString(iterator->name,MaxTextExtent,"%s-%lu",
     PixelIteratorId,iterator->id);
   GetExceptionInfo(&iterator->exception);
-  iterator->view=OpenCacheView(image);
+  iterator->view=view;
   SetGeometry(image,&iterator->region);
+  iterator->region.width=image->columns;
+  iterator->region.height=image->rows;
+  iterator->region.x=0;
+  iterator->region.y=0;
   iterator->pixel_wand=NewPixelWands(iterator->region.width);
-  iterator->y=(-1);
+  iterator->y=0;
   iterator->debug=IsEventLogging();
   if (iterator->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
-  iterator->signature=MagickSignature;
+  iterator->signature=WandSignature;
   return(iterator);
 }
 
@@ -302,10 +316,11 @@ WandExport MagickBooleanType PixelClearIteratorException(
   PixelIterator *iterator)
 {
   assert(iterator != (PixelIterator *) NULL);
-  assert(iterator->signature == MagickSignature);
+  assert(iterator->signature == WandSignature);
   if (iterator->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
-  return(SetExceptionInfo(&iterator->exception,UndefinedException));
+  ClearMagickException(&iterator->exception);
+  return(MagickTrue);
 }
 
 /*
@@ -338,31 +353,62 @@ WandExport MagickBooleanType PixelClearIteratorException(
 WandExport PixelIterator *NewPixelRegionIterator(MagickWand *wand,const long x,
   const long y,const unsigned long columns,const unsigned long rows)
 {
+  const char
+    *quantum;
+
   Image
     *image;
 
   PixelIterator
     *iterator;
 
+  unsigned long
+    depth;
+
+  ViewInfo
+    *view;
+
   assert(wand != (MagickWand *) NULL);
+  depth=QuantumDepth;
+  quantum=GetMagickQuantumDepth(&depth);
+  if (depth != QuantumDepth)
+    ThrowWandFatalException(WandError,"QuantumDepthMismatch",quantum);
+  if ((columns == 0) || (rows == 0))
+    ThrowWandFatalException(WandError,"ZeroRegionSize",quantum);
   image=GetImageFromMagickWand(wand);
   if (image == (Image *) NULL)
     return((PixelIterator *) NULL);
+  view=OpenCacheView(image);
+  if (view == (ViewInfo *) NULL)
+    return((PixelIterator *) NULL);
   iterator=(PixelIterator *) AcquireMagickMemory(sizeof(*iterator));
   if (iterator == (PixelIterator *) NULL)
-    ThrowMagickFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-      strerror(errno));
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
+        message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
   (void) ResetMagickMemory(iterator,0,sizeof(*iterator));
+  iterator->id=AcquireWandId();
+  (void) FormatMagickString(iterator->name,MaxTextExtent,"%s-%lu",
+    PixelIteratorId,iterator->id);
   GetExceptionInfo(&iterator->exception);
-  iterator->view=OpenCacheView(image);
+  iterator->view=view;
   SetGeometry(image,&iterator->region);
-  iterator->region.x=x;
-  iterator->region.y=y;
   iterator->region.width=columns;
   iterator->region.height=rows;
+  iterator->region.x=x;
+  iterator->region.y=y;
   iterator->pixel_wand=NewPixelWands(iterator->region.width);
-  iterator->y=(-1);
-  iterator->signature=MagickSignature;
+  iterator->y=0;
+  iterator->debug=IsEventLogging();
+  if (iterator->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
+  iterator->signature=WandSignature;
   return(iterator);
 }
 
@@ -371,6 +417,69 @@ WandExport PixelIterator *NewPixelRegionIterator(MagickWand *wand,const long x,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   P i x e l G e t C u r r e n t I t e r a t o r R o w                       %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  PixelGetCurrentIteratorRow() returns the current row as an array of pixel
+%  wands from the pixel iterator.
+%
+%  The format of the PixelGetCurrentIteratorRow method is:
+%
+%      PixelWand **PixelGetCurrentIteratorRow(PixelIterator *iterator,
+%        unsigned long *number_wands)
+%
+%  A description of each parameter follows:
+%
+%    o iterator: The pixel iterator.
+%
+%    o number_wands: The number of pixel wands.
+%
+*/
+WandExport PixelWand **PixelGetCurrentIteratorRow(PixelIterator *iterator,
+  unsigned long *number_wands)
+{
+  IndexPacket
+    *indexes;
+
+  register const PixelPacket
+    *p;
+
+  register long
+    x;
+
+  assert(iterator != (PixelIterator *) NULL);
+  assert(iterator->signature == WandSignature);
+  if (iterator->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
+  *number_wands=0;
+  iterator->active=MagickTrue;
+  p=AcquireCacheView(iterator->view,iterator->region.x,iterator->region.y+
+    iterator->y,iterator->region.width,1,&iterator->exception);
+  if (p == (const PixelPacket *) NULL)
+    return((PixelWand **) NULL);
+  indexes=GetCacheViewIndexes(iterator->view);
+  for (x=0; x < (long) iterator->region.width; x++)
+  {
+    PixelSetQuantumColor(iterator->pixel_wand[x],p);
+    if (GetCacheViewColorspace(iterator->view) == CMYKColorspace)
+      PixelSetBlackQuantum(iterator->pixel_wand[x],indexes[x]);
+    else
+      if (GetCacheViewStorageClass(iterator->view) == PseudoClass)
+        PixelSetIndex(iterator->pixel_wand[x],indexes[x]);
+    p++;
+  }
+  *number_wands=iterator->region.width;
+  return(iterator->pixel_wand);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   P i x e l G e t I t e r a t o r E x c e p t i o n                         %
 %                                                                             %
 %                                                                             %
@@ -406,15 +515,22 @@ WandExport char *PixelGetIteratorException(const PixelIterator *iterator,
     *description;
 
   assert(iterator != (const PixelIterator *) NULL);
-  assert(iterator->signature == MagickSignature);
+  assert(iterator->signature == WandSignature);
   if (iterator->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
   assert(severity != (ExceptionType *) NULL);
   *severity=iterator->exception.severity;
   description=(char *) AcquireMagickMemory(2*MaxTextExtent);
   if (description == (char *) NULL)
-    ThrowMagickFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-      strerror(errno));
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
+        message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
   *description='\0';
   if (iterator->exception.reason != (char *) NULL)
     (void) CopyMagickString(description,GetLocaleExceptionMessage(
@@ -435,6 +551,37 @@ WandExport char *PixelGetIteratorException(const PixelIterator *iterator,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   P i x e l G e t I t e r a t o r R o w                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  PixelGetIteratorRow() returns the crurent pixel iterator row.
+%
+%  The format of the PixelGetIteratorRow method is:
+%
+%      MagickBooleanType PixelGetIteratorRow(PixelIterator *iterator)
+%
+%  A description of each parameter follows:
+%
+%    o iterator: The pixel iterator.
+%
+*/
+WandExport long PixelGetIteratorRow(PixelIterator *iterator)
+{
+  assert(iterator != (const PixelIterator *) NULL);
+  assert(iterator->signature == WandSignature);
+  if (iterator->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
+  return(iterator->y);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   P i x e l G e t N e x t I t e r a t o r R o w                             %
 %                                                                             %
 %                                                                             %
@@ -444,9 +591,9 @@ WandExport char *PixelGetIteratorException(const PixelIterator *iterator,
 %  PixelGetNextIteratorRow() returns the next row as an array of pixel wands
 %  from the pixel iterator.
 %
-%  The format of the PixelGetNextRow method is:
+%  The format of the PixelGetNextIteratorRow method is:
 %
-%      PixelWand **PixelGetNextRow(PixelIterator *iterator,
+%      PixelWand **PixelGetNextIteratorRow(PixelIterator *iterator,
 %        unsigned long *number_wands)
 %
 %  A description of each parameter follows:
@@ -478,11 +625,12 @@ WandExport PixelWand **PixelGetNextIteratorRow(PixelIterator *iterator,
     x;
 
   assert(iterator != (PixelIterator *) NULL);
-  assert(iterator->signature == MagickSignature);
+  assert(iterator->signature == WandSignature);
   if (iterator->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
   *number_wands=0;
-  iterator->y++;
+  if (iterator->active != MagickFalse)
+    iterator->y++;
   if (PixelSetIteratorRow(iterator,iterator->y) == MagickFalse)
     return((PixelWand **) NULL);
   p=AcquireCacheView(iterator->view,iterator->region.x,iterator->region.y+
@@ -493,10 +641,85 @@ WandExport PixelWand **PixelGetNextIteratorRow(PixelIterator *iterator,
   for (x=0; x < (long) iterator->region.width; x++)
   {
     PixelSetQuantumColor(iterator->pixel_wand[x],p);
-    if (iterator->view->image->colorspace == CMYKColorspace)
+    if (GetCacheViewColorspace(iterator->view) == CMYKColorspace)
       PixelSetBlackQuantum(iterator->pixel_wand[x],indexes[x]);
     else
-      if (iterator->view->image->storage_class == PseudoClass)
+      if (GetCacheViewStorageClass(iterator->view) == PseudoClass)
+        PixelSetIndex(iterator->pixel_wand[x],indexes[x]);
+    p++;
+  }
+  *number_wands=iterator->region.width;
+  return(iterator->pixel_wand);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   P i x e l G e t P r e v i o u s I t e r a t o r R o w                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  PixelGetPreviousIteratorRow() returns the previous row as an array of pixel
+%  wands from the pixel iterator.
+%
+%  The format of the PixelGetPreviousIteratorRow method is:
+%
+%      PixelWand **PixelGetPreviousIteratorRow(PixelIterator *iterator,
+%        unsigned long *number_wands)
+%
+%  A description of each parameter follows:
+%
+%    o iterator: The pixel iterator.
+%
+%    o number_wands: The number of pixel wands.
+%
+*/
+
+WandExport PixelWand **PixelGetPreviousRow(PixelIterator *iterator)
+{
+  unsigned long
+    number_wands;
+
+  return(PixelGetPreviousIteratorRow(iterator,&number_wands));
+}
+
+WandExport PixelWand **PixelGetPreviousIteratorRow(PixelIterator *iterator,
+  unsigned long *number_wands)
+{
+  IndexPacket
+    *indexes;
+
+  register const PixelPacket
+    *p;
+
+  register long
+    x;
+
+  assert(iterator != (PixelIterator *) NULL);
+  assert(iterator->signature == WandSignature);
+  if (iterator->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
+  *number_wands=0;
+  if (iterator->active != MagickFalse)
+    iterator->y--;
+  if (PixelSetIteratorRow(iterator,iterator->y) == MagickFalse)
+    return((PixelWand **) NULL);
+  p=AcquireCacheView(iterator->view,iterator->region.x,iterator->region.y+
+    iterator->y,iterator->region.width,1,&iterator->exception);
+  if (p == (const PixelPacket *) NULL)
+    return((PixelWand **) NULL);
+  indexes=GetCacheViewIndexes(iterator->view);
+  for (x=0; x < (long) iterator->region.width; x++)
+  {
+    PixelSetQuantumColor(iterator->pixel_wand[x],p);
+    if (GetCacheViewColorspace(iterator->view) == CMYKColorspace)
+      PixelSetBlackQuantum(iterator->pixel_wand[x],indexes[x]);
+    else
+      if (GetCacheViewStorageClass(iterator->view) == PseudoClass)
         PixelSetIndex(iterator->pixel_wand[x],indexes[x]);
     p++;
   }
@@ -531,10 +754,43 @@ WandExport PixelWand **PixelGetNextIteratorRow(PixelIterator *iterator,
 WandExport void PixelResetIterator(PixelIterator *iterator)
 {
   assert(iterator != (PixelIterator *) NULL);
-  assert(iterator->signature == MagickSignature);
+  assert(iterator->signature == WandSignature);
   if (iterator->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
-  iterator->y=(-1);
+  iterator->active=MagickFalse;
+  iterator->y=0;
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   P i x e l S e t F i r s t I t e r a t o r R o w                           %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  PixelSetFirstIteratorRow() sets the pixel iterator to the first pixel row.
+%
+%  The format of the PixelSetFirstIteratorRow method is:
+%
+%      void PixelSetFirstIteratorRow(PixelIterator *iterator)
+%
+%  A description of each parameter follows:
+%
+%    o iterator: The magick iterator.
+%
+*/
+WandExport void PixelSetFirstIteratorRow(PixelIterator *iterator)
+{
+  assert(iterator != (PixelIterator *) NULL);
+  assert(iterator->signature == WandSignature);
+  if (iterator->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
+  iterator->active=MagickTrue;
+  iterator->y=iterator->region.y;
 }
 
 /*
@@ -559,17 +815,17 @@ WandExport void PixelResetIterator(PixelIterator *iterator)
 %
 %    o iterator: The pixel iterator.
 %
-%
 */
 WandExport MagickBooleanType PixelSetIteratorRow(PixelIterator *iterator,
   const long row)
 {
   assert(iterator != (const PixelIterator *) NULL);
-  assert(iterator->signature == MagickSignature);
+  assert(iterator->signature == WandSignature);
   if (iterator->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
-  if ((row < iterator->region.y) || (row >= (long) iterator->region.height))
+  if ((row < 0) || (row >= (long) iterator->region.height))
     return(MagickFalse);
+  iterator->active=MagickTrue;
   iterator->y=row;
   return(MagickTrue);
 }
@@ -579,6 +835,38 @@ WandExport MagickBooleanType PixelSetIteratorRow(PixelIterator *iterator,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   P i x e l S e t L a s t I t e r a t o r R o w                             %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  PixelSetLastIteratorRow() sets the pixel iterator to the last pixel row.
+%
+%  The format of the PixelSetLastIteratorRow method is:
+%
+%      void PixelSetLastIteratorRow(PixelIterator *iterator)
+%
+%  A description of each parameter follows:
+%
+%    o iterator: The magick iterator.
+%
+*/
+WandExport void PixelSetLastIteratorRow(PixelIterator *iterator)
+{
+  assert(iterator != (PixelIterator *) NULL);
+  assert(iterator->signature == WandSignature);
+  if (iterator->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
+  iterator->active=MagickFalse;
+  iterator->y=(long) iterator->region.height-1;
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   P i x e l S y n c I t e r a t o r                                         %
 %                                                                             %
 %                                                                             %
@@ -595,7 +883,6 @@ WandExport MagickBooleanType PixelSetIteratorRow(PixelIterator *iterator,
 %
 %    o iterator: The pixel iterator.
 %
-%
 */
 WandExport MagickBooleanType PixelSyncIterator(PixelIterator *iterator)
 {
@@ -609,30 +896,34 @@ WandExport MagickBooleanType PixelSyncIterator(PixelIterator *iterator)
     *p;
 
   assert(iterator != (const PixelIterator *) NULL);
-  assert(iterator->signature == MagickSignature);
+  assert(iterator->signature == WandSignature);
   if (iterator->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",iterator->name);
+  if (SetCacheViewStorageClass(iterator->view,DirectClass) == MagickFalse)
+    return(MagickFalse);
   p=GetCacheView(iterator->view,iterator->region.x,iterator->region.y+
     iterator->y,iterator->region.width,1);
   if (p == (PixelPacket *) NULL)
     {
-      InheritException(&iterator->exception,&iterator->view->image->exception);
+      InheritException(&iterator->exception,
+        GetCacheViewException(iterator->view));
       return(MagickFalse);
     }
   indexes=GetCacheViewIndexes(iterator->view);
   for (x=0; x < (long) iterator->region.width; x++)
   {
     PixelGetQuantumColor(iterator->pixel_wand[x],p);
-    if (iterator->view->image->colorspace == CMYKColorspace)
+    if (GetCacheViewColorspace(iterator->view) == CMYKColorspace)
       indexes[x]=PixelGetBlackQuantum(iterator->pixel_wand[x]);
     else
-      if (iterator->view->image->storage_class == PseudoClass)
+      if (GetCacheViewStorageClass(iterator->view) == PseudoClass)
         indexes[x]=PixelGetIndex(iterator->pixel_wand[x]);
     p++;
   }
   if (SyncCacheView(iterator->view) == MagickFalse)
     {
-      InheritException(&iterator->exception,&iterator->view->image->exception);
+      InheritException(&iterator->exception,
+        GetCacheViewException(iterator->view));
       return(MagickFalse);
     }
   return(MagickTrue);
